*===================================================================;
*  Title:  PROGRAM P3 PART 1:  CALCULATE RISK-ADJUSTED PROVIDER
*          RATES FOR AHRQ PATIENT SAFETY INDICATORS
*
*  Description:
*         USES PROC SCORE TO CALCULATE RISK-ADJUSTED RATES FOR
*         PATIENT SAFETY INDICATORS.
*         ADJUSTS FOR: AGE, SEXCAT, DRG AND COMORBIDITY
*
*          >>>  VERSION 3.2, March 2008 <<<
*
*===================================================================;
FILENAME CONTROL "C:\PATHNAME\CONTROL_PSI.SAS"; *<==USER MUST modify;

%INCLUDE CONTROL;

%MACRO LBL1;
    LABEL RPPS01 = 'COMPLICATIONS OF ANESTHESIA (Risk Adj)'
          RPPS03 = 'DECUBITUS ULCER (Risk Adj)'
          RPPS04 = 'DEATH AMONG SURGICAL (Risk Adj)'
          RPPS06 = 'IATROGENIC PNEUMOTHORAX (Risk Adj)'
          RPPS07 = 'INFECTION DUE TO MEDICAL CARE (Risk Adj)'
          RPPS08 = 'POSTOPERATIVE HIP FRACTURE (Risk Adj)'
          RPPS09 = 'POSTOP HEMORRHAGE OR HEMATOMA (RA)'
          RPPS10 = 'POSTOP PHYSIO METABOL DERANGMNT (RA)'
          RPPS11 = 'POSTOP RESPIRATORY FAILURE (Risk Adj)'
          RPPS12 = 'POSTOPERATIVE PE OR DVT (Risk Adj)'
          RPPS13 = 'POSTOPERATIVE SEPSIS (Risk Adj)'
          RPPS14 = 'POSTOPERATIVE WOUND DEHISCENCE (RA)'
          RPPS15 = 'ACCIDENTAL PUNCTURE/LACERATION (RA)'
          RPPS17 = 'BIRTH TRAUMA - INJURY TO NEONATE (RA)'
          RPPS18 = 'OB TRAUMA - VAGINAL W INSTRUMENT (RA)'
          RPPS19 = 'OB TRAUMA - VAGINAL W/O INSTRUMENT (RA)'
          LPPS01 = 'COMPLICATIONS OF ANESTHESIA (Lower CL)'
          LPPS03 = 'DECUBITUS ULCER (Lower CL)'
          LPPS04 = 'DEATH AMONG SURGICAL (Lower CL)'
          LPPS06 = 'IATROGENIC PNEUMOTHORAX (Lower CL)'
          LPPS07 = 'INFECTION DUE TO MEDICAL CARE (Lower CL)'
          LPPS08 = 'POSTOPERATIVE HIP FRACTURE (Lower CL)'
          LPPS09 = 'POSTOP HEMORRHAGE OR HEMATOMA (LL)'
          LPPS10 = 'POSTOP PHYSIO METABOL DERANGMNT (LL)'
          LPPS11 = 'POSTOP RESPIRATORY FAILURE (Lower CL)'
          LPPS12 = 'POSTOPERATIVE PE OR DVT (Lower CL)'
          LPPS13 = 'POSTOPERATIVE SEPSIS (Lower CL)'
          LPPS14 = 'POSTOPERATIVE WOUND DEHISCENCE (LL)'
          LPPS15 = 'ACCIDENTAL PUNCTURE/LACERATION (LL)'
          LPPS17 = 'BIRTH TRAUMA - INJURY TO NEONATE (LL)'
          LPPS18 = 'OB TRAUMA - VAGINAL W INSTRUMENT (LL)'
          LPPS19 = 'OB TRAUMA - VAGINAL W/O INSTRUMENT (LL)'
          UPPS01 = 'COMPLICATIONS OF ANESTHESIA (Upper CL)'
          UPPS03 = 'DECUBITUS ULCER (Upper CL)'
          UPPS04 = 'DEATH AMONG SURGICAL (Upper CL)'
          UPPS06 = 'IATROGENIC PNEUMOTHORAX (Upper CL)'
          UPPS07 = 'INFECTION DUE TO MEDICAL CARE (Upper CL)'
          UPPS08 = 'POSTOPERATIVE HIP FRACTURE (Upper CL)'
          UPPS09 = 'POSTOP HEMORRHAGE OR HEMATOMA (UL)'
          UPPS10 = 'POSTOP PHYSIO METABOL DERANGMNT (UL)'
          UPPS11 = 'POSTOP RESPIRATORY FAILURE (Upper CL)'
          UPPS12 = 'POSTOPERATIVE PE OR DVT (Upper CL)'
          UPPS13 = 'POSTOPERATIVE SEPSIS (Upper CL)'
          UPPS14 = 'POSTOPERATIVE WOUND DEHISCENCE (UL)'
          UPPS15 = 'ACCIDENTAL PUNCTURE/LACERATION (UL)'
          UPPS17 = 'BIRTH TRAUMA - INJURY TO NEONATE (UL)'
          UPPS18 = 'OB TRAUMA - VAGINAL W INSTRUMENT (UL)'
          UPPS19 = 'OB TRAUMA - VAGINAL W/O INSTRUMENT (UL)'
          ;

 * -------------------------------------------------------------- ;
 * --- RE-LABEL DAY DEPENDENT INDICATORS ------------------------ ;
 * -------------------------------------------------------------- ;
 %IF &PRDAY. = 0 %THEN %DO;
    LABEL RPPS03 = 'DECUBITUS ULCER-NO PRDAY (Risk Adj)'
          RPPS08 = 'POSTOP HIP FRACTURE-NO PRDAY (Risk Adj)'
          RPPS09 = 'POSTOP HEMOR OR HEMAT-NO PRDAY (RA)'
          RPPS10 = 'POSTOP PHYSIO METABO DE-NO PRDAY (RA)'
          RPPS11 = 'POSTOP RESP FAILURE-NO PRDAY (Risk Adj)'
          RPPS12 = 'POSTOP PE OR DVT-NO PRDAY (Risk Adj)'
          RPPS14 = 'POSTOP WOUND DEHISCENCE-NO PRDAY (RA)'
          ;
 %END;

%MEND;

%MACRO LBL2;
    LABEL SPPS01 = 'COMPLICATIONS OF ANESTHESIA (Smoothed)'
          SPPS03 = 'DECUBITUS ULCER (Smoothed)'
          SPPS04 = 'DEATH AMONG SURGICAL (Smoothed)'
          SPPS06 = 'IATROGENIC PNEUMOTHORAX (Smoothed)'
          SPPS07 = 'INFECTION DUE TO MEDICAL CARE (Smoothed)'
          SPPS08 = 'POSTOPERATIVE HIP FRACTURE (Smoothed)'
          SPPS09 = 'POSTOP HEMORRHAGE OR HEMATOMA (Smoothed)'
          SPPS10 = 'POSTOP PHYSIO METABOL DERANGEMENT (Smthd)'
          SPPS11 = 'POSTOP RESPIRATORY FAILURE (Smoothed)'
          SPPS12 = 'POSTOPERATIVE PE OR DVT (Smoothed)'
          SPPS13 = 'POSTOPERATIVE SEPSIS (Smoothed)'
          SPPS14 = 'POSTOPERATIVE WOUND DEHISCENCE (Smthd)'
          SPPS15 = 'ACCIDENTAL PUNCTURE/LACERATION(Smoothed)'
          SPPS17 = 'BIRTH TRAUMA - INJURY TO NEONATE (Smthd)'
          SPPS18 = 'OB TRAUMA - VAGINAL W INSTRUMENT (Smthd)'
          SPPS19 = 'OB TRAUMA - VAGINAL W/O INST (Smthd)'
          XPPS01 = 'COMPLICATIONS OF ANESTHESIA (Smthe SE)'
          XPPS03 = 'DECUBITUS ULCER (Smthe SE)'
          XPPS04 = 'DEATH AMONG SURGICAL (Smthe SE)'
          XPPS06 = 'IATROGENIC PNEUMOTHORAX (Smthe SE)'
          XPPS07 = 'INFECTION DUE TO MEDICAL CARE (Smthe SE)'
          XPPS08 = 'POSTOPERATIVE HIP FRACTURE (Smthe SE)'
          XPPS09 = 'POSTOP HEMORRHAGE OR HEMATOMA (Smthe SE)'
          XPPS10 = 'POSTOP PHYSIO METABOL DERANGEMENT (SmtSE)'
          XPPS11 = 'POSTOP RESPIRATORY FAILURE (Smthe SE)'
          XPPS12 = 'POSTOPERATIVE PE OR DVT (Smthe SE)'
          XPPS13 = 'POSTOPERATIVE SEPSIS (Smthe SE)'
          XPPS14 = 'POSTOPERATIVE WOUND DEHISCENCE (SmtSE)'
          XPPS15 = 'ACCIDENTAL PUNCTURE/LACERATION(Smthe SE)'
          XPPS17 = 'BIRTH TRAUMA - INJURY TO NEONATE (SmtSE)'
          XPPS18 = 'OB TRAUMA - VAGINAL W INSTRUMENT (SmtSE)'
          XPPS19 = 'OB TRAUMA - VAGINAL W/O INST (SmtSE)'
          ;

 * -------------------------------------------------------------- ;
 * --- RE-LABEL DAY DEPENDENT INDICATORS ------------------------ ;
 * -------------------------------------------------------------- ;
 %IF &PRDAY. = 0 %THEN %DO;
    LABEL SPPS03 = 'DECUBITUS ULCER-NO PRDAY (Smoothed)'
          SPPS08 = 'POSTOP HIP FRACTURE-NO PRDAY (Smoothed)'
          SPPS09 = 'POSTOP HEMOR OR HEMAT-NO PRDAY (Smthd)'
          SPPS10 = 'POSTOP PHYSIO METABO DE-NO PRDAY (Smthd)'
          SPPS11 = 'POSTOP RESP FAILURE-NO PRDAY (Smoothed)'
          SPPS12 = 'POSTOP PE OR DVT-NO PRDAY (Smoothed)'
          SPPS14 = 'POSTOP WOUND DEHISCENCE-NO PRDAY (Smthd)'
          ;
 %END;

%MEND;

%MACRO LBL3;
    LABEL EPPS01 = 'COMPLICATIONS OF ANESTHESIA (Expected)'
          EPPS03 = 'DECUBITUS ULCER (Expected)'
          EPPS04 = 'DEATH AMONG SURGICAL (Expected)'
          EPPS06 = 'IATROGENIC PNEUMOTHORAX (Expected)'
          EPPS07 = 'INFECTION DUE TO MEDICAL CARE (Expected)'
          EPPS08 = 'POSTOPERATIVE HIP FRACTURE (Expected)'
          EPPS09 = 'POSTOP HEMORRHAGE OR HEMATOMA (Expected)'
          EPPS10 = 'POSTOP PHYSIO METABOL DERANGEMENT (Exp)'
          EPPS11 = 'POSTOP RESPIRATORY FAILURE (Expected)'
          EPPS12 = 'POSTOPERATIVE PE OR DVT (Expected)'
          EPPS13 = 'POSTOPERATIVE SEPSIS (Expected)'
          EPPS14 = 'POSTOPERATIVE WOUND DEHISCENCE (Exp)'
          EPPS15 = 'ACCIDENTAL PUNCTURE/LACERATION(Expected)'
          EPPS17 = 'BIRTH TRAUMA - INJURY TO NEONATE (Exp)'
          EPPS18 = 'OB TRAUMA - VAGINAL W INSTRUMENT (Exp)'
          EPPS19 = 'OB TRAUMA - VAGINAL W/O INST (Exp)'
          ;

 * -------------------------------------------------------------- ;
 * --- RE-LABEL DAY DEPENDENT INDICATORS ------------------------ ;
 * -------------------------------------------------------------- ;
 %IF &PRDAY. = 0 %THEN %DO;
    LABEL EPPS03 = 'DECUBITUS ULCER-NO PRDAY (Expected)'
          EPPS08 = 'POSTOP HIP FRACTURE-NO PRDAY (Expected)'
          EPPS09 = 'POSTOP HEMOR OR HEMAT-NO PRDAY (Exp)'
          EPPS10 = 'POSTOP PHYSIO METABO DE-NO PRDAY (Exp)'
          EPPS11 = 'POSTOP RESP FAILURE-NO PRDAY (Expected)'
          EPPS12 = 'POSTOP PE OR DVT-NO PRDAY (Expected)'
          EPPS14 = 'POSTOP WOUND DEHISCENCE-NO PRDAY (Exp)'
          ;
 %END;

%MEND;

%MACRO PRT(PS,TEXT,CASE);
    PROC PRINT DATA=OUTP3.&OUTFILP3. LABEL SPLIT='*';
 %IF &CASE. = 1 %THEN %DO;
        VAR HOSPID AGECAT SEXCAT PAYCAT RACECAT 
            TPPS&PS. PPPS&PS. OPPS&PS.;
        LABEL HOSPID  = "HOSPID"
              AGECAT  = "AGECAT"
              SEXCAT  = "SEXCAT"
              PAYCAT  = "PAYCAT"
              RACECAT = "RACECAT"
              TPPS&PS. = "TPPS&PS.*(Numerator)"
              PPPS&PS. = "PPPS&PS.*(Denominator)"
              OPPS&PS. = "OPPS&PS.*(Observed)"
          ;
        FORMAT AGECAT AGECAT.
               SEXCAT SEXCAT.
               PAYCAT PAYCAT.
               RACECAT RACECAT.
               TPPS&PS. PPPS&PS. 13.0 OPPS&PS. 8.6;
 %END;
 %ELSE %DO;
        VAR HOSPID AGECAT SEXCAT PAYCAT RACECAT
            TPPS&PS. PPPS&PS. OPPS&PS. EPPS&PS. RPPS&PS. LPPS&PS. UPPS&PS. SPPS&PS. XPPS&PS.;
        LABEL HOSPID  = "HOSPID"
              AGECAT  = "AGECAT"
              SEXCAT  = "SEXCAT"
              PAYCAT  = "PAYCAT"
              RACECAT = "RACECAT"
              TPPS&PS. = "TPPS&PS.*(Numerator)"
              PPPS&PS. = "PPPS&PS.*(Denominator)"
              OPPS&PS. = "OPPS&PS.*(Observed)"
              EPPS&PS. = "EPPS&PS.*(Expected)"
              RPPS&PS. = "RPPS&PS.*(Risk Adj)"
              LPPS&PS. = "LPPS&PS.*(Lower CL)"
              UPPS&PS. = "UPPS&PS.*(Upper CL)"
              SPPS&PS. = "SPPS&PS.*(Smoothed)"
              XPPS&PS. = "XPPS&PS.*(Smoth SE)"
              ;
        FORMAT AGECAT AGECAT.
               SEXCAT SEXCAT.
               PAYCAT PAYCAT.
               RACECAT RACECAT.
               TPPS&PS. PPPS&PS. 13.0 OPPS&PS. RPPS&PS. SPPS&PS. EPPS&PS. 8.6;
  %END;
        TITLE4 "FINAL OUTPUT";
        TITLE5 "Indicator # &PS.: &TEXT";
        RUN;
%MEND;

%MACRO PRT2;
   %IF &PRINT. = 1 %THEN %DO;
       %PRT(01,COMPLICATIONS OF ANESTHESIA,0);
       %PRT(02,DEATH IN LOW MORTALITY DRGS,1);
       %PRT(03,DECUBITUS ULCER,0);
       %PRT(04,DEATH AMONG SURGICAL,0);
       %PRT(05,FOREIGN BODY LEFT IN DURING PROCEDURE,1);
       %PRT(06,IATROGENIC PNEUMOTHORAX,0);
       %PRT(07,INFECTION DUE TO MEDICAL CARE,0);
       %PRT(08,POST-OPERATIVE HIP FRACTURE,0);
       %IF &PRDAY. = 1 %THEN %DO;
           %PRT(09,POST-OPERATIVE HEMORRHAGE OR HEMATOMA,0);
       %END;
       %ELSE %DO;
           %PRT(09,POST-OPERATIVE HEMORRHAGE OR HEMATOMA-NO PRDAY,0);
       %END;
           %PRT(10,POST-OPERATIVE PHYSIOLOGIC AND METABOLIC DERANGEMENTS,0);
           %PRT(11,POST-OPERATIVE RESPIRATORY FAILURE,0);
       %IF &PRDAY. = 1 %THEN %DO;
           %PRT(12,POST-OPERATIVE PULMONARY EMBOLISM OR DEEP VEIN THROMBOSIS,0);
       %END;
       %ELSE %DO;
           %PRT(12,POST-OPERATIVE PULMONARY EMBOLISM OR DEEP VEIN THROMBOSIS-NO PRDAY,0);
       %END;
       %PRT(13,POST-OPERATIVE SEPSIS,0);
       %PRT(14,POST-OPERATIVE WOUND DEHISCENCE,0);
       %PRT(15,ACCIDENTAL PUNCTURE/LACERATION,0);
       %PRT(16,TRANSFUSION REACTION,1);
       %PRT(17,BIRTH TRAUMA,0);
       %PRT(18,OBSTETRIC TRAUMA - VAGINAL WITH INSTRUMENT,0);
       %PRT(19,OBSTETRIC TRAUMA - VAGINAL WITHOUT INSTRUMENT,0);
       %PRT(20,OBSTETRIC TRAUMA - C-SECTION,1);
   %END;
%MEND;

%MACRO TEXT;
   %IF &TEXTP3. = 1  %THEN %DO;
       DATA _NULL_;
          SET OUTP3.&OUTFILP3;
          FILE PSTEXTP3 LRECL=4500;
          IF _N_=1 THEN PUT
            "Hosp Id" "," "Age" "," "Sex" "," "Payer" "," "Race" "," "Type" ","
            "TPPS01" "," "TPPS02" "," "TPPS03" "," "TPPS04" ","
            "TPPS05" "," "TPPS06" "," "TPPS07" "," "TPPS08" ","
            "TPPS09" "," "TPPS10" "," "TPPS11" "," "TPPS12" ","
            "TPPS13" "," "TPPS14" "," "TPPS15" "," "TPPS16" ","
            "TPPS17" "," "TPPS18" "," "TPPS19" "," "TPPS20" ","
            "PPPS01" "," "PPPS02" "," "PPPS03" "," "PPPS04" ","
            "PPPS05" "," "PPPS06" "," "PPPS07" "," "PPPS08" ","
            "PPPS09" "," "PPPS10" "," "PPPS11" "," "PPPS12" ","
            "PPPS13" "," "PPPS14" "," "PPPS15" "," "PPPS16" ","
            "PPPS17" "," "PPPS18" "," "PPPS19" "," "PPPS20" ","
            "OPPS01" "," "OPPS02" "," "OPPS03" "," "OPPS04" ","
            "OPPS05" "," "OPPS06" "," "OPPS07" "," "OPPS08" ","
            "OPPS09" "," "OPPS10" "," "OPPS11" "," "OPPS12" ","
            "OPPS13" "," "OPPS14" "," "OPPS15" "," "OPPS16" ","
            "OPPS17" "," "OPPS18" "," "OPPS19" "," "OPPS20" ","
            "EPPS01" "," "EPPS03" "," "EPPS04" "," "EPPS06" ","
            "EPPS07" "," "EPPS08" "," "EPPS09" "," "EPPS10" ","
            "EPPS11" "," "EPPS12" "," "EPPS13" "," "EPPS14" ","
            "EPPS15" "," "EPPS17" "," "EPPS18" "," "EPPS19" "," 
            "RPPS01" "," "RPPS03" "," "RPPS04" "," "RPPS06" ","
            "RPPS07" "," "RPPS08" "," "RPPS09" "," "RPPS10" ","
            "RPPS11" "," "RPPS12" "," "RPPS13" "," "RPPS14" ","
            "RPPS15" "," "RPPS17" "," "RPPS18" "," "RPPS19" "," 
            "LPPS01" "," "LPPS03" "," "LPPS04" "," "LPPS06" ","
            "LPPS07" "," "LPPS08" "," "LPPS09" "," "LPPS10" ","
            "LPPS11" "," "LPPS12" "," "LPPS13" "," "LPPS14" ","
            "LPPS15" "," "LPPS17" "," "LPPS18" "," "LPPS19" "," 
            "UPPS01" "," "UPPS03" "," "UPPS04" "," "UPPS06" ","
            "UPPS07" "," "UPPS08" "," "UPPS09" "," "UPPS10" ","
            "UPPS11" "," "UPPS12" "," "UPPS13" "," "UPPS14" ","
            "UPPS15" "," "UPPS17" "," "UPPS18" "," "UPPS19" "," 
            "SPPS01" "," "SPPS03" "," "SPPS04" "," "SPPS06" ","
            "SPPS07" "," "SPPS08" "," "SPPS09" "," "SPPS10" ","
            "SPPS11" "," "SPPS12" "," "SPPS13" "," "SPPS14" ","
            "SPPS15" "," "SPPS17" "," "SPPS18" "," "SPPS19" "," 
            "XPPS01" "," "XPPS03" "," "XPPS04" "," "XPPS06" ","
            "XPPS07" "," "XPPS08" "," "XPPS09" "," "XPPS10" ","
            "XPPS11" "," "XPPS12" "," "XPPS13" "," "XPPS14" ","
            "XPPS15" "," "XPPS17" "," "XPPS18" "," "XPPS19"  
            ;
        PUT HOSPID 13. "," AGECAT 3. "," SEXCAT 3. "," PAYCAT 3. "," RACECAT 3. "," _TYPE_ 2. ","
           (TPPS01-TPPS20) (7.0 ",") "," 
           (PPPS01-PPPS20) (13.2 ",") ","
           (OPPS01-OPPS20) (12.10 ",") ","
           (EPPS01 EPPS03 EPPS04 EPPS06-EPPS15 EPPS17-EPPS19) (12.10 ",") ","
           (RPPS01 RPPS03 RPPS04 RPPS06-RPPS15 RPPS17-RPPS19) (12.10 ",") ","
           (LPPS01 LPPS03 LPPS04 LPPS06-LPPS15 LPPS17-LPPS19) (12.10 ",") ","
           (UPPS01 UPPS03 UPPS04 UPPS06-UPPS15 UPPS17-UPPS19) (12.10 ",") ","
           (SPPS01 SPPS03 SPPS04 SPPS06-SPPS15 SPPS17-SPPS19) (12.10 ",") ","
           (XPPS01 XPPS03 XPPS04 XPPS06-XPPS15 XPPS17-XPPS19) (12.10 ",");
       RUN;
   %END;
 %MEND;


TITLE2 'PROGRAM P3 PART I';
TITLE3 'AHRQ PATIENT SAFETY INDICATORS: CALCULATE RISK-ADJUSTED PROVIDER RATES';

* -------------------------------------------------------------- ;
* --- PSI ADJUSTED RATES  -------------------------------------- ;
* -------------------------------------------------------------- ;
* --- THIS STEP SELECTS THE OBSERVATIONS FROM THE PROGRAM 1    - ;
* --- OUTPUT FOR EACH PATIENT SAFETY INDICATOR IN TURN.        - ;
* --- EACH ITERATION PASSES VARIABLES THAT CONTROL THE         - ;
* --- COVARIATES FOR THAT PATIENT SAFETY INDICATOR:            - ;
* --- N - OBSERVATION NUMBER FROM THE COVAR FILE               - ;
* ---     ALSO IDENTIFIES THE FORMAT USED TO INDEX COVARIATES  - ;
* --- PS - THE PATIENT SAFETY INDICATOR NAME WITHOUT THE       - ;
* ---      PREFIX (P)                                          - ;
* --- A - THE NUMBER OF DEMOGRAPHIC CATEGORIES (AGE AND SEX)   - ;
* --- D - THE NUMBER OF DRG CATEGORIES                         - ;
* --- S - DETERMINES WHETHER AGE*SEX INTERACTIONS ARE NEEDED   - ;
* -------------------------------------------------------------- ;

 %LET COMOB = CHF      VALVE    PULMCIRC PERIVASC HTN_C
	  	  PARA     NEURO    CHRNLUNG DM       DMCX 
		  HYPOTHY  RENLFAIL LIVER    ULCER    AIDS 
		  LYMPH    METS     TUMOR    ARTH     OBESE
	  	  WGHTLOSS BLDLOSS  ANEMDEF  ALCOHOL  DRUG 
		  PSYCH    DEPRESS
 ;

 %MACRO MOD3(N,PS,A,D,S);

    * --- THIS SET CREATES TEMP1 WHICH CONTAINS THE DEPENDENT      - ;
    * --- VARIABLE (TPS) AND INDEPENDENT VARIABLES USED IN         - ;
    * --- REGRESSION                                               - ;

    DATA TEMP1(KEEP=KEY HOSPID AGECAT SEXCAT PAYCAT RACECAT 
                    T&PS. SEX AGE1-AGE%EVAL(2*&A.)
                    D1-D&D. &COMOB. DISCWT);
    LENGTH SEX AGE1-AGE%EVAL(2*&A.) D1-D&D. AIDX2 DIDX DIDX2 3;
    SET INP3.&INFILP3X.;

        * --- KEEP ONLY DISCHARGES NEEDED FOR THIS PS ------------------ ;
        IF T&PS. NE .;

        * --- INITIALIZE COVARIATE DUMMIES TO ZERO --------------------- ;
        ARRAY AGEX{%EVAL(2*&A.)} AGE1-AGE%EVAL(2*&A.);
        ARRAY DX{&D.} D1-D&D.;

        SEX = 0;
        DO I = 1 TO %EVAL(2*&A.);
          AGEX(I)=0;
        END;
        DO I = 1 TO &D.;
          DX(I)=0;
        END;

        * --- SET INDEX FOR AGE ARRAY (AIDX) AND DRG ARRAY (DIDX)    --- ;
        * --- CATEGORIES VARY BY PATIENT SAFETY INDICATOR AND ARE    --- ;
        * --- DETERMINED BY THE FORMAT CORRESPONDING TO N            --- ;
        AIDX2 = PUT(AGE,AGE2FMT.);

        DIDX = PUT(PUT(DRG,Z3.),$DRGFMT.);
        DIDX2 = PUT(DIDX,D&N.PSFMT.);
        IF DIDX2 IN (-1) THEN DO;
           IF 100 <= DIDX <= 199 THEN DIDX = 199;
           ELSE IF 200 <= DIDX <= 299 THEN DIDX = 299;
           ELSE IF 300 <= DIDX <= 399 THEN DIDX = 399;
           ELSE IF 400 <= DIDX <= 499 THEN DIDX = 499;
           ELSE IF 500 <= DIDX <= 599 THEN DIDX = 599;
           ELSE IF 600 <= DIDX <= 699 THEN DIDX = 699;
           ELSE IF 700 <= DIDX <= 799 THEN DIDX = 799;
           ELSE IF 800 <= DIDX <= 899 THEN DIDX = 899;
           ELSE IF 900 <= DIDX <= 999 THEN DIDX = 999;
           ELSE IF 1000 <= DIDX <= 1099 THEN DIDX = 1099;
           ELSE IF 1100 <= DIDX <= 1199 THEN DIDX = 1199;
           ELSE IF 1200 <= DIDX <= 1299 THEN DIDX = 1299;
           ELSE IF 1300 <= DIDX <= 1399 THEN DIDX = 1399;
           ELSE IF 1400 <= DIDX <= 1499 THEN DIDX = 1499;
           ELSE IF 1500 <= DIDX <= 1599 THEN DIDX = 1599;
           ELSE IF 1600 <= DIDX <= 1699 THEN DIDX = 1699;
           ELSE IF 1700 <= DIDX <= 1799 THEN DIDX = 1799;
           ELSE IF 1800 <= DIDX <= 1899 THEN DIDX = 1899;
           ELSE IF 1900 <= DIDX <= 1999 THEN DIDX = 1999;
           ELSE IF 2000 <= DIDX <= 2099 THEN DIDX = 2099;
           ELSE IF 2100 <= DIDX <= 2199 THEN DIDX = 2199;
           ELSE IF 2200 <= DIDX <= 2299 THEN DIDX = 2299;
           ELSE IF 2300 <= DIDX <= 2399 THEN DIDX = 2399;
           ELSE IF 2400 <= DIDX <= 2499 THEN DIDX = 2499;
           ELSE IF 2500 <= DIDX <= 2599 THEN DIDX = 2599;
           DIDX2 = PUT(DIDX,D&N.PSFMT.);
        END;

        * --- SET APPROPRIATE COVARIATE DUMMY TO 1 FOR THIS DISCHARGE  - ;
        IF SEXCAT IN (&S.) THEN SEX = 1;
        IF AIDX2 > 0 THEN DO; AGEX(AIDX2) = 1; AGEX(&A.+AIDX2) = 1 * SEX; END;
        IF DIDX2 > 0 THEN DX(DIDX2) = 1;

    RUN;


    * --- THIS DATA STEP READS THE REGRESSION COEFFICIENTS FOR EACH - ;
    * --- COVARIATE.                                                - ;

    DATA TEMP2(KEEP=INTERCEP SEX AGE1-AGE%EVAL(2*&A.) D1-D&D. &COMOB. 
                    _NAME_ _TYPE_ _LINK_ _LNLIKE_ _STATUS_);
        LENGTH INTERCEP SEX AGE1-AGE%EVAL(2*&A.) D1-D&D. &COMOB.
               _LNLIKE_ 8 F1-F%EVAL(275-(2*&A.+&D.+27)) 8 STATUS 3
               _NAME_ _TYPE_ _LINK_ $8 _STATUS_ $11;
        INFILE COVARP MISSOVER LRECL=4500;
        INPUT  N INTERCEP SEX AGE1-AGE%EVAL(2*&A.) 
               D1-D&D. &COMOB. F1-F%EVAL(275-((2*&A.)+&D.+27))
               _NAME_ _TYPE_ _LINK_ _LNLIKE_ STATUS _STATUS_;

        IF N = &N.;

    RUN;

    * --- THIS PROC CALCULATES A PREDICTED PSO FOR EACH OBSERVATION  - ;
    * --- USING THE ACTUAL COVARIATES AND THE ESTIMATED COEFFICIENTS - ;
    * --- THE PROVIDER EFFECT IS CALCULATED BY SUBTRACTING THE       - ;
    * --- PREDICTION FROM THE ACTUAL ON EACH OBSERVATION, AND        - ;
    * --- CALCULATING THE MEAN DIFFERENCE FOR THE PROVIDER.          - ;

    PROC SCORE DATA=TEMP1 SCORE=TEMP2 TYPE=PARMS OUT=TEMP1Y;
    VAR SEX AGE1-AGE%EVAL(2*&A.) D1-D&D. &COMOB.;
    RUN;

    %let dsid=%sysfunc(open(temp1y));
    %let dnum=%sysfunc(attrn(&dsid,nobs));
    %let drc=%sysfunc(close(&dsid));

    %if &dnum ne 0 %then %do;

    DATA   TEMP1Y;
    SET    TEMP1Y;

    EHAT = EXP(MHAT)/(1 + EXP(MHAT));
    PHAT = EHAT * (1 - EHAT);

    RUN;

    PROC   SUMMARY DATA=TEMP1Y;
    CLASS  HOSPID AGECAT SEXCAT PAYCAT RACECAT;
    VAR    T&PS. EHAT PHAT;
    OUTPUT OUT=R&PS. SUM(T&PS. EHAT PHAT)=T&PS. EHAT PHAT
                     SUMWGT(T&PS.)=P&PS.;
    WEIGHT DISCWT;
    RUN;

    DATA   R&PS.(KEEP=HOSPID AGECAT SEXCAT PAYCAT RACECAT _TYPE_
                      E&PS. R&PS. L&PS. U&PS. S&PS. X&PS. V&PS.);
    SET    R&PS.;
   
    IF _TYPE_ &TYPELVLP;

    %INCLUDE MSXP;
    IF &N. =  1 THEN SUB_N = 1;
    IF &N. =  3 THEN SUB_N = 2;
    IF &N. =  4 THEN SUB_N = 3;
    IF &N. =  6 THEN SUB_N = 4;
    IF &N. =  7 THEN SUB_N = 5;
    IF &N. =  8 THEN SUB_N = 6;
    IF &N. =  9 THEN SUB_N = 7;
    IF &N. = 10 THEN SUB_N = 8;
    IF &N. = 11 THEN SUB_N = 9;
    IF &N. = 12 THEN SUB_N = 10;
    IF &N. = 13 THEN SUB_N = 11;
    IF &N. = 14 THEN SUB_N = 12;
    IF &N. = 15 THEN SUB_N = 13;
    IF &N. = 17 THEN SUB_N = 14;
    IF &N. = 18 THEN SUB_N = 15;
    IF &N. = 19 THEN SUB_N = 16;

    E&PS. = EHAT / P&PS.;
    THAT = T&PS. / P&PS.;
 
    IF _TYPE_ IN (0,16) THEN DO;
       R&PS.   = (THAT / E&PS.) * ARRYP3(SUB_N);
       SE&PS.  = (ARRYP3(SUB_N) / E&PS.) * (1 / P&PS.) * SQRT(PHAT);
       V&PS.   = SE&PS.**2;
       SN&PS.  = ARRYP2(SUB_N) / (ARRYP2(SUB_N) + (V&PS. / P&PS.));
       S&PS.   = (R&PS. * SN&PS.) + ((1 -  SN&PS.) * ARRYP3(SUB_N));
       X&PS.   = SQRT(ARRYP2(SUB_N)- (SN&PS. * ARRYP2(SUB_N)));
    END;
    ELSE DO;
       R&PS.   = (THAT / E&PS.);
       SE&PS.  = (1 / E&PS.) * (1 / P&PS.) * SQRT(PHAT);
       V&PS.   = .;
       S&PS.   = .;
       X&PS.   = .;
    END;

    L&PS.   = R&PS. - (1.96 * SE&PS.);
    IF L&PS. < 0 THEN L&PS. = 0;
    U&PS.   = R&PS. + (1.96 * SE&PS.);

    RUN;

    %end;
    %else %do;

    DATA   R&PS.;
       HOSPID=.;AGECAT=.;SEXCAT=.;RACECAT=.;PAYCAT=.;_TYPE_=0;E&PS=.;R&PS=.;L&PS=.;U&PS=.;S&PS=.;X&PS=.;V&PS.=.; 
       OUTPUT;
    RUN;

    %end;

    PROC   DATASETS NOLIST;
    DELETE TEMP1 TEMP1Y TEMP2;
    RUN;

%MEND;

%MOD3(1,PPS01,7,60,2);
%MOD3(3,PPS03,7,227,2);
%MOD3(4,PPS04,7,213,2);
%MOD3(6,PPS06,7,96,2);
%MOD3(7,PPS07,7,173,2);
%MOD3(8,PPS08,7,16,2);
%MOD3(9,PPS09,7,100,2);
%MOD3(10,PPS10,7,36,2);
%MOD3(11,PPS11,7,89,2);
%MOD3(12,PPS12,7,134,2);
%MOD3(13,PPS13,7,58,2);
%MOD3(14,PPS14,7,21,2);
%MOD3(15,PPS15,7,162,2);
%MOD3(17,PPS17,7,2,2);
%MOD3(18,PPS18,7,2,0);
%MOD3(19,PPS19,7,2,0);

* --- MERGES THE AREA ADJUSTED RATES FOR EACH PS. PREFIX FOR THE - ;
* --- ADJUSTED RATES IS (R) AND EXPECTED RATES IS (E).           - ;

DATA RISKADJ;
    MERGE RPPS01(KEEP=HOSPID AGECAT SEXCAT PAYCAT RACECAT EPPS01 RPPS01 LPPS01 UPPS01 SPPS01 XPPS01 VPPS01)
          RPPS03(KEEP=HOSPID AGECAT SEXCAT PAYCAT RACECAT EPPS03 RPPS03 LPPS03 UPPS03 SPPS03 XPPS03 VPPS03)
          RPPS04(KEEP=HOSPID AGECAT SEXCAT PAYCAT RACECAT EPPS04 RPPS04 LPPS04 UPPS04 SPPS04 XPPS04 VPPS04)
          RPPS06(KEEP=HOSPID AGECAT SEXCAT PAYCAT RACECAT EPPS06 RPPS06 LPPS06 UPPS06 SPPS06 XPPS06 VPPS06)
          RPPS07(KEEP=HOSPID AGECAT SEXCAT PAYCAT RACECAT EPPS07 RPPS07 LPPS07 UPPS07 SPPS07 XPPS07 VPPS07)
          RPPS08(KEEP=HOSPID AGECAT SEXCAT PAYCAT RACECAT EPPS08 RPPS08 LPPS08 UPPS08 SPPS08 XPPS08 VPPS08)
          RPPS09(KEEP=HOSPID AGECAT SEXCAT PAYCAT RACECAT EPPS09 RPPS09 LPPS09 UPPS09 SPPS09 XPPS09 VPPS09)
          RPPS10(KEEP=HOSPID AGECAT SEXCAT PAYCAT RACECAT EPPS10 RPPS10 LPPS10 UPPS10 SPPS10 XPPS10 VPPS10)
          RPPS11(KEEP=HOSPID AGECAT SEXCAT PAYCAT RACECAT EPPS11 RPPS11 LPPS11 UPPS11 SPPS11 XPPS11 VPPS11)
          RPPS12(KEEP=HOSPID AGECAT SEXCAT PAYCAT RACECAT EPPS12 RPPS12 LPPS12 UPPS12 SPPS12 XPPS12 VPPS12)
          RPPS13(KEEP=HOSPID AGECAT SEXCAT PAYCAT RACECAT EPPS13 RPPS13 LPPS13 UPPS13 SPPS13 XPPS13 VPPS13)
          RPPS14(KEEP=HOSPID AGECAT SEXCAT PAYCAT RACECAT EPPS14 RPPS14 LPPS14 UPPS14 SPPS14 XPPS14 VPPS14)
          RPPS15(KEEP=HOSPID AGECAT SEXCAT PAYCAT RACECAT EPPS15 RPPS15 LPPS15 UPPS15 SPPS15 XPPS15 VPPS15)
          RPPS17(KEEP=HOSPID AGECAT SEXCAT PAYCAT RACECAT EPPS17 RPPS17 LPPS17 UPPS17 SPPS17 XPPS17 VPPS17)
          RPPS18(KEEP=HOSPID AGECAT SEXCAT PAYCAT RACECAT EPPS18 RPPS18 LPPS18 UPPS18 SPPS18 XPPS18 VPPS18)
          RPPS19(KEEP=HOSPID AGECAT SEXCAT PAYCAT RACECAT EPPS19 RPPS19 LPPS19 UPPS19 SPPS19 XPPS19 VPPS19);
    BY HOSPID AGECAT SEXCAT PAYCAT RACECAT;

    %LBL1;
    %LBL2;
    %LBL3;

RUN;

*==================================================================;
*  Title:  PROGRAM A3  PART II:  MERGE PROVIDER RATES FOR AHRQ
*  PATIENT SAFETY INDICATORS
*
*  Description:  MERGED RATES FOR PATIENT SAFETY INDICATORS
*
*          >>>  VERSION 3.1, February 2007 <<<
*
*===================================================================;

TITLE2 'PROGRAM P3 PART II';
TITLE3 'AHRQ PATIENT SAFETY INDICATORS: PROVIDER-LEVEL MERGED FILES';

* ---------------------------------------------------------------- ;
* --- PATIENT SAFETY INDICATOR MERGED RATES                   - ;
* ---------------------------------------------------------------- ;

DATA OUTP3.&OUTFILP3.;
    MERGE INP3.&INFILP3Y.(
               KEEP=HOSPID AGECAT SEXCAT PAYCAT RACECAT _TYPE_ 
                    TPPS01-TPPS20 PPPS01-PPPS20 OPPS01-OPPS20)
          RISKADJ(
               KEEP=HOSPID AGECAT SEXCAT PAYCAT RACECAT 
                    EPPS01 EPPS03 EPPS04 EPPS06-EPPS15 EPPS17-EPPS19
                    RPPS01 RPPS03 RPPS04 RPPS06-RPPS15 RPPS17-RPPS19
                    LPPS01 LPPS03 LPPS04 LPPS06-LPPS15 LPPS17-LPPS19
                    UPPS01 UPPS03 UPPS04 UPPS06-UPPS15 UPPS17-UPPS19
                    SPPS01 SPPS03 SPPS04 SPPS06-SPPS15 SPPS17-SPPS19
                    XPPS01 XPPS03 XPPS04 XPPS06-XPPS15 XPPS17-XPPS19
                    VPPS01 VPPS03 VPPS04 VPPS06-VPPS15 VPPS17-VPPS19);
    BY HOSPID AGECAT SEXCAT PAYCAT RACECAT;

    ARRAY ARRY1{16} EPPS01 EPPS03 EPPS04 EPPS06-EPPS15 EPPS17-EPPS19;
    ARRAY ARRY2{16} RPPS01 RPPS03 RPPS04 RPPS06-RPPS15 RPPS17-RPPS19;
    ARRAY ARRY3{16} LPPS01 LPPS03 LPPS04 LPPS06-LPPS15 LPPS17-LPPS19;
    ARRAY ARRY4{16} UPPS01 UPPS03 UPPS04 UPPS06-UPPS15 UPPS17-UPPS19;
    ARRAY ARRY5{16} SPPS01 SPPS03 SPPS04 SPPS06-SPPS15 SPPS17-SPPS19;
    ARRAY ARRY6{16} XPPS01 XPPS03 XPPS04 XPPS06-XPPS15 XPPS17-XPPS19;
    ARRAY ARRY7{16} VPPS01 VPPS03 VPPS04 VPPS06-VPPS15 VPPS17-VPPS19;
    ARRAY ARRY8{16} PPPS01 PPPS03 PPPS04 PPPS06-PPPS15 PPPS17-PPPS19;

    DO I = 1 TO 16;
       IF ARRY8(I) <= 2 THEN DO;
          ARRY1(I) = .; ARRY2(I) = .; ARRY3(I) = .; ARRY4(I) = .;
          ARRY5(I) = .; ARRY6(I) = .; ARRY7(I) = .; 
       END;
    END;

    DROP I;

RUN;

* -------------------------------------------------------------- ;
* --- CONTENTS AND MEANS OF PROVIDER-LEVEL MERGED FILE --------- ;
* -------------------------------------------------------------- ;

PROC CONTENTS DATA=OUTP3.&OUTFILP3. POSITION;
RUN;

PROC MEANS DATA=OUTP3.&OUTFILP3. (WHERE=(_TYPE_ IN (16))) N NMISS MIN MAX MEAN SUM;
    TITLE4 "SUMMARY OF PROVIDER-LEVEL RATES (_TYPE_=16)";
RUN;

* -------------------------------------------------------------- ;
* --- PRINT PROVIDER MERGED FILE ------------------------------- ;
* -------------------------------------------------------------- ;

%PRT2


* -------------------------------------------------------------- ;
* --- WRITE SAS OUTPUT DATA SET TO COMMA-DELIMITED TEXT FILE --- ;
* --- FOR EXPORT INTO SPREADSHEETS ----------------------------- ;
* -------------------------------------------------------------- ;

%TEXT
