*============================================================================;
* Title:
*     Format Library for Patient Safety Indicators Program;
*
* Description:
*     FORMAT LIB FOR AHRQ PATIENT SAFETY INDICATORS;
*
*          >>>  VERSION 3.2, March 2008 <<<
*
*     Note: Codes valid from 10/1/1994 to 9/30/2008
*
*============================================================================;
FILENAME CONTROL "C:\PATHNAME\CONTROL_PSI.SAS";          *<==USER MUST modify;

%INCLUDE CONTROL;

 /* -------------------------------------------------------------- */
 /* --- FORMATS FOR COMORBIDITIES -------------------------------- */
 /* -------------------------------------------------------------- */

%INCLUDE CMFORMAT;

PROC FORMAT DDNAME=LIBRARY;

 /* -------------------------------------------------------------- */
 /* --- FORMATS FOR STRATIFIERS ---------------------------------- */
 /* -------------------------------------------------------------- */

 /* --- FORMAT FOR SEX CATEGORY ---------------------------------- */
VALUE SEXCAT
 1 = 'Male'
 2 = 'Female'
  ;

 /* --- FORMAT FOR AGE CATEGORY ---------------------------------- */
VALUE AGECAT
 0 = '00 to 17'
 1 = '18 to 39'
 2 = '40 to 64'
 3 = '65 to 74'
 4 = '75+'
  ;

 /* --- FORMAT FOR RACE CATEGORY --------------------------------- */
VALUE RACECAT
 1 = 'White'
 2 = 'Black'
 3 = 'Hispanic'
 4 = 'Asian and NH/PI'
 5 = 'Amer Indian/AN'
 6 = 'Other'
  ;

 /* --- FORMAT FOR PAYER CATEGORY ---------------------------------- */
VALUE PAYCAT
 1 = 'Medicare'
 2 = 'Medicaid'
 3 = 'Private'
 4 = 'Self Pay'
 5 = 'No Charge'
 6 = 'Other'
  ;

 * -------------------------------------------------------------- ;
 * --- PSI DEFINING FORMATS ------------------------------------- ;
 * -------------------------------------------------------------- ;

 * -------------------------------------------------------------- ;
 * --- FORMATS - PROVIDER AND AREA LEVEL INDICATORS ------------- ;
 * -------------------------------------------------------------- ;

 /* MEDICAL DRGS                                                    */
 /* USED TO DEFINE PSIS #  3: DECUBITUS ULCER                       */
 /*                        5: FOREIGN BODY LEFT IN DURING PROCEDURE */
 /*                        6: IATROGENIC PNEUMOTHORAX               */
 /*                        7: INFECTION DUE TO MEDICAL CARE         */
 /*                       15: TECBNICAL DIFFICULTY WITH PROCEDURE   */
 /*                       16: TRANSFUSION REACTION                  */
 /*                       21: FOREIGN BODY LEFT IN DURING PROCEDURE */
 /*                       22: IATROGENIC PNEUMOTHORAX               */
 /*                       23: INFECTION DUE TO MEDICAL CARE         */
 /*                       25: TECBNICAL DIFFICULTY WITH PROCEDURE   */
 /*                       26: TRANSFUSION REACTION                  */
 /* DIAGNOSTIC RELATED GROUPS (DRGS): */

VALUE $MEDICDR
"009" = "1" /* SPINAL DISORDERS AND INJURIES */
"010" = "1" /* NERVOUS SYSTEM NEOPLASMS WITH CC */
"011" = "1" /* NERVOUS SYSTEM NEOPLASMS WITH CC */
"012" = "1" /* DEGENERATIVE NERVOUS SYSTEM DISORDERS */
"013" = "1" /* MULTIPLE SCLEROSIS AND CEREBELLAR ATAXIA */
"014" = "1" /* SPECIFIC CEREBROVASCULAR DISORDERS EXCEPT TRANSIENT ISCHEMIC ATTACK */
"015" = "1" /* TRANSIENT ISCHEMIC ATTACK AND PRECEREBRAL OCCLUSIONS */
"016" = "1" /* NONSPECIFIC CEREBROVASCULAR DISORDERS WITH CC */
"017" = "1" /* NONSPECIFIC CEREBROVASCULAR DISORDERS WITHOUT CC */
"018" = "1" /* CRANIAL AND PERIPHERAL NERVE DISORDERS WITH CC */
"019" = "1" /* CRANIAL AND PERIPHERAL NERVE DISORDERS WITHOUT CC */
"020" = "1" /* NERVOUS SYSTEM INFECTION EXCEPT VIRAL MENINGITIS */
"021" = "1" /* VIRAL MENINGITIS */
"022" = "1" /* HYPERTENSIVE ENCEPHALOPATHY */
"023" = "1" /* NONTRAUMATIC STUPOR AND COMA */
"024" = "1" /* SEIZURE AND HEADACHE, AGE GREATER THAN 17 WITH CC */
"025" = "1" /* SEIZURE AND HEADACHE, AGE GREATER THAN 17 WITHOUT CC */
"026" = "1" /* SEIZURE AND HEADACHE, AGE 0-17 */
"027" = "1" /* TRAUMATIC STUPOR AND COMA, COMA GREATER THAN ONE HOUR */
"028" = "1" /* TRAUMATIC STUPOR AND COMA, COMA LESS THAN ONE HOUR, AGE GREATER THAN 17 WITH CC */
"029" = "1" /* TRAUMATIC STUPOR AND COMA, COMA LESS THAN ONE HOUR, AGE GREATER THAN 17 WITHOUT CC */
"030" = "1" /* TRAUMATIC STUPOR AND COMA, COMA LESS THAN ONE HOUR, AGE 0-17 */
"031" = "1" /* CONCUSSION, AGE GREATER THAN 17 WITH CC */
"032" = "1" /* CONCUSSION, AGE GREATER THAN 17 WITHOUT CC */
"033" = "1" /* CONCUSSION, AGE 0-17 */
"034" = "1" /* OTHER DISORDERS OF NERVOUS SYSTEM WITH CC */
"035" = "1" /* OTHER DISORDERS OF NERVOUS SYSTEM WITHOUT CC */
"043" = "1" /* HYPHEMA */
"044" = "1" /* ACUTE MAJOR EYE INFECTIONS */
"045" = "1" /* NEUROLOGICAL EYE DISORDERS */
"046" = "1" /* OTHER DISORDERS OF THE EYE, AGE GREATER THAN 17 WITH CC */
"047" = "1" /* OTHER DISORDER OF THE EYE, AGE GREATER THAN 17 WITHOUT CC */
"048" = "1" /* OTHER DISORDERS OF THE EYE, AGE 0-17 */
"064" = "1" /* EAR, NOSE, MOUTH AND THROAT MALIGNANCY */
"065" = "1" /* DISEQUILIBRIA */
"066" = "1" /* EPISTAXIS */
"067" = "1" /* EPIGLOTTITIS */
"068" = "1" /* OTITIS MEDIA AND URI, AGE GREATER THAN 17 WITH CC */
"069" = "1" /* OTITIS MEDIA AND URI, AGE GREATER THAN 17 WITHOUT CC */
"070" = "1" /* OTITIS MEDIA AND URI, AGE 0-17 */
"071" = "1" /* LARYNGOTRACHEITIS */
"072" = "1" /* NASAL TRAUMA AND DEFORMITY */
"073" = "1" /* OTHER EAR, NOSE, MOUTH AND THROAT DIAGNOSES, AGE GREATER THAN 17 */
"074" = "1" /* OTHER EAR, NOSE, MOUTH AND THROAT DIAGNOSES, AGE 0-17 */
"078" = "1" /* PULMONARY EMBOLISM */
"079" = "1" /* RESPIRATORY INFECTIONS AND INFLAMMATIONS, AGE GREATER THAN 17 WITH CC */
"080" = "1" /* RESPIRATORY INFECTIONS AND INFLAMMATIONS, AGE GREATER THAN 17 WITHOUT CC */
"081" = "1" /* RESPIRATORY INFECTIONS AND INFLAMMATIONS, AGE 0-17 */
"082" = "1" /* RESPIRATORY NEOPLASMS */
"083" = "1" /* MAJOR CHEST TRAUMA WITH CC */
"084" = "1" /* MAJOR CHEST TRAUMA WITHOUT CC*/
"085" = "1" /* PLEURAL EFFUSION WITH CC */
"086" = "1" /* PLEURAL EFFUSION WITHOUT CC */
"087" = "1" /* PULMONARY EDEMA AND RESPIRATORY FAILURE */
"088" = "1" /* CHRONIC OBSTRUCTIVE PULMONARY DISEASE */
"089" = "1" /* SIMPLE PNEUMONIA AND PLEURISY, AGE GREATER THAN 17 WITH CC */
"090" = "1" /* SIMPLE PNEUMONIA AND PLEURISY, AGE GREATER THAN 17 WITHOUT CC */
"091" = "1" /* SIMPLE PNEUMONIA AND PLEURISY, AGE 0-17 */
"092" = "1" /* INTERSTITIAL LUNG DISEASE WITH CC */
"093" = "1" /* INTERSTITIAL LUNG DISEASE WITHOUT CC */
"094" = "1" /* PNEUMOTHORAX WITH CC */
"095" = "1" /* PNEUMOTHORAX WITHOUT CC */
"096" = "1" /* BRONCHITIS AND ASTHMA, AGE GREATER THAN 17 WITH CC */
"097" = "1" /* BRONCHITIS AND ASTHMA, AGE GREATER THAN 17 WITHOUT CC */
"098" = "1" /* BRONCHITIS AND ASTHMA, AGE 0-17 */
"099" = "1" /* RESPIRATORY SIGNS AND SYMPTOMS WITH CC */
"100" = "1" /* RESPIRATORY SIGNS AND SYMPTOMS WITHOUT CC */
"101" = "1" /* OTHER RESPIRATORY SYSTEM DIAGNOSES WITH CC */
"102" = "1" /* OTHER RESPIRATORY SYSTEM DIAGNOSES WITHOUT CC */
"121" = "1" /* CIRCULATORY DISORDERS WITH ACUTE MYOCARDIAL INFARCTION AND MAJOR COMPLICATION, DISCHARGED ALIVE */
"122" = "1" /* CIRCULATORY DISORDERS WITH ACUTE MYOCARDIAL INFARCTION WITHOUT MAJOR COMPLICATION, DISCHARGED ALIVE */
"123" = "1" /* CIRCULATORY DISORDERS WITH ACUTE MYOCARDIAL INFARCTION, EXPIRED */
"124" = "1" /* CIRCULATORY DISORDERS EXCEPT ACUTE MYOCARDIAL INFARCTION WITH CARDIAC CATHETERIZATION AND COMPLEX DIAGNOSIS */
"125" = "1" /* CIRCULATORY DISORDERS EXCEPT ACUTE MYOCARDIAL INFARCTION WITH CARDIAC CATHETERIZATION WITHOUT COMPLEX DIAGNOSIS */
"126" = "1" /* ACUTE AND SUB ACUTE ENDOCARDITIS */
"127" = "1" /* HEART FAILURE AND SHOCK */
"128" = "1" /* DEEP VEIN THROMBOPHLEBITIS */
"129" = "1" /* CARDIAC ARREST, UNEXPLAINED */
"130" = "1" /* PERIPHERAL VASCULAR DISORDERS WITH CC */
"131" = "1" /* PERIPHERAL VASCULAR DISORDERS WITHOUT CC */
"132" = "1" /* ATHEROSCLEROSIS WITH CC */
"133" = "1" /* ATHEROSCLEROSIS WITHOUT CC */
"134" = "1" /* HYPERTENSION */
"135" = "1" /* CARDIAC CONGENITAL AND VALVULAR DISORDERS, AGE GREATER THAN 17 WITH CC */
"136" = "1" /* CARDIAC CONGENITAL AND VALVULAR DISORDERS, AGE GREATER THAN 17 WITHOUT CC */
"137" = "1" /* CARDIAC CONGENITAL AND VALVULAR DISORDERS, AGE 0 - 17*/
"138" = "1" /* CARDIAC ARRHYTHMIA AND CONDUCTION DISORDERS WITH CC */
"139" = "1" /* CARDIAC ARRHYTHMIA AND CONDUCTION DISORDERS WITHOUT CC */
"140" = "1" /* ANGINA PECTORIS*/
"141" = "1" /* SYNCOPE AND COLLAPSE WITH CC */
"142" = "1" /* SYNCOPE AND COLLAPSE WITHOUT CC */
"143" = "1" /* CHEST PAIN */
"144" = "1" /* OTHER CIRCULATORY SYSTEM DIAGNOSES WITH CC*/
"145" = "1" /* OTHER CIRCULATORY SYSTEM DIAGNOSES WITHOUT CC */
"172" = "1" /* DIGESTIVE MALIGNANCY WITH CC */
"173" = "1" /* DIGESTIVE MALIGNANCY WITHOUT CC */
"174" = "1" /* GI HEMORRHAGE WITH CC*/
"175" = "1" /* GI HEMORRHAGE WITHOUT CC */
"176" = "1" /* COMPLICATED PEPTIC ULCER */
"177" = "1" /* UNCOMPLICATED PEPTIC ULCER WITH CC */
"178" = "1" /* UNCOMPLICATED PEPTIC ULCER WITHOUT CC*/
"179" = "1" /* INFLAMMATORY BOWEL DISEASE */
"180" = "1" /* GI OBSTRUCTION WITH CC */
"181" = "1" /* GI OBSTRUCTION WITHOUT CC */
"182" = "1" /* ESOPHAGITIS, GASTROENTERITIS AND MISCELLANEOUS DIGESTIVE DISORDERS, AGE GREATER THAN 17 WITH CC */
"183" = "1" /* ESOPHAGITIS, GASTROENTERITIS AND MISCELLANEOUS DIGESTIVE DISORDERS, AGE GREATER THAN 17 WITHOUT CC */
"184" = "1" /* ESOPHAGITIS, GASTROENTERITIS AND MISCELLANESOU DIGESTIVE DISORDERS, AGE 0-17 */
"185" = "1" /* DENTAL AND ORAL DISEASES EXCEPT EXTRACTIONS AND RESTORATIONS, AGE GREATER THAN 17 */
"186" = "1" /* DENTAL AND ORAL DISEASED EXCEPT EXTRACTIONS AND RESTORATIONS, AGE 0-17 */
"187" = "1" /* DENTAL EXTRACTIONS AND RESTORATIONS */
"188" = "1" /* OTHER DIGESTIVE SYSTEM DIAGNOSES, AGE GREATER THAN 17 WITH CC */
"189" = "1" /* OTHER DIGESTIVE SYSTEM DIAGNOSES, AGE GREATER THAN 17 WITHOUT CC */
"190" = "1" /* OTHER DIGESTIVE SYSTEM DIAGNOSES, AGE 0-17 */
"202" = "1" /* CIRRHOSIS AND ALCOHOLIC HEPATITIS */
"203" = "1" /* MALIGNANCY OF HEPATOBILIARY SYSTEM OR PANCREAS */
"204" = "1" /* DISORDERS OF PANCREAS EXCEPT MALIGNANCY */
"205" = "1" /* DISORDERS OF LIVER EXCEPT MALIGNANCY, CIRRHOSIS AND ALCOHOLIC HEPATITIS WITH CC */
"206" = "1" /* DISORDERS OF LIVER EXCEPT MALIGNANCY, CIRRHOSIS AND ALCOHOLIC HEPATITIS WITHOUT CC */
"207" = "1" /* DISORDERS OF THE BILIARY TRACT WITH CC */
"208" = "1" /* DISORDERS OF THE BILIARY TRACT WITHOUT CC */
"235" = "1" /* FRACTURES OF FEMUR */
"236" = "1" /* FRACTURES OF HIP AND PELVIS */
"237" = "1" /* SPRAINS, STRAINS AND DISLOCATIONS OF HIP, PELVIS AND THIGH */
"238" = "1" /* OSTEOMYELITIS */
"239" = "1" /* PATHOLOGICAL FRACTURES AND MUSCULOSKELETAL AND CONNECTIVE TISSUE MALIGNANCY */
"240" = "1" /* CONNECTIVE TISSUE DISORDERS WITH CC */
"241" = "1" /* CONNECTIVE TISSUE DISORDERS WITHOUT CC */
"242" = "1" /* SEPTIC ARTHRITIS */
"243" = "1" /* MEDICAL BACK PROBLEMS */
"244" = "1" /* BONE DISEASES AND SPECIFIC ARTHROPATHIES WITH CC */
"245" = "1" /* BONE DISEASES AND SPECIFIC ARTHROPATHIES WITHOUT CC */
"246" = "1" /* NONSPECIFIC ARTHROPATHIES */
"247" = "1" /* SIGNS AND SYMPTOMS OF MUSCULOSKELETAL SYSTEM AND CONNECTIVE TISSUE */
"248" = "1" /* TENDONITIS, MYOSITIS AND BURSITIS */
"249" = "1" /* AFTERCARE, MUSCULOSKELETAL SYSTEM AND CONNECTIVE TISSUE */
"250" = "1" /* FRACTURES, SPRAINS, STRAINS AND DISLOCATIONS OF FOREARM, HAND AND FOOT, AGE GREATER THAN 17 WITH CC */
"251" = "1" /* FRACTURES, SPRAINS, STRAINS AND DISLOCATIONS OF FOREARM, HAND AND FOOT, AGE GREATER THAN 17 WITHOUT CC */
"252" = "1" /* FRACTURES, SPRAINS, STRAINS AND DISLOCATIONS OF FOREARM, HAND AND FOOT, AGE 0-17 */
"253" = "1" /* FRACTURES, SPRAINS, STRAINS AND DISLOCATIONS OF UPPER ARM AND LOWER LEG EXCEPT FOOT, AGE GREATER THAN 17 WITH CC */
"254" = "1" /* FRACTURES, SPRAINS, STRAINS AND DISLOCATIONS OF UPPER ARM AND LOWER LEG EXCEPT FOOT, AGE GREATER THAN 17 WITHOUT CC */
"255" = "1" /* FRACTURES, SPRAINS, STRAINS AND DISLOCATIONS OF UPPER ARM AND LOWER LEG EXCEPT FOOT, AGE 0-17 */
"256" = "1" /* OTHER MUSCULOSKELETAL SYSTEM AND CONNECTIVE TISSUE DIAGNOSES */
"271" = "1" /* SKIN ULCERS */
"272" = "1" /* MAJOR SKIN DISORDERS WITH CC */
"273" = "1" /* MAJOR SKIN DISORDERS WITHOUT CC */
"274" = "1" /* MALIGNANT BREAST DISORDERS WITH CC */
"275" = "1" /* MALIGNANT BREAST DISORDERS WITHOUT CC */
"276" = "1" /* NONMALIGNANT BREAST DISORDERS */
"277" = "1" /* CELLULITIS, AGE GREATER THAN 17 WITH CC */
"278" = "1" /* CELLULTIIS, AGE GREATER THAN 17 WITHOUT CC */
"279" = "1" /* CELLULITIS, AGE 0-17 */
"280" = "1" /* TRAUMA TO SKIN, SUBCUTANEOUS TISSUE AND BREAST, AGE GREATER THAN 17 WITH CC */
"281" = "1" /* TRAUMA TO SKIN, SUBCUTANEOUS TISSUE AND BREAST, AGE GREATER THAN 17 WITHOUT CC */
"282" = "1" /* TRAUMA TO SKIN, SUBCUTANEOUS TISSUE AND BREAST, AGE 0-17 */
"283" = "1" /* MINOR SKIN DISORDERS WITH CC */
"284" = "1" /* MINOR SKIN DISORDERS WITHOUT CC */
"294" = "1" /* DIABETES, AGE GREATER THAN 35 */
"295" = "1" /* DIABETES, AGE 0-35 */
"296" = "1" /* NUTRITIONAL AND MISCELLANEOUS METABOLIC DISORDERS, AGE GREATER THAN 17 WITH CC */
"297" = "1" /* NUTRITIONAL AND MISCELLANEOUS METABOLIC DISORDERS, AGE GREATER THAN 17 WITHOUT CC */
"298" = "1" /* NUTRITIONAL AND MISCELLANEOUS METABOLIC DISORDERS, AGE 0-17 */
"299" = "1" /* INBORN ERRORS OF METABOLISM */
"300" = "1" /* ENDOCRINE DISORDERS WITH CC */
"301" = "1" /* ENDOCRINE DISORDERS WITHOUT CC */
"316" = "1" /* RENAL FAILURE */
"317" = "1" /* ADMISSION FOR RENAL DIALYSIS */
"318" = "1" /* KIDNEY AND URINARY TRACT NEOPLASMS WITH CC */
"319" = "1" /* KIDNEY AND URINARY TRACT NEOPLASMS WITHOUT CC */
"320" = "1" /* KIDNEY AND URINARY TRACT INFECTIONS, AGE GREATER THAN 17 WITH CC */
"321" = "1" /* KIDNEY AND URINARY TRACT INFECTIONS, AGE GREATER THAN 17 WITHOUT CC */
"322" = "1" /* KIDNEY AND URINARY TRACT INFECTION, AGE 0-17 */
"323" = "1" /* URINARY STONES WITH CC AND/ OR ESW LITHOTRIPSY */
"324" = "1" /* URINARY STONES WITHOUT CC */
"325" = "1" /* KIDNEY AND URINARY TRACT SIGNS AND SYMPTOMS, AGE GREATER THAN 17 WITH CC */
"326" = "1" /* KIDNEY AND URINARY TRACT SIGNS AND SYMPTOMS, AGE GREATER THAN 17 WITHOUT CC */
"327" = "1" /* KIDNEY AND URINARY TRACT SIGNS AND SYMPTOMS, AGE 0-17 */
"328" = "1" /* URETHRAL STRICTURE, AGE GREATER THAN 17 WITH CC */
"329" = "1" /* URETHRAL STRICTURE, AGE GREATER THAN 17 WITHOUT CC */
"330" = "1" /* URETHRAL STRICTURE, AGE AGE 0-17 */
"331" = "1" /* OTHER KIDNEY AND URINARY TRACT DIAGNOSES, AGE GREATER THAN 17 WITH CC */
"332" = "1" /* OTHER KIDNEY AND URINARY TRACT DIAGNOSES, AGE GREATER THAN 17 WITHOUT CC */
"333" = "1" /* OTHER KIDNEY AND URINARY TRACT DIAGNOSES, AGE 0-17 */
"346" = "1" /* MALIGNANCY OF MALE REPRODUCTIVE SYSTEM WITH CC */
"347" = "1" /* MALIGNANCY OF MALE REPRODUCTIVE SYSTEM WITHOUT CC */
"348" = "1" /* BENIGN PROSTATIC HYPERTROPHY WITH CC */
"349" = "1" /* BENIGN PROSTATIC HYPERTROPHY WITHOUT CC */
"350" = "1" /* INFLAMMATION OF THE MALE REPRODUCTIVE SYSTEM */
"351" = "1" /* STERILIZATION, MALE */
"352" = "1" /* OTHER MALE REPRODUCTIVE SYSTEM DIAGNOSES */
"366" = "1" /* MALIGNANCY OF FEMALE REPRODUCTIVE SYSTEM WITH CC */
"367" = "1" /* MALIGNANCY OF FEMALE REPRODUCTIVE SYSTEM WITHOUT CC */
"368" = "1" /* INFECTIONS OF FEMALE REPRODUCTIVE SYSTEM */
"369" = "1" /* MENSTRUAL AND OTHER FEMALE REPRODUCTIVE SYSTEM DISORDERS */
"372" = "1" /* VAGINAL DELIVERY WITH COMPLICATING DIAGNOSES */
"373" = "1" /* VAGINAL DELIVERY WITHOUT COMPLICATING DIAGNOSES */
"376" = "1" /* POSTPARTUM AND POSTABORTION DIAGNOSES WITHOUT OR PROCEDURE */
"378" = "1" /* ENTOPIC PREGNANCY */
"379" = "1" /* THREATENED ABORTION */
"380" = "1" /* ABORTION WITHOUT D AND G */
"382" = "1" /* FALSE LABOR */
"383" = "1" /* OTHER ANTEPARTUM DIAGNOSES WITH MEDICAL COMPLICATIONS */
"384" = "1" /* OTHER ANTEPARTUM DIAGNOSES WITHOUT MEDICAL COMPLICATIONS */
"395" = "1" /* RED BLOOD CELL DISORDERS, AGE GREATER THAN 17 */
"396" = "1" /* RED BLOOD CELL DISORDERS, AGE 0-17 */
"397" = "1" /* COAGULATION DISORDERS */
"398" = "1" /* RETICULOENDOTHELIAL AND IMMUNITY DISORDERS WITH CC */
"399" = "1" /* RETICULOENDOTHELIAL AND IMMUNITY DISORDERS WITHOUT CC */
"403" = "1" /* LYMPHOMA AND NONACUTE LEUKEMIA WITH CC */
"404" = "1" /* LYMPHOMA AND NONACUTE LEUKEMIA WITHOUT CC */
"405" = "1" /* ACUTE LEUKEMIA WITHOUT MAJOR OR PROCEDURE, AGE 0-17 */
"409" = "1" /* RADIOTHERAPY */
"410" = "1" /* CHEMOTHERAPY WITHOUT ACUTE LEUKEMIA AS SECONDARY DIAGNOSIS */
"411" = "1" /* HISTORY OF MALIGNANCY WITHOUT ENDOSCOPY */
"412" = "1" /* HISTORY OF MALIGNANCY WITH ENDOSCOPY */
"413" = "1" /* OTHER MYELOPROLIFERATIVE DISORDERS OR POORLY DIFFERENTIATED NEOPLASM DIAGNOSES WITH CC */
"414" = "1" /* OTHER MYELOPROLIFERATIVE DISORDERS OR POORLY DIFFERENTIATED NEOPLASM DIAGNOSES WITHOUT CC */
"416" = "1" /* SEPTICEMIA, AGE GREATER THAN 17 */
"417" = "1" /* SEPTICEMIA, AGE 0-17 */
"418" = "1" /* POSTOPERATIVE AND POSTTRAUMATIC INFECTIONS */
"419" = "1" /* FEVER OF UNKNOWN ORIGIN, AGE GREATER THAN 17 WITH CC */
"420" = "1" /* FEVER OF UNKNOWN ORIGIN, AGE GREATER THAN 17 WITHOUT CC */
"421" = "1" /* VIRAL ILLNESS, AGE GREATER THAN 17 */
"422" = "1" /* VIRAL ILLNESS AND FEVER OF UNKNOWN ORIGIN, AGE 0-17 */
"423" = "1" /* OTHER INFECTIOUS AND PARASITIC DISEASES DIAGNOSES */
"425" = "1" /* ACUTE ADJUSTMENT REACTIONS AND DISTURBANCES OF PSYCHOSOCIAL DYSFUNCTION */
"426" = "1" /* DEPRESSIVE NEUROSES */
"427" = "1" /* NEUROSES EXCEPT DEPRESSIVE */
"428" = "1" /* DISORDERS OF PERSONALITY AND IMPULSE CONTROL */
"429" = "1" /* ORGANIC DISTURBANCES AND MENTAL RETARDATION */
"430" = "1" /* PSYCHOSES */
"431" = "1" /* CHILDHOOD MENTAL DISORDERS */
"432" = "1" /* OTHER MENTAL DISORDER DIAGNOSES */
"433" = "1" /* ALCOHOL/DRUG ABUSE OR DEPENDENCE, LEFT AGAINST MEDICAL ADVICE */
"434" = "1" /* ALCOHOL/DRUG ABUSE OR DEPENDENCE, DETOXIFICATION OR OTHER SYMPTOMATIC TREATMENT WITH CC */
"435" = "1" /* ALCOHOL/DRUG ABUSE OR DEPENDENCE, DETOXIFICATION OR OTHER SYMPTOMATIC TREATMENT WITHOUT CC */
"436" = "1" /* ALCOHOL/DRUG DEPENDENCE WITH REHABILITATION THERAPY */
"437" = "1" /* ALCOHOL DRUG DEPENDENCE WITH COMBINED REHABILITATION AND DETOXIFICATION THERAPY */
"444" = "1" /* TRAUMATIC INJURY, AGE GREATER THAN 17 WITH CC */
"445" = "1" /* TRAUMATIC INJURY, AGE GREATER THAN 17 WITHOUT CC */
"446" = "1" /* TRAUMATIC INJURY, AGE 0-17 */
"447" = "1" /* ALLERGIC REACTIONS, AGE GREATER THAN 17 */
"448" = "1" /* ALLERGIC REACTIONS, AGE 0-17 */
"449" = "1" /* POISONING AND TOXIC EFFECTS OF DRUGS, AGE GREATER THAN 17 WITH CC */
"450" = "1" /* POISONING AND TOXIC EFFECTS OF DRUGS, AGE GREATER THAN 17 WITHOUT CC */
"451" = "1" /* POISONING AND TOXIC EFFECTS OF DRUGS, AGE 0-17 */
"452" = "1" /* COMPLICATIONS OF TREATMENT WITH CC */
"453" = "1" /* COMPLICATIONS OF TREATMENT WITHOUT CC */
"454" = "1" /* OTHER INJURY, POISONING AND TOXIC EFFECT DIAGNOSES WITH CC */
"455" = "1" /* OTHER INJURY, POISONING AND TOXIC EFFECT DIAGNOSES WITHOUT CC */
"456" = "1" /* NO LONGER VALID */
"457" = "1" /* NO LONGER VALID */
"460" = "1" /* NO LONGER VALID */
"462" = "1" /* REHABILITATION */
"463" = "1" /* SIGNS AND SYMPTOMS WITH CC */
"464" = "1" /* SIGNS AND SYMPTOMS WITHOUT CC */
"465" = "1" /* AFTERCARE WITH HISTORY OF MALIGNANCY AS SECONDARY DIAGNOSIS */
"466" = "1" /* AFTERCARE WITHOUT HISTORY OF MALIGNANCY AS SECONDARY DIAGNOSIS */
"467" = "1" /* OTHER FACTORS INFLUENCING HEALTH STATUS */
"473" = "1" /* ACUTE LEUKEMIA WITHOUT MAJOR OR PROCEDURE, AGE GREATER THAN 17 */
"475" = "1" /* RESPIRATORY SYSTEM DIAGNOSIS WITH VENTILATOR SUPPORT */
"487" = "1" /* OTHER MULTIPLE SIGNIFICANT TRAUMA */
"489" = "1" /* HIV WITH MAJOR RELATED CONDITION */
"490" = "1" /* HIV WITH OR WITHOUT OTHER RELATED CONDITION */
"492" = "1" /* CHEMOTHERAPY WITH ACUTE LEUKEMIA AS SECONDARY DIAGNOSIS */
"505" = "1" /* EXTENSIVE 3RD DEGREE BURNS W/O SKIN GRAFT */
"508" = "1" /* FULL THICKNESS BURN W/O SKIN GRFT OR INHAL INJ W CC OR SIG TRAUMA */
"509" = "1" /* FULL THICKNESS BURN W/O SKIN GRFT OR INH INJ W/O CC OR SIG TRAUMA */
"510" = "1" /* NON-EXTENSIVE BURNS W CC OR SIGNIFICANT TRAUMA */
"511" = "1" /* NON-EXTENSIVE BURNS W/O CC OR SIGNIFICANT TRAUMA */
"521" = "1" /* ALCOHOL/DRUG ABUSE OR DEPENDENCE W CC */
"522" = "1" /* ALC/DRUG ABUSE OR DEPEND W REHABILITATION THERAPY W/O CC */
"523" = "1" /* ALC/DRUG ABUSE OR DEPEND W/O REHABILITATION THERAPY W/O CC */
"524" = "1" /* TRANSIENT ISCHEMIA */
"559" = "1" /* ACUTE ISCHEMIC STROKE WITH USE OF THROMBOLYTIC AGENT */
"560" = "1" /* BACTERIAL & TUBERCULOUS INFECTIONS OF NERVOUS SYSTEM */
"561" = "1" /* NON-BACTERIAL INFECTIONS OF NERVOUS SYSTEM EXCEPT VIRAL MENINGITIS */
"562" = "1" /* SEIZURE AGE > 17 W CC */
"563" = "1" /* SEIZURE AGE > 17 W/O CC */
"564" = "1" /* HEADACHES AGE >17 */
"565" = "1" /* RESPIRATORY SYSTEM DIAGNOSIS WITH VENTILATOR SUPPORT 96+ HOURS */
"566" = "1" /* RESPIRATORY SYSTEM DIAGNOSIS WITH VENTILATOR SUPPORT < 96 HOURS */
"571" = "1" /* MAJOR ESOPHAGEAL DISORDERS */
"572" = "1" /* MAJOR GASTROINTESTINAL DISORDERS AND PERITONEAL INFECTIONS */
"574" = "1" /* MAJOR HEMATOLOGIC/IMMUNOLOGIC DIAG EXC SICKLE CELL CRISIS & COAGUL */
"575" = "1" /* SEPTICEMIA W MV96+ HOURS  AGE  >17 */
"576" = "1" /* SEPTICEMIA W/O MV96+ HOURS AGE  >17 */
OTHER  = "0" ;

 /* SURGICAL DRGS                                                                */
 /* USED TO DEFINE PSIS #  1: COMPLICATIONS OF ANESTHESIA                        */
 /*                        3: DECUBITUS ULCER                                    */
 /*                        5: FOREIGN BODY LEFT IN DURING PROCEDURE              */
 /*                        6: IATROGENIC PNEUMOTHORAX                            */
 /*                        7: INFECTION DUE TO MEDICAL CARE                      */
 /*                        8: POST-OP HIP FRACTURE                               */
 /*                        9: POST-OP HEMORRHAGE OR HEMATOMA                     */
 /*                       10: POST-OP PHYSIOLOGIC AND METABOLIC DERANGEMENTS     */
 /*                       11: POST-OP RESPIRATORY FAILURE                        */
 /*                       12: POST-OP PULMONARY EMBOLISM OR DEEP VEIN THROMBOSIS */
 /*                       13: POST-OP SEPSIS                                     */
 /*                       15: TECBNICAL DIFFICULTY WITH PROCEDURE                */
 /*                       16: TRANSFUSION REACTION                               */
 /*                       21: FOREIGN BODY LEFT IN DURING PROCEDURE (AREA)       */
 /*                       22: IATROGENIC PNEUMOTHORAX (AREA)                     */
 /*                       23: INFECTION DUE TO MEDICAL CARE (AREA)               */
 /*                       25: TECBNICAL DIFFICULTY WITH PROCEDURE (AREA)         */
 /*                       26: TRANSFUSION REACTION (AREA)                        */
 /*                       27: POST-OP HEMORRHAGE OR HEMATOMA (AREA)              */
 /* DIAGNOSTIC RELATED GROUPS (DRGS): */

VALUE $SURGIDR
"001" = "1" /* CRANIOTOMY, AGE GREATER THAN 17 EXCEPT FOR TRAUMA */
"002" = "1" /* CRANIOTOMY FOR TRAUMA, AGE GREATER THAN 17 */
"003" = "1" /* CRANIOTOMY, AGE 0-17 */
"004" = "1" /* SPINAL PROCEDURES */
"005" = "1" /* EXTRACRANIAL VASCULAR PROCEDURES */
"006" = "1" /* CARPAL TUNNEL RELEASE */
"007" = "1" /* PERIPHERAL AND CRANIAL NERVE AND OTHER NERVOUS SYSTEM PROCEDURES WITH CC */
"008" = "1" /* PERIPHERAL AND CRANIAL NERVE AND OTHER NERVOUS SYSTEM PROCEDURES WITHOUT CC */
"036" = "1" /* RETINAL PROCEDURES */
"037" = "1" /* ORBITAL PROCEDURES */
"038" = "1" /* PRIMARY IRIS PROCEDURES */
"039" = "1" /* LENS PROCEDURES WITH OR WITHOUT VITRECTOMY */
"040" = "1" /* EXTRAOCULAR PROCEDURES EXCEPT ORBIT, AGE GREATER THAN 17 */
"041" = "1" /* EXTRAOCULAR PROCEDURES EXCEPT ORBIT, AGE 0-17 */
"042" = "1" /* INTRAOCULAR PROCEDURES EXCEPT RETINA, IRIS AND LENS */
"049" = "1" /* MAJOR HEAD AND NECK PROCEDURES */
"050" = "1" /* SIALOADENECTOMY */
"051" = "1" /* SALIVARY GLAND PROCEDURES EXCEPT SIALOADENECTOMY */
"052" = "1" /* CLEFT LIP AND PALATE REPAIR */
"053" = "1" /* SINUS AND MASTOID PROCEDURES, AGE GREATER THAN 17 */
"054" = "1" /* SINUS AND MASTOID PROCEDURES, AGE 0-17 */
"055" = "1" /* MISCELLANEOUS EAR, NOSE, MOUTH AND THROAT PROCEDURES */
"056" = "1" /* RHINOPLASTY */
"057" = "1" /* TONSILLECTOMY AND ADENOIDECTOMY PROCEDURES EXCEPT TONSILLECTOMY AND/OR ADENOIDECTOMY ONLY, AGE GREATER THAN 17 */
"058" = "1" /* TONSILLECTOMY AND ADNOIDECTOMY PROCEDURES EXCEPT TONSILLECTOMY AND/OR ADENOIDECTOMY ONLY, AGE 0-17 */
"059" = "1" /* TONSILLECTOMY AND/OR ADENOIDECTOMY ONLY, AGE GREATER THAN 17 */
"060" = "1" /* TONSILLECTOMY AND/OR ADENOIDECTOMY ONLY, AGE 0 - 17 */
"061" = "1" /* MYRINGOTOMY WITH TUBE INSERTION, AGE GREATER THAN 17 */
"062" = "1" /* MYRINGOTOMY WITH TUBE INSERTION, AGE 0-17 */
"063" = "1" /* OTHER EAR, NOSE, MOUTH AND THROAT OR PROCEDURES */
"075" = "1" /* MAJOR CHEST PROCEDURES */
"076" = "1" /* OTHER RESPIRATORY SYSTEM OR PROCEDURES WITH CC */
"077" = "1" /* OTHER RESPIRATORY SYSTEM OR PROCEDURES WITHOUT CC */
"103" = "1" /* HEART TRANSPLANT */
"104" = "1" /* CARDIAC VALVE AND OTHER MAJOR CARDIOTHORACIC PROCEDURES WITH CARDIAC CATHETERIZATION */
"105" = "1" /* CARDIAC VALVE AND OTHER MAJOR CARDIOTHORACIC PROCEDURES WITHOUT CARDIAC CATHETERIZATION */
"106" = "1" /* CORONARY BYPASS WITH PTCA */
"107" = "1" /* CORONARY BYPASS WITH CARDIAC CATHETERIZATION */
"108" = "1" /* OTHER CARDIOTHORACIC PROCEDURES */
"109" = "1" /* CORONARY BYPASS WITHOUT CARDIAC CATHETERIZATION */
"110" = "1" /* MAJOR CARDIOVASCULAR PROCEDURES WITH CC */
"111" = "1" /* MAJOR CARDIOVASCULAR PROCEDURES WITHOUT CC */
"112" = "1" /* PERCUTANEOUS CARDIOVASCULAR PROCEDURES */
"113" = "1" /* AMPUTATION FOR CIRCULATORY SYSTEM DISORDERS EXCEPT UPPER LIMB AND TOE */
"114" = "1" /* UPPER LIMB AND TOES AMPUTATION FOR CIRCULATORY SITE */
"115" = "1" /* PERMANENT CARDIAC PACEMAKER IMPLANT WITH ACUTE MYOCARDIAL INFARCTION, HEART FAILURE OR SHOCK OR ACID LEAD OR GENERATOR PROCEDURE */
"116" = "1" /* OTHER PERMANENT CARDIAC PACEMAKER IMPLANT OR PTCA WITH CORONARY ARTERIAL STENT */
"117" = "1" /* CARDIAC PACEMAKER REVISION EXCEPT DEVICE REPLACEMENT */
"118" = "1" /* CARDIAC PACEMAKER DEVICE REPLACEMENT */
"119" = "1" /* VEIN LIGATION AND STRIPPING */
"120" = "1" /* OTHER CIRCULATORY SYSTEM OR PROCEDURES */
"146" = "1" /* RECTAL RESECTION WITH CC */
"147" = "1" /* RECTAL RESECTION WITHOUT CC */
"148" = "1" /* MAJOR SMALL AND LARGE BOWEL PROCEDURES WITH CC */
"149" = "1" /* MAJOR SMALL AND LARGE BOWEL PROCEDURES WITHOUT CC */
"150" = "1" /* PERITONEAL ADHESIOLYSIS WITH CC */
"151" = "1" /* PERITONEAL ADHESIOLYSIS WITHOUT CC */
"152" = "1" /* MINOR SMALL AND LARGE BOWEL PROCEDURES WITH CC */
"153" = "1" /* MINOR SMALL AND LARGE BOWEL PROCEDURES WITHOUT CC */
"154" = "1" /* STOMACH, ESOPHAGEAL AND DUODENAL PROCEDURES, AGE GREATER THAN 17 WITH CC */
"155" = "1" /* STOMACH, ESOPHAGEAL AND DUODENAL PROCEDURES, AGE GREATER THAN 17 WIHOUT CC*/
"156" = "1" /* STOMACH, ESOPHAGEAL AND DUODENAL PROCEDURES, AGE 0-17 */
"157" = "1" /* ANAL AND STOMAL PROCEDURES WITH CC */
"158" = "1" /* ANAL AND STOMAL PROCEDURES WITHOUT CC */
"159" = "1" /* HERMIA PROCEDURES EXCEPT INGUINAL AND FEMORAL , AGE GREATER THAN 17 WITH CC */
"160" = "1" /* HERNIA PROCEDURES EXCEPT INGUINAL AND FEMORAL, AGE GREATER THAN 17 WITHOUT CC */
"161" = "1" /* INGUINAL AND FEMORAL HERNIA PROCEDURES, AGE GREATER THAN 17 WITH CC */
"162" = "1" /* INGUINAL AND FEMORAL HERNIA PROCEDURES, AGE GREATER THAN 17 WITHOUT CC */
"163" = "1" /* HERNIA PROCEDURES, AGE 0-17 */
"164" = "1" /* APPENDECTOMY WITH COMPLICATED PRINCIPAL DIAGNOSIS WITH CC */
"165" = "1" /* APPENDECTOMY WITH COMPLICATED PRINCIPAL DIAGNOSIS WIHTOUT CC */
"166" = "1" /* APPENDECTOMY WITHOUT COMPLICATED PRINCIPAL IAGNOSIS WITH CC */
"167" = "1" /* APPENDECTOMY WITHOUT COMPLICATED PRINCIPAL DIAGNOSIS WITHOUT CC */
"168" = "1" /* MOUTH PROCEDURES WITH CC */
"169" = "1" /* MOUTH PROCEDURES WITHOUT CC */
"170" = "1" /* OTHER DIGESTIVE SYSTEM OR PROCEDURES WITH CC */
"171" = "1" /* OTHER DIGESTIVE SYSTEM OR PROCEDURES WITHOUT CC */
"191" = "1" /* PANCREAS, LIVER AND SHUNT PROCEDURES WITH CC */
"192" = "1" /* PANCREAS, LIVER AND SHUNT PROCEDURES WITHOUT CC */
"193" = "1" /* BILIARY TRACT PROCEDURES EXCEPT ONLY CHOLECYSTECTOIMY WITH OR WITHOUT COMMON DUCT EXPLORATION WITH CC */
"194" = "1" /* BILIARY TRACT PROCEDURES EXCEPT ONLY CHOLECYSTECTOMY WITH OR WITHOUT COMMON DUCT EXPLORATION WITHOUT CC */
"195" = "1" /* CHOLECYSTECTOMY WITH COMMON DUCT EXPLORATION WITH CC */
"196" = "1" /* CHOLECYSTECTOMY WITH COMMON DUCT EXPLORATION WITHOUT CC */
"197" = "1" /* CHOLECYSTECTOMY EXCEPT BY LAPAROSCOPE WITHOUT COMMON DUCT EXPLORATION WITH CC */
"198" = "1" /* CHOLECYSTECTOMY EXCEPT BY LAPAROSCOPE WITHOUT COMMON DUCT EXPORTATION WITHOUT CC */
"199" = "1" /* HEPATOBILIARY DIAGNOSTIC PROCEDURE FOR MALIGNANCY */
"200" = "1" /* HEPATOBILIARY DIAGNOSTIC PROCEDURE FOR NONMALIGNANCY */
"201" = "1" /* OTHER HEPATOBILIARY OR PANCREAS OR PROCEDURES */
"209" = "1" /* MAJOR JOINT AND LIMB REATTACHMENT PROCEDURES OF LOWER EXTREMITY */
"210" = "1" /* HIP AND FEMUR PROCEDURES EXCEPT MAJOR JOINT PROCEDURES, AGE GREATER THAN 17 WITH CC */
"211" = "1" /* HIP AND FEMUR PROCEDURES EXCEPT MAJOR JOINT PROCEDURES, AGE GREATER THAN 17 WITHOUT CC */
"212" = "1" /* HIP AND FEMUR PROCEDURES EXCEPT MAJOR JOINT PROCEDURE, AGE 0-17 */
"213" = "1" /* AMPUTATION FOR MUSCULOSKELETAL SYSTEM AND CONNECTIVE TISSUE DISORDERS */
"214" = "1" /* NO LONGER VALID */
"215" = "1" /* NO LONGER VALID */
"216" = "1" /* BIOPSIES OF MUSCULOSKELETAL SYSTEM AND CONNECTIVE TISSUE */
"217" = "1" /* WOUND DEBRIDEMENT AND SKIN GRAFT EXCEPT HAND FOR MUSCULOSKELETAL AND CONNECTIVE TISSUE DISORDERS */
"218" = "1" /* LOWER EXTREMITY AND HUMERUS PROCEURES EXCEPT HIP, FOOT AND FEMUR, AGE GREATER THAN 17 WITH CC */
"219" = "1" /* LOWER EXTREMITY AND HUMERUS PROCEDURES EXCEPT HIP, FOOT AND FEMUR, AGE GREATER THAN 17 WITHOUT CC */
"220" = "1" /* LOWER EXTREMITY AND HUMERUS PROCEDURES EXCEPT HIP, FOOT AND FEMUR, AGE 0-17 */
"221" = "1" /* NO LONGER VALID */
"222" = "1" /* NO LONGER VALID */
"223" = "1" /* MAJOR SHOULDER/ELBOW PROCEDURES OR OTHER UPPER EXTREMITY PROCEDURES WITH CC */
"224" = "1" /* SHOULDER, ELBOW OR FOREARM PROCEDURES EXCEPT MAJOR JOINT PROCEDURES WITHOUT CC */
"225" = "1" /* FOOT PROCEDURES*/
"226" = "1" /* SOFT TISSUE PROCEDURES WITH CC */
"227" = "1" /* SOFT TISSUE PROCEDURES WITHOUT CC */
"228" = "1" /* MAJOR THUMB OR JOINT PROCEDURES OR OTHER HAND OR WRIST PROCEDURES WITH CC */
"229" = "1" /* HAND OR WRIST PROCEDURES EXCEPT MAJOR JOINT PROCEDURES WITHOUT CC */
"230" = "1" /* LOCAL EXCISION AND REMOVAL OF INTERNAL FIXATION DEVICES OF HIP AND FEMUR */
"231" = "1" /* LOCAL EXCISION AND REMOVAL OF INTERNAL FIXATION DEVICES EXCEPT HIP AND FEMUR */
"232" = "1" /* ARTHROSCOPY */
"233" = "1" /* OTHER MUSCULOSKELETAL SYSTEM AND CONNECTIVE TISSUE OR PROCEDURES WITH CC */
"234" = "1" /* OTHER MUSCULOSKELETAL SYSTEM AND CONNECTIVE TISSUE OR PROCEDURES WITHOUT CC */
"257" = "1" /* TOTAL MASTECTOMY FOR MALIGNANCY WITH CC */
"258" = "1" /* TOTAL MASTECTOMY FOR MALIGNANCY WITHOUT CC */
"259" = "1" /* SUBTOTAL MASTECTOMY FOR MALIGNANCY WITH CC */
"260" = "1" /* SUBTOTAL MASTECTOMY FOR MALIGNANCY WITHOUT CC */
"261" = "1" /* BREAST PROCEDURE FOR NONMALIGNANCY EXCEPT BIOPSY AND LOCAL EXCISION */
"262" = "1" /* BREAST BIOPSY AND LOCAL EXCISION FOR NONMALIGNANCY */
"263" = "1" /* SKIN GRAFT AND/OR DEBRIDEMENT FOR SKIN ULCER OR CELLULITIS WITH CC */
"264" = "1" /* SKIN GRAFT AND OR DEBRIDEMENT FOR SKIN ULCER OR CELLULITIS WITHOUT CC */
"265" = "1" /* SKIN GRAFT AND OR DEBRIDEMENT EXCEPT FOR SKIN ULCER OR CELLULITIS WITH CC */
"266" = "1" /* SKIN GRAFT AND/OR DEBRIDEMENT EXCEPT FOR SKIN ULCER OR CELLUTLITIES WITHOUT CC */
"267" = "1" /* PERIANAL AND PILONIDAL PROCEDURES */
"268" = "1" /* SKIN, SUBCUTANEOUS TISSUE AND BREAST PLASTIC PROCEDURES */
"269" = "1" /* OTHER SKIN, SUBCUTANEOUS TISSUE AND BREAST PROCEDURES WITH CC */
"270" = "1" /* OTHER SKIN, SUBCUTANEOUS TISSUE AND BREAST PROCEDURS WITHOUT CC */
"285" = "1" /* AMPUTAETION OF LOWER LIMB FOR ENDOCRINE, NUTRITIONAL AND METABOLIC DISORDERS */
"286" = "1" /* ADRENAL AND PITUITARY PROCEDURES */
"287" = "1" /* SKIN GRAFTS AND WOUND DEBRIDEMENTS FOR ENDOCRINE, NUTRITIONAL AND METABOLIC DISORDERS */
"288" = "1" /* OR PROCEDURES FOR OBESITY */
"289" = "1" /* PARATHYROID PROCEDURES */
"290" = "1" /* THYROID PROCEDURES */
"291" = "1" /* THYROGLOSSAL PROCEDURES */
"292" = "1" /* OTHER ENDOCRINE, NUTRITIONAL AND METABOLIC OR PROCEDURES WITH CC */
"293" = "1" /* OTHER ENDOCRINE, NUTRITIONAL AND METABOLIC OR PROCEDURES WITHOUT CC */
"302" = "1" /* KIDNEY TRANSPLANT */
"303" = "1" /* KIDNEY, URETER AND MAJOR BLADDER PROCEDURES FOR NEOPLASM */
"304" = "1" /* KIDNEY, URETER AND MAJOR BLADDER PROCEDURES FOR NONNEOPLASMS WITH CC */
"305" = "1" /* KIDNEY, URETER AND MAJOR BLADDER PROCEDURES FOR NONEOPLSMS WITHOUT CC */
"306" = "1" /* PROSTATECTOMY WITH CC */
"307" = "1" /* PROSTATECTOMY WITHOUT CC */
"308" = "1" /* MINOR BLADDER PROCEDURES WITH CC */
"309" = "1" /* MINOR BLADDER PROCEDURES WITHOUT CC */
"310" = "1" /* TRANSURETHRAL PROCEDURES WITH CC */
"311" = "1" /* TRANSURETHRAL PROCEDURES WITHOUT CC */
"312" = "1" /* URETHRAL PROCEDURES, AGE GREATER THAN 17 WITH CC */
"313" = "1" /* URETHRAL PROCEDURES, AGE GREATER THAN 17 WITHOUT CC */
"314" = "1" /* URETHRAL PROCEDURES, AGE 0-17 */
"315" = "1" /* OTHER KIDNEY AND URINARY TRACT OR PROCEDURES */
"334" = "1" /* MAJOR MALE PELVIC PROCEDURES WITH CC */
"335" = "1" /* MAJOR MALE PELVIC PROCEDURES WITHOUT CC */
"336" = "1" /* TRANSURETHRAL PROSTATECTOMY WITH CC */
"337" = "1" /* TRANSURETHRAL PROSTATECTOMY WITHOUT CC */
"338" = "1" /* TESTES PROCEDURES FOR MALIGNANCY */
"339" = "1" /* TESTES PROCEDURES FOR NONMALIGNANCY, AGE GREATER THAN 17 */
"340" = "1" /* TESTES PROCEDURES FOR NONMALIGNANCY, AGE 0-17 */
"341" = "1" /* PENIS PROCEDURES */
"342" = "1" /* CIRCUMCISION, AGE GREATER THAN 17 */
"343" = "1" /* CIRCUMCISION, AGE 0-17 */
"344" = "1" /* OTHER MALE REPRODUCTIVE SYSTEM OR PROCEDURES FOR MALIGNANCY */
"345" = "1" /* OTHER MALE REPRODUCTIVE SYSTEM OR PROCEDURES EXCEPT FOR MALIGNANCY */
"353" = "1" /* PELVIC EVISCERATION, RADICAL HYSTERECTOMY AND RADICAL VULVECTOMY */
"354" = "1" /* UTERINE AND ADNEXA PROCEDURES FOR NONOVARIAN/ADNEXAL MALIGNANCY WITH CC */
"355" = "1" /* UTERINE AND ADNEXA PROCEDURES FOR NONOVARIAN/ADNEXA PROCEDURES WITHOUT CC*/
"356" = "1" /* FEMALE REPRODUCTIVE SYSTEM RECONSTRUCTIVE PROCEDURES */
"357" = "1" /* UTERINE AND ADNEXA PROCEDURES FOR OVARIAN OR ADNEXAL MALIGNANCY */
"358" = "1" /* UTERINE AND ADNEXA PROCEDURES FOR NONMALIGNANCY WITH CC */
"359" = "1" /* UTERINE AND ADNEXA PROCEDURES FOR NONMALIGNANCY WITHOUT CC */
"360" = "1" /* VAGINA, CERVIX AND VULVA PROCEDURES */
"361" = "1" /* LAPAROSCOPY AND INCISIONAL TUBAL INTERRUPTION */
"362" = "1" /* ENDOSCOPIC TUBAL INTERRUPTION */
"363" = "1" /* D AND C, CONIZATION AND RADIOIMPLANT FOR MALIGNANCY */
"364" = "1" /* D AND C, CONIZATION EXCEPT FOR MALIGNANCY */
"365" = "1" /* OTHER FEMALE REPRODUCTIVE SYSTEM OR PROCEDURES */
"370" = "1" /* CESAREAN SECTION WITH CC */
"371" = "1" /* CESAREAN SECTION WITHOUT CC */
"374" = "1" /* VAGINAL DELIVERY WITH STERILIZATION AND/OR D AND C */
"375" = "1" /* VAGINAL DELIVERY WITH OR PROCEDURE EXCEPT STERILIZATION AND/OR D AND C */
"377" = "1" /* POSTPARTUM AND POSTABORTION DIAGNOSES WITH OR PROCEDURE */
"381" = "1" /* ABORTION WITH D AND C ASPIRATION CURETTAGE OR HYSTERECTOMY */
"392" = "1" /* SPLENECTOMY, AGE GREATER THAN 17 */
"393" = "1" /* SPLENECTOMY, AGE 0-17 */
"394" = "1" /* OTHER OR PROCEDURES OF THE BLOOD AND BLOOD-FORMING ORGANS */
"400" = "1" /* LYMPHOMA AND LEUKEMIA WITH MAJOR OR PROCEDURES */
"401" = "1" /* LYMPHOMA AND NONACUTE LEUKEMIA WITH OTHER OR PROCEDURE WITH CC */
"402" = "1" /* LYMPHOMA AND NONACUTE LEUKEMIA WITH OTHER OR PROCEDURE WITHOUT CC */
"406" = "1" /* MYELOPROLIFERATIVE DISORDERS OR POORLY DIFFERENTIATED NEOPLASMS WITH MAJOR OR PROCEDURES WITH CC */
"407" = "1" /* MYELOPROLIFERATIVE DISORDERS OR POORLY DIFFERENTIATED NEOPLASMS WITH MAJOR OR PROCEDURES WITHOUT CC */
"408" = "1" /* MYELOPROLIFERATIVE DISORDERS OR POORLY DIFFERENTIATED NEOPLASMS WITH OTHER OR PROCEDURES */
"415" = "1" /* OR PROCEDURE FOR INFECTIOUS AND PARASITIC DISEASES */
"424" = "1" /* OR PROCEDURES WITH PRINCIPAL DIAGNOSIS OF MENTAL ILLNESS */
"439" = "1" /* SKIN GRAFTS FOR INJURIES */
"440" = "1" /* WOUND DEBRIDEMENTS FOR INJURIES */
"441" = "1" /* HAND PROCEDURES FOR INJURIES*/
"442" = "1" /* OTHER OR PROCEDURES FOR INJURIES WITH CC */
"443" = "1" /* OTHER OR PROCEDURES FOR INJURIES WITHOUT CC */
"458" = "1" /* NO LONGER VALID */
"459" = "1" /* NO LONGER VALID */
"461" = "1" /* OR PROCEDURES WITH DIAGNOSES OF OTHER CONTACT WITH HEALTH SERVICES */
"468" = "1" /* EXTENSIVE OR PROCEDURE UNRELATED TO PRINCIPAL DIAGNOSIS */
"471" = "1" /* BILATERAL OR MULTIPLE MAJOR JOINT PROCEDURES OF LOWER EXTREMITY */
"472" = "1" /* NO LONGER VALID */
"476" = "1" /* PROSTATIC OR PROCEDURE UNRELATED TO PRINCIPAL DIAGNOSIS */
"477" = "1" /* NONEXTENSIVE OR PROCEDURE UNRELATED TO PRINCIPAL DIAGNOSIS */
"478" = "1" /* OTHER VASCULAR PROCEDURES WITH CC */
"479" = "1" /* OTHER VASCULAR PROCEDURES WITHOUT CC */
"480" = "1" /* LIVER TRANSPLANT */
"481" = "1" /* BONE MARROW TRANSPLANT */
"482" = "1" /* TRACHEOSTOMY FOR FACE, MOUTH AND NECK DIAGNOSES */
"483" = "1" /* TRACHEOSTOMY EXCEPT FOR FACE, MOUTH AND NECK DIAGNOSES */
"484" = "1" /* CRANIOTOMY FOR MULTIPLE SIGNIFICANT TRAUMA */
"485" = "1" /* LIMB REATTACHMENT, HIP AND FEMUR PROCEDURES FOR MULTIPLE SIGNIFICANT TRAUMA */
"486" = "1" /* OTHER OR PROCEDURES FOR MULTIPLE SIGNIFICANT TRAUMA */
"488" = "1" /* HIV WITH EXTENSIVE OR PROCEDURE */
"491" = "1" /* MAJOR JOINT AND LIMB REATTACHMENT PROCEDURES OF UPPER EXTREMITY */
"493" = "1" /* LAPAROSCOPIC CHOLECYSTECTOMY WITHOUT COMMON DUCT EXPLORATION WITH CC */
"494" = "1" /* LAPAROSCOPIC CHOLECYSTECTOMY WITHOUT COMMON DUCT EXPLORATION WITHOUT CC */
"495" = "1" /* LUNG TRANSPLANT */
"496" = "1" /* COMBINED ANTERIOR/POSTERIOR SPINAL FUSION */
"497" = "1" /* SPINAL FUSION WITH CC */
"498" = "1" /* SPINAL FUSION WITHOUT CC */
"499" = "1" /* BACK AND NECK PROCEDURES EXCEPT SPINAL FUSION WITH CC */
"500" = "1" /* BACK AND NECK PROCEDURES EXCEPT SPINAL FUSION WITHOUT CC */
"501" = "1" /* KNEE PROCEDURES WITH PRINCIPAL DIAGNOSIS OF INFECTION, WITH CC */
"502" = "1" /* KNEE PROCEDURES WITH PRINCIPAL DIAGNOSIS OF INFECTION, WITHOUT CC */
"503" = "1" /* KNEE PROCEDURES WITHOUT PRINCIPAL DIAGNOSIS OF INFECTION */
"504" = "1" /* EXTENSIVE 3RD DEGREE BURNS W SKIN GRAFT */
"506" = "1" /* FULL THICKNESS BURN W SKIN GRAFT OR INHAL INJ W CC OR SIG TRAUMA */
"507" = "1" /* FULL THICKNESS BURN W SKIN GRFT OR INHAL INJ W/O CC OR SIG TRAUMA */
"512" = "1" /* SIMULTANEOUS PANCREAS/KIDNEY TRANSPLANT */
"513" = "1" /* PANCREAS TRANSPLANT */
"514" = "1" /* CARDIAC DEFIBRILLATOR IMPLANT W CARDIAC CATH */
"515" = "1" /* CARDIAC DEFIBRILLATOR IMPLANT WITHOUT CARDIAC CATH */
"516" = "1" /* PERCUTANEOUS CARDIOVASC PROC W AMI */
"517" = "1" /* PERC CARDIO PROC W NON-DRUG ELUTING STENT WITHOUT AMI */
"518" = "1" /* PERC CARDIO PROC WITHOUT CORONARY ARTERY STENT OR AMI */
"519" = "1" /* CERVICAL SPINAL FUSION W CC */
"520" = "1" /* CERVICAL SPINAL FUSION WITHOUT CC */
"525" = "1" /* HEART ASSIST SYSTEM IMPLANT */
"526" = "1" /* PERCUTNEOUS CARDIOVASULAR PROC W DRUG ELUTING STENT W AMI */
"527" = "1" /* PERCUTNEOUS CARDIOVASULAR PROC W DRUG ELUTING STENT WITHOUT AMI */
"528" = "1" /* INTRACRANIAL VASCULAR PROC W PDX HEMORRHAGE */
"529" = "1" /* VENTRICULAR SHUNT PROCEDURES W CC */
"530" = "1" /* VENTRICULAR SHUNT PROCEDURES W/O CC */
"531" = "1" /* SPINAL PROCEDURES W CC */
"532" = "1" /* SPINAL PROCEDURES W/O CC */
"533" = "1" /* EXTRACRANIAL PROCEDURES W CC */
"534" = "1" /* EXTRACRANIAL PROCEDURES W/O CC */
"535" = "1" /* CARDIAC DEFIB IMPLANT W CARDIAC CATH W AMI/HF/SHOCK */
"536" = "1" /* CARDIAC DEFIB IMPLANT W CARDIAC CATH W/O AMI/HF/SHOCK */
"537" = "1" /* LOCAL EXCIS & REMOV OF INT FIX DEV EXCEPT HIP & FEMUR W CC */
"538" = "1" /* LOCAL EXCIS & REMOV OF INT FIX DEV EXCEPT HIP & FEMUR W/O CC */
"539" = "1" /* LYMPHOMA & LEUKEMIA W MAJOR OR PROCEDURE W CC */
"540" = "1" /* LYMPHOMA & LEUKEMIA W MAJOR OR PROCEDURE W/O CC */
"541" = "1" /* TRACH W MV 96+HRS OR PDX EXC FACE, MTH, FACE & NECK DX W/MAJ OR */
"542" = "1" /* TRACH W MV 96+HRS OR PDX EXC FACE, MTH, FACE & NECK DX W/O MJ OR */
"543" = "1" /* CRANIOTOMY WITH IMPLANTATION OF CHEMOTHERAPEUTIC AGENT OR ACUTE 
               COMPLEX CENTRAL NERVOUS SYSTEM PRINCIPAL DIAGNOSIS */
"544" = "1" /* MAJOR JOINT REPLACEMENT OR REATTACHMENT OF LOWER EXTREMITY */
"545" = "1" /* REVISION OF HIP OR KNEE REPLACEMENT */
"546" = "1" /* SPINAL FUSION EXC CERV WITH CURVATURE OF THE SPINE OR MALIG */
"547" = "1" /* CORONARY BYPASS W CARDIAC CATH W MAJOR CV DX */
"548" = "1" /* CORONARY BYPASS W CARDIAC CATH W/O MAJOR CV DX */
"549" = "1" /* CORONARY BYPASS W/O CARDIAC CATH W MAJOR CV DX */
"550" = "1" /* CORONARY BYPASS W/O CARDIAC CATH W/O MAJOR CV DX */
"551" = "1" /* PERMANENT CARDIAC PACEMAKER IMPL W MAJ CV DX OR AICD LEAD OR GNRTR */
"552" = "1" /* OTHER PERMANENT CARDIAC PACEMAKER IMPLANT W/O MAJOR CV DX */
"553" = "1" /* OTHER VASCULAR PROCEDURES W CC W MAJOR CV DX */
"554" = "1" /* OTHER VASCULAR PROCEDURES W CC W/O MAJOR CV DX */
"555" = "1" /* PERCUTANEOUS CARDIOVASCULAR PROC W MAJOR CV DX */
"556" = "1" /* PERCUTANEOUS CARDIOVASC PROC W NON-DRUG-ELUTING STENT W/O MAJ CV DX */
"557" = "1" /* PERCUTANEOUS CARDIOVASCULAR PROC W DRUG-ELUTING STENT W MAJOR CV DX */
"558" = "1" /* PERCUTANEOUS CARDIOVASCULAR PROC W DRUG-ELUTING STENT W/O MAJ CV DX */
"567" = "1" /* STOMACH, ESOPHAGEAL & DUODENAL PROC AGE > 17 W CC W MAJOR GI DX */
"568" = "1" /* STOMACH, ESOPHAGEAL & DUODENAL PROCEDURES PROC AGE > 17 W CC W/O MAJOR GI DX */
"569" = "1" /* MAJOR SMALL & LARGE BOWEL PROCEDURES W CC W MAJOR GI DX */
"570" = "1" /* MAJOR SMALL & LARGE BOWEL PROCEDURES W CC W/O MAJOR GI DX */
"573" = "1" /* MAJOR BLADDER PROCEDURES */
"577" = "1" /* CAROTID ARTERY STENT PROCEDURE */
"578" = "1" /* INFECTIOUS & PARASITIC DISEASES W OR PROCEDURE */
"579" = "1" /* POSTOPERATIVE OR POST-TRAUMATIC INFECTIONS W OR PROCEDURE */
OTHER = "0" ;

 /* ------------------------------ */
 /* COMPLICATIONS OF ANESTHESIA    */
 /*   PROVIDER INDICATOR / PSI # 1 */
 /* ------------------------------ */

 /* ICD-9-CM DIAGNOSIS CODES: */

VALUE $ANES1ID
"9681"  = "1" /* HALOTHANE */
"9682"  = "1" /* OTHER GASEOUS ANESTHETICS */
"9683"  = "1" /* INTRAVENEOUS ANESTHETICS */
"9684"  = "1" /* OTHER AND UNSPECIFIED GENERAL ANESTHETICS */
"9687"  = "1" /* SPINAL ANESTHETICS */
"E8551" = "1" /* OTHER NERVOUS SYSTEM DEPRESSANTS */
OTHER   = "0" ;

VALUE $ANES2ID
"E8763" = "1" /* OTHER AND UNSPECIFIED MISADVENTURES DURING MEDICAL CARE,   ENDOTRACHEAL TUBE WRONGLY PLACED DURING ANESTHETIC   PROCEDURE */
"E9381" = "1" /* HALOTHANE */
"E9382" = "1" /* OTHER GASEOUS ANESTHETICS */
"E9383" = "1" /* INTRAVENOUS ANESTHETICS */
"E9384" = "1" /* OTHER AND UNSPECIFIED GENERAL ANESTHETICS */
"E9385" = "1" /* SURFACE AND INFILTRATION ANESTHETICS */
"E9386" = "1" /* PERIPHERAL NERVE AND PLEXUS BLOCKING ANESTHETICS */
"E9387" = "1" /* SPINAL ANESTHETICS */
"E9389" = "1" /* OTHER AND UNSPECIFIED LOCAL ANESTHETICS */
OTHER   = "0" ;

 /* EXCLUDE: ACTIVE DRUG DEPENDENCE */
 /* ICD-9-CM DIAGNOSIS CODES: */

VALUE $ADRUGID
"30400" = "1" /* OPIOID TYPE DEPENDENCE-UNSPECIFIED */
"30401" = "1" /* OPIOID TYPE DEPENDENCE-CONTINUOUS */
"30402" = "1" /* OPIOID TYPE DEPENDENCE-EPISODIC */
"30410" = "1" /* BARBITURATE AND SIMILARLY ACTING SEDATIVE OR HYPNOTIC DEPENDENCE-UNSPECIFIED */
"30411" = "1" /* BARBITURATE AND SIMILARLY ACTING SEDATIVE OR HYPNOTIC DEPENDENCE-CONTINUOUS */
"30412" = "1" /* BARBITURATE AND SIMILARLY ACTING SEDATIVE OR HYPNOTIC DEPENDENCE-EPISODIC */
"30420" = "1" /* COCAINE DEPENDENCE-UNSPECIFIED */
"30421" = "1" /* COCAINE DEPENDENCE-CONTINUOUS */
"30422" = "1" /* COCAINE DEPENDENCE-EPISODIC */
"30430" = "1" /* CANNABIS DEPENDENCE UNSPECIFIED */
"30431" = "1" /* CANNABIS DEPENDENCE CONTINUOUS */
"30432" = "1" /* CANNABIS DEPENDENCE EPISODIC */
"30440" = "1" /* AMPHETAMINE AND OTHER PSYCHO STIMULANT DEPENDENCE-UNSPECIFIED */
"30441" = "1" /* AMPHETAMINE AND OTHER PSYCHO STIMULANT DEPENDENCE-CONTINUOUS */
"30442" = "1" /* AMPHETAMINE AND OTHER PSYCHO STIMULANT DEPENDENCE-EPISODIC */
"30450" = "1" /* HALLUCINOGEN DEPENDENCE UNSPCIFIED */
"30451" = "1" /* HALLUCINOGEN DEPENDENCE-CONTINUOUS */
"30452" = "1" /* HALLUCINOGEN DEPENDENCE-EPISODIC */
"30460" = "1" /* OTHER SPECIFIED DRUG DEPENDENCE-UNSPECIFIED */
"30461" = "1" /* OTHER SPECIFIED DRUG DEPENDENCE-CONTINUOUS */
"30462" = "1" /* OTHER SPECIFIED DRUG DEPENDENCE-EPISODIC */
"30470" = "1" /* COMBINATIONS OF OPIOID TYPE DRUG WITH ANY OTHER-UNSPECIFIED */
"30471" = "1" /* COMBINATIONS OF OPIOID TYPE DRUG WITH ANY OTHER-CONTINUOUS */
"30472" = "1" /* COMBINATIONS OF OPIOID TYPE DRUG WITH ANY OTHER-EPISODIC */
"30480" = "1" /* COMBINATIONS OF DRUG EXCLUDING OPIOID TYPE DRUG-UNSPECIFIED */
"30481" = "1" /* COMBINATIONS OF DRUG EXCLUDING OPIOID TYPE DRUG-CONTINUOUS */
"30482" = "1" /* COMBINATIONS OF DRUG EXCLUDING OPIOID TYPE DRUG-EPISODIC */
"30490" = "1" /* UNSPECIFIED DRUG DEPENDENCE -UNSPECIFIED */
"30491" = "1" /* UNSPECIFIED DRUG DEPENDENCE -CONTINUOUS */
"30492" = "1" /* UNSPECIFIED DRUG DEPENDENCE -EPISODIC */
OTHER   = "0" ;

 /* EXCLUDE: ACTIVE NONDEPENDENT ABUSE OF DRUGS */
 /* ICD-9-CM DIAGNOSIS CODES: */

VALUE $ANONDID
"30520" = "1" /* CANNABIS ABUSE-UNSPECIFIED */
"30521" = "1" /* CANNABIS ABUSE-CONTINUOUS */
"30522" = "1" /* CANNABIS ABUSE-EPISODIC */
"30530" = "1" /* HALLUCINOGEN ABUSE- UNSPECIFIED */
"30531" = "1" /* HALLUCINOGEN ABUSE-CONTINUOUS */
"30532" = "1" /* HALLUCINOGEN ABUSE- EPISODIC */
"30540" = "1" /* BARBITURATE AND SIMILARLY ACTING SEDATIVE OR HYPNOTIC ABUSE-UNSPECIFIED */
"30541" = "1" /* BARBITURATE AND SIMILARLY ACTING SEDATIVE OR HYPNOTIC ABUSE-CONTINUOUS */
"30542" = "1" /* BARBITURATE AND SIMILARLY ACTING SEDATIVE OR HYPNOTIC ABUSE-EPISODIC */
"30550" = "1" /* OPIOID ABUSE-UNSPECIFIED */
"30551" = "1" /* OPIOD ABUSE-CONTINUOUS */
"30552" = "1" /* OPIOID ABUSE-EPISODIC */
"30560" = "1" /* COCAINE ABUSE-UNSPECIFIED */
"30561" = "1" /* COCAINE ABUSE-CONTINUOUS */
"30562" = "1" /* COCAINE ABUSE-EPISODIC */
"30570" = "1" /* AMPHETAMINE OR RELATED ACTING SYMPATHOMIMETIC ABUSE-UNSPECIFIED */
"30571" = "1" /* AMPHETAMINE OR RELATED ACTING SYMPATHOMIMETIC ABUSE-CONTINUOUS */
"30572" = "1" /* AMPHETAMINE OR RELATED ACTING SYMPATHOMIMETIC ABUSE- EPISODIC */
"30580" = "1" /* ANTIDEPRESSANT TYPE ABUSE-UNSPECIFIED */
"30581" = "1" /* ANTIDEPRESSANT TYPE ABUSE-CONTINUOUS */
"30582" = "1" /* ANTIDEPRESSANT TYPE ABUSE-EPISODIC */
"30590" = "1" /* OTHER, MIXED, OR UNSPECIFIED DRUG ABUSE-UNSPECIFIED */
"30591" = "1" /* OTHER, MIXED, OR UNSPECIFIED DRUG ABUSE-CONTINUOUS */
"30592" = "1" /* OTHER, MIXED, OR UNSPECIFIED DRUG ABUSE-EPISODIC */
OTHER   = "0" ;

 /* EXCLUDE: SELF-INFLICTED INJURY */
 /* ICD-9-CM DIAGNOSIS CODES: */

VALUE $SELFIID
"E9500" = "1" /* ANALGESICS, ANTIPYRETICS, AND ANTIRHEUMATICS */
"E9501" = "1" /* BARBITURATES */
"E9502" = "1" /* OTHER SEDATIVE AND HYPNOTICS */
"E9503" = "1" /* TRANQUILIZIERS AND OTHER PSYCHOTROPIC AGENTS */
"E9504" = "1" /* OTHER SPECIFIED DRUGS AND MEDICINAL SUBSTANCES */
"E9505" = "1" /* UNSPECIFIED DRUG OR MEDICINAL SUBSTANCE */
"E9506" = "1" /* AGRICULTURAL AND HORTICULTURAL CHEMICAL AND PHARMACEUTICAL PREPARATIONS OTHER THAN PLANT FOODS AND FERTILIZERS */
"E9507" = "1" /* CORROSIVE AND CAUSTIC SUBSTANCES */
"E9508" = "1" /* ARSENIC AND ITS COMPOUNDS */
"E9509" = "1" /* OTHER AND UNSPECIFIED SOLID AND LIQUID SUBSTANCES */
"E9510" = "1" /* GAS DISTRIBUTED BY PIPELINE */
"E9511" = "1" /* LIQUEFIED PETROLEUM GAS DISTRIBUTED IN MOBILE CONTAINERS */
"E9518" = "1" /* OTHERS UTILITY GASES */
"E9520" = "1" /* MOTOR VEHICLE EXHAUST GAS */
"E9521" = "1" /* OTHER CARBON MONOXIDE */
"E9528" = "1" /* OTHER SPECIFIED GASES AND VAPORS */
"E9529" = "1" /* UNSPECIFIED GASES AND VAPORS */
"E9530" = "1" /* HANGING */
"E9531" = "1" /* SUFFOCATION BY PLASTIC BAG */
"E9538" = "1" /* OTHER SPECIFIED MEANS */
"E954"  = "1" /* SUICIDE AND SELF-INFLICTED INJURY BY SUBMERSION [DROWNING] */
"E9550" = "1" /* HANDGUN */
"E9551" = "1" /* SHOTGUN */
"E9552" = "1" /* HUNTING RIFLE */
"E9553" = "1" /* MILITARY FIREARMS */
"E9554" = "1" /* OTHER AND UNSPECIFIED FIREARMS */
"E9555" = "1" /* EXPLOSIVES */
"E9559" = "1" /* UNSPECIFIED */
"E956"  = "1" /* SUICIDE AND SELF INFLICTED INJURY BY CUTTING AND PIERCING INSTRUMENT */
"E9570" = "1" /* RESIDENTIAL PREMISES */
"E9571" = "1" /* OTHER MAN-MADE STRUCTURES */
"E9572" = "1" /* NATURAL SITES */
"E9579" = "1" /* UNSPECIFIED */
"E9580" = "1" /* JUMPING OR LYING BEFORE MOVING OBJECT */
"E9581" = "1" /* BURNS, FIRE */
"E9582" = "1" /* SCALD */
"E9583" = "1" /* EXTREMES OF COLD */
"E9584" = "1" /* ELECTROCUTION */
"E9585" = "1" /* CRASHING OF MOTOR VEHICLE */
"E9586" = "1" /* CRASHING OF AIRCRAFT */
"E9587" = "1" /* CAUSTIC SUBSTANCES EXCEPT POISONING */
"E9588" = "1" /* OTHER SPECIFIED MEANS */
"E9589" = "1" /* UNSPECIFIED MEANS */
OTHER   = "0" ;

 /* ------------------------------ */
 /* DEATH IN LOW MORTALITY DRGS    */
 /*   PROVIDER INDICATOR / PSI # 2 */
 /* ------------------------------ */

 /* DIAGNOSTIC RELATED GROUPS DRGS */

VALUE $LOWMODR
"006" = "1" /* CARPAL TUNNEL RELEASE  */
"021" = "1" /* VIRAL MENINGITIS  */
"036" = "1" /* RETINAL PROCEDURES  */
"037" = "1" /* ORBITAL PROCEDURES  */
"039" = "1" /* LENS PROCEDURES WITH OR WITHOUT VITRECTOMY  */
"040" = "1" /* EXTRAOCULAR PROCEDURES EXCEPT ORBIT AGE >17  */
"042" = "1" /* INTRAOCULAR PROCEDURES EXCEPT RETINA, IRIS & LENS  */
"043" = "1" /* HYPHEMA  */
"044" = "1" /* ACUTE MAJOR EYE INFECTIONS  */
"045" = "1" /* NEUROLOGICAL EYE DISORDERS  */
"049" = "1" /* MAJOR HEAD & NECK PROCEDURES  */
"050" = "1" /* SIALOADENECTOMY  */
"051" = "1" /* SALIVARY GLAND PROCEDURES EXCEPT SIALOADENECTOMY  */
"052" = "1" /* CLEFT LIP & PALATE REPAIR  */
"053" = "1" /* SINUS & MASTOID PROCEDURES AGE >17  */
"055" = "1" /* MISCELLANEOUS EAR, NOSE, MOUTH & THROAT PROCEDURES  */
"056" = "1" /* RHINOPLASTY  */
"057" = "1" /* T&A PROC, EXCEPT TONSILLECTOMY &/OR ADENOIDECTOMY ONLY, */
"059" = "1" /* TONSILLECTOMY &/OR ADENOIDECTOMY ONLY, AGE >17  */
"063" = "1" /* OTHER EAR, NOSE, MOUTH & THROAT O.R. PROCEDURES  */
"065" = "1" /* DYSEQUILIBRIUM  */
"067" = "1" /* EPIGLOTTITIS  */
"068" = "1" /* OTITIS MEDIA & URI AGE >17 W CC  */
"069" = "1" /* OTITIS MEDIA & URI AGE >17 WO CC  */
"072" = "1" /* NASAL TRAUMA & DEFORMITY  */
"096" = "1" /* BRONCHITIS & ASTHMA AGE >17 W CC  */
"097" = "1" /* BRONCHITIS & ASTHMA AGE >17 WO CC  */
"118" = "1" /* CARDIAC PACEMAKER DEVICE REPLACEMENT  */
"125" = "1" /* CIRCULATORY DISORDERS EXCEPT AMI, W CARD CATH WO COMPLEX */
"134" = "1" /* HYPERTENSION  */
"140" = "1" /* ANGINA PECTORIS  */
"141" = "1" /* SYNCOPE & COLLAPSE W CC  */
"142" = "1" /* SYNCOPE & COLLAPSE WO CC  */
"143" = "1" /* CHEST PAIN  */
"166" = "1" /* APPENDECTOMY WO COMPLICATED PRINCIPAL DIAG W CC  */
"167" = "1" /* APPENDECTOMY WO COMPLICATED PRINCIPAL DIAG WO CC  */
"177" = "1" /* UNCOMPLICATED PEPTIC ULCER W CC  */
"178" = "1" /* UNCOMPLICATED PEPTIC ULCER WO CC  */
"179" = "1" /* INFLAMMATORY BOWEL DISEASE  */
"187" = "1" /* DENTAL EXTRACTIONS & RESTORATIONS  */
"223" = "1" /* MAJOR SHOULDER/ELBOW PROC, OR OTHER UPPER EXTREMITY PROC */
"224" = "1" /* SHOULDER,ELBOW OR FOREARM PROC,EXC MAJOR JOINT PROC, WO */
"225" = "1" /* FOOT PROCEDURES  */
"228" = "1" /* MAJOR THUMB OR JOINT PROC,OR OTH HAND OR WRIST PROC W CC */
"229" = "1" /* HAND OR WRIST PROC, EXCEPT MAJOR JOINT PROC, WO CC  */
"231" = "1" /* LOCAL EXCISION & REMOVAL OF INT FIX DEVICES EXCEPT HIP & */
"232" = "1" /* ARTHROSCOPY  */
"237" = "1" /* SPRAINS, STRAINS, & DISLOCATIONS OF HIP, PELVIS & THIGH */
"244" = "1" /* BONE DISEASES & SPECIFIC ARTHROPATHIES W CC  */
"245" = "1" /* BONE DISEASES & SPECIFIC ARTHROPATHIES WO CC  */
"246" = "1" /* NON-SPECIFIC ARTHROPATHIES  */
"247" = "1" /* SIGNS & SYMPTOMS OF MUSCULOSKELETAL SYSTEM & CONN TISSUE */
"257" = "1" /* TOTAL MASTECTOMY FOR MALIGNANCY W CC  */
"258" = "1" /* TOTAL MASTECTOMY FOR MALIGNANCY WO CC  */
"261" = "1" /* BREAST PROC FOR NON-MALIGNANCY EXCEPT BIOPSY & LOCAL EXC */
"262" = "1" /* BREAST BIOPSY & LOCAL EXCISION FOR NON-MALIGNANCY  */
"267" = "1" /* PERIANAL & PILONIDAL PROCEDURES  */
"268" = "1" /* SKIN, SUBCUTANEOUS TISSUE & BREAST PLASTIC PROCEDURES  */
"276" = "1" /* NON-MALIGANT BREAST DISORDERS  */
"288" = "1" /* O.R. PROCEDURES FOR OBESITY  */
"289" = "1" /* PARATHYROID PROCEDURES  */
"290" = "1" /* THYROID PROCEDURES  */
"291" = "1" /* THYROGLOSSAL PROCEDURES  */
"295" = "1" /* DIABETES AGE 0-35  */
"323" = "1" /* URINARY STONES W CC, &/OR ESW LITHOTRIPSY  */
"324" = "1" /* URINARY STONES WO CC  */
"334" = "1" /* MAJOR MALE PELVIC PROCEDURES W CC  */
"335" = "1" /* MAJOR MALE PELVIC PROCEDURES WO CC  */
"336" = "1" /* TRANSURETHRAL PROSTATECTOMY W CC  */
"337" = "1" /* TRANSURETHRAL PROSTATECTOMY WO CC  */
"339" = "1" /* TESTES PROCEDURES, NON-MALIGNANCY AGE >17  */
"341" = "1" /* PENIS PROCEDURES  */
"342" = "1" /* CIRCUMCISION AGE >17  */
"350" = "1" /* INFLAMMATION OF THE MALE REPRODUCTIVE SYSTEM  */
"353" = "1" /* PELVIC EVISCERATION, RADICAL HYSTERECTOMY & RADICAL VULV  */
"356" = "1" /* FEMALE REPRODUCTIVE SYSTEM RECONSTRUCTIVE PROCEDURES  */
"358" = "1" /* UTERINE & ADNEXA PROC FOR NON-MALIGNANCY W CC  */
"359" = "1" /* UTERINE & ADNEXA PROC FOR NON-MALIGNANCY WO CC  */
"360" = "1" /* VAGINA, CERVIX & VULVA PROCEDURES  */
"361" = "1" /* LAPAROSCOPY & INCISIONAL TUBAL INTERRUPTION  */
"362" = "1" /* ENDOSCOPIC TUBAL INTERRUPTION  */
"364" = "1" /* D&C, CONIZATION EXCEPT FOR MALIGNANCY  */
"369" = "1" /* MENSTRUAL & OTHER FEMALE REPRODUCTIVE SYSTEM DISORDERS  */
"370" = "1" /* CESAREAN SECTION W CC  */
"371" = "1" /* CESAREAN SECTION WO CC  */
"372" = "1" /* VAGINAL DELIVERY W COMPLICATING DIAGNOSES  */
"373" = "1" /* VAGINAL DELIVERY WO COMPLICATING DIAGNOSES  */
"374" = "1" /* VAGINAL DELIVERY W STERILIZATION &/OR D&C  */
"375" = "1" /* VAGINAL DELIVERY W O.R. PROC EXCEPT STERIL &/OR D&C   */
"376" = "1" /* POSTPARTUM & POST ABORTION DIAGNOSES WO O.R. PROCEDURE  */
"377" = "1" /* POSTPARTUM & POST ABORTION DIAGNOSES W O.R. PROCEDURE  */
"378" = "1" /* ECTOPIC PREGNANCY  */
"379" = "1" /* THREATENED ABORTION  */
"380" = "1" /* ABORTION WO D&C  */
"381" = "1" /* ABORTION W D&C, ASPIRATION CURETTAGE OR HYSTEROTOMY  */
"382" = "1" /* FALSE LABOR  */
"383" = "1" /* OTHER ANTEPARTUM DIAGNOSES W MEDICAL COMPLICATIONS  */
"384" = "1" /* OTHER ANTEPARTUM DIAGNOSES WO MEDICAL COMPLICATIONS  */
"421" = "1" /* VIRAL ILLNESS AGE >17  */
"425" = "1" /* ACUTE ADJUSTMENT REACTION & PSYCHOSOCIAL DYSFUNCTION   */
"426" = "1" /* DEPRESSIVE NEUROSES  */
"427" = "1" /* NEUROSES EXCEPT DEPRESSIVE  */
"428" = "1" /* DISORDERS OF PERSONALITY & IMPULSE CONTROL  */
"430" = "1" /* PSYCHOSES  */
"431" = "1" /* CHILDHOOD MENTAL DISORDERS  */
"432" = "1" /* OTHER MENTAL DISORDER DIAGNOSES  */
"433" = "1" /* ALCOHOL/DRUG ABUSE OR DEPENDENCE, LEFT AMA  */
"434" = "1" /* ALC/DRUG ABUSE OR DEPEND, DETOX OR OTH SYMPT TREAT W CC */
"435" = "1" /* ALC/DRUG ABUSE OR DEPEND, DETOX OR OTH SYMPT TREAT WO CC */
"436" = "1" /* ALC/DRUG DEPENDENCE W REHABILITATION THERAPY (NO LONGER */
"437" = "1" /* ALC/DRUG DEPENDENCE, COMBINED REHAB & DETOX THERAPY (NO */
"439" = "1" /* SKIN GRAFTS FOR INJURIES  */
"441" = "1" /* HAND PROCEDURES FOR INJURIES  */
"447" = "1" /* ALLERGIC REACTIONS AGE >17  */
"471" = "1" /* BILATERAL OR MULTIPLE MAJOR JOINT PROCS OF LOWER EXTREMI */
"491" = "1" /* MAJOR JOINT & LIMB REATTACHMENT PROCEDURES OF UPPER EXTR */
"496" = "1" /* COMBINED ANTERIOR/POSTERIOR SPINAL FUSION  */
"497" = "1" /* SPINAL FUSION EXCEPT CERVICAL W CC   */
"498" = "1" /* SPINAL FUSION EXCEPT CERVICAL WO CC   */
"499" = "1" /* BACK & NECK PROCEDURES EXCEPT SPINAL FUSION W CC  */
"500" = "1" /* BACK & NECK PROCEDURES EXCEPT SPINAL FUSION WO CC  */
"503" = "1" /* KNEE PROCEDURES WO PDX OF INFECTION  */
"517" = "1" /* PERC CARDIO PROC W NON-DRUG ELUTING STENT WO AMI  */
"518" = "1" /* PERC CARDIO PROC WO CORONARY ARTERY STENT OR AMI   */
"521" = "1" /* ALCOHOL/DRUG ABUSE OR DEPENDENCE W CC  */
"522" = "1" /* ALC/DRUG ABUSE OR DEPEND W REHABILITATION THERAPY WO CC */
"523" = "1" /* ALC/DRUG ABUSE OR DEPEND WO REHABILITATION THERAPY WO CC */
"524" = "1" /* TRANSIENT ISCHEMIA  */
"527" = "1" /* PERCUTNEOUS CARDIOVASULAR PROC W DRUG ELUTING STENT WO A */
"536" = "1" /* CARDIAC DEFIB IMPLANT W CARDIAC CATH WO AMI/HF/SHOCK  */
OTHER = "0" ;

 /* EXCLUDE: OTHER CHEST TRAUMA */
 /* ICD-9-CM DIAGNOSIS CODES ( INCLUDES 4TH AND 5TH DIGITS): */

VALUE $CTRAUMD
"80700" = "1" /* FRACTURE RIB NOS-CLOSED */
"80701" = "1" /* FRACTURE ONE RIB-CLOSED */
"80702" = "1" /* FRACTURE TWO RIBS-CLOSED */
"80703" = "1" /* FRACTURE THREE RIBS-CLOS */
"80704" = "1" /* FRACTURE FOUR RIBS-CLOSE */
"80705" = "1" /* FRACTURE FIVE RIBS-CLOSE */
"80706" = "1" /* FRACTURE SIX RIBS-CLOSED */
"80707" = "1" /* FRACTURE SEVEN RIBS-CLOS */
"80708" = "1" /* FX EIGHT/MORE RIB-CLOSED */
"80709" = "1" /* FX MULT RIBS NOS-CLOSED */
"80710" = "1" /* FRACTURE RIB NOS-OPEN */
"80711" = "1" /* FRACTURE ONE RIB-OPEN */
"80712" = "1" /* FRACTURE TWO RIBS-OPEN */
"80713" = "1" /* FRACTURE THREE RIBS-OPEN */
"80714" = "1" /* FRACTURE FOUR RIBS-OPEN */
"80715" = "1" /* FRACTURE FIVE RIBS-OPEN */
"80716" = "1" /* FRACTURE SIX RIBS-OPEN */
"80717" = "1" /* FRACTURE SEVEN RIBS-OPEN */
"80718" = "1" /* FX EIGHT/MORE RIBS-OPEN */
"80719" = "1" /* FX MULT RIBS NOS-OPEN */
"8072"  = "1" /* FRACTURE OF STERNUM-CLOS */
"8073"  = "1" /* FRACTURE OF STERNUM-OPEN */
"8074"  = "1" /* FLAIL CHEST */
"8075"  = "1" /* FX LARYNX/TRACHEA-CLOSED */
"8076"  = "1" /* FX LARYNX/TRACHEA-OPEN */
"8090"  = "1" /* FRACTURE TRUNK BONE-CLOS */
"8091"  = "1" /* FRACTURE TRUNK BONE-OPEN */
"8600"  = "1" /* TRAUM PNEUMOTHORAX-CLOSE */
"8601"  = "1" /* TRAUM PNEUMOTHORAX-OPEN */
"8602"  = "1" /* TRAUM HEMOTHORAX-CLOSED */
"8603"  = "1" /* TRAUM HEMOTHORAX-OPEN */
"8604"  = "1" /* TRAUM PNEUMOHEMOTHOR-CL */
"8605"  = "1" /* TRAUM PNEUMOHEMOTHOR-OPN */
"86100" = "1" /* HEART INJURY NOS-CLOSED */
"86101" = "1" /* HEART CONTUSION-CLOSED */
"86102" = "1" /* HEART LACERATION-CLOSED */
"86103" = "1" /* HEART CHAMBER LACERAT-CL */
"86110" = "1" /* HEART INJURY NOS-OPEN */
"86111" = "1" /* HEART CONTUSION-OPEN */
"86112" = "1" /* HEART LACERATION-OPEN */
"86113" = "1" /* HEART CHAMBER LACER-OPN */
"86120" = "1" /* LUNG INJURY NOS-CLOSED */
"86121" = "1" /* LUNG CONTUSION-CLOSED */
"86122" = "1" /* LUNG LACERATION-CLOSED */
"86130" = "1" /* LUNG INJURY NOS-OPEN */
"86131" = "1" /* LUNG CONTUSION-OPEN */
"86132" = "1" /* LUNG LACERATION-OPEN */
"8620"  = "1" /* DIAPHRAGM INJURY-CLOSED */
"8621"  = "1" /* DIAPHRAGM INJURY-OPEN */
"86221" = "1" /* BRONCHUS INJURY-CLOSED */
"86222" = "1" /* ESOPHAGUS INJURY-CLOSED */
"86229" = "1" /* INTRATHORACIC INJ NEC-CL */
"86231" = "1" /* BRONCHUS INJURY-OPEN */
"86232" = "1" /* ESOPHAGUS INJURY-OPEN */
"86239" = "1" /* INTRATHORAC INJ NEC-OPEN */
"8628"  = "1" /* INTRATHORACIC INJ NOS-CL */
"8629"  = "1" /* INTRATHORAC INJ NOS-OPEN */
"8750"  = "1" /* OPEN WOUND OF CHEST */
"8751"  = "1" /* OPEN WOUND CHEST-COMPL */
"8760"  = "1" /* OPEN WOUND OF BACK */
"8761"  = "1" /* OPEN WOUND BACK-COMPL */
"9010"  = "1" /* INJURY THORACIC AORTA */
"9011"  = "1" /* INJ INNOMIN/SUBCLAV ART */
"9012"  = "1" /* INJ SUPERIOR VENA CAVA */
"9013"  = "1" /* INJ INNOMIN/SUBCLAV VEIN */
"90140" = "1" /* INJ PULMONARY VESSEL NOS */
"90141" = "1" /* INJURY PULMONARY ARTERY */
"90142" = "1" /* INJURY PULMONARY VEIN */
"90181" = "1" /* INJ INTERCOSTAL ART/VEIN */
"90182" = "1" /* INJ INT MAMMARY ART/VEIN */
"90183" = "1" /* INJ MULT THORACIC VESSEL */
"90189" = "1" /* INJ THORACIC VESSEL NEC */
"9019"  = "1" /* INJ THORACIC VESSEL NOS */
"9110 " = "1" /* ABRASION TRUNK */
"9111 " = "1" /* ABRASION TRUNK-INFECTED */
"9118 " = "1" /* SUPERFIC INJU TRUNK NEC */
"9119 " = "1" /* SUPERFIC INJU TRUNK NEC-INF */
"9220 " = "1" /* CONTUSION OF BREAST */
"9221 " = "1" /* CONTUSION OF CHEST WALL */
"9223 " = "1" /* BACK CONTUSION */
"92231" = "1" /* BACK CONTUSION */
"92233" = "1" /* INTERSCPLR REG CONTUSION */
"9228 " = "1" /* MULIPLE CONTUSION TRUNK */
"9229 " = "1" /* CONTUSION OF TRUNK */
"92611" = "1" /* CRUSHING INJURY BACK */
"92619" = "1" /* CRUSHING INJ TRUNK NEC */
"9268"  = "1" /* MULT CRUSHING INJ TRUNK */
"9269"  = "1" /* CRUSHING INJ TRUNK NOS */
"9290"  = "1" /* CRUSH INJ MULT SITE NEC */
"9299"  = "1" /* CRUSHING INJURY NOS */
"9541 " = "1" /* INJ SYMPA NERVE NEC */
"9548 " = "1" /* INJURY TRUNK NERVE NEC */
"9549 " = "1" /* INJURY TRUNK NERVE NOS */
"95911" = "1" /* INJURY OF CHEST WALL NEC */
"95919" = "1" /* TRUNK INJURY-SITES NEC */
"9599 " = "1" /* INJURY-SITE NOS */
OTHER   = "0" ;

 /* EXCLUDE: TRAUMA */
 /* ICD-9-CM DIAGNOSIS CODES ( INCLUDES 4TH AND 5TH DIGITS): */

VALUE $TRAUMID
"80000" = "1" /* CLOSED SKULL VAULT FX */
"80001" = "1" /* CL SKULL VLT FX W/O COMA */
"80002" = "1" /* CL SKULL VLT FX-BRF COMA */
"80003" = "1" /* CL SKULL VLT FX-MOD COMA */
"80004" = "1" /* CL SKL VLT FX-PROLN COMA */
"80005" = "1" /* CL SKUL VLT FX-DEEP COMA */
"80006" = "1" /* CL SKULL VLT FX-COMA NOS */
"80009" = "1" /* CL SKL VLT FX-CONCUS NOS */
"80010" = "1" /* CL SKL VLT FX/CEREBR LAC */
"80011" = "1" /* CL SKULL VLT FX W/O COMA */
"80012" = "1" /* CL SKULL VLT FX-BRF COMA */
"80013" = "1" /* CL SKULL VLT FX-MOD COMA */
"80014" = "1" /* CL SKL VLT FX-PROLN COMA */
"80015" = "1" /* CL SKUL VLT FX-DEEP COMA */
"80016" = "1" /* CL SKULL VLT FX-COMA NOS */
"80019" = "1" /* CL SKL VLT FX-CONCUS NOS */
"80020" = "1" /* CL SKL VLT FX/MENING HEM */
"80021" = "1" /* CL SKULL VLT FX W/O COMA */
"80022" = "1" /* CL SKULL VLT FX-BRF COMA */
"80023" = "1" /* CL SKULL VLT FX-MOD COMA */
"80024" = "1" /* CL SKL VLT FX-PROLN COMA */
"80025" = "1" /* CL SKUL VLT FX-DEEP COMA */
"80026" = "1" /* CL SKULL VLT FX-COMA NOS */
"80029" = "1" /* CL SKL VLT FX-CONCUS NOS */
"80030" = "1" /* CL SKULL VLT FX/HEM NEC */
"80031" = "1" /* CL SKULL VLT FX W/O COMA */
"80032" = "1" /* CL SKULL VLT FX-BRF COMA */
"80033" = "1" /* CL SKULL VLT FX-MOD COMA */
"80034" = "1" /* CL SKL VLT FX-PROLN COMA */
"80035" = "1" /* CL SKUL VLT FX-DEEP COMA */
"80036" = "1" /* CL SKULL VLT FX-COMA NOS */
"80039" = "1" /* CL SKL VLT FX-CONCUS NOS */
"80040" = "1" /* CL SKL VLT FX/BR INJ NEC */
"80041" = "1" /* CL SKULL VLT FX W/O COMA */
"80042" = "1" /* CL SKULL VLT FX-BRF COMA */
"80043" = "1" /* CL SKULL VLT FX-MOD COMA */
"80044" = "1" /* CL SKL VLT FX-PROLN COMA */
"80045" = "1" /* CL SKUL VLT FX-DEEP COMA */
"80046" = "1" /* CL SKULL VLT FX-COMA NOS */
"80049" = "1" /* CL SKL VLT FX-CONCUS NOS */
"80050" = "1" /* OPN SKULL VAULT FRACTURE */
"80051" = "1" /* OPN SKUL VLT FX W/O COMA */
"80052" = "1" /* OPN SKUL VLT FX-BRF COMA */
"80053" = "1" /* OPN SKUL VLT FX-MOD COMA */
"80054" = "1" /* OPN SKL VLT FX-PROLN COM */
"80055" = "1" /* OPN SKL VLT FX-DEEP COMA */
"80056" = "1" /* OPN SKUL VLT FX-COMA NOS */
"80059" = "1" /* OP SKL VLT FX-CONCUS NOS */
"80060" = "1" /* OPN SKL VLT FX/CEREB LAC */
"80061" = "1" /* OPN SKUL VLT FX W/O COMA */
"80062" = "1" /* OPN SKUL VLT FX-BRF COMA */
"80063" = "1" /* OPN SKUL VLT FX-MOD COMA */
"80064" = "1" /* OPN SKL VLT FX-PROLN COM */
"80065" = "1" /* OPN SKL VLT FX-DEEP COMA */
"80066" = "1" /* OPN SKUL VLT FX-COMA NOS */
"80069" = "1" /* OP SKL VLT FX-CONCUS NOS */
"80070" = "1" /* OPN SKL VLT FX/MENIN HEM */
"80071" = "1" /* OPN SKUL VLT FX W/O COMA */
"80072" = "1" /* OPN SKUL VLT FX-BRF COMA */
"80073" = "1" /* OPN SKUL VLT FX-MOD COMA */
"80074" = "1" /* OPN SKL VLT FX-PROLN COM */
"80075" = "1" /* OPN SKL VLT FX-DEEP COMA */
"80076" = "1" /* OPN SKUL VLT FX-COMA NOS */
"80079" = "1" /* OP SKL VLT FX-CONCUS NOS */
"80080" = "1" /* OPN SKULL VLT FX/HEM NEC */
"80081" = "1" /* OPN SKUL VLT FX W/O COMA */
"80082" = "1" /* OPN SKUL VLT FX-BRF COMA */
"80083" = "1" /* OPN SKUL VLT FX-MOD COMA */
"80084" = "1" /* OPN SKL VLT FX-PROLN COM */
"80085" = "1" /* OPN SKL VLT FX-DEEP COMA */
"80086" = "1" /* OPN SKUL VLT FX-COMA NOS */
"80089" = "1" /* OP SKL VLT FX-CONCUS NOS */
"80090" = "1" /* OP SKL VLT FX/BR INJ NEC */
"80091" = "1" /* OPN SKUL VLT FX W/O COMA */
"80092" = "1" /* OPN SKUL VLT FX-BRF COMA */
"80093" = "1" /* OPN SKUL VLT FX-MOD COMA */
"80094" = "1" /* OPN SKL VLT FX-PROLN COM */
"80095" = "1" /* OP SKUL VLT FX-DEEP COMA */
"80096" = "1" /* OPN SKUL VLT FX-COMA NOS */
"80099" = "1" /* OP SKL VLT FX-CONCUS NOS */
"80100" = "1" /* CLOS SKULL BASE FRACTURE */
"80101" = "1" /* CL SKUL BASE FX W/O COMA */
"80102" = "1" /* CL SKUL BASE FX-BRF COMA */
"80103" = "1" /* CL SKUL BASE FX-MOD COMA */
"80104" = "1" /* CL SKL BASE FX-PROL COMA */
"80105" = "1" /* CL SKL BASE FX-DEEP COMA */
"80106" = "1" /* CL SKUL BASE FX-COMA NOS */
"80109" = "1" /* CL SKULL BASE FX-CONCUSS */
"80110" = "1" /* CL SKL BASE FX/CEREB LAC */
"80111" = "1" /* CL SKUL BASE FX W/O COMA */
"80112" = "1" /* CL SKUL BASE FX-BRF COMA */
"80113" = "1" /* CL SKUL BASE FX-MOD COMA */
"80114" = "1" /* CL SKL BASE FX-PROL COMA */
"80115" = "1" /* CL SKL BASE FX-DEEP COMA */
"80116" = "1" /* CL SKUL BASE FX-COMA NOS */
"80119" = "1" /* CL SKULL BASE FX-CONCUSS */
"80120" = "1" /* CL SKL BASE FX/MENIN HEM */
"80121" = "1" /* CL SKUL BASE FX W/O COMA */
"80122" = "1" /* CL SKUL BASE FX/BRF COMA */
"80123" = "1" /* CL SKUL BASE FX-MOD COMA */
"80124" = "1" /* CL SKL BASE FX-PROL COMA */
"80125" = "1" /* CL SKL BASE FX-DEEP COMA */
"80126" = "1" /* CL SKUL BASE FX-COMA NOS */
"80129" = "1" /* CL SKULL BASE FX-CONCUSS */
"80130" = "1" /* CL SKULL BASE FX/HEM NEC */
"80131" = "1" /* CL SKUL BASE FX W/O COMA */
"80132" = "1" /* CL SKUL BASE FX-BRF COMA */
"80133" = "1" /* CL SKUL BASE FX-MOD COMA */
"80134" = "1" /* CL SKL BASE FX-PROL COMA */
"80135" = "1" /* CL SKL BASE FX-DEEP COMA */
"80136" = "1" /* CL SKUL BASE FX-COMA NOS */
"80139" = "1" /* CL SKULL BASE FX-CONCUSS */
"80140" = "1" /* CL SK BASE FX/BR INJ NEC */
"80141" = "1" /* CL SKUL BASE FX W/O COMA */
"80142" = "1" /* CL SKUL BASE FX-BRF COMA */
"80143" = "1" /* CL SKUL BASE FX-MOD COMA */
"80144" = "1" /* CL SKL BASE FX-PROL COMA */
"80145" = "1" /* CL SKL BASE FX-DEEP COMA */
"80146" = "1" /* CL SKUL BASE FX-COMA NOS */
"80149" = "1" /* CL SKULL BASE FX-CONCUSS */
"80150" = "1" /* OPEN SKULL BASE FRACTURE */
"80151" = "1" /* OPN SKL BASE FX W/O COMA */
"80152" = "1" /* OPN SKL BASE FX-BRF COMA */
"80153" = "1" /* OPN SKL BASE FX-MOD COMA */
"80154" = "1" /* OP SKL BASE FX-PROL COMA */
"80155" = "1" /* OP SKL BASE FX-DEEP COMA */
"80156" = "1" /* OPN SKL BASE FX-COMA NOS */
"80159" = "1" /* OPN SKUL BASE FX-CONCUSS */
"80160" = "1" /* OP SKL BASE FX/CEREB LAC */
"80161" = "1" /* OPN SKL BASE FX W/O COMA */
"80162" = "1" /* OPN SKL BASE FX-BRF COMA */
"80163" = "1" /* OPN SKL BASE FX-MOD COMA */
"80164" = "1" /* OP SKL BASE FX-PROL COMA */
"80165" = "1" /* OP SKL BASE FX-DEEP COMA */
"80166" = "1" /* OPN SKL BASE FX-COMA NOS */
"80169" = "1" /* OPN SKUL BASE FX-CONCUSS */
"80170" = "1" /* OP SKL BASE FX/MENIN HEM */
"80171" = "1" /* OPN SKL BASE FX W/O COMA */
"80172" = "1" /* OPN SKL BASE FX-BRF COMA */
"80173" = "1" /* OPN SKL BASE FX-MOD COMA */
"80174" = "1" /* OP SKL BASE FX-PROL COMA */
"80175" = "1" /* OP SKL BASE FX-DEEP COMA */
"80176" = "1" /* OPN SKL BASE FX-COMA NOS */
"80179" = "1" /* OPN SKUL BASE FX-CONCUSS */
"80180" = "1" /* OPN SKUL BASE FX/HEM NEC */
"80181" = "1" /* OPN SKL BASE FX W/O COMA */
"80182" = "1" /* OPN SKL BASE FX-BRF COMA */
"80183" = "1" /* OPN SKL BASE FX-MOD COMA */
"80184" = "1" /* OP SKL BASE FX-PROL COMA */
"80185" = "1" /* OP SKL BASE FX-DEEP COMA */
"80186" = "1" /* OPN SKL BASE FX-COMA NOS */
"80189" = "1" /* OPN SKUL BASE FX-CONCUSS */
"80190" = "1" /* OP SK BASE FX/BR INJ NEC */
"80191" = "1" /* OP SKUL BASE FX W/O COMA */
"80192" = "1" /* OPN SKL BASE FX-BRF COMA */
"80193" = "1" /* OPN SKL BASE FX-MOD COMA */
"80194" = "1" /* OP SKL BASE FX-PROL COMA */
"80195" = "1" /* OP SKL BASE FX-DEEP COMA */
"80196" = "1" /* OPN SKL BASE FX-COMA NOS */
"80199" = "1" /* OPN SKUL BASE FX-CONCUSS */
"8020"  = "1" /* NASAL BONE FX-CLOSED */
"8021"  = "1" /* NASAL BONE FX-OPEN */
"80220" = "1" /* MANDIBLE FX NOS-CLOSED */
"80221" = "1" /* FX CONDYL PROC MANDIB-CL */
"80222" = "1" /* SUBCONDYLAR FX MANDIB-CL */
"80223" = "1" /* FX CORON PROC MANDIB-CL */
"80224" = "1" /* FX RAMUS NOS-CLOSED */
"80225" = "1" /* FX ANGLE OF JAW-CLOSED */
"80226" = "1" /* FX SYMPHY MANDIB BODY-CL */
"80227" = "1" /* FX ALVEOLAR BORD MAND-CL */
"80228" = "1" /* FX MANDIBLE BODY NEC-CL */
"80229" = "1" /* MULT FX MANDIBLE-CLOSED */
"80230" = "1" /* MANDIBLE FX NOS-OPEN */
"80231" = "1" /* FX CONDYL PROC MAND-OPEN */
"80232" = "1" /* SUBCONDYL FX MANDIB-OPEN */
"80233" = "1" /* FX CORON PROC MANDIB-OPN */
"80234" = "1" /* FX RAMUS NOS-OPEN */
"80235" = "1" /* FX ANGLE OF JAW-OPEN */
"80236" = "1" /* FX SYMPHY MANDIB BDY-OPN */
"80237" = "1" /* FX ALV BORD MAND BDY-OPN */
"80238" = "1" /* FX MANDIBLE BODY NEC-OPN */
"80239" = "1" /* MULT FX MANDIBLE-OPEN */
"8024"  = "1" /* FX MALAR/MAXILLARY-CLOSE */
"8025"  = "1" /* FX MALAR/MAXILLARY-OPEN */
"8026"  = "1" /* FX ORBITAL FLOOR-CLOSED */
"8027"  = "1" /* FX ORBITAL FLOOR-OPEN */
"8028"  = "1" /* FX FACIAL BONE NEC-CLOSE */
"8029"  = "1" /* FX FACIAL BONE NEC-OPEN */
"80300" = "1" /* CLOSE SKULL FRACTURE NEC */
"80301" = "1" /* CL SKULL FX NEC W/O COMA */
"80302" = "1" /* CL SKULL FX NEC-BRF COMA */
"80303" = "1" /* CL SKULL FX NEC-MOD COMA */
"80304" = "1" /* CL SKL FX NEC-PROLN COMA */
"80305" = "1" /* CL SKUL FX NEC-DEEP COMA */
"80306" = "1" /* CL SKULL FX NEC-COMA NOS */
"80309" = "1" /* CL SKULL FX NEC-CONCUSS */
"80310" = "1" /* CL SKL FX NEC/CEREBR LAC */
"80311" = "1" /* CL SKULL FX NEC W/O COMA */
"80312" = "1" /* CL SKULL FX NEC-BRF COMA */
"80313" = "1" /* CL SKULL FX NEC-MOD COMA */
"80314" = "1" /* CL SKL FX NEC-PROLN COMA */
"80315" = "1" /* CL SKUL FX NEC-DEEP COMA */
"80316" = "1" /* CL SKULL FX NEC-COMA NOS */
"80319" = "1" /* CL SKULL FX NEC-CONCUSS */
"80320" = "1" /* CL SKL FX NEC/MENING HEM */
"80321" = "1" /* CL SKULL FX NEC W/O COMA */
"80322" = "1" /* CL SKULL FX NEC-BRF COMA */
"80323" = "1" /* CL SKULL FX NEC-MOD COMA */
"80324" = "1" /* CL SKL FX NEC-PROLN COMA */
"80325" = "1" /* CL SKUL FX NEC-DEEP COMA */
"80326" = "1" /* CL SKULL FX NEC-COMA NOS */
"80329" = "1" /* CL SKULL FX NEC-CONCUSS */
"80330" = "1" /* CL SKULL FX NEC/HEM NEC */
"80331" = "1" /* CL SKULL FX NEC W/O COMA */
"80332" = "1" /* CL SKULL FX NEC-BRF COMA */
"80333" = "1" /* CL SKULL FX NEC-MOD COMA */
"80334" = "1" /* CL SKL FX NEC-PROLN COMA */
"80335" = "1" /* CL SKUL FX NEC-DEEP COMA */
"80336" = "1" /* CL SKULL FX NEC-COMA NOS */
"80339" = "1" /* CL SKULL FX NEC-CONCUSS */
"80340" = "1" /* CL SKL FX NEC/BR INJ NEC */
"80341" = "1" /* CL SKULL FX NEC W/O COMA */
"80342" = "1" /* CL SKULL FX NEC-BRF COMA */
"80343" = "1" /* CL SKULL FX NEC-MOD COMA */
"80344" = "1" /* CL SKL FX NEC-PROLN COMA */
"80345" = "1" /* CL SKUL FX NEC-DEEP COMA */
"80346" = "1" /* CL SKULL FX NEC-COMA NOS */
"80349" = "1" /* CL SKULL FX NEC-CONCUSS */
"80350" = "1" /* OPEN SKULL FRACTURE NEC */
"80351" = "1" /* OPN SKUL FX NEC W/O COMA */
"80352" = "1" /* OPN SKUL FX NEC-BRF COMA */
"80353" = "1" /* OPN SKUL FX NEC-MOD COMA */
"80354" = "1" /* OPN SKL FX NEC-PROL COMA */
"80355" = "1" /* OPN SKL FX NEC-DEEP COMA */
"80356" = "1" /* OPN SKUL FX NEC-COMA NOS */
"80359" = "1" /* OPN SKULL FX NEC-CONCUSS */
"80360" = "1" /* OPN SKL FX NEC/CEREB LAC */
"80361" = "1" /* OPN SKUL FX NEC W/O COMA */
"80362" = "1" /* OPN SKUL FX NEC-BRF COMA */
"80363" = "1" /* OPN SKUL FX NEC-MOD COMA */
"80364" = "1" /* OPN SKL FX NEC-PROLN COM */
"80365" = "1" /* OPN SKL FX NEC-DEEP COMA */
"80366" = "1" /* OPN SKUL FX NEC-COMA NOS */
"80369" = "1" /* OPN SKULL FX NEC-CONCUSS */
"80370" = "1" /* OPN SKL FX NEC/MENIN HEM */
"80371" = "1" /* OPN SKUL FX NEC W/O COMA */
"80372" = "1" /* OPN SKUL FX NEC-BRF COMA */
"80373" = "1" /* OPN SKUL FX NEC-MOD COMA */
"80374" = "1" /* OPN SKL FX NEC-PROL COMA */
"80375" = "1" /* OPN SKL FX NEC-DEEP COMA */
"80376" = "1" /* OPN SKUL FX NEC-COMA NOS */
"80379" = "1" /* OPN SKULL FX NEC-CONCUSS */
"80380" = "1" /* OPN SKULL FX NEC/HEM NEC */
"80381" = "1" /* OPN SKUL FX NEC W/O COMA */
"80382" = "1" /* OPN SKUL FX NEC-BRF COMA */
"80383" = "1" /* OPN SKUL FX NEC-MOD COMA */
"80384" = "1" /* OPN SKL FX NEC-PROL COMA */
"80385" = "1" /* OPN SKL FX NEC-DEEP COMA */
"80386" = "1" /* OPN SKUL FX NEC-COMA NOS */
"80389" = "1" /* OPN SKULL FX NEC-CONCUSS */
"80390" = "1" /* OP SKL FX NEC/BR INJ NEC */
"80391" = "1" /* OPN SKUL FX NEC W/O COMA */
"80392" = "1" /* OPN SKUL FX NEC-BRF COMA */
"80393" = "1" /* OPN SKUL FX NEC-MOD COMA */
"80394" = "1" /* OPN SKL FX NEC-PROL COMA */
"80395" = "1" /* OPN SKL FX NEC-DEEP COMA */
"80396" = "1" /* OPN SKUL FX NEC-COMA NOS */
"80399" = "1" /* OPN SKULL FX NEC-CONCUSS */
"80400" = "1" /* CL SKUL FX W OTH BONE FX */
"80401" = "1" /* CL SKL W OTH FX W/O COMA */
"80402" = "1" /* CL SKL W OTH FX-BRF COMA */
"80403" = "1" /* CL SKL W OTH FX-MOD COMA */
"80404" = "1" /* CL SKL/OTH FX-PROLN COMA */
"80405" = "1" /* CL SKUL/OTH FX-DEEP COMA */
"80406" = "1" /* CL SKL W OTH FX-COMA NOS */
"80409" = "1" /* CL SKUL W OTH FX-CONCUSS */
"80410" = "1" /* CL SK W OTH FX/CEREB LAC */
"80411" = "1" /* CL SKL W OTH FX W/O COMA */
"80412" = "1" /* CL SKL W OTH FX-BRF COMA */
"80413" = "1" /* CL SKL W OTH FX-MOD COMA */
"80414" = "1" /* CL SKL/OTH FX-PROLN COMA */
"80415" = "1" /* CL SKUL/OTH FX-DEEP COMA */
"80416" = "1" /* CL SKL W OTH FX-COMA NOS */
"80419" = "1" /* CL SKUL W OTH FX-CONCUSS */
"80420" = "1" /* CL SKL/OTH FX/MENING HEM */
"80421" = "1" /* CL SKL W OTH FX W/O COMA */
"80422" = "1" /* CL SKL W OTH FX-BRF COMA */
"80423" = "1" /* CL SKL W OTH FX-MOD COMA */
"80424" = "1" /* CL SKL/OTH FX-PROLN COMA */
"80425" = "1" /* CL SKUL/OTH FX-DEEP COMA */
"80426" = "1" /* CL SKL W OTH FX-COMA NOS */
"80429" = "1" /* CL SKUL W OTH FX-CONCUSS */
"80430" = "1" /* CL SKUL W OTH FX/HEM NEC */
"80431" = "1" /* CL SKL W OTH FX W/O COMA */
"80432" = "1" /* CL SKL W OTH FX-BRF COMA */
"80433" = "1" /* CL SKL W OTH FX-MOD COMA */
"80434" = "1" /* CL SKL/OTH FX-PROLN COMA */
"80435" = "1" /* CL SKUL/OTH FX-DEEP COMA */
"80436" = "1" /* CL SKL W OTH FX-COMA NOS */
"80439" = "1" /* CL SKUL W OTH FX-CONCUSS */
"80440" = "1" /* CL SKL/OTH FX/BR INJ NEC */
"80441" = "1" /* CL SKL W OTH FX W/O COMA */
"80442" = "1" /* CL SKL W OTH FX-BRF COMA */
"80443" = "1" /* CL SKL W OTH FX-MOD COMA */
"80444" = "1" /* CL SKL/OTH FX-PROLN COMA */
"80445" = "1" /* CL SKUL/OTH FX-DEEP COMA */
"80446" = "1" /* CL SKL W OTH FX-COMA NOS */
"80449" = "1" /* CL SKUL W OTH FX-CONCUSS */
"80450" = "1" /* OPN SKULL FX/OTH BONE FX */
"80451" = "1" /* OPN SKUL/OTH FX W/O COMA */
"80452" = "1" /* OPN SKUL/OTH FX-BRF COMA */
"80453" = "1" /* OPN SKUL/OTH FX-MOD COMA */
"80454" = "1" /* OPN SKL/OTH FX-PROL COMA */
"80455" = "1" /* OPN SKL/OTH FX-DEEP COMA */
"80456" = "1" /* OPN SKUL/OTH FX-COMA NOS */
"80459" = "1" /* OPN SKULL/OTH FX-CONCUSS */
"80460" = "1" /* OPN SKL/OTH FX/CEREB LAC */
"80461" = "1" /* OPN SKUL/OTH FX W/O COMA */
"80462" = "1" /* OPN SKUL/OTH FX-BRF COMA */
"80463" = "1" /* OPN SKUL/OTH FX-MOD COMA */
"80464" = "1" /* OPN SKL/OTH FX-PROL COMA */
"80465" = "1" /* OPN SKL/OTH FX-DEEP COMA */
"80466" = "1" /* OPN SKUL/OTH FX-COMA NOS */
"80469" = "1" /* OPN SKULL/OTH FX-CONCUSS */
"80470" = "1" /* OPN SKL/OTH FX/MENIN HEM */
"80471" = "1" /* OPN SKUL/OTH FX W/O COMA */
"80472" = "1" /* OPN SKUL/OTH FX-BRF COMA */
"80473" = "1" /* OPN SKUL/OTH FX-MOD COMA */
"80474" = "1" /* OPN SKL/OTH FX-PROL COMA */
"80475" = "1" /* OPN SKL/OTH FX-DEEP COMA */
"80476" = "1" /* OPN SKUL/OTH FX-COMA NOS */
"80479" = "1" /* OPN SKULL/OTH FX-CONCUSS */
"80480" = "1" /* OPN SKL W OTH FX/HEM NEC */
"80481" = "1" /* OPN SKUL/OTH FX W/O COMA */
"80482" = "1" /* OPN SKUL/OTH FX-BRF COMA */
"80483" = "1" /* OPN SKUL/OTH FX-MOD COMA */
"80484" = "1" /* OPN SKL/OTH FX-PROL COMA */
"80485" = "1" /* OPN SKL/OTH FX-DEEP COMA */
"80486" = "1" /* OPN SKUL/OTH FX-COMA NOS */
"80489" = "1" /* OPN SKULL/OTH FX-CONCUSS */
"80490" = "1" /* OP SKL/OTH FX/BR INJ NEC */
"80491" = "1" /* OPN SKUL/OTH FX W/O COMA */
"80492" = "1" /* OPN SKUL/OTH FX-BRF COMA */
"80493" = "1" /* OPN SKUL/OTH FX-MOD COMA */
"80494" = "1" /* OPN SKL/OTH FX-PROL COMA */
"80495" = "1" /* OPN SKL/OTH FX-DEEP COMA */
"80496" = "1" /* OPN SKUL/OTH FX-COMA NOS */
"80499" = "1" /* OPN SKULL/OTH FX-CONCUSS */
"80500" = "1" /* FX CERVICAL VERT NOS-CL */
"80501" = "1" /* FX C1 VERTEBRA-CLOSED */
"80502" = "1" /* FX C2 VERTEBRA-CLOSED */
"80503" = "1" /* FX C3 VERTEBRA-CLOSED */
"80504" = "1" /* FX C4 VERTEBRA-CLOSED */
"80505" = "1" /* FX C5 VERTEBRA-CLOSED */
"80506" = "1" /* FX C6 VERTEBRA-CLOSED */
"80507" = "1" /* FX C7 VERTEBRA-CLOSED */
"80508" = "1" /* FX MULT CERVICAL VERT-CL */
"80510" = "1" /* FX CERVICAL VERT NOS-OPN */
"80511" = "1" /* FX C1 VERTEBRA-OPEN */
"80512" = "1" /* FX C2 VERTEBRA-OPEN */
"80513" = "1" /* FX C3 VERTEBRA-OPEN */
"80514" = "1" /* FX C4 VERTEBRA-OPEN */
"80515" = "1" /* FX C5 VERTEBRA-OPEN */
"80516" = "1" /* FX C6 VERTEBRA-OPEN */
"80517" = "1" /* FX C7 VERTEBRA-OPEN */
"80518" = "1" /* FX MLT CERVICAL VERT-OPN */
"8052"  = "1" /* FX DORSAL VERTEBRA-CLOSE */
"8053"  = "1" /* FX DORSAL VERTEBRA-OPEN */
"8054"  = "1" /* FX LUMBAR VERTEBRA-CLOSE */
"8055"  = "1" /* FX LUMBAR VERTEBRA-OPEN */
"8056"  = "1" /* FX SACRUM/COCCYX-CLOSED */
"8057"  = "1" /* FX SACRUM/COCCYX-OPEN */
"8058"  = "1" /* VERTEBRAL FX NOS-CLOSED */
"8059"  = "1" /* VERTEBRAL FX NOS-OPEN */
"80600" = "1" /* C1-C4 FX-CL/CORD INJ NOS */
"80601" = "1" /* C1-C4 FX-CL/COM CORD LES */
"80602" = "1" /* C1-C4 FX-CL/ANT CORD SYN */
"80603" = "1" /* C1-C4 FX-CL/CEN CORD SYN */
"80604" = "1" /* C1-C4 FX-CL/CORD INJ NEC */
"80605" = "1" /* C5-C7 FX-CL/CORD INJ NOS */
"80606" = "1" /* C5-C7 FX-CL/COM CORD LES */
"80607" = "1" /* C5-C7 FX-CL/ANT CORD SYN */
"80608" = "1" /* C5-C7 FX-CL/CEN CORD SYN */
"80609" = "1" /* C5-C7 FX-CL/CORD INJ NEC */
"80610" = "1" /* C1-C4 FX-OP/CORD INJ NOS */
"80611" = "1" /* C1-C4 FX-OP/COM CORD LES */
"80612" = "1" /* C1-C4 FX-OP/ANT CORD SYN */
"80613" = "1" /* C1-C4 FX-OP/CEN CORD SYN */
"80614" = "1" /* C1-C4 FX-OP/CORD INJ NEC */
"80615" = "1" /* C5-C7 FX-OP/CORD INJ NOS */
"80616" = "1" /* C5-C7 FX-OP/COM CORD LES */
"80617" = "1" /* C5-C7 FX-OP/ANT CORD SYN */
"80618" = "1" /* C5-C7 FX-OP/CEN CORD SYN */
"80619" = "1" /* C5-C7 FX-OP/CORD INJ NEC */
"80620" = "1" /* T1-T6 FX-CL/CORD INJ NOS */
"80621" = "1" /* T1-T6 FX-CL/COM CORD LES */
"80622" = "1" /* T1-T6 FX-CL/ANT CORD SYN */
"80623" = "1" /* T1-T6 FX-CL/CEN CORD SYN */
"80624" = "1" /* T1-T6 FX-CL/CORD INJ NEC */
"80625" = "1" /* T7-T12 FX-CL/CRD INJ NOS */
"80626" = "1" /* T7-T12 FX-CL/COM CRD LES */
"80627" = "1" /* T7-T12 FX-CL/ANT CRD SYN */
"80628" = "1" /* T7-T12 FX-CL/CEN CRD SYN */
"80629" = "1" /* T7-T12 FX-CL/CRD INJ NEC */
"80630" = "1" /* T1-T6 FX-OP/CORD INJ NOS */
"80631" = "1" /* T1-T6 FX-OP/COM CORD LES */
"80632" = "1" /* T1-T6 FX-OP/ANT CORD SYN */
"80633" = "1" /* T1-T6 FX-OP/CEN CORD SYN */
"80634" = "1" /* T1-T6 FX-OP/CORD INJ NEC */
"80635" = "1" /* T7-T12 FX-OP/CRD INJ NOS */
"80636" = "1" /* T7-T12 FX-OP/COM CRD LES */
"80637" = "1" /* T7-T12 FX-OP/ANT CRD SYN */
"80638" = "1" /* T7-T12 FX-OP/CEN CRD SYN */
"80639" = "1" /* T7-T12 FX-OP/CRD INJ NEC */
"8064"  = "1" /* CL LUMBAR FX W CORD INJ */
"8065"  = "1" /* OPN LUMBAR FX W CORD INJ */
"80660" = "1" /* FX SACRUM-CL/CRD INJ NOS */
"80661" = "1" /* FX SACR-CL/CAUDA EQU LES */
"80662" = "1" /* FX SACR-CL/CAUDA INJ NEC */
"80669" = "1" /* FX SACRUM-CL/CRD INJ NEC */
"80670" = "1" /* FX SACRUM-OP/CRD INJ NOS */
"80671" = "1" /* FX SACR-OP/CAUDA EQU LES */
"80672" = "1" /* FX SACR-OP/CAUDA INJ NEC */
"80679" = "1" /* FX SACRUM-OP/CRD INJ NEC */
"8068"  = "1" /* VERT FX NOS-CL W CRD INJ */
"8069"  = "1" /* VERT FX NOS-OP W CRD INJ */
"80700" = "1" /* FRACTURE RIB NOS-CLOSED */
"80701" = "1" /* FRACTURE ONE RIB-CLOSED */
"80702" = "1" /* FRACTURE TWO RIBS-CLOSED */
"80703" = "1" /* FRACTURE THREE RIBS-CLOS */
"80704" = "1" /* FRACTURE FOUR RIBS-CLOSE */
"80705" = "1" /* FRACTURE FIVE RIBS-CLOSE */
"80706" = "1" /* FRACTURE SIX RIBS-CLOSED */
"80707" = "1" /* FRACTURE SEVEN RIBS-CLOS */
"80708" = "1" /* FX EIGHT/MORE RIB-CLOSED */
"80709" = "1" /* FX MULT RIBS NOS-CLOSED */
"80710" = "1" /* FRACTURE RIB NOS-OPEN */
"80711" = "1" /* FRACTURE ONE RIB-OPEN */
"80712" = "1" /* FRACTURE TWO RIBS-OPEN */
"80713" = "1" /* FRACTURE THREE RIBS-OPEN */
"80714" = "1" /* FRACTURE FOUR RIBS-OPEN */
"80715" = "1" /* FRACTURE FIVE RIBS-OPEN */
"80716" = "1" /* FRACTURE SIX RIBS-OPEN */
"80717" = "1" /* FRACTURE SEVEN RIBS-OPEN */
"80718" = "1" /* FX EIGHT/MORE RIBS-OPEN */
"80719" = "1" /* FX MULT RIBS NOS-OPEN */
"8072"  = "1" /* FRACTURE OF STERNUM-CLOS */
"8073"  = "1" /* FRACTURE OF STERNUM-OPEN */
"8074"  = "1" /* FLAIL CHEST */
"8075"  = "1" /* FX LARYNX/TRACHEA-CLOSED */
"8076"  = "1" /* FX LARYNX/TRACHEA-OPEN */
"8080"  = "1" /* FRACTURE ACETABULUM-CLOS */
"8081"  = "1" /* FRACTURE ACETABULUM-OPEN */
"8082"  = "1" /* FRACTURE OF PUBIS-CLOSED */
"8083"  = "1" /* FRACTURE OF PUBIS-OPEN */
"80841" = "1" /* FRACTURE OF ILIUM-CLOSED */
"80842" = "1" /* FRACTURE ISCHIUM-CLOSED */
"80843" = "1" /* PELV FX-CLOS/PELV DISRUP */
"80849" = "1" /* PELVIC FRACTURE NEC-CLOS */
"80851" = "1" /* FRACTURE OF ILIUM-OPEN */
"80852" = "1" /* FRACTURE OF ISCHIUM-OPEN */
"80853" = "1" /* PELV FX-OPEN/PELV DISRUP */
"80859" = "1" /* PELVIC FRACTURE NEC-OPEN */
"8088"  = "1" /* PELVIC FRACTURE NOS-CLOS */
"8089"  = "1" /* PELVIC FRACTURE NOS-OPEN */
"8090"  = "1" /* FRACTURE TRUNK BONE-CLOS */
"8091"  = "1" /* FRACTURE TRUNK BONE-OPEN */
"81000" = "1" /* FX CLAVICLE NOS-CLOSED */
"81001" = "1" /* FX CLAVICL, STERN END-CL */
"81002" = "1" /* FX CLAVICLE SHAFT-CLOSED */
"81003" = "1" /* FX CLAVICL, ACROM END-CL */
"81010" = "1" /* FX CLAVICLE NOS-OPEN */
"81011" = "1" /* FX CLAVIC, STERN END-OPN */
"81012" = "1" /* FX CLAVICLE SHAFT-OPEN */
"81013" = "1" /* FX CLAVIC, ACROM END-OPN */
"81100" = "1" /* FX SCAPULA NOS-CLOSED */
"81101" = "1" /* FX SCAPUL, ACROM PROC-CL */
"81102" = "1" /* FX SCAPUL, CORAC PROC-CL */
"81103" = "1" /* FX SCAP, GLEN CAV/NCK-CL */
"81109" = "1" /* FX SCAPULA NEC-CLOSED */
"81110" = "1" /* FX SCAPULA NOS-OPEN */
"81111" = "1" /* FX SCAPUL, ACROM PROC-OP */
"81112" = "1" /* FX SCAPUL, CORAC PROC-OP */
"81113" = "1" /* FX SCAP, GLEN CAV/NCK-OP */
"81119" = "1" /* FX SCAPULA NEC-OPEN */
"81200" = "1" /* FX UP END HUMERUS NOS-CL */
"81201" = "1" /* FX SURG NCK HUMERUS-CLOS */
"81202" = "1" /* FX ANATOM NCK HUMERUS-CL */
"81203" = "1" /* FX GR TUBEROS HUMERUS-CL */
"81209" = "1" /* FX UPPER HUMERUS NEC-CL */
"81210" = "1" /* FX UPPER HUMERUS NOS-OPN */
"81211" = "1" /* FX SURG NECK HUMERUS-OPN */
"81212" = "1" /* FX ANAT NECK HUMERUS-OPN */
"81213" = "1" /* FX GR TUBEROS HUMER-OPEN */
"81219" = "1" /* FX UPPER HUMERUS NEC-OPN */
"81220" = "1" /* FX HUMERUS NOS-CLOSED */
"81221" = "1" /* FX HUMERUS SHAFT-CLOSED */
"81230" = "1" /* FX HUMERUS NOS-OPEN */
"81231" = "1" /* FX HUMERUS SHAFT-OPEN */
"81240" = "1" /* FX LOWER HUMERUS NOS-CL */
"81241" = "1" /* SUPRCONDYL FX HUMERUS-CL */
"81242" = "1" /* FX HUMER, LAT CONDYL-CL */
"81243" = "1" /* FX HUMER, MED CONDYL-CL */
"81244" = "1" /* FX HUMER, CONDYL NOS-CL */
"81249" = "1" /* FX LOWER HUMERUS NEC-CL */
"81250" = "1" /* FX LOWER HUMER NOS-OPEN */
"81251" = "1" /* SUPRACONDYL FX HUMER-OPN */
"81252" = "1" /* FX HUMER, LAT CONDYL-OPN */
"81253" = "1" /* FX HUMER, MED CONDYL-OPN */
"81254" = "1" /* FX HUMER, CONDYL NOS-OPN */
"81259" = "1" /* FX LOWER HUMER NEC-OPEN */
"81300" = "1" /* FX UPPER FOREARM NOS-CL */
"81301" = "1" /* FX OLECRAN PROC ULNA-CL */
"81302" = "1" /* FX CORONOID PROC ULNA-CL */
"81303" = "1" /* MONTEGGIA"S FX-CLOSED */
"81304" = "1" /* FX UPPER ULNA NEC/NOS-CL */
"81305" = "1" /* FX RADIUS HEAD-CLOSED */
"81306" = "1" /* FX RADIUS NECK-CLOSED */
"81307" = "1" /* FX UP RADIUS NEC/NOS-CL */
"81308" = "1" /* FX UP RADIUS W ULNA-CLOS */
"81310" = "1" /* FX UPPER FOREARM NOS-OPN */
"81311" = "1" /* FX OLECRAN PROC ULNA-OPN */
"81312" = "1" /* FX CORONOID PRO ULNA-OPN */
"81313" = "1" /* MONTEGGIA"S FX-OPEN */
"81314" = "1" /* FX UP ULNA NEC/NOS-OPEN */
"81315" = "1" /* FX RADIUS HEAD-OPEN */
"81316" = "1" /* FX RADIUS NECK-OPEN */
"81317" = "1" /* FX UP RADIUS NEC/NOS-OPN */
"81318" = "1" /* FX UP RADIUS W ULNA-OPEN */
"81320" = "1" /* FX SHAFT FOREARM NOS-CL */
"81321" = "1" /* FX RADIUS SHAFT-CLOSED */
"81322" = "1" /* FX ULNA SHAFT-CLOSED */
"81323" = "1" /* FX SHAFT RAD W ULNA-CLOS */
"81330" = "1" /* FX SHAFT FOREARM NOS-OPN */
"81331" = "1" /* FX RADIUS SHAFT-OPEN */
"81332" = "1" /* FX ULNA SHAFT-OPEN */
"81333" = "1" /* FX SHAFT RAD W ULNA-OPEN */
"81340" = "1" /* FX LOWER FOREARM NOS-CL */
"81341" = "1" /* COLLES" FRACTURE-CLOSED */
"81342" = "1" /* FX DISTAL RADIUS NEC-CL */
"81343" = "1" /* FX DISTAL ULNA-CLOSED */
"81344" = "1" /* FX LOW RADIUS W ULNA-CL */
"81345" = "1" /* TORUS FRACTURE OF RADIUS */
"81350" = "1" /* FX LOWER FOREARM NOS-OPN */
"81351" = "1" /* COLLES" FRACTURE-OPEN */
"81352" = "1" /* FX DISTAL RADIUS NEC-OPN */
"81353" = "1" /* FX DISTAL ULNA-OPEN */
"81354" = "1" /* FX LOW RADIUS W ULNA-OPN */
"81380" = "1" /* FX FOREARM NOS-CLOSED */
"81381" = "1" /* FX RADIUS NOS-CLOSED */
"81382" = "1" /* FRACTURE ULNA NOS-CLOSED */
"81383" = "1" /* FX RADIUS W ULNA NOS-CL */
"81390" = "1" /* FX FOREARM NOS-OPEN */
"81391" = "1" /* FRACTURE RADIUS NOS-OPEN */
"81392" = "1" /* FRACTURE ULNA NOS-OPEN */
"81393" = "1" /* FX RADIUS W ULNA NOS-OPN */
"81400" = "1" /* FX CARPAL BONE NOS-CLOSE */
"81401" = "1" /* FX NAVICULAR, WRIST-CLOS */
"81402" = "1" /* FX LUNATE, WRIST-CLOSED */
"81403" = "1" /* FX TRIQUETRAL, WRIST-CL */
"81404" = "1" /* FX PISIFORM-CLOSED */
"81405" = "1" /* FX TRAPEZIUM BONE-CLOSED */
"81406" = "1" /* FX TRAPEZOID BONE-CLOSED */
"81407" = "1" /* FX CAPITATE BONE-CLOSED */
"81408" = "1" /* FX HAMATE BONE-CLOSED */
"81409" = "1" /* FX CARPAL BONE NEC-CLOSE */
"81410" = "1" /* FX CARPAL BONE NOS-OPEN */
"81411" = "1" /* FX NAVICULAR, WRIST-OPEN */
"81412" = "1" /* FX LUNATE, WRIST-OPEN */
"81413" = "1" /* FX TRIQUETRAL, WRIST-OPN */
"81414" = "1" /* FX PISIFORM-OPEN */
"81415" = "1" /* FX TRAPEZIUM BONE-OPEN */
"81416" = "1" /* FX TRAPEZOID BONE-OPEN */
"81417" = "1" /* FX CAPITATE BONE-OPEN */
"81418" = "1" /* FX HAMATE BONE-OPEN */
"81419" = "1" /* FX CARPAL BONE NEC-OPEN */
"81500" = "1" /* FX METACARPAL NOS-CLOSED */
"81501" = "1" /* FX 1ST METACARP BASE-CL */
"81502" = "1" /* FX METACARP BASE NEC-CL */
"81503" = "1" /* FX METACARPAL SHAFT-CLOS */
"81504" = "1" /* FX METACARPAL NECK-CLOSE */
"81509" = "1" /* MULT FX METACARPUS-CLOSE */
"81510" = "1" /* FX METACARPAL NOS-OPEN */
"81511" = "1" /* FX 1ST METACARP BASE-OPN */
"81512" = "1" /* FX METACARP BASE NEC-OPN */
"81513" = "1" /* FX METACARPAL SHAFT-OPEN */
"81514" = "1" /* FX METACARPAL NECK-OPEN */
"81519" = "1" /* MULT FX METACARPUS-OPEN */
"8170"  = "1" /* MULTIPLE FX HAND-CLOSED */
"8171"  = "1" /* MULTIPLE FX HAND-OPEN */
"8180"  = "1" /* FX ARM MULT/NOS-CLOSED */
"8181"  = "1" /* FX ARM MULT/NOS-OPEN */
"8190"  = "1" /* FX ARMS W RIB/STERNUM-CL */
"8191"  = "1" /* FX ARMS W RIB/STERN-OPEN */
"82000" = "1" /* FX FEMUR INTRCAPS NOS-CL */
"82001" = "1" /* FX UP FEMUR EPIPHY-CLOS */
"82002" = "1" /* FX FEMUR, MIDCERVIC-CLOS */
"82003" = "1" /* FX BASE FEMORAL NCK-CLOS */
"82009" = "1" /* FX FEMUR INTRCAPS NEC-CL */
"82010" = "1" /* FX FEMUR INTRCAP NOS-OPN */
"82011" = "1" /* FX UP FEMUR EPIPHY-OPEN */
"82012" = "1" /* FX FEMUR, MIDCERVIC-OPEN */
"82013" = "1" /* FX BASE FEMORAL NCK-OPEN */
"82019" = "1" /* FX FEMUR INTRCAP NEC-OPN */
"82020" = "1" /* TROCHANTERIC FX NOS-CLOS */
"82021" = "1" /* INTERTROCHANTERIC FX-CL */
"82022" = "1" /* SUBTROCHANTERIC FX-CLOSE */
"82030" = "1" /* TROCHANTERIC FX NOS-OPEN */
"82031" = "1" /* INTERTROCHANTERIC FX-OPN */
"82032" = "1" /* SUBTROCHANTERIC FX-OPEN */
"8208"  = "1" /* FX NECK OF FEMUR NOS-CL */
"8209"  = "1" /* FX NECK OF FEMUR NOS-OPN */
"82100" = "1" /* FX FEMUR NOS-CLOSED */
"82101" = "1" /* FX FEMUR SHAFT-CLOSED */
"82110" = "1" /* FX FEMUR NOS-OPEN */
"82111" = "1" /* FX FEMUR SHAFT-OPEN */
"82120" = "1" /* FX LOW END FEMUR NOS-CL */
"82121" = "1" /* FX FEMORAL CONDYLE-CLOSE */
"82122" = "1" /* FX LOW FEMUR EPIPHY-CLOS */
"82123" = "1" /* SUPRACONDYL FX FEMUR-CL */
"82129" = "1" /* FX LOW END FEMUR NEC-CL */
"82130" = "1" /* FX LOW END FEMUR NOS-OPN */
"82131" = "1" /* FX FEMORAL CONDYLE-OPEN */
"82132" = "1" /* FX LOW FEMUR EPIPHY-OPEN */
"82133" = "1" /* SUPRACONDYL FX FEMUR-OPN */
"82139" = "1" /* FX LOW END FEMUR NEC-OPN */
"8220"  = "1" /* FRACTURE PATELLA-CLOSED */
"8221"  = "1" /* FRACTURE PATELLA-OPEN */
"82300" = "1" /* FX UPPER END TIBIA-CLOSE */
"82301" = "1" /* FX UPPER END FIBULA-CLOS */
"82302" = "1" /* FX UP TIBIA W FIBULA-CL */
"82310" = "1" /* FX UPPER END TIBIA-OPEN */
"82311" = "1" /* FX UPPER END FIBULA-OPEN */
"82312" = "1" /* FX UP TIBIA W FIBULA-OPN */
"82320" = "1" /* FX SHAFT TIBIA-CLOSED */
"82321" = "1" /* FX SHAFT FIBULA-CLOSED */
"82322" = "1" /* FX SHAFT FIB W TIB-CLOS */
"82330" = "1" /* FX TIBIA SHAFT-OPEN */
"82331" = "1" /* FX FIBULA SHAFT-OPEN */
"82332" = "1" /* FX SHAFT TIBIA W FIB-OPN */
"8234"  = "1" /* TORUS FRACTURE */
"82340" = "1" /* TORUS FRACTURE, TIBIA ALONE */
"82341" = "1" /* TORUS FRACTURE, FIBULA ALONE */
"82342" = "1" /* TORUS FRACTURE, FIBULA WITH TIBIA */
"82380" = "1" /* FX TIBIA NOS-CLOSED */
"82381" = "1" /* FX FIBULA NOS-CLOSED */
"82382" = "1" /* FX TIBIA W FIBULA NOS-CL */
"82390" = "1" /* FX TIBIA NOS-OPEN */
"82391" = "1" /* FX FIBULA NOS-OPEN */
"82392" = "1" /* FX TIBIA W FIB NOS-OPEN */
"8240"  = "1" /* FX MEDIAL MALLEOLUS-CLOS */
"8241"  = "1" /* FX MEDIAL MALLEOLUS-OPEN */
"8242"  = "1" /* FX LATERAL MALLEOLUS-CL */
"8243"  = "1" /* FX LATERAL MALLEOLUS-OPN */
"8244"  = "1" /* FX BIMALLEOLAR-CLOSED */
"8245"  = "1" /* FX BIMALLEOLAR-OPEN */
"8246"  = "1" /* FX TRIMALLEOLAR-CLOSED */
"8247"  = "1" /* FX TRIMALLEOLAR-OPEN */
"8248"  = "1" /* FX ANKLE NOS-CLOSED */
"8249"  = "1" /* FX ANKLE NOS-OPEN */
"8250"  = "1" /* FRACTURE CALCANEUS-CLOSE */
"8251"  = "1" /* FRACTURE CALCANEUS-OPEN */
"82520" = "1" /* FX FOOT BONE NOS-CLOSED */
"82521" = "1" /* FX ASTRAGALUS-CLOSED */
"82522" = "1" /* FX NAVICULAR, FOOT-CLOS */
"82523" = "1" /* FX CUBOID-CLOSED */
"82524" = "1" /* FX CUNEIFORM, FOOT-CLOS */
"82525" = "1" /* FX METATARSAL-CLOSED */
"82529" = "1" /* FX FOOT BONE NEC-CLOSED */
"82530" = "1" /* FX FOOT BONE NOS-OPEN */
"82531" = "1" /* FX ASTRAGALUS-OPEN */
"82532" = "1" /* FX NAVICULAR, FOOT-OPEN */
"82533" = "1" /* FX CUBOID-OPEN */
"82534" = "1" /* FX CUNEIFORM, FOOT-OPEN */
"82535" = "1" /* FX METATARSAL-OPEN */
"82539" = "1" /* FX FOOT BONE NEC-OPEN */
"8270"  = "1" /* FX LOWER LIMB NEC-CLOSED */
"8271"  = "1" /* FX LOWER LIMB NEC-OPEN */
"8280"  = "1" /* FX LEGS W ARM/RIB-CLOSED */
"8281"  = "1" /* FX LEGS W ARM/RIB-OPEN */
"8290"  = "1" /* FRACTURE NOS-CLOSED */
"8291"  = "1" /* FRACTURE NOS-OPEN */
"8300"  = "1" /* DISLOCATION JAW-CLOSED */
"8301"  = "1" /* DISLOCATION JAW-OPEN */
"83100" = "1" /* DISLOC SHOULDER NOS-CLOS */
"83101" = "1" /* ANT DISLOC HUMERUS-CLOSE */
"83102" = "1" /* POST DISLOC HUMERUS-CLOS */
"83103" = "1" /* INFER DISLOC HUMERUS-CL */
"83104" = "1" /* DISLOC ACROMIOCLAVIC-CL */
"83109" = "1" /* DISLOC SHOULDER NEC-CLOS */
"83110" = "1" /* DISLOC SHOULDER NOS-OPEN */
"83111" = "1" /* ANT DISLOC HUMERUS-OPEN */
"83112" = "1" /* POST DISLOC HUMERUS-OPEN */
"83113" = "1" /* INFER DISLOC HUMERUS-OPN */
"83114" = "1" /* DISLOC ACROMIOCLAVIC-OPN */
"83119" = "1" /* DISLOC SHOULDER NEC-OPEN */
"83200" = "1" /* DISLOCAT ELBOW NOS-CLOSE */
"83201" = "1" /* ANT DISLOC ELBOW-CLOSED */
"83202" = "1" /* POST DISLOC ELBOW-CLOSED */
"83203" = "1" /* MED DISLOC ELBOW-CLOSED */
"83204" = "1" /* LAT DISLOC ELBOW-CLOSED */
"83209" = "1" /* DISLOCAT ELBOW NEC-CLOSE */
"83210" = "1" /* DISLOCAT ELBOW NOS-OPEN */
"83211" = "1" /* ANT DISLOC ELBOW-OPEN */
"83212" = "1" /* POST DISLOC ELBOW-OPEN */
"83213" = "1" /* MED DISLOC ELBOW-OPEN */
"83214" = "1" /* LAT DISLOCAT ELBOW-OPEN */
"83219" = "1" /* DISLOCAT ELBOW NEC-OPEN */
"83300" = "1" /* DISLOC WRIST NOS-CLOSED */
"83301" = "1" /* DISLOC DIST RADIOULN-CL */
"83302" = "1" /* DISLOC RADIOCARPAL-CLOS */
"83303" = "1" /* DISLOCA MIDCARPAL-CLOSED */
"83304" = "1" /* DISLOC CARPOMETACARP-CL */
"83305" = "1" /* DISLOC METACARPAL-CLOSED */
"83309" = "1" /* DISLOC WRIST NEC-CLOSED */
"83310" = "1" /* DISLOCAT WRIST NOS-OPEN */
"83311" = "1" /* DISLOC DIST RADIOULN-OPN */
"83312" = "1" /* DISLOC RADIOCARPAL-OPEN */
"83313" = "1" /* DISLOCAT MIDCARPAL-OPEN */
"83314" = "1" /* DISLOC CARPOMETACARP-OPN */
"83315" = "1" /* DISLOCAT METACARPAL-OPEN */
"83319" = "1" /* DISLOCAT WRIST NEC-OPEN */
"83500" = "1" /* DISLOCAT HIP NOS-CLOSED */
"83501" = "1" /* POSTERIOR DISLOC HIP-CL */
"83502" = "1" /* OBTURATOR DISLOC HIP-CL */
"83503" = "1" /* ANT DISLOC HIP NEC-CLOS */
"83510" = "1" /* DISLOCATION HIP NOS-OPEN */
"83511" = "1" /* POSTERIOR DISLOC HIP-OPN */
"83512" = "1" /* OBTURATOR DISLOC HIP-OPN */
"83513" = "1" /* ANT DISLOC HIP NEC-OPEN */
"8360"  = "1" /* TEAR MED MENISC KNEE-CUR */
"8361"  = "1" /* TEAR LAT MENISC KNEE-CUR */
"8362"  = "1" /* TEAR MENISCUS NEC-CURREN */
"8363"  = "1" /* DISLOCAT PATELLA-CLOSED */
"8364"  = "1" /* DISLOCATION PATELLA-OPEN */
"83650" = "1" /* DISLOCAT KNEE NOS-CLOSED */
"83651" = "1" /* ANT DISLOC PROX TIBIA-CL */
"83652" = "1" /* POST DISL PROX TIBIA-CL */
"83653" = "1" /* MED DISLOC PROX TIBIA-CL */
"83654" = "1" /* LAT DISLOC PROX TIBIA-CL */
"83659" = "1" /* DISLOCAT KNEE NEC-CLOSED */
"83660" = "1" /* DISLOCAT KNEE NOS-OPEN */
"83661" = "1" /* ANT DISL PROX TIBIA-OPEN */
"83662" = "1" /* POST DISL PROX TIBIA-OPN */
"83663" = "1" /* MED DISL PROX TIBIA-OPEN */
"83664" = "1" /* LAT DISL PROX TIBIA-OPEN */
"83669" = "1" /* DISLOCAT KNEE NEC-OPEN */
"8370"  = "1" /* DISLOCATION ANKLE-CLOSED */
"8371"  = "1" /* DISLOCATION ANKLE-OPEN */
"83800" = "1" /* DISLOCAT FOOT NOS-CLOSED */
"83801" = "1" /* DISLOC TARSAL NOS-CLOSED */
"83802" = "1" /* DISLOC MIDTARSAL-CLOSED */
"83803" = "1" /* DISLOC TARSOMETATARS-CL */
"83804" = "1" /* DISLOC METATARSAL NOS-CL */
"83805" = "1" /* DISL METATARSOPHALANG-CL */
"83806" = "1" /* DISL INTERPHALAN FOOT-CL */
"83809" = "1" /* DISLOCAT FOOT NEC-CLOSED */
"83810" = "1" /* DISLOCAT FOOT NOS-OPEN */
"83811" = "1" /* DISLOC TARSAL NOS-OPEN */
"83812" = "1" /* DISLOC MIDTARSAL-OPEN */
"83813" = "1" /* DISL TARSOMETATARSAL-OPN */
"83814" = "1" /* DISL METATARSAL NOS-OPEN */
"83815" = "1" /* DISLOC METATARSOPHAL-OPN */
"83816" = "1" /* DIS INTERPHALAN FOOT-OPN */
"83819" = "1" /* DISLOCAT FOOT NEC-OPEN */
"83900" = "1" /* DISLOC CERV VERT NOS-CL */
"83901" = "1" /* DISLOC 1ST CERV VERT-CL */
"83902" = "1" /* DISLOC 2ND CERV VERT-CL */
"83903" = "1" /* DISLOC 3RD CERV VERT-CL */
"83904" = "1" /* DISLOC 4TH CERV VERT-CL */
"83905" = "1" /* DISLOC 5TH CERV VERT-CL */
"83906" = "1" /* DISLOC 6TH CERV VERT-CL */
"83907" = "1" /* DISLOC 7TH CERV VERT-CL */
"83908" = "1" /* DISLOC MULT CERV VERT-CL */
"83910" = "1" /* DISLOC CERV VERT NOS-OPN */
"83911" = "1" /* DISLOC LST CERV VERT-OPN */
"83912" = "1" /* DISLOC 2ND CERV VERT-OPN */
"83913" = "1" /* DISLOC 3RD CERV VERT-OPN */
"83914" = "1" /* DISLOC 4TH CERV VERT-OPN */
"83915" = "1" /* DISLOC 5TH CERV VERT-OPN */
"83916" = "1" /* DISLOC 6TH CERV VERT-OPN */
"83917" = "1" /* DISLOC 7TH CERV VERT-OPN */
"83918" = "1" /* DISLOC MLT CERV VERT-OPN */
"83920" = "1" /* DISLOCAT LUMBAR VERT-CL */
"83921" = "1" /* DISLOC THORACIC VERT-CL */
"83930" = "1" /* DISLOCAT LUMBAR VERT-OPN */
"83931" = "1" /* DISLOC THORACIC VERT-OPN */
"83940" = "1" /* DISLOCAT VERTEBRA NOS-CL */
"83941" = "1" /* DISLOCAT COCCYX-CLOSED */
"83942" = "1" /* DISLOCAT SACRUM-CLOSED */
"83949" = "1" /* DISLOCAT VERTEBRA NEC-CL */
"83950" = "1" /* DISLOC VERTEBRA NOS-OPEN */
"83951" = "1" /* DISLOCAT COCCYX-OPEN */
"83952" = "1" /* DISLOCAT SACRUM-OPEN */
"83959" = "1" /* DISLOC VERTEBRA NEC-OPEN */
"83961" = "1" /* DISLOCAT STERNUM-CLOSED */
"83969" = "1" /* DISLOCAT SITE NEC-CLOSED */
"83971" = "1" /* DISLOCATION STERNUM-OPEN */
"83979" = "1" /* DISLOCAT SITE NEC-OPEN */
"8398"  = "1" /* DISLOCATION NEC-CLOSED */
"8399"  = "1" /* DISLOCATION NEC-OPEN */
"8500"  = "1" /* CONCUSSION W/O COMA */
"8501"  = "1" /* CONCUSSION-BRIEF COMA */
"85011" = "1" /* CONCUS-BRIEF COMA <31 MN */
"85012" = "1" /* CONCUS-BRF COMA 31-59 MN */
"8502"  = "1" /* CONCUSSION-MODERATE COMA */
"8503"  = "1" /* CONCUSSION-PROLONG COMA */
"8504"  = "1" /* CONCUSSION-DEEP COMA */
"8505"  = "1" /* CONCUSSION W COMA NOS */
"8509"  = "1" /* CONCUSSION NOS */
"85100" = "1" /* CEREBRAL CORTX CONTUSION */
"85101" = "1" /* CORTEX CONTUSION-NO COMA */
"85102" = "1" /* CORTEX CONTUS-BRIEF COMA */
"85103" = "1" /* CORTEX CONTUS-MOD COMA */
"85104" = "1" /* CORTX CONTUS-PROLNG COMA */
"85105" = "1" /* CORTEX CONTUS-DEEP COMA */
"85106" = "1" /* CORTEX CONTUS-COMA NOS */
"85109" = "1" /* CORTEX CONTUS-CONCUS NOS */
"85110" = "1" /* CORTEX CONTUSION/OPN WND */
"85111" = "1" /* OPN CORTX CONTUS-NO COMA */
"85112" = "1" /* OPN CORT CONTUS-BRF COMA */
"85113" = "1" /* OPN CORT CONTUS-MOD COMA */
"85114" = "1" /* OPN CORT CONTU-PROL COMA */
"85115" = "1" /* OPN CORT CONTU-DEEP COMA */
"85116" = "1" /* OPN CORT CONTUS-COMA NOS */
"85119" = "1" /* OPN CORTX CONTUS-CONCUSS */
"85120" = "1" /* CEREBRAL CORTEX LACERAT */
"85121" = "1" /* CORTEX LACERAT W/O COMA */
"85122" = "1" /* CORTEX LACERA-BRIEF COMA */
"85123" = "1" /* CORTEX LACERAT-MOD COMA */
"85124" = "1" /* CORTEX LACERAT-PROL COMA */
"85125" = "1" /* CORTEX LACERAT-DEEP COMA */
"85126" = "1" /* CORTEX LACERAT-COMA NOS */
"85129" = "1" /* CORTEX LACERAT-CONCUSS */
"85130" = "1" /* CORTEX LACER W OPN WOUND */
"85131" = "1" /* OPN CORTEX LACER-NO COMA */
"85132" = "1" /* OPN CORTX LAC-BRIEF COMA */
"85133" = "1" /* OPN CORTX LACER-MOD COMA */
"85134" = "1" /* OPN CORTX LAC-PROLN COMA */
"85135" = "1" /* OPN CORTEX LAC-DEEP COMA */
"85136" = "1" /* OPN CORTX LACER-COMA NOS */
"85139" = "1" /* OPN CORTX LACER-CONCUSS */
"85140" = "1" /* CEREBEL/BRAIN STM CONTUS */
"85141" = "1" /* CEREBELL CONTUS W/O COMA */
"85142" = "1" /* CEREBELL CONTUS-BRF COMA */
"85143" = "1" /* CEREBELL CONTUS-MOD COMA */
"85144" = "1" /* CEREBEL CONTUS-PROL COMA */
"85145" = "1" /* CEREBEL CONTUS-DEEP COMA */
"85146" = "1" /* CEREBELL CONTUS-COMA NOS */
"85149" = "1" /* CEREBELL CONTUS-CONCUSS */
"85150" = "1" /* CEREBEL CONTUS W OPN WND */
"85151" = "1" /* OPN CEREBE CONT W/O COMA */
"85152" = "1" /* OPN CEREBE CONT-BRF COMA */
"85153" = "1" /* OPN CEREBE CONT-MOD COMA */
"85154" = "1" /* OPN CEREBE CONT-PROL COM */
"85155" = "1" /* OPN CEREBE CONT-DEEP COM */
"85156" = "1" /* OPN CEREBE CONT-COMA NOS */
"85159" = "1" /* OPN CEREBEL CONT-CONCUSS */
"85160" = "1" /* CEREBEL/BRAIN STEM LACER */
"85161" = "1" /* CEREBEL LACERAT W/O COMA */
"85162" = "1" /* CEREBEL LACER-BRIEF COMA */
"85163" = "1" /* CEREBEL LACERAT-MOD COMA */
"85164" = "1" /* CEREBEL LACER-PROLN COMA */
"85165" = "1" /* CEREBELL LACER-DEEP COMA */
"85166" = "1" /* CEREBEL LACERAT-COMA NOS */
"85169" = "1" /* CEREBEL LACER-CONCUSSION */
"85170" = "1" /* CEREBEL LACER W OPEN WND */
"85171" = "1" /* OPN CEREBEL LAC W/O COMA */
"85172" = "1" /* OPN CEREBEL LAC-BRF COMA */
"85173" = "1" /* OPN CEREBEL LAC-MOD COMA */
"85174" = "1" /* OPN CEREBE LAC-PROL COMA */
"85175" = "1" /* OPN CEREBE LAC-DEEP COMA */
"85176" = "1" /* OPN CEREBEL LAC-COMA NOS */
"85179" = "1" /* OPN CEREBELL LAC-CONCUSS */
"85180" = "1" /* BRAIN LACERATION NEC */
"85181" = "1" /* BRAIN LACER NEC W/O COMA */
"85182" = "1" /* BRAIN LAC NEC-BRIEF COMA */
"85183" = "1" /* BRAIN LACER NEC-MOD COMA */
"85184" = "1" /* BRAIN LAC NEC-PROLN COMA */
"85185" = "1" /* BRAIN LAC NEC-DEEP COMA */
"85186" = "1" /* BRAIN LACER NEC-COMA NOS */
"85189" = "1" /* BRAIN LACER NEC-CONCUSS */
"85190" = "1" /* BRAIN LAC NEC W OPEN WND */
"85191" = "1" /* OPN BRAIN LACER W/O COMA */
"85192" = "1" /* OPN BRAIN LAC-BRIEF COMA */
"85193" = "1" /* OPN BRAIN LACER-MOD COMA */
"85194" = "1" /* OPN BRAIN LAC-PROLN COMA */
"85195" = "1" /* OPEN BRAIN LAC-DEEP COMA */
"85196" = "1" /* OPN BRAIN LACER-COMA NOS */
"85199" = "1" /* OPEN BRAIN LACER-CONCUSS */
"85200" = "1" /* TRAUM SUBARACHNOID HEM */
"85201" = "1" /* SUBARACHNOID HEM-NO COMA */
"85202" = "1" /* SUBARACH HEM-BRIEF COMA */
"85203" = "1" /* SUBARACH HEM-MOD COMA */
"85204" = "1" /* SUBARACH HEM-PROLNG COMA */
"85205" = "1" /* SUBARACH HEM-DEEP COMA */
"85206" = "1" /* SUBARACH HEM-COMA NOS */
"85209" = "1" /* SUBARACH HEM-CONCUSSION */
"85210" = "1" /* SUBARACH HEM W OPN WOUND */
"85211" = "1" /* OPN SUBARACH HEM-NO COMA */
"85212" = "1" /* OP SUBARACH HEM-BRF COMA */
"85213" = "1" /* OP SUBARACH HEM-MOD COMA */
"85214" = "1" /* OP SUBARACH HEM-PROL COM */
"85215" = "1" /* OP SUBARACH HEM-DEEP COM */
"85216" = "1" /* OP SUBARACH HEM-COMA NOS */
"85219" = "1" /* OPN SUBARACH HEM-CONCUSS */
"85220" = "1" /* TRAUMATIC SUBDURAL HEM */
"85221" = "1" /* SUBDURAL HEM W/O COMA */
"85222" = "1" /* SUBDURAL HEM-BRIEF COMA */
"85223" = "1" /* SUBDURAL HEMORR-MOD COMA */
"85224" = "1" /* SUBDURAL HEM-PROLNG COMA */
"85225" = "1" /* SUBDURAL HEM-DEEP COMA */
"85226" = "1" /* SUBDURAL HEMORR-COMA NOS */
"85229" = "1" /* SUBDURAL HEM-CONCUSSION */
"85230" = "1" /* SUBDURAL HEM W OPN WOUND */
"85231" = "1" /* OPEN SUBDUR HEM W/O COMA */
"85232" = "1" /* OPN SUBDUR HEM-BRF COMA */
"85233" = "1" /* OPN SUBDUR HEM-MOD COMA */
"85234" = "1" /* OPN SUBDUR HEM-PROL COMA */
"85235" = "1" /* OPN SUBDUR HEM-DEEP COMA */
"85236" = "1" /* OPN SUBDUR HEM-COMA NOS */
"85239" = "1" /* OPN SUBDUR HEM-CONCUSS */
"85240" = "1" /* TRAUMATIC EXTRADURAL HEM */
"85241" = "1" /* EXTRADURAL HEM W/O COMA */
"85242" = "1" /* EXTRADUR HEM-BRIEF COMA */
"85243" = "1" /* EXTRADURAL HEM-MOD COMA */
"85244" = "1" /* EXTRADUR HEM-PROLN COMA */
"85245" = "1" /* EXTRADURAL HEM-DEEP COMA */
"85246" = "1" /* EXTRADURAL HEM-COMA NOS */
"85249" = "1" /* EXTADURAL HEM-CONCUSS */
"85250" = "1" /* EXTRADURAL HEM W OPN WND */
"85251" = "1" /* EXTRADURAL HEMOR-NO COMA */
"85252" = "1" /* EXTRADUR HEM-BRIEF COMA */
"85253" = "1" /* EXTRADURAL HEM-MOD COMA */
"85254" = "1" /* EXTRADUR HEM-PROLN COMA */
"85255" = "1" /* EXTRADUR HEM-DEEP COMA */
"85256" = "1" /* EXTRADURAL HEM-COMA NOS */
"85259" = "1" /* EXTRADURAL HEM-CONCUSS */
"85300" = "1" /* TRAUMATIC BRAIN HEM NEC */
"85301" = "1" /* BRAIN HEM NEC W/O COMA */
"85302" = "1" /* BRAIN HEM NEC-BRIEF COMA */
"85303" = "1" /* BRAIN HEM NEC-MOD COMA */
"85304" = "1" /* BRAIN HEM NEC-PROLN COMA */
"85305" = "1" /* BRAIN HEM NEC-DEEP COMA */
"85306" = "1" /* BRAIN HEM NEC-COMA NOS */
"85309" = "1" /* BRAIN HEM NEC-CONCUSSION */
"85310" = "1" /* BRAIN HEM NEC W OPN WND */
"85311" = "1" /* BRAIN HEM OPN W/O COMA */
"85312" = "1" /* BRAIN HEM OPN-BRF COMA */
"85313" = "1" /* BRAIN HEM OPEN-MOD COMA */
"85314" = "1" /* BRAIN HEM OPN-PROLN COMA */
"85315" = "1" /* BRAIN HEM OPEN-DEEP COMA */
"85316" = "1" /* BRAIN HEM OPEN-COMA NOS */
"85319" = "1" /* BRAIN HEM OPN-CONCUSSION */
"85400" = "1" /* BRAIN INJURY NEC */
"85401" = "1" /* BRAIN INJURY NEC-NO COMA */
"85402" = "1" /* BRAIN INJ NEC-BRIEF COMA */
"85403" = "1" /* BRAIN INJ NEC-MOD COMA */
"85404" = "1" /* BRAIN INJ NEC-PROLN COMA */
"85405" = "1" /* BRAIN INJ NEC-DEEP COMA */
"85406" = "1" /* BRAIN INJ NEC-COMA NOS */
"85409" = "1" /* BRAIN INJ NEC-CONCUSSION */
"85410" = "1" /* BRAIN INJURY W OPN WND */
"85411" = "1" /* OPN BRAIN INJ W/O COMA */
"85412" = "1" /* OPN BRAIN INJ-BRIEF COMA */
"85413" = "1" /* OPN BRAIN INJ-MOD COMA */
"85414" = "1" /* OPN BRAIN INJ-PROLN COMA */
"85415" = "1" /* OPN BRAIN INJ-DEEP COMA */
"85416" = "1" /* OPEN BRAIN INJ-COMA NOS */
"85419" = "1" /* OPN BRAIN INJ-CONCUSSION */
"8600"  = "1" /* TRAUM PNEUMOTHORAX-CLOSE */
"8601"  = "1" /* TRAUM PNEUMOTHORAX-OPEN */
"8602"  = "1" /* TRAUM HEMOTHORAX-CLOSED */
"8603"  = "1" /* TRAUM HEMOTHORAX-OPEN */
"8604"  = "1" /* TRAUM PNEUMOHEMOTHOR-CL */
"8605"  = "1" /* TRAUM PNEUMOHEMOTHOR-OPN */
"86100" = "1" /* HEART INJURY NOS-CLOSED */
"86101" = "1" /* HEART CONTUSION-CLOSED */
"86102" = "1" /* HEART LACERATION-CLOSED */
"86103" = "1" /* HEART CHAMBER LACERAT-CL */
"86110" = "1" /* HEART INJURY NOS-OPEN */
"86111" = "1" /* HEART CONTUSION-OPEN */
"86112" = "1" /* HEART LACERATION-OPEN */
"86113" = "1" /* HEART CHAMBER LACER-OPN */
"86120" = "1" /* LUNG INJURY NOS-CLOSED */
"86121" = "1" /* LUNG CONTUSION-CLOSED */
"86122" = "1" /* LUNG LACERATION-CLOSED */
"86130" = "1" /* LUNG INJURY NOS-OPEN */
"86131" = "1" /* LUNG CONTUSION-OPEN */
"86132" = "1" /* LUNG LACERATION-OPEN */
"8620"  = "1" /* DIAPHRAGM INJURY-CLOSED */
"8621"  = "1" /* DIAPHRAGM INJURY-OPEN */
"86221" = "1" /* BRONCHUS INJURY-CLOSED */
"86222" = "1" /* ESOPHAGUS INJURY-CLOSED */
"86229" = "1" /* INTRATHORACIC INJ NEC-CL */
"86231" = "1" /* BRONCHUS INJURY-OPEN */
"86232" = "1" /* ESOPHAGUS INJURY-OPEN */
"86239" = "1" /* INTRATHORAC INJ NEC-OPEN */
"8628"  = "1" /* INTRATHORACIC INJ NOS-CL */
"8629"  = "1" /* INTRATHORAC INJ NOS-OPEN */
"8630"  = "1" /* STOMACH INJURY-CLOSED */
"8631"  = "1" /* STOMACH INJURY-OPEN */
"86320" = "1" /* SMALL INTEST INJ NOS-CL */
"86321" = "1" /* DUODENUM INJURY-CLOSED */
"86329" = "1" /* SMALL INTEST INJ NEC-CL */
"86330" = "1" /* SMALL INTEST INJ NOS-OPN */
"86331" = "1" /* DUODENUM INJURY-OPEN */
"86339" = "1" /* SMALL INTEST INJ NEC-OPN */
"86340" = "1" /* COLON INJURY NOS-CLOSED */
"86341" = "1" /* ASCENDING COLON INJ-CLOS */
"86342" = "1" /* TRANSVERSE COLON INJ-CL */
"86343" = "1" /* DESCENDING COLON INJ-CL */
"86344" = "1" /* SIGMOID COLON INJ-CLOSED */
"86345" = "1" /* RECTUM INJURY-CLOSED */
"86346" = "1" /* COLON INJ MULT SITE-CLOS */
"86349" = "1" /* COLON INJURY NEC-CLOSED */
"86350" = "1" /* COLON INJURY NOS-OPEN */
"86351" = "1" /* ASCENDING COLON INJ-OPEN */
"86352" = "1" /* TRANSVERSE COLON INJ-OPN */
"86353" = "1" /* DESCENDING COLON INJ-OPN */
"86354" = "1" /* SIGMOID COLON INJ-OPEN */
"86355" = "1" /* RECTUM INJURY-OPEN */
"86356" = "1" /* COLON INJ MULT SITE-OPEN */
"86359" = "1" /* COLON INJURY NEC-OPEN */
"86380" = "1" /* GI INJURY NOS-CLOSED */
"86381" = "1" /* PANCREAS, HEAD INJ-CLOSE */
"86382" = "1" /* PANCREAS, BODY INJ-CLOSE */
"86383" = "1" /* PANCREAS, TAIL INJ-CLOSE */
"86384" = "1" /* PANCREAS INJURY NOS-CLOS */
"86385" = "1" /* APPENDIX INJURY-CLOSED */
"86389" = "1" /* GI INJURY NEC-CLOSED */
"86390" = "1" /* GI INJURY NOS-OPEN */
"86391" = "1" /* PANCREAS, HEAD INJ-OPEN */
"86392" = "1" /* PANCREAS, BODY INJ-OPEN */
"86393" = "1" /* PANCREAS, TAIL INJ-OPEN */
"86394" = "1" /* PANCREAS INJURY NOS-OPEN */
"86395" = "1" /* APPENDIX INJURY-OPEN */
"86399" = "1" /* GI INJURY NEC-OPEN */
"86400" = "1" /* LIVER INJURY NOS-CLOSED */
"86401" = "1" /* LIVER HEMATOMA/CONTUSION */
"86402" = "1" /* LIVER LACERATION, MINOR */
"86403" = "1" /* LIVER LACERATION, MOD */
"86404" = "1" /* LIVER LACERATION, MAJOR */
"86405" = "1" /* LIVER LACERAT UNSPCF CLS */
"86409" = "1" /* LIVER INJURY NEC-CLOSED */
"86410" = "1" /* LIVER INJURY NOS-OPEN */
"86411" = "1" /* LIVER HEMATOM/CONTUS-OPN */
"86412" = "1" /* LIVER LACERAT, MINOR-OPN */
"86413" = "1" /* LIVER LACERAT, MOD-OPEN */
"86414" = "1" /* LIVER LACERAT, MAJOR-OPN */
"86415" = "1" /* LIVER LACERAT UNSPCF OPN */
"86419" = "1" /* LIVER INJURY NEC-OPEN */
"86500" = "1" /* SPLEEN INJURY NOS-CLOSED */
"86501" = "1" /* SPLEEN HEMATOMA-CLOSED */
"86502" = "1" /* SPLEEN CAPSULAR TEAR */
"86503" = "1" /* SPLEEN PARENCHYMA LACER */
"86504" = "1" /* SPLEEN DISRUPTION-CLOS */
"86509" = "1" /* SPLEEN INJURY NEC-CLOSED */
"86510" = "1" /* SPLEEN INJURY NOS-OPEN */
"86511" = "1" /* SPLEEN HEMATOMA-OPEN */
"86512" = "1" /* SPLEEN CAPSULAR TEAR-OPN */
"86513" = "1" /* SPLEEN PARNCHYM LAC-OPN */
"86514" = "1" /* SPLEEN DISRUPTION-OPEN */
"86519" = "1" /* SPLEEN INJURY NEC-OPEN */
"86600" = "1" /* KIDNEY INJURY NOS-CLOSED */
"86601" = "1" /* KIDNEY HEMATOMA-CLOSED */
"86602" = "1" /* KIDNEY LACERATION-CLOSED */
"86603" = "1" /* KIDNEY DISRUPTION-CLOSED */
"86610" = "1" /* KIDNEY INJURY NOS-OPEN */
"86611" = "1" /* KIDNEY HEMATOMA-OPEN */
"86612" = "1" /* KIDNEY LACERATION-OPEN */
"86613" = "1" /* KIDNEY DISRUPTION-OPEN */
"8670"  = "1" /* BLADDER/URETHRA INJ-CLOS */
"8671"  = "1" /* BLADDER/URETHRA INJ-OPEN */
"8672"  = "1" /* URETER INJURY-CLOSED */
"8673"  = "1" /* URETER INJURY-OPEN */
"8674"  = "1" /* UTERUS INJURY-CLOSED */
"8675"  = "1" /* UTERUS INJURY-OPEN */
"8676"  = "1" /* PELVIC ORGAN INJ NEC-CL */
"8677"  = "1" /* PELVIC ORGAN INJ NEC-OPN */
"8678"  = "1" /* PELVIC ORGAN INJ NOS-CL */
"8679"  = "1" /* PELVIC ORGAN INJ NOS-OPN */
"86800" = "1" /* INTRA-ABDOM INJ NOS-CLOS */
"86801" = "1" /* ADRENAL GLAND INJURY-CL */
"86802" = "1" /* BILIARY TRACT INJURY-CL */
"86803" = "1" /* PERITONEUM INJURY-CLOSED */
"86804" = "1" /* RETROPERITONEUM INJ-CL */
"86809" = "1" /* INTRA-ABDOM INJ NEC-CLOS */
"86810" = "1" /* INTRA-ABDOM INJ NOS-OPEN */
"86811" = "1" /* ADRENAL GLAND INJURY-OPN */
"86812" = "1" /* BILIARY TRACT INJURY-OPN */
"86813" = "1" /* PERITONEUM INJURY-OPEN */
"86814" = "1" /* RETROPERITONEUM INJ-OPEN */
"86819" = "1" /* INTRA-ABDOM INJ NEC-OPEN */
"8690"  = "1" /* INTERNAL INJ NOS-CLOSED */
"8691"  = "1" /* INTERNAL INJURY NOS-OPEN */
"8700"  = "1" /* LAC EYELID SKN/PERIOCULR */
"8701"  = "1" /* FULL-THICKNES LAC EYELID */
"8702"  = "1" /* LAC EYELID INV LACRM PAS */
"8703"  = "1" /* PENETR WND ORBIT W/O FB */
"8704"  = "1" /* PENETRAT WND ORBIT W FB */
"8708"  = "1" /* OPN WND OCULAR ADNEX NEC */
"8709"  = "1" /* OPN WND OCULAR ADNEX NOS */
"8710"  = "1" /* OCULAR LAC W/O PROLAPSE */
"8711"  = "1" /* OCULAR LACERA W PROLAPSE */
"8712"  = "1" /* RUPTURE EYE W TISSU LOSS */
"8713"  = "1" /* AVULSION OF EYE */
"8714"  = "1" /* LACERATION OF EYE NOS */
"8715"  = "1" /* PENETRAT MAGNET FB EYE */
"8716"  = "1" /* PENETRAT FB NEC EYE */
"8717"  = "1" /* OCULAR PENETRATION NOS */
"8719"  = "1" /* OPN WOUND OF EYEBALL NOS */
"87200" = "1" /* OPN WOUND EXTERN EAR NOS */
"87201" = "1" /* OPEN WOUND OF AURICLE */
"87202" = "1" /* OPN WOUND AUDITORY CANAL */
"87210" = "1" /* OPN WND EX EAR NOS-COMPL */
"87211" = "1" /* OPEN WOUND AURICLE-COMPL */
"87212" = "1" /* OPEN WND AUD CANAL-COMPL */
"87261" = "1" /* OPEN WOUND OF EAR DRUM */
"87262" = "1" /* OPEN WOUND OF OSSICLES */
"87263" = "1" /* OPEN WND EUSTACHIAN TUBE */
"87264" = "1" /* OPEN WOUND OF COCHLEA */
"87269" = "1" /* OPEN WOUND OF EAR NEC */
"87271" = "1" /* OPEN WND EAR DRUM-COMPL */
"87272" = "1" /* OPEN WND OSSICLES-COMPL */
"87273" = "1" /* OPN WND EUSTACH TB-COMPL */
"87274" = "1" /* OPEN WOUND COCHLEA-COMPL */
"87279" = "1" /* OPEN WOUND EAR NEC-COMPL */
"8728"  = "1" /* OPEN WOUND OF EAR NOS */
"8729"  = "1" /* OPEN WOUND EAR NOS-COMPL */
"8730"  = "1" /* OPEN WOUND OF SCALP */
"8731"  = "1" /* OPEN WOUND SCALP-COMPL */
"87320" = "1" /* OPEN WOUND OF NOSE NOS */
"87321" = "1" /* OPEN WOUND NASAL SEPTUM */
"87322" = "1" /* OPEN WOUND NASAL CAVITY */
"87323" = "1" /* OPEN WOUND NASAL SINUS */
"87329" = "1" /* MULT OPEN WOUND NOSE */
"87330" = "1" /* OPEN WND NOSE NOS-COMPL */
"87331" = "1" /* OPN WND NAS SEPTUM-COMPL */
"87332" = "1" /* OPEN WND NASAL CAV-COMPL */
"87333" = "1" /* OPEN WND NAS SINUS-COMPL */
"87339" = "1" /* MULT OPEN WND NOSE-COMPL */
"87340" = "1" /* OPEN WOUND OF FACE NOS */
"87341" = "1" /* OPEN WOUND OF CHEEK */
"87342" = "1" /* OPEN WOUND OF FOREHEAD */
"87343" = "1" /* OPEN WOUND OF LIP */
"87344" = "1" /* OPEN WOUND OF JAW */
"87349" = "1" /* OPEN WOUND OF FACE NEC */
"87350" = "1" /* OPEN WND FACE NOS-COMPL */
"87351" = "1" /* OPEN WOUND CHEEK-COMPL */
"87352" = "1" /* OPEN WND FOREHEAD-COMPL */
"87353" = "1" /* OPEN WOUND LIP-COMPLICAT */
"87354" = "1" /* OPEN WOUND JAW-COMPLICAT */
"87359" = "1" /* OPEN WND FACE NEC-COMPL */
"87360" = "1" /* OPEN WOUND OF MOUTH NOS */
"87361" = "1" /* OPEN WOUND BUCCAL MUCOSA */
"87362" = "1" /* OPEN WOUND OF GUM */
"87363" = "1" /* BROKEN TOOTH */
"87364" = "1" /* OPN WND TONGUE/MOUTH FLR */
"87365" = "1" /* OPEN WOUND OF PALATE */
"87369" = "1" /* OPEN WOUND MOUTH NEC */
"87370" = "1" /* OPEN WND MOUTH NOS-COMPL */
"87371" = "1" /* OPN WND BUC MUCOSA-COMPL */
"87372" = "1" /* OPEN WOUND GUM-COMPL */
"87373" = "1" /* BROKEN TOOTH-COMPLICATED */
"87374" = "1" /* OPEN WOUND TONGUE-COMPL */
"87375" = "1" /* OPEN WOUND PALATE-COMPL */
"87379" = "1" /* OPEN WND MOUTH NOS-COMPL */
"8738"  = "1" /* OPEN WOUND OF HEAD NEC */
"8739"  = "1" /* OPEN WND HEAD NEC-COMPL */
"87400" = "1" /* OPN WND LARYNX W TRACHEA */
"87401" = "1" /* OPEN WOUND OF LARYNX */
"87402" = "1" /* OPEN WOUND OF TRACHEA */
"87410" = "1" /* OPN WND LARY W TRAC-COMP */
"87411" = "1" /* OPEN WOUND LARYNX-COMPL */
"87412" = "1" /* OPEN WOUND TRACHEA-COMPL */
"8742"  = "1" /* OPEN WOUND THYROID GLAND */
"8743"  = "1" /* OPEN WOUND THYROID-COMPL */
"8744"  = "1" /* OPEN WOUND OF PHARYNX */
"8745"  = "1" /* OPEN WOUND PHARYNX-COMPL */
"8748"  = "1" /* OPEN WOUND OF NECK NEC */
"8749"  = "1" /* OPN WOUND NECK NEC-COMPL */
"8750"  = "1" /* OPEN WOUND OF CHEST */
"8751"  = "1" /* OPEN WOUND CHEST-COMPL */
"8760"  = "1" /* OPEN WOUND OF BACK */
"8761"  = "1" /* OPEN WOUND BACK-COMPL */
"8770"  = "1" /* OPEN WOUND OF BUTTOCK */
"8771"  = "1" /* OPEN WOUND BUTTOCK-COMPL */
"8780"  = "1" /* OPEN WOUND OF PENIS */
"8781"  = "1" /* OPEN WOUND PENIS-COMPL */
"8782"  = "1" /* OPN WOUND SCROTUM/TESTES */
"8783"  = "1" /* OPN WND SCROT/TEST-COMPL */
"8784"  = "1" /* OPEN WOUND OF VULVA */
"8785"  = "1" /* OPEN WOUND VULVA-COMPL */
"8786"  = "1" /* OPEN WOUND OF VAGINA */
"8787"  = "1" /* OPEN WOUND VAGINA-COMPL */
"8788"  = "1" /* OPEN WOUND GENITAL NEC */
"8789"  = "1" /* OPN WND GENITAL NEC-COMP */
"8790"  = "1" /* OPEN WOUND OF BREAST */
"8791"  = "1" /* OPEN WOUND BREAST-COMPL */
"8792"  = "1" /* OPN WND ANTERIOR ABDOMEN */
"8793"  = "1" /* OPN WND ANT ABDOMEN-COMP */
"8794"  = "1" /* OPN WND LATERAL ABDOMEN */
"8795"  = "1" /* OPN WND LAT ABDOMEN-COMP */
"8796"  = "1" /* OPEN WOUND OF TRUNK NEC */
"8797"  = "1" /* OPEN WND TRUNK NEC-COMPL */
"8798"  = "1" /* OPEN WOUND SITE NOS */
"8799"  = "1" /* OPN WOUND SITE NOS-COMPL */
"88000" = "1" /* OPEN WOUND OF SHOULDER */
"88001" = "1" /* OPEN WOUND OF SCAPULA */
"88002" = "1" /* OPEN WOUND OF AXILLA */
"88003" = "1" /* OPEN WOUND OF UPPER ARM */
"88009" = "1" /* MULT OPEN WOUND SHOULDER */
"88010" = "1" /* OPEN WND SHOULDER-COMPL */
"88011" = "1" /* OPEN WOUND SCAPULA-COMPL */
"88012" = "1" /* OPEN WOUND AXILLA-COMPL */
"88013" = "1" /* OPEN WND UPPER ARM-COMPL */
"88019" = "1" /* MULT OPN WND SHOULD-COMP */
"88020" = "1" /* OPN WND SHOULDR W TENDON */
"88021" = "1" /* OPN WND SCAPULA W TENDON */
"88022" = "1" /* OPEN WND AXILLA W TENDON */
"88023" = "1" /* OPEN WND UP ARM W TENDON */
"88029" = "1" /* MLT OPN WND SHLDR W TEND */
"88100" = "1" /* OPEN WOUND OF FOREARM */
"88101" = "1" /* OPEN WOUND OF ELBOW */
"88102" = "1" /* OPEN WOUND OF WRIST */
"88110" = "1" /* OPEN WOUND FOREARM-COMPL */
"88111" = "1" /* OPEN WOUND ELBOW-COMPLIC */
"88112" = "1" /* OPEN WOUND WRIST-COMPLIC */
"88120" = "1" /* OPEN WND FOREARM W TENDN */
"88121" = "1" /* OPN WOUND ELBOW W TENDON */
"88122" = "1" /* OPN WOUND WRIST W TENDON */
"8820"  = "1" /* OPEN WOUND OF HAND */
"8821"  = "1" /* OPN WOUND HAND-COMPLICAT */
"8822"  = "1" /* OPEN WOUND HAND W TENDON */
"8840"  = "1" /* OPEN WOUND ARM MULT/NOS */
"8841"  = "1" /* OPEN WOUND ARM NOS-COMPL */
"8842"  = "1" /* OPN WND ARM NOS W TENDON */
"8870"  = "1" /* AMPUT BELOW ELB, UNILAT */
"8871"  = "1" /* AMP BELOW ELB, UNIL-COMP */
"8872"  = "1" /* AMPUT ABV ELBOW, UNILAT */
"8873"  = "1" /* AMPUT ABV ELB, UNIL-COMP */
"8874"  = "1" /* AMPUTAT ARM, UNILAT NOS */
"8875"  = "1" /* AMPUT ARM, UNIL NOS-COMP */
"8876"  = "1" /* AMPUTATION ARM, BILAT */
"8877"  = "1" /* AMPUTAT ARM, BILAT-COMPL */
"8900"  = "1" /* OPEN WOUND OF HIP/THIGH */
"8901"  = "1" /* OPEN WND HIP/THIGH-COMPL */
"8902"  = "1" /* OPN WND HIP/THIGH W TEND */
"8910"  = "1" /* OPEN WND KNEE/LEG/ANKLE */
"8911"  = "1" /* OPEN WND KNEE/LEG-COMPL */
"8912"  = "1" /* OPN WND KNEE/LEG W TENDN */
"8920"  = "1" /* OPEN WOUND OF FOOT */
"8921"  = "1" /* OPEN WOUND FOOT-COMPL */
"8922"  = "1" /* OPEN WOUND FOOT W TENDON */
"8940"  = "1" /* OPEN WOUND OF LEG NEC */
"8941"  = "1" /* OPEN WOUND LEG NEC-COMPL */
"8942"  = "1" /* OPN WND LEG NEC W TENDON */
"8960"  = "1" /* AMPUTATION FOOT, UNILAT */
"8961"  = "1" /* AMPUT FOOT, UNILAT-COMPL */
"8962"  = "1" /* AMPUTATION FOOT, BILAT */
"8963"  = "1" /* AMPUTAT FOOT, BILAT-COMP */
"8970"  = "1" /* AMPUT BELOW KNEE, UNILAT */
"8971"  = "1" /* AMPUTAT BK, UNILAT-COMPL */
"8972"  = "1" /* AMPUT ABOVE KNEE, UNILAT */
"8973"  = "1" /* AMPUT ABV KN, UNIL-COMPL */
"8974"  = "1" /* AMPUTAT LEG, UNILAT NOS */
"8975"  = "1" /* AMPUT LEG, UNIL NOS-COMP */
"8976"  = "1" /* AMPUTATION LEG, BILAT */
"8977"  = "1" /* AMPUTAT LEG, BILAT-COMPL */
"90000" = "1" /* INJUR CAROTID ARTERY NOS */
"90001" = "1" /* INJ COMMON CAROTID ARTER */
"90002" = "1" /* INJ EXTERNAL CAROTID ART */
"90003" = "1" /* INJ INTERNAL CAROTID ART */
"9001"  = "1" /* INJ INTERNL JUGULAR VEIN */
"90081" = "1" /* INJ EXTERN JUGULAR VEIN */
"90082" = "1" /* INJ MLT HEAD/NECK VESSEL */
"90089" = "1" /* INJ HEAD/NECK VESSEL NEC */
"9009"  = "1" /* INJ HEAD/NECK VESSEL NOS */
"9010"  = "1" /* INJURY THORACIC AORTA */
"9011"  = "1" /* INJ INNOMIN/SUBCLAV ART */
"9012"  = "1" /* INJ SUPERIOR VENA CAVA */
"9013"  = "1" /* INJ INNOMIN/SUBCLAV VEIN */
"90140" = "1" /* INJ PULMONARY VESSEL NOS */
"90141" = "1" /* INJURY PULMONARY ARTERY */
"90142" = "1" /* INJURY PULMONARY VEIN */
"90181" = "1" /* INJ INTERCOSTAL ART/VEIN */
"90182" = "1" /* INJ INT MAMMARY ART/VEIN */
"90183" = "1" /* INJ MULT THORACIC VESSEL */
"90189" = "1" /* INJ THORACIC VESSEL NEC */
"9019"  = "1" /* INJ THORACIC VESSEL NOS */
"9020"  = "1" /* INJURY ABDOMINAL AORTA */
"90210" = "1" /* INJ INFER VENA CAVA NOS */
"90211" = "1" /* INJURY HEPATIC VEINS */
"90219" = "1" /* INJ INFER VENA CAVA NEC */
"90220" = "1" /* INJ CELIAC/MESEN ART NOS */
"90221" = "1" /* INJURY GASTRIC ARTERY */
"90222" = "1" /* INJURY HEPATIC ARTERY */
"90223" = "1" /* INJURY SPLENIC ARTERY */
"90224" = "1" /* INJURY CELIAC AXIS NEC */
"90225" = "1" /* INJ SUPER MESENTERIC ART */
"90226" = "1" /* INJ BRNCH SUP MESENT ART */
"90227" = "1" /* INJ INFER MESENTERIC ART */
"90229" = "1" /* INJ MESENTERIC VESS NEC */
"90231" = "1" /* INJ SUPERIOR MESENT VEIN */
"90232" = "1" /* INJ INFERIOR MESENT VEIN */
"90233" = "1" /* INJURY PORTAL VEIN */
"90234" = "1" /* INJURY SPLENIC VEIN */
"90239" = "1" /* INJ PORT/SPLEN VESS NEC */
"90240" = "1" /* INJURY RENAL VESSEL NOS */
"90241" = "1" /* INJURY RENAL ARTERY */
"90242" = "1" /* INJURY RENAL VEIN */
"90249" = "1" /* INJURY RENAL VESSEL NEC */
"90250" = "1" /* INJURY ILIAC VESSEL NOS */
"90251" = "1" /* INJ HYPOGASTRIC ARTERY */
"90252" = "1" /* INJURY HYPOGASTRIC VEIN */
"90253" = "1" /* INJURY ILIAC ARTERY */
"90254" = "1" /* INJURY ILIAC VEIN */
"90255" = "1" /* INJURY UTERINE ARTERY */
"90256" = "1" /* INJURY UTERINE VEIN */
"90259" = "1" /* INJURY ILIAC VESSEL NEC */
"90281" = "1" /* INJURY OVARIAN ARTERY */
"90282" = "1" /* INJURY OVARIAN VEIN */
"90287" = "1" /* INJ MULT ABD/PELV VESSEL */
"90289" = "1" /* INJ ABDOMINAL VESSEL NEC */
"9029"  = "1" /* INJ ABDOMINAL VESSEL NOS */
"90300" = "1" /* INJ AXILLARY VESSEL NOS */
"90301" = "1" /* INJURY AXILLARY ARTERY */
"90302" = "1" /* INJURY AXILLARY VEIN */
"9031"  = "1" /* INJURY BRACHIAL VESSELS */
"9032"  = "1" /* INJURY RADIAL VESSELS */
"9033"  = "1" /* INJURY ULNAR VESSELS */
"9034"  = "1" /* INJURY PALMAR ARTERY */
"9035"  = "1" /* INJURY FINGER VESSELS */
"9038"  = "1" /* INJURY ARM VESSELS NEC */
"9039"  = "1" /* INJURY ARM VESSEL NOS */
"9040"  = "1" /* INJ COMMON FEMORAL ARTER */
"9041"  = "1" /* INJ SUPERFIC FEMORAL ART */
"9042"  = "1" /* INJURY FEMORAL VEIN */
"9043"  = "1" /* INJURY SAPHENOUS VEIN */
"90440" = "1" /* INJ POPLITEAL VESSEL NOS */
"90441" = "1" /* INJURY POPLITEAL ARTERY */
"90442" = "1" /* INJURY POPLITEAL VEIN */
"90450" = "1" /* INJURY TIBIAL VESSEL NOS */
"90451" = "1" /* INJ ANTER TIBIAL ARTERY */
"90452" = "1" /* INJ ANTERIOR TIBIAL VEIN */
"90453" = "1" /* INJ POST TIBIAL ARTERY */
"90454" = "1" /* INJ POST TIBIAL VEIN */
"9046"  = "1" /* INJ DEEP PLANTAR VESSEL */
"9047"  = "1" /* INJURY LEG VESSELS NEC */
"9048"  = "1" /* INJURY LEG VESSEL NOS */
"9049"  = "1" /* BLOOD VESSEL INJURY NOS */
"925"   = "1" /* CRUSH INJ FACE/SCALP/NCK */
"9251"  = "1" /* CRUSH INJ FACE SCALP */
"9252"  = "1" /* CRUSH INJ NECK */
"9260"  = "1" /* CRUSH INJ EXT GENITALIA */
"92611" = "1" /* CRUSHING INJURY BACK */
"92612" = "1" /* CRUSHING INJURY BUTTOCK */
"92619" = "1" /* CRUSHING INJ TRUNK NEC */
"9268"  = "1" /* MULT CRUSHING INJ TRUNK */
"9269"  = "1" /* CRUSHING INJ TRUNK NOS */
"92700" = "1" /* CRUSH INJ SHOULDER REG */
"92701" = "1" /* CRUSH INJ SCAPUL REGION */
"92702" = "1" /* CRUSH INJ AXILLARY REG */
"92703" = "1" /* CRUSHING INJ UPPER ARM */
"92709" = "1" /* CRUSH INJ SHOULDER & ARM */
"92710" = "1" /* CRUSHING INJURY FOREARM */
"92711" = "1" /* CRUSHING INJURY ELBOW */
"92720" = "1" /* CRUSHING INJURY OF HAND */
"92721" = "1" /* CRUSHING INJURY OF WRIST */
"9273"  = "1" /* CRUSHING INJURY FINGER */
"9278"  = "1" /* MULT CRUSHING INJURY ARM */
"9279"  = "1" /* CRUSHING INJURY ARM NOS */
"92800" = "1" /* CRUSHING INJURY THIGH */
"92801" = "1" /* CRUSHING INJURY HIP */
"92810" = "1" /* CRUSHING INJ LOWER LEG */
"92811" = "1" /* CRUSHING INJURY KNEE */
"92820" = "1" /* CRUSHING INJURY FOOT */
"92821" = "1" /* CRUSHING INJURY ANKLE */
"9283"  = "1" /* CRUSHING INJURY TOE */
"9288"  = "1" /* MULT CRUSHING INJURY LEG */
"9289"  = "1" /* CRUSHING INJURY LEG NOS */
"9290"  = "1" /* CRUSH INJ MULT SITE NEC */
"9299"  = "1" /* CRUSHING INJURY NOS */
"9400"  = "1" /* CHEMICAL BURN PERIOCULAR */
"9401"  = "1" /* BURN PERIOCULAR AREA NEC */
"9402"  = "1" /* ALKAL BURN CORNEA/CONJUN */
"9403"  = "1" /* ACID BURN CORNEA/CONJUNC */
"9404"  = "1" /* BURN CORNEA/CONJUNCT NEC */
"9405"  = "1" /* BURN W EYEBALL DESTRUCT */
"9409"  = "1" /* BURN EYE & ADNEXA NOS */
"94100" = "1" /* BURN NOS HEAD-UNSPEC */
"94101" = "1" /* BURN NOS EAR */
"94102" = "1" /* BURN NOS EYE */
"94103" = "1" /* BURN NOS LIP */
"94104" = "1" /* BURN NOS CHIN */
"94105" = "1" /* BURN NOS NOSE */
"94106" = "1" /* BURN NOS SCALP */
"94107" = "1" /* BURN NOS FACE NEC */
"94108" = "1" /* BURN NOS NECK */
"94109" = "1" /* BURN NOS HEAD-MULT */
"94110" = "1" /* 1ST DEG BURN HEAD NOS */
"94111" = "1" /* 1ST DEG BURN EAR */
"94112" = "1" /* 1ST DEG BURN EYE */
"94113" = "1" /* 1ST DEG BURN LIP */
"94114" = "1" /* 1ST DEG BURN CHIN */
"94115" = "1" /* 1ST DEG BURN NOSE */
"94116" = "1" /* 1ST DEG BURN SCALP */
"94117" = "1" /* 1ST DEG BURN FACE NEC */
"94118" = "1" /* 1ST DEG BURN NECK */
"94119" = "1" /* 1ST DEG BURN HEAD-MULT */
"94120" = "1" /* 2ND DEG BURN HEAD NOS */
"94121" = "1" /* 2ND DEG BURN EAR */
"94122" = "1" /* 2ND DEG BURN EYE */
"94123" = "1" /* 2ND DEG BURN LIP */
"94124" = "1" /* 2ND DEG BURN CHIN */
"94125" = "1" /* 2ND DEG BURN NOSE */
"94126" = "1" /* 2ND DEG BURN SCALP */
"94127" = "1" /* 2ND DEG BURN FACE NEC */
"94128" = "1" /* 2ND DEG BURN NECK */
"94129" = "1" /* 2ND DEG BURN HEAD-MULT */
"94130" = "1" /* 3RD DEG BURN HEAD NOS */
"94131" = "1" /* 3RD DEG BURN EAR */
"94132" = "1" /* 3RD DEG BURN EYE */
"94133" = "1" /* 3RD DEG BURN LIP */
"94134" = "1" /* 3RD DEG BURN CHIN */
"94135" = "1" /* 3RD DEG BURN NOSE */
"94136" = "1" /* 3RD DEG BURN SCALP */
"94137" = "1" /* 3RD DEG BURN FACE NEC */
"94138" = "1" /* 3RD DEG BURN NECK */
"94139" = "1" /* 3RD DEG BURN HEAD-MULT */
"94140" = "1" /* DEEP 3 DEG BURN HEAD NOS */
"94141" = "1" /* DEEP 3RD DEG BURN EAR */
"94142" = "1" /* DEEP 3RD DEG BURN EYE */
"94143" = "1" /* DEEP 3RD DEG BURN LIP */
"94144" = "1" /* DEEP 3RD DEG BURN CHIN */
"94145" = "1" /* DEEP 3RD DEG BURN NOSE */
"94146" = "1" /* DEEP 3RD DEG BURN SCALP */
"94147" = "1" /* DEEP 3RD BURN FACE NEC */
"94148" = "1" /* DEEP 3RD DEG BURN NECK */
"94149" = "1" /* DEEP 3 DEG BRN HEAD-MULT */
"94150" = "1" /* 3RD BURN W LOSS-HEAD NOS */
"94151" = "1" /* 3RD DEG BURN W LOSS-EAR */
"94152" = "1" /* 3RD DEG BURN W LOSS-EYE */
"94153" = "1" /* 3RD DEG BURN W LOSS-LIP */
"94154" = "1" /* 3RD DEG BURN W LOSS-CHIN */
"94155" = "1" /* 3RD DEG BURN W LOSS-NOSE */
"94156" = "1" /* 3RD DEG BRN W LOSS-SCALP */
"94157" = "1" /* 3RD BURN W LOSS-FACE NEC */
"94158" = "1" /* 3RD DEG BURN W LOSS-NECK */
"94159" = "1" /* 3RD BRN W LOSS-HEAD MULT */
"94200" = "1" /* BURN NOS TRUNK-UNSPEC */
"94201" = "1" /* BURN NOS BREAST */
"94202" = "1" /* BURN NOS CHEST WALL */
"94203" = "1" /* BURN NOS ABDOMINAL WALL */
"94204" = "1" /* BURN NOS BACK */
"94205" = "1" /* BURN NOS GENITALIA */
"94209" = "1" /* BURN NOS TRUNK NEC */
"94210" = "1" /* 1ST DEG BURN TRUNK NOS */
"94211" = "1" /* 1ST DEG BURN BREAST */
"94212" = "1" /* 1ST DEG BURN CHEST WALL */
"94213" = "1" /* 1ST DEG BURN ABDOMN WALL */
"94214" = "1" /* 1ST DEG BURN BACK */
"94215" = "1" /* 1ST DEG BURN GENITALIA */
"94219" = "1" /* 1ST DEG BURN TRUNK NEC */
"94220" = "1" /* 2ND DEG BURN TRUNK NOS */
"94221" = "1" /* 2ND DEG BURN BREAST */
"94222" = "1" /* 2ND DEG BURN CHEST WALL */
"94223" = "1" /* 2ND DEG BURN ABDOMN WALL */
"94224" = "1" /* 2ND DEG BURN BACK */
"94225" = "1" /* 2ND DEG BURN GENITALIA */
"94229" = "1" /* 2ND DEG BURN TRUNK NEC */
"94230" = "1" /* 3RD DEG BURN TRUNK NOS */
"94231" = "1" /* 3RD DEG BURN BREAST */
"94232" = "1" /* 3RD DEG BURN CHEST WALL */
"94233" = "1" /* 3RD DEG BURN ABDOMN WALL */
"94234" = "1" /* 3RD DEG BURN BACK */
"94235" = "1" /* 3RD DEG BURN GENITALIA */
"94239" = "1" /* 3RD DEG BURN TRUNK NEC */
"94240" = "1" /* DEEP 3RD BURN TRUNK NOS */
"94241" = "1" /* DEEP 3RD DEG BURN BREAST */
"94242" = "1" /* DEEP 3RD BURN CHEST WALL */
"94243" = "1" /* DEEP 3RD BURN ABDOM WALL */
"94244" = "1" /* DEEP 3RD DEG BURN BACK */
"94245" = "1" /* DEEP 3RD BURN GENITALIA */
"94249" = "1" /* DEEP 3RD BURN TRUNK NEC */
"94250" = "1" /* 3RD BRN W LOSS-TRUNK NOS */
"94251" = "1" /* 3RD BURN W LOSS-BREAST */
"94252" = "1" /* 3RD BRN W LOSS-CHEST WLL */
"94253" = "1" /* 3RD BRN W LOSS-ABDOM WLL */
"94254" = "1" /* 3RD DEG BURN W LOSS-BACK */
"94255" = "1" /* 3RD BRN W LOSS-GENITALIA */
"94259" = "1" /* 3RD BRN W LOSS-TRUNK NEC */
"94300" = "1" /* BURN NOS ARM-UNSPEC */
"94301" = "1" /* BURN NOS FOREARM */
"94302" = "1" /* BURN NOS ELBOW */
"94303" = "1" /* BURN NOS UPPER ARM */
"94304" = "1" /* BURN NOS AXILLA */
"94305" = "1" /* BURN NOS SHOULDER */
"94306" = "1" /* BURN NOS SCAPULA */
"94309" = "1" /* BURN NOS ARM-MULTIPLE */
"94310" = "1" /* 1ST DEG BURN ARM NOS */
"94311" = "1" /* 1ST DEG BURN FOREARM */
"94312" = "1" /* 1ST DEG BURN ELBOW */
"94313" = "1" /* 1ST DEG BURN UPPER ARM */
"94314" = "1" /* 1ST DEG BURN AXILLA */
"94315" = "1" /* 1ST DEG BURN SHOULDER */
"94316" = "1" /* 1ST DEG BURN SCAPULA */
"94319" = "1" /* 1ST DEG BURN ARM-MULT */
"94320" = "1" /* 2ND DEG BURN ARM NOS */
"94321" = "1" /* 2ND DEG BURN FOREARM */
"94322" = "1" /* 2ND DEG BURN ELBOW */
"94323" = "1" /* 2ND DEG BURN UPPER ARM */
"94324" = "1" /* 2ND DEG BURN AXILLA */
"94325" = "1" /* 2ND DEG BURN SHOULDER */
"94326" = "1" /* 2ND DEG BURN SCAPULA */
"94329" = "1" /* 2ND DEG BURN ARM-MULT */
"94330" = "1" /* 3RD DEG BURN ARM NOS */
"94331" = "1" /* 3RD DEG BURN FOREARM */
"94332" = "1" /* 3RD DEG BURN ELBOW */
"94333" = "1" /* 3RD DEG BURN UPPER ARM */
"94334" = "1" /* 3RD DEG BURN AXILLA */
"94335" = "1" /* 3RD DEG BURN SHOULDER */
"94336" = "1" /* 3RD DEG BURN SCAPULA */
"94339" = "1" /* 3RD DEG BURN ARM-MULT */
"94340" = "1" /* DEEP 3 DEG BURN ARM NOS */
"94341" = "1" /* DEEP 3 DEG BURN FOREARM */
"94342" = "1" /* DEEP 3 DEG BURN ELBOW */
"94343" = "1" /* DEEP 3 DEG BRN UPPER ARM */
"94344" = "1" /* DEEP 3 DEG BURN AXILLA */
"94345" = "1" /* DEEP 3 DEG BURN SHOULDER */
"94346" = "1" /* DEEP 3 DEG BURN SCAPULA */
"94349" = "1" /* DEEP 3 DEG BURN ARM-MULT */
"94350" = "1" /* 3RD BURN W LOSS-ARM NOS */
"94351" = "1" /* 3RD BURN W LOSS-FOREARM */
"94352" = "1" /* 3RD BURN W LOSS-ELBOW */
"94353" = "1" /* 3RD BRN W LOSS-UPPER ARM */
"94354" = "1" /* 3RD BURN W LOSS-AXILLA */
"94355" = "1" /* 3RD BURN W LOSS-SHOULDER */
"94356" = "1" /* 3RD BURN W LOSS-SCAPULA */
"94359" = "1" /* 3RD BURN W LOSS ARM-MULT */
"94400" = "1" /* BURN NOS HAND-UNSPEC */
"94401" = "1" /* BURN NOS FINGER */
"94402" = "1" /* BURN NOS THUMB */
"94403" = "1" /* BURN NOS MULT FINGERS */
"94404" = "1" /* BURN NOS FINGER W THUMB */
"94405" = "1" /* BURN NOS PALM */
"94406" = "1" /* BURN NOS BACK OF HAND */
"94407" = "1" /* BURN NOS WRIST */
"94408" = "1" /* BURN NOS HAND-MULTIPLE */
"94410" = "1" /* 1ST DEG BURN HAND NOS */
"94411" = "1" /* 1ST DEG BURN FINGER */
"94412" = "1" /* 1ST DEG BURN THUMB */
"94413" = "1" /* 1ST DEG BURN MULT FINGER */
"94414" = "1" /* 1 DEG BURN FINGR W THUMB */
"94415" = "1" /* 1ST DEG BURN PALM */
"94416" = "1" /* 1 DEG BURN BACK OF HAND */
"94417" = "1" /* 1ST DEG BURN WRIST */
"94418" = "1" /* 1ST DEG BURN HAND-MULT */
"94420" = "1" /* 2ND DEG BURN HAND NOS */
"94421" = "1" /* 2ND DEG BURN FINGER */
"94422" = "1" /* 2ND DEG BURN THUMB */
"94423" = "1" /* 2ND DEG BURN MULT FINGER */
"94424" = "1" /* 2 DEG BURN FINGR W THUMB */
"94425" = "1" /* 2ND DEG BURN PALM */
"94426" = "1" /* 2 DEG BURN BACK OF HAND */
"94427" = "1" /* 2ND DEG BURN WRIST */
"94428" = "1" /* 2ND DEG BURN HAND-MULT */
"94430" = "1" /* 3RD DEG BURN HAND NOS */
"94431" = "1" /* 3RD DEG BURN FINGER */
"94432" = "1" /* 3RD DEG BURN THUMB */
"94433" = "1" /* 3RD DEG BURN MULT FINGER */
"94434" = "1" /* 3 DEG BURN FINGR W THUMB */
"94435" = "1" /* 3RD DEG BURN PALM */
"94436" = "1" /* 3 DEG BURN BACK OF HAND */
"94437" = "1" /* 3RD DEG BURN WRIST */
"94438" = "1" /* 3RD DEG BURN HAND-MULT */
"94440" = "1" /* DEEP 3 DEG BRN HAND NOS */
"94441" = "1" /* DEEP 3 DEG BURN FINGER */
"94442" = "1" /* DEEP 3 DEG BURN THUMB */
"94443" = "1" /* DEEP 3RD BRN MULT FINGER */
"94444" = "1" /* DEEP 3RD BRN FNGR W THMB */
"94445" = "1" /* DEEP 3 DEG BURN PALM */
"94446" = "1" /* DEEP 3RD BRN BACK OF HND */
"94447" = "1" /* DEEP 3 DEG BURN WRIST */
"94448" = "1" /* DEEP 3 DEG BRN HAND-MULT */
"94450" = "1" /* 3RD BRN W LOSS-HAND NOS */
"94451" = "1" /* 3RD BURN W LOSS-FINGER */
"94452" = "1" /* 3RD BURN W LOSS-THUMB */
"94453" = "1" /* 3RD BRN W LOSS-MULT FNGR */
"94454" = "1" /* 3RD BRN W LOSS-FNGR/THMB */
"94455" = "1" /* 3RD BURN W LOSS-PALM */
"94456" = "1" /* 3RD BRN W LOSS-BK OF HND */
"94457" = "1" /* 3RD BURN W LOSS-WRIST */
"94458" = "1" /* 3RD BRN W LOSS HAND-MULT */
"94500" = "1" /* BURN NOS LEG-UNSPEC */
"94501" = "1" /* BURN NOS TOE */
"94502" = "1" /* BURN NOS FOOT */
"94503" = "1" /* BURN NOS ANKLE */
"94504" = "1" /* BURN NOS LOWER LEG */
"94505" = "1" /* BURN NOS KNEE */
"94506" = "1" /* BURN NOS THIGH */
"94509" = "1" /* BURN NOS LEG-MULTIPLE */
"94510" = "1" /* 1ST DEG BURN LEG NOS */
"94511" = "1" /* 1ST DEG BURN TOE */
"94512" = "1" /* 1ST DEG BURN FOOT */
"94513" = "1" /* 1ST DEG BURN ANKLE */
"94514" = "1" /* 1ST DEG BURN LOWER LEG */
"94515" = "1" /* 1ST DEG BURN KNEE */
"94516" = "1" /* 1ST DEG BURN THIGH */
"94519" = "1" /* 1ST DEG BURN LEG-MULT */
"94520" = "1" /* 2ND DEG BURN LEG NOS */
"94521" = "1" /* 2ND DEG BURN TOE */
"94522" = "1" /* 2ND DEG BURN FOOT */
"94523" = "1" /* 2ND DEG BURN ANKLE */
"94524" = "1" /* 2ND DEG BURN LOWER LEG */
"94525" = "1" /* 2ND DEG BURN KNEE */
"94526" = "1" /* 2ND DEG BURN THIGH */
"94529" = "1" /* 2ND DEG BURN LEG-MULT */
"94530" = "1" /* 3RD DEG BURN LEG NOS */
"94531" = "1" /* 3RD DEG BURN TOE */
"94532" = "1" /* 3RD DEG BURN FOOT */
"94533" = "1" /* 3RD DEG BURN ANKLE */
"94534" = "1" /* 3RD DEG BURN LOW LEG */
"94535" = "1" /* 3RD DEG BURN KNEE */
"94536" = "1" /* 3RD DEG BURN THIGH */
"94539" = "1" /* 3RD DEG BURN LEG-MULT */
"94540" = "1" /* DEEP 3RD DEG BRN LEG NOS */
"94541" = "1" /* DEEP 3RD DEG BURN TOE */
"94542" = "1" /* DEEP 3RD DEG BURN FOOT */
"94543" = "1" /* DEEP 3RD DEG BURN ANKLE */
"94544" = "1" /* DEEP 3RD DEG BRN LOW LEG */
"94545" = "1" /* DEEP 3RD DEG BURN KNEE */
"94546" = "1" /* DEEP 3RD DEG BURN THIGH */
"94549" = "1" /* DEEP 3 DEG BURN LEG-MULT */
"94550" = "1" /* 3 DEG BRN W LOSS-LEG NOS */
"94551" = "1" /* 3 DEG BURN W LOSS-TOE */
"94552" = "1" /* 3 DEG BURN W LOSS-FOOT */
"94553" = "1" /* 3 DEG BURN W LOSS-ANKLE */
"94554" = "1" /* 3 DEG BRN W LOSS-LOW LEG */
"94555" = "1" /* 3 DEG BURN W LOSS-KNEE */
"94556" = "1" /* 3 DEG BURN W LOSS-THIGH */
"94559" = "1" /* 3 DEG BRN W LOSS LEG-MLT */
"9460"  = "1" /* BURN NOS MULTIPLE SITE */
"9461"  = "1" /* 1ST DEG BURN MULT SITE */
"9462"  = "1" /* 2ND DEG BURN MULT SITE */
"9463"  = "1" /* 3RD DEG BURN MULT SITE */
"9464"  = "1" /* DEEP 3 DEG BRN MULT SITE */
"9465"  = "1" /* 3RD BRN W LOSS-MULT SITE */
"9470"  = "1" /* BURN OF MOUTH & PHARYNX */
"9471"  = "1" /* BURN LARYNX/TRACHEA/LUNG */
"9472"  = "1" /* BURN OF ESOPHAGUS */
"9473"  = "1" /* BURN OF GI TRACT */
"9474"  = "1" /* BURN OF VAGINA & UTERUS */
"9478"  = "1" /* BURN INTERNAL ORGAN NEC */
"9479"  = "1" /* BURN INTERNAL ORGAN NOS */
"94800" = "1" /* BDY BRN < 10%/3D DEG NOS */
"94810" = "1" /* 10-19% BDY BRN/3 DEG NOS */
"94811" = "1" /* 10-19% BDY BRN/10-19% 3D */
"94820" = "1" /* 20-29% BDY BRN/3 DEG NOS */
"94821" = "1" /* 20-29% BDY BRN/10-19% 3D */
"94822" = "1" /* 20-29% BDY BRN/20-29% 3D */
"94830" = "1" /* 30-39% BDY BRN/3 DEG NOS */
"94831" = "1" /* 30-39% BDY BRN/10-19% 3D */
"94832" = "1" /* 30-39% BDY BRN/20-29% 3D */
"94833" = "1" /* 30-39% BDY BRN/30-39% 3D */
"94840" = "1" /* 40-49% BDY BRN/3 DEG NOS */
"94841" = "1" /* 40-49% BDY BRN/10-19% 3D */
"94842" = "1" /* 40-49% BDY BRN/20-29% 3D */
"94843" = "1" /* 40-49% BDY BRN/30-39% 3D */
"94844" = "1" /* 40-49% BDY BRN/40-49% 3D */
"94850" = "1" /* 50-59% BDY BRN/3 DEG NOS */
"94851" = "1" /* 50-59% BDY BRN/10-19% 3D */
"94852" = "1" /* 50-59% BDY BRN/20-29% 3D */
"94853" = "1" /* 50-59% BDY BRN/30-39% 3D */
"94854" = "1" /* 50-59% BDY BRN/40-49% 3D */
"94855" = "1" /* 50-59% BDY BRN/50-59% 3D */
"94860" = "1" /* 60-69% BDY BRN/3 DEG NOS */
"94861" = "1" /* 60-69% BDY BRN/10-19% 3D */
"94862" = "1" /* 60-69% BDY BRN/20-29% 3D */
"94863" = "1" /* 60-69% BDY BRN/30-39% 3D */
"94864" = "1" /* 60-69% BDY BRN/40-49% 3D */
"94865" = "1" /* 60-69% BDY BRN/50-59% 3D */
"94866" = "1" /* 60-69% BDY BRN/60-69% 3D */
"94870" = "1" /* 70-79% BDY BRN/3 DEG NOS */
"94871" = "1" /* 70-79% BDY BRN/10-19% 3D */
"94872" = "1" /* 70-79% BDY BRN/20-29% 3D */
"94873" = "1" /* 70-79% BDY BRN/30-39% 3D */
"94874" = "1" /* 70-79% BDY BRN/40-49% 3D */
"94875" = "1" /* 70-79% BDY BRN/50-59% 3D */
"94876" = "1" /* 70-79% BDY BRN/60-69% 3D */
"94877" = "1" /* 70-79% BDY BRN/70-79% 3D */
"94880" = "1" /* 80-89% BDY BRN/3 DEG NOS */
"94881" = "1" /* 80-89% BDY BRN/10-19% 3D */
"94882" = "1" /* 80-89% BDY BRN/20-29% 3D */
"94883" = "1" /* 80-89% BDY BRN/30-39% 3D */
"94884" = "1" /* 80-89% BDY BRN/40-49% 3D */
"94885" = "1" /* 80-89% BDY BRN/50-59% 3D */
"94886" = "1" /* 80-89% BDY BRN/60-69% 3D */
"94887" = "1" /* 80-89% BDY BRN/70-79% 3D */
"94888" = "1" /* 80-89% BDY BRN/80-89% 3D */
"94890" = "1" /* 90% + BDY BRN/3D DEG NOS */
"94891" = "1" /* 90% + BDY BRN/10-19% 3RD */
"94892" = "1" /* 90% + BDY BRN/20-29% 3RD */
"94893" = "1" /* 90% + BDY BRN/30-39% 3RD */
"94894" = "1" /* 90% + BDY BRN/40-49% 3RD */
"94895" = "1" /* 90% + BDY BRN/50-59% 3RD */
"94896" = "1" /* 90% + BDY BRN/60-69% 3RD */
"94897" = "1" /* 90% + BDY BRN/70-79% 3RD */
"94898" = "1" /* 90% + BDY BRN/80-89% 3RD */
"94899" = "1" /* 90% + BDY BRN/90% + 3RD */
"9490"  = "1" /* BURN NOS */
"9491"  = "1" /* 1ST DEGREE BURN NOS */
"9492"  = "1" /* 2ND DEGREE BURN NOS */
"9493"  = "1" /* 3RD DEGREE BURN NOS */
"9494"  = "1" /* DEEP 3RD DEG BURN NOS */
"9495"  = "1" /* 3RD BURN W LOSS-SITE NOS */
"95200" = "1" /* C1-C4 SPIN CORD INJ NOS */
"95201" = "1" /* COMPLETE LES CORD/C1-C4 */
"95202" = "1" /* ANTERIOR CORD SYND/C1-C4 */
"95203" = "1" /* CENTRAL CORD SYND/C1-C4 */
"95204" = "1" /* C1-C4 SPIN CORD INJ NEC */
"95205" = "1" /* C5-C7 SPIN CORD INJ NOS */
"95206" = "1" /* COMPLETE LES CORD/C5-C7 */
"95207" = "1" /* ANTERIOR CORD SYND/C5-C7 */
"95208" = "1" /* CENTRAL CORD SYND/C5-C7 */
"95209" = "1" /* C5-C7 SPIN CORD INJ NEC */
"95210" = "1" /* T1-T6 SPIN CORD INJ NOS */
"95211" = "1" /* COMPLETE LES CORD/T1-T6 */
"95212" = "1" /* ANTERIOR CORD SYND/T1-T6 */
"95213" = "1" /* CENTRAL CORD SYND/T1-T6 */
"95214" = "1" /* T1-T6 SPIN CORD INJ NEC */
"95215" = "1" /* T7-T12 SPIN CORD INJ NOS */
"95216" = "1" /* COMPLETE LES CORD/T7-T12 */
"95217" = "1" /* ANTERIOR CORD SYN/T7-T12 */
"95218" = "1" /* CENTRAL CORD SYN/T7-T12 */
"95219" = "1" /* T7-T12 SPIN CORD INJ NEC */
"9522"  = "1" /* LUMBAR SPINAL CORD INJUR */
"9523"  = "1" /* SACRAL SPINAL CORD INJUR */
"9524"  = "1" /* CAUDA EQUINA INJURY */
"9528"  = "1" /* SPIN CORD INJ-MULT SITE */
"9529"  = "1" /* SPINAL CORD INJURY NOS */
"9530"  = "1" /* CERVICAL ROOT INJURY */
"9531"  = "1" /* DORSAL ROOT INJURY */
"9532"  = "1" /* LUMBAR ROOT INJURY */
"9533"  = "1" /* SACRAL ROOT INJURY */
"9534"  = "1" /* BRACHIAL PLEXUS INJURY */
"9535"  = "1" /* LUMBOSACRAL PLEX INJURY */
"9538"  = "1" /* MULT NERVE ROOT/PLEX INJ */
"9539"  = "1" /* INJ NERVE ROOT/PLEX NOS */
"9580"  = "1" /* AIR EMBOLISM */
"9581"  = "1" /* FAT EMBOLISM */
"9582"  = "1" /* SECONDARY/RECUR HEMORR */
"9583"  = "1" /* POSTTRAUM WND INFEC NEC */
"9584"  = "1" /* TRAUMATIC SHOCK */
"9585"  = "1" /* TRAUMATIC ANURIA */
"9586"  = "1" /* VOLKMANN"S ISCH CONTRACT */
"9587"  = "1" /* TRAUM SUBCUTAN EMPHYSEMA */
"9588"  = "1" /* EARLY COMPLIC TRAUMA NEC */
OTHER   = "0" ;

 /* EXCLUDE: TRAUMA */
 /* DIAGNOSTIC RELATED GROUPS (DRGS): */

VALUE $TRAUMDR
"002" = "1" /* CRANIOTOMY FOR TRAUMA, AGE GREATER THAN 17 */
"027" = "1" /* TRAUMATIC STUPOR AND COMA, COMA GREATER THAN ONE HOUR */
"028" = "1" /* TRAUMATIC STUPOR AND COMA, COMA LESS THAN ONE HOUR, AGE GREATER THAN 17 WITH CC */
"029" = "1" /* TRAUMATIC STUPOR AND COMA, COMA LESS THAN ONE HOUR, AGE GREATER THAN 17 WITHOUT CC */
"030" = "1" /* TRAUMATIC STUPOR AND COMA, COMA LESS THAN ONE HOUR, AGE 0-17 */
"031" = "1" /* CONCUSSION, AGE GREATER THAN 17 WITHOUT CC */
"032" = "1" /* CONCUSSION, AGE GREATER THAN 17 WITHOUT CC */
"033" = "1" /* CONCUSSION AGE 0-17 */
"072" = "1" /* NASAL TRAUMA AND DEFORMITY */
"083" = "1" /* MAJOR CHEST TRAUMA WITH CC */
"084" = "1" /* MAJOR CHEST TRAUMA WITHOUT CC */
"235" = "1" /* FRACTURES OF FEMUR */
"236" = "1" /* FRACTURE OF HIP AND PELVIS */
"237" = "1" /* SPRAINS, STRAINS AND DISLOCATIONS OF HIP, PELVIS AND THIGH */
"440" = "1" /* WOUND DEBRIDEMENTS FOR INJURIES */
"441" = "1" /* HAND PROCEDURES FOR INJURIES */
"442" = "1" /* OTHER OR PROCEDURES FOR INJURIES WITH CC */
"443" = "1" /* OHER OR PROCEDURES FOR INJRUIES WITHOUT CC */
"444" = "1" /* AUMATIC INJRY, AGE GREATER THAN 17 WITH CC */
"445" = "1" /* AUMATIC INJRY, AGE GREATER THAN 17 WITHOUT CC */
"446" = "1" /* AUMATIC INJURY, AGE 0-17 */
"456" = "1" /* NO LONGER VALID */
"457" = "1" /* NO LONGER VALID */
"458" = "1" /* NO LONGER VALID */
"459" = "1" /* NO LONGER VALID */
"460" = "1" /* NO LONGER VALID */
"484" = "1" /* CRANIOTOMY FOR MULTIPLE SIGNIFICANT TRAUMA */
"485" = "1" /* LIMB REATTACHMENT, HIP AND FEMUR PROCEDURES FOR MULTIPLE SIGNIFICANT TRAUMA */
"486" = "1" /* OTHER OR PROCEDURES FOR MULTIPLE SIGNIFICANT TRAUMA */
"487" = "1" /* OTHER MULTIPLE SIGNIFICANT TRAUMAS */
"491" = "1" /* MAJOR JOINT AND LIMB REATTACHMENT PROCEDURES OF UPPER EXTREMITY */
"504" = "1" /* TOTAL HEPATECTOMY */
"505" = "1" /* EXTENSIVE 3RD DEGREE BURNS W/O SKIN GRAFT */
"506" = "1" /* FULL THICKNESS BURN W SKIN GRAFT OR INHAL INJ W CC OR SIG TRAUMA */
"507" = "1" /* FULL THICKNESS BURN W SKIN GRFT OR INHAL INJ W/O CC OR SIG TRAUMA*/
"508" = "1" /* FULL THICKNESS BURN W/O SKIN GRFT OR INHAL INJ W CC OR SIG TRAUMA*/
"509" = "1" /* FULL THICKNESS BURN W/O SKIN GRFT OR INH INJ W/O CC OR SIG TRAUMA*/
"510" = "1" /* NON-EXTENSIVE BURNS W CC OR SIGNIFICANT TRAUMA*/
"511" = "1" /* NON-EXTENSIVE BURNS W/O CC OR SIGNIFICANT TRAUMA*/
OTHER = "0" ;

 /* EXCLUDE: IMMUNOCOMPROMISED */
 /* ICD-9-CM DIAGNOSIS CODES (INCLUDES ALL 4TH AND 5TH DIGITS) */

VALUE $IMMUNID
"042"   = "1" /* HUMAN IMMUNODEFICIENCY VIRUS DISEASE */
"1363"  = "1" /* PNEUMOCYSTOSIS */
"23873" = "1" /* HI GRDE MYELODYS SYN LES */
"23876" = "1" /* MYELOFI W MYELO METAPLAS */
"260  " = "1" /* KWASHIORKOR */
"261  " = "1" /* NUTRITIONAL MARASMUS */
"262  " = "1" /* OTH SEVERE MALNUTRITION */
"27900" = "1" /* HYPOGAMMAGLOBULINEM NOS */
"27901" = "1" /* SELECTIVE IGA IMMUNODEF */
"27902" = "1" /* SELECTIVE IGM IMMUNODEF */
"27903" = "1" /* SELECTIVE IG DEFIC NEC */
"27904" = "1" /* CONG HYPOGAMMAGLOBULINEM */
"27905" = "1" /* IMMUNODEFIC W HYPER-IGM */
"27906" = "1" /* COMMON VARIABL IMMUNODEF */
"27909" = "1" /* HUMORAL IMMUNITY DEF NEC */
"27910" = "1" /* IMMUNDEF T-CELL DEF NOS */
"27911" = "1" /* DIGEORGES SYNDROME */
"27912" = "1" /* WISKOTT-ALDRICH SYNDROME */
"27913" = "1" /* NEZELOFS SYNDROME */
"27919" = "1" /* DEFIC CELL IMMUNITY NOS */
"2792"  = "1" /* COMBINED IMMUNITY DEFICIENCY */
"2793"  = "1" /* UNSPECIFIED IMMUNITY DEFICIENCY */
"2794"  = "1" /* AUTOIMMUNE DISEASE, NOT ELSEWHERE CLASSIFIED */
"2798"  = "1" /* OTHER SPECIFIED DISORDERS INVOLVING THE IMMUNE MECHANISM */
"2799"  = "1" /* UNSPECIFIED DISORDER OF IMMUNE MECHANISM */
"28409" = "1" /* CONST APLASTC ANEMIA NEC */
"2841 " = "1" /* PANCYTOPENIA */
"2880 " = "1" /* AGRANULOCYTOSIS */
"28800" = "1" /* NEUTROPENIA NOS */
"28801" = "1" /* CONGENITAL NEUTROPENIA */
"28802" = "1" /* CYCLIC NEUTROPENIA */
"28803" = "1" /* DRUG INDUCED NEUTROPENIA */
"28809" = "1" /* NEUTROPENIA NEC */
"2881 " = "1" /* FUNCTION DIS NEUTROPHILS */
"2882 " = "1" /* GENETIC ANOMALY LEUKOCYT */
"2884 " = "1" /* HEMOPHAGOCYTIC SYNDROMES */
"28850" = "1" /* LEUKOCYTOPENIA NOS */
"28851" = "1" /* LYMPHOCYTOPENIA */
"28859" = "1" /* DECREASED WBC COUNT NEC */
"28953" = "1" /* NEUTROPENIC SPLENOMEGALY */
"28983" = "1" /* MYELOFIBROSIS */
"40301" = "1" /* MAL HYP KIDNEY W CHR KID */
"40311" = "1" /* BEN HYP KIDNEY W CHR KID */
"40391" = "1" /* HYP KIDNEY NOS W CHR KID */
"40402" = "1" /* MAL HY HRT/KID W CHR KID */
"40403" = "1" /* MAL HYP HRT/KID W HF/KID */
"40412" = "1" /* BEN HYP HT/KID W CHR KID */
"40413" = "1" /* BEN HYP HT/KID W HF/KID */
"40492" = "1" /* HYP HT/KID NOS W CHR KID */
"40493" = "1" /* HYP HRT/KID NOS W HF/KID */
"5793 " = "1" /* INTEST POSTOP NONABSORB */
"585  " = "1" /* CHRONIC RENAL FAILURE */
"5855 " = "1" /* CHRON KIDNEY DIS STAGE V */
"5856 " = "1" /* END STAGE RENAL DISEASE */
"9968"  = "1" /* COMPLICATIONS OF TRANSPLANTED ORGAN */
"99680" = "1" /* COMP ORGAN TRANSPLNT NOS */
"99681" = "1" /* COMPL KIDNEY TRANSPLANT */
"99682" = "1" /* COMPL LIVER TRANSPLANT */
"99683" = "1" /* COMPL HEART TRANSPLANT */
"99684" = "1" /* COMPL LUNG TRANSPLANT */
"99685" = "1" /* COMPL MARROW TRANSPLANT */
"99686" = "1" /* COMPL PANCREAS TRANSPLNT */
"99687" = "1" /* COMP INTESTINE TRANSPLNT */
"99689" = "1" /* COMP OTH ORGAN TRANSPLNT */
"V420"  = "1" /* KIDNEY REPLACED BY TRANSPLANT */
"V421"  = "1" /* HEART REPLACED BY TRANSPLANT */
"V426"  = "1" /* LUNG REPLACED BY TRANSPLANT */
"V427"  = "1" /* LIVER REPLACED BY TRANSPLANT */
"V428"  = "1" /* OTHER SPECIFIED ORGAN OR TISSUE */
"V4281" = "1" /* BONE MARROW SPECIFIED BY TRANSPLANT */
"V4282" = "1" /* PERIPHERAL STEM CELLS REPLACED BY TRANSPLANT */
"V4283" = "1" /* PANCREAS REPLACED BY TRANSPLANT */
"V4284" = "1" /* INTESTINES REPLACE BY TRANSPLANT */
"V4289" = "1" /* OTHER REPLACED BY TRANSPLANT */
"V451 " = "1" /* RENAL DIALYSIS STATUS */
"V560 " = "1" /* RENAL DIALYSIS ENCOUNTER */
"V561 " = "1" /* FT/ADJ XTRCORP DIAL CATH */
"V562 " = "1" /* FIT/ADJ PERIT DIAL CATH */
OTHER   = "0" ;

 /* EXCLUDE: IMMUNOCOMPROMISED */
 /* ICD-9-CM PROCEDURE CODES (INCLUDES 4TH AND 5TH DIGITS:) */

VALUE $IMMUNIP
"0018" = "1" /* INFUS IMMUNOSUP ANTIBODY */
"335"  = "1" /* LUNG TRANSPLANT */
"3350" = "1" /* LUNG TRANSPLANT NOS */
"3351" = "1" /* UNILAT LUNG TRANSPLANT */
"3352" = "1" /* BILAT LUNG TRANSPLANT */
"336"  = "1" /* COMBINED HEART-LUNG TRANSPLANTATION */
"375"  = "1" /* HEART TRANSPLANTATION */
"3751" = "1" /* HEART TRANSPLANTATION */
"410"  = "1" /* OPERATIONS ON BONE MAROW AND SPLEEN */
"4100" = "1" /* BONE MARROW TRNSPLNT NOS */
"4101" = "1" /* AUTO BONE MT W/O PURG */
"4102" = "1" /* ALO BONE MARROW TRNSPLNT */
"4103" = "1" /* ALLOGRFT BONE MARROW NOS */
"4104" = "1" /* AUTO HEM STEM CT W/O PUR */
"4105" = "1" /* ALLO HEM STEM CT W/O PUR */
"4106" = "1" /* CORD BLD STEM CELL TRANS */
"4107" = "1" /* AUTO HEM STEM CT W PURG */
"4108" = "1" /* ALLO HEM STEM CT W PURG */
"4109" = "1" /* AUTO BONE MT W PURGING */
"5051" = "1" /* AUXILIARY LIVER TRANSPL */
"5059" = "1" /* LIVER TRANSPLANT NEC */
"5280" = "1" /* PANCREATIC TRANSPLANT, NOS */
"5281" = "1" /* REIMPLANTATION OF PANCREATIC TISSUE */
"5282" = "1" /* REIMPLANTATION OF PANCREATIC TISSUE */
"5283" = "1" /* HETEROTRANSPLANT OF PANCREAS */
"5285" = "1" /* ALLOTRANSPLANTATION OF CELLS OF ISLETS OF LNGERHANS */
"5286" = "1" /* TRANSPLANTATION OF CELLS OF ISLETS OF LANGERHANS, NOS */
"5569" = "1" /* OTHER KIDNEY TRANSPLANTATION */
OTHER  = "0" ;

 /* EXCLUDE: IMMUNOCOMPROMISED */
 /* DIAGNOSTIC RELATED GROUPS (DRGS): */

VALUE $IMMUNDR
"488" = "1" /* HIV WITH EXTENSIVE OR PROCEDURE */
"489" = "1" /* HIV WITH MAJOR RELATED CONDITION */
"490" = "1" /* HIV WITH OR WITHOUT OTHER RELATED CONDITION */
OTHER = "0" ;

 /* EXCLUDE: CANCER */
 /* ICD-9-CM DIAGNOSIS CODES( ALL 4TH AND 5TH DIGITS) : */

VALUE $CANCEID
"1400"  = "1" /* MAL NEO UPPER VERMILION*/
"1401"  = "1" /* MAL NEO LOWER VERMILION */
"1403"  = "1" /* MAL NEO UPPER LIP, INNER */
"1404"  = "1" /* MAL NEO LOWER LIP, INNER */
"1405"  = "1" /* MAL NEO LIP, INNER NOS */
"1406"  = "1" /* MAL NEO LIP, COMMISSURE */
"1408"  = "1" /* MAL NEO LIP NEC */
"1409"  = "1" /* MAL NEO LIP/VERMIL NOS */
"1410"  = "1" /* MAL NEO TONGUE BASE */
"1411"  = "1" /* MAL NEO DORSAL TONGUE */
"1412"  = "1" /* MAL NEO TIP/LAT TONGUE */
"1413"  = "1" /* MAL NEO VENTRAL TONGUE */
"1414"  = "1" /* MAL NEO ANT 2/3 TONGUE */
"1415"  = "1" /* MAL NEO TONGUE JUNCTION */
"1416"  = "1" /* MAL NEO LINGUAL TONSIL */
"1418"  = "1" /* MALIG NEO TONGUE NEC */
"1419"  = "1" /* MALIG NEO TONGUE NOS */
"1420"  = "1" /* MALIG NEO PAROTID */
"1421"  = "1" /* MALIG NEO SUBMANDIBULAR */
"1422"  = "1" /* MALIG NEO SUBLINGUAL */
"1428"  = "1" /* MAL NEO MAJ SALIVARY NEC */
"1429"  = "1" /* MAL NEO SALIVARY NOS */
"1430"  = "1" /* MALIG NEO UPPER GUM */
"1431"  = "1" /* MALIG NEO LOWER GUM */
"1438"  = "1" /* MALIG NEO GUM NEC */
"1439"  = "1" /* MALIG NEO GUM NOS */
"1440"  = "1" /* MAL NEO ANT FLOOR MOUTH */
"1441"  = "1" /* MAL NEO LAT FLOOR MOUTH */
"1448"  = "1" /* MAL NEO MOUTH FLOOR NEC */
"1449"  = "1" /* MAL NEO MOUTH FLOOR NOS */
"1450"  = "1" /* MAL NEO CHEEK MUCOSA */
"1451"  = "1" /* MAL NEO MOUTH VESTIBULE */
"1452"  = "1" /* MALIG NEO HARD PALATE */
"1453"  = "1" /* MALIG NEO SOFT PALATE */
"1454"  = "1" /* MALIGNANT NEOPLASM UVULA */
"1455"  = "1" /* MALIGNANT NEO PALATE NOS */
"1456"  = "1" /* MALIG NEO RETROMOLAR */
"1458"  = "1" /* MALIG NEOPLASM MOUTH NEC */
"1459"  = "1" /* MALIG NEOPLASM MOUTH NOS */
"1460"  = "1" /* MALIGNANT NEOPL TONSIL */
"1461"  = "1" /* MAL NEO TONSILLAR FOSSA */
"1462"  = "1" /*  MAL NEO TONSIL PILLARS */
"1463"  = "1" /* MALIGN NEOPL VALLECULA */
"1464"  = "1" /* MAL NEO ANT EPIGLOTTIS */
"1465"  = "1" /* MAL NEO EPIGLOTTIS JUNCT */
"1466"  = "1" /* MAL NEO LAT OROPHARYNX */
"1467"  = "1" /* MAL NEO POST OROPHARYNX */
"1468"  = "1" /* MAL NEO OROPHARYNX NEC */
"1469"  = "1" /* MALIG NEO OROPHARYNX NOS */
"1470"  = "1" /* MAL NEO SUPER NASOPHARYN */
"1471"  = "1" /* MAL NEO POST NASOPHARYNX */
"1472"  = "1" /* MAL NEO LAT NASOPHARYNX */
"1473"  = "1" /* MAL NEO ANT NASOPHARYNX */
"1478"  = "1" /* MAL NEO NASOPHARYNX NEC */
"1479"  = "1" /* MAL NEO NASOPHARYNX NOS */
"1480"  = "1" /* MAL NEO POSTCRICOID */
"1481"  = "1" /* MAL NEO PYRIFORM SINUS */
"1482"  = "1" /* MAL NEO ARYEPIGLOTT FOLD */
"1483"  = "1" /* MAL NEO POST HYPOPHARYNX */
"1488"  = "1" /* MAL NEO HYPOPHARYNX NEC */
"1489"  = "1" /* MAL NEO HYPOPHARYNX NOS */
"1490"  = "1" /* MAL NEO PHARYNX NOS */
"1491"  = "1" /* MAL NEO WALDEYERS RING */
"1498"  = "1" /* MAL NEO ORAL/PHARYNX NEC */
"1499"  = "1" /* MAL NEO OROPHRYN ILL-DEF */
"1500"  = "1" /* MAL NEO CERVICAL ESOPHAG */
"1501"  = "1" /* MAL NEO THORACIC ESOPHAG */
"1502"  = "1" /* MAL NEO ABDOMIN ESOPHAG */
"1503"  = "1" /* MAL NEO UPPER 3RD ESOPH */
"1504"  = "1" /* MAL NEO MIDDLE 3RD ESOPH */
"1505"  = "1" /* MAL NEO LOWER 3RD ESOPH */
"1508"  = "1" /* MAL NEO ESOPHAGUS NEC */
"1509"  = "1" /* MAL NEO ESOPHAGUS NOS */
"1510"  = "1" /* MAL NEO STOMACH CARDIA */
"1511"  = "1" /* MALIGNANT NEO PYLORUS */
"1512"  = "1" /* MAL NEO PYLORIC ANTRUM */
"1513"  = "1" /* MAL NEO STOMACH FUNDUS */
"1514"  = "1" /* MAL NEO STOMACH BODY */
"1515"  = "1" /* MAL NEO STOM LESSER CURV */
"1516"  = "1" /* MAL NEO STOM GREAT CURV */
"1518"  = "1" /* MALIG NEOPL STOMACH NEC */
"1519"  = "1" /* MALIG NEOPL STOMACH NOS */
"1520"  = "1" /* MALIGNANT NEOPL DUODENUM */
"1521"  = "1" /* MALIGNANT NEOPL JEJUNUM */
"1522"  = "1" /* MALIGNANT NEOPLASM ILEUM */
"1523"  = "1" /* MAL NEO MECKELS DIVERT */
"1528"  = "1" /* MAL NEO SMALL BOWEL NEC */
"1529"  = "1" /* MAL NEO SMALL BOWEL NOS */
"1530"  = "1" /* MAL NEO HEPATIC FLEXURE */
"1531"  = "1" /* MAL NEO TRANSVERSE COLON */
"1532"  = "1" /* MAL NEO DESCEND COLON */
"1533"  = "1" /* MAL NEO SIGMOID COLON */
"1534"  = "1" /* MALIGNANT NEOPLASM CECUM */
"1535"  = "1" /* MALIGNANT NEO APPENDIX */
"1536"  = "1" /* MALIG NEO ASCEND COLON */
"1537"  = "1" /* MAL NEO SPLENIC FLEXURE */
"1538"  = "1" /* MALIGNANT NEO COLON NEC */
"1539"  = "1" /* MALIGNANT NEO COLON NOS */
"1540"  = "1" /* MAL NEO RECTOSIGMOID JCT */
"1541"  = "1" /* MALIGNANT NEOPL RECTUM */
"1542"  = "1" /* MALIG NEOPL ANAL CANAL */
"1543"  = "1" /* MALIGNANT NEO ANUS NOS */
"1548"  = "1" /* MAL NEO RECTUM/ANUS NEC */
"1550"  = "1" /* MAL NEO LIVER, PRIMARY */
"1551"  = "1" /* MAL NEO INTRAHEPAT DUCTS */
"1552"  = "1" /* MALIGNANT NEO LIVER NOS */
"1560"  = "1" /* MALIG NEO GALLBLADDER */
"1561"  = "1" /* MAL NEO EXTRAHEPAT DUCTS */
"1562"  = "1" /* MAL NEO AMPULLA OF VATER */
"1568"  = "1" /* MALIG NEO BILIARY NEC */
"1569"  = "1" /* MALIG NEO BILIARY NOS */
"1570"  = "1" /* MAL NEO PANCREAS HEAD */
"1571"  = "1" /* MAL NEO PANCREAS BODY */
"1572"  = "1" /* MAL NEO PANCREAS TAIL */
"1573"  = "1" /* MAL NEO PANCREATIC DUCT */
"1574"  = "1" /* MAL NEO ISLET LANGERHANS */
"1578"  = "1" /* MALIG NEO PANCREAS NEC */
"1579"  = "1" /* MALIG NEO PANCREAS NOS */
"1580"  = "1" /* MAL NEO RETROPERITONEUM */
"1588"  = "1" /* MAL NEO PERITONEUM NEC */
"1589"  = "1" /* MAL NEO PERITONEUM NOS */
"1590"  = "1" /* MALIG NEO INTESTINE NOS */
"1591"  = "1" /* MALIGNANT NEO SPLEEN NEC */
"1598"  = "1" /* MAL NEO GI/INTRA-ABD NEC */
"1599"  = "1" /* MAL NEO GI TRACT ILL-DEF */
"1600"  = "1" /* MAL NEO NASAL CAVITIES */
"1601"  = "1" /* MALIG NEO MIDDLE EAR */
"1602"  = "1" /* MAL NEO MAXILLARY SINUS */
"1603"  = "1" /* MAL NEO ETHMOIDAL SINUS */
"1604"  = "1" /* MALIG NEO FRONTAL SINUS */
"1605"  = "1" /* MAL NEO SPHENOID SINUS */
"1608"  = "1" /* MAL NEO ACCESS SINUS NEC */
"1609"  = "1" /* MAL NEO ACCESS SINUS NOS */
"1610"  = "1" /* MALIGNANT NEO GLOTTIS */
"1611"  = "1" /* MALIG NEO SUPRAGLOTTIS */
"1612"  = "1" /* MALIG NEO SUBGLOTTIS */
"1613"  = "1" /* MAL NEO CARTILAGE LARYNX */
"1618"  = "1" /* MALIGNANT NEO LARYNX NEC */
"1619"  = "1" /* MALIGNANT NEO LARYNX NOS */
"1620"  = "1" /* MALIGNANT NEO TRACHEA */
"1622"  = "1" /* MALIG NEO MAIN BRONCHUS */
"1623"  = "1" /* MAL NEO UPPER LOBE LUNG */
"1624"  = "1" /* MAL NEO MIDDLE LOBE LUNG */
"1625"  = "1" /* MAL NEO LOWER LOBE LUNG */
"1628"  = "1" /* MAL NEO BRONCH/LUNG NEC */
"1629"  = "1" /* MAL NEO BRONCH/LUNG NOS */
"1630"  = "1" /* MAL NEO PARIETAL PLEURA */
"1631"  = "1" /* MAL NEO VISCERAL PLEURA */
"1638"  = "1" /* MALIG NEOPL PLEURA NEC */
"1639"  = "1" /* MALIG NEOPL PLEURA NOS */
"1640"  = "1" /* MALIGNANT NEOPL THYMUS */
"1641"  = "1" /* MALIGNANT NEOPL HEART */
"1642"  = "1" /* MAL NEO ANT MEDIASTINUM */
"1643"  = "1" /* MAL NEO POST MEDIASTINUM */
"1648"  = "1" /* MAL NEO MEDIASTINUM NEC */
"1649"  = "1" /* MAL NEO MEDIASTINUM NOS */
"1650"  = "1" /* MAL NEO UPPER RESP NOS */
"1658"  = "1" /* MAL NEO THORAX/RESP NEC */
"1659"  = "1" /* MAL NEO RESP SYSTEM NOS */
"1700"  = "1" /* MAL NEO SKULL/FACE BONE */
"1701"  = "1" /* MALIGNANT NEO MANDIBLE */
"1702"  = "1" /* MALIG NEO VERTEBRAE */
"1703"  = "1" /* MAL NEO RIBS/STERN/CLAV */
"1704"  = "1" /* MAL NEO LONG BONES ARM */
"1705"  = "1" /* MAL NEO BONES WRIST/HAND */
"1706"  = "1" /* MAL NEO PELVIC GIRDLE */
"1707"  = "1" /* MAL NEO LONG BONES LEG */
"1708"  = "1" /* MAL NEO BONES ANKLE/FOOT */
"1709"  = "1" /* MALIG NEOPL BONE NOS */
"1710"  = "1" /* MAL NEO SOFT TISSUE HEAD */
"1712"  = "1" /* MAL NEO SOFT TISSUE ARM */
"1713"  = "1" /* MAL NEO SOFT TISSUE LEG */
"1714"  = "1" /* MAL NEO SOFT TIS THORAX */
"1715"  = "1" /* MAL NEO SOFT TIS ABDOMEN */
"1716"  = "1" /* MAL NEO SOFT TIS PELVIS */
"1717"  = "1" /* MAL NEOPL TRUNK NOS */
"1718"  = "1" /* MAL NEO SOFT TISSUE NEC */
"1719"  = "1" /* MAL NEO SOFT TISSUE NOS */
"1720"  = "1" /* MALIG MELANOMA LIP */
"1721"  = "1" /* MALIG MELANOMA EYELID */
"1722"  = "1" /* MALIG MELANOMA EAR */
"1723"  = "1" /* MAL MELANOM FACE NEC/NOS */
"1724"  = "1" /* MAL MELANOMA SCALP/NECK */
"1725"  = "1" /* MALIG MELANOMA TRUNK */
"1726"  = "1" /* MALIG MELANOMA ARM */
"1727"  = "1" /* MALIG MELANOMA LEG */
"1728"  = "1" /* MALIG MELANOMA SKIN NEC */
"1729"  = "1" /* MALIG MELANOMA SKIN NOS */
"1740"  = "1" /* MALIG NEO NIPPLE */
"1741"  = "1" /* MAL NEO BREAST-CENTRAL */
"1742"  = "1" /* MAL NEO BREAST UP-INNER */
"1743"  = "1" /* MAL NEO BREAST LOW-INNER */
"1744"  = "1" /* MAL NEO BREAST UP-OUTER */
"1745"  = "1" /* MAL NEO BREAST LOW-OUTER */
"1746"  = "1" /* MAL NEO BREAST-AXILLARY */
"1748"  = "1" /* MALIGN NEOPL BREAST NEC */
"1749"  = "1" /* MALIGN NEOPL BREAST NOS */
"1750"  = "1" /* MAL NEO MALE NIPPLE */
"1759"  = "1" /* MAL NEO MALE BREAST NEC */
"1760"  = "1" /* SKIN - KAPOSIS SARCOMA */
"1761"  = "1" /* SFT TISUE - KPSIS SRCMA */
"1762"  = "1" /* PALATE - KPSIS SARCOMA */
"1763"  = "1" /* GI SITES - KPSIS SRCOMA */
"1764"  = "1" /* LUNG - KAPOSIS SARCOMA */
"1765"  = "1" /* LYM NDS - KPSIS SARCOMA */
"1768"  = "1" /* SPF STS - KPSIS SARCOMA */
"1769"  = "1" /* KAPOSIS SARCOMA NOS */
"179"   = "1" /* MALIG NEOPL UTERUS NOS */
"1800"  = "1" /* MALIG NEO ENDOCERVIX */
"1801"  = "1" /* MALIG NEO EXOCERVIX */
"1808"  = "1" /* MALIG NEO CERVIX NEC */
"1809"  = "1" /* MAL NEO CERVIX UTERI NOS */
"181"   = "1" /* MALIGNANT NEOPL PLACENTA */
"1820"  = "1" /* MALIG NEO CORPUS UTERI */
"1821"  = "1" /* MAL NEO UTERINE ISTHMUS */
"1828"  = "1" /* MAL NEO BODY UTERUS NEC */
"1830"  = "1" /* MALIGN NEOPL OVARY */
"1832"  = "1" /* MAL NEO FALLOPIAN TUBE */
"1833"  = "1" /* MAL NEO BROAD LIGAMENT */
"1834"  = "1" /* MALIG NEO PARAMETRIUM */
"1835"  = "1" /* MAL NEO ROUND LIGAMENT */
"1838"  = "1" /* MAL NEO ADNEXA NEC */
"1839"  = "1" /* MAL NEO ADNEXA NOS */
"1840"  = "1" /* MALIGN NEOPL VAGINA */
"1841"  = "1" /* MAL NEO LABIA MAJORA */
"1842"  = "1" /* MAL NEO LABIA MINORA */
"1843"  = "1" /* MALIGN NEOPL CLITORIS */
"1844"  = "1" /* MALIGN NEOPL VULVA NOS */
"1848"  = "1" /* MAL NEO FEMALE GENIT NEC */
"1849"  = "1" /* MAL NEO FEMALE GENIT NOS */
"185"   = "1" /* MALIGN NEOPL PROSTATE */
"1860"  = "1" /* MAL NEO UNDESCEND TESTIS */
"1869"  = "1" /* MALIG NEO TESTIS NEC */
"1871"  = "1" /* MALIGN NEOPL PREPUCE */
"1872"  = "1" /* MALIG NEO GLANS PENIS */
"1873"  = "1" /* MALIG NEO PENIS BODY */
"1874"  = "1" /* MALIG NEO PENIS NOS */
"1875"  = "1" /* MALIG NEO EPIDIDYMIS */
"1876"  = "1" /* MAL NEO SPERMATIC CORD */
"1877"  = "1" /* MALIGN NEOPL SCROTUM */
"1878"  = "1" /* MAL NEO MALE GENITAL NEC */
"1879"  = "1" /* MAL NEO MALE GENITAL NOS */
"1880"  = "1" /* MAL NEO BLADDER-TRIGONE */
"1881"  = "1" /* MAL NEO BLADDER-DOME */
"1882"  = "1" /* MAL NEO BLADDER-LATERAL */
"1883"  = "1" /* MAL NEO BLADDER-ANTERIOR */
"1884"  = "1" /* MAL NEO BLADDER-POST */
"1885"  = "1" /* MAL NEO BLADDER NECK */
"1886"  = "1" /* MAL NEO URETERIC ORIFICE */
"1887"  = "1" /* MALIG NEO URACHUS */
"1888"  = "1" /* MALIG NEO BLADDER NEC */
"1889"  = "1" /* MALIG NEO BLADDER NOS */
"1890"  = "1" /* MALIG NEOPL KIDNEY */
"1891"  = "1" /* MALIG NEO RENAL PELVIS */
"1892"  = "1" /* MALIGN NEOPL URETER */
"1893"  = "1" /* MALIGN NEOPL URETHRA */
"1894"  = "1" /* MAL NEO PARAURETHRAL */
"1898"  = "1" /* MAL NEO URINARY NEC */
"1899"  = "1" /* MAL NEO URINARY NOS */
"1900"  = "1" /* MALIGN NEOPL EYEBALL */
"1901"  = "1" /* MALIGN NEOPL ORBIT */
"1902"  = "1" /* MAL NEO LACRIMAL GLAND */
"1903"  = "1" /* MAL NEO CONJUNCTIVA */
"1904"  = "1" /* MALIGN NEOPL CORNEA */
"1905"  = "1" /* MALIGN NEOPL RETINA */
"1906"  = "1" /* MALIGN NEOPL CHOROID */
"1907"  = "1" /* MAL NEO LACRIMAL DUCT */
"1908"  = "1" /* MALIGN NEOPL EYE NEC */
"1909"  = "1" /* MALIGN NEOPL EYE NOS */
"1910"  = "1" /* MALIGN NEOPL CEREBRUM */
"1911"  = "1" /* MALIG NEO FRONTAL LOBE */
"1912"  = "1" /* MAL NEO TEMPORAL LOBE */
"1913"  = "1" /* MAL NEO PARIETAL LOBE */
"1914"  = "1" /* MAL NEO OCCIPITAL LOBE */
"1915"  = "1" /* MAL NEO CEREB VENTRICLE */
"1916"  = "1" /* MAL NEO CEREBELLUM NOS */
"1917"  = "1" /* MAL NEO BRAIN STEM */
"1918"  = "1" /* MALIG NEO BRAIN NEC */
"1919"  = "1" /* MALIG NEO BRAIN NOS */
"1920"  = "1" /* MAL NEO CRANIAL NERVES */
"1921"  = "1" /* MAL NEO CEREBRAL MENING */
"1922"  = "1" /* MAL NEO SPINAL CORD */
"1923"  = "1" /* MAL NEO SPINAL MENINGES */
"1928"  = "1" /* MAL NEO NERVOUS SYST NEC */
"1929"  = "1" /* MAL NEO NERVOUS SYST NOS */
"193"   = "1" /* MALIGN NEOPL THYROID */
"1940"  = "1" /* MALIGN NEOPL ADRENAL */
"1941"  = "1" /* MALIG NEO PARATHYROID */
"1943"  = "1" /* MALIG NEO PITUITARY */
"1944"  = "1" /* MALIGN NEO PINEAL GLAND */
"1945"  = "1" /* MAL NEO CAROTID BODY */
"1946"  = "1" /* MAL NEO PARAGANGLIA NEC */
"1948"  = "1" /* MAL NEO ENDOCRINE NEC */
"1949"  = "1" /* MAL NEO ENDOCRINE NOS */
"1950"  = "1" /* MAL NEO HEAD/FACE/NECK */
"1951"  = "1" /* MALIGN NEOPL THORAX */
"1952"  = "1" /* MALIG NEO ABDOMEN */
"1953"  = "1" /* MALIGN NEOPL PELVIS */
"1954"  = "1" /* MALIGN NEOPL ARM */
"1955"  = "1" /* MALIGN NEOPL LEG */
"1958"  = "1" /* MALIG NEO SITE NEC */
"1960"  = "1" /* MAL NEO LYMPH-HEAD/NECK */
"1961"  = "1" /* MAL NEO LYMPH-INTRATHOR */
"1962"  = "1" /* MAL NEO LYMPH INTRA-ABD */
"1963"  = "1" /* MAL NEO LYMPH-AXILLA/ARM */
"1965"  = "1" /* MAL NEO LYMPH-INGUIN/LEG */
"1966"  = "1" /* MAL NEO LYMPH-INTRAPELV */
"1968"  = "1" /* MAL NEO LYMPH NODE-MULT */
"1969"  = "1" /* MAL NEO LYMPH NODE NOS */
"1970"  = "1" /* SECONDARY MALIG NEO LUNG */
"1971"  = "1" /* SEC MAL NEO MEDIASTINUM */
"1972"  = "1" /* SECOND MALIG NEO PLEURA */
"1973"  = "1" /* SEC MALIG NEO RESP NEC */
"1974"  = "1" /* SEC MALIG NEO SM BOWEL */
"1975"  = "1" /* SEC MALIG NEO LG BOWEL */
"1976"  = "1" /* SEC MAL NEO PERITONEUM */
"1977"  = "1" /* SECOND MALIG NEO LIVER */
"1978"  = "1" /* SEC MAL NEO GI NEC */
"1980"  = "1" /* SECOND MALIG NEO KIDNEY */
"1981"  = "1" /* SEC MALIG NEO URIN NEC */
"1982"  = "1" /* SECONDARY MALIG NEO SKIN */
"1983"  = "1" /* SEC MAL NEO BRAIN/SPINE */
"1984"  = "1" /* SEC MALIG NEO NERVE NEC */
"1985"  = "1" /* SECONDARY MALIG NEO BONE */
"1986"  = "1" /* SECOND MALIG NEO OVARY */
"1987"  = "1" /* SECOND MALIG NEO ADRENAL */
"19881" = "1" /* SECOND MALIG NEO BREAST */
"19882" = "1" /* SECOND MALIG NEO GENITAL */
"19889" = "1" /* SECONDARY MALIG NEO NEC */
"1990"  = "1" /* MALIG NEO DISSEMINATED */
"1991"  = "1" /* MALIGNANT NEOPLASM NOS */
"20000" = "1" /* RETCLSRC UNSP XTRNDL ORG */
"20001" = "1" /* RETICULOSARCOMA HEAD */
"20002" = "1" /* RETICULOSARCOMA THORAX */
"20003" = "1" /* RETICULOSARCOMA ABDOM */
"20004" = "1" /* RETICULOSARCOMA AXILLA */
"20005" = "1" /* RETICULOSARCOMA INGUIN */
"20006" = "1" /* RETICULOSARCOMA PELVIC */
"20007" = "1" /* RETICULOSARCOMA SPLEEN */
"20008" = "1" /* RETICULOSARCOMA MULT */
"20010" = "1" /* LYMPHSRC UNSP XTRNDL ORG */
"20011" = "1" /* LYMPHOSARCOMA HEAD */
"20012" = "1" /* LYMPHOSARCOMA THORAX */
"20013" = "1" /* LYMPHOSARCOMA ABDOM */
"20014" = "1" /* LYMPHOSARCOMA AXILLA */
"20015" = "1" /* LYMPHOSARCOMA INGUIN */
"20016" = "1" /* LYMPHOSARCOMA PELVIC */
"20017" = "1" /* LYMPHOSARCOMA SPLEEN */
"20018" = "1" /* LYMPHOSARCOMA MULT */
"20020" = "1" /* BRKT TMR UNSP XTRNDL ORG */
"20021" = "1" /* BURKITTS TUMOR HEAD */
"20022" = "1" /* BURKITTS TUMOR THORAX */
"20023" = "1" /* BURKITTS TUMOR ABDOM */
"20024" = "1" /* BURKITTS TUMOR AXILLA */
"20025" = "1" /* BURKITTS TUMOR INGUIN */
"20026" = "1" /* BURKITTS TUMOR PELVIC */
"20027" = "1" /* BURKITTS TUMOR SPLEEN */
"20028" = "1" /* BURKITTS TUMOR MULT */
"20030" = "1" /* MARGNL ZONE LYM XTRNDL */
"20031" = "1" /* MARGIN ZONE LYM HEAD */
"20032" = "1" /* MARGIN ZONE LYM THORAX */
"20033" = "1" /* MARGIN ZONE LYM ABDOM */
"20034" = "1" /* MARGIN ZONE LYM AXILLA */
"20035" = "1" /* MARGIN ZONE LYM INGUIN */
"20036" = "1" /* MARGIN ZONE LYM PELVIC */
"20037" = "1" /* MARGIN ZONE LYMPH SPLEEN */
"20038" = "1" /* MARGIN ZONE LYMPH MULTIP */
"20040" = "1" /* MANTLE CELL LYM XTRRNDL */
"20041" = "1" /* MANTLE CELL LYMPH HEAD */
"20042" = "1" /* MANTLE CELL LYMPH THORAX */
"20043" = "1" /* MANTLE CELL LYMPH ABDOM */
"20044" = "1" /* MANTLE CELL LYMPH AXILLA */
"20045" = "1" /* MANTLE CELL LYMPH INGUIN */
"20046" = "1" /* MANTLE CELL LYMPH PELVIC */
"20047" = "1" /* MANTLE CELL LYMPH SPLEEN */
"20048" = "1" /* MANTLE CELL LYMPH MULTIP */
"20050" = "1" /* PRIMARY CNS LYMPH XTRNDL */
"20051" = "1" /* PRIMARY CNS LYMPH HEAD */
"20052" = "1" /* PRIMARY CNS LYMPH THORAX */
"20053" = "1" /* PRIMARY CNS LYMPH ABDOM */
"20054" = "1" /* PRIMARY CNS LYMPH AXILLA */
"20055" = "1" /* PRIMARY CNS LYM INGUIN */
"20056" = "1" /* PRIMARY CNS LYMPH PELVIC */
"20057" = "1" /* PRIMARY CNS LYMPH SPLEEN */
"20058" = "1" /* PRIMARY CNS LYMPH MULTIP */
"20060" = "1" /* ANAPLASTIC LYMPH XTRNDL */
"20061" = "1" /* ANAPLASTIC LYMPH HEAD */
"20062" = "1" /* ANAPLASTIC LYMPH THORAX */
"20063" = "1" /* ANAPLASTIC LYMPH ABDOM */
"20064" = "1" /* ANAPLASTIC LYMPH AXILLA */
"20065" = "1" /* ANAPLASTIC LYMPH INGUIN */
"20066" = "1" /* ANAPLASTIC LYMPH PELVIC */
"20067" = "1" /* ANAPLASTIC LYMPH SPLEEN */
"20068" = "1" /* ANAPLASTIC LYMPH MULTIP */
"20070" = "1" /* LARGE CELL LYMPH XTRNDL */
"20071" = "1" /* LARGE CELL LYMPHOMA HEAD */
"20072" = "1" /* LARGE CELL LYMPH THORAX */
"20073" = "1" /* LARGE CELL LYMPH ABDOM */
"20074" = "1" /* LARGE CELL LYMPH AXILLA */
"20075" = "1" /* LARGE CELL LYMPH INGUIN */
"20076" = "1" /* LARGE CELL LYMPH PELVIC */
"20077" = "1" /* LARGE CELL LYMPH SPLEEN */
"20078" = "1" /* LARGE CELL LYMPH MULTIP */
"20080" = "1" /* OTH VARN UNSP XTRNDL ORG */
"20081" = "1" /* MIXED LYMPHOSARC HEAD */
"20082" = "1" /* MIXED LYMPHOSARC THORAX */
"20083" = "1" /* MIXED LYMPHOSARC ABDOM */
"20084" = "1" /* MIXED LYMPHOSARC AXILLA */
"20085" = "1" /* MIXED LYMPHOSARC INGUIN */
"20086" = "1" /* MIXED LYMPHOSARC PELVIC */
"20087" = "1" /* MIXED LYMPHOSARC SPLEEN */
"20088" = "1" /* MIXED LYMPHOSARC MULT */
"20100" = "1" /* HDGK PRG UNSP XTRNDL ORG */
"20101" = "1" /* HODGKINS PARAGRAN HEAD */
"20102" = "1" /* HODGKINS PARAGRAN THORAX */
"20103" = "1" /* HODGKINS PARAGRAN ABDOM */
"20104" = "1" /* HODGKINS PARAGRAN AXILLA */
"20105" = "1" /* HODGKINS PARAGRAN INGUIN */
"20106" = "1" /* HODGKINS PARAGRAN PELVIC */
"20107" = "1" /* HODGKINS PARAGRAN SPLEEN */
"20108" = "1" /* HODGKINS PARAGRAN MULT */
"20110" = "1" /* HDGK GRN UNSP XTRNDL ORG */
"20111" = "1" /* HODGKINS GRANULOM HEAD */
"20112" = "1" /* HODGKINS GRANULOM THORAX */
"20113" = "1" /* HODGKINS GRANULOM ABDOM */
"20114" = "1" /* HODGKINS GRANULOM AXILLA */
"20115" = "1" /* HODGKINS GRANULOM INGUIN */
"20116" = "1" /* HODGKINS GRANULOM PELVIC */
"20117" = "1" /* HODGKINS GRANULOM SPLEEN */
"20118" = "1" /* HODGKINS GRANULOM MULT */
"20120" = "1" /* HDGK SRC UNSP XTRNDL ORG */
"20121" = "1" /* HODGKINS SARCOMA HEAD */
"20122" = "1" /* HODGKINS SARCOMA THORAX */
"20123" = "1" /* HODGKINS SARCOMA ABDOM */
"20124" = "1" /* HODGKINS SARCOMA AXILLA */
"20125" = "1" /* HODGKINS SARCOMA INGUIN */
"20126" = "1" /* HODGKINS SARCOMA PELVIC */
"20127" = "1" /* HODGKINS SARCOMA SPLEEN */
"20128" = "1" /* HODGKINS SARCOMA MULT */
"20140" = "1" /* LYM-HST UNSP XTRNDL ORGN */
"20141" = "1" /* HODG LYMPH-HISTIO HEAD */
"20142" = "1" /* HODG LYMPH-HISTIO THORAX */
"20143" = "1" /* HODG LYMPH-HISTIO ABDOM */
"20144" = "1" /* HODG LYMPH-HISTIO AXILLA */
"20145" = "1" /* HODG LYMPH-HISTIO INGUIN */
"20146" = "1" /* HODG LYMPH-HISTIO PELVIC */
"20147" = "1" /* HODG LYMPH-HISTIO SPLEEN */
"20148" = "1" /* HODG LYMPH-HISTIO MULT */
"20150" = "1" /* NDR SCLR UNSP XTRNDL ORG */
"20151" = "1" /* HODG NODUL SCLERO HEAD */
"20152" = "1" /* HODG NODUL SCLERO THORAX */
"20153" = "1" /* HODG NODUL SCLERO ABDOM */
"20154" = "1" /* HODG NODUL SCLERO AXILLA */
"20155" = "1" /* HODG NODUL SCLERO INGUIN */
"20156" = "1" /* HODG NODUL SCLERO PELVIC */
"20157" = "1" /* HODG NODUL SCLERO SPLEEN */
"20158" = "1" /* HODG NODUL SCLERO MULT */
"20160" = "1" /* MXD CELR UNSP XTRNDL ORG */
"20161" = "1" /* HODGKINS MIX CELL HEAD */
"20162" = "1" /* HODGKINS MIX CELL THORAX */
"20163" = "1" /* HODGKINS MIX CELL ABDOM */
"20164" = "1" /* HODGKINS MIX CELL AXILLA */
"20165" = "1" /* HODGKINS MIX CELL INGUIN */
"20166" = "1" /* HODGKINS MIX CELL PELVIC */
"20167" = "1" /* HODGKINS MIX CELL SPLEEN */
"20168" = "1" /* HODGKINS MIX CELL MULT */
"20170" = "1" /* LYM DPLT UNSP XTRNDL ORG */
"20171" = "1" /* HODG LYMPH DEPLET HEAD */
"20172" = "1" /* HODG LYMPH DEPLET THORAX */
"20173" = "1" /* HODG LYMPH DEPLET ABDOM */
"20174" = "1" /* HODG LYMPH DEPLET AXILLA */
"20175" = "1" /* HODG LYMPH DEPLET INGUIN */
"20176" = "1" /* HODG LYMPH DEPLET PELVIC */
"20177" = "1" /* HODG LYMPH DEPLET SPLEEN */
"20178" = "1" /* HODG LYMPH DEPLET MULT */
"20190" = "1" /* HDGK DIS UNSP XTRNDL ORG */
"20191" = "1" /* HODGKINS DIS NOS HEAD */
"20192" = "1" /* HODGKINS DIS NOS THORAX */
"20193" = "1" /* HODGKINS DIS NOS ABDOM */
"20194" = "1" /* HODGKINS DIS NOS AXILLA */
"20195" = "1" /* HODGKINS DIS NOS INGUIN */
"20196" = "1" /* HODGKINS DIS NOS PELVIC */
"20197" = "1" /* HODGKINS DIS NOS SPLEEN */
"20198" = "1" /* HODGKINS DIS NOS MULT */
"20200" = "1" /* NDLR LYM UNSP XTRNDL ORG */
"20201" = "1" /* NODULAR LYMPHOMA HEAD */
"20202" = "1" /* NODULAR LYMPHOMA THORAX */
"20203" = "1" /* NODULAR LYMPHOMA ABDOM */
"20204" = "1" /* NODULAR LYMPHOMA AXILLA */
"20205" = "1" /* NODULAR LYMPHOMA INGUIN */
"20206" = "1" /* NODULAR LYMPHOMA PELVIC */
"20207" = "1" /* NODULAR LYMPHOMA SPLEEN */
"20208" = "1" /* NODULAR LYMPHOMA MULT */
"20210" = "1" /* MYCS FNG UNSP XTRNDL ORG */
"20211" = "1" /* MYCOSIS FUNGOIDES HEAD */
"20212" = "1" /* MYCOSIS FUNGOIDES THORAX */
"20213" = "1" /* MYCOSIS FUNGOIDES ABDOM */
"20214" = "1" /* MYCOSIS FUNGOIDES AXILLA */
"20215" = "1" /* MYCOSIS FUNGOIDES INGUIN */
"20216" = "1" /* MYCOSIS FUNGOIDES PELVIC */
"20217" = "1" /* MYCOSIS FUNGOIDES SPLEEN */
"20218" = "1" /* MYCOSIS FUNGOIDES MULT */
"20220" = "1" /* SZRY DIS UNSP XTRNDL ORG */
"20221" = "1" /* SEZARYS DISEASE HEAD */
"20222" = "1" /* SEZARYS DISEASE THORAX */
"20223" = "1" /* SEZARYS DISEASE ABDOM */
"20224" = "1" /* SEZARYS DISEASE AXILLA */
"20225" = "1" /* SEZARYS DISEASE INGUIN */
"20226" = "1" /* SEZARYS DISEASE PELVIC */
"20227" = "1" /* SEZARYS DISEASE SPLEEN */
"20228" = "1" /* SEZARYS DISEASE MULT */
"20230" = "1" /* MLG HIST UNSP XTRNDL ORG */
"20231" = "1" /* MAL HISTIOCYTOSIS HEAD */
"20232" = "1" /* MAL HISTIOCYTOSIS THORAX */
"20233" = "1" /* MAL HISTIOCYTOSIS ABDOM */
"20234" = "1" /* MAL HISTIOCYTOSIS AXILLA */
"20235" = "1" /* MAL HISTIOCYTOSIS INGUIN */
"20236" = "1" /* MAL HISTIOCYTOSIS PELVIC */
"20237" = "1" /* MAL HISTIOCYTOSIS SPLEEN */
"20238" = "1" /* MAL HISTIOCYTOSIS MULT */
"20240" = "1" /* LK RTCTL UNSP XTRNDL ORG */
"20241" = "1" /* HAIRY-CELL LEUKEM HEAD */
"20242" = "1" /* HAIRY-CELL LEUKEM THORAX */
"20243" = "1" /* HAIRY-CELL LEUKEM ABDOM */
"20244" = "1" /* HAIRY-CELL LEUKEM AXILLA */
"20245" = "1" /* HAIRY-CELL LEUKEM INGUIN */
"20246" = "1" /* HAIRY-CELL LEUKEM PELVIC */
"20247" = "1" /* HAIRY-CELL LEUKEM SPLEEN */
"20248" = "1" /* HAIRY-CELL LEUKEM MULT */
"20250" = "1" /* LTR-SIWE UNSP XTRNDL ORG */
"20251" = "1" /* LETTERER-SIWE DIS HEAD */
"20252" = "1" /* LETTERER-SIWE DIS THORAX */
"20253" = "1" /* LETTERER-SIWE DIS ABDOM */
"20254" = "1" /* LETTERER-SIWE DIS AXILLA */
"20255" = "1" /* LETTERER-SIWE DIS INGUIN */
"20256" = "1" /* LETTERER-SIWE DIS PELVIC */
"20257" = "1" /* LETTERER-SIWE DIS SPLEEN */
"20258" = "1" /* LETTERER-SIWE DIS MULT */
"20260" = "1" /* MLG MAST UNSP XTRNDL ORG */
"20261" = "1" /* MAL MASTOCYTOSIS HEAD */
"20262" = "1" /* MAL MASTOCYTOSIS THORAX */
"20263" = "1" /* MAL MASTOCYTOSIS ABDOM */
"20264" = "1" /* MAL MASTOCYTOSIS AXILLA */
"20265" = "1" /* MAL MASTOCYTOSIS INGUIN */
"20266" = "1" /* MAL MASTOCYTOSIS PELVIC */
"20267" = "1" /* MAL MASTOCYTOSIS SPLEEN */
"20268" = "1" /* MAL MASTOCYTOSIS MULT */
"20270" = "1" /* PERIPH T CELL LYM XTRNDL */
"20271" = "1" /* PERIPH T CELL LYMPH HEAD */
"20272" = "1" /* PERIPH T CELL LYM THORAX */
"20273" = "1" /* PERIPH T CELL LYM ABDOM */
"20274" = "1" /* PERIPH T CELL LYM AXILLA */
"20275" = "1" /* PERIPH T CELL LYM INGUIN */
"20276" = "1" /* PERIPH T CELL LYM PELVIC */
"20277" = "1" /* PERIPH T CELL LYM SPLEEN */
"20278" = "1" /* PERIPH T CELL LYM MULTIP */
"20280" = "1" /* OTH LYMP UNSP XTRNDL ORG */
"20281" = "1" /* LYMPHOMAS NEC HEAD */
"20282" = "1" /* LYMPHOMAS NEC THORAX */
"20283" = "1" /* LYMPHOMAS NEC ABDOM */
"20284" = "1" /* LYMPHOMAS NEC AXILLA */
"20285" = "1" /* LYMPHOMAS NEC INGUIN */
"20286" = "1" /* LYMPHOMAS NEC PELVIC */
"20287" = "1" /* LYMPHOMAS NEC SPLEEN */
"20288" = "1" /* LYMPHOMAS NEC MULT */
"20290" = "1" /* UNSP LYM UNSP XTRNDL ORG */
"20291" = "1" /* LYMPHOID MAL NEC HEAD */
"20292" = "1" /* LYMPHOID MAL NEC THORAX */
"20293" = "1" /* LYMPHOID MAL NEC ABDOM */
"20294" = "1" /* LYMPHOID MAL NEC AXILLA */
"20295" = "1" /* LYMPHOID MAL NEC INGUIN */
"20296" = "1" /* LYMPHOID MAL NEC PELVIC */
"20297" = "1" /* LYMPHOID MAL NEC SPLEEN */
"20298" = "1" /* LYMPHOID MAL NEC MULT */
"2030"  = "1" /* MULTIPLE MYELOMA */
"20300" = "1" /* MULT MYELM W/O REMISSION */
"20301" = "1" /* MULT MYELM W REMISSION */
"2031"  = "1" /* PLASMA CELL LEUKEMIA */
"20310" = "1" /* PLSM CELL LEUK W/O RMSON */
"20311" = "1" /* PLSM CELL LEUK W RMSON */
"2038"  = "1" /* IMMUNOPROLIFERAT NEO NEC */
"20380" = "1" /* OTH IMNPRFL NPL W/O RMSN */
"20381" = "1" /* OTH IMNPRFL NPL W RMSN */
"2040"  = "1" /* ACUTE LYMPHOID LEUKEMIA */
"20400" = "1" /* ACT LYM LEUK W/O RMSION */
"20401" = "1" /* ACT LYM LEUK W RMSION */
"2041"  = "1" /* CHR LYMPHOID LEUKEMIA */
"20410" = "1" /* CHR LYM LEUK W/O RMSION */
"20411" = "1" /* CHR LYM LEUK W RMSION */
"2042"  = "1" /* SUBAC LYMPHOID LEUKEMIA */
"20420" = "1" /* SBAC LYM LEUK W/O RMSION */
"20421" = "1" /* SBAC LYM LEUK W RMSION */
"2048"  = "1" /* LYMPHOID LEUKEMIA NEC */
"20480" = "1" /* OTH LYM LEUK W/O RMSION */
"20481" = "1" /* OTH LYM LEUK W RMSION */
"2049"  = "1" /* LYMPHOID LEUKEMIA NOS */
"20490" = "1" /* UNS LYM LEUK W/O RMSION */
"20491" = "1" /* UNS LYM LEUK W RMSION */
"2050"  = "1" /* ACUTE MYELOID LEUKEMIA */
"20500" = "1" /* ACT MYL LEUK W/O RMSION */
"20501" = "1" /* ACT MYL LEUK W RMSION */
"2051"  = "1" /* CHRONIC MYELOID LEUKEMIA */
"20510" = "1" /* CHR MYL LEUK W/O RMSION */
"20511" = "1" /* CHR MYL LEUK W RMSION */
"2052"  = "1" /* SUBACUT MYELOID LEUKEMIA */
"20520" = "1" /* SBAC MYL LEUK W/O RMSION */
"20521" = "1" /* SBAC MYL LEUK W RMSION */
"2053"  = "1" /* MYELOID SARCOMA */
"20530" = "1" /* MYL SRCOMA W/O RMSION */
"20531" = "1" /* MYL SRCOMA W RMSION */
"2058"  = "1" /* MYELOID LEUKEMIA NEC */
"20580" = "1" /* OTH MYL LEUK W/O RMSION */
"20581" = "1" /* OTH MYL LEUK W RMSION */
"2059"  = "1" /* MYELOID LEUKEMIA NOS */
"20590" = "1" /* UNS MYL LEUK W/O RMSION */
"20591" = "1" /* UNS MYL LEUK W RMSION */
"2060"  = "1" /* ACUTE MONOCYTIC LEUKEMIA */
"20600" = "1" /* ACT MONO LEUK W/O RMSION */
"20601" = "1" /* ACT MONO LEUK W RMSION */
"2061"  = "1" /* CHR MONOCYTIC LEUKEMIA */
"20610" = "1" /* CHR MONO LEUK W/O RMSION */
"20611" = "1" /* CHR MONO LEUK W RMSION */
"2062"  = "1" /* SUBAC MONOCYTIC LEUKEMIA */
"20620" = "1" /* SBAC MONO LEUK W/O RMSON */
"20621" = "1" /* SBAC MONO LEUK W RMSION */
"2068"  = "1" /* MONOCYTIC LEUKEMIA NEC */
"20680" = "1" /* OTH MONO LEUK W/O RMSION */
"20681" = "1" /* OTH MONO LEUK W RMSION */
"2069"  = "1" /* MONOCYTIC LEUKEMIA NOS */
"20690" = "1" /* UNS MONO LEUK W/O RMSION */
"20691" = "1" /* UNS MONO LEUK W RMSION */
"2070"  = "1" /* ACUTE ERYTHREMIA */
"20700" = "1" /* ACT ERTH/ERYLK W/O RMSON */
"20701" = "1" /* ACT ERTH/ERYLK W RMSON */
"2071"  = "1" /* CHRONIC ERYTHREMIA */
"20710" = "1" /* CHR ERYTHRM W/O REMISION */
"20711" = "1" /* CHR ERYTHRM W REMISION */
"2072"  = "1" /* MEGAKARYOCYTIC LEUKEMIA */
"20720" = "1" /* MGKRYCYT LEUK W/O RMSION */
"20721" = "1" /* MGKRYCYT LEUK W RMSION */
"2078"  = "1" /* SPECIFIED LEUKEMIA NEC */
"20780" = "1" /* OTH SPF LEUK W/O REMSION */
"20781" = "1" /* OTH SPF LEUK W REMSION */
"2080"  = "1" /* ACT LEUK UNS CL W/O RMSN */
"20800" = "1" /* ACT LEUK UNS CL W/O RMSN */
"20801" = "1" /* ACT LEUK UNS CL W RMSON */
"2081"  = "1" /* CHRONIC LEUKEMIA NOS */
"20810" = "1" /* CHR LEUK UNS CL W/O RMSN */
"20811" = "1" /* CHR LEUK UNS CL W RMSON */
"2082"  = "1" /* SUBACUTE LEUKEMIA NOS */
"20820" = "1" /* SBAC LEUK UNS CL W/O RMS */
"20821" = "1" /* SBAC LEUK UNS CL W RMSON */
"2088"  = "1" /* LEUKEMIA-UNSPEC CELL NEC */
"20880" = "1" /* OTH LEUK UNS CL W/O RMSN */
"20881" = "1" /* OTH LEUK UNS CL W RMSON */
"2089"  = "1" /* LEUKEMIA-UNSPEC CELL NOS */
"20890" = "1" /* LEUKEMIA NOS W/O REMSION */
"20891" = "1" /* LEUKEMIA NOS W REMISSIO */
"2386"  = "1" /* NEOPLASM OF UNCERTAIN BEHAVIOR OF OTHER AND UNSPECIFIED SITES AND TISSUES, PLASMA CELLS */
"2733"  = "1" /* DISORDERS OF PLASMA PROTEIN METABOLISM-MACROGLOBULINEMIA */
"V1000" = "1" /* GASTROINTESTINAL TRACT, UNSPECIFIED */
"V1001" = "1" /* TONGUE */
"V1002" = "1" /* OTHER AND UNSPECIFIED ORAL CAVITY AND PHARYNX */
"V1003" = "1" /* ESOPHAGUS */
"V1004" = "1" /* STOMACH */
"V1005" = "1" /* LARGE INTESTINE */
"V1006" = "1" /* RECTUM, RECTOSIGMOID JUNCTION, AND ANUS */
"V1007" = "1" /* LIVER */
"V1009" = "1" /* OTHER */
"V1011" = "1" /* BRONCHUS AND LUNG */
"V1012" = "1" /* TRACHEA */
"V1020" = "1" /* RESPIRATORY ORGAN, UNSPECIFIED */
"V1021" = "1" /* LARYNX */
"V1022" = "1" /* NASAL CAVITIES, MIDDLE EAR, AND ACCESSORY SINUSES */
"V1029" = "1" /* OTHER */
"V103"  = "1" /* BREAST */
"V1040" = "1" /* FEMALE GENITAL ORGAN, UNSPECIFIED */
"V1041" = "1" /* CERVIX UTERI */
"V1042" = "1" /* OTHER PARTS OF UTERUS */
"V1043" = "1" /* OVARY */
"V1044" = "1" /* OTHER FEMALE GENITAL ORGANS */
"V1045" = "1" /* MALE GENITAL ORGAN, UNSPECIFIED */
"V1046" = "1" /* PROSTATE */
"V1047" = "1" /* TESTIS */
"V1048" = "1" /* EPIDDIDYMIS */
"V1049" = "1" /* OTHER MALE GENITAL ORGANS */
"V1050" = "1" /* URINARY ORGAN, UNSPECIFIED */
"V1051" = "1" /* BLADDER */
"V1052" = "1" /* KIDNEY */
"V1053" = "1" /* RENAL PELVIS */
"V1059" = "1" /* OTHER */
"V1060" = "1" /* LEUKEMIA, UNSPECIFIED */
"V1061" = "1" /* LYMPHID LEUKEMIA */
"V1062" = "1" /* MYELOID LEUKEMIA */
"V1063" = "1" /* MONOCYTIC LEUKEMIA */
"V1069" = "1" /* OTHER */
"V1071" = "1" /* LYMPHOSARCOMA AND RETICULOSARCOMA */
"V1072" = "1" /* HODGKINS DISEASE */
"V1079" = "1" /* OTHER */
"V1081" = "1" /* BONE */
"V1082" = "1" /* MALIGNANT MELANOMA OF SKIN */
"V1083" = "1" /* OPTHER MALIGNANT NEOPLASM OF SKIN */
"V1084" = "1" /* EYE */
"V1085" = "1" /* BRAIN */
"V1086" = "1" /* OTHER PARTS OF NERVOUS SYSTEM */
"V1087" = "1" /* THYROID */
"V1088" = "1" /* OTHER ENDOCRINE GLANDS AND RELATED STRUCTURES */
"V1089" = "1" /* OTHER */
"V109"  = "1" /* UNSPECIFIED PERSONAL HISTORY OF MALIGNANT NEOPLASM */
OTHER   = "0" ;

 /* EXCLUDE: CANCER */
 /* DIAGNOSTIC RELATED GROUPS (DRGS) */

VALUE $CANCEDR
"010" = "1" /* NERVOUS SYSTEM NEOPLASMS WITH CC */
"011" = "1" /* NERVOUS SYSTEM NEOPLASMS WITHOUT CC */
"064" = "1" /* EAR, NOSE, MOUTH AND THROAT MALIGNANCY */
"082" = "1" /* RESPIRATORY NEOPLASMS */
"172" = "1" /* DIGESTIVE MALIGNANCY WITH CC */
"173" = "1" /* DIGESTIVE MALIGNANCY WITHOUT CC */
"199" = "1" /* HEPATOBILIARY DIAGNOSTIC PROCEDURE FOR MALIGNANCY */
"203" = "1" /* MALIGNANCY OF HEPATOBILIARY SYSTEM OR PANCREAS */
"239" = "1" /* PATHOLOGICAL FRACTURES AND MUSCULOSKELETAL AND CONNECTIVE TISSUE MALIGNANCY */
"257" = "1" /* TOTAL MASTECTOMY FOR MALIGNANCY WITH CC */
"258" = "1" /* TOTAL MASTECTOMY FOR MALIGNANCY WITHOUT CC */
"259" = "1" /* SUBTOTAL MASTECTOMY FOR MALIGNANCY WITH CC */
"260" = "1" /* SUBTOTAL MASTECTOMY FOR MALIGNANCY WITHOUT CC */
"274" = "1" /* MALIGNANT BREAST DISORDERS WITH CC */
"275" = "1" /* MALIGNANT BREAST DISORDERS WITHOUT CC */
"303" = "1" /* KIDNEY, URETER AND MAJOR BLADDER PROCEDURES FOR NEOPLASM */
"318" = "1" /* KIDNEY AND URINARY TRACT NEOPLSMS WITH CC */
"319" = "1" /* KIDNEY AND URINARY TRACT NEOPLASMS WITHOUT CC */
"338" = "1" /* TESTES PROCEDURES FOR MALIGNANCY */
"344" = "1" /* OTHER MALE REPRODUCTIVE SYSTEM OR PROCEDURES FOR MALIGNANCY */
"346" = "1" /* MALIGNANCY OF MALE REPRODUCTIVE SYSTEM WITH CC */
"347" = "1" /* MALIGNANCY OF MALE REPRODUCTIVE SYSTEM WITHOUT CC */
"354" = "1" /* UTERINE AND ADNEXA PROCEDURES FOR NONOVARIAN/ADNEXAL MALIGNANCY WITH CC */
"355" = "1" /* UTERINE AND ADNEXA PROCEDURES FOR NONOVARIAN/ADNEXAL MALIGNANCY WITHOUT CC */
"357" = "1" /* UTERINE AND ADNEXA PROCEDURES FOR OVARIAN OR ADNEXAL MALIGNANCY */
"363" = "1" /* D AND C, CONIZATION AND RADIOIMPLANT FOR MALIGNANCY */
"367" = "1" /* MALIGNANCY OF FEMALE REPRODUCTIVE SYSTEM WITHOUT CC */
"400" = "1" /* LYMPHOMA AND LEUKEMIA WITH MAJOR OR PROCEDURES */
"401" = "1" /* LYMPHOMA AND NONACUTE LEUKEMIA WITH OTHER OR PROCEDURE WITH CC */
"402" = "1" /* LYMPHOMA AND NONACUTE LEUKEMIA WITH OTHER OR PROCEDURE WITHOUT CC */
"403" = "1" /* LYMPHOMA AND NONACUTE LEUKEMIA WITH CC */
"404" = "1" /* LYMPHOMA AND NONACUTE LEUKEMIA WITHOUT CC */
"405" = "1" /* ACUTE LEUKEMAI WITHOUT MAJOR OR PROCEDURE, AGE 0-17 */
"406" = "1" /* MYELOPROLIFERATIVE DISORDERS OR POORLY DIFFERENTIATED NEOPLASMS WITH MAJOR OR PROCEDURES WITH CC */
"407" = "1" /* MYELOPROLIFERATIVE DISORDERS OR POORLY DIFFERENTIATED NEOPLASMS WITH MAJOR OR PROCEDURE WITHOUT CC */
"408" = "1" /* MYELOPROLIFERATIVE DISORDERS OR POORLY DIFFERENTIATED NEOPLASMS WITH OTHER OR PROCEDURES */
"409" = "1" /* RADIOTHERAPY */
"410" = "1" /* CHEMOTHERAPY WITHOUT ACUTE LEUKEMIA AS SECONDARY DIAGNOSIS */
"411" = "1" /* HISTORY OF MALIGNANCY WITHOUT ENDOSCOPY */
"412" = "1" /* HISTORY OF MALIGNANCY WITH ENDOSCOPY */
"413" = "1" /* OTHER MYELOPROLIFERATIVE DISORDERS OR POORLY DIFFERENTIATED NEOPLASM DIAGNOSES WITH CC */
"414" = "1" /* OTHER MYELOPROLIFERATIVE DISORDERS OR POORLY DIFFERENTIATED NEOPLASM DIAGNOSES WITHOUT CC */
"473" = "1" /* ACUTE LEUKEMIA WITHOUT MAJOR OR PROCEDURE, AGE GREATER THAN 17 */
"492" = "1" /* CHEMOTHERAPY WITH ACUTE LEUKEMIA AS SECONDARY DIAGNOSIS */
"539" = "1" /* LYMPHOMA & LEUKEMIA W MAJOR OR PROCEDURE W CC */
"540" = "1" /* LYMPHOMA & LEUKEMIA W MAJOR OR PROCEDURE W/O CC */
OTHER = "0" ;

 /* ------------------------------ */
 /* DECUBITUS ULCER                */
 /*   PROVIDER INDICATOR / PSI # 3 */
 /* ------------------------------ */

 /* ICD-9-CM DIAGNOSIS CODES: */

VALUE $DECUBID
"7070"  = "1" /* DECUBITUS ULCER */
"70700" = "1" /* DECUBITUS ULCER SITE NOS */
"70701" = "1" /* DECUBITUS ULCER, ELBOW */
"70702" = "1" /* DECUBITUS ULCER, UP BACK */
"70703" = "1" /* DECUBITUS ULCER, LOW BACK */
"70704" = "1" /* DECUBITUS ULCER, HIP */
"70705" = "1" /* DECUBITUS ULCER, BUTTOCK */
"70706" = "1" /* DECUBITUS ULCER, ANKLE */
"70707" = "1" /* DECUBITUS ULCER, HEEL */
"70709" = "1" /* DECUBITUS ULCER, SITE NEC */
OTHER   = "0" ;

 /* EXCLUDE: DEBRIDEMENT OR PEDICLE GRAFT */
 /* ICD-9-CM PROCEDURE CODES: */

VALUE $DEBRIDP
"8345"  = "1" /* OTHER MYECTOMY */           
"8622"  = "1" /* EXC WOUND DEBRIDEMENT */    
"8628"  = "1" /* NONEXCIS DEBRIDEMENT WND */ 
"8670"  = "1" /* PEDICLE GRAFT/FLAP NOS */   
"8671"  = "1" /* CUT & PREP PEDICLE GRAFT */ 
"8672"  = "1" /* PEDICLE GRAFT ADVANCEMEN */ 
"8674"  = "1" /* ATTACH PEDICLE GRAFT NEC */ 
"8675"  = "1" /* REVISION OF PEDICLE GRFT */ 
OTHER   = "0" ;

 /* EXCLUDE: SPINA BIFIDA OR ANOXIC BRAIN DAMAGE */
 /* ICD-9-CM DIAGNOSIS CODES: */

VALUE $SPINABD
"3481 "  = "1" /* ANOXIC BRAIN DAMAGE */
"74100"  = "1" /* SPINA BIFIDA, W HYDROCEPHALUS UNSPECIFIED REGION */
"74101"  = "1" /* SPINA BIFIDA, W HYDROCEPHALUS CERVICAL REGION */
"74102"  = "1" /* SPINA BIFIDA, W HYDROCEPHALUS DORSAL REGION */
"74103"  = "1" /* SPINA BIFIDA, W HYDROCEPHALUS LUMBAR REGION */
"74190"  = "1" /* SPINA BIFIDA, W/O HYDROCEPHALUS UNSPECIFIED REGION */
"74191"  = "1" /* SPINA BIFIDA, W/O HYDROCEPHALUS CERVICAL REGION */
"74192"  = "1" /* SPINA BIFIDA, W/O HYDROCEPHALUS DORSAL REGION */
"74193"  = "1" /* SPINA BIFIDA, W/O HYDROCEPHALUS LUMBAR REGION */
"7685 "  = "1" /* SEVERE BIRTH ASPHYXIA */
OTHER   = "0" ;

 /* EXCLUDE: HEMIPLEGIA, PARAPLEGIA, OR QUADRIPLEGIA */
 /* ICD-9-CM DIAGNOSIS CODES: */

VALUE $HEMIPID
"33371" = "1" /* ATHETOID CEREBRAL PALSY */  
"3420"  = "1" /* FLACCID HEMIPLEGIA */
"34200" = "1" /* FLCCD HMIPLGA UNSPF SIDE */
"34201" = "1" /* FLCCD HMIPLGA DOMNT SIDE */
"34202" = "1" /* FLCCD HMIPLG NONDMNT SDE */
"3421"  = "1" /* SPASTIC HEMIPLEGIA */
"34210" = "1" /* SPSTC HMIPLGA UNSPF SIDE */
"34211" = "1" /* SPSTC HMIPLGA DOMNT SIDE */
"34212" = "1" /* SPSTC HMIPLG NONDMNT SDE */
"34280" = "1" /* OT SP HMIPLGA UNSPF SIDE */
"34281" = "1" /* OT SP HMIPLGA DOMNT SIDE */
"34282" = "1" /* OT SP HMIPLG NONDMNT SDE */
"3429"  = "1" /* HEMIPLEGIA, UNSPECIFIED */
"34290" = "1" /* UNSP HEMIPLGA UNSPF SIDE */
"34291" = "1" /* UNSP HEMIPLGA DOMNT SIDE */
"34292" = "1" /* UNSP HMIPLGA NONDMNT SDE */
"3430"  = "1" /* INFANTILE CEREBRAL PALSY, DIPLEGIC */
"3431"  = "1" /* INFANTILE CEREBRAL PALSY, HEMIPLEGIC */
"3432"  = "1" /* INFANTILE CEREBRAL PALSY, QUADRIPLEGIC */
"3433"  = "1" /* INFANTILE CEREBRAL PALSY, MONOPLEGIC */
"3434"  = "1" /* INFANTILE CEREBRAL PALSY INFANTILE HEMIPLEGIA */
"3438"  = "1" /* INFANTILE CEREBRAL PALSY OTHER SPECIFIED INFANTILE CEREBRAL PALSY */
"3439"  = "1" /* INFANTILE CEREBRAL PALSY, INFANTILE CEREBRAL PALSY, UNSPECIFIED */
"3440"  = "1" /* QUADRIPLEGIA AND QUADRIPARESIS */
"34400" = "1" /* QUADRIPLEGIA, UNSPECIFD */
"34401" = "1" /* QUADRPLG C1-C4, COMPLETE */
"34402" = "1" /* QUADRPLG C1-C4, INCOMPLT */
"34403" = "1" /* QUADRPLG C5-C7, COMPLETE */
"34404" = "1" /* QUADRPLG C5-C7, INCOMPLT */
"34409" = "1" /* OTHER QUADRIPLEGIA */
"3441"  = "1" /* PARAPLEGIA */
"3442"  = "1" /* DIPLEGIA OF UPPER LIMBS */
"3443"  = "1" /* MONOPLEGIA OF LOWER LIMB */
"34430" = "1" /* MONPLGA LWR LMB UNSP SDE */
"34431" = "1" /* MONPLGA LWR LMB DMNT SDE */
"34432" = "1" /* MNPLG LWR LMB NONDMNT SD */
"3444"  = "1" /* MONOPLEGIA OF UPPER LIMB */
"34440" = "1" /* MONPLGA UPR LMB UNSP SDE */
"34441" = "1" /* MONPLGA UPR LMB DMNT SDE */
"34442" = "1" /* MNPLG UPR LMB NONDMNT SD */
"3445"  = "1" /* UNSPECIFIED MONOPLEGIA */
"34460" = "1" /* CAUDA EQUINA SYNDROME, WITHOUT MENTION OF NEUROGENIC BLADDER */
"34461" = "1" /* CAUDA EQUINA SYNDROME, WITH NEUROGENIC BLADDER */
"3448"  = "1" /* OTHER SPECIFIED PARALYTIC SYNDROMES */
"34481" = "1" /* LOCKED-IN STATE */
"34489" = "1" /* OTH SPCF PARALYTIC SYND */
"3449"  = "1" /* PARALYSIS, UNSPECIFIED */
"43820" = "1" /* LATE EF-HEMPLGA SIDE NOS */
"43821" = "1" /* LATE EF-HEMPLGA DOM SIDE */
"43822" = "1" /* LATE EF-HEMIPLGA NON-DOM */
"43830" = "1" /* LATE EF-MPLGA UP LMB NOS */
"43831" = "1" /* LATE EF-MPLGA UP LMB DOM */
"43832" = "1" /* LT EF-MPLGA UPLMB NONDOM */
"43840" = "1" /* LTE EF-MPLGA LOW LMB NOS */
"43841" = "1" /* LTE EF-MPLGA LOW LMB DOM */
"43842" = "1" /* LT EF-MPLGA LOWLMB NONDM */
"43850" = "1" /* LT EF OTH PARAL SIDE NOS */
"43851" = "1" /* LT EF OTH PARAL DOM SIDE */
"43852" = "1" /* LT EF OTH PARALS NON-DOM */
"43853" = "1" /* LT EF OTH PARALS-BILAT */
"7687"  = "1" /* HYPOXIC-ISCHEMIC ENCEPH */  
OTHER   = "0" ;

 /* ------------------------------ */
 /* DEATH AMONG SURGICAL           */
 /*   PROVIDER INDICATOR / PSI # 4 */
 /* ------------------------------ */

 /* FTR-ACUTE RENAL FAILURE */
 /* ICD-9-CM DIAGNOSIS CODES (ALL 4TH AND 5TH DIGITS INCLUDED): */

VALUE $FTR1DX
"5845"  = "1" /* ACUTE RENAL FAILURE - W LESION OF TUBULAR NECROSIS */
"5846"  = "1" /* ACUTE RENAL FAILURE - W LESION OF RENAL CORTICAL NECROSIS */
"5847"  = "1" /* ACUTE RENAL FAILURE - W LESION OF RENAL MEDULLARY NECROSIS */
"5848"  = "1" /* ACUTE RENAL FAILURE - W OTHER SPECIFIED PATHOLOGICAL LESION */
"5849"  = "1" /* ACUTE RENAL FAILURE - UNSPECIFIED */
"6393"  = "1"  /* POSTABORTION RENAL FAILURE */
"66930" = "1" /* ACUTE RENAL FAILURE FOLLOWING LABOR AND DELIVERY */
"66932" = "1" /* ACUTE RENAL FAILURE FOLLOWING LABOR AND DELIVERY */
"66934" = "1" /* ACUTE RENAL FAILURE FOLLOWING LABOR AND DELIVERY */
OTHER   = "0" ;

 /* EXCLUDE: ABORTION-RELATED RENAL FAILURE */
 /* ICD-9-CM DIAGNOSIS CODES (ALL 4TH AND 5TH DIGITS INCLUDED): */

VALUE $FTR1EX
"63430"  = "1" /* SPON AB W REN FAIL-UNSP */
"63431"  = "1" /* SPON AB W REN FAIL-INC */
"63432"  = "1" /* SPON AB W REN FAIL-COMP */
"63530"  = "1" /* LEG ABOR W REN FAIL-UNSP */
"63531"  = "1" /* LEG ABOR W REN FAIL-INC */
"63532"  = "1" /* LEG ABOR W REN FAIL-COMP */
"63630"  = "1" /* ILLEG AB W REN FAIL-UNSP */
"63631"  = "1" /* ILLEG AB W REN FAIL-INC */
"63632"  = "1" /* ILLEG AB W REN FAIL-COMP */
"63730"  = "1" /* AB NOS W RENAL FAIL-UNSP */
"63731"  = "1" /* AB NOS W RENAL FAIL-INC */
"63732"  = "1" /* AB NOS W RENAL FAIL-COMP */
"6383"   = "1" /* ATTEM ABORT W RENAL FAIL */
OTHER   = "0" ;

 /* EXCLUDE: ACUTE MYOCARDIAL INFARCTION */
 /* ICD-9-CM DIAGNOSIS CODES: */

VALUE $AMIID
"41000" = "1" /* AMI OF ANTEROLATERAL WALL  EPISODE OF CARE UNSPECIFIED */
"41001" = "1" /* AMI OF ANTEROLATERAL WALL - INITIAL EPISODE OF CARE */
"41010" = "1" /* AMI OF OTHER ANTERIOR WALL  EPISODE OF CARE UNSPECIFIED */
"41011" = "1" /* AMI OF OTHER ANTERIOR WALL  INITIAL EPISODE OF CARE */
"41020" = "1" /* AMI OF INFEROLATERAL WALL  EPISODE OF CARE UNSPECIFIED */
"41021" = "1" /* AMI OF INFEROLATERAL WALL  INITIAL EPISODE OF CARE */
"41030" = "1" /* AMI OF INFEROPOSTERIOR WALL  EPISODE OF CARE UNSPECIFIED */
"41031" = "1" /* AMI OF INFEROPOSTERIOR WALL  INITIAL EPISODE OF CARE */
"41040" = "1" /* AMI OF INFERIOR WALL - EPISODE OF CARE UNSPECIFIED */
"41041" = "1" /* AMI OF INFERIOR WALL - INITIAL EPISODE OF CARE */
"41050" = "1" /* AMI OF OTHER LATERAL WALL - EPISODE OF CARE UNSPECIFIED */
"41051" = "1" /* AMI OF OTHER LATERAL WALL - INITIAL EPISODE OF CARE */
"41060" = "1" /* AMI TRUE POSTERIOR WALL INFARCTION - EPISODE OF CARE UNSPECIFIED */
"41061" = "1" /* AMI TRUE POSTERIOR WALL INFARCTION - INITIAL EPISODE OF CARE */
"41070" = "1" /* AMI SUBENDOCARDIAL INFARCTION - EPISODE OF CARE UNSPECIFIED */
"41071" = "1" /* AMI SUBENDOCARDIAL INFARCTION - INITIAL EPISODE OF CARE */
"41080" = "1" /* AMI OF OTHER SPECIFIED SITES - EPISODE OF CARE UNSPECIFIED */
"41081" = "1" /* AMI OF OTHER SPECIFIED SITES - INITIAL EPISODE OF CARE */
"41090" = "1" /* AMI UNSPECIFIED SITE - EPISODE OF CARE UNSPECIFIED */
"41091" = "1" /* AMI UNSPECIFIED SITE - INITIAL EPISODE OF CARE */
OTHER   = "0" ;

 /* EXCLUDE: CARDIAC ARRHYTHMIA */
 /* ICD-9-CM DIAGNOSIS CODES: */

VALUE $CARDRID
"4260"  = "1" /* ATRIOVENTRICULAR BLOCK, COMPLETE */
"4270"  = "1" /* PAROXYSMAL SUPRAVENTRICULAR TACHYCARDIA */
"4271"  = "1" /* PAROSYSMAL VENTRICULAR TACHYCARDIA */
"4272"  = "1" /* PAROXYSMAL TACHYCARDIA, UNSPECIFIED */
"42731" = "1" /* ATRIAL FIBRILLATION */
"42732" = "1" /* ATRIAL FLUTTER */
"42741" = "1" /* VENTRICULAR FIBRILLATION */
"42742" = "1" /* VENTRICULAR FLUTTER */
"4279"  = "1" /* CARDIAC DYSRHYTHMIA */
OTHER   = "0" ;

 /* EXCLUDE: CARDIAC ARREST */
 /* ICD-9-CM DIAGNOSIS CODES: */

VALUE $CARDIID
"4275"  = "1" /* CARDIAC ARREST */
OTHER   = "0" ;

 /* EXCLUDE: HEMORRHAGE */
 /* ICD-9-CM DIAGNOSIS CODES: */

VALUE $HEMORID
"2851"  = "1" /* ACUTE POSTHEMORRHAGIC ANEMIA */
"4590"  = "1" /* OTHER DISORDERS OF CIRCULATORY SYSTEM, HEMORRHAGE, UNSPECIFIED */
"9582"  = "1" /* CERTAIN EARLY COMPLICATIONS OF TRAUMA, SECONDARY AND RECURRENT HEMORRHAGE */
"99811" = "1" /* HEMORRHAGE COMPLICATING A PROCEDURE */
OTHER   = "0" ;


 /* EXCLUDE: SHOCK */
 /* ICD-9-CM DIAGNOSIS CODES: */

VALUE $SHOCKID
"63450" = "1" /* SPON ABORT W SHOCK-UNSP */
"63451" = "1" /* SPON ABORT W SHOCK-INC */
"63452" = "1" /* SPON ABORT W SHOCK-COMP */
"63550" = "1" /* LEGAL ABORT W SHOCK-UNSP */
"63551" = "1" /* LEGAL ABORT W SHOCK-INC */
"63552" = "1" /* LEGAL ABORT W SHOCK-COMP */
"63650" = "1" /* ILLEG ABORT W SHOCK-UNSP */
"63651" = "1" /* ILLEG ABORT W SHOCK-INC */
"63652" = "1" /* ILLEG ABORT W SHOCK-COMP */
"63750" = "1" /* ABORT NOS W SHOCK-UNSP */
"63751" = "1" /* ABORT NOS W SHOCK-INC */
"63752" = "1" /* ABORT NOS W SHOCK-COMP */
"6385"  = "1" /* ATTEM ABORTION W SHOCK */
"6395"  = "1" /* COMPLICATIONS FOLLOWING ABORTION AND ECTOPIC AND MOLAR PREGNANCIES, SHOCK */
"66910" = "1" /* SHOCK DURING OR FOLLOWING LABOR AND DELIVERY*/
"66911" = "1" /* SHOCK DURING OR FOLLOWING LABOR AND DELIVERY*/
"66912" = "1" /* SHOCK DURING OR FOLLOWING LABOR AND DELIVERY*/
"66913" = "1" /* SHOCK DURING OR FOLLOWING LABOR AND DELIVERY*/
"66914" = "1" /* SHOCK DURING OR FOLLOWING LABOR AND DELIVERY*/
"7855"  = "1" /* SHOCK NOS */
"78550" = "1" /* SHOCK, UNSPECIFIED */
"78551" = "1" /* CARDIOGENIC SHOCK */
"78552" = "1" /* SEPTIC SHOCK */
"78559" = "1" /* SHOCK WITHOUT MENTION OF TRAUMA, OTHER  */
"9950"  = "1" /* OTHER ANAPHYLACTIC SHOCK */
"9954"  = "1" /* SHOCK DUE TO ANESTHESIA */
"9980"  = "1" /* POSTOPERATIVE SHOCK */
"9994"  = "1" /* ANAPHYLACTIC SHOCK, DUE TO SERUM*/
OTHER   = "0"  ;

 /* FTR-DVT PE */
 /* ICD-9-CM DIAGNOSIS CODES: */

VALUE $FTR2DX
"4151"  = "1" /* PULMONARY EMBOLISM AND INFARCTION */
"41511" = "1" /* IATROGENIC PULMONARY EMBOLISM */
"41519" = "1" /* OTHER PULMONARY EMBOLISM AND INFARCTION*/
"45111" = "1" /* PHLEBITIS AND THORBOPHLEBITIS FEMORAL VEIN (DEEP) (SUPERFICIAL) */
"45119" = "1" /* PHLEBITIS AND THORBOPHLEBITIS, OTHER DEEP VESSEL OF LOWER EXTREMITIES */
"4512"  = "1" /* PHLEBITIS AND THORBOPHLEBITIS, LOWER EXTREMITIES */
"45181" = "1" /* ILIAC VEIN */
"4519"  = "1" /* UNSPECIFIED SITE */
"45340" = "1" /* DVT-EMBLSM LOWER EXT NOS */
"45341" = "1" /* DVT-EMB PROX LOWER EXT */
"45342" = "1" /* DVT-EMB DISTAL LOWER EXT */
"4538"  = "1" /* OTHER VENOUS EMBOLISM AND THROMBOSIS OF OTHER SPECIFIED VEINS */
"4539"  = "1" /* OTHER VENOUS EMBOLISM AND THROMBOSIS OF UNSPECIFIED SITE */
OTHER   = "0" ;

 /* EXCLUDE: ABORTION-RELATED AND POSTPARTUM OBSTETRIC PULMONARY EMBOLISM */
 /* ICD-9-CM DIAGNOSIS CODES: */

VALUE $OBEMBOL
"63460" = "1" /* SPON ABORT W EMBOL-UNSP */
"63461" = "1" /* SPON ABORT W EMBOL-INC */
"63462" = "1" /* SPON ABORT W EMBOL-COMP */
"63560" = "1" /* LEGAL ABORT W EMBOL-UNSP */
"63561" = "1" /* LEGAL ABORT W EMBOL-INC */
"63562" = "1" /* LEGAL ABORT W EMBOL-COMP */
"63660" = "1" /* ILLEG AB W EMBOLISM-UNSP */
"63661" = "1" /* ILLEG AB W EMBOLISM-INC */
"63662" = "1" /* ILLEG AB W EMBOLISM-COMP */
"63760" = "1" /* AB NOS W EMBOLISM-UNSP */
"63761" = "1" /* AB NOS W EMBOLISM-INC */
"63762" = "1" /* AB NOS W EMBOLISM-COMP */
"6386"  = "1" /* ATTEMP ABORT W EMBOLISM */
"6396"  = "1" /* POSTABORTION EMBOLISM */
"67320" = "1" /* OBSTETRICAL BLOOD-CLOT EMBOLISM */
"67321" = "1" /* OBSTETRICAL BLOOD-CLOT EMBOLISM */
"67322" = "1" /* OBSTETRICAL BLOOD-CLOT EMBOLISM */
"67323" = "1" /* OBSTETRICAL BLOOD-CLOT EMBOLISM */
"67324" = "1" /* OBSTETRICAL BLOOD-CLOT EMBOLISM */
OTHER = "0";

 /* FTR-PNEUMONIA */
 /* ICD-9-CM DIAGNOSIS CODES: */

VALUE $FTR3DX
"4820"  = "1" /* K. PNEUMONIAE PNEUMONIA */
"4821"  = "1" /* PSEUDOMONAL PNEUMONIA */
"4822"  = "1" /* H.INFLUENZAE PNEUMONIA */
"4823"  = "1" /* STREPTOCOCCAL PNEUMONIA */
"48230" = "1" /* STREPTOCOCCAL PNEUMN NOS */
"48231" = "1" /* PNEUMONIA STRPTOCOCCUS A */
"48232" = "1" /* PNEUMONIA STRPTOCOCCUS B */
"48239" = "1" /* PNEUMONIA OTH STREP */
"4824"  = "1" /* STAPHYLOCOCCAL PNEUMONIA */
"48240" = "1" /* STAPHYLOCOCCAL PNEU NOS */
"48241" = "1" /* STAPH AUREUS PNEUMONIA */
"48249" = "1" /* STAPH PNEUMONIA NEC */
"4828"  = "1" /* BACTERIAL PNEUMONIA NEC */
"48281" = "1" /* PNEUMONIA ANAEROBES */
"48282" = "1" /* PNEUMONIA E COLI */
"48283" = "1" /* PNEUMO OTH GRM-NEG BACT */
"48284" = "1" /* LEGIONNAIRES  DISEASE */
"48289" = "1" /* PNEUMONIA OTH SPCF BACT */
"4829"  = "1" /* BACTERIAL PNEUMONIA NOS */
"485"   = "1" /* BRONCHOPNEUMONIA, ORGANISM UNSPECIFIED */
"486"   = "1" /* PNEUMONIA, ORGANISM UNSPECIFIED */
"5070"  = "1" /* DUE TO INHALATION OF FOOD OR VOMITUS */
"514"   = "1" /* PULMONARY CONGESTION AND HYPOSTASIS */
OTHER   = "0" ;

 /* EXCLUDE */
 /* ICD-9-CM DIAGNOSIS CODES: */

VALUE $FTR3EXA
"9973"  = "1" /* RESPIRATORY COMPLICATIONS */
OTHER   = "0" ;

VALUE $FTR3EXB
"4800"  = "1" /* VIRAL PNEUMONIA */
"4801"  = "1" /* VIRAL PNEUMONIA */
"4802"  = "1" /* VIRAL PNEUMONIA */
"4803"  = "1" /* PNEUMONIA DUE TO SARS */
"4808"  = "1" /* VIRAL PNEUMONIA */
"4809"  = "1" /* VIRAL PNEUMONIA */
"481"   = "1" /* PNEUMOCOCCAL PNEUMONIA [STREPTOCOCCUS PNEUMONIAE PNEUMONIA] */
"483"   = "1" /* PNEUMONIA DUE TO OTHER SPECIFIED ORGANISM */
"4830"  = "1" /* PNEUMONIA DUE TO OTHER SPECIFIED ORGANISM */
"4831"  = "1" /* PNEUMONIA DUE TO OTHER SPECIFIED ORGANISM */
"4838"  = "1" /* PNEUMONIA DUE TO OTHER SPECIFIED ORGANISM */
"4841"  = "1" /* PNEUMONIA IN INFECTIOUS DISEASES CLASSIFIED ELSEWHERE */
"4843"  = "1" /* PNEUMONIA IN INFECTIOUS DISEASES CLASSIFIED ELSEWHERE */
"4845"  = "1" /* PNEUMONIA IN INFECTIOUS DISEASES CLASSIFIED ELSEWHERE */
"4846"  = "1" /* PNEUMONIA IN INFECTIOUS DISEASES CLASSIFIED ELSEWHERE */
"4847"  = "1" /* PNEUMONIA IN INFECTIOUS DISEASES CLASSIFIED ELSEWHERE */
"4848"  = "1" /* PNEUMONIA IN INFECTIOUS DISEASES CLASSIFIED ELSEWHERE */
"4870"  = "1" /* INFLUENZA */
"4871"  = "1" /* INFLUENZA */
"4878"  = "1" /* INFLUENZA */
"488 "  = "1" /* FLU DT AVIAN FLU VIRUS */
OTHER   = "0" ;

  /* FTR-SEPSIS */
  /* ICD-9-CM DIAGNOSIS CODES: */

VALUE $FTR4DX
"0380"  = "1" /* STREPTOCOCCAL SEPTICEMIA */
"0381"  = "1" /* STAPHYLOCOCCAL SEPTICEMIA */
"03810" = "1" /* STAPHYLOCOCCAL SEPTICEMIA, UNSPECIFIED */
"03811" = "1" /* STAPHYLOCOCCUS AUREUS SEPTICEMIA */
"03819" = "1" /* OTHER STAPHYLOCOCCAL SEPTICEMIA */
"0382"  = "1" /* PNEUMOCOCCAL SEPTICEMIA (STREPTOCOCCUS PNEUMONIAE SEPTICEMIA) */
"0383"  = "1" /* SEPTICEMIA DUE TO ANAEROBES */
"03840" = "1" /* GRAM-NEGATIVE ORGANISM, UNSPECIFIED */
"03841" = "1" /* HEMOPHILUS INFLUENZAE */
"03842" = "1" /* ESCHERICHIA COLI */
"03843" = "1" /* PSEUDOMONAS */
"03844" = "1" /* SERRATIA */
"03849" = "1" /* SEPTICEMIA DUE TO OTHER GRAM-NEGATIVE ORGANISMS */
"0388"  = "1" /* OTHER SPECIFIED SEPTICEMIAS */
"0389"  = "1" /* UNSPECIFIED SEPTICEMIA */
"78552" = "1" /* SEPTIC SHOCK */             
"78559" = "1" /* SHOCK W/O TRAUMA NEC */     
"99591" = "1" /* SYSTEMIC INFLAMMATORY RESPONSE SYNDROME DUE TO INFECTIOUS 
                 PROCESS WITHOUT ORGAN DYSFUNCTION */
"99592" = "1" /* SYSTEMIC INFLAMMATORY RESPONSE SYNDROME DUE TO INFECTIOUS
                 PROCESS WITH ORGAN DYSFUNCTION */
"9980"  = "1" /* POSTOPERATIVE SHOCK */  
OTHER   = "0" ;

 /* EXCLUDE: INFECTION */
 /* ICD-9-CM DIAGNOSIS CODES: */

VALUE $INFECID
"0010" = "1" /* CHOLERA D/T VIB CHOLERAE */
"0011" = "1" /* CHOLERA D/T VIB EL TOR */
"0019" = "1" /* CHOLERA NOS */
"0020" = "1" /* TYPHOID FEVER */
"0021" = "1" /* PARATYPHOID FEVER A */
"0022" = "1" /* PARATYPHOID FEVER B */
"0023" = "1" /* PARATYPHOID FEVER C */
"0029" = "1" /* PARATYPHOID FEVER NOS */
"0030" = "1" /* SALMONELLA ENTERITIS */
"0031" = "1" /* SALMONELLA SEPTICEMIA */
"00320" = "1" /* LOCAL SALMONELLA INF NOS */
"00321" = "1" /* SALMONELLA MENINGITIS */
"00322" = "1" /* SALMONELLA PNEUMONIA */
"00323" = "1" /* SALMONELLA ARTHRITIS */
"00324" = "1" /* SALMONELLA OSTEOMYELITIS */
"00329" = "1" /* LOCAL SALMONELLA INF NEC */
"0038" = "1" /* SALMONELLA INFECTION NEC */
"0039" = "1" /* SALMONELLA INFECTION NOS */
"0040" = "1" /* SHIGELLA DYSENTERIAE */
"0041" = "1" /* SHIGELLA FLEXNERI */
"0042" = "1" /* SHIGELLA BOYDII */
"0043" = "1" /* SHIGELLA SONNEI */
"0048" = "1" /* SHIGELLA INFECTION NEC */
"0049" = "1" /* SHIGELLOSIS NOS */
"0050" = "1" /* STAPH FOOD POISONING */
"0051" = "1" /* BOTULISM */
"0052" = "1" /* FOOD POIS D/T C. PERFRIN */
"0053" = "1" /* FOOD POIS: CLOSTRID NEC */
"0054" = "1" /* FOOD POIS: V. PARAHAEM */
"00581" = "1" /* FOOD POISN D/T V. VULNIF */
"00589" = "1" /* BACT FOOD POISONING NEC */
"0059" = "1" /* FOOD POISONING NOS */
"00800" = "1" /* INTEST INFEC E COLI NOS */
"00801" = "1" /* INT INF E COLI ENTRPATH */
"00802" = "1" /* INT INF E COLI ENTRTOXGN */
"00803" = "1" /* INT INF E COLI ENTRNVSV */
"00804" = "1" /* INT INF E COLI ENTRHMRG */
"00809" = "1" /* INT INF E COLI SPCF NEC */
"0081" = "1" /* ARIZONA ENTERITIS */
"0082" = "1" /* AEROBACTER ENTERITIS */
"0083" = "1" /* PROTEUS ENTERITIS */
"00841" = "1" /* STAPHYLOCOCC ENTERITIS */
"00842" = "1" /* PSEUDOMONAS ENTERITIS */
"00843" = "1" /* INT INFEC CAMPYLOBACTER */
"00844" = "1" /* INT INF YRSNIA ENTRCLTCA */
"00845" = "1" /* INT INF CLSTRDIUM DFCILE */
"00846" = "1" /* INTES INFEC OTH ANEROBES */
"00847" = "1" /* INT INF OTH GRM NEG BCTR */
"00849" = "1" /* BACTERIAL ENTERITIS NEC */
"0085" = "1" /* BACTERIAL ENTERITIS NOS */
"0200" = "1" /* BUBONIC PLAGUE */
"0201" = "1" /* CELLULOCUTANEOUS PLAGUE */
"0202" = "1" /* SEPTICEMIC PLAGUE */
"0203" = "1" /* PRIMARY PNEUMONIC PLAGUE */
"0204" = "1" /* SECONDARY PNEUMON PLAGUE */
"0205" = "1" /* PNEUMONIC PLAGUE NOS */
"0208" = "1" /* OTHER TYPES OF PLAGUE */
"0209" = "1" /* PLAGUE NOS */
"0210" = "1" /* ULCEROGLANDUL TULAREMIA */
"0211" = "1" /* ENTERIC TULAREMIA */
"0212" = "1" /* PULMONARY TULAREMIA */
"0213" = "1" /* OCULOGLANDULAR TULAREMIA */
"0218" = "1" /* TULAREMIA NEC */
"0219" = "1" /* TULAREMIA NOS */
"0220" = "1" /* CUTANEOUS ANTHRAX */
"0221" = "1" /* PULMONARY ANTHRAX */
"0222" = "1" /* GASTROINTESTINAL ANTHRAX */
"0223" = "1" /* ANTHRAX SEPTICEMIA */
"0228" = "1" /* OTHER ANTHRAX MANIFEST */
"0229" = "1" /* ANTHRAX NOS */
"0230" = "1" /* BRUCELLA MELITENSIS */
"0231" = "1" /* BRUCELLA ABORTUS */
"0232" = "1" /* BRUCELLA SUIS */
"0233" = "1" /* BRUCELLA CANIS */
"0238" = "1" /* BRUCELLOSIS NEC */
"0239" = "1" /* BRUCELLOSIS NOS */
"024" = "1" /* GLANDERS */
"025" = "1" /* MELIOIDOSIS */
"0260" = "1" /* SPIRILLARY FEVER */
"0261" = "1" /* STREPTOBACILLARY FEVER */
"0269" = "1" /* RAT-BITE FEVER NOS */
"0270" = "1" /* LISTERIOSIS */
"0271" = "1" /* ERYSIPELOTHRIX INFECTION */
"0272" = "1" /* PASTEURELLOSIS */
"0278" = "1" /* ZOONOTIC BACT DIS NEC */
"0279" = "1" /* ZOONOTIC BACT DIS NOS */
"0320" = "1" /* FAUCIAL DIPHTHERIA */
"0321" = "1" /* NASOPHARYNX DIPHTHERIA */
"0322" = "1" /* ANT NASAL DIPHTHERIA */
"0323" = "1" /* LARYNGEAL DIPHTHERIA */
"03281" = "1" /* CONJUNCTIVAL DIPHTHERIA */
"03282" = "1" /* DIPHTHERITIC MYOCARDITIS */
"03283" = "1" /* DIPHTHERITIC PERITONITIS */
"03284" = "1" /* DIPHTHERITIC CYSTITIS */
"03285" = "1" /* CUTANEOUS DIPHTHERIA */
"03289" = "1" /* DIPHTHERIA NEC */
"0329" = "1" /* DIPHTHERIA NOS */
"0330" = "1" /* BORDETELLA PERTUSSIS */
"0331" = "1" /* BORDETELLA PARAPERTUSSIS */
"0338" = "1" /* WHOOPING COUGH NEC */
"0339" = "1" /* WHOOPING COUGH NOS */
"0340" = "1" /* STREP SORE THROAT */
"0341" = "1" /* SCARLET FEVER */
"035" = "1" /* ERYSIPELAS */
"0360" = "1" /* MENINGOCOCCAL MENINGITIS */
"0361" = "1" /* MENINGOCOCC ENCEPHALITIS */
"0362" = "1" /* MENINGOCOCCEMIA */
"0363" = "1" /* MENINGOCOCC ADRENAL SYND */
"03640" = "1" /* MENINGOCOCC CARDITIS NOS */
"03641" = "1" /* MENINGOCOCC PERICARDITIS */
"03642" = "1" /* MENINGOCOCC ENDOCARDITIS */
"03643" = "1" /* MENINGOCOCC MYOCARDITIS */
"03681" = "1" /* MENINGOCOCC OPTIC NEURIT */
"03682" = "1" /* MENINGOCOCC ARTHROPATHY */
"03689" = "1" /* MENINGOCOCCAL INFECT NEC */
"0369" = "1" /* MENINGOCOCCAL INFECT NOS */
"037" = "1" /* TETANUS */
"0380" = "1" /* STREPTOCOCCAL SEPTICEMIA */
"03810" = "1" /* STAPHYLCOCC SEPTICEM NOS */
"03811" = "1" /* STAPH AUREUS SEPTICEMIA */
"03819" = "1" /* STAPHYLCOCC SEPTICEM NEC */
"0382" = "1" /* PNEUMOCOCCAL SEPTICEMIA */
"0383" = "1" /* ANAEROBIC SEPTICEMIA */
"03840" = "1" /* GRAM-NEG SEPTICEMIA NOS */
"03841" = "1" /* H. INFLUENAE SEPTICEMIA */
"03842" = "1" /* E COLI SEPTICEMIA */
"03843" = "1" /* PSEUDOMONAS SEPTICEMIA */
"03844" = "1" /* SERRATIA SEPTICEMIA */
"03849" = "1" /* GRAM-NEG SEPTICEMIA NEC */
"0388" = "1" /* SEPTICEMIA NEC */
"0389" = "1" /* SEPTICEMIA NOS */
"0390" = "1" /* CUTANEOUS ACTINOMYCOSIS */
"0391" = "1" /* PULMONARY ACTINOMYCOSIS */
"0392" = "1" /* ABDOMINAL ACTINOMYCOSIS */
"0393" = "1" /* CERVICOFAC ACTINOMYCOSIS */
"0394" = "1" /* MADURA FOOT */
"0398" = "1" /* ACTINOMYCOSIS NEC */
"0399" = "1" /* ACTINOMYCOSIS NOS */
"0400" = "1" /* GAS GANGRENE */
"0401" = "1" /* RHINOSCLEROMA */
"0402" = "1" /* WHIPPLE"S DISEASE */
"0403" = "1" /* NECROBACILLOSIS */
"04041" = "1" /* INFANT BOTULISM */
"04042" = "1" /* WOUND BOTULISM */
"04081" = "1" /* TROPICAL PYOMYOSITIS */
"04082" = "1" /* TOXIC SHOCK SYNDROME */
"04089" = "1" /* BACTERIAL DISEASES NEC */
"04100" = "1" /* STREPTOCOCCUS UNSPECF */
"04101" = "1" /* STREPTOCOCCUS GROUP A */
"04102" = "1" /* STREPTOCOCCUS GROUP B */
"04103" = "1" /* STREPTOCOCCUS GROUP C */
"04104" = "1" /* ENTEROCOCCUS GROUP D */
"04105" = "1" /* STREPTOCOCCUS GROUP G */
"04109" = "1" /* OTHER STREPTOCOCCUS */
"04110" = "1" /* STAPHYLOCOCCUS UNSPCFIED */
"04111" = "1" /* STAPHYLOCOCCUS AUREUS */
"04119" = "1" /* OTHER STAPHYLOCOCCUS */
"0412" = "1" /* PNEUMOCOCCUS INFECT NOS */
"0413" = "1" /* KLEBSIELLA INFECT NOS */
"0414" = "1" /* E. COLI INFECT NOS */
"0415" = "1" /* H. INFLUENZAE INFECT NOS */
"0416" = "1" /* PROTEUS INFECTION NOS */
"0417" = "1" /* PSEUDOMONAS INFECT NOS */
"04182" = "1" /* BACTEROIDES FRAGILIS */
"04183" = "1" /* CLOSTRIDIUM PERFRINGENS */
"04184" = "1" /* OTHER ANAEROBES */
"04185" = "1" /* OTH GRAM NEGATV BACTERIA */
"04186" = "1" /* HELICOBACTER PYLORI */
"04189" = "1" /* OTH SPECF BACTERIA */
"0419" = "1" /* BACTERIAL INFECTION NOS */
"0980" = "1" /* ACUTE GC INFECT LOWER GU */
"09810" = "1" /* GC (ACUTE) UPPER GU NOS */
"09811" = "1" /* GC CYSTITIS (ACUTE) */
"09812" = "1" /* GC PROSTATITIS (ACUTE) */
"09813" = "1" /* GC ORCHITIS (ACUTE) */
"09814" = "1" /* GC SEM VESICULIT (ACUTE) */
"09815" = "1" /* GC CERVICITIS (ACUTE) */
"09816" = "1" /* GC ENDOMETRITIS (ACUTE) */
"09817" = "1" /* ACUTE GC SALPINGITIS */
"09819" = "1" /* GC (ACUTE) UPPER GU NEC */
"0982" = "1" /* CHR GC INFECT LOWER GU */
"09830" = "1" /* CHR GC UPPER GU NOS */
"09831" = "1" /* GC CYSTITIS, CHRONIC */
"09832" = "1" /* GC PROSTATITIS, CHRONIC */
"09833" = "1" /* GC ORCHITIS, CHRONIC */
"09834" = "1" /* GC SEM VESICULITIS, CHR */
"09835" = "1" /* GC CERVICITIS, CHRONIC */
"09836" = "1" /* GC ENDOMETRITIS, CHRONIC */
"09837" = "1" /* GC SALPINGITIS (CHRONIC) */
"09839" = "1" /* CHR GC UPPER GU NEC */
"09840" = "1" /* GONOCOCCAL CONJUNCTIVIT */
"09841" = "1" /* GONOCOCCAL IRIDOCYCLITIS */
"09842" = "1" /* GONOCOCCAL ENDOPHTHALMIA */
"09843" = "1" /* GONOCOCCAL KERATITIS */
"09849" = "1" /* GONOCOCCAL EYE NEC */
"09850" = "1" /* GONOCOCCAL ARTHRITIS */
"09851" = "1" /* GONOCOCCAL SYNOVITIS */
"09852" = "1" /* GONOCOCCAL BURSITIS */
"09853" = "1" /* GONOCOCCAL SPONDYLITIS */
"09859" = "1" /* GC INFECT JOINT NEC */
"0986" = "1" /* GONOCOCCAL INFEC PHARYNX */
"0987" = "1" /* GC INFECT ANUS & RECTUM */
"09881" = "1" /* GONOCOCCAL KERATOSIS */
"09882" = "1" /* GONOCOCCAL MENINGITIS */
"09883" = "1" /* GONOCOCCAL PERICARDITIS */
"09884" = "1" /* GONOCOCCAL ENDOCARDITIS */
"09885" = "1" /* GONOCOCCAL HEART DIS NEC */
"09886" = "1" /* GONOCOCCAL PERITONITIS */
"09889" = "1" /* GONOCOCCAL INF SITE NEC */
"3200" = "1" /* HEMOPHILUS MENINGITIS */
"3201" = "1" /* PNEUMOCOCCAL MENINGITIS */
"3202" = "1" /* STREPTOCOCCAL MENINGITIS */
"3203" = "1" /* STAPHYLOCOCC MENINGITIS */
"3207" = "1" /* MENING IN OTH BACT DIS */
"32081" = "1" /* ANAEROBIC MENINGITIS */
"32082" = "1" /* MNINGTS GRAM-NEG BCT NEC */
"32089" = "1" /* MENINGITIS OTH SPCF BACT */
"3209" = "1" /* BACTERIAL MENINGITIS NOS */
"3229" = "1" /* MENINGITIS NOS            */
"3240" = "1" /* INTRACRANIAL ABSCESS */
"3241" = "1" /* INTRASPINAL ABSCESS */
"3249" = "1" /* CNS ABSCESS NOS */
"36000" = "1" /* PURULENT ENDOPHTHALM NOS */
"36001" = "1" /* ACUTE ENDOPHTHALMITIS */
"36002" = "1" /* PANOPHTHALMITIS */
"36004" = "1" /* VITREOUS ABSCESS */
"37055" = "1" /* CORNEAL ABSCESS */
"37200" = "1" /* ACUTE CONJUNCTIVITIS NOS */
"37203" = "1" /* MUCOPUR CONJUNCTIVIT NEC */
"37204" = "1" /* PSEUDOMEMB CONJUNCTIVIT */
"37220" = "1" /* BLEPHAROCONJUNCTIVIT NOS */
"37221" = "1" /* ANGULAR BLEPHAROCONJUNCT */
"37230" = "1" /* CONJUNCTIVITIS NOS */
"37300" = "1" /* BLEPHARITIS NOS */
"37301" = "1" /* ULCERATIVE BLEPHARITIS */
"37311" = "1" /* HORDEOLUM EXTERNUM */
"37312" = "1" /* HORDEOLUM INTERNUM */
"37313" = "1" /* ABSCESS OF EYELID */
"37500" = "1" /* DACRYOADENITIS NOS */
"37501" = "1" /* ACUTE DACRYOADENITIS */
"37530" = "1" /* DACRYOCYSTITIS NOS */
"37531" = "1" /* ACUTE CANALICULITIS */
"37532" = "1" /* ACUTE DACRYOCYSTITIS */
"37600" = "1" /* ACUTE INFLAM NOS, ORBIT */
"37601" = "1" /* ORBITAL CELLULITIS */
"37602" = "1" /* ORBITAL PERIOSTITIS */
"37603" = "1" /* ORBITAL OSTEOMYELITIS */
"37604" = "1" /* TENONITIS */
"38010" = "1" /* INFEC OTITIS EXTERNA NOS */
"38011" = "1" /* ACUTE INFECTION OF PINNA */
"38012" = "1" /* ACUTE SWIMMERS" EAR */
"38013" = "1" /* AC INFECT EXTERN EAR NEC */
"38014" = "1" /* MALIGNANT OTITIS EXTERNA */
"38150" = "1" /* EUSTACHIAN SALPING NOS */
"38151" = "1" /* AC EUSTACHIAN SALPING */
"38200" = "1" /* AC SUPP OTITIS MEDIA NOS */
"38201" = "1" /* AC SUPP OM W DRUM RUPT */
"38202" = "1" /* AC SUPP OM IN OTH DIS */
"3821" = "1" /* CHR TUBOTYMP SUPP OTITIS MEDIA */
"3822" = "1" /* CHR ATTICOANTRAL SUPP OTITIS MEDIA */
"3823" = "1" /* CHR SUPP OTITIS MEDIA NOS */
"3824" = "1" /* SUPPUR OTITIS MEDIA NOS */
"3829" = "1" /* OTITIS MEDIA NOS */
"38300" = "1" /* AC MASTOIDITIS W/O COMPL */
"38301" = "1" /* SUBPERI MASTOID ABSCESS */
"38302" = "1" /* AC MASTOIDITIS-COMPL NEC */
"38320" = "1" /* PETROSITIS NOS */
"38321" = "1" /* ACUTE PETROSITIS */
"38400" = "1" /* ACUTE MYRINGITIS NOS */
"38630" = "1" /* LABYRINTHITIS NOS */
"38631" = "1" /* SEROUS LABYRINTHITIS */
"38632" = "1" /* CIRCUMSCRI LABYRINTHITIS */
"38633" = "1" /* SUPPURATIV LABYRINTHITIS */
"4200" = "1" /* AC PERICARDIT IN OTH DIS */
"42090" = "1" /* ACUTE PERICARDITIS NOS */
"42099" = "1" /* ACUTE PERICARDITIS NEC */
"4210" = "1" /* AC/SUBAC BACT ENDOCARD */
"4211" = "1" /* AC/SUBAC INFECT ENDOCARD */
"4219" = "1" /* AC/SUBAC ENDOCARDIT NOS */
"42292" = "1" /* SEPTIC MYOCARDITIS */
"4610" = "1" /* AC MAXILLARY SINUSITIS */
"4611" = "1" /* AC FRONTAL SINUSITIS */
"4612" = "1" /* AC ETHMOIDAL SINUSITIS */
"4613" = "1" /* AC SPHENOIDAL SINUSITIS */
"4618" = "1" /* OTHER ACUTE SINUSITIS */
"4619" = "1" /* ACUTE SINUSITIS NOS */
"462" = "1" /* ACUTE PHARYNGITIS */
"463" = "1" /* ACUTE TONSILLITIS */
"46430" = "1" /* AC EPIGLOTTITIS NO OBSTR */
"46431" = "1" /* AC EPIGLOTTITIS W OBSTR */
"4660" = "1" /* ACUTE BRONCHITIS */
"475" = "1" /* PERITONSILLAR ABSCESS */
"47822" = "1" /* PARAPHARYNGEAL ABSCESS */
"47824" = "1" /* RETROPHARYNGEAL ABSCESS */
"481" = "1" /* PNEUMOCOCCAL PNEUMONIA */
"4820" = "1" /* K. PNEUMONIAE PNEUMONIA */
"4821" = "1" /* PSEUDOMONAL PNEUMONIA */
"4822" = "1" /* H.INFLUENZAE PNEUMONIA */
"48230" = "1" /* STREPTOCOCCAL PNEUMN NOS */
"48231" = "1" /* PNEUMONIA STRPTOCOCCUS A */
"48232" = "1" /* PNEUMONIA STRPTOCOCCUS B */
"48239" = "1" /* PNEUMONIA OTH STREP */
"48240" = "1" /* STAPHYLOCOCCAL PNEU NOS */
"48241" = "1" /* STAPH AUREUS PNEUMONIA */
"48249" = "1" /* STAPH PNEUMONIA NEC */
"48281" = "1" /* PNEUMONIA ANAEROBES */
"48282" = "1" /* PNEUMONIA E COLI */
"48283" = "1" /* PNEUMO OTH GRM-NEG BACT */
"48284" = "1" /* LEGIONNAIRES" DISEASE */
"48289" = "1" /* PNEUMONIA OTH SPCF BACT */
"4829" = "1" /* BACTERIAL PNEUMONIA NOS */
"4843" = "1" /* PNEUMONIA IN WHOOPING COUGH */
"4845" = "1" /* PNEUMONIA IN ANTHRAX */
"4848" = "1" /* PNEUMONIA IN OTHER INF DIS  */
"485" = "1" /* BRONCHOPNEUMONIA ORG NOS */
"486" = "1" /* PNEUMONIA, ORGANISM NOS */
"490" = "1" /* BRONCHITIS NOS */
"49122" = "1" /* OBS CHR BRONC W AC BRONC */
"4941" = "1" /* BRONCHIECTASIS W AC EXAC */
"5100" = "1" /* EMPYEMA WITH FISTULA */
"5109" = "1" /* EMPYEMA W/O FISTULA */
"5111" = "1" /* BACT PLEUR/EFFUS NOT TB */
"5130" = "1" /* ABSCESS OF LUNG */
"5131" = "1" /* ABSCESS OF MEDIASTINUM */
"51901" = "1" /* TRACHEOSTOMY INFECTION */
"5192" = "1" /* MEDIASTINITIS */
"5220" = "1" /* PULPITIS */
"5225" = "1" /* PERIAPICAL ABSCESS */
"5227" = "1" /* PERIAPICAL ABSC W SINUS */
"5230" = "1" /* ACUTE GINGIVITIS */
"52300" = "1" /* ACUTE GINGITITIS, PLAQUE */ 
"52301" = "1" /* AC GINGIVITIS,NONPLAQUE */  
"5233" = "1" /* ACUTE PERIODONTITIS       */
"52330" = "1" /* AGGRES PERIODONTITIS NOS */ 
"52331" = "1" /* AGGRES PERIODONTITIS,LOC */ 
"52332" = "1" /* AGGRES PERIODONTITIS,GEN */ 
"52333" = "1" /* ACUTE PERIODONTITIS */  
"5264" = "1" /* INFLAMMATION OF JAW */
"5273" = "1" /* SALIVARY GLAND ABSCESS */
"5283" = "1" /* CELLULITIS/ABSCESS MOUTH */
"53641" = "1" /* GASTROSTOMY INFECTION     */
"5400" = "1" /* AC APPEND W PERITONITIS */
"5401" = "1" /* ABSCESS OF APPENDIX */
"5409" = "1" /* ACUTE APPENDICITIS NOS */
"541" = "1" /* APPENDICITIS NOS */
"542" = "1" /* OTHER APPENDICITIS */
"56201" = "1" /* DVRTCLI SML INT W/O HMRG */
"56203" = "1" /* DVRTCLI SML INT W HMRHG */
"56211" = "1" /* DVRTCLI COLON W/O HMRHG */
"56213" = "1" /* DVRTCLI COLON W HMRHG */
"566" = "1" /* ANAL & RECTAL ABSCESS */
"5670" = "1" /* PERITONITIS IN INFEC DIS */
"5671" = "1" /* PNEUMOCOCCAL PERITONITIS */
"5672" = "1" /* SUPPURAT PERITONITIS NEC */
"56721" = "1" /* PERITONITIS (ACUTE) GEN  */
"56722" = "1" /* PERITONEAL ABSCESS  */
"56723" = "1" /* SPONTAN BACT PERITONITIS  */
"56729" = "1" /* SUPPURAT PERITONITIS NEC  */
"56731" = "1" /* PSOAS MUSCLE ABSCESS  */
"56738" = "1" /* RETROPERITON ABSCESS NEC  */
"56739" = "1" /* RETROPERITON INFECT NEC  */
"56781" = "1" /* CHOLEPERITONITIS  */
"56782" = "1" /* SCLEROSING MESENTERITIS  */
"56789" = "1" /* PERITONITIS NEC  */
"5679" = "1" /* PERITONITIS NOS */
"5695" = "1" /* INTESTINAL ABSCESS */
"56961" = "1" /* COLOSTY/ENTEROST INFECTN */
"5720" = "1" /* ABSCESS OF LIVER */
"5721" = "1" /* PORTAL PYEMIA */
"57400" = "1" /* CHOLELITH W AC CHOLECYST */
"57401" = "1" /* CHOLELITH/AC GB INF-OBST */
"57430" = "1" /* CHOLEDOCHOLITH/AC GB INF */
"57431" = "1" /* CHOLEDOCHLITH/AC GB-OBST */
"57460" = "1" /* GALL&BIL CAL W/AC W/O OB */
"57461" = "1" /* GALL&BIL CAL W/AC W OBS */
"57480" = "1" /* GAL&BIL CAL W/AC&CHR W/O */
"57481" = "1" /* GAL&BIL CAL W/AC&CH W OB */
"5750" = "1" /* ACUTE CHOLECYSTITIS */
"57510" = "1" /* CHOLECYSTITIS UNSPEC */
"57512" = "1" /* AC&CHRON CHOLECYSTITIS */
"5754" = "1" /* PERFORATION GALLBLADDER */
"5761" = "1" /* CHOLANGITIS */
"5763" = "1" /* PERFORATION OF BILE DUCT */
"5770" = "1" /* ACUTE PANCREATITIS */
"59010" = "1" /* AC PYELONEPHRITIS NOS */
"59011" = "1" /* AC PYELONEPHR W MED NECR */
"5902" = "1" /* RENAL/PERIRENAL ABSCESS */
"5903" = "1" /* PYELOURETERITIS CYSTICA */
"59080" = "1" /* PYELONEPHRITIS NOS        */
"59081" = "1" /* PYELONEPHRIT IN OTH DIS */
"5909" = "1" /* INFECTION OF KIDNEY NOS */
"5950" = "1" /* ACUTE CYSTITIS */
"5954" = "1" /* CYSTITIS IN OTH DIS */
"59581" = "1" /* CYSTITIS CYSTICA */
"59589" = "1" /* CYSTITIS NEC */
"5959" = "1" /* CYSTITIS NOS */
"5970" = "1" /* URETHRAL ABSCESS */
"59800" = "1" /* URETHR STRICT:INFECT NOS */
"59801" = "1" /* URETH STRICT:OTH INFECT */
"5990" = "1" /* URIN TRACT INFECTION NOS */
"6010" = "1" /* ACUTE PROSTATITIS */
"6012" = "1" /* ABSCESS OF PROSTATE */
"6013" = "1" /* PROSTATOCYSTITIS */
"6014" = "1" /* PROSTATITIS IN OTH DIS */
"6018" = "1" /* PROSTATITIS  */
"6019" = "1" /* PROSTATITIS NOS */
"6031" = "1" /* INFECTED HYDROCELE */
"6040" = "1" /* ORCHITIS WITH ABSCESS */
"60490" = "1" /* ORCHITIS/EPIDIDYMIT NOS */
"60491" = "1" /* ORCHITIS IN OTH DISEASE */
"6071" = "1" /* BALANOPOSTHITIS */
"6072" = "1" /* INFLAM DIS, PENIS NEC */
"6080" = "1" /* SEMINAL VESICULITIS */
"6084" = "1" /* MALE GEN INFLAM DIS NEC */
"6110" = "1" /* INFLAM DISEASE OF BREAST */
"6140" = "1" /* AC SALPINGO-OOPHORITIS */
"6141" = "1" /* CHRON SALPINGITIS OOPHORITIS */
"6142" = "1" /* SALPINGO-OOPHORITIS NOS */
"6143" = "1" /* ACUTE PARAMETRITIS */
"6144" = "1" /* CHRON OR UNSP CELLULITIS */
"6145" = "1" /* AC PELV PERITONITIS-FEM */
"6149" = "1" /* PID NOS */
"6150" = "1" /* AC UTERINE INFLAMMATION */
"6159" = "1" /* UTERINE INFLAM DIS NOS */
"6160" = "1" /* CERVICITIS */
"61610" = "1" /* VAGINITIS NOS */
"6163" = "1" /* BARTHOLIN"S GLND ABSCESS */
"6164" = "1" /* ABSCESS OF VULVA NEC */
"63400" = "1" /* SPON ABOR W PEL INF-UNSP */
"63401" = "1" /* SPON ABOR W PELV INF-INC */
"63402" = "1" /* SPON ABOR W PEL INF-COMP */
"63500" = "1" /* LEG ABOR W PELV INF-UNSP */
"63501" = "1" /* LEG ABOR W PELV INF-INC */
"63502" = "1" /* LEG ABOR W PELV INF-COMP */
"63600" = "1" /* ILLEG AB W PELV INF-UNSP */
"63601" = "1" /* ILLEG AB W PELV INF-INC */
"63602" = "1" /* ILLEG AB W PELV INF-COMP */
"63700" = "1" /* ABORT NOS W PEL INF-UNSP */
"63701" = "1" /* ABORT NOS W PEL INF-INC */
"63702" = "1" /* ABORT NOS W PEL INF-COMP */
"6380" = "1" /* ATTEM ABORT W PELVIC INF */
"6390" = "1" /* POSTABORTION GU INFECT */
"64650" = "1" /* BACTERIURIA PREG-UNSPEC */
"64651" = "1" /* ASYM BACTERIURIA-DELIVER */
"64652" = "1" /* ASY BACTERURIA-DEL W P/P */
"64653" = "1" /* ASY BACTERIURIA-ANTEPART */
"64654" = "1" /* ASY BACTERIURIA-POSTPART */
"64660" = "1" /* GU INFECT IN PREG-UNSPEC */
"64661" = "1" /* GU INFECTION-DELIVERED */
"64662" = "1" /* GU INFECTION-DELIV W P/P */
"64663" = "1" /* GU INFECTION-ANTEPARTUM */
"64664" = "1" /* GU INFECTION-POSTPARTUM */
"64710" = "1" /* GONORRHEA IN PREG-UNSPEC */
"64711" = "1" /* GONORRHEA-DELIVERED */
"64712" = "1" /* GONORRHEA-DELIVER W P/P */
"64713" = "1" /* GONORRHEA-ANTEPARTUM */
"64714" = "1" /* GONORRHEA-POSTPARTUM */
"64780" = "1" /* INF DIS IN PREG NEC-UNSP */
"64781" = "1" /* INFECT DIS NEC-DELIVERED */
"64782" = "1" /* INFECT DIS NEC-DEL W P/P */
"64783" = "1" /* INFECT DIS NEC-ANTEPART */
"64784" = "1" /* INFECT DIS NEC-POSTPART */
"64790" = "1" /* INFECT IN PREG NOS-UNSP */
"64791" = "1" /* INFECT NOS-DELIVERED */
"64792" = "1" /* INFECT NOS-DELIVER W P/P */
"64793" = "1" /* INFECT NOS-ANTEPARTUM */
"64794" = "1" /* INFECT NOS-POSTPARTUM */
"65840" = "1" /* AMNIOTIC INFECTION-UNSP */
"65841" = "1" /* AMNIOTIC INFECTION-DELIV */
"65843" = "1" /* AMNIOTIC INFECT-ANTEPART */
"67000" = "1" /* MAJOR PUERP INFECT-UNSP */
"67002" = "1" /* MAJOR PUERP INF-DEL P/P */
"67004" = "1" /* MAJOR PUERP INF-POSTPART */
"67500" = "1" /* INFECT NIPPLE PREG-UNSP */
"67501" = "1" /* INFECT NIPPLE-DELIVERED */
"67502" = "1" /* INFECT NIPPLE-DEL W P/P */
"67503" = "1" /* INFECT NIPPLE-ANTEPARTUM */
"67504" = "1" /* INFECT NIPPLE-POSTPARTUM */
"67510" = "1" /* BREAST ABSCESS PREG-UNSP */
"67511" = "1" /* BREAST ABSCESS-DELIVERED */
"67512" = "1" /* BREAST ABSCESS-DEL W P/P */
"67513" = "1" /* BREAST ABSCESS-ANTEPART */
"67514" = "1" /* BREAST ABSCESS-POSTPART */
"67580" = "1" /* BREAST INF PREG NEC-UNSP */
"67581" = "1" /* BREAST INFECT NEC-DELIV */
"67582" = "1" /* BREAST INF NEC-DEL W P/P */
"67583" = "1" /* BREAST INF NEC-ANTEPART */
"67584" = "1" /* BREAST INF NEC-POSTPART */
"67590" = "1" /* BREAST INF PREG NOS-UNSP */
"67591" = "1" /* BREAST INFECT NOS-DELIV */
"67592" = "1" /* BREAST INF NOS-DEL W P/P */
"67593" = "1" /* BREAST INF NOS-ANTEPART */
"67594" = "1" /* BREAST INF NOS-POSTPART */
"6800" = "1" /* CARBUNCLE OF FACE */
"6801" = "1" /* CARBUNCLE OF NECK */
"6802" = "1" /* CARBUNCLE OF TRUNK */
"6803" = "1" /* CARBUNCLE OF ARM */
"6804" = "1" /* CARBUNCLE OF HAND */
"6805" = "1" /* CARBUNCLE OF BUTTOCK */
"6806" = "1" /* CARBUNCLE OF LEG */
"6807" = "1" /* CARBUNCLE OF FOOT */
"6808" = "1" /* CARBUNCLE, SITE NEC */
"6809" = "1" /* CARBUNCLE NOS */
"68100" = "1" /* CELLULITIS, FINGER NOS */
"68101" = "1" /* FELON */
"68102" = "1" /* ONYCHIA OF FINGER */
"68110" = "1" /* CELLULITIS, TOE NOS */
"68111" = "1" /* ONYCHIA OF TOE */
"6819" = "1" /* CELLULITIS OF DIGIT NOS */
"6820" = "1" /* CELLULITIS OF FACE */
"6821" = "1" /* CELLULITIS OF NECK */
"6822" = "1" /* CELLULITIS OF TRUNK */
"6823" = "1" /* CELLULITIS OF ARM */
"6824" = "1" /* CELLULITIS OF HAND */
"6825" = "1" /* CELLULITIS OF BUTTOCK */
"6826" = "1" /* CELLULITIS OF LEG */
"6827" = "1" /* CELLULITIS OF FOOT */
"6828" = "1" /* CELLULITIS, SITE NEC */
"6829" = "1" /* CRLLULITIS, SITE NOS */
"683" = "1" /* ACUTE LYMPHADENITIS */
"684" = "1" /* IMPETIGO */
"68600" = "1" /* PYODERMA NOS */
"68601" = "1" /* PYODERMA GANGRENOSUM */
"68609" = "1" /* PYODERMA OTHER */
"6868" = "1" /* LOCAL SKIN INFECTION NEC */
"6869" = "1" /* LOCAL SKIN INFECTION NOS */
"69581" = "1" /* RITTER"S DISEASE */
"70700" = "1" /* DECUBITUS ULCER SITE NOS */
"70701" = "1" /* DECUBITUS ULCER,ELBOW */
"70702" = "1" /* DECUBITUS ULCER,UP BACK */
"70703" = "1" /* DECUBITUS ULCER,LOW BACK */
"70704" = "1" /* DECUBITUS ULCER,HIP */
"70705" = "1" /* DECUBITUS ULCER,BUTTOCK */
"70706" = "1" /* DECUBITUS ULCER,ANKLE */
"70707" = "1" /* DECUBITUS ULCER,HEEL */
"70709" = "1" /* DECUBITUS ULCER,SITE NEC */
"71100" = "1" /* PYOGEN ARTHRITIS-UNSPEC */
"71101" = "1" /* PYOGEN ARTHRITIS-SHLDER */
"71102" = "1" /* PYOGEN ARTHRITIS-UP/ARM */
"71103" = "1" /* PYOGEN ARTHRITIS-FOREARM */
"71104" = "1" /* PYOGEN ARTHRITIS-HAND */
"71105" = "1" /* PYOGEN ARTHRITIS-PELVIS */
"71106" = "1" /* PYOGEN ARTHRITIS-L/LEG */
"71107" = "1" /* PYOGEN ARTHRITIS-ANKLE */
"71108" = "1" /* PYOGEN ARTHRITIS NEC */
"71109" = "1" /* PYOGEN ARTHRITIS-MULT */
"71190" = "1" /* INF ARTHRITIS NOS-UNSPEC */
"71191" = "1" /* INF ARTHRITIS NOS-SHLDER */
"71192" = "1" /* INF ARTHRITIS NOS-UP/ARM */
"71193" = "1" /* INF ARTHRIT NOS-FOREARM */
"71194" = "1" /* INF ARTHRIT NOS-HAND */
"71195" = "1" /* INF ARTHRIT NOS-PELVIS */
"71196" = "1" /* INF ARTHRIT NOS-L/LEG */
"71197" = "1" /* INF ARTHRIT NOS-ANKLE */
"71198" = "1" /* INF ARTHRIT NOS-OTH SITE */
"71199" = "1" /* INF ARTHRITIS NOS-MULT */
"7280" = "1" /* INFECTIVE MYOSITIS */
"72886" = "1" /* NECROTIZING FASCIITIS */
"73000" = "1" /* AC OSTEOMYELITIS-UNSPEC */
"73001" = "1" /* AC OSTEOMYELITIS-SHLDER */
"73002" = "1" /* AC OSTEOMYELITIS-UP/ARM */
"73003" = "1" /* AC OSTEOMYELITIS-FOREARM */
"73004" = "1" /* AC OSTEOMYELITIS-HAND */
"73005" = "1" /* AC OSTEOMYELITIS-PELVIS */
"73006" = "1" /* AC OSTEOMYELITIS-L/LEG */
"73007" = "1" /* AC OSTEOMYELITIS-ANKLE */
"73008" = "1" /* AC OSTEOMYELITIS NEC */
"73009" = "1" /* AC OSTEOMYELITIS-MULT */
"73010" = "1" /* CHR OSTEOMYELITIS-UNSP */
"73011" = "1" /* CHR OSTEOMYELIT-SHLDER */
"73012" = "1" /* CHR OSTEOMYELIT-UP/ARM */
"73013" = "1" /* CHR OSTEOMYELIT-FOREARM */
"73014" = "1" /* CHR OSTEOMYELIT-HAND */
"73015" = "1" /* CHR OSTEOMYELIT-PELVIS */
"73016" = "1" /* CHR OSTEOMYELIT-L/LEG */
"73017" = "1" /* CHR OSTEOMYELIT-ANKLE */
"73018" = "1" /* CHR OSTEOMYELIT NEC */
"73019" = "1" /* CHR OSTEOMYELIT-MULT */
"73020" = "1" /* OSTEOMYELITIS NOS-UNSPEC */
"73021" = "1" /* OSTEOMYELITIS NOS-SHLDER */
"73022" = "1" /* OSTEOMYELITIS NOS-UP/ARM */
"73023" = "1" /* OSTEOMYELIT NOS-FOREARM */
"73024" = "1" /* OSTEOMYELITIS NOS-HAND */
"73025" = "1" /* OSTEOMYELITIS NOS-PELVIS */
"73026" = "1" /* OSTEOMYELITIS NOS-L/LEG */
"73027" = "1" /* OSTEOMYELITIS NOS-ANKLE */
"73028" = "1" /* OSTEOMYELIT NOS-OTH SITE */
"73029" = "1" /* OSTEOMYELITIS NOS-MULT */
"73030" = "1" /* PERIOSTITIS-UNSPEC */
"73031" = "1" /* PERIOSTITIS-SHLDER */
"73032" = "1" /* PERIOSTITIS-UP/ARM */
"73033" = "1" /* PERIOSTITIS-FOREARM */
"73034" = "1" /* PERIOSTITIS-HAND */
"73035" = "1" /* PERIOSTITIS-PELVIS */
"73036" = "1" /* PERIOSTITIS-L/LEG */
"73037" = "1" /* PERIOSTITIS-ANKLE */
"73038" = "1" /* PERIOSTITIS NEC */
"73039" = "1" /* PERIOSTITIS-MULT */
"73080" = "1" /* BONE INFECT NEC-UNSPEC */
"73081" = "1" /* BONE INFECT NEC-SHLDER */
"73082" = "1" /* BONE INFECT NEC-UP/ARM */
"73083" = "1" /* BONE INFECT NEC-FOREARM */
"73084" = "1" /* BONE INFECT NEC-HAND */
"73085" = "1" /* BONE INFECT NEC-PELVIS */
"73086" = "1" /* BONE INFECT NEC-L/LEG */
"73087" = "1" /* BONE INFECT NEC-ANKLE */
"73088" = "1" /* BONE INFECT NEC-OTH SITE */
"73089" = "1" /* BONE INFECT NEC-MULT */
"73090" = "1" /* BONE INFEC NOS-UNSP SITE */
"73091" = "1" /* BONE INFECT NOS-SHLDER */
"73092" = "1" /* BONE INFECT NOS-UP/ARM */
"73093" = "1" /* BONE INFECT NOS-FOREARM */
"73094" = "1" /* BONE INFECT NOS-HAND */
"73095" = "1" /* BONE INFECT NOS-PELVIS */
"73096" = "1" /* BONE INFECT NOS-L/LEG */
"73097" = "1" /* BONE INFECT NOS-ANKLE */
"73098" = "1" /* BONE INFECT NOS-OTH SITE */
"73099" = "1" /* BONE INFECT NOS-MULT */
"7713" = "1" /* TETANUS NEONATORUM */
"7714" = "1" /* OMPHALITIS OF NEWBORN */
"7715" = "1" /* NEONATAL INFEC MASTITIS */
"77181" = "1" /* NB SEPTICEMIA SEPSIS */
"77182" = "1" /* NB URINARY TRACT INFECTN */
"77183" = "1" /* BACTEREMIA OF NEWBORN */
"77189" = "1" /* PERINATAL INFECTION NEC */
"7775" = "1" /* NECROT ENTEROCOLITIS NB */
"7854" = "1" /* GANGRENE */
"78552" = "1" /* SEPTIC SHOCK */
"7907" = "1" /* BACTEREMIA */
"9101" = "1" /* ABRASION HEAD-INFECTED */
"9103" = "1" /* BLISTER HEAD-INFECTED */
"9105" = "1" /* INSECT BITE HEAD-INFECT */
"9107" = "1" /* FOREIGN BODY HEAD-INFECT */
"9109" = "1" /* SUPERF INJ HEAD NEC-INF */
"9111" = "1" /* ABRASION TRUNK-INFECTED */
"9113" = "1" /* BLISTER TRUNK-INFECTED */
"9115" = "1" /* INSECT BITE TRUNK-INFEC */
"9117" = "1" /* FOREIGN BODY TRUNK-INFEC */
"9119" = "1" /* SUPERF INJ TRNK NEC-INF */
"9121" = "1" /* ABRASION SHLDR/ARM-INFEC */
"9123" = "1" /* BLISTER SHOULDER/ARM-INF */
"9125" = "1" /* INSECT BITE SHLD/ARM-INF */
"9127" = "1" /* FB SHOULDER/ARM-INFECT */
"9129" = "1" /* SUPERF INJ SHLDR NEC-INF */
"9131" = "1" /* ABRASION FOREARM-INFECT */
"9133" = "1" /* BLISTER FOREARM-INFECTED */
"9135" = "1" /* INSECT BITE FOREARM-INF */
"9137" = "1" /* FOREIGN BODY FOREARM-INF */
"9139" = "1" /* SUPRF INJ FORARM NEC-INF */
"9141" = "1" /* ABRASION HAND-INFECTED */
"9143" = "1" /* BLISTER HAND-INFECTED */
"9145" = "1" /* INSECT BITE HAND-INFECT */
"9147" = "1" /* FOREIGN BODY HAND-INFECT */
"9149" = "1" /* SUPERF INJ HAND NEC-INF */
"9151" = "1" /* ABRASION FINGER-INFECTED */
"9153" = "1" /* BLISTER FINGER-INFECTED */
"9155" = "1" /* INSECT BITE FINGER-INFEC */
"9157" = "1" /* FOREIGN BODY FINGER-INF */
"9159" = "1" /* SUPRF INJ FINGER NEC-INF */
"9161" = "1" /* ABRASION HIP/LEG-INFECT */
"9163" = "1" /* BLISTER HIP & LEG-INFECT */
"9165" = "1" /* INSECT BITE HIP/LEG-INF */
"9167" = "1" /* FOREIGN BDY HIP/LEG-INF */
"9169" = "1" /* SUPERF INJ LEG NEC-INFEC */
"9171" = "1" /* ABRASION FOOT/TOE-INFEC */
"9173" = "1" /* BLISTER FOOT & TOE-INFEC */
"9175" = "1" /* INSECT BITE FOOT/TOE-INF */
"9177" = "1" /* FOREIGN BDY FOOT/TOE-INF */
"9179" = "1" /* SUPERF INJ FOOT NEC-INF */
"9191" = "1" /* ABRASION NEC-INFECTED */
"9193" = "1" /* BLISTER NEC-INFECTED */
"9195" = "1" /* INSECT BITE NEC-INFECTED */
"9197" = "1" /* SUPERFICIAL FB NEC-INFEC */
"9199" = "1" /* SUPERFIC INJ NEC-INFECT */
"99590" = "1" /* SIRS, NOS */
"99591" = "1" /* SIRS-INFECT W/O ORG DYSF */
"99592" = "1" /* SIRS-INFECT W ORGAN DYSF */
"99660" = "1" /* INFECT INFLAMM DEVICE IMPLANT GRAFT NOS */
"99661" = "1" /* INFECT INFLAMM CARDIAC DEVICE IMPLANT GRAFT */
"99662" = "1" /* INFECT INFLAMM VASCULAR DEVICE IMPLANT GRAFT  */
"99663" = "1" /* INFECT INFLAMM NERV DEVICE IMPLANT GRAFT  */
"99664" = "1" /* INFECT INFLAMM URINARY CATH */
"99665" = "1" /* INFECT INFLAMM GU DEVICE IMPLANT GRAFT  */
"99666" = "1" /* INFECT INFLAMM JOINT PROSTH */
"99667" = "1" /* INFECT INFLAMM OTH ORTHOP DEVICE IMPLANT GRAFT NOS */
"99669" = "1" /* INFECT INFLAMM OTH DEVICE IMPLANT GRAFT */
"99762" = "1" /* INFECTION AMPUTAT STUMP */
"99851" = "1" /* INFECTED POSTOP SEROMA */
"99859" = "1" /* OTHER POSTOP INFECTION */
"9993" = "1" /* INFEC COMPL MED CARE NEC */
OTHER   = "0" ;

 /* EXCLUDE: INFECTION */
 /* DIAGNOSTIC RELATED GROUPS (DRGS) */

VALUE $INFECDR
"020" = "1" /* NERVOUS SYSTEM INFECTION EXCEPT VIRAL MENINGITIS */
"068" = "1" /* OTITIS MEDIA AND URI, AGE GREATER THAN 17 WITH CC */
"069" = "1" /* OTITIS MEDIA AND URI, AGE GREATER THAN 17 WITHOUT CC */
"070" = "1" /* OTITIS MEDIA & URI AGE 0-17 */
"079" = "1" /* RESPIRATORY INFECTIONS AND INFLAMMATIONS, AGE GREATER THAN 17 WITH CC */
"080" = "1" /* RESPIRATORY INF" = "1" /* ECTIONS AND INFLAMMATIONS, AGE GREATER THAN 17 WITHOUT CC */
"081" = "1" /* RESPIRATORY INFECTIONS & INFLAMMATIONS AGE 0-17 */
"089" = "1" /* SIMPLE PNEUMONIA AND PLEURISY, AGE GREATER THAN 17 WITH CC */
"090" = "1" /* SIMPLE PNEUMONIA AND PLEURISY, AGE GREATER THAN 17 WITHOUT CC */
"091" = "1" /* SIMPLE PNEUMONIA AND PLEURISY, AGE 0-17 */
"126" = "1" /* ACUTE AND SUBACUTE ENDOCARDITIS */
"238" = "1" /* OSTEOMYELITIS */
"242" = "1" /* SEPTIC ARTHRITIS */
"277" = "1" /* CELLULITIS, AGE GREATER THAN 17 WITH CC */
"278" = "1" /* CELLULITIS, AGE GREATER THAN 17 WITHOUT CC */
"279" = "1" /* CELLULITIS AGE 0-17 */
"320" = "1" /* KIDNEY AND URINARY TRACT INFECTIONS, AGE GREATER THAN 17 WITH CC */
"321" = "1" /* KIDNEY AND URINARY TRACT INFECTIONS, AGE GREATER THAN 17 WITHOUT CC */
"322" = "1" /* KIDNEY & URINARY TRACT INFECTIONS AGE 0-17 */
"368" = "1" /* INFECTIONS OF FEMALE REPRODUCTIVE SYSTEM */
"415" = "1" /* OR PROCEDURE FOR INFECTIOUS AND PARASITIC DISEASES */
"416" = "1" /* SEPTICEMIA, AGE GREATER THAN 17 */
"417" = "1" /* SEPTICEMIA AGE 0-17 */
"423" = "1" /* OTHER INFECTIOUS & PARASITIC DISEASES DIAGNOSES */
"560" = "1" /* BACTERIAL & TUBERCULOUS INFECTIONS OF NERVOUS SYSTEM */
"561" = "1" /* NON-BACTERIAL INFECTIONS OF NERVOUS SYSTEM EXCEPT VIRAL MENINGITIS */
"575" = "1" /* SEPTICEMIA W MV96+ HOURS  AGE  >17 */
"576" = "1" /* SEPTICEMIA W/O MV96+ HOURS AGE  >17 */
"578" = "1" /* INFECTIOUS & PARASITIC DISEASES W OR PROCEDURE */
"579" = "1" /* POSTOPERATIVE OR POST-TRAUMATIC INFECTIONS W OR PROCEDURE */
OTHER = "0" ;

 /* FTR-SHOCK OR CARDIAC ARREST */
 /* ICD-9-CM DIAGNOSIS CODES: */

VALUE $FTR5DX
"4275"  = "1" /* CARDIAC ARREST */
"6395"  = "1" /* COMPLICATIONS FOLLOWING ABORTION AND ECTOPIC AND MOLAR PREGNANCIES, SHOCK */
"66910" = "1" /* OBSTETRIC SHOCK-UNSPEC */
"66911" = "1" /* OBSTETRIC SHOCK-DELIVER */
"66912" = "1" /* OBSTET SHOCK-DELIV W P/P */
"66913" = "1" /* OBSTETRIC SHOCK-ANTEPAR */
"66914" = "1" /* OBSTETRIC SHOCK-POSTPART */
"7855"  = "1" /* SHOCK NOS */
"78550" = "1" /* SHOCK, UNSPECIFIED */
"78551" = "1" /* CARDIOGENIC SHOCK */
"78552" = "1" /* SEPTIC SHOCK */
"78559" = "1" /* SHOCK WITHOUT MENTION OF TRAUMA, OTHER */
"7991"  = "1" /* RESPIRATORY ARREST */
"9950"  = "1" /* OTHER ANAPHYLACTIC SHOCK */
"9954"  = "1" /* SHOCK DUE TO ANESTHESIA */
"9980"  = "1" /* POSTOPERATIVE SHOCK */
"9994"  = "1" /* ANAPHYLACTIC SHOCK DUE TO SERUM */
OTHER   = "0" ;

/* ICD-9-CM PROCEDURE CODES: */

VALUE $FTR5PR
"9393" = "1" /* NONMECHANICAL METHODS OF RESUSCITATION */
"9960" = "1" /* CARDIOPULMONARY RESUSCITATION, NOS */
"9963" = "1" /* CLOSED CHEST CARDIAC MASSAGE */
OTHER  = "0" ;

 /* EXCLUDE ABORTION-RELATED SHOCK */
 /* ICD-9-CM DIAGNOSIS CODES: */

VALUE $FTR5EX
"63450"  = "1" /* SPON ABORT W SHOCK-UNSP */
"63451"  = "1" /* SPON ABORT W SHOCK-INC */
"63452"  = "1" /* SPON ABORT W SHOCK-COMP */
"63550"  = "1" /* LEGAL ABORT W SHOCK-UNSP */
"63551"  = "1" /* LEGAL ABORT W SHOCK-INC */
"63552"  = "1" /* LEGAL ABORT W SHOCK-COMP */
"63650"  = "1" /* ILLEG ABORT W SHOCK-UNSP */
"63651"  = "1" /* ILLEG ABORT W SHOCK-INC */
"63652"  = "1" /* ILLEG ABORT W SHOCK-COMP */
"63750"  = "1" /* ABORT NOS W SHOCK-UNSP */
"63751"  = "1" /* ABORT NOS W SHOCK-INC */
"63752"  = "1" /* ABORT NOS W SHOCK-COMP */
"6385"   = "1" /* ATTEM ABORTION W SHOCK */
OTHER  = "0" ;

 /* FTR-GI HEMORRHAGE-ACUTE ULCER */
 /* ICD-9-CM DIAGNOSIS CODES: */

VALUE $FTR6DX
"4560"  = "1" /* ESOPHAGEAL VARICES WITH BLEEDING */
"45620" = "1" /* ESOPHAGEAL VARICES IN DISEASES CLASSIFIED ELSEWHERE WITH BLEEDING */
"5307"  = "1" /* GASTROESOPHAGEAL LACERATION-HEMORRHAGE SYNDROME */
"53082" = "1" /* ESOPHAGEAL HEMORRHAGE */
"53100" = "1" /* ACUTE WITH HEMORRHAGE - WITHOUT MENTION OF OBSTRUCTION */
"53101" = "1" /* ACUTE WITH HEMORRHAGE - WITH OBSTRUCTION */
"53110" = "1" /* ACUTE WITH PERFORATION - WITHOUT MENTION OF OBSTRUCTION */
"53111" = "1" /* ACUTE WITH PERFORATION - WITH OBSTRUCTION */
"53120" = "1" /* ACUTE WITH HEMORRHAGE AND PERFORATION - WITHOUT MENTION OF OBSTRUCTION */
"53121" = "1" /* ACUTE WITH HEMORRHAGE AND PERFORATION - WITH OBSTRUCTION */
"53130" = "1" /* ACUTE WITHOUT MENTION OF HEMORRHAGE OR PERFORATION  WITHOUT MENTION OF OBSTRUCTION */
"53131" = "1" /* ACUTE WITHOUT MENTION OF HEMORRHAGE OR PERFORATION - WITH   OBSTRUCTION */
"53190" = "1" /* UNSPECIFIED AS ACUTE OR CHRONIC, WITHOUT MENTION OF HEMORRHAGE OR PERFORATION - WITHOUT MENTION OF OBSTRUCTION */
"53191" = "1" /* UNSPECIFIED AS ACUTE OR CHRONIC, WITHOUT MENTION OF HEMORRHAGE OR PERFORATION - WITH OBSTRUCTION */
"53200" = "1" /* ACUTE WITH HEMORRHAGE - WITHOUT MENTION OF OBSTRUCTION */
"53201" = "1" /* ACUTE WITH HEMORRHAGE - WITH OBSTRUCTION */
"53210" = "1" /* ACUTE WITH PERFORATION - WITHOUT MENTION OF OBSTRUCTION */
"53211" = "1" /* ACUTE WITH PERFORATION - WITH OBSTRUCTION */
"53220" = "1" /* ACUTE WITH HEMORRHAGE AND PERFORATION - WITHOUT MENTION OF OBSTRUCTION */
"53221" = "1" /* ACUTE WITH HEMORRHAGE AND PERFORATION - WITH OBSTRUCTION */
"53230" = "1" /* ACUTE WITHOUT MENTION OF HEMORRHAGE OR PERFORATION -WTHOUT MENTION OF OBSTRUCTION */
"53231" = "1" /* ACUTE WITHOUT MENTION OF HEMORRHAGE OR PERFORATION - WITH OBSTRUCTION */
"53290" = "1" /* UNSPECIFIED AS ACUTE OR CHRONIC, WITHOUT MENTION OF HEMORRHAGE OR PERFORATION - WITHOUT MENTION OF OBSTRUCTION */
"53291" = "1" /* UNSPECIFIED AS ACUTE OR CHRONIC, WITHOUT MENTION OF HEMORRHAGE OR PERFORATION - WITH OBSTRUCTION */
"53300" = "1" /* SITE UNSPECIFIED ACUTE WITH HEMORRHAGE - WITHOUT MENTION OF OBSTRUCTION */
"53301" = "1" /* SITE UNSPECIFIED ACUTE WITH HEMORRHAGE - WITH OBSTRUCTION */
"53310" = "1" /* SITE UNSPECIFIED ACUTE WITH PERFORATION - WITHOUT MENTION OF OBSTRUCTION */
"53311" = "1" /* SITE UNSPECIFIED ACUTE WITH -PERFORATION WITH OBSTRUCTION */
"53320" = "1" /* SITE UNSPECIFIED ACTE WITH HEMORRHAGE AND PERFORATION - WITHOUT MENTION OF OBSTRUCTION */
"53321" = "1" /* SITE UNSPECIFIED ACUTE WITH HEMORRHAGE AND PERFORATION - WITHOUT MENTION OF OBSTRUCTION */
"53330" = "1" /* SITE UNSPECIFIED ACUTE WITHOUT MENTION OF HEMORRHAGE AND PERFFORATION - WITHOUT MENTION OF OBSTRUCTION */
"53331" = "1" /* SITE UNSPECIFIED ACUTE WITHOUT MENTION OF HEMORRHAGE AND PERFORATION - WITH OBSTRUCTION */
"53390" = "1" /* SITE UNSPECIFIED UNSPECIFIED AS ACUTE OR CHRONIC, WITHOUT MENTION OF HEMORRHAGE OR PERFORATION - WITHOUT MENTION OF OBSTRUCTION */
"53391" = "1" /* SITE UNSPECIFIED UNSPECIFIED AS ACUTE OR CHRONIC, WITHOUT MENTION OF HEMORRHAGE OR PERFORATION - WITH OBSTRUCTION */
"53400" = "1" /* ACUTE WITH HEMORRHAGE -WITHOUT MENTION OF OBSTRUCTION */
"53401" = "1" /* ACUTE WITH HEMORRHAGE - WITH OBSTRUCTION */
"53410" = "1" /* ACUTE WITH PERFORATION - WITHOUT MENTION OF OBSTRUCTION */
"53411" = "1" /* ACUTE WITH PERFORATION  WITH OBSTRUCTION */
"53420" = "1" /* ACUTE WITH HEMORRHAGE AND PERFORATION - WITHOUT MENTION OF OBSTRUCTION */
"53421" = "1" /* ACUTE WITH HEMORRHAGE AND PERFORATION - WITH OBSTRUCTION */
"53430" = "1" /* ACUTE WITHOUT MENTION OF HEMORRHAGE OR PERFORATION -WITHOUT MENTION OF OBSTRUCTION */
"53431" = "1" /* ACUTE WITHOUT MENTION OF HEMORRHAGE OR PERFORATION - WITH OBSTRUCTION */
"53490" = "1" /* UNSPECIFIED AS ACUTE OR CHRONIC, WITHOUT MENTION OF HEMORRHAGE OR PERFORATION - WITHOUT MENTION OF OBSTRUCTION */
"53491" = "1" /* UNSPECIFIED AS ACUTE OR CHRONIC, WITHOUT MENTION OF HEMORRHAGE OR PERFORATION - WITH OBSTRUCTION */
"53501" = "1" /* ACUTE GASTRITIS - WITH HEMORRHAGE */
"53511" = "1" /* ATROPHIC GASTRITIS - WITH HEMORRHAGE */
"53521" = "1" /* GASTRIC MUCOSAL HYPERTROPHY -WITH HEMORRHAGE */
"53531" = "1" /* ALCOHOLIC GASTRITIS - WITH HEMORRHAGE */
"53541" = "1" /* OTHER SPECIFIED GASTRITIS - WITH HEMORRHAGE */
"53551" = "1" /* UNSPECIFIED GASTRITIS AND GASTRODUODENITIS - WITH HEMORRHAGE */
"53561" = "1" /* DUODENITIS - WITH HEMORRHAGE */
"53783" = "1" /* ANGIODYSPLASIA OF STOMACH AND DUODENUM WITH HEMORRHAGE */
"53784" = "1" /* DIEULAFOY LESION (HEMORRHAGIC) OF STOMACH AND DUODENUM */
"56202" = "1" /* DIVERTICULOSIS OF SMALL INTESTINE WITH HEMORRHAGE */
"56203" = "1" /* DIVERTICULITIS OF SMALL INTESTINE WITH HEMORRHAGE */
"56212" = "1" /* DIVERTICULOSIS OF COLON WITH HEMORRHAGE */
"56213" = "1" /* DIVERICULITIS OF COLON WITH HEMORRHAGE */
"5693"  = "1" /* HEMORRHAGE OF RECTUM AND ANUS */
"56985" = "1" /* ANGIODYSPLASIA OF INTESTINE WITH HEMORRHAGE */
"56986" = "1" /* DIEULAFOY LESION (HEMORRHAGIC) OF INTESTINE */
"5780"  = "1" /* HEMATEMESIS */
"5781"  = "1" /* BLOOD IN STOOL */
"5789"  = "1" /* HEMORRHAGE OF GASTROINTESTINAL TRACT, UNSPECIFIED */
OTHER   = "0" ;

 /* EXCLUDE: ALCOHOLISM */
 /* ICD-9-CM DIAGNOSIS CODES: */

VALUE $ALCHLSM
"2910"  = "1"  /* ALCOHOL WITHDRAWAL DELIRIUM */
"2911"  = "1"  /* ALCOHOL AMNESTIC SYNDROME */
"2912"  = "1"  /* OTHER ALCOHOLIC DEMENTIA */
"2913"  = "1"  /* ALCOHOL WITHDRAWAL HALLUCINOSIS */
"2914"  = "1"  /* IDIOSYNCRATIC ALCOHOL INTOXICATION */
"2915"  = "1"  /* ALCOHOLIC JEALOUSY */
"29181" = "1"  /* OTHER SPECIFIED ALCOHOLIC PSYCHOSES, ALCOHOL WITHDRAWAL */
"29182" = "1"  /* ALCOH INDUCE SLEEP DISOR */
"29189" = "1"  /* OTHER SPECIFIED ALCOHOLIC PSYCHOSES, OTHER */
"2919"  = "1"  /* UNSPECIFIED ALCOHOLIC PSYCHOSIS */
"30300" = "1"  /* ACUTE ALCOHOL INTOXICATION */
"30301" = "1"  /* ACUTE ALCOHOLIC INTOXICATION */
"30302" = "1"  /* ACUTE ALCOHOLIC INTOXICATION */
"30303" = "1"  /* ACUTE ALCOHOLIC INTOXICATION */
"30390" = "1"  /* OTHER AND UNSPECIFIED ALCOHOL DEPENDENCE */
"30391" = "1"  /* OTHER AND UNSPECIFIED ALCOHOL DEPENDENCE */
"30392" = "1"  /* OTHER AND UNSPECIFIED ALCOHOL DEPENDENCE */
"30393" = "1"  /* OTHER AND UNSPECIFIED ALCOHOL DEPENDENCE */
"30500" = "1"  /* NONDEPENDENT ABUSE OF DRUGS, ALCOHOL ABUSE */
"30501" = "1"  /* NONDEPENDENT ABUSE OF DRUGS, ALCOHOL ABUSE */
"30502" = "1"  /* NONDEPENDENT ABUSE OF DRUGS, ALCOHOL ABUSE */
"30503" = "1"  /* NONDEPENDENT ABUSE OF DRUGS, ALCOHOL ABUSE */
"4255"  = "1"  /* ALCOHOLIC CARDIOMYOPATHY  */
"5710"  = "1"  /* ALCOHOLIC FATTY LIVER */
"5711"  = "1"  /* ACUTE ALCOHOLIC HEPATITIS */
"5712"  = "1"  /* ALCOHOLIC CIRRHOSIS OF LIVER */
"5713"  = "1"  /* ALCOHOLIC LIVER DAMAGE, UNSPECIFIED */
"53530" = "1"  /* ALCOHOLIC GASTRITIS, WITHOUT MENTION OF HEMORRHAGE  */
"53531" = "1"  /* ALCOHOLIC GASTRITIS, WITH HEMORRHAGE   */
"9800"  = "1"  /* TOXIC EFFECT OF ALCOHOL, ETHYL ALCOHOL */
"9809"  = "1"  /* TOXIC EFFECT OF ALCOHOL, UNSPECIFIED ALCOHOL */
OTHER   = "0";

 /* EXCLUDE */
 /* ICD-9-CM DIAGNOSIS CODES: */

VALUE $FTR6EX
"2800" = "1" /* SECONDARY TO BLOOD LOSS [CHRONIC] */
"2851" = "1" /* ACUTE POSTHEMORRHAGIC ANEMIA */
OTHER  = "0" ;

 /* ------------------------------------- */
 /* FOREIGN BODY LEFT IN DURING PROCEDURE */
 /*   PROVIDER INDICATOR / PSI #  5       */
 /*   AREA     INDICATOR / PSI # 21       */
 /* ------------------------------------- */

 /* ICD-9-CM DIAGNOSIS CODES: */

VALUE $FOREIID
"9984"  = "1" /* FOREIGN BODY ACCIDENTALLY LEFT DURING A PROCEDURE */
"9987"  = "1" /* ACUTE REACTION TO FOREIGN SUBSTANCE ACCIDENTALLY LEFT DURING   A PROCEDURE */
"E8710" = "1" /* SURGICAL OPERATION */
"E8711" = "1" /* INFUSION OR TRANSFUSION */
"E8712" = "1" /* KIDNEY DIALYSIS OR OTHER PERFUSION */
"E8713" = "1" /* INJECTION OR VACCINATION */
"E8714" = "1" /* ENDOSCOPIC EXAMINATION */
"E8715" = "1" /* ASPIRATION OF FLUID OR TISSUE, PUNCTURE, AND CATHETERIZATION */
"E8716" = "1" /* HEART CATHETERIZATION */
"E8717" = "1" /* REMOVAL OF CATHETER OR PACKING */
"E8718" = "1" /* OTHER SPECIFIED PROCEDURES */
"E8719" = "1" /* UNSPECIFIED PROCEDURE */
OTHER   = "0" ;

 /* ------------------------------- */
 /* IATROGENIC PNEUMOTHORAX         */
 /*   PROVIDER INDICATOR / PSI #  6 */
 /*   AREA     INDICATOR / PSI # 22 */
 /* ------------------------------- */

 /* ICD-9-CM DIAGNOSIS CODES: */;

VALUE $IATROID
"5121"  = "1" /* IATROGENIC PNEUMOTHORAX */
OTHER   = "0" ;

 /* EXCLUDE: PLEURAL EFFUSION */
 /* ICD-9-CM DIAGNOSIS CODES: */;

VALUE $PLEURAD
"0101 "  = "1" /* TUBERCULOUS PLEURISY IN PRIMARY PROGRESSIVE TUBERCULOSIS */
"01010"  = "1" /* TUBERCULOUS PLEURISY IN PRIMARY PROGRESSIVE TUBERCULOSIS, UNSPECIFIED */
"01011"  = "1" /* TPIPPT, BACTERIAL OR HISTOLOGICAL EXAM NOT DONE */
"01012"  = "1" /* TPIPPT, BACTERIAL OR HISTOLOGICAL EXAM UNKNOWN */
"01013"  = "1" /* TPIPPT, TUBERCLE BACILI FOUND BY MICROSCOPY */
"01014"  = "1" /* TPIPPT, TUBERCLE BACILI NOT FOUND BY MICROSCOPY BUT BY BACTERIAL CULTURE */
"01015"  = "1" /* TPIPPT, TUBERCLE BACILI NOT FOUND BY BACTERIOLOGICAL BUT CONFIRMED HISTOLOGICALLY */
"01016"  = "1" /* TPIPPT, TUBERCLE BACILI NOT FOUND BY BACTERILOGICAL OR HISTOLOGICAL BUT CONFIRMED OTHER METHODS */
"0117 "  = "1" /* TUBRCULOUS PNEUMOTHORAX */
"01170"  = "1" /* TUBRCULOUS PNEUMOTHORAX, UNSPECIFIED */
"01171"  = "1" /* TPNEU, BACTERIAL OR HISTOLOGICAL EXAM NOT DONE */
"01172"  = "1" /* TPNEU, BACTERIAL OR HISTOLOGICAL EXAM UNKNOWN */
"01173"  = "1" /* TPNEU, TUBERCLE BACILI FOUND BY MICROSCOPY */
"01174"  = "1" /* TPNEU, TUBERCLE BACILI NOT FOUND BY MICROSCOPY BUT BY BACTERIAL CULTURE */
"01175"  = "1" /* TPNEU, TUBERCLE BACILI NOT FOUND BY BACTERIOLOGICAL BUT CONFIRMED HISTOLOGICALLY */
"01176"  = "1" /* TPENU, TUBERCLE BACILI NOT FOUND BY BACTERILOGICAL OR HISTOLOGICAL BUT CONFIRMED OTHER METHODS */
"0120 "  = "1" /* TUBERCULOUS PLEURISY */
"01200"  = "1" /* TUBERCULOUS PLEURISY, UNSPECIFIED */
"01201"  = "1" /* TP, BACTERIAL OR HISTOLOGICAL EXAM NOT DONE */
"01202"  = "1" /* TP, BACTERIAL OR HISTOLOGICAL EXAM UNKNOWN */
"01203"  = "1" /* TP, TUBERCLE BACILI FOUND BY MICROSCOPY */
"01204"  = "1" /* TP, TUBERCLE BACILI NOT FOUND BY MICROSCOPY BUT BY BACTERIAL CULTURE */
"01205"  = "1" /* TP, TUBERCLE BACILI NOT FOUND BY BACTERIOLOGICAL BUT CONFIRMED HISTOLOGICALLY */
"01206"  = "1" /* TP, TUBERCLE BACILI NOT FOUND BY BACTERILOGICAL OR HISTOLOGICAL BUT CONFIRMED OTHER METHODS */
"1972 "  = "1" /* SECOND MALIG NEO PLEURA */  
"5111 "  = "1" /* WITH EFUSION, WITH MENTION OF A BACTERIAL CAUSE OTHER THAN TUBERCULOSIS */
"5118 "  = "1" /* OTHER SPECIFIED FORM OF EFFUSION, EXCEPT TUBERCULOUS */
"5119 "  = "1" /* UNSPECIFIED PLEURAL EFFUSION */
OTHER    = "0" ;

 /* EXCLUDE: THORACIC SURGERY */
 /* ICD-9-CM PROCEDURE CODES: */

VALUE $THORAIP
"3121" = "1" /* MEDIASTINAL TRACHEOSTOMY */
"3145" = "1" /* OPEN BIOPSY OF LARYNX OR TRACHEA */
"3173" = "1" /* CLOSURE OF OTHER FISTULA OF TRACHEA */
"3179" = "1" /* OTHER REPAIR AND PLASTIC OPERATIONS ON TRACHEA */
"3199" = "1" /* OTHER OPERATIONS ON TRACHEA */
"3209" = "1" /* OTHER LOCAL EXCISION OR DESTRUCTION OF LESION OR TISSUE OF BRONCHUS */
"321"  = "1" /* OTHER EXCISION OF BRONCHUS */
"3220" = "1" /* THORAC EXC LUNG LESION */
"3221" = "1" /* PLICATION OF EMPHYSEMATIOUS BLEB */
"3222" = "1" /* LUNG VOLUME REDUCTION SURGERY */
"3223" = "1" /* OPEN ABLTN LUNG LES/TISS */
"3224" = "1" /* PERC ABLTN LUNG LES/TISS */ 
"3225" = "1" /* THOR ABLTN LUNG LES/TISS */ 
"3226" = "1" /* ABLTN LUNG TISS NEC/NOS */  
"3228" = "1" /* ENDOSCOPIC EXCISION OR DETRUCTION OF LESION OR TISSUE OF LUNG */
"3229" = "1" /* OTHER LOCAL EXCISION OR DESTRUCTION OF LESION OR TISSUE OF LUNG */
"323"  = "1" /* SEGMENTAL RESECTION OF LUNG */
"3230" = "1" /* THORAC SEG LUNG RESECT */
"3239" = "1" /* OTH SEG LUNG RESECT NOS */
"324"  = "1" /* LOBECTOMY OF LUNG */
"3241" = "1" /* THORAC LOBECTOMY LUNG */
"325"  = "1" /* COMPLETE PNEUMONECTOMY */
"3250" = "1" /* THORACOSPC PNEUMONECTOMY */
"3259" = "1" /* OTHER PNEUMONECTOMY NOS */
"326"  = "1" /* RADICAL DISSECTION OF THORACIC STRUCTURES */
"329"  = "1" /* OTHER EXCISION OF LUNG */
"330"  = "1" /* INCISION OF BRONCHUS */
"331"  = "1" /* INCISION OF LUNG */
"3320" = "1" /* THORACOSCOPC LUNG BIOPSY */
"3325" = "1" /* OPEN BIOPSY OF BRONCHUS */
"3326" = "1" /* CLOSE [PERCUTANEOUS][NEEDLE] BIOPSY OF LUNG */
"3327" = "1" /* CLOSED ENDOSCOPIC BIOPSY OF LUNG */
"3328" = "1" /* OPEN BIOPSY OF LUNG */
"3331" = "1" /* DESTRUCTION OF PHRENIC NERVE FOR COLLAPSE OF LUNG (NO LONGER PERFORMED) */
"3332" = "1" /* ARTIFICAL PNEUMOTHORAX FOR COLLAPSE OF LUNG */
"3334" = "1" /* THORACOPLASTY */
"3339" = "1" /* OTHER SURGICAL COLLAPSE OF LUNG */
"3341" = "1" /* SUTURE OF LACERATION OF BRONCHUS */
"3342" = "1" /* CLOSURE OF BRONCHIAL FISTULA */
"3343" = "1" /* CLOSURE OF LACERATION OF LUNG */
"3348" = "1" /* OTHER REPAIR OAND PLASTIC OPERATIONS ON BRONCHUS */
"3349" = "1" /* OTHER REPAIR AND PLASTIC OPERATIONS ON LUNG */
"335"  = "1" /* LUNG TRANSPLANTATION */
"3350" = "1" /* LUNG TRANSPLANTATION, NOS */
"3351" = "1" /* UNILATERAL LUNG TRANSPLANTATION */
"3352" = "1" /* BILATERAL LUNG TRANSPLANTATION */
"336"  = "1" /* COMBINED HEART-LUNG TRANSPLANTATION */
"3392" = "1" /* LIGATION OF BRONCHUS */
"3393" = "1" /* PUNCTURE OF LUNG */
"3398" = "1" /* OTHER OPERATIONS ON BRONCHUS */
"3399" = "1" /* OTHER OPERATIONS ON LUNG */
"3329" = "1" /* OTHER DIAGNOSTIC PROCEDURE ON LUNG AND BRONCHUS */
"3333" = "1" /* PNEUMOPERITONEUM FOR COLLAPSE OF LUNG */
"3401" = "1" /* INCISION OF CHEST WALL */
"3402" = "1" /* EXPLORATORY THORACOTOMY */
"3403" = "1" /* REOPENING OF RECENT THORACOTOMY SITE */
"3405" = "1" /* CREATION OF PLEUROPERITONEAL SHUNT */
"3409" = "1" /* OTHER INCISION OF PLEURA */
"341"  = "1" /* INCISION OF MEDIASTINUM */
"3420" = "1" /* THORACOSCOPIC PLEURAL BX */
"3421" = "1" /* TRANSPLEURAL THORACOSOCOPY */
"3422" = "1" /* MEDIASTINOSCOPY */
"3423" = "1" /* BIOPSY OF CHEST WALL */
"3424" = "1" /* PLEURAL BIOPSY */
"3425" = "1" /* CLOSED [PERCUTANEOUS][NEEDLE] BIOPSY OF MEDIASTINUM */
"3426" = "1" /* OPEN BIOPSY OF MEDIASTINUM */
"3427" = "1" /* BIOPSY OF DIAPHRAGM */
"3428" = "1" /* OTHER DIAGNOSTIC PROCEDURES ON CHEST WALL, PLEURA, AND DIAPHRAGM */
"3429" = "1" /* OTHER DIAGNOSTIC PROCEDURES ON MEDIASTINUM */
"343"  = "1" /* EXCISION OR DESTRUCTION OF LESION OR TISSUE OF MEDIASTINUM */
"344"  = "1" /* EXCISION OR DESTRUCTION OF LESION OF CHEST WALL */
"3451" = "1" /* DECORTICATION OF LUNG */
"3452" = "1" /* THORACOSCOPC DECORT LUNG */
"3459" = "1" /* OTHER EXCISION OF PLEURA */
"3471" = "1" /* SUTURE OF LACERATION OF CHEST WALL */
"3472" = "1" /* CLOSURE OF THORACOSTOMY */
"3473" = "1" /* CLOSURE OF OTHER FISTULA OF THORAX */
"3474" = "1" /* REPAIR OF PECTUS DEFORMITY */
"3479" = "1" /* OTHER REPAIR OF CHEST WALL */
"3481" = "1" /* EXCISION OF LESION OR TISSUE OF DIAPHRAGM */
"3482" = "1" /* SUTURE OF LACERATION OF DIAPHRAGM */
"3483" = "1" /* CLOSURE OF FISTULA OF DIAPHRAGM */
"3484" = "1" /* OTHER REPAIR OF DIAPHRAGM */
"3485" = "1" /* IMPLANTATION OF DIAPHRAGMATIC PACEMAKER */
"3489" = "1" /* OTHER OPERATIONS ON DIAPHRAGM */
"3493" = "1" /* REPAIR OF PLEURA */
"3499" = "1" /* OTHER */
"4061" = "1" /* CANNULATION OF THORACIC DUCT */
"4062" = "1" /* FISTULIZATION OF THORACIC DUCT */
"4063" = "1" /* CLOSURE OF FISTULA OF THORACIC DUCT */
"4064" = "1" /* LIGATION OF THORACIC DUCT */
"4069" = "1" /* OTHER OPERATIONS ON THORACIC DUCT */
"4201" = "1" /* INCISION OF ESOPHAGEAL WEB */
"4209" = "1" /* OTHER INCISION OF EXOPHAGUS */
"4210" = "1" /* ESOPHAGOSTOMY, NOS */
"4211" = "1" /* CERVICAL ESOPHAGOSTOMY */
"4212" = "1" /* EXTERIORIZATION OF ESOPHAGEAL POUCH */
"4219" = "1" /* OTHER EXTERNAL FISTULIZATION OF ESOPHAGUS */
"4221" = "1" /* OPERATIVE ESOPHAGOSCOPY BY INCISION */
"4225" = "1" /* OPEN BIOPSY OF ESOPHAGUS */
"4231" = "1" /* LOCAL EXCISION OF ESOPHAGEAL DIVERTICULUM */
"4232" = "1" /* LOCAL EXCISION OF OTHER LESION OR TISSUE OF ESOPHAGUS */
"4239" = "1" /* OTHER DESTRUCTION OF LESION OR TISSUE OF ESOPHAGUS */
"4240" = "1" /* ESOPHAGECTOMY, NOS */
"4241" = "1" /* PARTIAL ESOPHAGECTOMY */
"4242" = "1" /* TOTAL ESOPHAGECTOMY */
"4251" = "1" /* INTRATHORACIC ESOPHAGOESOPHAGOSTOMY */
"4252" = "1" /* INTRATHORACIC ESOPHAGOGASTROSTOMY */
"4253" = "1" /* INTRATHORACIC ESOPHAGEAL ANASTOMOSIS WITH INTERPOSITION OF SMALL BOWEL */
"4254" = "1" /* OTHER INTRATHORACIC ESOPHAGOENTEROSOTOMY */
"4255" = "1" /* INTRATHORACIC ESOPHAGEAL ANASTOMOSIS WITH INTERPOSITION OF COLON */
"4256" = "1" /* OTHER INTRATHORACIC ESOPHAGOCOLOSTOMY */
"4258" = "1" /* INTRATHORACIC ESOPHAGEAL ANASTOMOSIS WITH OTHER INTERPOSITION */
"4259" = "1" /* OTHER INTRATHORACIC ANASTOMOSIS OF ESOPHAGUS */
"4261" = "1" /* ANTESTERNAL ESOPHAGOESOPHAGOSTOMY */
"4262" = "1" /* ANTESTERNAL ESOPHAGOGASTROSTOMY */
"4263" = "1" /* ANTESTERNAL ESOPHAGEAL ANASTOMOSIS WITH INTERPOSITION OF SMALL BOWEL */
"4264" = "1" /* OTHER ANTESTERNAL ESOPHAGOENTEROSTOMY */
"4265" = "1" /* ANTESTERNAL ESOPHAGEAL ANASTOMOSIS WITH INTERPOSITION OF COLON */
"4266" = "1" /* OTHER ANTESTERNAL ESOPHAGOCOLOSTOMY */
"4268" = "1" /* OTHER ANTESTERNAL ESOPHAGEAL ANASTOMOSIS WITH INTERPOSISION */
"4269" = "1" /* OTHER ANTESTERNAL ANASTOMOSIS OF ESOPHAGUS */
"427"  = "1" /* ESOPHAGOMYOTOMY */
"4281" = "1" /* INSERTION OF PERMANENT TUBE INTO ESOPHAGUS */
"4282" = "1" /* SUTURE OF LACERATION OF ESOPHAGUS */
"4283" = "1" /* CLOSURE OF ESOPHAGOSTOMY */
"4284" = "1" /* REPAIR OF ESOPHAGEAL FISTULA, NEC */
"4285" = "1" /* REPAIR OF ESOPHAGEAL STRICTURE */
"4286" = "1" /* PRODUCTION OF SUBCUTANEOUS TUNNEL WITHOUT ESOPHAGEAL ANASTOMOSIS */
"4287" = "1" /* OTHER GRAFT OF ESOPHAGUS */
"4289" = "1" /* OTHER REPAIR OF ESOPHAGUS */
"4465" = "1" /* ESOPHAGOGASTROPLASTY */
"4466" = "1" /* OTHER PROCEDURES FOR CREATION OF ESOPHAGOGASTRIC SPHINTERIC COMPETENCE */
"8104" = "1" /* DORSAL AND DORSO-LUMBAR FUSION, ANTERIOR TECHNIQUE */
"8134" = "1" /* REFUSION OF DORSAL AND DORSOLUMBAR SPINE, ANTERIOR TECHNIQUE */
OTHER  = "0" ;

 /* EXCLUDE: LUNG OR PLEURAL BIOPSY */
 /* ICD-9-CM PROCEDURE CODES: */

VALUE $LUNGBIP
"3326" = "1" /* CLOSED [PERCUTANEOUS] [NEEDLE] BIOPSY OF LUNG */
"3328" = "1" /* OPEN BIOPSY OF LUNG */
"3424" = "1" /* PLEURAL BIOPSY */
OTHER  = "0" ;

 /* EXCLUDE: DIAPHRAGMATIC SURGERY REPAIR */
 /* ICD-9-CM PROCEDURE CODES: */

VALUE $DIAPHRP
"537 " = "1" /* ABD REPAIR-DIAPHR HERNIA */
"5380" = "1" /* THOR REP-DIAPH HERN NOS */
"5381" = "1" /* DIAPHRAGMATIC PLICATION */
"5382" = "1" /* PARASTERN HERNIA REPAIR */
OTHER  = "0" ;

 /* EXCLUDE: CARDIAC SURGERY */
 /* DIAGNOSTIC RELATED GROUPS (DRGS): */

VALUE $CARDSDR
"103" = "1" /* HEART TRANSPLANT */
"104" = "1" /* CARDIAC VALVE AND OTHER MAJOR CARDIOTHORACIC PROCEDURES WITH CARDIAC CATHETERIZATION */
"105" = "1" /* CARDIAC VALVE AND OTHER MAJOR CARDIOTHORACIC PROCEDURES WITHOUT CARDIAC CATHETERIZATION */
"106" = "1" /* CORONARY BYPASS WITH PTCA */
"107" = "1" /* CORONARY BYPASS WITH CARDIAC CATHETERIZATION */
"108" = "1" /* OTHER CARDIOTHORACIC PROCEDURES */
"109" = "1" /* CORONARY BYPASS WITHOUT CARDIAC CATHETERIZATION */
"110" = "1" /* MAJOR CARDIOVASCULAR PROCEDURES WITH CC */
"111" = "1" /* MAJOR CARDIOVASCULAR PROCEDURES WITHOUT CC */
"525" = "1" /* HEART ASSIST SYSTEM IMPLANT */
"547" = "1" /* CORONARY BYPASS W CARDIAC CATH W MAJOR CV DX */
"548" = "1" /* CORONARY BYPASS W CARDIAC CATH W/O MAJOR CV DX */
"549" = "1" /* CORONARY BYPASS W/O CARDIAC CATH W MAJOR CV DX */
"550" = "1" /* CORONARY BYPASS W/O CARDIAC CATH W/O MAJOR CV DX */
OTHER = "0" ;

 /* ------------------------------- */
 /* INFECTION DUE TO MEDICAL CARE   */
 /*   PROVIDER INDICATOR / PSI #  7 */
 /*   AREA     INDICATOR / PSI # 23 */
 /* ------------------------------- */

 /* ICD-9-CM DIAGNOSIS CODES: */;

VALUE $IDTMCID
"99662" = "1" /* DUE TO OTHER VASCULAR DEVICE, IMPLANT, AND GRAFT */
"9993"  = "1" /* OTHER INFECTION */
"99931" = "1" /* INFECT DT CENT VEN CATH */
OTHER   = "0" ;

 /* ------------------------------- */
 /* POST-OPERATIVE HIP FRACTURE     */
 /*   PROVIDER INDICATOR / PSI # 8  */
 /* ------------------------------- */

/* ICD-9-CM DIAGNOSIS CODES: (INCLUDES ALL 5TH DIGITS) */

VALUE $HIPFXID
"82000" = "1" /* FX FEMUR INTRCAPS NOS-CL */
"82001" = "1" /* FX UP FEMUR EPIPHY-CLOS */
"82002" = "1" /* FX FEMUR, MIDCERVIC-CLOS */
"82003" = "1" /* FX BASE FEMORAL NCK-CLOS */
"82009" = "1" /* FX FEMUR INTRCAPS NEC-CL */
"82010" = "1" /* FX FEMUR INTRCAP NOS-OPN */
"82011" = "1" /* FX UP FEMUR EPIPHY-OPEN */
"82012" = "1" /* FX FEMUR, MIDCERVIC-OPEN */
"82013" = "1" /* FX BASE FEMORAL NCK-OPEN */
"82019" = "1" /* FX FEMUR INTRCAP NEC-OPN */
"82020" = "1" /* TROCHANTERIC FX NOS-CLOS */
"82021" = "1" /* INTERTROCHANTERIC FX-CL */
"82022" = "1" /* SUBTROCHANTERIC FX-CLOSE */
"82030" = "1" /* TROCHANTERIC FX NOS-OPEN */
"82031" = "1" /* INTERTROCHANTERIC FX-OPN */
"82032" = "1" /* SUBTROCHANTERIC FX-OPEN */
"8208"  = "1" /* UNSPECIFIED PART OF NECK OF FEMUR, CLOSED */
"8209"  = "1" /* UNSPECIFIED PART OF NECK OF FEMUR, OPEN */
OTHER   = "0" ;

 /* EXCLUDE: HIP FRACTURE REPAIR AS ONLY OR PROCEDURE */
 /* ICD-9-CM PROCEDURE CODES: */

VALUE $HIPFXIP
"7855"  = "1" /* INTERNAL FIXATION-FEMUR */
"7915"  = "1" /* CLOSED RED-INT FIX FEMUR */
"7925"  = "1" /* OPEN REDUCTION-FEMUR FX */
"7935"  = "1" /* OPEN REDUC-INT FIX FEMUR */
"8151"  = "1" /* TOTAL HIP REPLACEMENT */
"8152"  = "1" /* PARTIAL HIP REPLACEMENT */
OTHER   = "0" ;

 /* EXCLUDE: SEIZURE */
 /* ICD-9-CM DIAGNOSIS CODES: */

VALUE $SEIZUID
"34500" = "1" /* GENERALIZED NONCONVULSIVE EPILEPSY - WITHOUT MENTION OF INTRACTABLE EPILEPSY */
"34501" = "1" /* GENERALIZED NONCONVULSIVE EPILEPSY - WITH INTRACTABLE EPILEPSY */
"34510" = "1" /* GENERALIZED CONVULSIVE EPILEPSY - WITHOUT MENTION OF INTRACTABLE EPILEPSY */
"34511" = "1" /* GENERALIZED CONVULSIVE EPILEPSY - WITH INTRACTABLE EPILEPSY */
"3452"  = "1" /* EPILEPSY-PETIT MAL STATUS */
"3453"  = "1" /* EPILEPSY-GRAND MAL STATUS */
"34540" = "1" /* PARTIAL EPILEPSY, WITH IMPAIRMENT OF CONSCIOUSNESS - WITH INTRACTABLE EPILEPSY */
"34541" = "1" /* PARTIAL EPILEPSY, WITH IMPAIRMENT OF CONSCIOUSNESS - WITHOUT MENTION OF INTRACTABLE EPILEPSY */
"34550" = "1" /* PARTIAL EPILEPSY, WITHOUT MENTION OF IMPAIRMENT OF CONSCIOUSNESS, - WITHOUT MENTION OF INTRACTABLE EPILEPSY */
"34551" = "1" /* PARTIAL EPILEPSY, WITHOUT MENTION OF IMPAIRMENT OF CONSCIOUSNESS-WITH INTRACTABLE EPILEPSY */
"34560" = "1" /* INFANTILE SPASMS - WITHOUT MENTION OF INTRACTABLE EPILEPSY */
"34561" = "1" /* INFANTILE SPASMS - WITH INTRACTABLE EPILEPSY */
"34570" = "1" /* EPILEPSIA PARTIALIS CONTINUA - WITHOUT MENTION OF INTRACTABLE EPILEPSY */
"34571" = "1" /* EPILEPSIA PARTIALIS CONTINUA - WITH INTRACTABLE EPILEPSY */
"34580" = "1" /* OTHER FORMS OF EPILEPSY - WITHOUT MENTION OF INTRACTABLE EPILEPSY */
"34581" = "1" /* OTHER FORMS OF EPILEPSY - WITH INTRACTABLE EPILEPSY */
"34590" = "1" /* EPILEPSY, UNSPECIFIED - WITHOUT MENTION OF INTRACTABLE EPILEPSY */
"34591" = "1" /* EPILEPSY, UNSPECIFIED - WITH INTRACTABLE EPILEPSY */
"78031" = "1" /* FEBRILE CONVULSIONS */
"78032" = "1" /* COMPLX FEBRILE CONVULSNS */ 
"78039" = "1" /* OTHER CONVULSIONS */
"7803"  = "1" /* CONVULSIONS (OLD CODE NO LONGER VALID) */
OTHER   = "0" ;

 /* EXCLUDE: SYNCOPE */
 /* ICD-9-CM DIAGNOSIS CODES: */

VALUE $SYNCOID
"7802"  = "1" /* SYNCOPE AND COLLAPSE */
OTHER   = "0" ;

 /* EXCLUDE: STROKE */
 /* ICD-9-CM DIAGNOSIS CODES: */

VALUE $STROKID
"430"   = "1" /* SUBARACHNOID HEMORRHAGE */
"431"   = "1" /* INTRACEREBRAL HEMORRHAGE */
"4320"  = "1" /* NONTRAUMATIC EXTRADURAL HEMORRHAGE */
"4321"  = "1" /* SUBDURAL HEMORRHAGE */
"4329"  = "1" /* UNSPECIFIED INTRACRANIAL HEMORRHAGE */
"43301" = "1" /* BASILAR ARTERY, WITH CEREBRAL INFARCTION */
"43311" = "1" /* CAROTID ARTERY, WITH CEREBRAL INFARCTION */
"43321" = "1" /* VERTEBRAL ARTERY - WITH CEREBRAL INFARCTION */
"43331" = "1" /* MULTIPLE AND BILATERAL WITH CEREBRAL INFARCTION */
"43381" = "1" /* OTHER SPECIFIED PRECEBRAL ARTERY WITH CEREBRAL INFARCTION */
"43391" = "1" /* OCCLUSION AND STENOSIS OF PRECEREBRAL ARTERIES, UNSPECIFIED PRECEREBRAL ARTERY WITH CEREBRAL INFARCTION */
"43401" = "1" /* CEREBRAL THROMBOSIS - WITH CEREBRAL INFARCTION */
"43411" = "1" /* CEREBRAL EMBOLISM - WITH CEREBRAL INFARCTION */
"43491" = "1" /* CEREBRAL ARTERY OCCLUSION, UNSPECIFIED - WITH CEREBRAL INFARCTION */
"436"   = "1" /* ACUTE, BUT ILL-DEFINED CEREBROVASCULAR DISEASE */
"99702" = "1" /* POSTOPERATIVE CEREBROVASCULAR ACCIDENT */
OTHER   = "0" ;

 /* EXCLUDE: COMA */
 /* ICD-9-CM DIAGNOSIS CODES: */

VALUE $COMAID
"2510"  = "1" /* OTHER DISORDERS OF PANCREATIC INTERNAL SECRETION, HYPOGLYCEMIC COMA */
"5722"  = "1" /* LIVER ABSCESS AND SEQUELAE OF CHRONIC LIVER DISEASE, HEPATIC COMA */
"78001" = "1" /* GENERAL SYMPTOMS, ALTERATION OF CONSCIOUSNESS, COMA */
"25020" = "1" /* DIABETES WITH HYPEROSMOLARITY, TYPE 2 [NONINSULIN DEPENDENT TYPE][NIDDM TYPE][ADULT-ONSET] OR UNSPECIFIED TYPE, NOT STATED   AS UNCONTROLLED */
"25021" = "1" /* DIABETES WITH HYPEROSMOLARITY, TYPE 1 [INSULIN DEPENDENT TYPE] [NIDDM-TYPE] [JUVENILE TYPE], NOT STATED AS UNCONTROLLED */
"25022" = "1" /* DIABETES WITH HYPEROSMOLARITY, TYPE 2 */
"25023" = "1" /* DIABETES MELLITUS, DIABETES WITH HYPEROSMOLARITY , TYPE 1 [INSULIN DEPENDENT TYPE][NIDMM-TYPE][JUVENILE TYPE]   UNCONTROLLED */
"25030" = "1" /* DIABETES WITH OTHER COMA, TYPE 2 NOT STATED AS UNCONTROLLED */
"25031" = "1" /* DIABETES WITH OTHER COMA, TYPE 1 NOT STATED AS UNCONTROLLED */
"25032" = "1" /* DIABETES MELLITUS, DIABETES WITH OTHER COMA, TYPE 11 UNCONTROLLED */
"25033" = "1" /* DIABETES MELLITUS, DIABETES WITH OTHER COMA, TYPE 1 UNCONTROLLED */
"78003" = "1" /* GENERAL SYMPTOMS, ALTERATION OF CONSCIOUSNESS PERSISTENT VEGETATIVE STATE */
OTHER   = "0" ;

 /* EXCLUDE: POISONING */
 /* ICD-9-CM DIAGNOSIS CODES ( INCLUDES 4TH AND 5TH DIGITS): */

VALUE $POISOID
"9600"  = "1" /* POISONING-PENICILLINS */
"9601"  = "1" /* POIS-ANTIFUNGAL ANTIBIOT */
"9602"  = "1" /* POISON-CHLORAMPHENICOL */
"9603"  = "1" /* POIS-ERYTHROMYC/MACROLID */
"9604"  = "1" /* POISONING-TETRACYCLINE */
"9605"  = "1" /* POIS-CEPHALOSPORIN GROUP */
"9606"  = "1" /* POIS-ANTIMYCOBAC ANTIBIO */
"9607"  = "1" /* POIS-ANTINEOP ANTIBIOTIC */
"9608"  = "1" /* POISONING-ANTIBIOTIC NEC */
"9609"  = "1" /* POISONING-ANTIBIOTIC NOS */
"9610"  = "1" /* POISONING-SULFONAMIDES */
"9611"  = "1" /* POIS-ARSENIC ANTI-INFEC */
"9612"  = "1" /* POIS-HEAV MET ANTI-INFEC */
"9613"  = "1" /* POIS-QUINOLINE/HYDROXYQU */
"9614"  = "1" /* POISONING-ANTIMALARIALS */
"9615"  = "1" /* POIS-ANTIPROTOZ DRUG NEC */
"9616"  = "1" /* POISONING-ANTHELMINTICS */
"9617"  = "1" /* POISONING-ANTIVIRAL DRUG */
"9618"  = "1" /* POIS-ANTIMYCOBAC DRG NEC */
"9619"  = "1" /* POIS-ANTI-INFECT NEC/NOS */
"9620"  = "1" /* POIS-CORTICOSTEROIDS */
"9621"  = "1" /* POISONING-ANDROGENS */
"9622"  = "1" /* POISONING-OVARIAN HORMON */
"9623"  = "1" /* POISON-INSULIN/ANTIDIAB */
"9624"  = "1" /* POIS-ANT PITUITARY HORM */
"9625"  = "1" /* POIS-POST PITUITARY HORM */
"9626"  = "1" /* POISONING-PARATHYROIDS */
"9627"  = "1" /* POISONING-THYROID/DERIV */
"9628"  = "1" /* POISON-ANTITHYROID AGENT */
"9629"  = "1" /* POISONING HORMON NEC/NOS */
"9630"  = "1" /* POIS-ANTIALLRG/ANTIEMET */
"9631"  = "1" /* POIS-ANTINEOPL/IMMUNOSUP */
"9632"  = "1" /* POISONING-ACIDIFYING AGT */
"9633"  = "1" /* POISONING-ALKALIZING AGT */
"9634"  = "1" /* POISONING-ENZYMES NEC */
"9635"  = "1" /* POISONING-VITAMINS NEC */
"9638"  = "1" /* POISONING-SYSTEM AGT NEC */
"9639"  = "1" /* POISONING-SYSTEM AGT NOS */
"9640"  = "1" /* POISONING-IRON/COMPOUNDS */
"9641"  = "1" /* POISON-LIVER/ANTIANEMICS */
"9642"  = "1" /* POISONING-ANTICOAGULANTS */
"9643"  = "1" /* POISONING-VITAMIN K */
"9644"  = "1" /* POISON-FIBRINOLYSIS AGNT */
"9645"  = "1" /* POISONING-COAGULANTS */
"9646"  = "1" /* POISONING-GAMMA GLOBULIN */
"9647"  = "1" /* POISONING-BLOOD PRODUCT */
"9648"  = "1" /* POISONING-BLOOD AGT NEC */
"9649"  = "1" /* POISONING-BLOOD AGT NOS */
"96500" = "1" /* POISONING-OPIUM NOS */
"96501" = "1" /* POISONING-HEROIN */
"96502" = "1" /* POISONING-METHADONE */
"96509" = "1" /* POISONING-OPIATES NEC */
"9651"  = "1" /* POISONING-SALICYLATES */
"9654"  = "1" /* POIS-AROM ANALGESICS NEC */
"9655"  = "1" /* POISONING-PYRAZOLE DERIV */
"9656"  = "1" /* POISONING-ANTIRHEUMATICS */
"96561" = "1" /* POIS-PROPIONIC ACID DERV */
"96569" = "1" /* POISON-ANTIRHEUMATIC NEC */
"9657"  = "1" /* POIS-NO-NARC ANALGES NEC */
"9658"  = "1" /* POIS-ANALGES/ANTIPYR NEC */
"9659"  = "1" /* POIS-ANALGES/ANTIPYR NOS */
"9660"  = "1" /* POISON-OXAZOLIDINE DERIV */
"9661"  = "1" /* POISON-HYDANTOIN DERIVAT */
"9662"  = "1" /* POISONING-SUCCINIMIDES */
"9663"  = "1" /* POIS-ANTICONVUL NEC/NOS */
"9664"  = "1" /* POIS-ANTI-PARKINSON DRUG */
"9670"  = "1" /* POISONING-BARBITURATES */
"9671"  = "1" /* POISONING-CHLORAL HYDRAT */
"9672"  = "1" /* POISONING-PARALDEHYDE */
"9673"  = "1" /* POISONING-BROMINE COMPND */
"9674"  = "1" /* POISONING-METHAQUALONE */
"9675"  = "1" /* POISONING-GLUTETHIMIDE */
"9676"  = "1" /* POISON-MIX SEDATIVE NEC */
"9678"  = "1" /* POIS-SEDATIVE/HYPNOT NEC */
"9679"  = "1" /* POIS-SEDATIVE/HYPNOT NOS */
"9680"  = "1" /* POIS-CNS MUSCLE DEPRESS */
"9681"  = "1" /* POISONING-HALOTHANE */
"9682"  = "1" /* POISON-GAS ANESTHET NEC */
"9683"  = "1" /* POISON-INTRAVEN ANESTHET */
"9684"  = "1" /* POIS-GEN ANESTH NEC/NOS */
"9685"  = "1" /* POIS-TOPIC/INFILT ANESTH */
"9686"  = "1" /* POIS-NERVE/PLEX-BLK ANES */
"9687"  = "1" /* POISON-SPINAL ANESTHETIC */
"9689"  = "1" /* POIS-LOCAL ANEST NEC/NOS */
"9690"  = "1" /* POISONING-ANTIDEPRESSANT */
"9691"  = "1" /* POIS-PHENOTHIAZINE TRANQ */
"9692"  = "1" /* POIS-BUTYROPHENONE TRANQ */
"9693"  = "1" /* POISON-ANTIPSYCHOTIC NEC */
"9694"  = "1" /* POIS-BENZODIAZEPINE TRAN */
"9695"  = "1" /* POISON-TRANQUILIZER NEC */
"9696"  = "1" /* POISONING-HALLUCINOGENS */
"9697"  = "1" /* POISON-PSYCHOSTIMULANTS */
"9698"  = "1" /* POISON-PSYCHOTROPIC NEC */
"9699"  = "1" /* POISON-PSYCHOTROPIC NOS */
"9700"  = "1" /* POISONING-ANALEPTICS */
"9701"  = "1" /* POISON-OPIATE ANTAGONIST */
"9708"  = "1" /* POIS-CNS STIMULANTS NEC */
"9709"  = "1" /* POIS-CNS STIMULANT NOS */
"9710"  = "1" /* POIS-PARASYMPATHOMIMETIC */
"9711"  = "1" /* POIS-PARASYMPATHOLYTICS */
"9712"  = "1" /* POISON-SYMPATHOMIMETICS */
"9713"  = "1" /* POISONING-SYMPATHOLYTICS */
"9719"  = "1" /* POIS-AUTONOMIC AGENT NOS */
"9720"  = "1" /* POIS-CARD RHYTHM REGULAT */
"9721"  = "1" /* POISONING-CARDIOTONICS */
"9722"  = "1" /* POISONING-ANTILIPEMICS */
"9723"  = "1" /* POIS-GANGLION BLOCK AGT */
"9724"  = "1" /* POIS-CORONARY VASODILAT */
"9725"  = "1" /* POISON-VASODILATOR NEC */
"9726"  = "1" /* POIS-ANTIHYPERTEN AGENT */
"9727"  = "1" /* POISON-ANTIVARICOSE DRUG */
"9728"  = "1" /* POISON-CAPILLARY ACT AGT */
"9729"  = "1" /* POIS-CARDIOVASC AGT NEC */
"9730"  = "1" /* POIS-ANTACID/ANTIGASTRIC */
"9731"  = "1" /* POIS-IRRITANT CATHARTICS */
"9732"  = "1" /* POIS-EMOLLIENT CATHARTIC */
"9733"  = "1" /* POISONING-CATHARTIC NEC */
"9734"  = "1" /* POISONING-DIGESTANTS */
"9735"  = "1" /* POISONING-ANTIDIARRH AGT */
"9736"  = "1" /* POISONING-EMETICS */
"9738"  = "1" /* POISONING-GI AGENTS NEC */
"9739"  = "1" /* POISONING-GI AGENT NOS */
"9740"  = "1" /* POIS-MERCURIAL DIURETICS */
"9741"  = "1" /* POIS-PURINE DIURETICS */
"9742"  = "1" /* POIS-H2CO3 ANHYDRA INHIB */
"9743"  = "1" /* POISONING-SALURETICS */
"9744"  = "1" /* POISONING-DIURETICS NEC */
"9745"  = "1" /* POIS-ELECTRO/CAL/WAT AGT */
"9746"  = "1" /* POISON-MINERAL SALTS NEC */
"9747"  = "1" /* POIS-URIC ACID METABOL */
"9750"  = "1" /* POISONING-OXYTOCIC AGENT */
"9751"  = "1" /* POIS-SMOOTH MUSCLE RELAX */
"9752"  = "1" /* POIS-SKELET MUSCLE RELAX */
"9753"  = "1" /* POISON-MUSCLE AGENT NEC */
"9754"  = "1" /* POISONING-ANTITUSSIVES */
"9755"  = "1" /* POISONING-EXPECTORANTS */
"9756"  = "1" /* POIS-ANTI-COLD DRUGS */
"9757"  = "1" /* POISONING-ANTIASTHMATICS */
"9758"  = "1" /* POIS-RESPIR DRUG NEC/NOS */
"9760"  = "1" /* POIS-LOCAL ANTI-INFECT */
"9761"  = "1" /* POISONING-ANTIPRURITICS */
"9762"  = "1" /* POIS-LOC ASTRING/DETERG */
"9763"  = "1" /* POIS-EMOL/DEMUL/PROTECT */
"9764"  = "1" /* POISON-HAIR/SCALP PREP */
"9765"  = "1" /* POIS-EYE ANTI-INFEC/DRUG */
"9766"  = "1" /* POISON-ENT PREPARATION */
"9767"  = "1" /* POIS-TOPICAL DENTAL DRUG */
"9768"  = "1" /* POIS-SKIN/MEMBR AGNT NEC */
"9769"  = "1" /* POIS-SKIN/MEMBR AGNT NOS */
"9770"  = "1" /* POISONING-DIETETICS */
"9771"  = "1" /* POISON-LIPOTROPIC DRUGS */
"9772"  = "1" /* POISONING-ANTIDOTES NEC */
"9773"  = "1" /* POISON-ALCOHOL DETERRENT */
"9774"  = "1" /* POIS-PHARMACEUT EXCIPIEN */
"9778"  = "1" /* POISON-MEDICINAL AGT NEC */
"9779"  = "1" /* POISON-MEDICINAL AGT NOS */
"9780"  = "1" /* POISONING-BCG VACCINE */
"9781"  = "1" /* POIS-TYPH/PARATYPH VACC */
"9782"  = "1" /* POISONING-CHOLERA VACCIN */
"9783"  = "1" /* POISONING-PLAGUE VACCINE */
"9784"  = "1" /* POISONING-TETANUS VACCIN */
"9785"  = "1" /* POIS-DIPHTHERIA VACCINE */
"9786"  = "1" /* POIS-PERTUSSIS VACCINE */
"9788"  = "1" /* POIS-BACT VACCIN NEC/NOS */
"9789"  = "1" /* POIS-MIX BACTER VACCINES */
"9790"  = "1" /* POISON-SMALLPOX VACCINE */
"9791"  = "1" /* POISON-RABIES VACCINE */
"9792"  = "1" /* POISON-TYPHUS VACCINE */
"9793"  = "1" /* POIS-YELLOW FEVER VACCIN */
"9794"  = "1" /* POISONING-MEASLES VACCIN */
"9795"  = "1" /* POIS-POLIOMYELIT VACCINE */
"9796"  = "1" /* POIS-VIRAL/RICK VACC NEC */
"9797"  = "1" /* POISONING-MIXED VACCINE */
"9799"  = "1" /* POIS-VACCINE/BIOLOG NEC */
"E8500" = "1" /* ACC POISON-HEROIN */
"E8501" = "1" /* ACC POISON-METHADONE */
"E8502" = "1" /* ACC POISON-OPIATES NEC */
"E8503" = "1" /* ACC POISON-SALICYLATES */
"E8504" = "1" /* ACC POISON-AROM ANALGESC */
"E8505" = "1" /* ACC POISON-PYRAZOLE DERV */
"E8506" = "1" /* ACC POISON-ANTIRHEUMATIC */
"E8507" = "1" /* ACC POISON-NONNARC ANALG */
"E8508" = "1" /* ACC POISON-ANALGESIC NEC */
"E8509" = "1" /* ACC POISON-ANALGESIC NOS */
"E851"  = "1" /* ACC POISON-BARBITURATES */
"E8520" = "1" /* ACC POISN-CHLORL HYDRATE */
"E8521" = "1" /* ACC POISON-PARALDEHYDE */
"E8522" = "1" /* ACC POISON-BROMINE CMPND */
"E8523" = "1" /* ACC POISON-METHAQUALONE */
"E8524" = "1" /* ACC POISON-GLUTETHIMIDE */
"E8525" = "1" /* ACC POISON-MIX SEDTV NEC */
"E8528" = "1" /* ACC POISON-SEDATIVES NEC */
"E8529" = "1" /* ACC POISON-SEDATIVES NOS */
"E8530" = "1" /* ACC POIS-PHENTHIAZ TRANQ */
"E8531" = "1" /* ACC POIS-BUTYRPHEN TRANQ */
"E8532" = "1" /* ACC POISN-BENZDIAZ TRANQ */
"E8538" = "1" /* ACC POISN-TRANQUILZR NEC */
"E8539" = "1" /* ACC POISN-TRANQUILZR NOS */
"E8540" = "1" /* ACC POISON-ANTIDEPRESSNT */
"E8541" = "1" /* ACC POISON-HALLUCINOGENS */
"E8542" = "1" /* ACC POISN-PSYCHSTIMULANT */
"E8543" = "1" /* ACC POISON-CNS STIMULANT */
"E8548" = "1" /* ACC POISN PSYCHOTROP NEC */
"E8550" = "1" /* ACC POISN-ANTICONVULSANT */
"E8551" = "1" /* ACC POISN-CNS DEPRES NEC */
"E8552" = "1" /* ACC POISN-LOCAL ANESTHET */
"E8553" = "1" /* ACC POISON-CHOLINERGICS */
"E8554" = "1" /* ACC POISN-ANTICHOLINERG */
"E8555" = "1" /* ACC POISON-ADRENERGICS */
"E8556" = "1" /* ACC POISN-SYMPATHOLYTICS */
"E8558" = "1" /* ACC POISON-CNS DRUG NEC */
"E8559" = "1" /* ACC POISON-CNS DRUG NOS */
"E856"  = "1" /* ACC POISON-ANTIBIOTICS */
"E857"  = "1" /* ACC POIS-OTH ANTI-INFECT */
"E8580" = "1" /* ACC POISON-HORMONES */
"E8581" = "1" /* ACC POISN-SYSTEMIC AGENT */
"E8582" = "1" /* ACC POISON-BLOOD AGENT */
"E8583" = "1" /* ACC POISN-CARDIOVASC AGT */
"E8584" = "1" /* ACC POISON-GI AGENT */
"E8585" = "1" /* ACC POISN-METABOL AGNT */
"E8586" = "1" /* ACC POISN-MUSCL/RESP AGT */
"E8587" = "1" /* ACC POISN-SKIN/EENT AGNT */
"E8588" = "1" /* ACC POISONING-DRUG NEC */
"E8589" = "1" /* ACC POISONING-DRUG NOS */
"E8600" = "1" /* ACC POISN-ALCOHOL BEVRAG */
"E8601" = "1" /* ACC POISON-ETHYL ALCOHOL */
"E8602" = "1" /* ACC POISN-METHYL ALCOHOL */
"E8603" = "1" /* ACC POISN-ISOPROPYL ALC */
"E8604" = "1" /* ACC POISON-FUSEL OIL */
"E8608" = "1" /* ACC POISON-ALCOHOL NEC */
"E8609" = "1" /* ACC POISON-ALCOHOL NOS */
"E8610" = "1" /* ACC POIS-SYNTH DETERGENT */
"E8611" = "1" /* ACC POISON-SOAP PRODUCTS */
"E8612" = "1" /* ACC POISON-POLISHES */
"E8613" = "1" /* ACC POISON-CLEANSER NEC */
"E8614" = "1" /* ACC POISON-DISINFECTANTS */
"E8615" = "1" /* ACC POISON-LEAD PAINTS */
"E8616" = "1" /* ACC POISON-PAINTS NEC */
"E8619" = "1" /* ACC POISON-CLEANSER NOS */
"E8620" = "1" /* ACC POISN-PETROL SOLVENT */
"E8621" = "1" /* ACC POISN-PETROLEUM FUEL */
"E8622" = "1" /* ACC POIS-LUBRICATING OIL */
"E8623" = "1" /* ACC POIS-PETROLEUM SOLID */
"E8624" = "1" /* ACC POISN-SOLVENTS NEC */
"E8629" = "1" /* ACC POISN-SOLVENT NOS */
"E8630" = "1" /* ACC POIS-CHLORINE PESTIC */
"E8631" = "1" /* ACC POIS-PHOSPH PESTICID */
"E8632" = "1" /* ACC POISON-CARBAMATES */
"E8633" = "1" /* ACC POISN-MIXED PESTICID */
"E8634" = "1" /* ACC POISON-PESTICIDE NEC */
"E8635" = "1" /* ACC POISON-HERBICIDES */
"E8636" = "1" /* ACC POISON-FUNGICIDES */
"E8637" = "1" /* ACC POISON-RODENTICIDES */
"E8638" = "1" /* ACC POISON-FUMIGANTS */
"E8639" = "1" /* ACC POIS-AGRCULT NEC/NOS */
"E8640" = "1" /* ACC POIS-CORROSIV AROMAT */
"E8641" = "1" /* ACC POISON-ACIDS */
"E8642" = "1" /* ACC POISN-CAUSTIC ALKALI */
"E8643" = "1" /* ACC POISON-CAUSTIC NEC */
"E8644" = "1" /* ACC POISON-CAUSTIC NOS */
"E8650" = "1" /* ACC POISON-MEAT */
"E8651" = "1" /* ACC POISON-SHELLFISH */
"E8652" = "1" /* ACC POISON-FISH NEC */
"E8653" = "1" /* ACC POISON-BERRIES/SEEDS */
"E8654" = "1" /* ACC POISON-PLANTS NEC */
"E8655" = "1" /* ACC POISON-MUSHROOMS */
"E8658" = "1" /* ACC POISON-FOOD NEC */
"E8659" = "1" /* ACC POISN-FOOD/PLANT NOS */
"E8660" = "1" /* ACC POISONING-LEAD */
"E8661" = "1" /* ACC POISONING-MERCURY */
"E8662" = "1" /* ACC POISONING-ANTIMONY */
"E8663" = "1" /* ACC POISONING-ARSENIC */
"E8664" = "1" /* ACC POISON-METALS NEC */
"E8665" = "1" /* ACC POISON-PLANT FOOD */
"E8666" = "1" /* ACC POISON-GLUES */
"E8667" = "1" /* ACC POISON-COSMETICS */
"E8668" = "1" /* ACC POIS-SOLID/LIQ NEC */
"E8669" = "1" /* ACC POIS-SOLID/LIQ NOS */
"E867"  = "1" /* ACC POISON-PIPED GAS */
"E8680" = "1" /* ACC POIS-LIQ PETROL GAS */
"E8681" = "1" /* ACC POIS-UTL GAS NEC/NOS */
"E8682" = "1" /* ACC POISON-EXHAUST GAS */
"E8683" = "1" /* ACC POIS-CO/DOMESTC FUEL */
"E8688" = "1" /* ACC POIS-CARBN MONOX NEC */
"E8689" = "1" /* ACC POIS-CARBN MONOX NOS */
"E8690" = "1" /* ACC POISN-NITROGEN OXIDE */
"E8691" = "1" /* ACC POISN-SULFUR DIOXIDE */
"E8692" = "1" /* ACC POISON-FREON */
"E8693" = "1" /* ACC POISON-TEAR GAS */
"E8694" = "1" /* SCNDHND TBCCO SMOKE */
"E8698" = "1" /* ACC POISON-GAS/VAPOR NEC */
"E8699" = "1" /* ACC POISON-GAS/VAPOR NOS */
"E9510" = "1" /* POISON-PIPED GAS */
"E9511" = "1" /* POISON-GAS IN CONTAINER */
"E9518" = "1" /* POISON-UTILITY GAS NEC */
"E9520" = "1" /* POISON-EXHAUST GAS */
"E9521" = "1" /* POISON-CO NEC */
"E9528" = "1" /* POISON-GAS/VAPOR NEC */
"E9529" = "1" /* POISON-GAS/VAPOR NOS */
"E9620" = "1" /* ASSAULT-POIS W MEDIC AGT */
"E9621" = "1" /* ASSAULT-POIS W SOLID/LIQ */
"E9622" = "1" /* ASSAULT-POIS W GAS/VAPOR */
"E9629" = "1" /* ASSAULT-POISONING NOS */
"E9800" = "1" /* UNDETERM POIS-ANALGESICS */
"E9801" = "1" /* UNDETERM POIS-BARBITURAT */
"E9802" = "1" /* UNDET POIS-SED/HYPN NEC */
"E9803" = "1" /* UNDETERM POIS-PSYCHOTROP */
"E9804" = "1" /* UNDET POIS-MED AGNT NEC */
"E9805" = "1" /* UNDET POIS-MED AGNT NOS */
"E9806" = "1" /* UNDET POIS-CORROS/CAUST */
"E9807" = "1" /* UNDET POIS-AGRICULT AGNT */
"E9808" = "1" /* UNDETER POIS-ARSENIC */
"E9809" = "1" /* UNDETER POIS-SOL/LIQ NEC */
"E9810" = "1" /* UNDETER POIS-PIPED GAS */
"E9811" = "1" /* UNDET POIS-CONTAINER GAS */
"E9818" = "1" /* UNDET POIS-UTIL GAS NEC */
"E9820" = "1" /* UNDETER POIS-EXHAUST GAS */
"E9821" = "1" /* UNDETERMIN POISON-CO NEC */
"E9828" = "1" /* UNDET POIS-GAS/VAPOR NEC */
"E9829" = "1" /* UNDET POIS-GAS/VAPOR NOS */
OTHER   = "0" ;

 /* EXCLUDE: DELIRIUM AND OTHER PSYCHOSES */
 /* ICD-9-CM DIAGNOSIS CODES (INCLUDES ALL 4TH AND 5TH DIGITS) */

VALUE $DELIRID
"2900"  = "1" /* SENILE DEMENTIA UNCOMP */
"29010" = "1" /* PRESENILE DEMENTIA */
"29011" = "1" /* PRESENILE DELIRIUM */
"29012" = "1" /* PRESENILE DELUSION */
"29013" = "1" /* PRESENILE DEPRESSION */
"29020" = "1" /* SENILE DELUSION */
"29021" = "1" /* SENILE DEPRESSIVE */
"2903"  = "1" /* SENILE DELIRIUM */
"29040" = "1" /* ARTERIOSCLER DEMENT NOS */
"29041" = "1" /* ARTERIOSCLER DELIRIUM */
"29042" = "1" /* ARTERIOSCLER DELUSION */
"29043" = "1" /* ARTERIOSCLER DEPRESSIVE */
"2908"  = "1" /* SENILE PSYCHOSIS NEC */
"2909"  = "1" /* SENILE PSYCHOT COND NOS */
"2910"  = "1" /* DELIRIUM TREMENS */
"2911"  = "1" /* ALCOHOL AMNESTIC SYND */
"2912"  = "1" /* ALCOHOLIC DEMENTIA NEC */
"2913"  = "1" /* ALCOHOL HALLUCINOSIS */
"2914"  = "1" /* PATHOLOGIC ALCOHOL INTOX */
"2915"  = "1" /* ALCOHOLIC JEALOUSY */
"2918"  = "1" /* ALCOHOLIC PSYCHOSIS NEC */
"29181" = "1" /* ALCOHOL WITHDRAWAL */
"29189" = "1" /* ALCOHOLIC PSYCHOSIS NEC */
"2919"  = "1" /* ALCOHOLIC PSYCHOSIS NOS */
"2920"  = "1" /* DRUG WITHDRAWAL SYNDROME */
"29211" = "1" /* DRUG PARANOID STATE */
"29212" = "1" /* DRUG HALLUCINOSIS */
"2922"  = "1" /* PATHOLOGIC DRUG INTOX */
"29281" = "1" /* DRUG-INDUCED DELIRIUM */
"29282" = "1" /* DRUG-INDUCED DEMENTIA */
"29283" = "1" /* DRUG AMNESTIC SYNDROME */
"29284" = "1" /* DRUG DEPRESSIVE SYNDROME */
"29285" = "1" /* DRUG INDUCED SLEEP DISOR */ 
"29289" = "1" /* DRUG MENTAL DISORDER NEC */
"2929"  = "1" /* DRUG MENTAL DISORDER NOS */
"2930"  = "1" /* ACUTE DELIRIUM */
"2931"  = "1" /* SUBACUTE DELIRIUM */
"29381" = "1" /* ORGANIC DELUSIONAL SYND */
"29382" = "1" /* ORGANIC HALLUCINOSIS SYN */
"29383" = "1" /* ORGANIC AFFECTIVE SYND */
"29384" = "1" /* ORGANIC ANXIETY SYNDROME */
"29389" = "1" /* TRANSIENT ORG MENTAL NEC */
"2939"  = "1" /* TRANSIENT ORG MENTAL NOS */
"2940"  = "1" /* AMNESTIC SYNDROME */
"2941"  = "1" /* DEMENTIA IN OTH DISEASES */
"29410" = "1" /* DEMENTIA W/O BEHAV DIST */
"29411" = "1" /* DEMENTIA W BEHAVIOR DIST */
"2948"  = "1" /* ORGANIC BRAIN SYND NEC */
"2949"  = "1" /* ORGANIC BRAIN SYND NOS */
"29500" = "1" /* SIMPL SCHIZOPHREN-UNSPEC */
"29501" = "1" /* SIMPL SCHIZOPHREN-SUBCHR */
"29502" = "1" /* SIMPLE SCHIZOPHREN-CHR */
"29503" = "1" /* SIMP SCHIZ-SUBCHR/EXACER */
"29504" = "1" /* SIMPL SCHIZO-CHR/EXACERB */
"29505" = "1" /* SIMPL SCHIZOPHREN-REMISS */
"29510" = "1" /* HEBEPHRENIA-UNSPEC */
"29511" = "1" /* HEBEPHRENIA-SUBCHRONIC */
"29512" = "1" /* HEBEPHRENIA-CHRONIC */
"29513" = "1" /* HEBEPHREN-SUBCHR/EXACERB */
"29514" = "1" /* HEBEPHRENIA-CHR/EXACERB */
"29515" = "1" /* HEBEPHRENIA-REMISSION */
"29520" = "1" /* CATATONIA-UNSPEC */
"29521" = "1" /* CATATONIA-SUBCHRONIC */
"29522" = "1" /* CATATONIA-CHRONIC */
"29523" = "1" /* CATATONIA-SUBCHR/EXACERB */
"29524" = "1" /* CATATONIA-CHR/EXACERB */
"29525" = "1" /* CATATONIA-REMISSION */
"29530" = "1" /* PARANOID SCHIZO-UNSPEC */
"29531" = "1" /* PARANOID SCHIZO-SUBCHR */
"29532" = "1" /* PARANOID SCHIZO-CHRONIC */
"29533" = "1" /* PARAN SCHIZO-SUBCHR/EXAC */
"29534" = "1" /* PARAN SCHIZO-CHR/EXACERB */
"29535" = "1" /* PARANOID SCHIZO-REMISS */
"29540" = "1" /* AC SCHIZOPHRENIA-UNSPEC */
"29541" = "1" /* AC SCHIZOPHRENIA-SUBCHR */
"29542" = "1" /* AC SCHIZOPHRENIA-CHR */
"29543" = "1" /* AC SCHIZO-SUBCHR/EXACERB */
"29544" = "1" /* AC SCHIZOPHR-CHR/EXACERB */
"29545" = "1" /* AC SCHIZOPHRENIA-REMISS */
"29550" = "1" /* LATENT SCHIZOPHREN-UNSP */
"29551" = "1" /* LAT SCHIZOPHREN-SUBCHR */
"29552" = "1" /* LATENT SCHIZOPHREN-CHR */
"29553" = "1" /* LAT SCHIZO-SUBCHR/EXACER */
"29554" = "1" /* LATENT SCHIZO-CHR/EXACER */
"29555" = "1" /* LAT SCHIZOPHREN-REMISS */
"29560" = "1" /* RESID SCHIZOPHREN-UNSP */
"29561" = "1" /* RESID SCHIZOPHREN-SUBCHR */
"29562" = "1" /* RESIDUAL SCHIZOPHREN-CHR */
"29563" = "1" /* RESID SCHIZO-SUBCHR/EXAC */
"29564" = "1" /* RESID SCHIZO-CHR/EXACERB */
"29565" = "1" /* RESID SCHIZOPHREN-REMISS */
"29570" = "1" /* SCHIZOAFFECTIVE-UNSPEC */
"29571" = "1" /* SCHIZOAFFECTIVE-SUBCHR */
"29572" = "1" /* SCHIZOAFFECTIVE-CHRONIC */
"29573" = "1" /* SCHIZOAFF-SUBCHR/EXACER */
"29574" = "1" /* SCHIZOAFFECT-CHR/EXACER */
"29575" = "1" /* SCHIZOAFFECTIVE-REMISS */
"29580" = "1" /* SCHIZOPHRENIA NEC-UNSPEC */
"29581" = "1" /* SCHIZOPHRENIA NEC-SUBCHR */
"29582" = "1" /* SCHIZOPHRENIA NEC-CHR */
"29583" = "1" /* SCHIZO NEC-SUBCHR/EXACER */
"29584" = "1" /* SCHIZO NEC-CHR/EXACERB */
"29585" = "1" /* SCHIZOPHRENIA NEC-REMISS */
"29590" = "1" /* SCHIZOPHRENIA NOS-UNSPEC */
"29591" = "1" /* SCHIZOPHRENIA NOS-SUBCHR */
"29592" = "1" /* SCHIZOPHRENIA NOS-CHR */
"29593" = "1" /* SCHIZO NOS-SUBCHR/EXACER */
"29594" = "1" /* SCHIZO NOS-CHR/EXACERB */
"29595" = "1" /* SCHIZOPHRENIA NOS-REMISS */
"29600" = "1" /* MANIC DISORDER-UNSPEC */
"29601" = "1" /* MANIC DISORDER-MILD */
"29602" = "1" /* MANIC DISORDER-MOD */
"29603" = "1" /* MANIC DISORDER-SEVERE */
"29604" = "1" /* MANIC DIS-SEVERE W PSYCH */
"29605" = "1" /* MANIC DIS-PARTIAL REMISS */
"29606" = "1" /* MANIC DIS-FULL REMISSION */
"29610" = "1" /* RECUR MANIC DIS-UNSPEC */
"29611" = "1" /* RECUR MANIC DIS-MILD */
"29612" = "1" /* RECUR MANIC DIS-MOD */
"29613" = "1" /* RECUR MANIC DIS-SEVERE */
"29614" = "1" /* RECUR MANIC-SEV W PSYCHO */
"29615" = "1" /* RECUR MANIC-PART REMISS */
"29616" = "1" /* RECUR MANIC-FULL REMISS */
"29620" = "1" /* DEPRESS PSYCHOSIS-UNSPEC */
"29621" = "1" /* DEPRESS PSYCHOSIS-MILD */
"29622" = "1" /* DEPRESSIVE PSYCHOSIS-MOD */
"29623" = "1" /* DEPRESS PSYCHOSIS-SEVERE */
"29624" = "1" /* DEPR PSYCHOS-SEV W PSYCH */
"29625" = "1" /* DEPR PSYCHOS-PART REMISS */
"29626" = "1" /* DEPR PSYCHOS-FULL REMISS */
"29630" = "1" /* RECURR DEPR PSYCHOS-UNSP */
"29631" = "1" /* RECURR DEPR PSYCHOS-MILD */
"29632" = "1" /* RECURR DEPR PSYCHOS-MOD */
"29633" = "1" /* RECUR DEPR PSYCH-SEVERE */
"29634" = "1" /* REC DEPR PSYCH-PSYCHOTIC */
"29635" = "1" /* RECUR DEPR PSYC-PART REM */
"29636" = "1" /* RECUR DEPR PSYC-FULL REM */
"29640" = "1" /* BIPOL AFF, MANIC-UNSPEC */
"29641" = "1" /* BIPOLAR AFF, MANIC-MILD */
"29642" = "1" /* BIPOLAR AFFEC, MANIC-MOD */
"29643" = "1" /* BIPOL AFF, MANIC-SEVERE */
"29644" = "1" /* BIPOL MANIC-SEV W PSYCH */
"29645" = "1" /* BIPOL AFF MANIC-PART REM */
"29646" = "1" /* BIPOL AFF MANIC-FULL REM */
"29650" = "1" /* BIPOLAR AFF, DEPR-UNSPEC */
"29651" = "1" /* BIPOLAR AFFEC, DEPR-MILD */
"29652" = "1" /* BIPOLAR AFFEC, DEPR-MOD */
"29653" = "1" /* BIPOL AFF, DEPR-SEVERE */
"29654" = "1" /* BIPOL DEPR-SEV W PSYCH */
"29655" = "1" /* BIPOL AFF DEPR-PART REM */
"29656" = "1" /* BIPOL AFF DEPR-FULL REM */
"29660" = "1" /* BIPOL AFF, MIXED-UNSPEC */
"29661" = "1" /* BIPOLAR AFF, MIXED-MILD */
"29662" = "1" /* BIPOLAR AFFEC, MIXED-MOD */
"29663" = "1" /* BIPOL AFF, MIXED-SEVERE */
"29664" = "1" /* BIPOL MIXED-SEV W PSYCH */
"29665" = "1" /* BIPOL AFF, MIX-PART REM */
"29666" = "1" /* BIPOL AFF, MIX-FULL REM */
"2967"  = "1" /* BIPOLAR AFFECTIVE NOS */
"29680" = "1" /* MANIC-DEPRESSIVE NOS */
"29681" = "1" /* ATYPICAL MANIC DISORDER */
"29682" = "1" /* ATYPICAL DEPRESSIVE DIS */
"29689" = "1" /* MANIC-DEPRESSIVE NEC */
"29690" = "1" /* AFFECTIVE PSYCHOSIS NOS */
"29699" = "1" /* AFFECTIVE PSYCHOSES NEC */
"2970"  = "1" /* PARANOID STATE, SIMPLE */
"2971"  = "1" /* PARANOIA */
"2972"  = "1" /* PARAPHRENIA */
"2973"  = "1" /* SHARED PARANOID DISORDER */
"2978"  = "1" /* PARANOID STATES NEC */
"2979"  = "1" /* PARANOID STATE NOS */
"2980"  = "1" /* REACT DEPRESS PSYCHOSIS */
"2981"  = "1" /* EXCITATIV TYPE PSYCHOSIS */
"2982"  = "1" /* REACTIVE CONFUSION */
"2983"  = "1" /* ACUTE PARANOID REACTION */
"2984"  = "1" /* PSYCHOGEN PARANOID PSYCH */
"2988"  = "1" /* REACT PSYCHOSIS NEC/NOS */
"2989"  = "1" /* PSYCHOSIS NOS */
"29900" = "1" /* INFANTILE AUTISM-ACTIVE */
"29901" = "1" /* INFANTILE AUTISM-RESID */
"29910" = "1" /* DISINTEGR PSYCH-ACTIVE */
"29911" = "1" /* DISINTEGR PSYCH-RESIDUAL */
"29980" = "1" /* CHILD PSYCHOS NEC-ACTIVE */
"29981" = "1" /* CHILD PSYCHOS NEC-RESID */
"29990" = "1" /* CHILD PSYCHOS NOS-ACTIVE */
"29991" = "1" /* CHILD PSYCHOS NOS-RESID */
OTHER   = "0" ;

 /* EXCLUDE: ANOXIC BRAIN INJURY */
 /* ICD-9-CM DIAGNOSIS CODES: */

VALUE $ANOXIID
"3481"  = "1" /* ANOXIC BRAIN DAMAGE */
OTHER   = "0" ;

 /* EXCLUDE: METASTATIC CANCER */
 /* ICD-9-CM DIAGNOSIS CODES: */

VALUE $METACID
"1960"  = "1" /* MAL NEO LYMPH-HEAD/NECK */
"1961"  = "1" /* MAL NEO LYMPH-INTRATHOR */
"1962"  = "1" /* MAL NEO LYMPH INTRA-ABD */
"1963"  = "1" /* MAL NEO LYMPH-AXILLA/ARM */
"1965"  = "1" /* MAL NEO LYMPH-INGUIN/LEG */
"1966"  = "1" /* MAL NEO LYMPH-INTRAPELV */
"1968"  = "1" /* MAL NEO LYMPH NODE-MULT */
"1969"  = "1" /* MAL NEO LYMPH NODE NOS */
"1970"  = "1" /* SECONDARY MALIG NEO LUNG */
"1971"  = "1" /* SEC MAL NEO MEDIASTINUM */
"1972"  = "1" /* SECOND MALIG NEO PLEURA */
"1973"  = "1" /* SEC MALIG NEO RESP NEC */
"1974"  = "1" /* SEC MALIG NEO SM BOWEL */
"1975"  = "1" /* SEC MALIG NEO LG BOWEL */
"1976"  = "1" /* SEC MAL NEO PERITONEUM */
"1977"  = "1" /* SECOND MALIG NEO LIVER */
"1978"  = "1" /* SEC MAL NEO GI NEC */
"1980"  = "1" /* SECOND MALIG NEO KIDNEY */
"1981"  = "1" /* SEC MALIG NEO URIN NEC */
"1982"  = "1" /* SECONDARY MALIG NEO SKIN */
"1983"  = "1" /* SEC MAL NEO BRAIN/SPINE */
"1984"  = "1" /* SEC MALIG NEO NERVE NEC */
"1985"  = "1" /* SECONDARY MALIG NEO BONE */
"1986"  = "1" /* SECOND MALIG NEO OVARY */
"1987"  = "1" /* SECOND MALIG NEO ADRENAL */
"19881" = "1" /* SECOND MALIG NEO BREAST */
"19882" = "1" /* SECOND MALIG NEO GENITAL */
"19889" = "1" /* SECONDARY MALIG NEO NEC */
"1990"  = "1" /* MALIGNANT NEOPLASM WITHOUT SPECIFICATION OF SITE, DISSEMINATED */
OTHER   = "0" ;

 /* EXCLUDE: LYMPHOID MALIGNANCY */
 /* ICD-9-CM DIAGNOSIS CODES INCLUDES 4TH AND 5TH DIGITS: */

VALUE $LYMPHID
"20000" = "1" /* RETCLSRC UNSP XTRNDL ORG */
"20001" = "1" /* RETICULOSARCOMA HEAD */
"20002" = "1" /* RETICULOSARCOMA THORAX */
"20003" = "1" /* RETICULOSARCOMA ABDOM */
"20004" = "1" /* RETICULOSARCOMA AXILLA */
"20005" = "1" /* RETICULOSARCOMA INGUIN */
"20006" = "1" /* RETICULOSARCOMA PELVIC */
"20007" = "1" /* RETICULOSARCOMA SPLEEN */
"20008" = "1" /* RETICULOSARCOMA MULT */
"20010" = "1" /* LYMPHSRC UNSP XTRNDL ORG */
"20011" = "1" /* LYMPHOSARCOMA HEAD */
"20012" = "1" /* LYMPHOSARCOMA THORAX */
"20013" = "1" /* LYMPHOSARCOMA ABDOM */
"20014" = "1" /* LYMPHOSARCOMA AXILLA */
"20015" = "1" /* LYMPHOSARCOMA INGUIN */
"20016" = "1" /* LYMPHOSARCOMA PELVIC */
"20017" = "1" /* LYMPHOSARCOMA SPLEEN */
"20018" = "1" /* LYMPHOSARCOMA MULT */
"20020" = "1" /* BRKT TMR UNSP XTRNDL ORG */
"20021" = "1" /* BURKITTS TUMOR HEAD */
"20022" = "1" /* BURKITTS TUMOR THORAX */
"20023" = "1" /* BURKITTS TUMOR ABDOM */
"20024" = "1" /* BURKITTS TUMOR AXILLA */
"20025" = "1" /* BURKITTS TUMOR INGUIN */
"20026" = "1" /* BURKITTS TUMOR PELVIC */
"20027" = "1" /* BURKITTS TUMOR SPLEEN */
"20028" = "1" /* BURKITTS TUMOR MULT */
"20080" = "1" /* OTH VARN UNSP XTRNDL ORG */
"20081" = "1" /* MIXED LYMPHOSARC HEAD */
"20082" = "1" /* MIXED LYMPHOSARC THORAX */
"20083" = "1" /* MIXED LYMPHOSARC ABDOM */
"20084" = "1" /* MIXED LYMPHOSARC AXILLA */
"20085" = "1" /* MIXED LYMPHOSARC INGUIN */
"20086" = "1" /* MIXED LYMPHOSARC PELVIC */
"20087" = "1" /* MIXED LYMPHOSARC SPLEEN */
"20088" = "1" /* MIXED LYMPHOSARC MULT */
"20100" = "1" /* HDGK PRG UNSP XTRNDL ORG */
"20101" = "1" /* HODGKINS PARAGRAN HEAD */
"20102" = "1" /* HODGKINS PARAGRAN THORAX */
"20103" = "1" /* HODGKINS PARAGRAN ABDOM */
"20104" = "1" /* HODGKINS PARAGRAN AXILLA */
"20105" = "1" /* HODGKINS PARAGRAN INGUIN */
"20106" = "1" /* HODGKINS PARAGRAN PELVIC */
"20107" = "1" /* HODGKINS PARAGRAN SPLEEN */
"20108" = "1" /* HODGKINS PARAGRAN MULT */
"20110" = "1" /* HDGK GRN UNSP XTRNDL ORG */
"20111" = "1" /* HODGKINS GRANULOM HEAD */
"20112" = "1" /* HODGKINS GRANULOM THORAX */
"20113" = "1" /* HODGKINS GRANULOM ABDOM */
"20114" = "1" /* HODGKINS GRANULOM AXILLA */
"20115" = "1" /* HODGKINS GRANULOM INGUIN */
"20116" = "1" /* HODGKINS GRANULOM PELVIC */
"20117" = "1" /* HODGKINS GRANULOM SPLEEN */
"20118" = "1" /* HODGKINS GRANULOM MULT */
"20120" = "1" /* HDGK SRC UNSP XTRNDL ORG */
"20121" = "1" /* HODGKINS SARCOMA HEAD */
"20122" = "1" /* HODGKINS SARCOMA THORAX */
"20123" = "1" /* HODGKINS SARCOMA ABDOM */
"20124" = "1" /* HODGKINS SARCOMA AXILLA */
"20125" = "1" /* HODGKINS SARCOMA INGUIN */
"20126" = "1" /* HODGKINS SARCOMA PELVIC */
"20127" = "1" /* HODGKINS SARCOMA SPLEEN */
"20128" = "1" /* HODGKINS SARCOMA MULT */
"20140" = "1" /* LYM-HST UNSP XTRNDL ORGN */
"20141" = "1" /* HODG LYMPH-HISTIO HEAD */
"20142" = "1" /* HODG LYMPH-HISTIO THORAX */
"20143" = "1" /* HODG LYMPH-HISTIO ABDOM */
"20144" = "1" /* HODG LYMPH-HISTIO AXILLA */
"20145" = "1" /* HODG LYMPH-HISTIO INGUIN */
"20146" = "1" /* HODG LYMPH-HISTIO PELVIC */
"20147" = "1" /* HODG LYMPH-HISTIO SPLEEN */
"20148" = "1" /* HODG LYMPH-HISTIO MULT */
"20150" = "1" /* NDR SCLR UNSP XTRNDL ORG */
"20151" = "1" /* HODG NODUL SCLERO HEAD */
"20152" = "1" /* HODG NODUL SCLERO THORAX */
"20153" = "1" /* HODG NODUL SCLERO ABDOM */
"20154" = "1" /* HODG NODUL SCLERO AXILLA */
"20155" = "1" /* HODG NODUL SCLERO INGUIN */
"20156" = "1" /* HODG NODUL SCLERO PELVIC */
"20157" = "1" /* HODG NODUL SCLERO SPLEEN */
"20158" = "1" /* HODG NODUL SCLERO MULT */
"20160" = "1" /* MXD CELR UNSP XTRNDL ORG */
"20161" = "1" /* HODGKINS MIX CELL HEAD */
"20162" = "1" /* HODGKINS MIX CELL THORAX */
"20163" = "1" /* HODGKINS MIX CELL ABDOM */
"20164" = "1" /* HODGKINS MIX CELL AXILLA */
"20165" = "1" /* HODGKINS MIX CELL INGUIN */
"20166" = "1" /* HODGKINS MIX CELL PELVIC */
"20167" = "1" /* HODGKINS MIX CELL SPLEEN */
"20168" = "1" /* HODGKINS MIX CELL MULT */
"20170" = "1" /* LYM DPLT UNSP XTRNDL ORG */
"20171" = "1" /* HODG LYMPH DEPLET HEAD */
"20172" = "1" /* HODG LYMPH DEPLET THORAX */
"20173" = "1" /* HODG LYMPH DEPLET ABDOM */
"20174" = "1" /* HODG LYMPH DEPLET AXILLA */
"20175" = "1" /* HODG LYMPH DEPLET INGUIN */
"20176" = "1" /* HODG LYMPH DEPLET PELVIC */
"20177" = "1" /* HODG LYMPH DEPLET SPLEEN */
"20178" = "1" /* HODG LYMPH DEPLET MULT */
"20190" = "1" /* HDGK DIS UNSP XTRNDL ORG */
"20191" = "1" /* HODGKINS DIS NOS HEAD */
"20192" = "1" /* HODGKINS DIS NOS THORAX */
"20193" = "1" /* HODGKINS DIS NOS ABDOM */
"20194" = "1" /* HODGKINS DIS NOS AXILLA */
"20195" = "1" /* HODGKINS DIS NOS INGUIN */
"20196" = "1" /* HODGKINS DIS NOS PELVIC */
"20197" = "1" /* HODGKINS DIS NOS SPLEEN */
"20198" = "1" /* HODGKINS DIS NOS MULT */
"202"   = "1" /* OTHER MALIGNANT NEOPLASMS OF LYMPHOID AND HISTIOCYTIC TISSUE */
"20200" = "1" /* NDLR LYM UNSP XTRNDL ORG */
"20201" = "1" /* NODULAR LYMPHOMA HEAD */
"20202" = "1" /* NODULAR LYMPHOMA THORAX */
"20203" = "1" /* NODULAR LYMPHOMA ABDOM */
"20204" = "1" /* NODULAR LYMPHOMA AXILLA */
"20205" = "1" /* NODULAR LYMPHOMA INGUIN */
"20206" = "1" /* NODULAR LYMPHOMA PELVIC */
"20207" = "1" /* NODULAR LYMPHOMA SPLEEN */
"20208" = "1" /* NODULAR LYMPHOMA MULT */
"20210" = "1" /* MYCS FNG UNSP XTRNDL ORG */
"20211" = "1" /* MYCOSIS FUNGOIDES HEAD */
"20212" = "1" /* MYCOSIS FUNGOIDES THORAX */
"20213" = "1" /* MYCOSIS FUNGOIDES ABDOM */
"20214" = "1" /* MYCOSIS FUNGOIDES AXILLA */
"20215" = "1" /* MYCOSIS FUNGOIDES INGUIN */
"20216" = "1" /* MYCOSIS FUNGOIDES PELVIC */
"20217" = "1" /* MYCOSIS FUNGOIDES SPLEEN */
"20218" = "1" /* MYCOSIS FUNGOIDES MULT */
"20220" = "1" /* SZRY DIS UNSP XTRNDL ORG */
"20221" = "1" /* SEZARYS DISEASE HEAD */
"20222" = "1" /* SEZARYS DISEASE THORAX */
"20223" = "1" /* SEZARYS DISEASE ABDOM */
"20224" = "1" /* SEZARYS DISEASE AXILLA */
"20225" = "1" /* SEZARYS DISEASE INGUIN */
"20226" = "1" /* SEZARYS DISEASE PELVIC */
"20227" = "1" /* SEZARYS DISEASE SPLEEN */
"20228" = "1" /* SEZARYS DISEASE MULT */
"20230" = "1" /* MLG HIST UNSP XTRNDL ORG */
"20231" = "1" /* MAL HISTIOCYTOSIS HEAD */
"20232" = "1" /* MAL HISTIOCYTOSIS THORAX */
"20233" = "1" /* MAL HISTIOCYTOSIS ABDOM */
"20234" = "1" /* MAL HISTIOCYTOSIS AXILLA */
"20235" = "1" /* MAL HISTIOCYTOSIS INGUIN */
"20236" = "1" /* MAL HISTIOCYTOSIS PELVIC */
"20237" = "1" /* MAL HISTIOCYTOSIS SPLEEN */
"20238" = "1" /* MAL HISTIOCYTOSIS MULT */
"20240" = "1" /* LK RTCTL UNSP XTRNDL ORG */
"20241" = "1" /* HAIRY-CELL LEUKEM HEAD */
"20242" = "1" /* HAIRY-CELL LEUKEM THORAX */
"20243" = "1" /* HAIRY-CELL LEUKEM ABDOM */
"20244" = "1" /* HAIRY-CELL LEUKEM AXILLA */
"20245" = "1" /* HAIRY-CELL LEUKEM INGUIN */
"20246" = "1" /* HAIRY-CELL LEUKEM PELVIC */
"20247" = "1" /* HAIRY-CELL LEUKEM SPLEEN */
"20248" = "1" /* HAIRY-CELL LEUKEM MULT */
"20250" = "1" /* LTR-SIWE UNSP XTRNDL ORG */
"20251" = "1" /* LETTERER-SIWE DIS HEAD */
"20252" = "1" /* LETTERER-SIWE DIS THORAX */
"20253" = "1" /* LETTERER-SIWE DIS ABDOM */
"20254" = "1" /* LETTERER-SIWE DIS AXILLA */
"20255" = "1" /* LETTERER-SIWE DIS INGUIN */
"20256" = "1" /* LETTERER-SIWE DIS PELVIC */
"20257" = "1" /* LETTERER-SIWE DIS SPLEEN */
"20258" = "1" /* LETTERER-SIWE DIS MULT */
"20260" = "1" /* MLG MAST UNSP XTRNDL ORG */
"20261" = "1" /* MAL MASTOCYTOSIS HEAD */
"20262" = "1" /* MAL MASTOCYTOSIS THORAX */
"20263" = "1" /* MAL MASTOCYTOSIS ABDOM */
"20264" = "1" /* MAL MASTOCYTOSIS AXILLA */
"20265" = "1" /* MAL MASTOCYTOSIS INGUIN */
"20266" = "1" /* MAL MASTOCYTOSIS PELVIC */
"20267" = "1" /* MAL MASTOCYTOSIS SPLEEN */
"20268" = "1" /* MAL MASTOCYTOSIS MULT */
"20280" = "1" /* OTH LYMP UNSP XTRNDL ORG */
"20281" = "1" /* LYMPHOMAS NEC HEAD */
"20282" = "1" /* LYMPHOMAS NEC THORAX */
"20283" = "1" /* LYMPHOMAS NEC ABDOM */
"20284" = "1" /* LYMPHOMAS NEC AXILLA */
"20285" = "1" /* LYMPHOMAS NEC INGUIN */
"20286" = "1" /* LYMPHOMAS NEC PELVIC */
"20287" = "1" /* LYMPHOMAS NEC SPLEEN */
"20288" = "1" /* LYMPHOMAS NEC MULT */
"20290" = "1" /* UNSP LYM UNSP XTRNDL ORG */
"20291" = "1" /* LYMPHOID MAL NEC HEAD */
"20292" = "1" /* LYMPHOID MAL NEC THORAX */
"20293" = "1" /* LYMPHOID MAL NEC ABDOM */
"20294" = "1" /* LYMPHOID MAL NEC AXILLA */
"20295" = "1" /* LYMPHOID MAL NEC INGUIN */
"20296" = "1" /* LYMPHOID MAL NEC PELVIC */
"20297" = "1" /* LYMPHOID MAL NEC SPLEEN */
"20298" = "1" /* LYMPHOID MAL NEC MULT */
"2030"  = "1" /* MULTIPLE MYELOMA */
"20300" = "1" /* MULT MYELM W/O REMISSION */
"20301" = "1" /* MULT MYELM W REMISSION */
"2031"  = "1" /* PLASMA CELL LEUKEMIA */
"20310" = "1" /* PLSM CELL LEUK W/O RMSON */
"20311" = "1" /* PLSM CELL LEUK W RMSON */
"2038"  = "1" /* IMMUNOPROLIFERAT NEO NEC */
"20380" = "1" /* OTH IMNPRFL NPL W/O RMSN */
"20381" = "1" /* OTH IMNPRFL NPL W RMSN */
"2040"  = "1" /* ACUTE LYMPHOID LEUKEMIA */
"20400" = "1" /* ACT LYM LEUK W/O RMSION */
"20401" = "1" /* ACT LYM LEUK W RMSION */
"2041"  = "1" /* CHR LYMPHOID LEUKEMIA */
"20410" = "1" /* CHR LYM LEUK W/O RMSION */
"20411" = "1" /* CHR LYM LEUK W RMSION */
"2042"  = "1" /* SUBAC LYMPHOID LEUKEMIA */
"20420" = "1" /* SBAC LYM LEUK W/O RMSION */
"20421" = "1" /* SBAC LYM LEUK W RMSION */
"2048"  = "1" /* LYMPHOID LEUKEMIA NEC */
"20480" = "1" /* OTH LYM LEUK W/O RMSION */
"20481" = "1" /* OTH LYM LEUK W RMSION */
"2049"  = "1" /* LYMPHOID LEUKEMIA NOS */
"20490" = "1" /* UNS LYM LEUK W/O RMSION */
"20491" = "1" /* UNS LYM LEUK W RMSION */
"2050"  = "1" /* ACUTE MYELOID LEUKEMIA */
"20500" = "1" /* ACT MYL LEUK W/O RMSION */
"20501" = "1" /* ACT MYL LEUK W RMSION */
"2051"  = "1" /* CHRONIC MYELOID LEUKEMIA */
"20510" = "1" /* CHR MYL LEUK W/O RMSION */
"20511" = "1" /* CHR MYL LEUK W RMSION */
"2052"  = "1" /* SUBACUT MYELOID LEUKEMIA */
"20520" = "1" /* SBAC MYL LEUK W/O RMSION */
"20521" = "1" /* SBAC MYL LEUK W RMSION */
"2053"  = "1" /* MYELOID SARCOMA */
"20530" = "1" /* MYL SRCOMA W/O RMSION */
"20531" = "1" /* MYL SRCOMA W RMSION */
"2058"  = "1" /* MYELOID LEUKEMIA NEC */
"20580" = "1" /* OTH MYL LEUK W/O RMSION */
"20581" = "1" /* OTH MYL LEUK W RMSION */
"2059"  = "1" /* MYELOID LEUKEMIA NOS */
"20590" = "1" /* UNS MYL LEUK W/O RMSION */
"20591" = "1" /* UNS MYL LEUK W RMSION */
"2060"  = "1" /* ACUTE MONOCYTIC LEUKEMIA */
"20600" = "1" /* ACT MONO LEUK W/O RMSION */
"20601" = "1" /* ACT MONO LEUK W RMSION */
"2061"  = "1" /* CHR MONOCYTIC LEUKEMIA */
"20610" = "1" /* CHR MONO LEUK W/O RMSION */
"20611" = "1" /* CHR MONO LEUK W RMSION */
"2062"  = "1" /* SUBAC MONOCYTIC LEUKEMIA */
"20620" = "1" /* SBAC MONO LEUK W/O RMSON */
"20621" = "1" /* SBAC MONO LEUK W RMSION */
"2068"  = "1" /* MONOCYTIC LEUKEMIA NEC */
"20680" = "1" /* OTH MONO LEUK W/O RMSION */
"20681" = "1" /* OTH MONO LEUK W RMSION */
"2069"  = "1" /* MONOCYTIC LEUKEMIA NOS */
"20690" = "1" /* UNS MONO LEUK W/O RMSION */
"20691" = "1" /* UNS MONO LEUK W RMSION */
"2070"  = "1" /* ACUTE ERYTHREMIA */
"20700" = "1" /* ACT ERTH/ERYLK W/O RMSON */
"20701" = "1" /* ACT ERTH/ERYLK W RMSON */
"2071"  = "1" /* CHRONIC ERYTHREMIA */
"20710" = "1" /* CHR ERYTHRM W/O REMISION */
"20711" = "1" /* CHR ERYTHRM W REMISION */
"2072"  = "1" /* MEGAKARYOCYTIC LEUKEMIA */
"20720" = "1" /* MGKRYCYT LEUK W/O RMSION */
"20721" = "1" /* MGKRYCYT LEUK W RMSION */
"2078"  = "1" /* SPECIFIED LEUKEMIA NEC */
"20780" = "1" /* OTH SPF LEUK W/O REMSION */
"20781" = "1" /* OTH SPF LEUK W REMSION */
"2080"  = "1" /* ACT LEUK UNS CL W/O RMSN */
"20800" = "1" /* ACT LEUK UNS CL W/O RMSN */
"20801" = "1" /* ACT LEUK UNS CL W RMSON */
"2081"  = "1" /* CHRONIC LEUKEMIA NOS */
"20810" = "1" /* CHR LEUK UNS CL W/O RMSN */
"20811" = "1" /* CHR LEUK UNS CL W RMSON */
"2082"  = "1" /* SUBACUTE LEUKEMIA NOS */
"20820" = "1" /* SBAC LEUK UNS CL W/O RMS */
"20821" = "1" /* SBAC LEUK UNS CL W RMSON */
"2088"  = "1" /* LEUKEMIA-UNSPEC CELL NEC */
"20880" = "1" /* OTH LEUK UNS CL W/O RMSN */
"20881" = "1" /* OTH LEUK UNS CL W RMSON */
"2089"  = "1" /* LEUKEMIA-UNSPEC CELL NOS */
"20890" = "1" /* LEUKEMIA NOS W/O REMSION */
"20891" = "1" /* LEUKEMIA NOS W REMISSION */
OTHER   = "0" ;

 /* EXCLUDE: BONE MALIGNANCY */
 /* ICD-9-CM DIAGNOSIS CODES (ALL 4TH AND 5TH DIGITS) : */

VALUE $BONEMID
"1700"  = "1" /* MAL NEO SKULL/FACE BONE */
"1701"  = "1" /* MALIGNANT NEO MANDIBLE */
"1702"  = "1" /* MALIG NEO VERTEBRAE */
"1703"  = "1" /* MAL NEO RIBS/STERN/CLAV */
"1704"  = "1" /* MAL NEO LONG BONES ARM */
"1705"  = "1" /* MAL NEO BONES WRIST/HAND */
"1706"  = "1" /* MAL NEO PELVIC GIRDLE */
"1707"  = "1" /* MAL NEO LONG BONES LEG */
"1708"  = "1" /* MAL NEO BONES ANKLE/FOOT */
"1709"  = "1" /* MALIG NEOPL BONE NOS */
OTHER   = "0" ;

 /* ------------------------------------- */
 /* POST-OPERATIVE HEMORRHAGE OR HEMATOMA */
 /*   PROVIDER INDICATOR / PSI # 9        */
 /* ------------------------------------- */

 /* POST-OPERATIVE HEMATOMA */
 /* ICD-9-CM DIAGNOSIS CODE: */

VALUE $POHMAID
"99812" = "1" /* HEMATOMA COMPLICATING A PROCEDURE */
OTHER   = "0" ;

 /* CONTROL OF POST-OPERATIVE HEMATOMA */
 /* ICD-9-CM PROCEDURE CODES: */

VALUE $HEMATIP
"1809" = "1" /* OTHER INCISION OF EXTERNAL EAR */
"540"  = "1" /* INCISION OF ABDOMINAL WALL */
"5412" = "1" /* REOPENING OF RECENT LAPAROTOMY SITE */
"5919" = "1" /* OTHER INCISION OF PERIVESICLE TISSUE */
"610"  = "1" /* INCISION AND DRAINAGE OF SCROTUM AND TUNICA AND VAGINALIS */
"6998" = "1" /* OTHER OPERATIONS ON SUPPORTING STRUCTURES OF UTERUS */
"7014" = "1" /* OTHER VAGINOTOMY */
"7109" = "1" /* OTHER INCISION OF VULVA AND PERINEUM */
"7591" = "1" /* EVACUATION OF OBSTETRICAL INCISIONAL HEMATOMA OF PERINEUM */
"7592" = "1" /* EVACUATION OF OTHER HEMATOMA OF VULVA OR VAGINA */
"8604" = "1" /* OTHER INCISION WITH DRAINAGE OF SKIN AND SUBCUTANEOUS TISSUE */
OTHER  = "0" ;

 /* POST-OPERATIVE HEMORRHAGE */
 /* ICD-9-CM DIAGNOSIS CODE: */

VALUE $POHMRID
"99811" = "1" /* HEMORRHAGE COMPLICATING A PROCEDURE */
OTHER   = "0" ;

 /* CONTROL OF POST-OPERATIVE HEMORRHAGE */
 /* ICD-9-CM PROCEDURE CODES: */

VALUE $HEMORIP
"287"  = "1" /* CONTROL OF HEMORRHAGE AFTER TONSILLECTOMY AND ADENOIDECTOMY */
"3880" = "1" /* OTHER SURGICAL OCCLUSION OF UNSPECIFIED SITE */
"3881" = "1" /* OTHER SURGICAL OCCLUSION OF INTRACRANIAL VESSELS */
"3882" = "1" /* OTHER SURGICAL OCCLUSION OF OTHER VESSELS OF HEAD AND NECK */
"3883" = "1" /* OTHER SURGICAL OCCLUSION OF UPPER LIMB VESSELS */
"3884" = "1" /* OTHER SURGICAL OCCLUSION OF AORTA, ABDOMINAL */
"3885" = "1" /* OTHER SURGICAL OCCLUSION OF THORACIC VESSEL */
"3886" = "1" /* OTHER SURGICAL OCCLUSION OF ABDOMINAL ARTERIES */
"3887" = "1" /* OTHER SURGICAL OCCLUSION OF VESSELS ABDOMINAL VEINS */
"3888" = "1" /* OTHER SURGICAL OCCLUSION OF LOWER LIMB ARTERIES */
"3889" = "1" /* OTHER SURGICAL OCCLUSION OF LOWER LIMB VEINS */
"3941" = "1" /* CONTROL OF HEMORRHAGE FOLLOWING VASCULAR SURGERY*/
"3998" = "1" /* CONTROL OF HEMORRHAGE NOS */
"4995" = "1" /* CONTROL OF (POSTOPERATIVE) HEMORRHAGE OF ANUS */
"5793" = "1" /* CONTROL OF (POSTOPERATIVE HEMORRHAGE OF BLADDER) */
"6094" = "1" /* CONTROL OF (POSTOPERATIVE) HEMORRHAGE OF PROSTATE */
OTHER  = "0" ;

 /* COMBINED */
 /* ICD-9-CM PROCEDURE CODES: */

VALUE $HEMIP
"1809" = "1" /* OTHER INCISION OF EXTERNAL EAR */
"287"  = "1" /* CONTROL OF HEMORRHAGE AFTER TONSILLECTOMY AND ADENOIDECTOMY */
"3880" = "1" /* OTHER SURGICAL OCCLUSION OF UNSPECIFIED SITE */
"3881" = "1" /* OTHER SURGICAL OCCLUSION OF INTRACRANIAL VESSELS */
"3882" = "1" /* OTHER SURGICAL OCCLUSION OF OTHER VESSELS OF HEAD AND NECK */
"3883" = "1" /* OTHER SURGICAL OCCLUSION OF UPPER LIMB VESSELS */
"3884" = "1" /* OTHER SURGICAL OCCLUSION OF AORTA, ABDOMINAL */
"3885" = "1" /* OTHER SURGICAL OCCLUSION OF THORACIC VESSEL */
"3886" = "1" /* OTHER SURGICAL OCCLUSION OF ABDOMINAL ARTERIES */
"3887" = "1" /* OTHER SURGICAL OCCLUSION OF VESSELS ABDOMINAL VEINS */
"3888" = "1" /* OTHER SURGICAL OCCLUSION OF LOWER LIMB ARTERIES */
"3889" = "1" /* OTHER SURGICAL OCCLUSION OF LOWER LIMB VEINS */
"3941" = "1" /* CONTROL OF HEMORRHAGE FOLLOWING VASCULAR SURGERY*/
"3998" = "1" /* CONTROL OF HEMORRHAGE NOS */
"4995" = "1" /* CONTROL OF (POSTOPERATIVE) HEMORRHAGE OF ANUS */
"540"  = "1" /* INCISION OF ABDOMINAL WALL */
"5412" = "1" /* REOPENING OF RECENT LAPAROTOMY SITE */
"5793" = "1" /* CONTROL OF (POSTOPERATIVE HEMORRHAGE OF BLADDER) */
"5919" = "1" /* OTHER INCISION OF PERIVESICLE TISSUE */
"6094" = "1" /* CONTROL OF (POSTOPERATIVE) HEMORRHAGE OF PROSTATE */
"610"  = "1" /* INCISION AND DRAINAGE OF SCROTUM AND TUNICA AND VAGINALIS */
"6998" = "1" /* OTHER OPERATIONS ON SUPPORTING STRUCTURES OF UTERUS */
"7014" = "1" /* OTHER VAGINOTOMY */
"7109" = "1" /* OTHER INCISION OF VULVA AND PERINEUM */
"7591" = "1" /* EVACUATION OF OBSTETRICAL INCISIONAL HEMATOMA OF PERINEUM */
"7592" = "1" /* EVACUATION OF OTHER HEMATOMA OF VULVA OR VAGINA */
"8604" = "1" /* OTHER INCISION WITH DRAINAGE OF SKIN AND SUBCUTANEOUS TISSUE */
OTHER  = "0" ;

 /* ----------------------------------------------------- */
 /* POST-OPERATIVE PHYSIOLOGIC AND METABOLIC DERANGEMENTS */
 /*   PROVIDER INDICATOR / PSI # 10                       */
 /* ----------------------------------------------------- */

 /* ICD-9-CM DIAGNOSIS CODES: */

VALUE $PHYSIDA
"25010" = "1" /* TYPE 2, OR UNSPECIFIED TYPE, NOT STATED AS UNCONTROLLED */
"25011" = "1" /* TYPE 1 NOT STATED AS UNCONTROLLED */
"25012" = "1" /* TYPE 2 OR UNSPECIFIED TYPE, UNCONTROLLED */
"25013" = "1" /* TYPE 1 UNCONTROLLED */
"25020" = "1" /* TYPE 2, OR UNSPECIFIED TYPE, NOT STATED AS UNCONTROLLED */
"25021" = "1" /* TYPE 1 NOT STATED AS UNCONTROLLED */
"25022" = "1" /* TYPE 2 OR UNSPECIFIED TYPE, UNCONTROLLED */
"25023" = "1" /* TYPE 1 UNCONTROLLED */
"25030" = "1" /* TYPE 2, OR UNSPECIFIED TYPE, NOT STATED AS UNCONTROLLED */
"25031" = "1" /* TYPE 1 NOT STATED AS UNCONTROLLED */
"25032" = "1" /* TYPE 2 OR UNSPECIFIED TYPE, UNCONTROLLED */
"25033" = "1" /* TYPE 1 UNCONTROLLED */
OTHER   = "0" ;

VALUE $PHYSIDB
"5845"  = "1" /* WITH LESION OF TUBULAR NECROSIS */
"5846"  = "1" /* WITH LESION OF RENAL CORTICAL NECROSIS */
"5847"  = "1" /* WITH LESION OF RENAL MEDULLARY [PAPILLARY] NECROSIS */
"5848"  = "1" /* WITH OTHER SPECIFIED PATHOLOGICAL LESION IN KIDNEY */
"5849"  = "1" /* ACUTE RENAL FAILURE, UNSPECIFIED */
"586"   = "1" /* RENAL FAILURE NOS */        
"9975"  = "1" /* SURG COMPL-URINARY TRACT */ 
OTHER   = "0" ;

 /* DIALYSIS */
 /* ICD-9-CM PROCEDURE CODES: */

VALUE $DIALYIP
"3995"  = "1" /* HEMODIALYSIS */
"5498"  = "1" /* PERITONEAL DIALYSIS */
OTHER   = "0" ;

 /* EXCLUDE: DIABETES */
 /* ICD-9-CM DIAGNOSIS CODES: */

VALUE $DIABEID
"25000" = "1" /* DMII WO CMP NT ST UNCNTR */
"25001" = "1" /* DMI WO CMP NT ST UNCNTRL */
"25002" = "1" /* DMII WO CMP UNCNTRLD */
"25003" = "1" /* DMI WO CMP UNCNTRLD */
"25010" = "1" /* DMII KETO NT ST UNCNTRLD */
"25011" = "1" /* DMI KETO NT ST UNCNTRLD */
"25012" = "1" /* DMII KETOACD UNCONTROLD */
"25013" = "1" /* DMI KETOACD UNCONTROLD */
"25020" = "1" /* DMII HPRSM NT ST UNCNTRL */
"25021" = "1" /* DMI HPRSM NT ST UNCNTRLD */
"25022" = "1" /* DMII HPROSMLR UNCONTROLD */
"25023" = "1" /* DMI HPROSMLR UNCONTROLD */
"25030" = "1" /* DMII O CM NT ST UNCNTRLD */
"25031" = "1" /* DMI O CM NT ST UNCNTRLD */
"25032" = "1" /* DMII OTH COMA UNCONTROLD */
"25033" = "1" /* DMI OTH COMA UNCONTROLD */
"25040" = "1" /* DMII RENL NT ST UNCNTRLD */
"25041" = "1" /* DMI RENL NT ST UNCNTRLD */
"25042" = "1" /* DMII RENAL UNCNTRLD */
"25043" = "1" /* DMI RENAL UNCNTRLD */
"25050" = "1" /* DMII OPHTH NT ST UNCNTRL */
"25051" = "1" /* DMI OPHTH NT ST UNCNTRLD */
"25052" = "1" /* DMII OPHTH UNCNTRLD */
"25053" = "1" /* DMI OPHTH UNCNTRLD */
"25060" = "1" /* DMII NEURO NT ST UNCNTRL */
"25061" = "1" /* DMI NEURO NT ST UNCNTRLD */
"25062" = "1" /* DMII NEURO UNCNTRLD */
"25063" = "1" /* DMI NEURO UNCNTRLD */
"25070" = "1" /* DMII CIRC NT ST UNCNTRLD */
"25071" = "1" /* DMI CIRC NT ST UNCNTRLD */
"25072" = "1" /* DMII CIRC UNCNTRLD */
"25073" = "1" /* DMI CIRC UNCNTRLD */
"25080" = "1" /* DMII OTH NT ST UNCNTRLD */
"25081" = "1" /* DMI OTH NT ST UNCNTRLD */
"25082" = "1" /* DMII OTH UNCNTRLD */
"25083" = "1" /* DMI OTH UNCNTRLD */
"25090" = "1" /* DMII UNSPF NT ST UNCNTRL */
"25091" = "1" /* DMI UNSPF NT ST UNCNTRLD */
"25092" = "1" /* DMII UNSPF UNCNTRLD */
"25093" = "1" /* DMI UNSPF UNCNTRLD */
OTHER   = "0" ;

 /* EXCLUDE: CHRONIC RENAL FAILURE */
 /* ICD-9-CM DIAGNOSIS CODES: */

VALUE $CRENLFD
"40301" = "1" /* MAL HYP KIDNEY W CHR KID */
"40311" = "1" /* BEN HYP KIDNEY W CHR KID */
"40391" = "1" /* HYP KIDNEY NOS W CHR KID */
"40402" = "1" /* MAL HY HRT/KID W CHR KID */
"40403" = "1" /* MAL HYP HRT/KID W HF/KID */
"40412" = "1" /* BEN HYP HT/KID W CHR KID */
"40413" = "1" /* BEN HYP HT/KID W HF/KID */
"40492" = "1" /* HYP HT/KID NOS W CHR KID */
"40493" = "1" /* HYP HRT/KID NOS W HF/KID */
"585  " = "1" /* CHRONIC RENAL FAILURE */
"5855 " = "1" /* CHRON KIDNEY DIS STAGE V */
"5856 " = "1" /* END STAGE RENAL DISEASE */
OTHER   = "0" ;

 /* EXCLUDE: CARDIAC ARRHYTHMIA */
 /* DIAGNOSTIC RELATED GROUPS (DRGS): */

VALUE $CARDRDR
"138" = "1" /* CARDIAC ARRHYTHMIA AND CONDUCTION DISORDERS WITH CC */
"139" = "1" /* CARDIAC ARRHYTHMIA AND CONDUCTION DISORDERS WITHOUT CC */
OTHER = "0" ;

 /* EXCLUDE: GASTROINTESTINAL (GI) HEMORRHAGE */
 /* ICD-9-CM DIAGNOSIS CODES: */

VALUE $GASTRID
"4560"  = "1" /* ESOPHAGEAL VARICES WITH BLEEDING */
"45620" = "1" /* ESOPHAGEAL VARICES IN DISEASES CLASSIFIED ELSEWHERE WITH BLEEDING */
"5307"  = "1" /* GASTROESOPHAGEAL LACERATION- HEMORRHAGE SYNDROME */
"53082" = "1" /* ESOPHAGEAL HEMORRHAGE */
"53100" = "1" /* GASTRIC ULCER ACUTE WITH HEMORRHAGE - WITHOUT MENTION OF OBSTRUCTION */
"53101" = "1" /* GASTRIC ULCER ACUTE WITH HEMORRHAGE - WITH OBSTRUCTION */
"53120" = "1" /* GASTRIC ULCER ACUTE WITH HEMORRHAGE AND PERFORATION - WITHOUT MENTION OF OBSTRUCTION */
"53121" = "1" /* GASTRIC ULCER, ACUTE WITH HEMORRHAGE AND PERFORATION - WITH OBSTRUCTION */
"53140" = "1" /* GASTRIC ULCER CHRONIC OR UNSPECIFIED WITH HEMORRHAGE -WITHOUT MENTION OF OBSTRUCTION */
"53141" = "1" /* GASTRIC ULCER CHRONIC OR UNSPECIFIED WITH HEMORRHAGE - WITH OBSTRUCTION */
"53160" = "1" /* GASTRIC ULCER CHRONIC OR UNSPECIFIED WITH HEMORRHAGE AND PERFORATION - WITHOUT MENTION OF OBSTRUCTION */
"53161" = "1" /* GASTRIC ULCER CHRONIC OR UNSPECIFIED WITH HEMORRHAGE AND PERFORATION - WITH OBSTRUCTION */
"53200" = "1" /* DUODENAL ULCER ACUTE WITH HEMORRHAGE - WITHOUT MENTION OF OBSTRUCTION */
"53201" = "1" /* DUODENAL ULCER ACUTE WITH HEMORRHAGE - WITH OBSTRUCTION */
"53220" = "1" /* DUODENAL ULCER ACUTE WITH HEMORRHAGE AND PERFORATION - WITHOUT MENTION OF OBSTRUCTION */
"53221" = "1" /* DUODENAL ULCER ACUTE WITH HEMORRHAGE AND PERFORATION - WITH OBSTRUCTION */
"53240" = "1" /* DUODENAL ULCER CHRONIC OR UNSPECIFIED WITH HEMORRHAGE - WITHOUT MENTION OF OBSTRUCTION */
"53241" = "1" /* DUODENAL ULCER CHRONIC OR UNSPECIFIED WITH HEMORRHAGE - WITH OBSTRUCTION */
"53260" = "1" /* DUODENAL ULCER CHRONIC OR UNSPECIFIED WITH HEMORRHAGE AND PERFORATION  WITHOUT MENTION OF OBSTRUCTION */
"53261" = "1" /* DUODENAL ULCER CHRONIC OR UNSPECIFIED WITH HEMORRHAGE AND PERFORATION - WITH OBSTRUCTION */
"53300" = "1" /* PEPTIC ULCER, SITE UNSPECIFIED ACUTE WITH HEMORRHAGE - WITHOUT MENTION OF OBSTRUCTION */
"53301" = "1" /* PEPTIC ULCER, SITE UNSPECIFIED, ACUTE WITH HEMORRHAGE - WITH OBSTRUCTION */
"53320" = "1" /* PEPTIC ULCER, SITE UNSPECIFIED, ACUTE WITH HEMORRHAGE AND PERFORATION - WITHOUT MENTION OF OBSTRUCTION */
"53321" = "1" /* PEPTIC ULCER, SITE UNSPECIFIED, ACUTE WITH HEMORRHAGE AND PERFORATION - WITH OBSTRUCTION */
"53340" = "1" /* PEPTIC ULCER, SITE UNSPECIFIED CHRONIC OR UNSPECIFIED WITH HEMORRHAGE - WITHOUT MENTION OF OBSTRUCTION */
"53341" = "1" /* PEPTIC ULCER, SITE UNSPECIFIED, CHRONIC OR UNSPECIFIED WITH HEMORRHAGE - WITH OBSTRUCTION */
"53360" = "1" /* PEPTIC ULCER, SITE UNSPECIFIED, CHRONIC OR UNSPECIFIED WITH HEMORRHAGE AND PERFORATION -WITHOUT MENTION OF OBSTRUCTION */
"53361" = "1" /* PEPTIC ULCER, SITE UNSPECIFIED, CHRONIC OR UNSPECIFIED WITH HEMORRHAGE AND PERFORATION - WITH OBSTRUCTION */
"53400" = "1" /* GASTROJEJUNAL ULCER, ACUTE WITH HEMORRHAGE - WITHOUT MENTION OF OBSTRUCTION */
"53401" = "1" /* GASTROJEJUNAL ULCER, ACUTE WITH HEMORRHAGE - WITH OBSTRUCTION */
"53420" = "1" /* GASTROJEJUNAL ULCER, ACUTE WITH HEMORRHAGE AND PERFORATION - WITHOUT MENTION OF OBSTRUCTION */
"53421" = "1" /* GASTROJEJUNAL ULCER, ACUTE WITH HEMORRHAGE AND PERFORATION - WITH OBSTRUCTION */
"53440" = "1" /* GASTROJEJUNAL ULCER, CHRONIC OR UNSPECIFIED WITH HEMORRHAGE - WITHOUT MENTION OF OBSTRUCTION */
"53441" = "1" /* GASTROJEJUNAL ULCER, CHRONIC OR UNSPECIFIED WITH HEMORRHAGE - WITH OBSTRUCTION */
"53460" = "1" /* GASTROJEJUNAL ULCER, CHRONIC OR UNSPECIFIED WITH HEMORRHAGE AND PERFORATION - WITHOUT MENTION OF OBSTRUCTION */
"53461" = "1" /* GASTROJEJUNAL ULCER, CHRONIC OR UNSPECIFIED WITH HEMORRHAGE AND PERFORATION - WITH OBSTRUCTION */
"53501" = "1" /* GASTRITIS AND DUODENITIS, ACUTE GASTRITIS WITH HEMORRHAGE */
"53511" = "1" /* GASTRITIS AND DUODENITIS, ATROPHIC GASTRITIS WITH HEMORRHAGE */
"53521" = "1" /* GASTRITIS AND DUODENITIS, GASTRIC MUCOSAL HYPERTROPHY, WITH HEMORRHAGE */
"53531" = "1" /* GASTRITIS AND DUODENITIS, ALCOHOLIC GASTRITIS, WITH HEMORRHAGE */
"53541" = "1" /* GASTRITIS AND DUODENITIS, OTHER SPECIFIED GASTRITIS - WITH HEMORRHAGE */
"53551" = "1" /* GASTRITIS AND DUODENITIS, UNSPECIFIED GASTRITIS AND GASTRODUODENITIS - WITH HEMORRHAGE */
"53561" = "1" /* GASTRITIS AND DUODENITIS, DUODENITIS - WITH HEMORRHAGE */
"53783" = "1" /* OTHER SPECIFIED DISORDERS OF STOMACH AND DUODENUM, ANGIODYSPLASIA OF STOMACH AND DUODENUM - WITH HEMORRHAGE */
"53784" = "1" /* DIEULAFOY LESION (HEMORRHAGIC) OF STOMACH AND DUODENUM */
"56202" = "1" /* DIVERTICULOSIS OF SMALL INTESTINE -WITH HEMORRHAGE */
"56203" = "1" /* DIVERTICULITIS OF SMALL INTESTINE -WITH HEMORRHAGE */
"56212" = "1" /* DIVERTICULOSIS OF COLON - WITH HEMORRHAGE */
"56213" = "1" /* DIVERTICULITIS OF COLON - WITH HEMORRHAGE */
"5693"  = "1" /* HEMORRHAGE OF RECTUM AND ANUS */
"56985" = "1" /* ANGIODYSPLASIA OF INTESTINE - WITH HEMORRHAGE */
"56986" = "1" /* DIEULAFOY LESION (HEMORRHAGIC) OF INTESTINE */
"5780"  = "1" /* GASTROINTESTINAL HEMORRHAGE, HEMATEMESIS */
"5781"  = "1" /* GASTROINTESTINAL HEMORRHAGE, BLOOD IN STOOL */
"5789"  = "1" /* GASTROINTESTINAL HEMORRHAGE, HEMORRHAGE OF GASTROINTESTINAL TRACT, UNSPECIFIED */
OTHER   = "0" ;

 /* ---------------------------------- */
 /* POST-OPERATIVE RESPIRATORY FAILURE */
 /*   PROVIDER INDICATOR / PSI # 11    */
 /* ---------------------------------- */

 /* ICD-9-CM DIAGNOSIS CODES: */

VALUE $ACURFID
"51881" = "1" /* ACUTE RESPIRATRY FAILURE */
"51884" = "1" /* ACUTE & CHRONC RESP FAIL */
OTHER   = "0" ;

 /* ICD-9-CM PROCEDURE CODES: */

VALUE $BLANK
OTHER  = "0" ;

VALUE $PR9604P
"9604" = "1" /* INSERT OF SENSTAKEN TUBE */
OTHER  = "0" ;

VALUE $PR9670P
"9670" = "1" /* CONTINUOUS MECHANICAL VENTILATION OF UNSPEC DURATION */
OTHER  = "0" ;

VALUE $PR9671P
"9671" = "1" /* CONTINUOUS MECHANICAL VENTILATION FOR LESS THAN 96 CONSECUTIVE HRS */
OTHER  = "0" ;

VALUE $PR9672P
"9672" = "1" /* CONTINUOUS MECHANICAL VENTILATION FOR 96 CONSECUTIVE HRS OR MORE*/
OTHER  = "0" ;

 /* EXCLUDE: NEUROMUSCULAR DISORDERS */;
 /* ICD-9-CM DIAGNOSIS CODES: */

VALUE $NEUROMD
"3570 " = "1" /* AC INFECT POLYNEURITIS */
"35800" = "1" /* MYSTHNA GRVS W/O AC EXAC */
"35801" = "1" /* MYASTHNA GRAVS W AC EXAC */
"3581 " = "1" /* MYASTHENIA IN OTH DIS */
"3582 " = "1" /* TOXIC MYONEURAL DISORDER */
"3588 " = "1" /* MYONEURAL DISORDERS NEC */
"3589 " = "1" /* MYONEURAL DISORDERS NOS */
"3590 " = "1" /* CONG HERED MUSC DYSTRPHY */
"3591 " = "1" /* HERED PROG MUSC DYSTRPHY */
"3592 " = "1" /* MYOTONIC DISORDERS */
"35922" = "1" /* MYOTONIA CONGENITA */
"35923" = "1" /* MYOTONIC CHONDRODYSTRPHY */
"3593 " = "1" /* FAMIL PERIODIC PARALYSIS */
"3594 " = "1" /* TOXIC MYOPATHY */
"3595 " = "1" /* MYOPATHY IN ENDOCRIN DIS */
"3596 " = "1" /* INFL MYOPATHY IN OTH DIS */
"35981" = "1" /* CRITICAL ILLNESS MYOPTHY */
"35989" = "1" /* MYOPATHIES NEC */
"3599 " = "1" /* MYOPATHY NOS */
OTHER   = "0" ;

 /* EXCLUDE: TRACHEOSTOMY */;
 /* ICD-9-CM PROCEDURE CODE: */;

VALUE $TRACHIP
"3121" = "1" /* MEDIASTINAL TRACHEOSTOMY */
"3129" = "1" /* OTHER PERM TRACHEOSTOMY */
"3174" = "1" /* REVISION OF TRACHEOSTOMY */
OTHER  = "0" ;

 /* --------------------------------------------------------- */
 /* POST-OPERATIVE PULMONARY EMBOLISM OR DEEP VEIN THROMBOSIS */
 /*   PROVIDER INDICATOR / PSI # 12                           */
 /* --------------------------------------------------------- */

 /* PULMONARY EMBOLISM */
 /* ICD-9-CM DIAGNOSIS CODES: */

VALUE $PULMOID
"4151"  = "1" /* PULMONARY EMBOLISM AND INFARCTION */
"41511" = "1" /* IAGTROGENIC PULMONARY EMBOLISM AND INFARCTION */
"41519" = "1" /* OTHER */
OTHER   = "0" ;

 /* DEEP VEIN THROMBOSIS */
 /* ICD-9-CM DIAGNOSIS CODES: */

VALUE $DEEPVID
"45111" = "1" /* PHLEBITIS AND THROMBOSIS OF FEMORAL VEIN (DEEP) (SUPERFICIAL) */
"45119" = "1" /* PHLEBITIS AND THROMBOPHLEBITIS - OF DEEP VESSEL OF LOWER EXTREMITIES - OTHER */
"4512"  = "1" /* PHLEBITIS AND THROMBOPHLEBITIS OF LOWER EXTREMITIES UNSPECIFIED */
"45181" = "1" /* PHLEBITIS AND THROMBOPHLEBITIS OF ILIAC VEIN */
"4519"  = "1" /* PHLEBITIS AND THROMBOPHLEBITIS OF OTHER SITES - OF UNSPECIFIED SITE */
"45340" = "1" /* DVT-EMBLSM LOWER EXT NOS */
"45341" = "1" /* DVT-EMB PROX LOWER EXT */
"45342" = "1" /* DVT-EMB DISTAL LOWER EXT */
"4538"  = "1" /* OTHER VENOUS EMBOLISM AND THROMBOSIS OF OTHER SPECIFIED VEINS */
"4539"  = "1" /* OTHER VENOUS EMBOLISM AND THROMBOSIS OF UNSPECIFIED SITE */
OTHER   = "0" ;

 /* EXCLUDE: INTERUPTION OF VENA CAVA */;
 /* ICD-9-CM PROCEDURE CODE: */;

VALUE $VENACIP
"387"  = "1" /* INTERUPTION OF VENA CAVA */
OTHER  = "0" ;

 /* EXCLUDE: HYPERCOAGULABLE STATE */;
 /* ICD-9-CM DIAGNOSIS CODE: */;

VALUE $HYPERID
"28981" = "1" /* PRIM HYPERCOAGULABLE ST */
"28982"  = "1" /* SEC HYPERCOAGULABLE ST */
OTHER  = "0" ;

 /* ------------------------------- */
 /* POST-OPERATIVE SEPSIS           */
 /*   PROVIDER INDICATOR / PSI # 13 */
 /* ------------------------------- */

 /* SEPTICEMIA */
 /* ICD-9-CM DIAGNOSIS CODES: */

VALUE $SEPTIID
"0380"  = "1" /* STREPTOCOCCAL SEPTICEMIA */
"0381"  = "1" /* STAPHYLOCOCCAL SEPTICEMIA */
"03810" = "1" /* STAPHYLOCOCCAL SEPTICEMIA, UNSPECIFIED */
"03811" = "1" /* STAPHYLOCOCCUS AUREUS SEPTICEMIA */
"03819" = "1" /* OTHER STAPHYLOCOCCAL SEPTICEMIA */
"0382"  = "1" /* PNEUMOCOCCAL SEPTICEMIA (STREPTOCOCCUS PNEUMONIAE SEPTICEMIA) */
"0383"  = "1" /* SEPTICEMIA DUE TO ANAEROBES */
"03840" = "1" /* GRAM-NEGATIVE ORGANISM, UNSPECIFIED */
"03841" = "1" /* HEMOPHILUS INFLUENZAE */
"03842" = "1" /* ESCHERICHIA COLI */
"03843" = "1" /* PSEUDOMONAS */
"03844" = "1" /* SERRATIA */
"03849" = "1" /* SEPTICEMIA DUE TO OTHER GRAM-NEGATIVE ORGANISMS */
"0388"  = "1" /* OTHER SPECIFIED SEPTICEMIAS */
"0389"  = "1" /* UNSPECIFIED SEPTICEMIA */
"78552" = "1" /* SEPTIC SHOCK */             
"78559" = "1" /* SHOCK W/O TRAUMA NEC */     
"99591" = "1" /* SYSTEMIC INFLAMMATORY RESPONSE SYNDROME DUE TO INFECTIOUS 
                 PROCESS WITHOUT ORGAN DYSFUNCTION */
"99592" = "1" /* SYSTEMIC INFLAMMATORY RESPONSE SYNDROME DUE TO INFECTIOUS
                 PROCESS WITH ORGAN DYSFUNCTION */
"9980"  = "1" /* POSTOPERATIVE SHOCK */      
OTHER   = "0" ;

 /* ------------------------------- */
 /* POST-OPERATIVE WOUND DEHISCENCE */
 /*   PROVIDER INDICATOR / PSI # 14 */
 /*   AREA     INDICATOR / PSI # 24 */
 /* ------------------------------- */

 /* ABDOMINOPELVIC */;
 /* ICD-9-CM PROCEDURE CODES: */;

VALUE $ABDOMIP
"3804" = "1" /* INCISION OF AORTA */
"3806" = "1" /* INCISION OF ABDOMINAL ARTERIES */
"3807" = "1" /* INCISION OF ABDOMINAL VEINS */
"3814" = "1" /* ENDARTERECTOMY OF AORTA */
"3816" = "1" /* ENDARTERECTOMY OF ABDOMINAL ARTERIES */
"3834" = "1" /* RESECTION OF AORTA WITH ANASTOMOSIS */
"3836" = "1" /* RESECTION OF ABDOMINAL ARTERIES WITH ANASTOMOSIS */
"3837" = "1" /* RESECTION OF ABDOMINAL VEINS WITH ANASTOMOSIS */
"3844" = "1" /* RESECTION OF AORTA, ABDOMINAL WITH REPLACEMENT */
"3846" = "1" /* RESECTION OF ABDOMINAL ARTERIES WITH REPLACEMENT */
"3847" = "1" /* RESECTION OF ABDOMINAL VEINS WITH REPLACEMENT */
"3857" = "1" /* LIGATION AND STRIPPING OF VARICOSE VEINS, ABDOMINCAL VEINS */
"3864" = "1" /* OTHER EXCISION OF AORTA, ABDOMINAL */
"3866" = "1" /* OTHER EXCISION OF ABDOMINAL ARTERIES */
"3867" = "1" /* OTHER EXCISION OF ABDOMINAL VEINS */
"3884" = "1" /* OTHER SURGICAL OCCLUSION OF AORTA, ABDOMINAL */
"3886" = "1" /* OTHER SURGICAL OCCLUSION OF ABDOMINAL ARTERIES */
"3887" = "1" /* OTHER SURGICAL OCCLUSION OF ABDOMINAL VEINS */
"391"  = "1" /* INTRA-ABDOMINAL VENOUS SHUNT */
"3924" = "1" /* AORTA-RENAL BYPASS */
"3925" = "1" /* AORTA-ILIAC-FEMORAL BYPASS */
"3926" = "1" /* OTHER INTRA-ABDOMINAL VASCULAR SHUNT OR BYPASS */
"4052" = "1" /* RADICAL EXCISION OF PERIAORTIC LYMPH NODES */
"4053" = "1" /* RADICAL EXCISION OF ILIAC LYMPH NODES */
"412"  = "1" /* SPLENOTOMY */
"4133" = "1" /* OPEN BIOPSY OF SPLEEN */
"4141" = "1" /* MARSUPIALIZATION OF SPLENIC CYST */
"4142" = "1" /* EXCISION OF LESION OR TISSUE OF SPLEEN */
"4143" = "1" /* PARTIAL SPLENECTOMY */
"415"  = "1" /* TOTAL SPLENECTOMY */
"4193" = "1" /* EXCISION OF ACCESSORY SPLEEN */
"4194" = "1" /* TRANSPLANTATION OF SPLEEN */
"4195" = "1" /* REPAIR AND PLASTIC OPERATIONS ON SPLEEN */
"4199" = "1" /* OTHER OPERATIONS ON SPLEEN */
"4240" = "1" /* ESOPHAGECTOMY, NOS */
"4241" = "1" /* PARTIAL EXOPHAGECTOMY */
"4242" = "1" /* TOTAL ESOPHAGECTOMY */
"4253" = "1" /* INTRATHORACIC ESOPHAGEAL ANASTOMOSIS WITH INTERPOSITION OF SMALL BOWEL */
"4254" = "1" /* OTHER INTRATHORACIC ESOPHAGOENTEROSTOMY */
"4255" = "1" /* INTRATHORACIC ESOPHAGEAL ANASTOMOSIS WITH INTERPOSITION OF COLON */
"4256" = "1" /* OTHER INTRATHORACIC ESOPHAGOCOLOSTOMY */
"4263" = "1" /* ANTESTERNAL ESOPHAGEAL ANASTOMOSIS WITH INTERPOSITION OF SMALL BOWEL */
"4264" = "1" /* OTHER ANTESTERNAL ESOPHAGOENTEROSTOMY */
"4265" = "1" /* ANTESTERNAL ESOPHAGEAL ANASTOMOSIS WITH INTERPOSITION OF COLON */
"4266" = "1" /* OTHER ANTESTERNAL ESOPHAGOCOLOSTOMY */
"4291" = "1" /* LIGATION OF ESOPHAGEAL VARICES */
"430"  = "1" /* GASTROSTOMY */
"433"  = "1" /* PYLOROMYOTOMY */
"4342" = "1" /* LOCAL EXCISION OF OTHER LESION OR TISSUE OF STOMACH */
"4349" = "1" /* OTHER DESTRUCTION OF LESION OR TISSUE OF STOMACH */
"435"  = "1" /* PARTIAL GASTRECTOMY WITH ANASTOMOSIS TO ESOPHAGUS */
"436"  = "1" /* PARTIAL GASTRECTOMY WITH ANASTOMOSIS TO DUODENUM */
"437"  = "1" /* PARTIAL GASTRECTOMY WITH ANASTOMOSIS TO JEJUNUM */
"4381" = "1" /* PARTIAL GASTRECTOMY WITH JEJUNA TRANSPOSITION */
"4389" = "1" /* OTHER PARTIAL GASTRECTOMY */
"4391" = "1" /* TOTAL GASTRECTOMY WITH INTESTINAL INTERPOSITION */
"4399" = "1" /* OTHER TOTAL GASTRECTOMY */
"4400" = "1" /* VAGOTOMY, NOS */
"4401" = "1" /* TRUNCAL VAGOTOMY */
"4402" = "1" /* HIGHLY SELECTIVE VAGOTOMY */
"4403" = "1" /* OTHER SELECTIVE VAGOTOMOY */
"4411" = "1" /* TRANSABDOMINAL GASTROSCOPY */
"4415" = "1" /* OPEN BIOPSY OF STOMACH */
"4421" = "1" /* DILATION OF PYLORUS BY INCISION */
"4429" = "1" /* OTHER PYLOROPLASTY */
"4431" = "1" /* HIGH GASTRIC BYPASS */
"4439" = "1" /* OTHER GASTROENTEROSTOMY */
"4440" = "1" /* SUTURE OF PEPTIC ULCER, NOS */
"4441" = "1" /* SUTURE OF GASTRIC ULCER SITE */
"4442" = "1" /* SUTURE OF DUODENAL ULCER SITE */
"445"  = "1" /* REVISION OF GASTRIC ANASTOMOSIS */
"4461" = "1" /* SUTURE OF LACERATION OF STOMACH */
"4463" = "1" /* CLOSURE OF OTHER GASTRIC FISTULA */
"4464" = "1" /* GASTROPEXY */
"4465" = "1" /* ESOPHAGOGASTROPLASTY */
"4466" = "1" /* OTHER PROCEDURES FOR CREATION OF ESOPHAGOGASTRIC SPHINCTERIC COMPETENCE */
"4469" = "1" /* OTHER REPAIR OF STOMACH */
"4491" = "1" /* LIGATION OF GASTRIC VARICES */
"4492" = "1" /* INTRAOPERATIVE MANIPULATION OF STOMACH */
"4500" = "1" /* INCISION OF INTESTINE, NOS */
"4501" = "1" /* INCISION OF DUODENUM */
"4502" = "1" /* OTHER INCISION OF SMALL INTESTINE */
"4503" = "1" /* INCISION OF LARGE INTESTINE */
"4531" = "1" /* OTHER LOCAL EXCISION OF LESION OF DUODENUM */
"4532" = "1" /* OTHER DESTRUCTION OF LESION OF DUODENUM */
"4533" = "1" /* LOCAL EXCISION OF LESION OR TISSUE OF SMALL INTESTINE, EXCEPT DUODENUM */
"4534" = "1" /* OTHER DESTRUCTION OF LESION OF SMALL INTESTINE, EXCEPT DUODENUM */
"4541" = "1" /* EXCISION OF LESION OR TISSUE OF LARGE INTESTINE */
"4549" = "1" /* OTHER DESTRUCTION OF LESION OF LARGE INTESTINE */
"4550" = "1" /* ISOLATION OF INTESTINAL SEGMENT, NOS */
"4551" = "1" /* ISOLATION OF SEGMENT OF SMALL INTESTINE */
"4552" = "1" /* ISOLATION OF SEGMENT OF LARGE INTESTINE */
"4561" = "1" /* MULTIPLE SEGMENTAL RESECTION OF SMALL INTESTINE */
"4562" = "1" /* OTHER PARTIAL RESECTION OF SMALL INTESTINE */
"4563" = "1" /* TOTAL REMOVAL OF SMALL INTESTINE */
"4571" = "1" /* MULTIPLE SEGMENTAL RESECTION OF LARGE INTESTINE */
"4572" = "1" /* CESECTOMY */
"4573" = "1" /* RIGHT HEMICOLECTOMY */
"4574" = "1" /* RESECTION OF TRANSVERSE COLON */
"4575" = "1" /* LEFT HEMICOLECTOMY */
"4576" = "1" /* SIGMOIDECTOMY */
"4579" = "1" /* OTHER PARTIAL EXCISION OF LARGE INTESTINE */
"458"  = "1" /* TOTAL INTRA-ABDOMINAL COLECTOMY */
"4590" = "1" /* INTESTINAL ANASTOMOSIS, NOS */
"4591" = "1" /* SMALL-TO-SMALL INTESTINAL ANASTOMOSIS */
"4592" = "1" /* ANASTOMOSIS OF SMALL INTESTINE TO RECTAL STUMP */
"4593" = "1" /* OTHER SMALL-TO-LARGE INTESTINAL ANASTOMOSIS */
"4594" = "1" /* LARGE-TO-LARGE INTESTINAL ANASTOMOSIS */
"4595" = "1" /* ANASTOMOSIS TO ANUS */
"4601" = "1" /* EXTERIORIZATION OF SMALL INTESTINE */
"4603" = "1" /* EXTERIORIZATION OF LARGE INTESTINE */
"4610" = "1" /* COLOSTOMY, NOS */
"4611" = "1" /* TEMPORARY COLOSTOMY */
"4613" = "1" /* PERMANENT COLOSTOMY */
"4620" = "1" /* ILEOSTOMY, NOS */
"4621" = "1" /* TEMPORARY ILESOSTOMY */
"4622" = "1" /* CONTINENT ILEOSTOMY */
"4623" = "1" /* OTHER PERMANENT ILEOSTOMY */
"4640" = "1" /* REVISION OF INTESTINA STOMA, NOS */
"4641" = "1" /* REVISION OF STOMA OF SMALL INTESTINE */
"4642" = "1" /* REPAIR OF PERICOLOSTOMY HERNIA */
"4643" = "1" /* OTHER REVISION OF STOMA OF LARGE INTESTINE */
"4650" = "1" /* CLOSURE OF INTESTINAL STOMA, NOS */
"4651" = "1" /* CLOSURE OF STOMA OF SMALL INTESTINE */
"4652" = "1" /* CLOSURE OF STOMA OF LARGE INTESTINE */
"4660" = "1" /* FIXATION OF INTESTINE, NOS */
"4661" = "1" /* FIXATION OF SMALL INTESTINE TO ABDOMINAL WALL */
"4662" = "1" /* OTHER FIXATION OF SMALL INTESTINE */
"4663" = "1" /* FIXATION OF LARGE INTESTINE TO ABDOMINAL WALL */
"4664" = "1" /* OTHER FIXATION OF LARGE INTESTINE */
"4672" = "1" /* CLOSURE OF FISTULA OF DUODENUM */
"4674" = "1" /* CLOSURE OF FISTULA OF SMALL INTESTINE, EXCEPT DUODENUM */
"4676" = "1" /* CLOSURE OF FISTULA OF LARGE INTESTINE */
"4680" = "1" /* INTRA-ABDOMINAL MANIPULATION OF INTESTINE, NOS */
"4681" = "1" /* INTRA-ABDOMINAL MANIPULATION OF SMALL INTESTINE */
"4682" = "1" /* INTRA-ABDOMINAL MANIPULATION OF LARGE INTESTINE */
"4691" = "1" /* MYOTOMY OF SIGMOID COLON */
"4692" = "1" /* MYOTOMY OF OTHER PARTS OF COLON */
"4693" = "1" /* REVISION OF ANASTOMOSIS OF SMALL INTESTINE */
"4694" = "1" /* REVISION OF ANASTOMOSIS OF LARGE INTESTINE */
"4699" = "1" /* OTHER OPERATIONS ON INTESTINES */
"4709" = "1" /* OTHER APPENDECTOMY */
"4719" = "1" /* OTHER INCIDENTAL APPENDECTOMY */
"472"  = "1" /* DRAINAGE OF APPENDICEAL ABSCESS */
"4791" = "1" /* APPENDECTOMY */
"4792" = "1" /* CLOSURE OF APPENDICEAL FISTULA */
"4799" = "1" /* OTHER */
"4841" = "1" /* SUBMUCOSAL RESECTION OF RECTUM */
"4849" = "1" /* OTHER PULL-THROUGH RESECTION OF RECTUM */
"485"  = "1" /* ABDOMINOPERINEAL RESECTION OF RECTUM */
"4875" = "1" /* ABDOMINAL PROCTOPEXY */
"500"  = "1" /* HEPATOTOMY */
"5012" = "1" /* OPEN BIOPSY OF LIVER */
"5021" = "1" /* MARSUPIALIZATION OF LESION OF LIVER */
"5022" = "1" /* PARTIAL HEPATECTOMY */
"5023" = "1" /* OPN ABLTN LIVER LES/TISS */ 
"5026" = "1" /* ABLTN LIVER LES/TISS NEC */ 
"5029" = "1" /* OTHER DESTRUCTION OF LESION OF LIVER */
"503"  = "1" /* LOBECTOMY OF LIVER */
"504"  = "1" /* TOTAL HEPATECTOMY */
"5051" = "1" /* AUXILIARY LIVER TRANSPLANT */
"5059" = "1" /* OTHER TRANSPLANT OF LIVER */
"5069" = "1" /* OTHER REPAIR OF LIVER */
"5103" = "1" /* OTHER CHOLECYSTOTSTOMY */
"5104" = "1" /* OTHER CHOLECYSTOTOMY */
"5113" = "1" /* OPEN BIOPSY OF GALLBLADDER OR BILE DUCTS */
"5121" = "1" /* OTHER PARTIAL CHOLECYSTECTOMY */
"5122" = "1" /* CHOLECYSTECTOMY */
"5131" = "1" /* ANASTOMOSIS OF GALLBLADDER TO HEPATIC DUCTS */
"5132" = "1" /* ANASTOMOSIS OF GALLBLADDER TO INTESTINE */
"5133" = "1" /* ANASTOMOSIS OF GALLBLADDER TO PANCREAS */
"5134" = "1" /* ANASTOMOSIS OF GALLBLADDER TO STOMACH */
"5135" = "1" /* OTHER GALLBLADDER ANASTOMOSIS */
"5136" = "1" /* CHOLEDOCHOENTEROSTOMY */
"5137" = "1" /* ANASTOMOSIS OF HEPATIC DUCT TO GASTROINTESTINAL TRACT */
"5139" = "1" /* OTHER BILE DUCT ANASTOMOSIS */
"5141" = "1" /* COMMON DUCT EXPLORATION FOR REMOVAL OF CALCULUS */
"5142" = "1" /* COMMON DUCT EXPLORATION FOR RELIEF OF OTHER OBSTRUCTION */
"5143" = "1" /* INSERTION OF CHOLEDOCHOHEPATIC TUBE FOR DECOMPRESSION */
"5149" = "1" /* INCISION OF OTHER BILE DUCTS FOR RELIEF OF OBSTRUCTION */
"5151" = "1" /* EXPLORATION OF COMMON DUCT */
"5159" = "1" /* INCISION OF OTHER BILE DUCT */
"5161" = "1" /* EXCISION OF CYSTIC DUCT REMNANT */
"5162" = "1" /* EXCISION OF AMPULLA OF VATER WITH REIMPLANTATION OF COMMON DUCT */
"5163" = "1" /* OTHER EXCISION OF COMMON DUCT */
"5169" = "1" /* EXCISION OF OTHER BILE DUCT */
"5171" = "1" /* SIMPLE SUTURE OF COMMON BILE DUCT */
"5172" = "1" /* CHOLEDOCHOPLASTY */
"5179" = "1" /* REPAIR OF OTHER BILE DUCTS */
"5181" = "1" /* DILATION OF SPHINCTER OF ODDI */
"5182" = "1" /* PANCREATIC SPHINCTEROTOMY */
"5183" = "1" /* PANCREATIC SPHINCTEROPLSTY */
"5189" = "1" /* OTHER OPERATIONS ON SPHINCTER OF ODDI */
"5192" = "1" /* CLOSURE OF CHOLECYSTOSTOMY */
"5193" = "1" /* CLOSURE OF OTHER BILIARY FISTULA */
"5194" = "1" /* REVISION OF ANASTOMOSIS OF BILIARY TRACT */
"5195" = "1" /* REMOVAL OF PROSTHETIC DEVICE FROM BILE DUCT */
"5199" = "1" /* OTHER OPERATIONS ON BILIARY TRACT */
"5201" = "1" /* DRAINAGE OF PANCREATIC CYST BY CATHETER */
"5209" = "1" /* OTHER PANCREATOTOMY */
"5212" = "1" /* OPEN BIOPSY OF PANCREAS */
"5222" = "1" /* OTHER EXCISION OR DESTRUCTION OF LESION OR TISSUE OF PANCREAS OR PANCREATIC DUCT */
"523"  = "1" /* GINGIVAL AND PERIODONTAL DISEASES */
"524"  = "1" /* INTERNAL DRAINAGE OF PANCREATIC CYST */
"5251" = "1" /* PROXIMAL PANCREATECTOMY */
"5252" = "1" /* DISTAL PANCREATECTOMY */
"5253" = "1" /* RADIAL SUBTOTAL PANCREATECTOMY */
"5259" = "1" /* OTHER PARTIAL PANCREATECTOMY */
"526"  = "1" /* TOTAL PANCREATECTOMY */
"527"  = "1" /* RADICAL PANCREATICODUODENCECTOMY */
"5280" = "1" /* PANCREATIC TRANSPLANT, NOS */
"5281" = "1" /* REIMPLANTATION */
"5282" = "1" /* HOMOTRANSPLANT OF PANCREAS */
"5283" = "1" /* HETEROTRANSPLANT OF PANCREAS */
"5292" = "1" /* CANNULATION OF PANCREATIC DUCT */
"5295" = "1" /* OTHER REPAIR OF PANCREAS */
"5296" = "1" /* ANASTOMOSIS OF PANCREAS */
"5299" = "1" /* OTHER OPERATIONS ON PANCREAS */
"5300" = "1" /* UNILATERAL REPAIR OF INGUINAL HERNIA, NOS */
"5301" = "1" /* REPAIR OF DIRECT INGUINAL HERNIA */
"5302" = "1" /* REPAIR OF INDIRECT INGUINAL HERNIA */
"5303" = "1" /* REPAIR OF DIRECT INGUINAL HERNIA WITH GRAFT OR PROSTHESIS */
"5304" = "1" /* REPAIR OF INDIRECT INGUINAL HERNIA WITH GRAFT OR PROSTHESIS */
"5305" = "1" /* REAPIR OF INGUINAL HERNIA WITH GRAFT OR PROSTHESIS, NOS */
"5310" = "1" /* BILATERAL REPAIR OF INGUINAL HERNIA, NOS */
"5311" = "1" /* BILATERAL REPAIR OF DIRECT INGUINAL YERNIA */
"5312" = "1" /* BILATERAL REPAIR OF INDIRECT INGUINAL HERNIA */
"5313" = "1" /* BILATERAL REPAIR OF INGUINAL HERNIA, ONE DIRECT AND ONE INDIRECT */
"5314" = "1" /* BILATERAL REPAIR OF DIRECT INGUINAL HERNIA WITH GRAFT OR PROSTHESIS */
"5315" = "1" /* BILATERAL REPAIR OF INDIRECT INGUINAL HERNIA WITH GRAFT OR PROSTHESIS */
"5316" = "1" /* BILATERAL REPAIR OF INGUINAL HERNIA, ONE DIRECT AND ONE INDIRECT, WITH GRAFT OR PROSTHESIS */
"5317" = "1" /* BILATERAL INGUINAL HERNIA REPAIR WITH GRAFT OR PROSTHESIS, NOS */
"5321" = "1" /* UNILATERAL REPAIR OF FEMORAL HERNIA */
"5329" = "1" /* OTHER UNILATERAL FEMORAL HERNIORHAPHY */
"5331" = "1" /* BILATERAL REPAIR OF FEMORAL HERNIA WITH GRAFT OR PROSTHESIS */
"5339" = "1" /* OTHER BILATERAL FEMORAL HERNIORRHAPHY */
"5341" = "1" /* REPAIR OF UMBILICAL HERNIA WITH PROSTHESIS */
"5349" = "1" /* OTHER UMBILICAL HERNIORRHAPHY */
"5351" = "1" /* INCISIONAL HERNIA REPAIR */
"5359" = "1" /* REPAIR OF OTHER HERNIA OF ANTERIOR ABDOMINAL WALL */
"5361" = "1" /* INCISIONAL HERNIA REPAIR WITH PROSTHESIS */
"5369" = "1" /* REPAIR OF OTHER HERNIA OF ANTERIOR ABDOMINAL WALL WITH PROSTHESIS */
"537"  = "1" /* REPAIR OF DIAPHRAGMATIC HERNIA, ABDOMINAL APPROACH */
"540"  = "1" /* INCISION OF ABDOMINAL WALL */
"5411" = "1" /* EXPLORATORY LAPAROTOMY */
"5419" = "1" /* OTHER LAPAROTOMY */
"5422" = "1" /* BIOPSY OF ABDOMINAL WALL OR UMBILICUS */
"5423" = "1" /* BIOPSY OF ABDOMINAL WALL OR UMBILICUS */
"543"  = "1" /* EXCISION OR DESTRUCTION OF LESION OR TISSUE OF ABDOMINAL WALL OR UMBILICUS */
"544"  = "1" /* EXCISION OR DESTRUCTION OF PERITONEAL TISSUE */
"5459" = "1" /* OTHER LYSIS OF PERITONEAL ADHESIONS */
"5463" = "1" /* OTHER SUTURE OF ABDOMINAL WALL */
"5464" = "1" /* SUTURE OF PERITONEUM */
"5471" = "1" /* REPAIR OF GASTROSCHISIS */
"5472" = "1" /* OTHER REPAIRE OF ABDOMINAL WALLS */
"5473" = "1" /* OTHER REPAIR OF PERITONEUM */
"5474" = "1" /* OTHER REPAIR OF OMENTUM */
"5475" = "1" /* OTHER REPAIR OF MESENTERY */
"5492" = "1" /* REMOVAL OF FOREIGN BODY FROM PERITONEAL CAVITY */
"5493" = "1" /* CREATION OF CUTANEOPERITONEAL FISTULA */
"5494" = "1" /* CREATION OF PERITONEOVASCULAR SHUNT */
"5495" = "1" /* INCISION OF PERITONEUM */
"5532" = "1" /* OPN ABLTN RENAL LES/TISS */ 
"5535" = "1" /* ABLTN RENAL LES/TISS NEC */ 
"5551" = "1" /* NEPHROURETERECTOMY */
"5552" = "1" /* NEPHRECTOMY OF REMAINING KIDNEY */
"5553" = "1" /* REMOVAL OF TRANSPLANTED OR REGECTED KDINEY */
"5554" = "1" /* BILATERAL NEPHRECTOMY */
"5561" = "1" /* RENAL AUTOTRANSPLANTATION */
"5569" = "1" /* ULCERATIVE COLITIS, UNSPECIFIED */
"557"  = "1" /* NEPHROPEXY */
"5583" = "1" /* CLOSURE OF OTHER FISTULA OF KIDNEY */
"5584" = "1" /* REDUCTION OF TORSION OF RENAL */
"5585" = "1" /* SYMPHYSIOTOMY FOR HORESHOE KIDNEY */
"5586" = "1" /* ANASTOMOSIS OF KIDNEY */
"5587" = "1" /* CORRECTION OF URETEROPELVIC JUNCTION */
"5591" = "1" /* DECAPSULATION OF KEDNEY */
"5597" = "1" /* IMPLANTATION OR REPLACEMENT OF MECHANICAL KIDNEY */
"5598" = "1" /* REMOVAL OF MECHANICAL KIDNEY */
"5651" = "1" /* FORMATION OF CUTANEOUS URETEROILEOSTOMY */
"5652" = "1" /* REVISION OF CUTANEOUS URETER OILEOSTOMY */
"5661" = "1" /* FORMATION OF OTHER CUTANEOUS URETEROSTOMY */
"5662" = "1" /* REVISION OF OTHER CUTANEOUS URETEROSTOMY */
"5671" = "1" /* URINARY DIVERSION TO INTESTINE */
"5672" = "1" /* REVISION OF URETEROINTESTINAL ANASTOMOSIS */
"5673" = "1" /* NEPHROCYSTANASTOMOSIS, NO */
"5674" = "1" /* URETERONEOXYSTOSTOMY */
"5675" = "1" /* TRANSURETEROURETEROSTOMY */
"5683" = "1" /* CLOSURE OF URETEROSTOMY */
"5684" = "1" /* CLOSURE OF OTHER FISTULA OF URETER */
"5685" = "1" /* URETEROPEXY */
"5686" = "1" /* REMOVAL OF LIGATURE FORM URETER */
"5689" = "1" /* OTHER REPAIR OF URETER */
"5695" = "1" /* LIGATION OF URETER */
"5771" = "1" /* RADICAL CYSTECTOMY */
"5779" = "1" /* OTHER TOTAL CYSTECTOMY */
"5782" = "1" /* CLOSRUE OF CYSTOSTOMY */
"5787" = "1" /* RECONSTRUCTION OF URNIARY BLADDER */
"5900" = "1" /* RETROPERITONEAL DISSECTION, NOS */
"5902" = "1" /* OTHER LYSIS OF PERIRENAL OR PERIURETERAL ADHESIONS */
"5909" = "1" /* OTHER INCISION OF PERIRENAL OR PERIURETERAL TISSUE */
"6012" = "1" /* OPEN BIOPSY OF PROSTATE */
"6014" = "1" /* OPEN BIOPSY OF SEMINAL VESICLES */
"6015" = "1" /* BIOPSY OF PERIPROSTATIC TISSUE */
"603"  = "1" /* SUPRAPUBIC PROSTATECTOMY */
"604"  = "1" /* RETROPUBIC PROSTATECTOMY */
"605"  = "1" /* RADICAL PROSTATECTOMY */
"6061" = "1" /* LOCAL EXCISION OF LESION OF PROSTATE */
"6072" = "1" /* INCISION OF SEMINAL VESICLE */
"6073" = "1" /* EXCISION OF SEMINAL VESICLE */
"6079" = "1" /* OTHER OPERATIONS ON SEMINAL VESICLES */
"6093" = "1" /* REPAIR OF PROSTATE */
"6509" = "1" /* OTHER OOPHORECTOMY */
"6512" = "1" /* OTHER BIOPSY OF OVARY */
"6521" = "1" /* MARSUPIALIZATION OF OVARIAN CYST */
"6522" = "1" /* WEDGE RESECTION OF OVARY*/
"6529" = "1" /* OTHER LOCAL EXCISION OR DESTRUCTION OF OVARY */
"6539" = "1" /* OTHER UNLILATERAL OOPHORECTOMY */
"6549" = "1" /* OTHER UNLIATERAL SALPINGOOPHORECTOMY */
"6551" = "1" /* OTHER REMOVAL OF BOTH OVARIES AT SAME OPERATIVE EPISODE */
"6552" = "1" /* OTHER REMOVAL OF REMAINING OVARY */
"6561" = "1" /* OTHER REMOVAL OF BOTH OVARIES AND TUBES AT SAME OPERATIVE EPISODE */
"6562" = "1" /* OTHER REMOVAL OF REMAINING OVARY AND TUBE */
"6571" = "1" /* OTHER SIMPLE SUTURE OF OVARY */
"6572" = "1" /* OTHER REIMPLANTATION OF OVARY */
"6573" = "1" /* OTHER SALPINGO OOPHOROPLASTY */
"6579" = "1" /* OTHER REPAIR OF OVARY */
"6589" = "1" /* OTHER LYSIS OF ADHESIONS OF AVARY AND FALLOPIAN TUBE */
"6592" = "1" /* TRANSPLANTATION OF OVARY */
"6593" = "1" /* MANUAL RUPTURE OF OVARIAN CYST */
"6594" = "1" /* OVARIAN DENERVATION */
"6595" = "1" /* RELEASE OF TORSION OF OVARY */
"6599" = "1" /* OTHER OPERATIONS ON OVARY */
"6601" = "1" /* SALPINGOTOMY */
"6602" = "1" /* SALPINGOSTOMY */
"6631" = "1" /* OTHER BILATERAL LIGATION AND CRUSHING OF FALLOPIAN TUBES */
"6632" = "1" /* OTHER BILATERAL LIGATION AND DIVISION OF FALLOPIAN TUBES */
"6639" = "1" /* OTHER BILATERAL DESTRUCTION OR OCCLUSION OF FALLOPIAN TUBES */
"664"  = "1" /* TOTAL UNILATERAL SALPINGECTOMY */
"6651" = "1" /* REMOVAL OF BOTH FALLOPIAN TUBES AT SAME OPERATIVE EPISODE */
"6652" = "1" /* REMOVAL OF REMAINING FALLOPIAN TUBE */
"6661" = "1" /* EXCISION OR DESTRUCTION OF LESION OF FALLOPIAN TUBE */
"6662" = "1" /* SALPINGECTOMY WITH REMOVAL OF TUBAL PREGNANCY */
"6663" = "1" /* BILATERAL PARTIAL SALPINGECTOMY, NOS */
"6669" = "1" /* OTHER PARTIAL SALPINGECTOMY */
"6671" = "1" /* SIMPLE SUTURE OF FALLOPIAN TUBE */
"6672" = "1" /* SALPINGO-OOPHOROSTOMY*/
"6673" = "1" /* SALPINGO-SALPINGOSTOMY */
"6674" = "1" /* SALPING0-UTEROSTOMY */
"6679" = "1" /* OTHER REPAIR OF FALLOPIAN TUBE */
"6692" = "1" /* UNILATERAL DESTRUCTION OR OCCLUSION OF FALLOPIAN TUBE */
"6697" = "1" /* BURYING OF FIMBRIAE IN UTERINE WALL */
"680"  = "1" /* OTHER INCISION AND EXCISION OF UTERUS */
"6813" = "1" /* OPEN BIOPSY OF UTERUS */
"6814" = "1" /* OPEN BIOPSY OF UTERINE LIGAMENTS */
"683"  = "1" /* SUBTOTAL ABDOMINAL HYSTERECTOMY */
"6839" = "1" /* OTH SUBTOT ABD HYSTERECT */
"684"  = "1" /* TOTAL ABDOMINAL HYSTERECTOMY */
"6841" = "1" /* LAP TOTAL ABDOMINAL HYST */ 
"6849" = "1" /* TOTAL ABD HYST NEC/NOS */   
"686"  = "1" /* RADICAL ABDOMINAL HYSTERECTOMY */
"6861" = "1" /* LAP RADICAL ABDOMNL HYST */ 
"6869" = "1" /* RADICAL ABD HYST NEC/NOS */ 
"688"  = "1" /* PELVIC EVISCERATION */
"6922" = "1" /* OTHER UTERINE SUSPENSION */
"693"  = "1" /* PARACERVICAL UTERINE DENERVATION */
"6941" = "1" /* SUTURE OF LACERATION OF UTERUS */
"6942" = "1" /* CLOSURE OF FISTULA OF UTERUS */
"6949" = "1" /* OTHER REPAIR OF UTERUS */
OTHER  = "0" ;

VALUE $ABDOM2P
"4499" = "1" /* GASTRIC OPERATION NEC */
OTHER  = "0" ;

 /* RECLOSURE OF POSTOPERATIVE DISRUPTION OF ABDOMINAL WALL */;
 /* ICD-9-CM PROCEDURE CODE: */

VALUE $RECLOIP
"5461"  = "1"  /* RECLOSURE OF POSTOPERATIVE DISRUPTION OF ABDOMINAL WALL */
OTHER   = "0" ;

 /* ----------------------------------- */
 /* ACCIDENTAL PUNCTURE OR LACERATION   */
 /*   PROVIDER INDICATOR / PSI # 15     */
 /*   AREA     INDICATOR / PSI # 25     */
 /* ----------------------------------- */

 /* ICD-9-CM DIAGNOSIS CODES: */

VALUE $TECHNID
"9982"  = "1" /* ACCIDENTAL PUNCTURE OR LACERATION DURING A PROCEDURE */
"E8700" = "1" /* ACCIDENTAL CUT, PUNCTURE, PERFORATION, OR HEMORRHAGE DURING MEDICAL CARE, SURGICAL OPERATION */
"E8701" = "1" /* INFUSION OR TRANSFUSION */
"E8702" = "1" /* KIDNEY DIALYSIS OR OTHER PERFUSION */
"E8703" = "1" /* INJECTION OR VACCINATION */
"E8704" = "1" /* ENDOSCOPIC EXAMINATION */
"E8705" = "1" /* ASPIRATION OF FLUID OR TISSUE, PUNCTURE, AND CATHETERIZATION */
"E8706" = "1" /* HEART CATHETERIZATION */
"E8707" = "1" /* ADMINISTRATION OF ENEMA */
"E8708" = "1" /* OTHER SPECIFIED MEDICAL CARE */
"E8709" = "1" /* ACCIDENTAL CUT, PUNCTURE, PERFORATION, OR HEMORRHAGE DURING MEDICAL CARE, UNSPECIFIED MEDICAL CARE */
OTHER   = "0" ;

 /* ICD-9-CM PROCEDURE CODES: */

VALUE $SPINEP
"0301" = "1" /* REMOVAL FB SPINAL CANAL */
"0302" = "1" /* REOPEN LAMINECTOMY SITE */
"0309" = "1" /* SPINAL CANAL EXPLOR NEC */
"0353" = "1" /* VERTEBRAL FX REPAIR */
"036" = "1" /* SPINAL CORD ADHESIOLYSIS */
"8100" = "1" /* SPINAL FUSION NOS */
"8101" = "1" /* ATLAS-AXIS FUSION */
"8102" = "1" /* OTHER CERVICAL FUS ANT */
"8103" = "1" /* OTHER CERVICAL FUS POST */
"8104" = "1" /* DORSAL/DORSOLUM FUS ANT */
"8105" = "1" /* DORSAL/DORSOLUM FUS POST */
"8106" = "1" /* LUMBAR/LUMBOSAC FUS ANT */
"8107" = "1" /* LUMBAR/LUMBOSAC FUS LAT */
"8108" = "1" /* LUMBAR/LUMBOSAC FUS POST */
"8130" = "1" /* SPINAL REFUSION NOS */
"8131" = "1" /* REFUSION OF ATLAS-AXIS */
"8132" = "1" /* REFUSION OF OTH CERV ANT */
"8133" = "1" /* REFUS OF OTH CERV POST */
"8134" = "1" /* REFUSION OF DORSAL ANT */
"8135" = "1" /* REFUSION OF DORSAL POST */
"8136" = "1" /* REFUSION OF LUMBAR ANT */
"8137" = "1" /* REFUSION OF LUMBAR LAT */
"8138" = "1" /* REFUSION OF LUMBAR POST */
"8139" = "1" /* REFUSION OF SPINE NEC */
"8162" = "1" /* FUS/REFUS 2-3 VERTEBRAE */
"8163" = "1" /* FUS/REFUS 4-8 VERTEBRAE */
"8164" = "1" /* FUS/REFUS 9 VERTEBRAE */
"8165" = "1" /* VERTEBROPLASTY */
"8166" = "1" /* KYPHOPLASTY */
"8451" = "1" /* INS SPINAL FUSION DEVICE */
"8452" = "1" /* INSERT RECOMBINANT BMP */
"8458" = "1" /* IMP INTRSPINE DECOMP DEV */
"8459" = "1" /* INSERT OTHR SPIN DEVICE */
"8460" = "1" /* INSERT DISC PROS NOS */
"8461" = "1" /* INS PART DISC PROS CERV */
"8462" = "1" /* INS TOT DISC PROST CERV */
"8463" = "1" /* INS SPIN DISC PROS THOR */
"8464" = "1" /* INS PART DISC PROS LUMB */
"8465" = "1" /* INS TOTL DISC PROS LUMB */
"8466" = "1" /* REVISE DISC PROST CERV */
"8467" = "1" /* REVISE DISC PROST THORA */
"8468" = "1" /* REVISE DISC PROSTH LUMB */
"8469" = "1" /* REVISE DISC PROSTH NOS */
"8480" = "1" /* INS/REPL INTERSPINE DEV */
"8481" = "1" /* REV INTERSPINE DEVICE */
"8482" = "1" /* INS/REPL PDCL STABIL DEV */
"8483" = "1" /* REV PEDCL DYN STABIL DEV */
"8485" = "1" /* REV FACET REPLACE DEVICE */
OTHER   = "0" ;

 /* ------------------------------- */
 /* TRANSFUSION REACTION            */
 /*   PROVIDER INDICATOR / PSI # 16 */
 /*   AREA     INDICATOR / PSI # 26 */
 /* ------------------------------- */

 /* ICD-9-CM DIAGNOSIS CODES: */

VALUE $TRANFID
"9996"  = "1" /* ABO INCOMPATIBILITY REACTION */
"9997"  = "1" /* RH INCOMPATIBILITY REACTION */
"E8760" = "1" /* MISMATCHED BLOOD IN TRANSFUSION */
OTHER   = "0" ;

 /* ------------------------------- */
 /* BIRTH TRAUMA                    */
 /*   PROVIDER INDICATOR / PSI # 17 */
 /* ------------------------------- */

 /* LIVEBORN */
 /* ICD-9-CM DIAGNOSIS CODES */

VALUE $LIVEBND
'V3000' = 1  /* SINGLE LB IN-HOSP W/O CS */
'V3001' = 1  /* SINGLE LB IN-HOSP W CS */
'V3100' = 1  /* TWIN-MATE LB-HOSP W/O CS */
'V3101' = 1  /* TWIN-MATE LB-IN HOS W CS */
'V3200' = 1  /* TWIN-MATE SB-HOSP W/O CS */
'V3201' = 1  /* TWIN-MATE SB-HOSP W CS */
'V3300' = 1  /* TWIN-NOS-IN HOSP W/O CS */
'V3301' = 1  /* TWIN-NOS-IN HOSP W CS */
'V3400' = 1  /* OTH MULT LB-HOSP W/O CS */
'V3401' = 1  /* OTH MULT LB-IN HOSP W CS */
'V3500' = 1  /* OTH MULT SB-HOSP W/O CS */
'V3501' = 1  /* OTH MULT SB-IN HOSP W CS */
'V3600' = 1  /* MULT LB/SB-IN HOS W/O CS */
'V3601' = 1  /* MULT LB/SB-IN HOSP W CS */
'V3700' = 1  /* MULT BRTH NOS-HOS W/O CS */
'V3701' = 1  /* MULT BIRTH NOS-HOSP W CS */
'V3900' = 1  /* LIVEBORN NOS-HOSP W/O CS */
'V3901' = 1  /* LIVEBORN NOS-HOSP W CS */
OTHER   = 0 ;

VALUE $V29D
"V290"  = "1" /* OBSERVATIONS FOR SUSPECTED INFECTIOUS CONDITION */
"V291"  = "1" /* OBSERVATIONS FOR SUSPECTED NEUROLOGICAL CONDITION */
"V292"  = "1" /* OBSERVATIONS FOR SUSPECTED RESPIRATORY CONDITION */
"V293"  = "1" /* OBSERVATIONS FOR SUSPECTED GENETIC OR METABOLIC CONDITION */
"V298"  = "1" /* OBSERVATIONS FOR OTHER SPECIFIED SUSPECTED CONDITION */
"V299"  = "1" /* OBSERVATIONS FOR UNSPECIFIED SUSPECTED CONDITION */
OTHER   = "0";

VALUE $LIVEB2D
'V301 ' = 1  /* SINGL LIVEBRN-BEFORE ADM */
'V302 ' = 1  /* SINGLE LIVEBORN-NONHOSP */
'V311 ' = 1  /* TWIN, MATE LB-BEFORE ADM */
'V312 ' = 1  /* TWIN, MATE LB-NONHOSP */
'V321 ' = 1  /* TWIN, MATE SB-BEFORE ADM */
'V322 ' = 1  /* TWIN, MATE SB-NONHOSP */
'V331 ' = 1  /* TWIN NOS-BEFORE ADMISSN */
'V332 ' = 1  /* TWIN NOS-NONHOSP */
'V341 ' = 1  /* OTH MULT NB-BEFORE ADM */
'V342 ' = 1  /* OTH MULTIPLE NB-NONHOSP */
'V351 ' = 1  /* OTH MULT SB-BEFORE ADM */
'V352 ' = 1  /* OTH MULTIPLE SB-NONHOSP */
'V361 ' = 1  /* MULT NB/SB-BEFORE ADM */
'V362 ' = 1  /* MULTIPLE NB/SB-NONHOSP */
'V371 ' = 1  /* MULT BRTH NOS-BEFORE ADM */
'V372 ' = 1  /* MULT BIRTH NOS-NONHOSP */
'V391 ' = 1  /* LIVEBORN NOS-BEFORE ADM */
'V392 ' = 1  /* LIVEBORN NOS-NONHOSP */
OTHER   = 0 ;

 /* BIRTH TRAUMA */
 /* ICD-9-CM DIAGNOSIS CODES: */

VALUE $BIRTIDA
"7670"  = "1" /* SUBDURAL AND CEREBRAL HEMORRHAGE (DUE TO TRAUMA OR TO INTRAPARTUM 
                 ANOXIA OR HYPOXIA) */
OTHER   = "0" ;

 /* EXCLUDE: PRETERM INFANT */
 /* ICD-9-CM DIAGNOSIS CODES: */

VALUE $PRETEID
"76501" = "1" /* EXTREME IMMATURITY, LESS THAN 500 GRAMS  */
"76502" = "1" /* EXTREME IMMATURITY, 500 - 749 GRAMS */
"76503" = "1" /* EXTREME IMMATURITY, 750 - 999 GRAMS  */
"76504" = "1" /* EXTREME IMMATURITY, 1000 - 1249 GRAMS  */
"76505" = "1" /* EXTREME IMMATURITY, 1250 - 1499 GRAMS */
"76506" = "1" /* EXTREME IMMATURITY, 1500 - 1749 GRAMS  */
"76507" = "1" /* EXTREME IMMATURITY, 1750 - 1999 GRAMS  */
"76508" = "1" /* EXTREME IMMATURITY, 2000 - 2499 GRAMS  */
"76511" = "1" /* OTHER PRETERM INFANTS, LESS THAN 500 GRAMS  */
"76512" = "1" /* OTHER PRETERM INFANTS, 500 - 749 GRAMS  */
"76513" = "1" /* OTHER PRETERM INFANTS, 750 - 999 GRAMS  */
"76514" = "1" /* OTHER PRETERM INFANTS, 1000 - 1249 GRAMS  */
"76515" = "1" /* OTHER PRETERM INFANTS, 1250 - 1499 GRAMS  */
"76516" = "1" /* OTHER PRETERM INFANTS, 1500 - 1749 GRAMS */
"76517" = "1" /* OTHER PRETERM INFANTS, 1750 - 1999 GRAMS  */
"76518" = "1" /* OTHER PRETERM INFANTS, 2000 - 2499 GRAMS  */
"76521" = "1" /* LESS THAN 24 COMPLETED WEEKS OF GESTATION  */
"76522" = "1" /* 24 COMPLETED WEEKS OF GESTATION  */
"76523" = "1" /* 25-26 COMPLETED WEEKS OF GESTATION */
"76524" = "1" /* 27-28 COMPLETED WEEKS OF GESTATION */
"76525" = "1" /* 29-30 COMPLETED WEEKS OF GESTATION */
"76526" = "1" /* 31-32 COMPLETED WEEKS OF GESTATION */
"76527" = "1" /* 33-34 COMPLETED WEEKS OF GESTATION  */
OTHER   = "0" ;

 /* BIRTH TRAUMA */
 /* ICD-9-CM DIAGNOSIS CODES: */

VALUE $BIRTIDB
"7673"  = "1" /* INJURIES TO SKELETON (EXCLUDES CLAVICLE) */
"7674"  = "1" /* INJURY TO SPINE AND SPINAL CORD */
OTHER   = "0" ;

 /* EXCLUDE: OSTEOGENESIS IMPERFECTA */;
 /* ICD-9-CM DIAGNOSIS CODES: */

VALUE $OSTEOID
"75651" = "1" /* OSTEOGENESIS IMPERFECTA */
OTHER   = "0" ;

 /* BIRTH TRAUMA */
 /* ICD-9-CM DIAGNOSIS CODES: */

VALUE $BIRTIDC
"76711" = "1" /* EPICRANIAL SUBAPO HEMORR */
"7677"  = "1" /* OTHER CRANIAL AND PERIPHERAL NERVE INJURIES */
"7678"  = "1" /* OTHER SPECIFIED BIRTH TRAUMA */
"7679"  = "1" /* BIRTH TRAUMA, UNSPECIFIED */
OTHER   = "0" ;

 /* --------------------------------------------- */
 /* OBSTETRIC TRAUMA - VAGINAL WITH INSTRUMENT    */
 /*   PROVIDER INDICATOR / PSI # 18               */
 /* OBSTETRIC TRAUMA - VAGINAL WITHOUT INSTRUMENT */
 /*   PROVIDER INDICATOR / PSI # 19               */
 /* --------------------------------------------- */

 /* VAGINAL DELIVERY */
 /* DIAGNOSTIC RELATED GROUPS (DRGS): */

VALUE $VAGINDR
"372" = "1" /* VAGINAL DELIVERY WITH COMPLICATING DIAGNOSES */
"373" = "1" /* VAGINAL DELIVERY WITHOUT COMPLICATING DIAGNOSES */
"374" = "1" /* VAGINAL DELIVERY WITH STERILIZATION AND/OR D AND C */
"375" = "1" /* VAGINAL DELIVERY WITH OR PROCEDURE EXCEPT STERILIZATION ANDOR D AND C */
OTHER = "0" ;

 /* INSTRUMENT ASSISTED DELIVERY */
 /* ICD-9-CM PROCEDURE CODES: */

VALUE $INSTRIP
"720"  = "1" /* LOW FORCEPS OPERATION */
"721"  = "1" /* LOW FORCEPS OPERATION WITH EPISIOTOMY */
"7221" = "1" /* MID FORCEPS OPERATION WITH EPISIOTOMY */
"7229" = "1" /* OTHER MID FORCEPS OPERATION */
"7231" = "1" /* HIGH FORCEPS OPERATION WITH EPISIOTOMY */
"7239" = "1" /* OTHER HIGH FORCEPS OPERATION */
"724"  = "1" /* FORCEPS ROTATION OF FETAL HEAD */
"7251" = "1" /* PARTIAL BREECH EXTRACTION WITH FORCEPS TO AFTERCOMING HEAD */
"7253" = "1" /* TOTAL BREECH EXTRACTION WITH FORCEPS TO AFTERCOMING HEAD */
"726"  = "1" /* FORCEPS APPLICATION TO AFTERCOMING HEAD */
"7271" = "1" /* VACUUM EXTRACTION WITH EPISIOTOMY */
"7279" = "1" /* VACUUM EXTRACTION DELIVERY NEC */
"728"  = "1" /* OTHER SPECIFIED INSTRUMENTAL DELIVERY */
"729"  = "1" /* UNSPECIFIED INSTRUMENTAL DELIVERY */
OTHER  = "0" ;

 /* OBSTETRIC TRAUMA */
 /* ICD-9-CM DIAGNOSIS CODES: */

VALUE $OBTRAID
"66420" = "1" /* DEL W 3 DEG LACERAT-UNSP */
"66421" = "1" /* DEL W 3 DEG LACERAT-DEL */
"66424" = "1" /* DEL W 3 DEG LAC-POSTPART */
"66430" = "1" /* TRAUMA TO PERINEUM AND VULVA DURING DELIVERY, 
                 FOURTH-DEGREE PERINEAL LACERATION */
"66431" = "1" /* TRAUMA TO PERINEUM AND VULVA DURING DELIVERY, 
                 FOURTH-DEGREE PERINEAL LACERATION */
"66434" = "1" /* TRAUMA TO PERINEUM AND VULVA DURING DELIVERY, 
                 FOURTH-DEGREE PERINEAL LACERATION */
OTHER   = "0" ;

 /* OBSTETRIC TRAUMA W 3RD DEGREE */
 /* ICD-9-CM DIAGNOSIS CODES: */

VALUE $OBTRIDB
"66530" = "1" /* OTHER OBSTETRICAL TRAUMA, LACERATION OF CERVIX */
"66531" = "1" /* OTHER OBSTETRICAL TRAUMA, LACERATION OF CERVIX */
"66534" = "1" /* OTHER OBSTETRICAL TRAUMA, LACERATION OF CERVIX */
"66540" = "1" /* OTHER OBSTETRICAL TRAUMA, HIGH VAGINAL LACERATIONS */
"66541" = "1" /* OTHER OBSTETRICAL TRAUMA, HIGH VAGINAL LACERATIONS */
"66544" = "1" /* OTHER OBSTETRICAL TRAUMA, HIGH VAGINAL LACERATIONS */
"66550" = "1" /* OTHER OBSTETRICAL TRAUMA, OTHER INJURY TO PELVIC ORGANS */
"66551" = "1" /* OTHER OBSTETRICAL TRAUMA, OTHER INJURY TO PELVIC ORGANS */
"66554" = "1" /* OTHER OBSTETRICAL TRAUMA, OTHER INJURY TO PELVIC ORGANS */
OTHER   = "0" ;

 /* OBSTETRIC TRAUMA */
 /* ICD-9-CM PROCEDURE CODES: */

VALUE $OBTRAIP
"7550" = "1" /* REPAIR OF CURRENT OBSTETRIC LACERATIONS UTERUS */
"7551" = "1" /* REPAIR OF CURRENT OBSTETRIC LACERATIONS OF CERVIX */
"7552" = "1" /* REPAIR OF CURRENT OBSTETRIC LACERATIONS OF CORPUS UTERI */
"7561" = "1" /* REPAIR OF CURRENT OBSTETRIC LACERATION OF BLADDER AND URETHRA */
"7562" = "1" /* REPARI OF CURRENT OBSTETRIC LACERATION OF RECTUM AND SPHINCTER ANI */
OTHER  = "0" ;

 /* ----------------------------------- */
 /* OBSTETRIC TRAUMA - CESAREAN SECTION */
 /*   PROVIDER INDICATOR / PSI # 20     */
 /* ----------------------------------- */

 /* CESAREAN DELIVERY */
 /* DIAGNOSTIC RELATED GROUPS (DRGS): */

VALUE $CESARDR
"370" = "1" /* CESAREAN SECTION WITH CC */
"371" = "1" /* CESAREAN SECTION WITHOUT CC */
OTHER = "0" ;


 * ----------------------------------------------------- ;
 * ---   AREA FORMATS     ------------------------------ ;
 * ---   TRANSLATES FIPS STATE AND COUNTY CODE (SSCCC)   ;
 * ---   TO MSA (METROPOLITAN STATISTICAL AREA)          ;
 * ----------------------------------------------------- ;

 * ----------------------------------------------------- ;
 * ---   AREA FORMATS     ------------------------------ ;
 * ---   Translates FIPS State and County Code (SSCCC)   ;
 * ---   to MSA (Metropolitan Statistical Area)          ;
 * ----------------------------------------------------- ;

/* MODIFIED FIPS */

VALUE $M1AREA
'02013' = '02001'
'02016' = '02001'
'02020' = '02001'
'02050' = '02001'
'02060' = '02001'
'02068' = '02001'
'02070' = '02001'
'02090' = '02001'
'02100' = '02001'
'02110' = '02001'
'02122' = '02001'
'02130' = '02001'
'02150' = '02001'
'02164' = '02001'
'02170' = '02001'
'02180' = '02001'
'02185' = '02001'
'02188' = '02001'
'02201' = '02001'
'02220' = '02001'
'02232' = '02001'
'02240' = '02001'
'02261' = '02001'
'02270' = '02001'
'02280' = '02001'
'02282' = '02001'
'02290' = '02001'
'15005' = '15009'
'24510' = '24007'
'29510' = '29191'
'32510' = '32025'
'51515' = '51019'
'51520' = '51119'
'51530' = '51163'
'51540' = '51003'
'51560' = '51005'
'51570' = '51041'
'51580' = '51005'
'51590' = '51143'
'51595' = '51081'
'51600' = '51059'
'51610' = '51059'
'51620' = '51175'
'51630' = '51177'
'51640' = '51077'
'51660' = '51165'
'51670' = '51149'
'51678' = '51163'
'51680' = '51031'
'51683' = '51153'
'51685' = '51153'
'51690' = '51089'
'51710' = '51129'
'51720' = '51195'
'51730' = '51053'
'51735' = '51199'
'51740' = '51129'
'51750' = '51121'
'51760' = '51087'
'51770' = '51161'
'51775' = '51161'
'51780' = '51083'
'51790' = '51015'
'51800' = '51123'
'51820' = '51015'
'51830' = '51095'
'51840' = '51069'
;

/* OMB 1999 METRO AREA */

VALUE $M2AREA
'01001' = '5240 '
'01003' = '5160 '
'01009' = '1000 '
'01015' = '0450 '
'01033' = '2650 '
'01045' = '2180 '
'01051' = '5240 '
'01055' = '2880 '
'01069' = '2180 '
'01073' = '1000 '
'01077' = '2650 '
'01079' = '2030 '
'01081' = '0580 '
'01083' = '3440 '
'01089' = '3440 '
'01097' = '5160 '
'01101' = '5240 '
'01103' = '2030 '
'01113' = '1800 '
'01115' = '1000 '
'01117' = '1000 '
'01125' = '8600 '
'02020' = '0380 '
'04005' = '2620 '
'04013' = '6200 '
'04015' = '4120 '
'04019' = '8520 '
'04021' = '6200 '
'04027' = '9360 '
'05007' = '2580 '
'05031' = '3700 '
'05033' = '2720 '
'05035' = '4920 '
'05045' = '4400 '
'05069' = '6240 '
'05085' = '4400 '
'05091' = '8360 '
'05119' = '4400 '
'05125' = '4400 '
'05131' = '2720 '
'05143' = '2580 '
'06001' = '5775 '
'06007' = '1620 '
'06013' = '5775 '
'06017' = '6920 '
'06019' = '2840 '
'06029' = '0680 '
'06037' = '4480 '
'06039' = '2840 '
'06041' = '7360 '
'06047' = '4940 '
'06053' = '7120 '
'06055' = '8720 '
'06059' = '5945 '
'06061' = '6920 '
'06065' = '6780 '
'06067' = '6920 '
'06071' = '6780 '
'06073' = '7320 '
'06075' = '7360 '
'06077' = '8120 '
'06079' = '7460 '
'06081' = '7360 '
'06083' = '7480 '
'06085' = '7400 '
'06087' = '7485 '
'06089' = '6690 '
'06095' = '8720 '
'06097' = '7500 '
'06099' = '5170 '
'06101' = '9340 '
'06107' = '8780 '
'06111' = '8735 '
'06113' = '9270 '
'06115' = '9340 '
'08001' = '2080 '
'08005' = '2080 '
'08013' = '1125 '
'08031' = '2080 '
'08035' = '2080 '
'08041' = '1720 '
'08059' = '2080 '
'08069' = '2670 '
'08077' = '2995 '
'08101' = '6560 '
'08123' = '3060 '
'09001' = '5483 '
'09003' = '3283 '
'09007' = '3283 '
'09009' = '5483 '
'09011' = '5523 '
'09013' = '3283 '
'10001' = '2190 '
'10003' = '9160 '
'11001' = '8840 '
'12001' = '2900 '
'12005' = '6015 '
'12009' = '4900 '
'12011' = '2680 '
'12015' = '6580 '
'12019' = '3600 '
'12021' = '5345 '
'12031' = '3600 '
'12033' = '6080 '
'12035' = '2020 '
'12039' = '8240 '
'12053' = '8280 '
'12057' = '8280 '
'12069' = '5960 '
'12071' = '2700 '
'12073' = '8240 '
'12081' = '7510 '
'12083' = '5790 '
'12085' = '2710 '
'12086' = '5000 '
'12089' = '3600 '
'12091' = '2750 '
'12095' = '5960 '
'12097' = '5960 '
'12099' = '8960 '
'12101' = '8280 '
'12103' = '8280 '
'12105' = '3980 '
'12109' = '3600 '
'12111' = '2710 '
'12113' = '6080 '
'12115' = '7510 '
'12117' = '5960 '
'12127' = '2020 '
'13013' = '0520 '
'13015' = '0520 '
'13021' = '4680 '
'13029' = '7520 '
'13045' = '0520 '
'13047' = '1560 '
'13051' = '7520 '
'13053' = '1800 '
'13057' = '0520 '
'13059' = '0500 '
'13063' = '0520 '
'13067' = '0520 '
'13073' = '0600 '
'13077' = '0520 '
'13083' = '1560 '
'13089' = '0520 '
'13095' = '0120 '
'13097' = '0520 '
'13103' = '7520 '
'13113' = '0520 '
'13117' = '0520 '
'13121' = '0520 '
'13135' = '0520 '
'13145' = '1800 '
'13151' = '0520 '
'13153' = '4680 '
'13169' = '4680 '
'13177' = '0120 '
'13189' = '0600 '
'13195' = '0500 '
'13215' = '1800 '
'13217' = '0520 '
'13219' = '0500 '
'13223' = '0520 '
'13225' = '4680 '
'13227' = '0520 '
'13245' = '0600 '
'13247' = '0520 '
'13255' = '0520 '
'13289' = '4680 '
'13295' = '1560 '
'13297' = '0520 '
'15003' = '3320 '
'16001' = '1080 '
'16005' = '6340 '
'16027' = '1080 '
'17007' = '6880 '
'17019' = '1400 '
'17027' = '7040 '
'17031' = '1600 '
'17037' = '1600 '
'17043' = '1600 '
'17063' = '1600 '
'17073' = '1960 '
'17083' = '7040 '
'17089' = '1600 '
'17091' = '3740 '
'17093' = '1600 '
'17097' = '1600 '
'17111' = '1600 '
'17113' = '1040 '
'17115' = '2040 '
'17119' = '7040 '
'17129' = '7880 '
'17133' = '7040 '
'17141' = '6880 '
'17143' = '6120 '
'17161' = '1960 '
'17163' = '7040 '
'17167' = '7880 '
'17179' = '6120 '
'17197' = '1600 '
'17201' = '6880 '
'17203' = '6120 '
'18001' = '2760 '
'18003' = '2760 '
'18011' = '3480 '
'18019' = '4520 '
'18021' = '8320 '
'18023' = '3920 '
'18029' = '1640 '
'18033' = '2760 '
'18035' = '5280 '
'18039' = '2330 '
'18043' = '4520 '
'18057' = '3480 '
'18059' = '3480 '
'18061' = '4520 '
'18063' = '3480 '
'18067' = '3850 '
'18069' = '2760 '
'18081' = '3480 '
'18089' = '2960 '
'18095' = '3480 '
'18097' = '3480 '
'18105' = '1020 '
'18109' = '3480 '
'18115' = '1640 '
'18127' = '2960 '
'18129' = '2440 '
'18141' = '7800 '
'18143' = '4520 '
'18145' = '3480 '
'18157' = '3920 '
'18159' = '3850 '
'18163' = '2440 '
'18165' = '8320 '
'18167' = '8320 '
'18173' = '2440 '
'18179' = '2760 '
'18183' = '2760 '
'19013' = '8920 '
'19049' = '2120 '
'19061' = '2200 '
'19103' = '3500 '
'19113' = '1360 '
'19153' = '2120 '
'19155' = '5920 '
'19163' = '1960 '
'19181' = '2120 '
'19193' = '7720 '
'20015' = '9040 '
'20045' = '4150 '
'20079' = '9040 '
'20091' = '3760 '
'20103' = '3760 '
'20121' = '3760 '
'20173' = '9040 '
'20177' = '8440 '
'20209' = '3760 '
'21015' = '1640 '
'21017' = '4280 '
'21019' = '3400 '
'21029' = '4520 '
'21037' = '1640 '
'21043' = '3400 '
'21047' = '1660 '
'21049' = '4280 '
'21059' = '5990 '
'21067' = '4280 '
'21077' = '1640 '
'21081' = '1640 '
'21089' = '3400 '
'21101' = '2440 '
'21111' = '4520 '
'21113' = '4280 '
'21117' = '1640 '
'21151' = '4280 '
'21185' = '4520 '
'21191' = '1640 '
'21209' = '4280 '
'21239' = '4280 '
'22001' = '3880 '
'22005' = '0760 '
'22015' = '7680 '
'22017' = '7680 '
'22019' = '3960 '
'22033' = '0760 '
'22051' = '5560 '
'22055' = '3880 '
'22057' = '3350 '
'22063' = '0760 '
'22071' = '5560 '
'22073' = '5200 '
'22075' = '5560 '
'22079' = '0220 '
'22087' = '5560 '
'22089' = '5560 '
'22093' = '5560 '
'22095' = '5560 '
'22097' = '3880 '
'22099' = '3880 '
'22103' = '5560 '
'22109' = '3350 '
'22119' = '7680 '
'22121' = '0760 '
'23001' = '4243 '
'23005' = '6403 '
'23019' = '0733 '
'24001' = '1900 '
'24003' = '0720 '
'24005' = '0720 '
'24009' = '8840 '
'24013' = '0720 '
'24015' = '9160 '
'24017' = '8840 '
'24021' = '8840 '
'24025' = '0720 '
'24027' = '0720 '
'24031' = '8840 '
'24033' = '8840 '
'24035' = '0720 '
'24043' = '3180 '
'24510' = '0720 '
'25001' = '0743 '
'25003' = '6323 '
'25005' = '1123 '
'25009' = '1123 '
'25013' = '8003 '
'25015' = '8003 '
'25017' = '1123 '
'25021' = '1123 '
'25023' = '1123 '
'25025' = '1123 '
'25027' = '1123 '
'26005' = '3000 '
'26017' = '6960 '
'26021' = '0870 '
'26025' = '3720 '
'26037' = '4040 '
'26045' = '4040 '
'26049' = '2640 '
'26065' = '4040 '
'26075' = '3520 '
'26077' = '3720 '
'26081' = '3000 '
'26087' = '2160 '
'26091' = '0440 '
'26093' = '0440 '
'26099' = '2160 '
'26111' = '6960 '
'26115' = '2160 '
'26121' = '3000 '
'26125' = '2160 '
'26139' = '3000 '
'26145' = '6960 '
'26147' = '2160 '
'26159' = '3720 '
'26161' = '0440 '
'26163' = '2160 '
'27003' = '5120 '
'27009' = '6980 '
'27019' = '5120 '
'27025' = '5120 '
'27027' = '2520 '
'27037' = '5120 '
'27053' = '5120 '
'27055' = '3870 '
'27059' = '5120 '
'27109' = '6820 '
'27119' = '2985 '
'27123' = '5120 '
'27137' = '2240 '
'27139' = '5120 '
'27141' = '5120 '
'27145' = '6980 '
'27163' = '5120 '
'27171' = '5120 '
'28033' = '4920 '
'28035' = '3285 '
'28045' = '0920 '
'28047' = '0920 '
'28049' = '3560 '
'28059' = '0920 '
'28073' = '3285 '
'28089' = '3560 '
'28121' = '3560 '
'29003' = '7000 '
'29019' = '1740 '
'29021' = '7000 '
'29037' = '3760 '
'29043' = '7920 '
'29047' = '3760 '
'29049' = '3760 '
'29071' = '7040 '
'29077' = '7920 '
'29095' = '3760 '
'29097' = '3710 '
'29099' = '7040 '
'29107' = '3760 '
'29113' = '7040 '
'29145' = '3710 '
'29165' = '3760 '
'29177' = '3760 '
'29183' = '7040 '
'29189' = '7040 '
'29219' = '7040 '
'29225' = '7920 '
'29510' = '7040 '
'30013' = '3040 '
'30063' = '5140 '
'30111' = '0880 '
'31025' = '5920 '
'31043' = '7720 '
'31055' = '5920 '
'31109' = '4360 '
'31153' = '5920 '
'31177' = '5920 '
'32003' = '4120 '
'32023' = '4120 '
'32031' = '6720 '
'33011' = '1123 '
'33015' = '1123 '
'33017' = '1123 '
'34001' = '0560 '
'34003' = '0875 '
'34005' = '6160 '
'34007' = '6160 '
'34009' = '0560 '
'34011' = '8760 '
'34013' = '5640 '
'34015' = '6160 '
'34017' = '3640 '
'34019' = '5015 '
'34021' = '8480 '
'34023' = '5015 '
'34025' = '5190 '
'34027' = '5640 '
'34029' = '5190 '
'34031' = '0875 '
'34033' = '6160 '
'34035' = '5015 '
'34037' = '5640 '
'34039' = '5640 '
'34041' = '5640 '
'35001' = '0200 '
'35013' = '4100 '
'35028' = '7490 '
'35043' = '0200 '
'35049' = '7490 '
'35061' = '0200 '
'36001' = '0160 '
'36005' = '5600 '
'36007' = '0960 '
'36011' = '8160 '
'36013' = '3610 '
'36015' = '2335 '
'36027' = '2281 '
'36029' = '1280 '
'36037' = '6840 '
'36043' = '8680 '
'36047' = '5600 '
'36051' = '6840 '
'36053' = '8160 '
'36055' = '6840 '
'36057' = '0160 '
'36059' = '5380 '
'36061' = '5600 '
'36063' = '1280 '
'36065' = '8680 '
'36067' = '8160 '
'36069' = '6840 '
'36071' = '5660 '
'36073' = '6840 '
'36075' = '8160 '
'36079' = '5600 '
'36081' = '5600 '
'36083' = '0160 '
'36085' = '5600 '
'36087' = '5600 '
'36091' = '0160 '
'36093' = '0160 '
'36095' = '0160 '
'36103' = '5380 '
'36107' = '0960 '
'36113' = '2975 '
'36115' = '2975 '
'36117' = '6840 '
'36119' = '5600 '
'37001' = '3120 '
'37003' = '3290 '
'37019' = '9200 '
'37021' = '0480 '
'37023' = '3290 '
'37025' = '1520 '
'37027' = '3290 '
'37035' = '3290 '
'37037' = '6640 '
'37051' = '2560 '
'37053' = '5720 '
'37057' = '3120 '
'37059' = '3120 '
'37063' = '6640 '
'37065' = '6895 '
'37067' = '3120 '
'37069' = '6640 '
'37071' = '1520 '
'37081' = '3120 '
'37101' = '6640 '
'37109' = '1520 '
'37115' = '0480 '
'37119' = '1520 '
'37127' = '6895 '
'37129' = '9200 '
'37133' = '3605 '
'37135' = '6640 '
'37147' = '3150 '
'37151' = '3120 '
'37159' = '1520 '
'37169' = '3120 '
'37179' = '1520 '
'37183' = '6640 '
'37191' = '2980 '
'37197' = '3120 '
'38015' = '1010 '
'38017' = '2520 '
'38035' = '2985 '
'38059' = '1010 '
'39003' = '4320 '
'39007' = '1680 '
'39011' = '4320 '
'39013' = '9000 '
'39015' = '1640 '
'39017' = '3200 '
'39019' = '1320 '
'39023' = '2000 '
'39025' = '1640 '
'39029' = '9320 '
'39033' = '4800 '
'39035' = '1680 '
'39041' = '1840 '
'39045' = '1840 '
'39049' = '1840 '
'39051' = '8400 '
'39055' = '1680 '
'39057' = '2000 '
'39061' = '1640 '
'39081' = '8080 '
'39085' = '1680 '
'39087' = '3400 '
'39089' = '1840 '
'39093' = '1680 '
'39095' = '8400 '
'39097' = '1840 '
'39099' = '9320 '
'39103' = '1680 '
'39109' = '2000 '
'39113' = '2000 '
'39129' = '1840 '
'39133' = '0080 '
'39139' = '4800 '
'39151' = '1320 '
'39153' = '0080 '
'39155' = '9320 '
'39165' = '1640 '
'39167' = '6020 '
'39173' = '8400 '
'40017' = '5880 '
'40027' = '5880 '
'40031' = '4200 '
'40037' = '8560 '
'40047' = '2340 '
'40083' = '5880 '
'40087' = '5880 '
'40109' = '5880 '
'40113' = '8560 '
'40125' = '5880 '
'40131' = '8560 '
'40135' = '2720 '
'40143' = '8560 '
'40145' = '8560 '
'41003' = '1890 '
'41005' = '6440 '
'41009' = '6440 '
'41029' = '4890 '
'41039' = '2400 '
'41047' = '7080 '
'41051' = '6440 '
'41053' = '7080 '
'41067' = '6440 '
'41071' = '6440 '
'42003' = '6280 '
'42007' = '6280 '
'42011' = '6680 '
'42013' = '0280 '
'42017' = '6160 '
'42019' = '6280 '
'42021' = '3680 '
'42025' = '0240 '
'42027' = '8050 '
'42029' = '6160 '
'42037' = '7560 '
'42041' = '3240 '
'42043' = '3240 '
'42045' = '6160 '
'42049' = '2360 '
'42051' = '6280 '
'42069' = '7560 '
'42071' = '4000 '
'42075' = '3240 '
'42077' = '0240 '
'42079' = '7560 '
'42081' = '9140 '
'42085' = '7610 '
'42091' = '6160 '
'42095' = '0240 '
'42099' = '3240 '
'42101' = '6160 '
'42103' = '5660 '
'42111' = '3680 '
'42125' = '6280 '
'42129' = '6280 '
'42131' = '7560 '
'42133' = '9280 '
'44001' = '6483 '
'44003' = '6483 '
'44007' = '6483 '
'44009' = '6483 '
'45003' = '0600 '
'45007' = '3160 '
'45015' = '1440 '
'45019' = '1440 '
'45021' = '3160 '
'45035' = '1440 '
'45037' = '0600 '
'45041' = '2655 '
'45045' = '3160 '
'45051' = '5330 '
'45063' = '1760 '
'45077' = '3160 '
'45079' = '1760 '
'45083' = '3160 '
'45085' = '8140 '
'45091' = '1520 '
'46083' = '7760 '
'46099' = '7760 '
'46103' = '6660 '
'47001' = '3840 '
'47009' = '3840 '
'47019' = '3660 '
'47021' = '5360 '
'47023' = '3580 '
'47037' = '5360 '
'47043' = '5360 '
'47047' = '4920 '
'47065' = '1560 '
'47073' = '3660 '
'47093' = '3840 '
'47105' = '3840 '
'47113' = '3580 '
'47115' = '1560 '
'47125' = '1660 '
'47147' = '5360 '
'47149' = '5360 '
'47155' = '3840 '
'47157' = '4920 '
'47163' = '3660 '
'47165' = '5360 '
'47167' = '4920 '
'47171' = '3660 '
'47173' = '3840 '
'47179' = '3660 '
'47187' = '5360 '
'47189' = '5360 '
'48009' = '9080 '
'48021' = '0640 '
'48027' = '3810 '
'48029' = '7240 '
'48037' = '8360 '
'48039' = '1145 '
'48041' = '1260 '
'48055' = '0640 '
'48061' = '1240 '
'48071' = '3360 '
'48085' = '1920 '
'48091' = '7240 '
'48099' = '3810 '
'48113' = '1920 '
'48121' = '1920 '
'48135' = '5800 '
'48139' = '1920 '
'48141' = '2320 '
'48157' = '3360 '
'48167' = '2920 '
'48181' = '7640 '
'48183' = '4420 '
'48187' = '7240 '
'48199' = '0840 '
'48201' = '3360 '
'48203' = '4420 '
'48209' = '0640 '
'48213' = '1920 '
'48215' = '4880 '
'48221' = '2800 '
'48231' = '1920 '
'48245' = '0840 '
'48251' = '2800 '
'48257' = '1920 '
'48291' = '3360 '
'48303' = '4600 '
'48309' = '8800 '
'48329' = '5800 '
'48339' = '3360 '
'48355' = '1880 '
'48361' = '0840 '
'48367' = '2800 '
'48375' = '0320 '
'48381' = '0320 '
'48397' = '1920 '
'48409' = '1880 '
'48423' = '8640 '
'48439' = '2800 '
'48441' = '0040 '
'48451' = '7200 '
'48453' = '0640 '
'48459' = '4420 '
'48469' = '8750 '
'48473' = '3360 '
'48479' = '4080 '
'48485' = '9080 '
'48491' = '0640 '
'48493' = '7240 '
'49011' = '7160 '
'49025' = '2620 '
'49035' = '7160 '
'49049' = '6520 '
'49057' = '7160 '
'50007' = '1303 '
'50011' = '1303 '
'50013' = '1303 '
'51003' = '1540 '
'51009' = '4640 '
'51013' = '8840 '
'51019' = '4640 '
'51023' = '6800 '
'51031' = '4640 '
'51036' = '6760 '
'51041' = '6760 '
'51043' = '8840 '
'51047' = '8840 '
'51053' = '6760 '
'51059' = '8840 '
'51061' = '8840 '
'51065' = '1540 '
'51073' = '5720 '
'51075' = '6760 '
'51079' = '1540 '
'51085' = '6760 '
'51087' = '6760 '
'51093' = '5720 '
'51095' = '5720 '
'51099' = '8840 '
'51107' = '8840 '
'51115' = '5720 '
'51127' = '6760 '
'51143' = '1950 '
'51145' = '6760 '
'51149' = '6760 '
'51153' = '8840 '
'51161' = '6800 '
'51169' = '3660 '
'51177' = '8840 '
'51179' = '8840 '
'51187' = '8840 '
'51191' = '3660 '
'51199' = '5720 '
'51510' = '8840 '
'51515' = '4640 '
'51520' = '3660 '
'51540' = '1540 '
'51550' = '5720 '
'51570' = '6760 '
'51590' = '1950 '
'51600' = '8840 '
'51610' = '8840 '
'51630' = '8840 '
'51650' = '5720 '
'51670' = '6760 '
'51680' = '4640 '
'51683' = '8840 '
'51685' = '8840 '
'51700' = '5720 '
'51710' = '5720 '
'51730' = '6760 '
'51735' = '5720 '
'51740' = '5720 '
'51760' = '6760 '
'51770' = '6800 '
'51775' = '6800 '
'51800' = '5720 '
'51810' = '5720 '
'51830' = '5720 '
'53005' = '6740 '
'53011' = '6440 '
'53021' = '6740 '
'53029' = '7600 '
'53033' = '7600 '
'53035' = '1150 '
'53053' = '8200 '
'53061' = '7600 '
'53063' = '7840 '
'53067' = '5910 '
'53073' = '0860 '
'53077' = '9260 '
'54003' = '8840 '
'54009' = '8080 '
'54011' = '3400 '
'54029' = '8080 '
'54037' = '8840 '
'54039' = '1480 '
'54051' = '9000 '
'54057' = '1900 '
'54069' = '9000 '
'54079' = '1480 '
'54099' = '3400 '
'54107' = '6020 '
'55009' = '3080 '
'55015' = '0460 '
'55017' = '2290 '
'55025' = '4720 '
'55031' = '2240 '
'55035' = '2290 '
'55059' = '3800 '
'55063' = '3870 '
'55073' = '8940 '
'55079' = '5080 '
'55087' = '0460 '
'55089' = '5080 '
'55093' = '5120 '
'55101' = '6600 '
'55105' = '3620 '
'55109' = '5120 '
'55117' = '7620 '
'55131' = '5080 '
'55133' = '5080 '
'55139' = '0460 '
'56021' = '1580 '
'56025' = '1350 '
;

/* OMB 2003 METRO AREA */

VALUE $M3AREA
'01001' = '33860'
'01003' = '19300'
'01005' = '21640'
'01007' = '13820'
'01009' = '13820'
'01015' = '11500'
'01017' = '46740'
'01021' = '13820'
'01031' = '21460'
'01033' = '22520'
'01037' = '10760'
'01043' = '18980'
'01045' = '21460'
'01047' = '42820'
'01049' = '22840'
'01051' = '33860'
'01055' = '23460'
'01061' = '20020'
'01063' = '46220'
'01065' = '46220'
'01067' = '20020'
'01069' = '20020'
'01071' = '42460'
'01073' = '13820'
'01077' = '22520'
'01079' = '19460'
'01081' = '12220'
'01083' = '26620'
'01085' = '33860'
'01087' = '46260'
'01089' = '26620'
'01095' = '10700'
'01097' = '33660'
'01101' = '33860'
'01103' = '19460'
'01109' = '45980'
'01113' = '17980'
'01115' = '13820'
'01117' = '13820'
'01121' = '45180'
'01123' = '10760'
'01125' = '46220'
'01127' = '13820'
'02020' = '11260'
'02090' = '21820'
'02110' = '27940'
'02130' = '28540'
'02150' = '28980'
'02170' = '11260'
'04003' = '43420'
'04005' = '22380'
'04007' = '37740'
'04009' = '40940'
'04011' = '40940'
'04013' = '38060'
'04015' = '29420'
'04019' = '46060'
'04021' = '38060'
'04023' = '35700'
'04025' = '39140'
'04027' = '49740'
'05005' = '34260'
'05007' = '22220'
'05009' = '25460'
'05013' = '15780'
'05019' = '11660'
'05025' = '38220'
'05027' = '31620'
'05031' = '27860'
'05033' = '22900'
'05035' = '32820'
'05045' = '30780'
'05047' = '22900'
'05051' = '26300'
'05053' = '30780'
'05055' = '37500'
'05057' = '26260'
'05063' = '12900'
'05069' = '38220'
'05079' = '38220'
'05085' = '30780'
'05087' = '22220'
'05091' = '45500'
'05093' = '14180'
'05099' = '26260'
'05101' = '25460'
'05103' = '15780'
'05105' = '30780'
'05107' = '48340'
'05111' = '27860'
'05115' = '40780'
'05119' = '30780'
'05123' = '22620'
'05125' = '30780'
'05131' = '22900'
'05139' = '20980'
'05143' = '22220'
'05145' = '42620'
'05149' = '40780'
'06001' = '41860'
'06007' = '17020'
'06013' = '41860'
'06015' = '18860'
'06017' = '40900'
'06019' = '23420'
'06023' = '21700'
'06025' = '20940'
'06027' = '13860'
'06029' = '12540'
'06031' = '25260'
'06033' = '17340'
'06035' = '45000'
'06037' = '31100'
'06039' = '31460'
'06041' = '41860'
'06045' = '46380'
'06047' = '32900'
'06053' = '41500'
'06055' = '34900'
'06057' = '46020'
'06059' = '31100'
'06061' = '40900'
'06065' = '40140'
'06067' = '40900'
'06069' = '41940'
'06071' = '40140'
'06073' = '41740'
'06075' = '41860'
'06077' = '44700'
'06079' = '42020'
'06081' = '41860'
'06083' = '42060'
'06085' = '41940'
'06087' = '42100'
'06089' = '39820'
'06095' = '46700'
'06097' = '42220'
'06099' = '33700'
'06101' = '49700'
'06103' = '39780'
'06107' = '47300'
'06109' = '38020'
'06111' = '37100'
'06113' = '40900'
'06115' = '49700'
'08001' = '19740'
'08005' = '19740'
'08013' = '14500'
'08014' = '19740'
'08019' = '19740'
'08031' = '19740'
'08035' = '19740'
'08037' = '20780'
'08039' = '19740'
'08041' = '17820'
'08043' = '15860'
'08047' = '19740'
'08059' = '19740'
'08065' = '20780'
'08067' = '20420'
'08069' = '22660'
'08075' = '44540'
'08077' = '24300'
'08085' = '33940'
'08087' = '22820'
'08093' = '19740'
'08101' = '39380'
'08117' = '43540'
'08119' = '17820'
'08123' = '24540'
'09001' = '14860'
'09003' = '25540'
'09005' = '45860'
'09007' = '25540'
'09009' = '35300'
'09011' = '35980'
'09013' = '25540'
'09015' = '48740'
'10001' = '20100'
'10003' = '37980'
'10005' = '42580'
'11001' = '47900'
'12001' = '23540'
'12003' = '27260'
'12005' = '37460'
'12009' = '37340'
'12011' = '33100'
'12015' = '39460'
'12017' = '26140'
'12019' = '27260'
'12021' = '34940'
'12023' = '29380'
'12027' = '11580'
'12031' = '27260'
'12033' = '37860'
'12035' = '37380'
'12039' = '45220'
'12041' = '23540'
'12049' = '48100'
'12051' = '17500'
'12053' = '45300'
'12055' = '42700'
'12057' = '45300'
'12061' = '46940'
'12065' = '45220'
'12069' = '36740'
'12071' = '15980'
'12073' = '45220'
'12081' = '42260'
'12083' = '36100'
'12085' = '38940'
'12086' = '33100'
'12087' = '28580'
'12089' = '27260'
'12091' = '23020'
'12093' = '36380'
'12095' = '36740'
'12097' = '36740'
'12099' = '33100'
'12101' = '45300'
'12103' = '45300'
'12105' = '29460'
'12107' = '37260'
'12109' = '27260'
'12111' = '38940'
'12113' = '37860'
'12115' = '42260'
'12117' = '36740'
'12119' = '45540'
'12127' = '19660'
'12129' = '45220'
'13003' = '20060'
'13007' = '10500'
'13009' = '33300'
'13013' = '12060'
'13015' = '12060'
'13017' = '22340'
'13021' = '31420'
'13025' = '15260'
'13027' = '46660'
'13029' = '42340'
'13031' = '44340'
'13033' = '12260'
'13035' = '12060'
'13039' = '41220'
'13045' = '12060'
'13047' = '16860'
'13051' = '42340'
'13053' = '17980'
'13055' = '44900'
'13057' = '12060'
'13059' = '12020'
'13063' = '12060'
'13067' = '12060'
'13069' = '20060'
'13071' = '34220'
'13073' = '12260'
'13077' = '12060'
'13079' = '31420'
'13081' = '18380'
'13083' = '16860'
'13085' = '12060'
'13087' = '12460'
'13089' = '12060'
'13095' = '10500'
'13097' = '12060'
'13101' = '46660'
'13103' = '42340'
'13113' = '12060'
'13115' = '40660'
'13117' = '12060'
'13121' = '12060'
'13127' = '15260'
'13129' = '15660'
'13135' = '12060'
'13137' = '18460'
'13139' = '23580'
'13141' = '33300'
'13143' = '12060'
'13145' = '17980'
'13149' = '12060'
'13151' = '12060'
'13153' = '47580'
'13155' = '22340'
'13159' = '12060'
'13167' = '20140'
'13169' = '31420'
'13171' = '12060'
'13173' = '46660'
'13175' = '20140'
'13177' = '10500'
'13179' = '25980'
'13183' = '25980'
'13185' = '46660'
'13189' = '12260'
'13191' = '15260'
'13195' = '12020'
'13197' = '17980'
'13199' = '12060'
'13207' = '31420'
'13209' = '47080'
'13213' = '19140'
'13215' = '17980'
'13217' = '12060'
'13219' = '12020'
'13221' = '12020'
'13223' = '12060'
'13225' = '22980'
'13227' = '12060'
'13229' = '48180'
'13231' = '12060'
'13233' = '16340'
'13239' = '21640'
'13245' = '12260'
'13247' = '12060'
'13249' = '11140'
'13255' = '12060'
'13257' = '45740'
'13261' = '11140'
'13273' = '10500'
'13275' = '45620'
'13277' = '45700'
'13279' = '47080'
'13285' = '29300'
'13289' = '31420'
'13293' = '45580'
'13295' = '16860'
'13297' = '12060'
'13299' = '48180'
'13305' = '27700'
'13313' = '19140'
'13321' = '10500'
'15001' = '25900'
'15003' = '26180'
'15007' = '28180'
'15009' = '27980'
'16001' = '14260'
'16005' = '38540'
'16011' = '13940'
'16015' = '14260'
'16019' = '26820'
'16027' = '14260'
'16031' = '15420'
'16039' = '34300'
'16041' = '30860'
'16043' = '39940'
'16045' = '14260'
'16051' = '26820'
'16053' = '46300'
'16055' = '17660'
'16057' = '34140'
'16065' = '39940'
'16067' = '15420'
'16069' = '30300'
'16073' = '14260'
'16075' = '36620'
'16077' = '38540'
'16081' = '27220'
'16083' = '46300'
'17001' = '39500'
'17003' = '16020'
'17005' = '41180'
'17007' = '40420'
'17011' = '36860'
'17013' = '41180'
'17019' = '16580'
'17021' = '45380'
'17027' = '41180'
'17029' = '16660'
'17031' = '16980'
'17035' = '16660'
'17037' = '16980'
'17043' = '16980'
'17049' = '20820'
'17053' = '16580'
'17057' = '15900'
'17063' = '16980'
'17065' = '34500'
'17071' = '15460'
'17073' = '19340'
'17077' = '16060'
'17081' = '34500'
'17083' = '41180'
'17089' = '16980'
'17091' = '28100'
'17093' = '16980'
'17095' = '23660'
'17097' = '16980'
'17099' = '36860'
'17103' = '19940'
'17105' = '38700'
'17107' = '30660'
'17109' = '31380'
'17111' = '16980'
'17113' = '14060'
'17115' = '19500'
'17117' = '41180'
'17119' = '41180'
'17121' = '16460'
'17123' = '37900'
'17127' = '37140'
'17129' = '44100'
'17131' = '19340'
'17133' = '41180'
'17137' = '27300'
'17141' = '40300'
'17143' = '37900'
'17147' = '16580'
'17155' = '36860'
'17161' = '19340'
'17163' = '41180'
'17165' = '25380'
'17167' = '44100'
'17171' = '27300'
'17175' = '37900'
'17177' = '23300'
'17179' = '37900'
'17183' = '19180'
'17187' = '23660'
'17195' = '44580'
'17197' = '16980'
'17199' = '32060'
'17201' = '40420'
'17203' = '37900'
'18001' = '19540'
'18003' = '23060'
'18005' = '18020'
'18007' = '29140'
'18011' = '26900'
'18013' = '26900'
'18015' = '29140'
'18017' = '30900'
'18019' = '31140'
'18021' = '45460'
'18023' = '23140'
'18027' = '47780'
'18029' = '17140'
'18031' = '24700'
'18033' = '12140'
'18035' = '34620'
'18037' = '27540'
'18039' = '21140'
'18041' = '18220'
'18043' = '31140'
'18047' = '17140'
'18051' = '21780'
'18053' = '31980'
'18055' = '14020'
'18057' = '26900'
'18059' = '26900'
'18061' = '31140'
'18063' = '26900'
'18065' = '35220'
'18067' = '29020'
'18069' = '26540'
'18071' = '42980'
'18073' = '16980'
'18077' = '31500'
'18079' = '35860'
'18081' = '26900'
'18083' = '47180'
'18085' = '47700'
'18089' = '16980'
'18091' = '33140'
'18093' = '13260'
'18095' = '11300'
'18097' = '26900'
'18099' = '38500'
'18103' = '37940'
'18105' = '14020'
'18107' = '18820'
'18109' = '26900'
'18111' = '16980'
'18113' = '28340'
'18115' = '17140'
'18119' = '14020'
'18125' = '27540'
'18127' = '16980'
'18129' = '21780'
'18133' = '26900'
'18141' = '43780'
'18143' = '42500'
'18145' = '26900'
'18151' = '11420'
'18153' = '45460'
'18157' = '29140'
'18159' = '29020'
'18163' = '21780'
'18165' = '45460'
'18167' = '45460'
'18169' = '47340'
'18173' = '21780'
'18175' = '31140'
'18177' = '39980'
'18179' = '23060'
'18183' = '23060'
'19011' = '16300'
'19013' = '47940'
'19015' = '14340'
'19017' = '47940'
'19021' = '44740'
'19033' = '32380'
'19041' = '43980'
'19045' = '17540'
'19049' = '19780'
'19057' = '15460'
'19059' = '44020'
'19061' = '20220'
'19075' = '47940'
'19077' = '19780'
'19085' = '36540'
'19099' = '35500'
'19103' = '26980'
'19105' = '16300'
'19111' = '28460'
'19113' = '16300'
'19115' = '34700'
'19121' = '19780'
'19123' = '36820'
'19125' = '37800'
'19127' = '32260'
'19129' = '36540'
'19139' = '34700'
'19153' = '19780'
'19155' = '36540'
'19163' = '19340'
'19169' = '11180'
'19179' = '36900'
'19181' = '19780'
'19183' = '26980'
'19187' = '22700'
'19193' = '43580'
'19195' = '32380'
'20005' = '11860'
'20009' = '24460'
'20015' = '48620'
'20017' = '21380'
'20035' = '49060'
'20037' = '38260'
'20043' = '41140'
'20045' = '29940'
'20051' = '25700'
'20055' = '23780'
'20057' = '19980'
'20059' = '28140'
'20061' = '31740'
'20079' = '48620'
'20085' = '45820'
'20087' = '45820'
'20091' = '28140'
'20099' = '37660'
'20103' = '28140'
'20107' = '28140'
'20111' = '21380'
'20113' = '32700'
'20121' = '28140'
'20125' = '17700'
'20139' = '45820'
'20143' = '41460'
'20149' = '31740'
'20155' = '26740'
'20161' = '31740'
'20169' = '41460'
'20173' = '48620'
'20175' = '30580'
'20177' = '45820'
'20191' = '48620'
'20197' = '45820'
'20209' = '28140'
'21005' = '23180'
'21007' = '37140'
'21009' = '23980'
'21011' = '34460'
'21013' = '33180'
'21015' = '17140'
'21017' = '30460'
'21019' = '26580'
'21021' = '19220'
'21023' = '17140'
'21029' = '31140'
'21035' = '34660'
'21037' = '17140'
'21047' = '17300'
'21049' = '30460'
'21059' = '36980'
'21061' = '14540'
'21067' = '30460'
'21073' = '23180'
'21075' = '46460'
'21077' = '17140'
'21081' = '17140'
'21083' = '32460'
'21089' = '26580'
'21091' = '36980'
'21093' = '21060'
'21101' = '21780'
'21103' = '31140'
'21107' = '31580'
'21111' = '31140'
'21113' = '30460'
'21117' = '17140'
'21123' = '21060'
'21125' = '30940'
'21135' = '32500'
'21137' = '19220'
'21139' = '37140'
'21145' = '37140'
'21149' = '36980'
'21151' = '40080'
'21161' = '32500'
'21163' = '31140'
'21165' = '34460'
'21169' = '23980'
'21173' = '34460'
'21177' = '16420'
'21179' = '31140'
'21185' = '31140'
'21191' = '17140'
'21199' = '43700'
'21203' = '40080'
'21209' = '30460'
'21211' = '31140'
'21215' = '31140'
'21217' = '15820'
'21221' = '17300'
'21223' = '31140'
'21227' = '14540'
'21233' = '21780'
'21235' = '18340'
'21239' = '30460'
'22001' = '18940'
'22005' = '12940'
'22007' = '38200'
'22011' = '19760'
'22015' = '43340'
'22017' = '43340'
'22019' = '29340'
'22023' = '29340'
'22029' = '35020'
'22031' = '43340'
'22033' = '12940'
'22037' = '12940'
'22043' = '10780'
'22045' = '35340'
'22047' = '12940'
'22049' = '40820'
'22051' = '35380'
'22053' = '27660'
'22055' = '29180'
'22057' = '26380'
'22061' = '40820'
'22063' = '12940'
'22065' = '45260'
'22067' = '12820'
'22069' = '35060'
'22071' = '35380'
'22073' = '33740'
'22075' = '35380'
'22077' = '12940'
'22079' = '10780'
'22087' = '35380'
'22089' = '35380'
'22091' = '12940'
'22095' = '35380'
'22097' = '36660'
'22099' = '29180'
'22101' = '34020'
'22103' = '35380'
'22105' = '25220'
'22109' = '26380'
'22111' = '33740'
'22113' = '10020'
'22115' = '22860'
'22117' = '14220'
'22119' = '33380'
'22121' = '12940'
'22125' = '12940'
'23001' = '30340'
'23005' = '38860'
'23011' = '12300'
'23013' = '40500'
'23019' = '12620'
'23023' = '38860'
'23031' = '38860'
'24001' = '19060'
'24003' = '12580'
'24005' = '12580'
'24009' = '47900'
'24013' = '12580'
'24015' = '37980'
'24017' = '47900'
'24019' = '15700'
'24021' = '47900'
'24025' = '12580'
'24027' = '12580'
'24031' = '47900'
'24033' = '47900'
'24035' = '12580'
'24037' = '30500'
'24039' = '41540'
'24041' = '20660'
'24043' = '25180'
'24045' = '41540'
'24047' = '36180'
'24510' = '12580'
'25001' = '12700'
'25003' = '38340'
'25005' = '39300'
'25009' = '14460'
'25011' = '44140'
'25013' = '44140'
'25015' = '44140'
'25017' = '14460'
'25021' = '14460'
'25023' = '14460'
'25025' = '14460'
'25027' = '49340'
'26005' = '10880'
'26007' = '10980'
'26015' = '24340'
'26017' = '13020'
'26019' = '45900'
'26021' = '35660'
'26023' = '17740'
'26025' = '12980'
'26027' = '43780'
'26033' = '42300'
'26037' = '29620'
'26041' = '21540'
'26043' = '27020'
'26045' = '29620'
'26049' = '22420'
'26055' = '45900'
'26057' = '10940'
'26061' = '26340'
'26065' = '29620'
'26067' = '24340'
'26073' = '34380'
'26075' = '27100'
'26077' = '28020'
'26079' = '45900'
'26081' = '24340'
'26083' = '26340'
'26087' = '19820'
'26089' = '45900'
'26091' = '10300'
'26093' = '19820'
'26099' = '19820'
'26103' = '32100'
'26107' = '13660'
'26109' = '31940'
'26111' = '33220'
'26113' = '15620'
'26115' = '33780'
'26121' = '34740'
'26123' = '24340'
'26125' = '19820'
'26139' = '26100'
'26145' = '40980'
'26147' = '19820'
'26149' = '44780'
'26155' = '37020'
'26159' = '28020'
'26161' = '11460'
'26163' = '19820'
'26165' = '15620'
'27003' = '33460'
'27007' = '13420'
'27009' = '41060'
'27013' = '31860'
'27015' = '35580'
'27017' = '20260'
'27019' = '33460'
'27021' = '14660'
'27025' = '33460'
'27027' = '22020'
'27035' = '14660'
'27037' = '33460'
'27039' = '40340'
'27041' = '10820'
'27047' = '10660'
'27049' = '39860'
'27053' = '33460'
'27055' = '29100'
'27059' = '33460'
'27067' = '48820'
'27083' = '32140'
'27085' = '26780'
'27091' = '21860'
'27099' = '12380'
'27103' = '31860'
'27105' = '49380'
'27109' = '40340'
'27111' = '22260'
'27119' = '24220'
'27123' = '33460'
'27131' = '22060'
'27137' = '20260'
'27139' = '33460'
'27141' = '33460'
'27145' = '41060'
'27147' = '36940'
'27157' = '40340'
'27163' = '33460'
'27167' = '47420'
'27169' = '49100'
'27171' = '33460'
'28001' = '35020'
'28003' = '18420'
'28005' = '32620'
'28011' = '17380'
'28015' = '24900'
'28023' = '32940'
'28025' = '48500'
'28027' = '17260'
'28029' = '27140'
'28033' = '32820'
'28035' = '25620'
'28039' = '37700'
'28043' = '24980'
'28045' = '25060'
'28047' = '25060'
'28049' = '27140'
'28057' = '46180'
'28059' = '37700'
'28061' = '29860'
'28067' = '29860'
'28069' = '32940'
'28071' = '37060'
'28073' = '25620'
'28075' = '32940'
'28081' = '46180'
'28083' = '24900'
'28085' = '15020'
'28087' = '18060'
'28089' = '27140'
'28093' = '32820'
'28105' = '44260'
'28109' = '38100'
'28111' = '25620'
'28113' = '32620'
'28115' = '46180'
'28121' = '27140'
'28127' = '27140'
'28131' = '25060'
'28133' = '26940'
'28137' = '32820'
'28143' = '32820'
'28149' = '46980'
'28151' = '24740'
'28163' = '49540'
'29001' = '28860'
'29003' = '41140'
'29007' = '33020'
'29013' = '28140'
'29017' = '16020'
'29019' = '17860'
'29021' = '41140'
'29023' = '38740'
'29025' = '28140'
'29027' = '27620'
'29031' = '16020'
'29037' = '28140'
'29043' = '44180'
'29045' = '28460'
'29047' = '28140'
'29049' = '28140'
'29051' = '27620'
'29059' = '44180'
'29063' = '41140'
'29069' = '28380'
'29071' = '41180'
'29077' = '44180'
'29089' = '17860'
'29091' = '48460'
'29095' = '28140'
'29097' = '27900'
'29099' = '41180'
'29101' = '47660'
'29105' = '30060'
'29107' = '28140'
'29111' = '39500'
'29113' = '41180'
'29119' = '22220'
'29127' = '25300'
'29135' = '27620'
'29145' = '27900'
'29147' = '32340'
'29151' = '27620'
'29159' = '42740'
'29161' = '40620'
'29165' = '28140'
'29167' = '44180'
'29169' = '22780'
'29173' = '25300'
'29175' = '33620'
'29177' = '28140'
'29183' = '41180'
'29187' = '22100'
'29189' = '41180'
'29195' = '32180'
'29197' = '28860'
'29201' = '43460'
'29209' = '14700'
'29213' = '14700'
'29219' = '41180'
'29221' = '41180'
'29225' = '44180'
'29510' = '41180'
'30009' = '13740'
'30013' = '24500'
'30029' = '28060'
'30031' = '14580'
'30041' = '25660'
'30043' = '25740'
'30049' = '25740'
'30063' = '33540'
'30093' = '15580'
'30111' = '13740'
'31001' = '25580'
'31007' = '42420'
'31019' = '28260'
'31025' = '36540'
'31035' = '25580'
'31043' = '43580'
'31047' = '30420'
'31051' = '43580'
'31053' = '23340'
'31055' = '36540'
'31067' = '13100'
'31073' = '30420'
'31079' = '24260'
'31093' = '24260'
'31099' = '28260'
'31109' = '30700'
'31111' = '35820'
'31113' = '35820'
'31117' = '35820'
'31119' = '35740'
'31121' = '24260'
'31139' = '35740'
'31141' = '18100'
'31153' = '36540'
'31155' = '36540'
'31157' = '42420'
'31159' = '30700'
'31167' = '35740'
'31177' = '36540'
'32001' = '21980'
'32003' = '29820'
'32005' = '23820'
'32007' = '21220'
'32011' = '21220'
'32023' = '37220'
'32029' = '39900'
'32031' = '39900'
'32510' = '16180'
'33001' = '29060'
'33005' = '28300'
'33007' = '13620'
'33009' = '30100'
'33011' = '31700'
'33013' = '18180'
'33015' = '14460'
'33017' = '14460'
'33019' = '17200'
'34001' = '12100'
'34003' = '35620'
'34005' = '37980'
'34007' = '37980'
'34009' = '36140'
'34011' = '47220'
'34013' = '35620'
'34015' = '37980'
'34017' = '35620'
'34019' = '35620'
'34021' = '45940'
'34023' = '35620'
'34025' = '35620'
'34027' = '35620'
'34029' = '35620'
'34031' = '35620'
'34033' = '37980'
'34035' = '35620'
'34037' = '35620'
'34039' = '35620'
'34041' = '10900'
'35001' = '10740'
'35005' = '40740'
'35006' = '24380'
'35009' = '17580'
'35013' = '29740'
'35015' = '16100'
'35017' = '43500'
'35025' = '26020'
'35028' = '31060'
'35029' = '19700'
'35031' = '23700'
'35035' = '10460'
'35039' = '21580'
'35041' = '38780'
'35043' = '10740'
'35045' = '22140'
'35047' = '29780'
'35049' = '42140'
'35055' = '45340'
'35057' = '10740'
'35061' = '10740'
'36001' = '10580'
'36005' = '35620'
'36007' = '13780'
'36009' = '36460'
'36011' = '12180'
'36013' = '27460'
'36015' = '21300'
'36019' = '38460'
'36021' = '26460'
'36023' = '18660'
'36027' = '39100'
'36029' = '15380'
'36033' = '31660'
'36035' = '24100'
'36037' = '12860'
'36043' = '46540'
'36045' = '48060'
'36047' = '35620'
'36051' = '40380'
'36053' = '45060'
'36055' = '40380'
'36057' = '11220'
'36059' = '35620'
'36061' = '35620'
'36063' = '15380'
'36065' = '46540'
'36067' = '45060'
'36069' = '40380'
'36071' = '39100'
'36073' = '40380'
'36075' = '45060'
'36077' = '36580'
'36079' = '35620'
'36081' = '35620'
'36083' = '10580'
'36085' = '35620'
'36087' = '35620'
'36089' = '36300'
'36091' = '10580'
'36093' = '10580'
'36095' = '10580'
'36099' = '42900'
'36101' = '18500'
'36103' = '35620'
'36107' = '13780'
'36109' = '27060'
'36111' = '28740'
'36113' = '24020'
'36115' = '24020'
'36117' = '40380'
'36119' = '35620'
'37001' = '15500'
'37003' = '25860'
'37007' = '16740'
'37013' = '47820'
'37019' = '48900'
'37021' = '11700'
'37023' = '25860'
'37025' = '16740'
'37027' = '25860'
'37029' = '21020'
'37031' = '33980'
'37035' = '25860'
'37037' = '20500'
'37045' = '43140'
'37049' = '35100'
'37051' = '22180'
'37053' = '47260'
'37055' = '28620'
'37057' = '45640'
'37059' = '49180'
'37063' = '20500'
'37065' = '40580'
'37067' = '49180'
'37069' = '39580'
'37071' = '16740'
'37079' = '24780'
'37081' = '24660'
'37083' = '40260'
'37085' = '20380'
'37087' = '11700'
'37089' = '11700'
'37093' = '22180'
'37097' = '44380'
'37101' = '39580'
'37103' = '35100'
'37105' = '41820'
'37107' = '28820'
'37109' = '30740'
'37115' = '11700'
'37119' = '16740'
'37125' = '43860'
'37127' = '40580'
'37129' = '48900'
'37131' = '40260'
'37133' = '27340'
'37135' = '20500'
'37137' = '35100'
'37139' = '21020'
'37141' = '48900'
'37143' = '21020'
'37145' = '20500'
'37147' = '24780'
'37151' = '24660'
'37153' = '40460'
'37155' = '31300'
'37157' = '24660'
'37159' = '41580'
'37161' = '22580'
'37165' = '29900'
'37167' = '10620'
'37169' = '49180'
'37171' = '34340'
'37175' = '14820'
'37179' = '16740'
'37181' = '25780'
'37183' = '39580'
'37189' = '14380'
'37191' = '24140'
'37193' = '35900'
'37195' = '48980'
'37197' = '49180'
'38007' = '19860'
'38015' = '13900'
'38017' = '22020'
'38035' = '24220'
'38049' = '33500'
'38059' = '13900'
'38075' = '33500'
'38077' = '47420'
'38089' = '19860'
'38093' = '27420'
'38101' = '33500'
'38105' = '48780'
'39003' = '30620'
'39005' = '11740'
'39007' = '11780'
'39009' = '11900'
'39011' = '47540'
'39013' = '48540'
'39015' = '17140'
'39017' = '17140'
'39019' = '15940'
'39021' = '46500'
'39023' = '44220'
'39025' = '17140'
'39027' = '48940'
'39029' = '20620'
'39031' = '18740'
'39033' = '15340'
'39035' = '17460'
'39037' = '24820'
'39039' = '19580'
'39041' = '18140'
'39043' = '41780'
'39045' = '18140'
'39047' = '47860'
'39049' = '18140'
'39051' = '45780'
'39053' = '38580'
'39055' = '17460'
'39057' = '19380'
'39059' = '15740'
'39061' = '17140'
'39063' = '22300'
'39077' = '35940'
'39081' = '48260'
'39083' = '34540'
'39085' = '17460'
'39087' = '26580'
'39089' = '18140'
'39091' = '13340'
'39093' = '17460'
'39095' = '45780'
'39097' = '18140'
'39099' = '49660'
'39101' = '32020'
'39103' = '17460'
'39107' = '16380'
'39109' = '19380'
'39113' = '19380'
'39117' = '18140'
'39119' = '49780'
'39123' = '45780'
'39129' = '18140'
'39133' = '10420'
'39135' = '19380'
'39139' = '31900'
'39141' = '17060'
'39143' = '23380'
'39145' = '39020'
'39147' = '45660'
'39149' = '43380'
'39151' = '15940'
'39153' = '10420'
'39155' = '49660'
'39157' = '35420'
'39159' = '18140'
'39161' = '46780'
'39165' = '17140'
'39167' = '37620'
'39169' = '49300'
'39173' = '45780'
'40009' = '21120'
'40013' = '20460'
'40017' = '36420'
'40019' = '11620'
'40021' = '45140'
'40027' = '36420'
'40031' = '30020'
'40037' = '46140'
'40047' = '21420'
'40051' = '36420'
'40065' = '11060'
'40071' = '38620'
'40079' = '22900'
'40081' = '36420'
'40083' = '36420'
'40085' = '11620'
'40087' = '36420'
'40101' = '34780'
'40109' = '36420'
'40111' = '46140'
'40113' = '46140'
'40115' = '33060'
'40117' = '46140'
'40119' = '44660'
'40121' = '32540'
'40123' = '10220'
'40125' = '43060'
'40131' = '46140'
'40135' = '22900'
'40137' = '20340'
'40139' = '25100'
'40143' = '46140'
'40145' = '46140'
'40147' = '12780'
'40153' = '49260'
'41003' = '18700'
'41005' = '38900'
'41007' = '11820'
'41009' = '38900'
'41011' = '18300'
'41013' = '39260'
'41015' = '15060'
'41017' = '13460'
'41019' = '40700'
'41027' = '26220'
'41029' = '32780'
'41033' = '24420'
'41035' = '28900'
'41039' = '21660'
'41043' = '10540'
'41045' = '36620'
'41047' = '41420'
'41049' = '37820'
'41051' = '38900'
'41053' = '41420'
'41059' = '37820'
'41061' = '29260'
'41065' = '17180'
'41067' = '38900'
'41071' = '38900'
'42001' = '23900'
'42003' = '38300'
'42005' = '38300'
'42007' = '38300'
'42011' = '39740'
'42013' = '11020'
'42015' = '42380'
'42017' = '37980'
'42019' = '38300'
'42021' = '27780'
'42025' = '10900'
'42027' = '44300'
'42029' = '37980'
'42033' = '20180'
'42035' = '30820'
'42037' = '14100'
'42039' = '32740'
'42041' = '25420'
'42043' = '25420'
'42045' = '37980'
'42047' = '41260'
'42049' = '21500'
'42051' = '38300'
'42055' = '16540'
'42061' = '26500'
'42063' = '26860'
'42069' = '42540'
'42071' = '29540'
'42073' = '35260'
'42075' = '30140'
'42077' = '10900'
'42079' = '42540'
'42081' = '48700'
'42083' = '14620'
'42085' = '49660'
'42087' = '30380'
'42089' = '20700'
'42091' = '37980'
'42093' = '14100'
'42095' = '10900'
'42097' = '44980'
'42099' = '25420'
'42101' = '37980'
'42103' = '35620'
'42107' = '39060'
'42109' = '42780'
'42111' = '43740'
'42119' = '30260'
'42121' = '36340'
'42123' = '47620'
'42125' = '38300'
'42129' = '38300'
'42131' = '42540'
'42133' = '49620'
'44001' = '39300'
'44003' = '39300'
'44005' = '39300'
'44007' = '39300'
'44009' = '39300'
'45003' = '12260'
'45007' = '11340'
'45013' = '25940'
'45015' = '16700'
'45017' = '17900'
'45019' = '16700'
'45021' = '23500'
'45023' = '16900'
'45029' = '47500'
'45031' = '22500'
'45033' = '19900'
'45035' = '16700'
'45037' = '12260'
'45039' = '17900'
'45041' = '22500'
'45043' = '23860'
'45045' = '24860'
'45047' = '24940'
'45051' = '34820'
'45053' = '25940'
'45055' = '17900'
'45057' = '29580'
'45059' = '24860'
'45063' = '17900'
'45069' = '13500'
'45071' = '35140'
'45073' = '42860'
'45075' = '36700'
'45077' = '24860'
'45079' = '17900'
'45081' = '17900'
'45083' = '43900'
'45085' = '44940'
'45087' = '46420'
'45091' = '16740'
'46005' = '26700'
'46011' = '15100'
'46013' = '10100'
'46027' = '46820'
'46029' = '47980'
'46035' = '33580'
'46045' = '10100'
'46057' = '47980'
'46061' = '33580'
'46065' = '38180'
'46081' = '43940'
'46083' = '43620'
'46087' = '43620'
'46093' = '39660'
'46099' = '43620'
'46103' = '39660'
'46117' = '38180'
'46125' = '43620'
'46127' = '43580'
'46135' = '49460'
'47001' = '28940'
'47003' = '43180'
'47009' = '28940'
'47011' = '17420'
'47013' = '29220'
'47015' = '34980'
'47019' = '27740'
'47021' = '34980'
'47023' = '27180'
'47029' = '35460'
'47031' = '46100'
'47035' = '18900'
'47037' = '34980'
'47043' = '34980'
'47045' = '20540'
'47047' = '32820'
'47051' = '46100'
'47053' = '26480'
'47057' = '34100'
'47059' = '24620'
'47063' = '34100'
'47065' = '16860'
'47073' = '28700'
'47075' = '15140'
'47079' = '37540'
'47081' = '34980'
'47087' = '18260'
'47089' = '34100'
'47093' = '28940'
'47099' = '29980'
'47105' = '28940'
'47107' = '11940'
'47111' = '34980'
'47113' = '27180'
'47115' = '16860'
'47117' = '30280'
'47119' = '17940'
'47125' = '17300'
'47127' = '46100'
'47131' = '46460'
'47133' = '18260'
'47139' = '17420'
'47141' = '18260'
'47145' = '25340'
'47147' = '34980'
'47149' = '34980'
'47153' = '16860'
'47155' = '42940'
'47157' = '32820'
'47159' = '34980'
'47161' = '17300'
'47163' = '28700'
'47165' = '34980'
'47167' = '32820'
'47169' = '34980'
'47171' = '27740'
'47173' = '28940'
'47177' = '32660'
'47179' = '27740'
'47183' = '32280'
'47187' = '34980'
'47189' = '34980'
'48001' = '37300'
'48003' = '11380'
'48005' = '31260'
'48007' = '18580'
'48009' = '48660'
'48011' = '11100'
'48013' = '41700'
'48015' = '26420'
'48019' = '41700'
'48021' = '12420'
'48025' = '13300'
'48027' = '28660'
'48029' = '41700'
'48037' = '45500'
'48039' = '26420'
'48041' = '17780'
'48049' = '15220'
'48051' = '17780'
'48055' = '12420'
'48057' = '47020'
'48059' = '10180'
'48061' = '15180'
'48065' = '11100'
'48071' = '26420'
'48073' = '27380'
'48077' = '48660'
'48085' = '19100'
'48091' = '41700'
'48097' = '23620'
'48099' = '28660'
'48107' = '31180'
'48113' = '19100'
'48115' = '29500'
'48117' = '25820'
'48119' = '19100'
'48121' = '19100'
'48135' = '36220'
'48139' = '19100'
'48141' = '21340'
'48143' = '44500'
'48147' = '14300'
'48157' = '26420'
'48167' = '26420'
'48175' = '47020'
'48179' = '37420'
'48181' = '43300'
'48183' = '30980'
'48187' = '41700'
'48189' = '38380'
'48199' = '13140'
'48201' = '26420'
'48203' = '32220'
'48209' = '12420'
'48213' = '11980'
'48215' = '32580'
'48219' = '30220'
'48221' = '24180'
'48223' = '44860'
'48227' = '13700'
'48231' = '19100'
'48233' = '14420'
'48235' = '41660'
'48245' = '13140'
'48249' = '10860'
'48251' = '19100'
'48253' = '10180'
'48257' = '19100'
'48259' = '41700'
'48261' = '28780'
'48265' = '28500'
'48273' = '28780'
'48277' = '37580'
'48281' = '28660'
'48291' = '26420'
'48303' = '31180'
'48309' = '47380'
'48321' = '13060'
'48323' = '20580'
'48325' = '41700'
'48329' = '33260'
'48339' = '26420'
'48341' = '20300'
'48347' = '34860'
'48349' = '18620'
'48353' = '45020'
'48355' = '18580'
'48361' = '13140'
'48363' = '33420'
'48367' = '19100'
'48375' = '11100'
'48381' = '11100'
'48389' = '37780'
'48393' = '37420'
'48395' = '17780'
'48397' = '19100'
'48401' = '30980'
'48407' = '26420'
'48409' = '18580'
'48415' = '43660'
'48423' = '46340'
'48425' = '24180'
'48427' = '40100'
'48439' = '19100'
'48441' = '10180'
'48449' = '34420'
'48451' = '41660'
'48453' = '12420'
'48459' = '30980'
'48463' = '46620'
'48465' = '19620'
'48469' = '47020'
'48471' = '26660'
'48473' = '26420'
'48477' = '14780'
'48479' = '29700'
'48481' = '20900'
'48485' = '48660'
'48487' = '46900'
'48489' = '39700'
'48491' = '12420'
'48493' = '41700'
'48497' = '19100'
'49003' = '14940'
'49005' = '30860'
'49007' = '39220'
'49011' = '36260'
'49021' = '16260'
'49023' = '39340'
'49029' = '36260'
'49035' = '41620'
'49043' = '41620'
'49045' = '41620'
'49047' = '46860'
'49049' = '39340'
'49051' = '25720'
'49053' = '41100'
'49057' = '36260'
'50003' = '13540'
'50007' = '15540'
'50009' = '13620'
'50011' = '15540'
'50013' = '15540'
'50017' = '30100'
'50021' = '40860'
'50023' = '12740'
'50027' = '30100'
'51003' = '16820'
'51007' = '40060'
'51009' = '31340'
'51011' = '31340'
'51013' = '47900'
'51015' = '44420'
'51019' = '31340'
'51023' = '40220'
'51031' = '31340'
'51033' = '40060'
'51036' = '40060'
'51041' = '40060'
'51043' = '47900'
'51045' = '40220'
'51047' = '19020'
'51049' = '40060'
'51053' = '40060'
'51059' = '47900'
'51061' = '47900'
'51065' = '16820'
'51067' = '40220'
'51069' = '49020'
'51071' = '13980'
'51073' = '47260'
'51075' = '40060'
'51079' = '16820'
'51085' = '40060'
'51087' = '40060'
'51089' = '32300'
'51093' = '47260'
'51095' = '47260'
'51097' = '40060'
'51101' = '40060'
'51107' = '47900'
'51109' = '40060'
'51115' = '47260'
'51121' = '13980'
'51125' = '16820'
'51127' = '40060'
'51143' = '19260'
'51145' = '40060'
'51149' = '40060'
'51153' = '47900'
'51155' = '13980'
'51161' = '40220'
'51165' = '25500'
'51169' = '28700'
'51177' = '47900'
'51179' = '47900'
'51181' = '47260'
'51183' = '40060'
'51185' = '14140'
'51187' = '47900'
'51191' = '28700'
'51199' = '47260'
'51510' = '47900'
'51515' = '31340'
'51520' = '28700'
'51540' = '16820'
'51550' = '47260'
'51570' = '40060'
'51590' = '19260'
'51600' = '47900'
'51610' = '47900'
'51630' = '47900'
'51650' = '47260'
'51660' = '25500'
'51670' = '40060'
'51680' = '31340'
'51683' = '47900'
'51685' = '47900'
'51690' = '32300'
'51700' = '47260'
'51710' = '47260'
'51730' = '40060'
'51735' = '47260'
'51740' = '47260'
'51750' = '13980'
'51760' = '40060'
'51770' = '40220'
'51775' = '40220'
'51790' = '44420'
'51800' = '47260'
'51810' = '47260'
'51820' = '44420'
'51830' = '47260'
'51840' = '49020'
'53003' = '30300'
'53005' = '28420'
'53007' = '48300'
'53009' = '38820'
'53011' = '38900'
'53015' = '31020'
'53017' = '48300'
'53021' = '28420'
'53025' = '34180'
'53027' = '10140'
'53029' = '36020'
'53033' = '42660'
'53035' = '14740'
'53037' = '21260'
'53041' = '16500'
'53045' = '43220'
'53053' = '42660'
'53057' = '34580'
'53059' = '38900'
'53061' = '42660'
'53063' = '44060'
'53067' = '36500'
'53071' = '47460'
'53073' = '13380'
'53075' = '39420'
'53077' = '49420'
'54003' = '25180'
'54005' = '16620'
'54009' = '48260'
'54011' = '26580'
'54015' = '16620'
'54017' = '17220'
'54019' = '36060'
'54027' = '49020'
'54029' = '48260'
'54033' = '17220'
'54037' = '47900'
'54039' = '16620'
'54043' = '16620'
'54049' = '21900'
'54051' = '48540'
'54053' = '38580'
'54055' = '14140'
'54057' = '19060'
'54061' = '34060'
'54065' = '25180'
'54069' = '48540'
'54073' = '37620'
'54077' = '34060'
'54079' = '16620'
'54081' = '13220'
'54091' = '17220'
'54099' = '26580'
'54105' = '37620'
'54107' = '37620'
'55009' = '24580'
'55015' = '11540'
'55017' = '20740'
'55021' = '31540'
'55025' = '31540'
'55027' = '13180'
'55031' = '20260'
'55033' = '32860'
'55035' = '20740'
'55037' = '27020'
'55039' = '22540'
'55043' = '38420'
'55045' = '33820'
'55049' = '31540'
'55055' = '48020'
'55059' = '16980'
'55061' = '24580'
'55063' = '29100'
'55069' = '32980'
'55071' = '31820'
'55073' = '48140'
'55075' = '31940'
'55079' = '33340'
'55083' = '24580'
'55087' = '11540'
'55089' = '33340'
'55093' = '33460'
'55097' = '44620'
'55101' = '39540'
'55105' = '27500'
'55109' = '33460'
'55111' = '12660'
'55117' = '43100'
'55127' = '48580'
'55131' = '33340'
'55133' = '33340'
'55139' = '36780'
'55141' = '49220'
'56001' = '29660'
'56005' = '23940'
'56013' = '40180'
'56021' = '16940'
'56025' = '16220'
'56033' = '43260'
'56037' = '40540'
'56039' = '27220'
'56041' = '21740'
;


/*============================================================================;
*  TITLE:
*          FORMAT LIBRARY FOR PATIENT SAFETY INDICATORS
*
*  DESCRIPTION:
*          COVARIATE FORMAT LIB FOR AHRQ PATIENT SAFETY INDICATORS;
*
*                 >>>  VERSION 2.1  <<<
*
*============================================================================;*/

 /* -------------------------------------------------------------- */
 /* ---   FORMAT FOR AGE STRATIFIERS                           --- */
 /* ---   MODULE 3. ASSIGNS ARRAY INDEX FOR AGE CATEGORIES     --- */
 /* -------------------------------------------------------------- */

VALUE AGEFMT
LOW - 4 = 1
 5 -  9 = 2
10 - 14 = 3
15 - 17 = 4
18 - 24 = 5
25 - 29 = 6
30 - 34 = 7
35 - 39 = 8
40 - 44 = 9
45 - 49 = 10
50 - 54 = 11
55 - 59 = 12
60 - 64 = 13
65 - 69 = 14
70 - 74 = 15
75 - 79 = 16
80 - 84 = 17
85 - HIGH = 18;

VALUE AGE2FMT
LOW - 24 = 1
25 - 29 = 1
30 - 34 = 2
35 - 39 = 2
40 - 44 = 3
45 - 49 = 3
50 - 54 = 4
55 - 59 = 4
60 - 64 = 5
65 - 69 = 5
70 - 74 = 6
75 - 79 = 6
80 - 84 = 7
85 - HIGH = 7
;
 /* -------------------------------------------------------------- */
 /* ---   FORMATS FOR DRG STRATIFIERS                          --- */
 /* ---   MODULE 3. ASSIGNS ARRAY INDEX FOR DRG CATEGORIES     --- */
 /* -------------------------------------------------------------- */

VALUE $DRGFMT
"001" = 101 /* CRANIOTOMY AGE >17 W CC  */
"002" = 101 /* CRANIOTOMY AGE >17 WO CC  */
"003" = 102 /* CRANIOTOMY AGE 0-17  */
"004" = 103 /* SPINAL PROCEDURES (NO LONGER VALID)  */
"005" = 104 /* EXTRACRANIAL VASCULAR PROCEDURES (NO LONGER VALID)  */
"006" = 105 /* CARPAL TUNNEL RELEASE  */
"007" = 106 /* PERIPH & CRANIAL NERVE & OTHER NERV SYST PROC W CC  */
"008" = 106 /* PERIPH & CRANIAL NERVE & OTHER NERV SYST PROC WO CC  */
"009" = 107 /* SPINAL DISORDERS & INJURIES  */
"010" = 108 /* NERVOUS SYSTEM NEOPLASMS W CC  */
"011" = 108 /* NERVOUS SYSTEM NEOPLASMS WO CC  */
"012" = 109 /* DEGENERATIVE NERVOUS SYSTEM DISORDERS  */
"013" = 110 /* MULTIPLE SCLEROSIS & CEREBELLAR ATAXIA  */
"014" = 111 /* INTRACRANIAL HEMORRHAGE & STROKE W INFARCT  */
"015" = 112 /* NONSPECIFIC CVA & PRECEREBRAL OCCLUSION WO INFARCT  */
"016" = 113 /* NONSPECIFIC CEREBROVASCULAR DISORDERS W CC  */
"017" = 113 /* NONSPECIFIC CEREBROVASCULAR DISORDERS WO CC  */
"018" = 114 /* CRANIAL & PERIPHERAL NERVE DISORDERS W CC  */
"019" = 114 /* CRANIAL & PERIPHERAL NERVE DISORDERS WO CC  */
"020" = 115 /* NERVOUS SYSTEM INFECTION EXCEPT VIRAL MENINGITIS (NO LONGER VALID)  */
"021" = 116 /* VIRAL MENINGITIS  */
"022" = 117 /* HYPERTENSIVE ENCEPHALOPATHY  */
"023" = 118 /* NONTRAUMATIC STUPOR & COMA  */
"024" = 119 /* SEIZURE & HEADACHE AGE >17 W CC (NO LONGER VALID)  */
"025" = 119 /* SEIZURE & HEADACHE AGE >17 WO CC (NO LONGER VALID)  */
"026" = 120 /* SEIZURE & HEADACHE AGE 0-17  */
"027" = 121 /* TRAUMATIC STUPOR & COMA, COMA >1 HR  */
"028" = 122 /* TRAUMATIC STUPOR & COMA, COMA <1 HR AGE >17 W CC  */
"029" = 122 /* TRAUMATIC STUPOR & COMA, COMA <1 HR AGE >17 WO CC  */
"030" = 123 /* TRAUMATIC STUPOR & COMA, COMA <1 HR AGE 0-17  */
"031" = 124 /* CONCUSSION AGE >17 W CC  */
"032" = 124 /* CONCUSSION AGE >17 WO CC  */
"033" = 125 /* CONCUSSION AGE 0-17  */
"034" = 126 /* OTHER DISORDERS OF NERVOUS SYSTEM W CC  */
"035" = 126 /* OTHER DISORDERS OF NERVOUS SYSTEM WO CC  */
"036" = 201 /* RETINAL PROCEDURES  */
"037" = 202 /* ORBITAL PROCEDURES  */
"038" = 203 /* PRIMARY IRIS PROCEDURES  */
"039" = 204 /* LENS PROCEDURES WITH OR WITHOUT VITRECTOMY  */
"040" = 205 /* EXTRAOCULAR PROCEDURES EXCEPT ORBIT AGE >17  */
"041" = 206 /* EXTRAOCULAR PROCEDURES EXCEPT ORBIT AGE 0-17  */
"042" = 207 /* INTRAOCULAR PROCEDURES EXCEPT RETINA, IRIS & LENS  */
"043" = 208 /* HYPHEMA  */
"044" = 209 /* ACUTE MAJOR EYE INFECTIONS  */
"045" = 210 /* NEUROLOGICAL EYE DISORDERS  */
"046" = 211 /* OTHER DISORDERS OF THE EYE AGE >17 W CC  */
"047" = 211 /* OTHER DISORDERS OF THE EYE AGE >17 WO CC  */
"048" = 212 /* OTHER DISORDERS OF THE EYE AGE 0-17  */
"049" = 301 /* MAJOR HEAD & NECK PROCEDURES  */
"050" = 302 /* SIALOADENECTOMY  */
"051" = 303 /* SALIVARY GLAND PROCEDURES EXCEPT SIALOADENECTOMY  */
"052" = 304 /* CLEFT LIP & PALATE REPAIR  */
"053" = 305 /* SINUS & MASTOID PROCEDURES AGE >17  */
"054" = 306 /* SINUS & MASTOID PROCEDURES AGE 0-17  */
"055" = 307 /* MISCELLANEOUS EAR, NOSE, MOUTH & THROAT PROCEDURES  */
"056" = 308 /* RHINOPLASTY  */
"057" = 309 /* T&A PROC, EXCEPT TONSILLECTOMY &/OR ADENOIDECTOMY ONLY, */
"058" = 310 /* T&A PROC, EXCEPT TONSILLECTOMY &/OR ADENOIDECTOMY ONLY, */
"059" = 311 /* TONSILLECTOMY &/OR ADENOIDECTOMY ONLY, AGE >17  */
"060" = 312 /* TONSILLECTOMY &/OR ADENOIDECTOMY ONLY, AGE 0-17  */
"061" = 313 /* MYRINGOTOMY W TUBE INSERTION AGE >17  */
"062" = 314 /* MYRINGOTOMY W TUBE INSERTION AGE 0-17  */
"063" = 315 /* OTHER EAR, NOSE, MOUTH & THROAT O.R. PROCEDURES  */
"064" = 316 /* EAR, NOSE, MOUTH & THROAT MALIGNANCY  */
"065" = 317 /* DYSEQUILIBRIUM  */
"066" = 318 /* EPISTAXIS  */
"067" = 319 /* EPIGLOTTITIS  */
"068" = 320 /* OTITIS MEDIA & URI AGE >17 W CC  */
"069" = 320 /* OTITIS MEDIA & URI AGE >17 WO CC  */
"070" = 321 /* OTITIS MEDIA & URI AGE 0-17  */
"071" = 322 /* LARYNGOTRACHEITIS  */
"072" = 323 /* NASAL TRAUMA & DEFORMITY  */
"073" = 324 /* OTHER EAR, NOSE, MOUTH & THROAT DIAGNOSES AGE >17  */
"074" = 325 /* OTHER EAR, NOSE, MOUTH & THROAT DIAGNOSES AGE 0-17  */
"075" = 401 /* MAJOR CHEST PROCEDURES  */
"076" = 402 /* OTHER RESP SYSTEM O.R. PROCEDURES W CC  */
"077" = 402 /* OTHER RESP SYSTEM O.R. PROCEDURES WO CC  */
"078" = 403 /* PULMONARY EMBOLISM  */
"079" = 404 /* RESPIRATORY INFECTIONS & INFLAMMATIONS AGE >17 W CC  */
"080" = 404 /* RESPIRATORY INFECTIONS & INFLAMMATIONS AGE >17 WO CC  */
"081" = 405 /* RESPIRATORY INFECTIONS & INFLAMMATIONS AGE 0-17  */
"082" = 406 /* RESPIRATORY NEOPLASMS  */
"083" = 407 /* MAJOR CHEST TRAUMA W CC  */
"084" = 407 /* MAJOR CHEST TRAUMA WO CC  */
"085" = 408 /* PLEURAL EFFUSION W CC  */
"086" = 408 /* PLEURAL EFFUSION WO CC  */
"087" = 409 /* PULMONARY EDEMA & RESPIRATORY FAILURE  */
"088" = 410 /* CHRONIC OBSTRUCTIVE PULMONARY DISEASE  */
"089" = 411 /* SIMPLE PNEUMONIA & PLEURISY AGE >17 W CC  */
"090" = 411 /* SIMPLE PNEUMONIA & PLEURISY AGE >17 WO CC  */
"091" = 412 /* SIMPLE PNEUMONIA & PLEURISY AGE 0-17  */
"092" = 413 /* INTERSTITIAL LUNG DISEASE W CC  */
"093" = 413 /* INTERSTITIAL LUNG DISEASE WO CC  */
"094" = 414 /* PNEUMOTHORAX W CC  */
"095" = 414 /* PNEUMOTHORAX WO CC  */
"096" = 415 /* BRONCHITIS & ASTHMA AGE >17 W CC  */
"097" = 415 /* BRONCHITIS & ASTHMA AGE >17 WO CC  */
"098" = 416 /* BRONCHITIS & ASTHMA AGE 0-17  */
"099" = 417 /* RESPIRATORY SIGNS & SYMPTOMS W CC  */
"100" = 417 /* RESPIRATORY SIGNS & SYMPTOMS WO CC  */
"101" = 418 /* OTHER RESPIRATORY SYSTEM DIAGNOSES W CC  */
"102" = 418 /* OTHER RESPIRATORY SYSTEM DIAGNOSES WO CC  */
"103" = 7701 /* HEART TRANSPLANT  */
"104" = 501 /* CARDIAC VALVE & OTH MAJOR CARDIOTHORACIC PROC W CARD CAT */
"105" = 502 /* CARDIAC VALVE & OTH MAJOR CARDIOTHORACIC PROC WO CARD CA */
"106" = 503 /* CORONARY BYPASS W PTCA  */
"107" = 504 /* CORONARY BYPASS W CARDIAC CATH (NO LONGER VALID)  */
"108" = 505 /* OTHER CARDIOTHORACIC PROCEDURES  */
"109" = 506 /* CORONARY BYPASS WO PTCA OR CARDIAC CATH (NO LONGER VALID */
"110" = 507 /* MAJOR CARDIOVASCULAR PROCEDURES W CC  */
"111" = 507 /* MAJOR CARDIOVASCULAR PROCEDURES WO CC  */
"112" = 508 /* PERCUTANEOUS CARDIOVASCULAR PROCEDURES (NO LONGER VALID) */
"113" = 509 /* AMPUTATION FOR CIRC SYSTEM DISORDERS EXCEPT UPPER LIMB & */
"114" = 510 /* UPPER LIMB & TOE AMPUTATION FOR CIRC SYSTEM DISORDERS  */
"115" = 511 /* PRM CARD PACEM IMPL W AMI/HR/SHOCK OR AICD LEAD OR GNRTR */
"116" = 512 /* OTHER PERMANENT CARDIAC PACEMAKER IMPLANT (NO LONGER VAL */
"117" = 513 /* CARDIAC PACEMAKER REVISION EXCEPT DEVICE REPLACEMENT  */
"118" = 514 /* CARDIAC PACEMAKER DEVICE REPLACEMENT  */
"119" = 515 /* VEIN LIGATION & STRIPPING  */
"120" = 516 /* OTHER CIRCULATORY SYSTEM O.R. PROCEDURES  */
"121" = 517 /* CIRCULATORY DISORDERS W AMI & MAJOR COMP, DISCHARGED ALI */
"122" = 517 /* CIRCULATORY DISORDERS W AMI WO MAJOR COMP, DISCHARGED AL */
"123" = 517 /* CIRCULATORY DISORDERS W AMI, EXPIRED  */
"124" = 518 /* CIRCULATORY DISORDERS EXCEPT AMI, W CARD CATH & COMPLEX */
"125" = 519 /* CIRCULATORY DISORDERS EXCEPT AMI, W CARD CATH W/O COMPLEX DIAG */
"126" = 520 /* ACUTE & SUBACUTE ENDOCARDITIS  */
"127" = 521 /* HEART FAILURE & SHOCK  */
"128" = 522 /* DEEP VEIN THROMBOPHLEBITIS  */
"129" = 523 /* CARDIAC ARREST, UNEXPLAINED  */
"130" = 524 /* PERIPHERAL VASCULAR DISORDERS W CC  */
"131" = 524 /* PERIPHERAL VASCULAR DISORDERS WO CC  */
"132" = 525 /* ATHEROSCLEROSIS W CC  */
"133" = 525 /* ATHEROSCLEROSIS WO CC  */
"134" = 526 /* HYPERTENSION  */
"135" = 527 /* CARDIAC CONGENITAL & VALVULAR DISORDERS AGE >17 W CC  */
"136" = 527 /* CARDIAC CONGENITAL & VALVULAR DISORDERS AGE >17 WO CC  */
"137" = 528 /* CARDIAC CONGENITAL & VALVULAR DISORDERS AGE 0-17  */
"138" = 529 /* CARDIAC ARRHYTHMIA & CONDUCTION DISORDERS W CC  */
"139" = 529 /* CARDIAC ARRHYTHMIA & CONDUCTION DISORDERS WO CC  */
"140" = 530 /* ANGINA PECTORIS  */
"141" = 531 /* SYNCOPE & COLLAPSE W CC  */
"142" = 531 /* SYNCOPE & COLLAPSE WO CC  */
"143" = 532 /* CHEST PAIN  */
"144" = 533 /* OTHER CIRCULATORY SYSTEM DIAGNOSES W CC  */
"145" = 533 /* OTHER CIRCULATORY SYSTEM DIAGNOSES WO CC  */
"146" = 601 /* RECTAL RESECTION W CC  */
"147" = 601 /* RECTAL RESECTION WO CC  */
"148" = 602 /* MAJOR SMALL & LARGE BOWEL PROCEDURES W CC (NO LONGER VALID)  */
"149" = 602 /* MAJOR SMALL & LARGE BOWEL PROCEDURES WO CC  */
"150" = 603 /* PERITONEAL ADHESIOLYSIS W CC  */
"151" = 603 /* PERITONEAL ADHESIOLYSIS WO CC  */
"152" = 604 /* MINOR SMALL & LARGE BOWEL PROCEDURES W CC  */
"153" = 604 /* MINOR SMALL & LARGE BOWEL PROCEDURES WO CC  */
"154" = 605 /* STOMACH, ESOPHAGEAL & DUODENAL PROCEDURES AGE >17 W CC (NO LONGER VALID)  */
"155" = 605 /* STOMACH, ESOPHAGEAL & DUODENAL PROCEDURES AGE >17 WO CC */
"156" = 606 /* STOMACH, ESOPHAGEAL & DUODENAL PROCEDURES AGE 0-17  */
"157" = 607 /* ANAL & STOMAL PROCEDURES W CC  */
"158" = 607 /* ANAL & STOMAL PROCEDURES WO CC  */
"159" = 608 /* HERNIA PROCEDURES EXCEPT INGUINAL & FEMORAL AGE >17 W CC */
"160" = 608 /* HERNIA PROCEDURES EXCEPT INGUINAL & FEMORAL AGE >17 WO C */
"161" = 609 /* INGUINAL & FEMORAL HERNIA PROCEDURES AGE >17 W CC  */
"162" = 609 /* INGUINAL & FEMORAL HERNIA PROCEDURES AGE >17 WO CC  */
"163" = 610 /* HERNIA PROCEDURES AGE 0-17  */
"164" = 611 /* APPENDECTOMY W COMPLICATED PRINCIPAL DIAG W CC  */
"165" = 611 /* APPENDECTOMY W COMPLICATED PRINCIPAL DIAG WO CC  */
"166" = 612 /* APPENDECTOMY WO COMPLICATED PRINCIPAL DIAG W CC  */
"167" = 612 /* APPENDECTOMY WO COMPLICATED PRINCIPAL DIAG WO CC  */
"168" = 326 /* MOUTH PROCEDURES W CC  */
"169" = 326 /* MOUTH PROCEDURES WO CC  */
"170" = 613 /* OTHER DIGESTIVE SYSTEM O.R. PROCEDURES W CC  */
"171" = 613 /* OTHER DIGESTIVE SYSTEM O.R. PROCEDURES WO CC  */
"172" = 614 /* DIGESTIVE MALIGNANCY W CC  */
"173" = 614 /* DIGESTIVE MALIGNANCY WO CC  */
"174" = 615 /* G.I. HEMORRHAGE W CC  */
"175" = 615 /* G.I. HEMORRHAGE WO CC  */
"176" = 616 /* COMPLICATED PEPTIC ULCER  */
"177" = 617 /* UNCOMPLICATED PEPTIC ULCER W CC  */
"178" = 617 /* UNCOMPLICATED PEPTIC ULCER WO CC  */
"179" = 618 /* INFLAMMATORY BOWEL DISEASE  */
"180" = 619 /* G.I. OBSTRUCTION W CC  */
"181" = 619 /* G.I. OBSTRUCTION WO CC  */
"182" = 620 /* ESOPHAGITIS, GASTROENT & MISC DIGEST DISORDERS AGE >17 W */
"183" = 620 /* ESOPHAGITIS, GASTROENT & MISC DIGEST DISORDERS AGE >17 W */
"184" = 621 /* ESOPHAGITIS, GASTROENT & MISC DIGEST DISORDERS AGE 0-17 */
"185" = 327 /* DENTAL & ORAL DIS EXCEPT EXTRACTIONS & RESTORATIONS, AGE */
"186" = 328 /* DENTAL & ORAL DIS EXCEPT EXTRACTIONS & RESTORATIONS, AGE */
"187" = 329 /* DENTAL EXTRACTIONS & RESTORATIONS  */
"188" = 622 /* OTHER DIGESTIVE SYSTEM DIAGNOSES AGE >17 W CC  */
"189" = 622 /* OTHER DIGESTIVE SYSTEM DIAGNOSES AGE >17 WO CC  */
"190" = 623 /* OTHER DIGESTIVE SYSTEM DIAGNOSES AGE 0-17  */
"191" = 701 /* PANCREAS, LIVER & SHUNT PROCEDURES W CC  */
"192" = 701 /* PANCREAS, LIVER & SHUNT PROCEDURES WO CC  */
"193" = 702 /* BILIARY TRACT PROC EXCEPT ONLY CHOLECYST W OR WO C.D.E. */
"194" = 702 /* BILIARY TRACT PROC EXCEPT ONLY CHOLECYST W OR WO C.D.E. */
"195" = 703 /* CHOLECYSTECTOMY W C.D.E. W CC  */
"196" = 703 /* CHOLECYSTECTOMY W C.D.E. WO CC  */
"197" = 704 /* CHOLECYSTECTOMY EXCEPT BY LAPAROSCOPE WO C.D.E. W CC  */
"198" = 704 /* CHOLECYSTECTOMY EXCEPT BY LAPAROSCOPE WO C.D.E. WO CC  */
"199" = 705 /* HEPATOBILIARY DIAGNOSTIC PROCEDURE FOR MALIGNANCY  */
"200" = 706 /* HEPATOBILIARY DIAGNOSTIC PROCEDURE FOR NON-MALIGNANCY  */
"201" = 707 /* OTHER HEPATOBILIARY OR PANCREAS O.R. PROCEDURES  */
"202" = 708 /* CIRRHOSIS & ALCOHOLIC HEPATITIS  */
"203" = 709 /* MALIGNANCY OF HEPATOBILIARY SYSTEM OR PANCREAS  */
"204" = 710 /* DISORDERS OF PANCREAS EXCEPT MALIGNANCY  */
"205" = 711 /* DISORDERS OF LIVER EXCEPT MALIG,CIRR,ALC HEPA W CC  */
"206" = 711 /* DISORDERS OF LIVER EXCEPT MALIG,CIRR,ALC HEPA WO CC  */
"207" = 712 /* DISORDERS OF THE BILIARY TRACT W CC  */
"208" = 712 /* DISORDERS OF THE BILIARY TRACT WO CC  */
"209" = 801 /* MAJOR JOINT & LIMB REATTACHMENT PROCEDURES OF LOWER EXTR */
"210" = 802 /* HIP & FEMUR PROCEDURES EXCEPT MAJOR JOINT AGE >17 W CC  */
"211" = 802 /* HIP & FEMUR PROCEDURES EXCEPT MAJOR JOINT AGE >17 WO CC */
"212" = 803 /* HIP & FEMUR PROCEDURES EXCEPT MAJOR JOINT AGE 0-17  */
"213" = 804 /* AMPUTATION FOR MUSCULOSKELETAL SYSTEM & CONN TISSUE DISO */
"214" = 805 /* BACK & NECK PROC W CC (NO LONGER VALID)  */
"215" = 805 /* BACK & NECK PROC WO CC (NO LONGER VALID)  */
"216" = 806 /* BIOPSIES OF MUSCULOSKELETAL SYSTEM & CONNECTIVE TISSUE  */
"217" = 807 /* WND DEBRID & SKN GRFT EXCEPT HAND,FOR MUSCSKELET & CONN */
"218" = 808 /* LOWER EXTREM & HUMER PROC EXCEPT HIP,FOOT,FEMUR AGE >17 */
"219" = 808 /* LOWER EXTREM & HUMER PROC EXCEPT HIP,FOOT,FEMUR AGE >17 */
"220" = 809 /* LOWER EXTREM & HUMER PROC EXCEPT HIP,FOOT,FEMUR AGE 0-17 */
"221" = 810 /* KNEE PROC W CC (NO LONGER VALID)  */
"222" = 810 /* KNEE PROC WO CC (NO LONGER VALID)  */
"223" = 811 /* MAJOR SHOULDER/ELBOW PROC, OR OTHER UPPER EXTREMITY PROC */
"224" = 811 /* SHOULDER,ELBOW OR FOREARM PROC,EXC MAJOR JOINT PROC, WO */
"225" = 812 /* FOOT PROCEDURES  */
"226" = 813 /* SOFT TISSUE PROCEDURES W CC  */
"227" = 813 /* SOFT TISSUE PROCEDURES WO CC  */
"228" = 814 /* MAJOR THUMB OR JOINT PROC,OR OTH HAND OR WRIST PROC W CC */
"229" = 814 /* HAND OR WRIST PROC, EXCEPT MAJOR JOINT PROC, WO CC  */
"230" = 815 /* LOCAL EXCISION & REMOVAL OF INT FIX DEVICES OF HIP & FEM */
"231" = 816 /* LOCAL EXCISION & REMOVAL OF INT FIX DEVICES EXCEPT HIP & */
"232" = 817 /* ARTHROSCOPY  */
"233" = 818 /* OTHER MUSCULOSKELET SYS & CONN TISS O.R. PROC W CC  */
"234" = 818 /* OTHER MUSCULOSKELET SYS & CONN TISS O.R. PROC WO CC  */
"235" = 819 /* FRACTURES OF FEMUR  */
"236" = 820 /* FRACTURES OF HIP & PELVIS  */
"237" = 821 /* SPRAINS, STRAINS, & DISLOCATIONS OF HIP, PELVIS & THIGH */
"238" = 822 /* OSTEOMYELITIS  */
"239" = 823 /* PATHOLOGICAL FRACTURES & MUSCULOSKELETAL & CONN TISS MAL */
"240" = 824 /* CONNECTIVE TISSUE DISORDERS W CC  */
"241" = 824 /* CONNECTIVE TISSUE DISORDERS WO CC  */
"242" = 825 /* SEPTIC ARTHRITIS  */
"243" = 826 /* MEDICAL BACK PROBLEMS  */
"244" = 827 /* BONE DISEASES & SPECIFIC ARTHROPATHIES W CC  */
"245" = 827 /* BONE DISEASES & SPECIFIC ARTHROPATHIES WO CC  */
"246" = 828 /* NON-SPECIFIC ARTHROPATHIES  */
"247" = 829 /* SIGNS & SYMPTOMS OF MUSCULOSKELETAL SYSTEM & CONN TISSUE */
"248" = 830 /* TENDONITIS, MYOSITIS & BURSITIS  */
"249" = 831 /* AFTERCARE, MUSCULOSKELETAL SYSTEM & CONNECTIVE TISSUE  */
"250" = 832 /* FX, SPRN, STRN & DISL OF FOREARM, HAND, FOOT AGE >17 W C */
"251" = 832 /* FX, SPRN, STRN & DISL OF FOREARM, HAND, FOOT AGE >17 WO */
"252" = 833 /* FX, SPRN, STRN & DISL OF FOREARM, HAND, FOOT AGE 0-17  */
"253" = 834 /* FX, SPRN, STRN & DISL OF UPARM,LOWLEG EX FOOT AGE >17 W */
"254" = 834 /* FX, SPRN, STRN & DISL OF UPARM,LOWLEG EX FOOT AGE >17 WO */
"255" = 835 /* FX, SPRN, STRN & DISL OF UPARM,LOWLEG EX FOOT AGE 0-17  */
"256" = 836 /* OTHER MUSCULOSKELETAL SYSTEM & CONNECTIVE TISSUE DIAGNOS */
"257" = 901 /* TOTAL MASTECTOMY FOR MALIGNANCY W CC  */
"258" = 901 /* TOTAL MASTECTOMY FOR MALIGNANCY WO CC  */
"259" = 902 /* SUBTOTAL MASTECTOMY FOR MALIGNANCY W CC  */
"260" = 902 /* SUBTOTAL MASTECTOMY FOR MALIGNANCY WO CC  */
"261" = 903 /* BREAST PROC FOR NON-MALIGNANCY EXCEPT BIOPSY & LOCAL EXC */
"262" = 904 /* BREAST BIOPSY & LOCAL EXCISION FOR NON-MALIGNANCY  */
"263" = 905 /* SKIN GRAFT &/OR DEBRID FOR SKN ULCER OR CELLULITIS W CC */
"264" = 905 /* SKIN GRAFT &/OR DEBRID FOR SKN ULCER OR CELLULITIS WO CC */
"265" = 906 /* SKIN GRAFT &/OR DEBRID EXCEPT FOR SKIN ULCER OR CELLULIT */
"266" = 906 /* SKIN GRAFT &/OR DEBRID EXCEPT FOR SKIN ULCER OR CELLULIT */
"267" = 907 /* PERIANAL & PILONIDAL PROCEDURES  */
"268" = 908 /* SKIN, SUBCUTANEOUS TISSUE & BREAST PLASTIC PROCEDURES  */
"269" = 909 /* OTHER SKIN, SUBCUT TISS & BREAST PROC W CC  */
"270" = 909 /* OTHER SKIN, SUBCUT TISS & BREAST PROC WO CC  */
"271" = 910 /* SKIN ULCERS  */
"272" = 911 /* MAJOR SKIN DISORDERS W CC  */
"273" = 911 /* MAJOR SKIN DISORDERS WO CC  */
"274" = 912 /* MALIGNANT BREAST DISORDERS W CC  */
"275" = 912 /* MALIGNANT BREAST DISORDERS WO CC  */
"276" = 913 /* NON-MALIGANT BREAST DISORDERS  */
"277" = 914 /* CELLULITIS AGE >17 W CC  */
"278" = 914 /* CELLULITIS AGE >17 WO CC  */
"279" = 915 /* CELLULITIS AGE 0-17  */
"280" = 916 /* TRAUMA TO THE SKIN, SUBCUT TISS & BREAST AGE >17 W CC  */
"281" = 916 /* TRAUMA TO THE SKIN, SUBCUT TISS & BREAST AGE >17 WO CC  */
"282" = 917 /* TRAUMA TO THE SKIN, SUBCUT TISS & BREAST AGE 0-17  */
"283" = 918 /* MINOR SKIN DISORDERS W CC  */
"284" = 918 /* MINOR SKIN DISORDERS WO CC  */
"285" = 1001 /* AMPUTAT OF LOWER LIMB FOR ENDOCRINE,NUTRIT,& METABOL DIS */
"286" = 1002 /* ADRENAL & PITUITARY PROCEDURES  */
"287" = 1003 /* SKIN GRAFTS & WOUND DEBRID FOR ENDOC, NUTRIT & METAB DIS */
"288" = 1004 /* O.R. PROCEDURES FOR OBESITY  */
"289" = 1005 /* PARATHYROID PROCEDURES  */
"290" = 1006 /* THYROID PROCEDURES  */
"291" = 1007 /* THYROGLOSSAL PROCEDURES  */
"292" = 1008 /* OTHER ENDOCRINE, NUTRIT & METAB O.R. PROC W CC  */
"293" = 1008 /* OTHER ENDOCRINE, NUTRIT & METAB O.R. PROC WO CC  */
"294" = 1009 /* DIABETES AGE >35  */
"295" = 1010 /* DIABETES AGE 0-35  */
"296" = 1011 /* NUTRITIONAL & MISC METABOLIC DISORDERS AGE >17 W CC  */
"297" = 1011 /* NUTRITIONAL & MISC METABOLIC DISORDERS AGE >17 WO CC  */
"298" = 1012 /* NUTRITIONAL & MISC METABOLIC DISORDERS AGE 0-17  */
"299" = 1013 /* INBORN ERRORS OF METABOLISM  */
"300" = 1014 /* ENDOCRINE DISORDERS W CC  */
"301" = 1014 /* ENDOCRINE DISORDERS WO CC  */
"302" = 1101 /* KIDNEY TRANSPLANT  */
"303" = 1102 /* KIDNEY,URETER & MAJOR BLADDER PROCEDURES FOR NEOPLASM  */
"304" = 1103 /* KIDNEY,URETER & MAJOR BLADDER PROC FOR NON-NEOPL W CC  */
"305" = 1103 /* KIDNEY,URETER & MAJOR BLADDER PROC FOR NON-NEOPL WO CC  */
"306" = 1104 /* PROSTATECTOMY W CC  */
"307" = 1104 /* PROSTATECTOMY WO CC  */
"308" = 1105 /* MINOR BLADDER PROCEDURES W CC  */
"309" = 1105 /* MINOR BLADDER PROCEDURES WO CC  */
"310" = 1106 /* TRANSURETHRAL PROCEDURES W CC  */
"311" = 1106 /* TRANSURETHRAL PROCEDURES WO CC  */
"312" = 1107 /* URETHRAL PROCEDURES, AGE >17 W CC  */
"313" = 1107 /* URETHRAL PROCEDURES, AGE >17 WO CC  */
"314" = 1108 /* URETHRAL PROCEDURES, AGE 0-17  */
"315" = 1109 /* OTHER KIDNEY & URINARY TRACT O.R. PROCEDURES  */
"316" = 1110 /* RENAL FAILURE  */
"317" = 1111 /* ADMIT FOR RENAL DIALYSIS  */
"318" = 1112 /* KIDNEY & URINARY TRACT NEOPLASMS W CC  */
"319" = 1112 /* KIDNEY & URINARY TRACT NEOPLASMS WO CC  */
"320" = 1113 /* KIDNEY & URINARY TRACT INFECTIONS AGE >17 W CC  */
"321" = 1113 /* KIDNEY & URINARY TRACT INFECTIONS AGE >17 WO CC  */
"322" = 1114 /* KIDNEY & URINARY TRACT INFECTIONS AGE 0-17  */
"323" = 1115 /* URINARY STONES W CC, &/OR ESW LITHOTRIPSY  */
"324" = 1115 /* URINARY STONES WO CC  */
"325" = 1116 /* KIDNEY & URINARY TRACT SIGNS & SYMPTOMS AGE >17 W CC  */
"326" = 1116 /* KIDNEY & URINARY TRACT SIGNS & SYMPTOMS AGE >17 WO CC  */
"327" = 1117 /* KIDNEY & URINARY TRACT SIGNS & SYMPTOMS AGE 0-17  */
"328" = 1118 /* URETHRAL STRICTURE AGE >17 W CC  */
"329" = 1118 /* URETHRAL STRICTURE AGE >17 WO CC  */
"330" = 1119 /* URETHRAL STRICTURE AGE 0-17  */
"331" = 1120 /* OTHER KIDNEY & URINARY TRACT DIAGNOSES AGE >17 W CC  */
"332" = 1120 /* OTHER KIDNEY & URINARY TRACT DIAGNOSES AGE >17 WO CC  */
"333" = 1121 /* OTHER KIDNEY & URINARY TRACT DIAGNOSES AGE 0-17  */
"334" = 1201 /* MAJOR MALE PELVIC PROCEDURES W CC  */
"335" = 1201 /* MAJOR MALE PELVIC PROCEDURES WO CC  */
"336" = 1202 /* TRANSURETHRAL PROSTATECTOMY W CC  */
"337" = 1202 /* TRANSURETHRAL PROSTATECTOMY WO CC  */
"338" = 1203 /* TESTES PROCEDURES, FOR MALIGNANCY  */
"339" = 1204 /* TESTES PROCEDURES, NON-MALIGNANCY AGE >17  */
"340" = 1205 /* TESTES PROCEDURES, NON-MALIGNANCY AGE 0-17  */
"341" = 1206 /* PENIS PROCEDURES  */
"342" = 1207 /* CIRCUMCISION AGE >17  */
"343" = 1208 /* CIRCUMCISION AGE 0-17  */
"344" = 1209 /* OTHER MALE REPRODUCTIVE SYSTEM O.R. PROCEDURES FOR MALIG */
"345" = 1210 /* OTHER MALE REPRODUCTIVE SYSTEM O.R. PROC EXCEPT FOR MALI */
"346" = 1211 /* MALIGNANCY, MALE REPRODUCTIVE SYSTEM, W CC  */
"347" = 1211 /* MALIGNANCY, MALE REPRODUCTIVE SYSTEM, WO CC  */
"348" = 1212 /* BENIGN PROSTATIC HYPERTROPHY W CC  */
"349" = 1212 /* BENIGN PROSTATIC HYPERTROPHY WO CC  */
"350" = 1213 /* INFLAMMATION OF THE MALE REPRODUCTIVE SYSTEM  */
"351" = 1214 /* STERILIZATION, MALE  */
"352" = 1215 /* OTHER MALE REPRODUCTIVE SYSTEM DIAGNOSES  */
"353" = 1301 /* PELVIC EVISCERATION, RADICAL HYSTERECTOMY & RADICAL VULV */
"354" = 1302 /* UTERINE,ADNEXA PROC FOR NON-OVARIAN/ADNEXAL MALIG W CC  */
"355" = 1302 /* UTERINE,ADNEXA PROC FOR NON-OVARIAN/ADNEXAL MALIG WO CC */
"356" = 1303 /* FEMALE REPRODUCTIVE SYSTEM RECONSTRUCTIVE PROCEDURES  */
"357" = 1304 /* UTERINE & ADNEXA PROC FOR OVARIAN OR ADNEXAL MALIGNANCY */
"358" = 1305 /* UTERINE & ADNEXA PROC FOR NON-MALIGNANCY W CC  */
"359" = 1305 /* UTERINE & ADNEXA PROC FOR NON-MALIGNANCY WO CC  */
"360" = 1306 /* VAGINA, CERVIX & VULVA PROCEDURES  */
"361" = 1307 /* LAPAROSCOPY & INCISIONAL TUBAL INTERRUPTION  */
"362" = 1308 /* ENDOSCOPIC TUBAL INTERRUPTION  */
"363" = 1309 /* D&C, CONIZATION & RADIO-IMPLANT, FOR MALIGNANCY  */
"364" = 1310 /* D&C, CONIZATION EXCEPT FOR MALIGNANCY  */
"365" = 1311 /* OTHER FEMALE REPRODUCTIVE SYSTEM O.R. PROCEDURES  */
"366" = 1312 /* MALIGNANCY, FEMALE REPRODUCTIVE SYSTEM W CC  */
"367" = 1312 /* MALIGNANCY, FEMALE REPRODUCTIVE SYSTEM WO CC  */
"368" = 1313 /* INFECTIONS, FEMALE REPRODUCTIVE SYSTEM  */
"369" = 1314 /* MENSTRUAL & OTHER FEMALE REPRODUCTIVE SYSTEM DISORDERS  */
"370" = 1401 /* CESAREAN SECTION W CC  */
"371" = 1401 /* CESAREAN SECTION WO CC  */
"372" = 1402 /* VAGINAL DELIVERY W COMPLICATING DIAGNOSES  */
"373" = 1403 /* VAGINAL DELIVERY W/O COMPLICATING DIAGNOSES */
"374" = 1404 /* VAGINAL DELIVERY W STERILIZATION &/OR D&C  */
"375" = 1405 /* VAGINAL DELIVERY W O.R. PROC EXCEPT STERIL &/OR D&C  */
"376" = 1406 /* POSTPARTUM & POST ABORTION DIAGNOSES WO O.R. PROCEDURE  */
"377" = 1407 /* POSTPARTUM & POST ABORTION DIAGNOSES W O.R. PROCEDURE  */
"378" = 1408 /* ECTOPIC PREGNANCY  */
"379" = 1409 /* THREATENED ABORTION  */
"380" = 1410 /* ABORTION WO D&C  */
"381" = 1411 /* ABORTION W D&C, ASPIRATION CURETTAGE OR HYSTEROTOMY  */
"382" = 1412 /* FALSE LABOR  */
"383" = 1413 /* OTHER ANTEPARTUM DIAGNOSES W MEDICAL COMPLICATIONS  */
"384" = 1414 /* OTHER ANTEPARTUM DIAGNOSES WO MEDICAL COMPLICATIONS  */
"385" = 1502 /* NEONATES, DIED OR TRANSFERRED TO ANOTHER ACUTE CARE FACI */
"386" = 1502 /* EXTREME IMMATURITY OR RESPIRATORY DISTRESS SYNDROME, NEO */
"387" = 1503 /* PREMATURITY W MAJOR PROBLEMS  */
"388" = 1504 /* PREMATURITY WO MAJOR PROBLEMS  */
"389" = 1505 /* FULL TERM NEONATE W MAJOR PROBLEMS  */
"390" = 1506 /* NEONATE W OTHER SIGNIFICANT PROBLEMS  */
"391" = 1507 /* NORMAL NEWBORN  */
"392" = 1601 /* SPLENECTOMY AGE >17  */
"393" = 1602 /* SPLENECTOMY AGE 0-17  */
"394" = 1603 /* OTHER O.R. PROCEDURES OF THE BLOOD AND BLOOD FORMING ORG */
"395" = 1604 /* RED BLOOD CELL DISORDERS AGE >17  */
"396" = 1605 /* RED BLOOD CELL DISORDERS AGE 0-17  */
"397" = 1606 /* COAGULATION DISORDERS  */
"398" = 1607 /* RETICULOENDOTHELIAL & IMMUNITY DISORDERS W CC  */
"399" = 1607 /* RETICULOENDOTHELIAL & IMMUNITY DISORDERS WO CC  */
"400" = 1701 /* LYMPHOMA & LEUKEMIA W MAJOR O.R. PROCEDURE (NO LONGER VA */
"401" = 1702 /* LYMPHOMA & NON-ACUTE LEUKEMIA W OTHER O.R. PROC W CC  */
"402" = 1702 /* LYMPHOMA & NON-ACUTE LEUKEMIA W OTHER O.R. PROC WO CC  */
"403" = 1703 /* LYMPHOMA & NON-ACUTE LEUKEMIA W CC  */
"404" = 1703 /* LYMPHOMA & NON-ACUTE LEUKEMIA WO CC  */
"405" = 1704 /* ACUTE LEUKEMIA WO MAJOR O.R. PROCEDURE AGE 0-17  */
"406" = 1705 /* MYELOPROLIF DISORD OR POORLY DIFF NEOPL W MAJ O.R.PROC W */
"407" = 1705 /* MYELOPROLIF DISORD OR POORLY DIFF NEOPL W MAJ O.R.PROC W */
"408" = 1706 /* MYELOPROLIF DISORD OR POORLY DIFF NEOPL W OTHER O.R.PROC */
"409" = 1707 /* RADIOTHERAPY  */
"410" = 1708 /* CHEMOTHERAPY WO ACUTE LEUKEMIA AS SECONDARY DIAGNOSIS  */
"411" = 1709 /* HISTORY OF MALIGNANCY WO ENDOSCOPY  */
"412" = 1710 /* HISTORY OF MALIGNANCY W ENDOSCOPY  */
"413" = 1711 /* OTHER MYELOPROLIF DIS OR POORLY DIFF NEOPL DIAG W CC  */
"414" = 1711 /* OTHER MYELOPROLIF DIS OR POORLY DIFF NEOPL DIAG WO CC  */
"415" = 1801 /* O.R. PROCEDURE FOR INFECTIOUS & PARASITIC DISEASES (NO LONGER VALID)  */
"416" = 1802 /* SEPTICEMIA AGE >17 (NO LONGER VALID)  */
"417" = 1803 /* SEPTICEMIA AGE 0-17  */
"418" = 1804 /* POSTOPERATIVE & POST-TRAUMATIC INFECTIONS  */
"419" = 1805 /* FEVER OF UNKNOWN ORIGIN AGE >17 W CC  */
"420" = 1805 /* FEVER OF UNKNOWN ORIGIN AGE >17 WO CC  */
"421" = 1806 /* VIRAL ILLNESS AGE >17  */
"422" = 1807 /* VIRAL ILLNESS & FEVER OF UNKNOWN ORIGIN AGE 0-17  */
"423" = 1808 /* OTHER INFECTIOUS & PARASITIC DISEASES DIAGNOSES  */
"424" = 1901 /* O.R. PROCEDURE W PRINCIPAL DIAGNOSES OF MENTAL ILLNESS  */
"425" = 1902 /* ACUTE ADJUSTMENT REACTION & PSYCHOSOCIAL DYSFUNCTION  */
"426" = 1903 /* DEPRESSIVE NEUROSES  */
"427" = 1904 /* NEUROSES EXCEPT DEPRESSIVE  */
"428" = 1905 /* DISORDERS OF PERSONALITY & IMPULSE CONTROL  */
"429" = 1906 /* ORGANIC DISTURBANCES & MENTAL RETARDATION  */
"430" = 1907 /* PSYCHOSES  */
"431" = 1908 /* CHILDHOOD MENTAL DISORDERS  */
"432" = 1909 /* OTHER MENTAL DISORDER DIAGNOSES  */
"433" = 2001 /* ALCOHOL/DRUG ABUSE OR DEPENDENCE, LEFT AMA  */
"434" = 2001 /* ALC/DRUG ABUSE OR DEPEND, DETOX OR OTH SYMPT TREAT W CC */
"435" = 2001 /* ALC/DRUG ABUSE OR DEPEND, DETOX OR OTH SYMPT TREAT WO CC */
"436" = 2001 /* ALC/DRUG DEPENDENCE W REHABILITATION THERAPY (NO LONGER */
"437" = 2001 /* ALC/DRUG DEPENDENCE, COMBINED REHAB & DETOX THERAPY (NO */
"438" = 2001 /* NO LONGER VALID  */
"439" = 2101 /* SKIN GRAFTS FOR INJURIES  */
"440" = 2102 /* WOUND DEBRIDEMENTS FOR INJURIES  */
"441" = 2103 /* HAND PROCEDURES FOR INJURIES  */
"442" = 2104 /* OTHER O.R. PROCEDURES FOR INJURIES W CC  */
"443" = 2104 /* OTHER O.R. PROCEDURES FOR INJURIES WO CC  */
"444" = 2105 /* TRAUMATIC INJURY AGE >17 W CC  */
"445" = 2105 /* TRAUMATIC INJURY AGE >17 WO CC  */
"446" = 2106 /* TRAUMATIC INJURY AGE 0-17  */
"447" = 2107 /* ALLERGIC REACTIONS AGE >17  */
"448" = 2108 /* ALLERGIC REACTIONS AGE 0-17  */
"449" = 2109 /* POISONING & TOXIC EFFECTS OF DRUGS AGE >17 W CC  */
"450" = 2109 /* POISONING & TOXIC EFFECTS OF DRUGS AGE >17 WO CC  */
"451" = 2110 /* POISONING & TOXIC EFFECTS OF DRUGS AGE 0-17  */
"452" = 2111 /* COMPLICATIONS OF TREATMENT W CC  */
"453" = 2111 /* COMPLICATIONS OF TREATMENT WO CC  */
"454" = 2112 /* OTHER INJURY, POISONING & TOXIC EFFECT DIAG W CC  */
"455" = 2112 /* OTHER INJURY, POISONING & TOXIC EFFECT DIAG WO CC  */
"456" = 2201 /* BURNS, TRANSFERRED TO ANOTHER ACUTE CARE FACILITY (NO LO */
"457" = 2201 /* EXTENSIVE BURNS WO O.R. PROCEDURE (NO LONGER VALID)  */
"458" = 2202 /* NON-EXTENSIVE BURNS W SKIN GRAFT (NO LONGER VALID)  */
"459" = 2203 /* NON-EXTENSIVE BURNS W WOUND DEBRIDEMENT OR OTHER O.R. PR */
"460" = 2204 /* NON-EXTENSIVE BURNS WO O.R. PROCEDURE (NO LONGER VALID) */
"461" = 2301 /* O.R. PROC W DIAGNOSES OF OTHER CONTACT W HEALTH SERVICES */
"462" = 2302 /* REHABILITATION  */
"463" = 2303 /* SIGNS & SYMPTOMS W CC  */
"464" = 2303 /* SIGNS & SYMPTOMS WO CC  */
"465" = 2304 /* AFTERCARE W HISTORY OF MALIGNANCY AS SECONDARY DIAGNOSIS */
"466" = 2305 /* AFTERCARE WO HISTORY OF MALIGNANCY AS SECONDARY DIAGNOSI */
"467" = 2306 /* OTHER FACTORS INFLUENCING HEALTH STATUS  */
"468" = 8801 /* EXTENSIVE O.R. PROCEDURE UNRELATED TO PRINCIPAL DIAGNOSI */
"469" = 8802 /* PRINCIPAL DIAGNOSIS INVALID AS DISCHARGE DIAGNOSIS  */
"470" = 8803 /* UNGROUPABLE  */
"471" = 837 /* BILATERAL OR MULTIPLE MAJOR JOINT PROCS OF LOWER EXTREMI */
"472" = 2205 /* BILATERAL OR MULTIPLE MAJOR JOINT PROCS OF LOWER EXTREMI */
"473" = 1712 /* ACUTE LEUKEMIA WO MAJOR O.R. PROCEDURE AGE >17  */
"474" = 419 /* NO LONGER VALID  */
"475" = 420 /* RESPIRATORY SYSTEM DIAGNOSIS WITH VENTILATOR SUPPORT (NO LONGER VALID)  */
"476" = 8804 /* PROSTATIC O.R. PROCEDURE UNRELATED TO PRINCIPAL DIAGNOSI */
"477" = 8805 /* NON-EXTENSIVE O.R. PROCEDURE UNRELATED TO PRINCIPAL DIAG */
"478" = 536 /* OTHER VASCULAR PROCEDURES W CC  */
"479" = 536 /* OTHER VASCULAR PROCEDURES WO CC  */
"480" = 7702 /* LIVER TRANSPLANT  */
"481" = 7703 /* BONE MARROW TRANSPLANT  */
"482" = 7704 /* TRACHEOSTOMY FOR FACE,MOUTH & NECK DIAGNOSES  */
"483" = 7705 /* TRAC W MECH VENT 96+HRS OR PDX EXCEPT FACE,MOUTH & NECK */
"484" = 2401 /* CRANIOTOMY FOR MULTIPLE SIGNIFICANT TRAUMA  */
"485" = 2402 /* LIMB REATTACHMENT, HIP AND FEMUR PROC FOR MULTIPLE SIGNI */
"486" = 2403 /* OTHER O.R. PROCEDURES FOR MULTIPLE SIGNIFICANT TRAUMA  */
"487" = 2404 /* OTHER MULTIPLE SIGNIFICANT TRAUMA  */
"488" = 2501 /* HIV W EXTENSIVE O.R. PROCEDURE  */
"489" = 2502 /* HIV W MAJOR RELATED CONDITION  */
"490" = 2503 /* HIV W OR WO OTHER RELATED CONDITION  */
"491" = 838 /* MAJOR JOINT & LIMB REATTACHMENT PROCEDURES OF UPPER EXTR */
"492" = 1713 /* CHEMOTHERAPY W ACUTE LEUKEMIA OR W USE OF HI DOSE CHEMOA */
"493" = 713 /* LAPAROSCOPIC CHOLECYSTECTOMY WO C.D.E. W CC  */
"494" = 713 /* LAPAROSCOPIC CHOLECYSTECTOMY WO C.D.E. WO CC  */
"495" = 7706 /* LUNG TRANSPLANT  */
"496" = 839 /* COMBINED ANTERIOR/POSTERIOR SPINAL FUSION  */
"497" = 840 /* SPINAL FUSION EXCEPT CERVICAL W CC  */
"498" = 840 /* SPINAL FUSION EXCEPT CERVICAL WO CC  */
"499" = 841 /* BACK & NECK PROCEDURES EXCEPT SPINAL FUSION W CC  */
"500" = 841 /* BACK & NECK PROCEDURES EXCEPT SPINAL FUSION WO CC  */
"501" = 842 /* KNEE PROCEDURES W PDX OF INFECTION W CC  */
"502" = 842 /* KNEE PROCEDURES W PDX OF INFECTION WO CC  */
"503" = 843 /* KNEE PROCEDURES WO PDX OF INFECTION  */
"504" = 2206 /* EXTENSIVE 3RD DEGREE BURNS W SKIN GRAFT  */
"505" = 2207 /* EXTENSIVE 3RD DEGREE BURNS WO SKIN GRAFT  */
"506" = 2208 /* FULL THICKNESS BURN W SKIN GRAFT OR INHAL INJ W CC OR SI */
"507" = 2208 /* FULL THICKNESS BURN W SKIN GRFT OR INHAL INJ WO CC OR SI */
"508" = 2209 /* FULL THICKNESS BURN WO SKIN GRFT OR INHAL INJ W CC OR SI */
"509" = 2209 /* FULL THICKNESS BURN WO SKIN GRFT OR INH INJ WO CC OR SIG */
"510" = 2210 /* NON-EXTENSIVE BURNS W CC OR SIGNIFICANT TRAUMA  */
"511" = 2210 /* NON-EXTENSIVE BURNS WO CC OR SIGNIFICANT TRAUMA  */
"512" = 7707 /* SIMULTANEOUS PANCREAS/KIDNEY TRANSPLANT  */
"513" = 7708 /* PANCREAS TRANSPLANT  */
"514" = 537 /* CARDIAC DEFIB IMPLANT W CARDIAC CATH (NO LONGER VALID)  */
"515" = 538 /* CARDIAC DEFIBRILLATOR IMPLANT WO CARDIAC CATH  */
"516" = 539 /* PERCUTANEOUS CARDIOVASC PROC W AMI  */
"517" = 540 /* PERC CARDIO PROC W NON-DRUG ELUTING STENT WO AMI  */
"518" = 541 /* PERC CARDIO PROC WO CORONARY ARTERY STENT OR AMI  */
"519" = 844 /* CERVICAL SPINAL FUSION W CC  */
"520" = 844 /* CERVICAL SPINAL FUSION WO CC  */
"521" = 2001 /* ALCOHOL/DRUG ABUSE OR DEPENDENCE W CC  */
"522" = 2001 /* ALC/DRUG ABUSE OR DEPEND W REHABILITATION THERAPY WO CC */
"523" = 2001 /* ALC/DRUG ABUSE OR DEPEND WO REHABILITATION THERAPY WO CC */
"524" = 127 /* TRANSIENT ISCHEMIA  */
"525" = 542 /* HEART ASSIST SYSTEM IMPLANT  */
"526" = 543 /* PERCUTNEOUS CARDIOVASULAR PROC W DRUG ELUTING STENT W AM */
"527" = 544 /* PERCUTNEOUS CARDIOVASULAR PROC W DRUG ELUTING STENT WO A */
"528" = 128 /* INTRACRANIAL VASCULAR PROC W PDX HEMORRHAGE  */
"529" = 129 /* VENTRICULAR SHUNT PROCEDURES W CC  */
"530" = 129 /* VENTRICULAR SHUNT PROCEDURES WO CC  */
"531" = 103 /* SPINAL PROCEDURES W CC  */
"532" = 103 /* SPINAL PROCEDURES WO CC  */
"533" = 104 /* EXTRACRANIAL PROCEDURES W CC  */
"534" = 104 /* EXTRACRANIAL PROCEDURES WO CC  */
"535" = 545 /* CARDIAC DEFIB IMPLANT W CARDIAC CATH W AMI/HF/SHOCK  */
"536" = 546 /* CARDIAC DEFIB IMPLANT W CARDIAC CATH W/O AMI/HF/SHOCK */
"537" = 845 /* LOCAL EXCIS & REMOV OF INT FIX DEV EXCEPT HIP & FEMUR W */
"538" = 845 /* LOCAL EXCIS & REMOV OF INT FIX DEV EXCEPT HIP & FEMUR WO */
"539" = 1714 /* LYMPHOMA & LEUKEMIA W MAJOR OR PROCEDURE W CC  */
"540" = 1714 /* LYMPHOMA & LEUKEMIA W MAJOR OR PROCEDURE WO CC  */
"541" = 7705 /* TRACH W MV 96+HRS OR PDX EXC FACE, MTH, FACE & NECK DX W */
"542" = 7705 /* TRACH W MV 96+HRS OR PDX EXC FACE, MTH, FACE & NECK DX W */
"543" = 101 /* CRANIOTOMY WITH IMPLANTATION OF CHEMOTHERAPEUTIC AGENT O */
"544" = 801 /* MAJOR JOINT REPLACEMENT OR REATTACHMENT OF LOWER EXTREMI */
"545" = 801 /* REVISION OF HIP OR KNEE REPLACEMENT  */
"546" = 846 /* SPINAL FUSION EXC CERV WITH CURVATURE OF THE SPINE OR MA */
"547" = 504 /* CORONARY BYPASS W CARDIAC CATH W MAJOR CV DX  */
"548" = 504 /* CORONARY BYPASS W CARDIAC CATH W/O MAJOR CV DX  */
"549" = 506 /* CORONARY BYPASS W/O CARDIAC CATH W MAJOR CV DX  */
"550" = 506 /* CORONARY BYPASS W/O CARDIAC CATH W/O MAJOR CV DX  */
"551" = 511 /* PERMANENT CARDIAC PACEMAKER IMPL W MAJ CV DX OR AICD LEA */
"552" = 512 /* OTHER PERMANENT CARDIAC PACEMAKER IMPLANT W/O MAJOR CV D */
"553" = 536 /* OTHER VASCULAR PROCEDURES W CC W MAJOR CV DX  */
"554" = 536 /* OTHER VASCULAR PROCEDURES W CC W/O MAJOR CV DX  */
"555" = 539 /* PERCUTANEOUS CARDIOVASCULAR PROC W MAJOR CV DX  */
"556" = 540 /* PERCUTANEOUS CARDIOVASC PROC W NON-DRUG-ELUTING STENT W/ */
"557" = 543 /* PERCUTANEOUS CARDIOVASCULAR PROC W DRUG-ELUTING STENT W */
"558" = 544 /* PERCUTANEOUS CARDIOVASCULAR PROC W DRUG-ELUTING STENT W/ */
"559" = 130 /* ACUTE ISCHEMIC STROKE WITH USE OF THROMBOLYTIC AGENT  */
"560" = 115 /* BACTERIAL & TUBERCULOUS INFECTIONS OF NERVOUS SYSTEM */
"561" = 115 /* NON-BACTERIAL INFECTIONS OF NERVOUS SYSTEM EXCEPT VIRAL MENINGIT */
"562" = 119 /* SEIZURE AGE > 17 W CC */
"563" = 119 /* SEIZURE AGE > 17 W/O CC */
"564" = 119 /* HEADACHES AGE >17 */
"565" = 420 /* RESPIRATORY SYSTEM DIAGNOSIS WITH VENTILATOR SUPPORT 96+ HOURS */
"566" = 420 /* RESPIRATORY SYSTEM DIAGNOSIS WITH VENTILATOR SUPPORT < 96 HOURS */
"567" = 605 /* STOMACH, ESOPHAGEAL & DUODENAL PROC AGE > 17 W CC W MAJOR GI DX */
"568" = 605 /* STOMACH, ESOPHAGEAL & DUODENAL PROCEDURES PROC AGE > 17 W CC W/O */
"569" = 602 /* MAJOR SMALL & LARGE BOWEL PROCEDURES W CC W MAJOR GI DX */
"570" = 602 /* MAJOR SMALL & LARGE BOWEL PROCEDURES W CC W/O MAJOR GI DX */
"571" = 624 /* MAJOR ESOPHAGEAL DISORDERS */
"572" = 625 /* MAJOR GASTROINTESTINAL DISORDERS AND PERITONEAL INFECTIONS */
"573" = 1122 /* MAJOR BLADDER PROCEDURES */
"574" = 1608 /* MAJOR HEMATOLOGIC/IMMUNOLOGIC DIAG EXC SICKLE CELL CRISIS & COAGUL */
"575" = 1802 /* SEPTICEMIA W MV96+ HOURS  AGE  >17 */
"576" = 1802 /* SEPTICEMIA W/O MV96+ HOURS AGE  >17 */
"577" = 131 /* CAROTID ARTERY STENT PROCEDURE */
"578" = 1801 /* INFECTIOUS & PARASITIC DISEASES W OR PROCEDURE */
"579" = 1801 /* POSTOPERATIVE OR POST-TRAUMATIC INFECTIONS W OR PROCEDURE */
OTHER = 8806 /* UNKNOWN DRG */
;

VALUE $MSDRG
"001" = "103"
"002" = "103"
"003" = "541"
"004" = "542"
"005" = "480"
"006" = "480"
"007" = "495"
"008" = "512"
"009" = "481"
"010" = "513"
"011" = "482"
"012" = "482"
"013" = "482"
"020" = "528"
"021" = "528"
"022" = "528"
"023" = "543"
"024" = "543"
"025" = "001"
"026" = "002"
"027" = "003"
"028" = "531"
"029" = "532"
"030" = "532"
"031" = "529"
"032" = "530"
"033" = "530"
"034" = "577"
"035" = "577"
"036" = "577"
"037" = "533"
"038" = "534"
"039" = "534"
"040" = "006"
"041" = "007"
"042" = "008"
"052" = "009"
"053" = "009"
"054" = "010"
"055" = "011"
"056" = "012"
"057" = "012"
"058" = "013"
"059" = "013"
"060" = "013"
"061" = "559"
"062" = "559"
"063" = "559"
"064" = "014"
"065" = "014"
"066" = "014"
"067" = "015"
"068" = "015"
"069" = "524"
"070" = "016"
"071" = "017"
"072" = "017"
"073" = "018"
"074" = "019"
"075" = "021"
"076" = "021"
"077" = "022"
"078" = "022"
"079" = "022"
"080" = "023"
"081" = "023"
"082" = "027"
"083" = "027"
"084" = "027"
"085" = "028"
"086" = "029"
"087" = "030"
"088" = "031"
"089" = "032"
"090" = "033"
"091" = "034"
"092" = "035"
"093" = "035"
"094" = "560"
"095" = "560"
"096" = "560"
"097" = "561"
"098" = "561"
"099" = "561"
"100" = "026"
"101" = "562"
"102" = "564"
"103" = "564"
"113" = "037"
"114" = "037"
"115" = "040"
"116" = "036"
"117" = "038"
"121" = "044"
"122" = "044"
"123" = "045"
"124" = "043"
"125" = "046"
"129" = "049"
"130" = "049"
"131" = "049"
"132" = "049"
"133" = "052"
"134" = "055"
"135" = "053"
"136" = "054"
"137" = "168"
"138" = "169"
"139" = "050"
"146" = "064"
"147" = "064"
"148" = "064"
"149" = "065"
"150" = "066"
"151" = "066"
"152" = "067"
"153" = "068"
"154" = "072"
"155" = "073"
"156" = "074"
"157" = "185"
"158" = "186"
"159" = "187"
"163" = "075"
"164" = "075"
"165" = "075"
"166" = "076"
"167" = "077"
"168" = "077"
"175" = "078"
"176" = "078"
"177" = "079"
"178" = "080"
"179" = "081"
"180" = "082"
"181" = "082"
"182" = "082"
"183" = "083"
"184" = "084"
"185" = "084"
"186" = "085"
"187" = "086"
"188" = "086"
"189" = "087"
"190" = "088"
"191" = "088"
"192" = "088"
"193" = "089"
"194" = "090"
"195" = "091"
"196" = "092"
"197" = "093"
"198" = "093"
"199" = "094"
"200" = "095"
"201" = "095"
"202" = "096"
"203" = "097"
"204" = "099"
"205" = "101"
"206" = "102"
"207" = "565"
"208" = "566"
"215" = "525"
"216" = "104"
"217" = "104"
"218" = "104"
"219" = "105"
"220" = "105"
"221" = "105"
"222" = "535"
"223" = "535"
"224" = "536"
"225" = "536"
"226" = "515"
"227" = "515"
"228" = "108"
"229" = "108"
"230" = "108"
"231" = "106"
"232" = "106"
"233" = "547"
"234" = "548"
"235" = "549"
"236" = "550"
"237" = "110"
"238" = "111"
"239" = "113"
"240" = "113"
"241" = "113"
"242" = "551"
"243" = "552"
"244" = "552"
"245" = "552"
"246" = "557"
"247" = "558"
"248" = "555"
"249" = "556"
"250" = "518"
"251" = "518"
"252" = "479"
"253" = "553"
"254" = "554"
"255" = "114"
"256" = "114"
"257" = "114"
"258" = "118"
"259" = "118"
"260" = "117"
"261" = "117"
"262" = "117"
"263" = "119"
"264" = "120"
"280" = "121"
"281" = "122"
"282" = "122"
"283" = "123"
"284" = "123"
"285" = "123"
"286" = "124"
"287" = "125"
"288" = "126"
"289" = "126"
"290" = "126"
"291" = "127"
"292" = "127"
"293" = "127"
"294" = "128"
"295" = "128"
"296" = "129"
"297" = "129"
"298" = "129"
"299" = "130"
"300" = "131"
"301" = "131"
"302" = "132"
"303" = "133"
"304" = "134"
"305" = "134"
"306" = "135"
"307" = "136"
"308" = "138"
"309" = "139"
"310" = "139"
"311" = "140"
"312" = "141"
"313" = "143"
"314" = "144"
"315" = "145"
"316" = "145"
"326" = "155"
"327" = "156"
"328" = "567"
"329" = "149"
"330" = "569"
"331" = "570"
"332" = "146"
"333" = "147"
"334" = "147"
"335" = "150"
"336" = "151"
"337" = "151"
"338" = "164"
"339" = "165"
"340" = "165"
"341" = "166"
"342" = "167"
"343" = "167"
"344" = "152"
"345" = "153"
"346" = "153"
"347" = "157"
"348" = "158"
"349" = "158"
"350" = "161"
"351" = "162"
"352" = "163"
"353" = "159"
"354" = "160"
"355" = "160"
"356" = "170"
"357" = "171"
"358" = "171"
"368" = "571"
"369" = "571"
"370" = "571"
"371" = "572"
"372" = "572"
"373" = "572"
"374" = "172"
"375" = "173"
"376" = "173"
"377" = "174"
"378" = "175"
"379" = "175"
"380" = "176"
"381" = "176"
"382" = "176"
"383" = "177"
"384" = "178"
"385" = "179"
"386" = "179"
"387" = "179"
"388" = "180"
"389" = "181"
"390" = "181"
"391" = "182"
"392" = "183"
"393" = "188"
"394" = "189"
"395" = "190"
"405" = "191"
"406" = "192"
"407" = "192"
"408" = "193"
"409" = "194"
"410" = "194"
"411" = "195"
"412" = "196"
"413" = "196"
"414" = "197"
"415" = "198"
"416" = "198"
"417" = "493"
"418" = "494"
"419" = "494"
"420" = "199"
"421" = "200"
"422" = "200"
"423" = "201"
"424" = "201"
"425" = "201"
"432" = "202"
"433" = "202"
"434" = "202"
"435" = "203"
"436" = "203"
"437" = "203"
"438" = "204"
"439" = "204"
"440" = "204"
"441" = "205"
"442" = "206"
"443" = "206"
"444" = "207"
"445" = "208"
"446" = "208"
"453" = "496"
"454" = "496"
"455" = "496"
"456" = "546"
"457" = "546"
"458" = "546"
"459" = "497"
"460" = "498"
"461" = "471"
"462" = "471"
"463" = "217"
"464" = "217"
"465" = "217"
"466" = "545"
"467" = "545"
"468" = "545"
"469" = "544"
"470" = "544"
"471" = "519"
"472" = "520"
"473" = "520"
"474" = "213"
"475" = "213"
"476" = "213"
"477" = "216"
"478" = "216"
"479" = "216"
"480" = "210"
"481" = "211"
"482" = "212"
"483" = "491"
"484" = "491"
"485" = "501"
"486" = "502"
"487" = "502"
"488" = "503"
"489" = "503"
"490" = "499"
"491" = "500"
"492" = "218"
"493" = "219"
"494" = "220"
"495" = "537"
"496" = "538"
"497" = "538"
"498" = "230"
"499" = "230"
"500" = "226"
"501" = "227"
"502" = "227"
"503" = "225"
"504" = "225"
"505" = "225"
"506" = "228"
"507" = "223"
"508" = "223"
"509" = "232"
"510" = "224"
"511" = "224"
"512" = "224"
"513" = "229"
"514" = "229"
"515" = "233"
"516" = "234"
"517" = "234"
"533" = "235"
"534" = "235"
"535" = "236"
"536" = "236"
"537" = "237"
"538" = "237"
"539" = "238"
"540" = "238"
"541" = "238"
"542" = "239"
"543" = "239"
"544" = "239"
"545" = "240"
"546" = "241"
"547" = "241"
"548" = "242"
"549" = "242"
"550" = "242"
"551" = "243"
"552" = "243"
"553" = "244"
"554" = "245"
"555" = "247"
"556" = "247"
"557" = "248"
"558" = "248"
"559" = "249"
"560" = "249"
"561" = "249"
"562" = "250"
"563" = "251"
"564" = "256"
"565" = "256"
"566" = "256"
"573" = "263"
"574" = "264"
"575" = "264"
"576" = "265"
"577" = "266"
"578" = "266"
"579" = "267"
"580" = "268"
"581" = "269"
"582" = "257"
"583" = "258"
"584" = "259"
"585" = "260"
"592" = "271"
"593" = "271"
"594" = "271"
"595" = "272"
"596" = "273"
"597" = "274"
"598" = "275"
"599" = "275"
"600" = "276"
"601" = "276"
"602" = "277"
"603" = "278"
"604" = "280"
"605" = "281"
"606" = "283"
"607" = "284"
"614" = "286"
"615" = "286"
"616" = "285"
"617" = "285"
"618" = "285"
"619" = "288"
"620" = "288"
"621" = "288"
"622" = "287"
"623" = "287"
"624" = "287"
"625" = "289"
"626" = "290"
"627" = "291"
"628" = "292"
"629" = "293"
"630" = "293"
"637" = "294"
"638" = "295"
"639" = "295"
"640" = "296"
"641" = "297"
"642" = "299"
"643" = "300"
"644" = "301"
"645" = "301"
"652" = "302"
"653" = "573"
"654" = "573"
"655" = "573"
"656" = "303"
"657" = "303"
"658" = "303"
"659" = "304"
"660" = "305"
"661" = "305"
"662" = "308"
"663" = "309"
"664" = "309"
"665" = "306"
"666" = "307"
"667" = "307"
"668" = "310"
"669" = "311"
"670" = "311"
"671" = "312"
"672" = "313"
"673" = "315"
"674" = "315"
"675" = "315"
"682" = "316"
"683" = "316"
"684" = "316"
"685" = "317"
"686" = "318"
"687" = "319"
"688" = "319"
"689" = "320"
"690" = "321"
"691" = "323"
"692" = "323"
"693" = "324"
"694" = "324"
"695" = "325"
"696" = "326"
"697" = "328"
"698" = "331"
"699" = "332"
"700" = "333"
"707" = "334"
"708" = "335"
"709" = "341"
"710" = "341"
"711" = "338"
"712" = "339"
"713" = "336"
"714" = "337"
"715" = "344"
"716" = "344"
"717" = "345"
"718" = "345"
"722" = "346"
"723" = "347"
"724" = "347"
"725" = "348"
"726" = "349"
"727" = "350"
"728" = "350"
"729" = "351"
"730" = "352"
"734" = "353"
"735" = "353"
"736" = "357"
"737" = "357"
"738" = "357"
"739" = "354"
"740" = "355"
"741" = "355"
"742" = "358"
"743" = "359"
"744" = "361"
"745" = "362"
"746" = "360"
"747" = "360"
"748" = "356"
"749" = "365"
"750" = "365"
"754" = "366"
"755" = "367"
"756" = "367"
"757" = "368"
"758" = "368"
"759" = "368"
"760" = "369"
"761" = "369"
"765" = "370"
"766" = "371"
"767" = "374"
"768" = "375"
"769" = "377"
"770" = "381"
"774" = "372"
"775" = "373"
"776" = "376"
"777" = "378"
"778" = "379"
"779" = "380"
"780" = "382"
"781" = "383"
"782" = "384"
"789" = "385"
"790" = "386"
"791" = "387"
"792" = "388"
"793" = "389"
"794" = "390"
"795" = "391"
"799" = "392"
"800" = "393"
"801" = "393"
"802" = "394"
"803" = "394"
"804" = "394"
"808" = "574"
"809" = "574"
"810" = "574"
"811" = "395"
"812" = "396"
"813" = "397"
"814" = "398"
"815" = "399"
"816" = "399"
"820" = "539"
"821" = "540"
"822" = "540"
"823" = "401"
"824" = "402"
"825" = "402"
"826" = "406"
"827" = "407"
"828" = "407"
"829" = "408"
"830" = "408"
"834" = "405"
"835" = "473"
"836" = "473"
"837" = "492"
"838" = "492"
"839" = "492"
"840" = "403"
"841" = "404"
"842" = "404"
"843" = "411"
"844" = "412"
"845" = "413"
"846" = "410"
"847" = "410"
"848" = "410"
"849" = "409"
"853" = "578"
"854" = "578"
"855" = "578"
"856" = "579"
"857" = "579"
"858" = "579"
"862" = "418"
"863" = "418"
"864" = "419"
"865" = "421"
"866" = "422"
"867" = "423"
"868" = "423"
"869" = "423"
"870" = "417"
"871" = "576"
"872" = "576"
"876" = "424"
"880" = "425"
"881" = "426"
"882" = "427"
"883" = "428"
"884" = "429"
"885" = "430"
"886" = "431"
"887" = "432"
"894" = "433"
"895" = "521"
"896" = "523"
"897" = "523"
"901" = "440"
"902" = "440"
"903" = "440"
"904" = "439"
"905" = "439"
"906" = "441"
"907" = "442"
"908" = "443"
"909" = "443"
"913" = "444"
"914" = "445"
"915" = "447"
"916" = "448"
"917" = "449"
"918" = "450"
"919" = "452"
"920" = "453"
"921" = "453"
"922" = "454"
"923" = "455"
"927" = "504"
"928" = "506"
"929" = "507"
"933" = "505"
"934" = "508"
"935" = "510"
"939" = "461"
"940" = "461"
"941" = "461"
"945" = "462"
"946" = "462"
"947" = "463"
"948" = "464"
"949" = "465"
"950" = "466"
"951" = "467"
"955" = "484"
"956" = "485"
"957" = "486"
"958" = "486"
"959" = "486"
"963" = "487"
"964" = "487"
"965" = "487"
"969" = "488"
"970" = "488"
"974" = "489"
"975" = "489"
"976" = "489"
"977" = "490"
"981" = "468"
"982" = "468"
"983" = "468"
"984" = "476"
"985" = "476"
"986" = "476"
"987" = "477"
"988" = "477"
"989" = "477"
"998" = "469"
"999" = "470"
OTHER = "470"
;

 /* --------------------------------------------------------------- */
 /* ---   INDICATOR SPECIFIC AGE, DRG AND COMORBIDITY STRATIFIERS - */
 /* --------------------------------------------------------------- */

VALUE D1PSFMT
101 = 1
104 = 2
106 = 3
401 = 4
402 = 5
502 = 6
504 = 7
506 = 8
507 = 9
509 = 10
516 = 11
536 = 12
537 = 13
601 = 14
602 = 15
603 = 16
605 = 17
607 = 18
608 = 19
609 = 20
611 = 21
612 = 22
613 = 23
701 = 24
704 = 25
713 = 26
801 = 0
802 = 28
808 = 29
811 = 30
813 = 31
816 = 32
818 = 33
837 = 34
838 = 35
839 = 36
840 = 37
841 = 38
843 = 39
844 = 40
901 = 41
1004 = 42
1006 = 43
1102 = 44
1103 = 45
1106 = 46
1201 = 47
1202 = 48
1302 = 49
1303 = 50
1304 = 51
1305 = 52
1306 = 53
1401 = 54
1404 = 55
1801 = 56
2104 = 57
2301 = 58
199 = 59
299 = 59
399 = 59
699 = 59
799 = 59
999 = 59
1199 = 59
1399 = 59
1799 = 59
1999 = 59
599 = 60
899 = 60
1099 = 60
1299 = 60
1499 = 60
1699 = 60
2199 = 60
2299 = 60
2499 = 60
2599 = 60
OTHER = -1;

VALUE D3PSFMT
101 = 1
103 = 2
104 = 3
106 = 4
107 = 5
108 = 6
109 = 7
110 = 8
111 = 9
112 = 10
113 = 11
114 = 12
115 = 13
116 = 14
118 = 15
119 = 16
121 = 17
122 = 18
124 = 19
126 = 20
127 = 21
129 = 22
316 = 23
317 = 24
318 = 25
320 = 26
324 = 27
327 = 28
401 = 29
402 = 30
403 = 31
404 = 32
406 = 33
407 = 34
408 = 35
409 = 36
410 = 37
411 = 38
413 = 39
414 = 40
415 = 41
417 = 42
418 = 43
420 = 44
501 = 45
502 = 46
503 = 47
504 = 48
505 = 49
506 = 50
507 = 51
509 = 52
510 = 53
511 = 54
512 = 55
514 = 56
516 = 57
517 = 58
518 = 59
519 = 60
520 = 61
521 = 62
522 = 63
524 = 64
525 = 65
526 = 66
527 = 67
529 = 68
530 = 69
531 = 70
532 = 71
533 = 72
536 = 73
537 = 74
538 = 75
539 = 76
540 = 77
541 = 78
543 = 79
544 = 80
545 = 81
601 = 82
602 = 83
603 = 84
604 = 85
605 = 86
607 = 87
608 = 88
609 = 89
611 = 90
613 = 91
614 = 92
615 = 93
616 = 94
617 = 95
618 = 96
619 = 97
620 = 98
622 = 99
701 = 100
702 = 101
703 = 102
704 = 103
707 = 104
708 = 105
709 = 106
710 = 107
711 = 108
712 = 109
713 = 110
801 = 111
802 = 112
804 = 113
806 = 114
807 = 115
808 = 116
811 = 117
812 = 118
813 = 119
815 = 120
816 = 121
818 = 122
819 = 123
820 = 124
822 = 125
823 = 126
824 = 127
825 = 128
826 = 129
827 = 130
829 = 131
830 = 132
831 = 133
832 = 134
834 = 135
836 = 136
837 = 137
838 = 138
839 = 139
840 = 140
841 = 141
842 = 142
844 = 143
845 = 144
1001 = 145
1003 = 146
1004 = 147
1008 = 148
1009 = 149
1010 = 150
1011 = 151
1014 = 152
1102 = 153
1103 = 154
1104 = 155
1105 = 156
1106 = 157
1107 = 158
1109 = 159
1110 = 160
1112 = 161
1113 = 162
1115 = 163
1116 = 164
1120 = 165
1202 = 166
1211 = 167
1212 = 168
1213 = 169
1302 = 170
1304 = 171
1305 = 172
1306 = 173
1311 = 174
1312 = 175
1313 = 176
1314 = 177
1601 = 178
1603 = 179
1604 = 180
1606 = 181
1607 = 182
1701 = 183
1702 = 184
1703 = 185
1705 = 186
1706 = 187
1707 = 188
1708 = 189
1711 = 190
1712 = 191
1713 = 192
1714 = 193
1801 = 194
1802 = 0
1804 = 196
1805 = 197
1806 = 198
1808 = 199
1901 = 200
1902 = 201
1903 = 202
1906 = 203
1907 = 204
2001 = 205
2102 = 206
2104 = 207
2105 = 208
2109 = 209
2111 = 210
2112 = 211
2210 = 212
2301 = 213
2302 = 214
2303 = 215
2304 = 216
2305 = 217
2306 = 218
2401 = 219
2402 = 220
2403 = 221
2404 = 222
2501 = 223
2502 = 224
2503 = 225
199 = 226
599 = 226
899 = 226
1199 = 226
1399 = 226
1799 = 226
2199 = 226
2299 = 226
299 = 227
399 = 227
699 = 227
799 = 227
1099 = 227
1299 = 227
1999 = 227
OTHER = -1;

%MACRO D4;

%IF &POAFG. = 0 %THEN %DO;

VALUE D4PSFMT
101 = 1
103 = 2
104 = 3
106 = 4
107 = 5
108 = 6
109 = 7
110 = 8
111 = 9
112 = 10
113 = 11
114 = 12
115 = 13
116 = 14
118 = 15
119 = 16
121 = 17
122 = 18
126 = 19
127 = 20
128 = 21
129 = 22
316 = 23
320 = 24
324 = 25
327 = 26
401 = 27
402 = 28
403 = 29
404 = 30
406 = 31
408 = 32
409 = 33
410 = 34
411 = 35
413 = 36
414 = 37
417 = 38
418 = 39
420 = 40
501 = 41
502 = 42
503 = 43
504 = 44
505 = 45
506 = 46
507 = 47
509 = 48
510 = 49
511 = 50
512 = 51
516 = 52
518 = 53
520 = 54
521 = 55
523 = 0
524 = 57
525 = 58
526 = 59
527 = 60
529 = 61
531 = 62
532 = 63
533 = 64
536 = 65
537 = 66
538 = 67
539 = 68
540 = 69
541 = 70
542 = 71
543 = 72
544 = 73
601 = 74
602 = 75
603 = 76
604 = 77
605 = 78
607 = 79
608 = 80
609 = 81
611 = 82
612 = 83
613 = 84
614 = 85
615 = 86
616 = 87
618 = 88
619 = 89
620 = 90
622 = 91
701 = 92
702 = 93
703 = 94
704 = 95
705 = 96
706 = 97
707 = 98
708 = 99
709 = 100
710 = 101
711 = 102
712 = 103
713 = 104
801 = 105
802 = 106
804 = 107
806 = 108
807 = 109
808 = 110
813 = 111
818 = 112
820 = 113
822 = 114
823 = 115
824 = 116
826 = 117
827 = 118
830 = 119
831 = 120
834 = 121
836 = 122
837 = 123
839 = 124
840 = 125
841 = 126
842 = 127
844 = 128
905 = 129
909 = 130
910 = 131
911 = 132
912 = 133
914 = 134
916 = 135
918 = 136
1001 = 137
1002 = 138
1003 = 139
1004 = 140
1008 = 141
1009 = 142
1010 = 143
1011 = 144
1014 = 145
1102 = 146
1103 = 147
1105 = 148
1106 = 149
1109 = 150
1110 = 151
1112 = 152
1113 = 153
1115 = 154
1120 = 155
1201 = 156
1202 = 157
1211 = 158
1302 = 159
1304 = 160
1305 = 161
1309 = 162
1311 = 163
1312 = 164
1401 = 165
1402 = 166
1406 = 167
1413 = 168
1601 = 169
1603 = 170
1604 = 171
1606 = 172
1607 = 173
1701 = 174
1702 = 175
1703 = 176
1705 = 177
1706 = 178
1707 = 179
1708 = 180
1711 = 181
1712 = 182
1713 = 183
1714 = 184
1801 = 185
1802 = 0
1804 = 187
1805 = 188
1806 = 189
1808 = 190
1902 = 191
1906 = 192
1907 = 193
2001 = 194
2104 = 195
2109 = 196
2111 = 197
2112 = 198
2206 = 199
2208 = 200
2302 = 201
2303 = 202
2305 = 203
2306 = 204
2401 = 205
2402 = 206
2403 = 207
2404 = 208
2501 = 209
2502 = 210
2503 = 211
199 = 212
899 = 212
999 = 212
1099 = 212
1199 = 212
1299 = 212
1799 = 212
2199 = 212
2299 = 212
299 = 213
399 = 213
499 = 213
599 = 213
699 = 213
1399 = 213
1499 = 213
1999 = 213
2399 = 213
OTHER = -1;

%END;
%ELSE %DO;

VALUE D4PSFMT
101 = 1
103 = 2
104 = 3
106 = 4
107 = 5
108 = 6
109 = 7
110 = 8
111 = 9
112 = 10
113 = 11
114 = 12
115 = 13
116 = 14
118 = 15
119 = 16
121 = 17
122 = 18
126 = 19
127 = 20
128 = 21
129 = 22
316 = 23
320 = 24
324 = 25
327 = 26
401 = 27
402 = 28
403 = 29
404 = 30
406 = 31
408 = 32
409 = 33
410 = 34
411 = 35
413 = 36
414 = 37
417 = 38
418 = 39
420 = 40
501 = 41
502 = 42
503 = 43
504 = 44
505 = 45
506 = 46
507 = 47
509 = 48
510 = 49
511 = 50
512 = 51
516 = 52
518 = 53
520 = 54
521 = 55
523 = 0
524 = 57
525 = 58
526 = 59
527 = 60
529 = 61
531 = 62
532 = 63
533 = 64
536 = 65
537 = 66
538 = 67
539 = 68
540 = 69
541 = 70
542 = 71
543 = 72
544 = 73
601 = 74
602 = 75
603 = 76
604 = 77
605 = 78
607 = 79
608 = 80
609 = 81
611 = 82
612 = 83
613 = 84
614 = 85
615 = 86
616 = 87
618 = 88
619 = 89
620 = 90
622 = 91
701 = 92
702 = 93
703 = 94
704 = 95
705 = 96
706 = 97
707 = 98
708 = 99
709 = 100
710 = 101
711 = 102
712 = 103
713 = 104
801 = 105
802 = 106
804 = 107
806 = 108
807 = 109
808 = 110
813 = 111
818 = 112
820 = 113
822 = 114
823 = 115
824 = 116
826 = 117
827 = 118
830 = 119
831 = 120
834 = 121
836 = 122
837 = 123
839 = 124
840 = 125
841 = 126
842 = 127
844 = 128
905 = 129
909 = 130
910 = 131
911 = 132
912 = 133
914 = 134
916 = 135
918 = 136
1001 = 137
1002 = 138
1003 = 139
1004 = 140
1008 = 141
1009 = 142
1010 = 143
1011 = 144
1014 = 145
1102 = 146
1103 = 147
1105 = 148
1106 = 149
1109 = 150
1110 = 151
1112 = 152
1113 = 153
1115 = 154
1120 = 155
1201 = 156
1202 = 157
1211 = 158
1302 = 159
1304 = 160
1305 = 161
1309 = 162
1311 = 163
1312 = 164
1401 = 165
1402 = 166
1406 = 167
1413 = 213
1601 = 169
1603 = 170
1604 = 171
1606 = 172
1607 = 173
1701 = 174
1702 = 175
1703 = 176
1705 = 177
1706 = 178
1707 = 179
1708 = 180
1711 = 181
1712 = 182
1713 = 183
1714 = 184
1801 = 185
1802 = 0
1804 = 187
1805 = 188
1806 = 189
1808 = 190
1902 = 191
1906 = 192
1907 = 213
2001 = 194
2104 = 195
2109 = 196
2111 = 197
2112 = 198
2206 = 199
2208 = 200
2302 = 201
2303 = 202
2305 = 203
2306 = 213
2401 = 205
2402 = 206
2403 = 207
2404 = 208
2501 = 209
2502 = 210
2503 = 211
199 = 212
899 = 212
999 = 212
1099 = 212
1199 = 212
1299 = 212
1799 = 212
2199 = 212
2299 = 212
299 = 213
399 = 213
499 = 213
599 = 213
699 = 213
1399 = 213
1499 = 213
1999 = 213
2399 = 213
OTHER = -1;

%END;

%MEND;

%D4;

VALUE D6PSFMT
101 = 1
103 = 2
104 = 3
106 = 4
108 = 5
111 = 6
119 = 7
128 = 8
402 = 9
404 = 10
406 = 11
409 = 12
410 = 13
411 = 14
413 = 15
414 = 16
418 = 17
420 = 18
509 = 19
511 = 20
512 = 0
513 = 22
514 = 23
516 = 24
517 = 25
518 = 26
521 = 27
524 = 28
529 = 29
533 = 30
536 = 31
537 = 32
538 = 33
541 = 34
545 = 35
546 = 36
601 = 37
602 = 38
603 = 39
605 = 40
613 = 41
614 = 42
615 = 43
619 = 44
620 = 45
622 = 46
701 = 47
704 = 48
708 = 49
709 = 50
710 = 51
711 = 52
712 = 53
713 = 54
801 = 55
802 = 56
807 = 57
808 = 58
818 = 59
823 = 60
839 = 61
840 = 62
841 = 63
901 = 64
905 = 65
909 = 66
914 = 67
1002 = 68
1009 = 69
1010 = 70
1011 = 71
1102 = 72
1103 = 73
1109 = 74
1110 = 75
1113 = 76
1120 = 77
1304 = 78
1305 = 79
1603 = 80
1604 = 81
1702 = 82
1703 = 83
1708 = 84
1801 = 85
1802 = 86
1804 = 87
2104 = 88
2109 = 89
2111 = 90
2302 = 91
2305 = 92
2403 = 93
2502 = 94
399 = 95
699 = 95
799 = 95
999 = 95
1099 = 95
1399 = 95
1699 = 95
1799 = 95
1899 = 95
2199 = 95
2299 = 95
2399 = 95
2499 = 95
2599 = 95
199 = 96
299 = 96
499 = 96
599 = 96
899 = 96
1199 = 96
1299 = 96
1999 = 96
2099 = 96
OTHER = -1;

VALUE D7PSFMT
101 = 1
103 = 2
104 = 3
106 = 4
109 = 5
110 = 6
111 = 7
112 = 8
113 = 9
114 = 10
115 = 11
116 = 12
119 = 13
121 = 14
122 = 15
126 = 16
127 = 17
128 = 18
401 = 19
402 = 20
403 = 21
404 = 22
408 = 23
409 = 24
410 = 25
411 = 26
413 = 27
414 = 28
415 = 29
417 = 30
418 = 31
420 = 0
501 = 33
502 = 34
503 = 35
504 = 36
505 = 37
506 = 38
507 = 39
509 = 40
511 = 41
512 = 42
516 = 43
517 = 44
518 = 45
519 = 46
520 = 47
521 = 48
524 = 49
525 = 50
526 = 51
527 = 52
529 = 53
531 = 54
532 = 55
533 = 56
536 = 57
537 = 58
538 = 59
539 = 60
540 = 61
541 = 62
542 = 63
543 = 64
544 = 65
545 = 66
546 = 67
601 = 68
602 = 69
603 = 70
604 = 71
605 = 72
607 = 73
608 = 74
611 = 75
613 = 76
615 = 77
616 = 78
617 = 79
618 = 80
619 = 81
620 = 82
622 = 83
701 = 84
702 = 85
704 = 86
706 = 87
707 = 88
708 = 89
710 = 90
711 = 91
712 = 92
713 = 93
801 = 94
802 = 95
804 = 96
806 = 97
807 = 98
808 = 99
813 = 100
816 = 101
818 = 102
822 = 103
824 = 104
826 = 105
830 = 106
831 = 107
836 = 108
839 = 109
840 = 110
841 = 111
842 = 112
844 = 113
845 = 114
905 = 115
909 = 116
910 = 117
911 = 118
914 = 119
918 = 120
1001 = 121
1003 = 122
1004 = 123
1008 = 124
1009 = 125
1010 = 126
1011 = 127
1013 = 128
1014 = 129
1103 = 130
1106 = 131
1109 = 132
1110 = 133
1113 = 134
1115 = 135
1120 = 136
1305 = 137
1401 = 138
1402 = 139
1403 = 140
1406 = 141
1413 = 142
1601 = 143
1603 = 144
1604 = 145
1606 = 146
1607 = 147
1801 = 148
1802 = 149
1804 = 150
1805 = 151
1806 = 152
1808 = 153
1907 = 154
2001 = 155
2101 = 156
2102 = 157
2104 = 158
2109 = 159
2111 = 160
2206 = 0
2208 = 162
2210 = 163
2301 = 164
2302 = 165
2303 = 166
2305 = 167
2401 = 168
2402 = 169
2403 = 170
2404 = 171
199 = 172
399 = 172
499 = 172
599 = 172
799 = 172
999 = 172
1099 = 172
1199 = 172
1399 = 172
2199 = 172
2299 = 172
2399 = 172
299 = 173
699 = 173
899 = 173
1299 = 173
1499 = 173
1999 = 173
OTHER = -1;

%MACRO D8;

%IF &POAFG. = 0 %THEN %DO;

VALUE D8PSFMT
402 = 1
504 = 2
506 = 3
507 = 4
509 = 5
516 = 6
536 = 0
539 = 8
540 = 9
543 = 10
602 = 11
905 = 12
1801 = 13
2301 = 14
199 = 15
299 = 15
499 = 15
599 = 15
1199 = 15
1699 = 15
1799 = 15
2199 = 15
2599 = 15
399 = 16
699 = 16
799 = 16
999 = 16
1099 = 16
1299 = 16
1399 = 16
OTHER = -1;

%END;
%ELSE %DO;

VALUE D8PSFMT
OTHER = -1;

%END;

%MEND;

%D8;

VALUE D9PSFMT
101 = 1
103 = 2
104 = 3
106 = 4
301 = 5
302 = 6
309 = 7
315 = 8
401 = 9
402 = 10
501 = 11
502 = 12
503 = 13
504 = 14
505 = 15
506 = 16
507 = 17
509 = 18
511 = 19
512 = 20
513 = 21
516 = 22
536 = 23
537 = 24
538 = 25
539 = 26
540 = 27
541 = 28
543 = 29
544 = 30
546 = 31
601 = 32
602 = 33
603 = 34
604 = 35
605 = 36
607 = 37
608 = 38
609 = 39
611 = 40
612 = 41
613 = 42
701 = 43
702 = 44
703 = 45
704 = 46
713 = 47
801 = 48
802 = 49
804 = 50
807 = 51
808 = 52
813 = 53
816 = 54
818 = 55
839 = 56
840 = 57
841 = 58
844 = 59
901 = 60
902 = 61
903 = 62
905 = 63
906 = 64
908 = 65
909 = 66
1001 = 67
1002 = 68
1004 = 69
1005 = 70
1006 = 71
1008 = 72
1102 = 73
1103 = 74
1104 = 75
1105 = 76
1109 = 77
1201 = 78
1202 = 79
1204 = 80
1301 = 81
1302 = 82
1303 = 83
1304 = 84
1305 = 0
1306 = 86
1311 = 87
1601 = 88
1603 = 89
1701 = 90
1705 = 91
1714 = 92
1801 = 93
2101 = 94
2102 = 95
2104 = 96
2301 = 97
2403 = 98
399 = 99
599 = 99
799 = 99
999 = 99
1299 = 99
1399 = 99
2299 = 99
2599 = 99
199 = 100
299 = 100
899 = 100
1099 = 100
1199 = 100
1799 = 100
1999 = 100
2199 = 100
2499 = 100
OTHER = -1;

VALUE D10PSFMT
101 = 1
104 = 2
401 = 3
402 = 4
501 = 5
502 = 6
504 = 7
505 = 8
506 = 9
507 = 0
509 = 11
516 = 12
536 = 13
540 = 14
544 = 15
602 = 16
605 = 17
613 = 18
701 = 19
713 = 20
801 = 21
807 = 22
840 = 23
841 = 24
905 = 25
1004 = 26
1102 = 27
1103 = 28
1109 = 29
1305 = 30
1702 = 31
1801 = 32
2104 = 33
2301 = 34
199 = 35
599 = 35
699 = 35
799 = 35
1199 = 35
1699 = 35
1799 = 35
1999 = 35
2199 = 35
2299 = 35
2499 = 35
2599 = 35
299 = 36
399 = 36
899 = 36
999 = 36
1099 = 36
1299 = 36
1399 = 36
OTHER = -1;

VALUE D11PSFMT
101 = 1
103 = 2
104 = 3
106 = 4
128 = 5
129 = 6
301 = 7
305 = 8
307 = 9
309 = 10
315 = 11
326 = 12
601 = 13
602 = 0
603 = 15
604 = 16
605 = 17
607 = 18
608 = 19
609 = 20
611 = 21
613 = 22
701 = 23
702 = 24
703 = 25
704 = 26
705 = 27
706 = 28
707 = 29
713 = 30
801 = 31
802 = 32
804 = 33
806 = 34
807 = 35
808 = 36
811 = 37
813 = 38
816 = 39
818 = 40
837 = 41
838 = 42
839 = 43
840 = 44
841 = 45
843 = 46
844 = 47
845 = 48
901 = 49
905 = 50
906 = 51
909 = 52
1001 = 53
1002 = 54
1004 = 55
1005 = 56
1006 = 57
1008 = 58
1102 = 59
1103 = 60
1105 = 61
1106 = 62
1109 = 63
1201 = 64
1202 = 65
1301 = 66
1302 = 67
1303 = 68
1304 = 69
1305 = 70
1306 = 71
1311 = 72
1601 = 73
1603 = 74
1701 = 75
1702 = 76
1705 = 77
1706 = 78
1714 = 79
1801 = 80
2101 = 81
2102 = 82
2104 = 83
2301 = 84
2402 = 85
2403 = 86
2501 = 87
399 = 88
1099 = 88
1299 = 88
1399 = 88
1999 = 88
2299 = 88
2499 = 88
199 = 89
299 = 89
699 = 89
899 = 89
999 = 89
1199 = 89
OTHER = -1;

VALUE D12PSFMT
101 = 1
103 = 2
104 = 3
106 = 4
128 = 5
129 = 6
301 = 7
305 = 8
315 = 9
326 = 10
401 = 11
402 = 12
501 = 13
502 = 14
503 = 15
504 = 16
505 = 17
506 = 18
507 = 19
509 = 20
510 = 21
511 = 22
512 = 23
513 = 24
514 = 25
515 = 26
516 = 27
536 = 28
537 = 29
538 = 30
539 = 31
540 = 32
541 = 33
542 = 34
543 = 35
544 = 36
545 = 37
546 = 38
601 = 39
602 = 40
603 = 41
604 = 42
605 = 43
607 = 44
608 = 45
609 = 46
611 = 47
612 = 48
613 = 49
701 = 50
702 = 51
703 = 52
704 = 53
705 = 54
706 = 55
707 = 56
713 = 57
801 = 0
802 = 59
804 = 60
806 = 61
807 = 62
808 = 63
811 = 64
812 = 65
813 = 66
814 = 67
815 = 68
816 = 69
818 = 70
837 = 71
838 = 72
839 = 73
840 = 74
841 = 75
842 = 76
843 = 77
844 = 78
845 = 79
901 = 80
902 = 81
903 = 82
905 = 83
906 = 84
909 = 85
1001 = 86
1002 = 87
1003 = 88
1004 = 89
1005 = 90
1006 = 91
1008 = 92
1102 = 93
1103 = 94
1104 = 95
1105 = 96
1106 = 97
1107 = 98
1109 = 99
1201 = 100
1202 = 101
1203 = 102
1206 = 103
1209 = 104
1301 = 105
1302 = 106
1303 = 107
1304 = 108
1305 = 109
1306 = 110
1309 = 111
1310 = 112
1311 = 113
1601 = 114
1603 = 115
1701 = 116
1702 = 117
1705 = 118
1706 = 119
1714 = 120
1801 = 121
1901 = 122
2101 = 123
2102 = 124
2104 = 125
2206 = 126
2208 = 127
2301 = 128
2401 = 129
2402 = 130
2403 = 131
2501 = 132
199 = 133
0299 = 133
0899 = 133
1099 = 133
1299 = 133
1399 = 133
0399 = 134
0999 = 134
2199 = 134
OTHER = -1;

VALUE D13PSFMT
101 = 1
103 = 2
104 = 3
106 = 4
401 = 5
402 = 6
501 = 7
502 = 8
503 = 9
504 = 10
505 = 11
506 = 12
507 = 13
509 = 14
510 = 15
516 = 16
536 = 17
537 = 18
539 = 19
540 = 20
544 = 21
602 = 0
603 = 23
604 = 24
605 = 25
608 = 26
613 = 27
701 = 28
702 = 29
704 = 30
713 = 31
801 = 32
802 = 33
807 = 34
837 = 35
839 = 36
840 = 37
841 = 38
844 = 39
905 = 40
909 = 41
1001 = 42
1002 = 43
1003 = 44
1004 = 45
1008 = 46
1103 = 47
1105 = 48
1106 = 49
1109 = 50
1202 = 51
1305 = 52
1601 = 53
2102 = 54
2104 = 55
2301 = 56
199 = 57
599 = 57
799 = 57
1199 = 57
1299 = 57
1699 = 57
2299 = 57
2499 = 57
299 = 58
399 = 58
699 = 58
899 = 58
999 = 58
1099 = 58
1399 = 58
1999 = 58
2199 = 58
OTHER = -1;

VALUE D14PSFMT
507 = 1
601 = 2
602 = 0
603 = 4
604 = 5
605 = 6
608 = 7
611 = 8
613 = 9
701 = 10
704 = 11
1004 = 12
1102 = 13
1201 = 14
1302 = 15
1305 = 16
1801 = 17
2104 = 18
2403 = 19
499 = 20
599 = 20
899 = 20
1099 = 20
1199 = 20
1299 = 20
1799 = 20
1999 = 20
2199 = 20
2299 = 20
2499 = 20
199 = 21
699 = 21
799 = 21
999 = 21
1399 = 21
1699 = 21
2399 = 21
OTHER = -1;

VALUE D15PSFMT
101 = 1
103 = 2
104 = 3
106 = 4
111 = 5
119 = 6
126 = 7
128 = 8
301 = 9
302 = 10
305 = 11
315 = 12
326 = 13
401 = 14
402 = 15
404 = 16
406 = 17
410 = 18
411 = 19
414 = 20
420 = 21
501 = 22
502 = 23
503 = 24
504 = 25
505 = 26
506 = 27
507 = 28
509 = 29
511 = 30
512 = 31
515 = 32
516 = 33
517 = 34
518 = 35
519 = 36
521 = 37
524 = 38
525 = 39
529 = 40
533 = 41
536 = 42
537 = 43
538 = 44
539 = 45
540 = 46
541 = 47
542 = 48
543 = 49
544 = 50
545 = 51
546 = 52
601 = 53
602 = 0
603 = 55
604 = 56
605 = 57
607 = 58
608 = 59
609 = 60
611 = 61
612 = 62
613 = 63
614 = 64
615 = 65
616 = 66
619 = 67
620 = 68
622 = 69
701 = 70
702 = 71
703 = 72
704 = 73
705 = 74
706 = 75
707 = 76
708 = 77
709 = 78
710 = 79
711 = 80
712 = 81
713 = 82
801 = 83
802 = 84
806 = 85
807 = 86
808 = 87
811 = 88
813 = 89
814 = 90
816 = 91
818 = 92
826 = 93
837 = 94
838 = 95
839 = 96
840 = 97
841 = 98
843 = 99
844 = 100
845 = 101
901 = 102
905 = 103
906 = 104
909 = 105
914 = 106
1002 = 107
1004 = 108
1006 = 109
1008 = 110
1011 = 111
1102 = 112
1103 = 113
1104 = 114
1105 = 115
1106 = 116
1107 = 117
1109 = 118
1110 = 119
1113 = 120
1115 = 121
1120 = 122
1201 = 123
1202 = 124
1206 = 125
1209 = 126
1301 = 127
1302 = 128
1303 = 129
1304 = 130
1305 = 131
1306 = 132
1307 = 133
1308 = 134
1309 = 135
1310 = 136
1311 = 137
1314 = 138
1601 = 139
1603 = 140
1604 = 141
1701 = 142
1702 = 143
1703 = 144
1705 = 145
1706 = 146
1714 = 147
1801 = 148
1802 = 149
1804 = 150
2101 = 151
2102 = 152
2104 = 153
2109 = 154
2111 = 155
2301 = 156
2302 = 157
2305 = 158
2402 = 159
2403 = 160
299 = 161
399 = 161
999 = 161
1199 = 161
1299 = 161
1399 = 161
2299 = 161
2499 = 161
2599 = 161
199 = 162
499 = 162
599 = 162
699 = 162
899 = 162
1099 = 162
1699 = 162
1799 = 162
1899 = 162
1999 = 162
2099 = 162
2199 = 162
2399 = 162
OTHER = -1;

VALUE D17PSFMT
OTHER = -1;

VALUE D18PSFMT
OTHER = -1;

VALUE D19PSFMT
OTHER = -1;

 /* --------------------------------------------------------------- */
 /* ---   OPERATING ROOM PROCEDURE CODES                           - */
 /* --------------------------------------------------------------- */

VALUE $ORPROC
"0044" = "1" /* PROC-VESSEL BIFURCATION */
"0050" = "1" /* IMPL CRT PACEMAKER SYS */
"0051" = "1" /* IMPL CRT DEFIBRILLAT SYS */
"0052" = "1" /* IMP/REP LEAD LF VEN SYS */
"0053" = "1" /* IMP/REP CRT PACEMAKR GEN */
"0054" = "1" /* IMP/REP CRT DEFIB GENAT */
"0056" = "1" /* INS/REP IMPL SENSOR LEAD */
"0057" = "1" /* IMP/REP SUBCUE CARD DEV */
"0061" = "1" /* PERC ANGIO PRECEREB VESS */
"0062" = "1" /* PERC ANGIO INTRACRAN VES */
"0066" = "1" /* PTCA OR CORONARY ATHER */
"0070" = "1" /* REV HIP REPL-ACETAB/FEM */
"0071" = "1" /* REV HIP REPL-ACETAB COMP */
"0072" = "1" /* REV HIP REPL-FEM COMP */
"0073" = "1" /* REV HIP REPL-LINER/HEAD */
"0074" = "1" /* HIP REPL SURF-METAL/POLY */
"0075" = "1" /* HIP REP SURF-METAL/METAL */
"0076" = "1" /* HIP REP SURF-CERMC/CERMC */
"0077" = "1" /* HIP REPL SURF-CERMC/POLY */
"0080" = "1" /* REV KNEE REPLACEMT-TOTAL */
"0081" = "1" /* REV KNEE REPL-TIBIA COMP */
"0082" = "1" /* REV KNEE REPL-FEMUR COMP */
"0083" = "1" /* REV KNEE REPLACE-PATELLA */
"0084" = "1" /* REV KNEE REPL-TIBIA LIN */
"0085" = "1" /* RESRF HIPTOTAL-ACET/FEM */
"0086" = "1" /* RESRF HIPPART-FEM HEAD */
"0087" = "1" /* RESRF HIPPART-ACETABLUM */
"0112" = "1" /* OPEN CEREB MENINGES BX */
"0114" = "1" /* OPEN BRAIN BIOPSY */
"0115" = "1" /* SKULL BIOPSY */
"0118" = "1" /* OTHER BRAIN DX PROCEDURE */
"0119" = "1" /* OTHER SKULL DX PROCEDURE */
"0121" = "1" /* CRANIAL SINUS I & D */
"0122" = "1" /* REMOV INTRACRAN STIMULAT */
"0123" = "1" /* REOPEN CRANIOTOMY SITE */
"0124" = "1" /* OTHER CRANIOTOMY */
"0125" = "1" /* OTHER CRANIECTOMY */
"0126" = "1" /* INS CATH-CRANIAL CAVITY */
"0127" = "1" /* REM CATH-CRANIAL CAVITY */
"0128" = "1" /* INTRACEREB CTH-BURR HOLE */
"0131" = "1" /* INCISE CEREBRAL MENINGES */
"0132" = "1" /* LOBOTOMY & TRACTOTOMY */
"0139" = "1" /* OTHER BRAIN INCISION */
"0141" = "1" /* THALAMUS OPERATIONS */
"0142" = "1" /* GLOBUS PALLIDUS OPS */
"0151" = "1" /* EX CEREB MENINGEAL LES */
"0152" = "1" /* HEMISPHERECTOMY */
"0153" = "1" /* BRAIN LOBECTOMY */
"0159" = "1" /* OTHER BRAIN EXCISION */
"016 " = "1" /* EXCISE SKULL LESION */
"0201" = "1" /* LINEAR CRANIECTOMY */
"0202" = "1" /* ELEVATE SKULL FX FRAGMNT */
"0203" = "1" /* SKULL FLAP FORMATION */
"0204" = "1" /* BONE GRAFT TO SKULL */
"0205" = "1" /* SKULL PLATE INSERTION */
"0206" = "1" /* CRANIAL OSTEOPLASTY NEC */
"0207" = "1" /* SKULL PLATE REMOVAL */
"0211" = "1" /* SIMPLE SUTURE OF DURA */
"0212" = "1" /* BRAIN MENINGE REPAIR NEC */
"0213" = "1" /* MENINGE VESSEL LIGATION */
"0214" = "1" /* CHOROID PLEXECTOMY */
"022 " = "1" /* VENTRICULOSTOMY */
"0231" = "1" /* VENTRICL SHUNT-HEAD/NECK */
"0232" = "1" /* VENTRI SHUNT-CIRCULA SYS */
"0233" = "1" /* VENTRICL SHUNT-THORAX */
"0234" = "1" /* VENTRICL SHUNT-ABDOMEN */
"0235" = "1" /* VENTRI SHUNT-UNINARY SYS */
"0239" = "1" /* OTHER VENTRICULAR SHUNT */
"0242" = "1" /* REPLACE VENTRICLE SHUNT */
"0243" = "1" /* REMOVE VENTRICLE SHUNT */
"0291" = "1" /* LYSIS CORTICAL ADHESION */
"0292" = "1" /* BRAIN REPAIR */
"0293" = "1" /* IMPLANT BRAIN STIMULATOR */
"0294" = "1" /* INSERT/REPLAC SKULL TONG */
"0299" = "1" /* SKULL & BRAIN OP NEC */
"0301" = "1" /* REMOVAL FB SPINAL CANAL */
"0302" = "1" /* REOPEN LAMINECTOMY SITE */
"0309" = "1" /* SPINAL CANAL EXPLOR NEC */
"031 " = "1" /* INTRASPIN NERVE ROOT DIV */
"0321" = "1" /* PERCUTANEOUS CHORDOTOMY */
"0329" = "1" /* OTHER CHORDOTOMY */
"0332" = "1" /* SPINAL CORD/MENINGES BX */
"0339" = "1" /* OTHER SPINAL DX PROC */
"034 " = "1" /* EXCIS SPINAL CORD LESION */
"0351" = "1" /* SPINE MENINGOCELE REPAIR */
"0352" = "1" /* MYELOMENINGOCEL REPAIR */
"0353" = "1" /* VERTEBRAL FX REPAIR */
"0359" = "1" /* SPINAL STRUCT REPAIR NEC */
"036 " = "1" /* SPINAL CORD ADHESIOLYSIS */
"0371" = "1" /* SUBARACH-PERITON SHUNT */
"0372" = "1" /* SUBARACH-URETERAL SHUNT */
"0379" = "1" /* OTH SPINAL THECAL SHUNT */
"0393" = "1" /* INSERT SPINAL STIMULATOR */
"0394" = "1" /* REMOVE SPINAL STIMULATOR */
"0397" = "1" /* REVISE SPINE THECA SHUNT */
"0398" = "1" /* REMOVE SPINE THECA SHUNT */
"0399" = "1" /* SPINE CANAL STRUC OP NEC */
"0401" = "1" /* EXCISION ACOUSTC NEUROMA */
"0402" = "1" /* TRIGEMINAL NERV DIVISION */
"0403" = "1" /* PERIPH NERVE DIV NEC */
"0404" = "1" /* PERIPH NERVE INCIS NEC */
"0405" = "1" /* GASSERIAN GANGLIONECTOMY */
"0406" = "1" /* PERIPH GANGLIONECT NEC */
"0407" = "1" /* PERIPH NERV EXCISION NEC */
"0412" = "1" /* OPEN PERIPH NERVE BIOPSY */
"0419" = "1" /* PERIPH NERVE DX PROC NEC */
"043 " = "1" /* PERIPHERAL NERVE SUTURE */
"0441" = "1" /* DECOMPRESS TRIGEM ROOT */
"0442" = "1" /* CRAN NERV ROOT DECOM NEC */
"0443" = "1" /* CARPAL TUNNEL RELEASE */
"0444" = "1" /* TARSAL TUNNEL RELEASE */
"0449" = "1" /* PER NERVE ADHESIOLYS NEC */
"045 " = "1" /* PERIPHERAL NERVE GRAFT */
"046 " = "1" /* PERIPH NERVE TRANSPOSIT */
"0471" = "1" /* HYPOGLOSS-FACIAL ANASTOM */
"0472" = "1" /* ACCESSORY-FACIAL ANASTOM */
"0473" = "1" /* ACCESS-HYPOGLOSS ANASTOM */
"0474" = "1" /* PERIPH NERV ANASTOM NEC */
"0475" = "1" /* POSTOP REVIS PER NERV OP */
"0476" = "1" /* LATE REPAIR PER NERV INJ */
"0479" = "1" /* OTHER NEUROPLASTY */
"0491" = "1" /* NEURECTASIS */
"0492" = "1" /* IMPLANT PERIPH STIMULAT */
"0493" = "1" /* REMOVE PERIPH STIMULATOR */
"0499" = "1" /* PERIPHERAL NERVE OPS NEC */
"050 " = "1" /* SYMPATH NERVE DIVISION */
"0511" = "1" /* SYMPATHETIC NERVE BIOPSY */
"0519" = "1" /* SYMPATH NRV DX PROC NEC */
"0521" = "1" /* SPHENOPALATIN GANGLIONEC */
"0522" = "1" /* CERVICAL SYMPATHECTOMY */
"0523" = "1" /* LUMBAR SYMPATHECTOMY */
"0524" = "1" /* PRESACRAL SYMPATHECTOMY */
"0525" = "1" /* PERIART SYMPATHECTOMY */
"0529" = "1" /* OTHER SYMPATHECTOMY */
"0581" = "1" /* SYMPATHETIC NERVE REPAIR */
"0589" = "1" /* SYMPATHETIC NERVE OP NEC */
"059 " = "1" /* OTHER NERVOUS SYSTEM OPS */
"0602" = "1" /* REOPEN THYROID FIELD WND */
"0609" = "1" /* INCIS THYROID FIELD NEC */
"0612" = "1" /* OPEN THYROID GLAND BX */
"0613" = "1" /* PARATHYROID BIOPSY */
"0619" = "1" /* THYR/PARATHY DX PROC NEC */
"062 " = "1" /* UNILAT THYROID LOBECTOMY */
"0631" = "1" /* EXCISION THYROID LESION */
"0639" = "1" /* PART THYROIDECTOMY NEC */
"064 " = "1" /* COMPLETE THYROIDECTOMY */
"0650" = "1" /* SUBSTERN THYROIDECT NOS */
"0651" = "1" /* PART SUBSTERN THYROIDECT */
"0652" = "1" /* TOT SUBSTERN THYROIDECT */
"066 " = "1" /* LINGUAL THYROID EXCISION */
"067 " = "1" /* THYROGLOSS DUCT EXCISION */
"0681" = "1" /* TOTAL PARATHYROIDECTOMY */
"0689" = "1" /* OTHER PARATHYROIDECTOMY */
"0691" = "1" /* THYROID ISTHMUS DIVISION */
"0692" = "1" /* THYROID VESSEL LIGATION */
"0693" = "1" /* THYROID SUTURE */
"0694" = "1" /* THYROID REIMPLANTATION */
"0695" = "1" /* PARATHYROID REIMPLANT */
"0698" = "1" /* OTHER THYROID OPERATIONS */
"0699" = "1" /* OTHER PARATHYROID OPS */
"0700" = "1" /* ADRENAL EXPLORATION NOS */
"0701" = "1" /* UNILAT ADRENAL EXPLORAT */
"0702" = "1" /* BILAT ADRENAL EXPLORAT */
"0712" = "1" /* OPEN ADRENAL GLAND BX */
"0713" = "1" /* TRANSFRONT PITUITARY BX */
"0714" = "1" /* TRANSPHEN PITUITARY BX */
"0715" = "1" /* PITUITARY BIOPSY NOS */
"0716" = "1" /* THYMUS BIOPSY */
"0717" = "1" /* PINEAL BIOPSY */
"0719" = "1" /* ENDOCRINE DX PROC NEC */
"0721" = "1" /* ADRENAL LESION EXCISION */
"0722" = "1" /* UNILATERAL ADRENALECTOMY */
"0729" = "1" /* PART ADRENALECTOMY NEC */
"073 " = "1" /* BILATERAL ADRENALECTOMY */
"0741" = "1" /* ADRENAL INCISION */
"0742" = "1" /* ADRENAL NERVE DIVISION */
"0743" = "1" /* ADRENAL VESSEL LIGATION */
"0744" = "1" /* ADRENAL REPAIR */
"0745" = "1" /* ADRENAL REIMPLANTATION */
"0749" = "1" /* ADRENAL OPERATION NEC */
"0751" = "1" /* PINEAL FIELD EXPLORATION */
"0752" = "1" /* PINEAL GLAND INCISION */
"0753" = "1" /* PARTIAL PINEALECTOMY */
"0754" = "1" /* TOTAL PINEALECTOMY */
"0759" = "1" /* PINEAL OPERATION NEC */
"0761" = "1" /* EXC PITUIT LES-TRANSFRON */
"0762" = "1" /* EXC PITUIT LES-TRANSPHEN */
"0763" = "1" /* PART EXCIS PITUITARY NOS */
"0764" = "1" /* TOT EXC PITUIT-TRANSFRON */
"0765" = "1" /* TOT EXC PITUIT-TRANSPHEN */
"0768" = "1" /* TOTAL EXC PITUITARY NEC */
"0769" = "1" /* TOTAL EXC PITUITARY NOS */
"0771" = "1" /* PITUITARY FOSSA EXPLORAT */
"0772" = "1" /* PITUITARY GLAND INCISION */
"0779" = "1" /* PITUITARY OPERATION NEC */
"0780" = "1" /* THYMECTOMY NOS */
"0781" = "1" /* PART EXCISION OF THYMUS */
"0782" = "1" /* TOTAL EXCISION OF THYMUS */
"0791" = "1" /* THYMUS FIELD EXPLORATION */
"0792" = "1" /* INCISION OF THYMUS */
"0793" = "1" /* REPAIR OF THYMUS */
"0794" = "1" /* THYMUS TRANSPLANTATION */
"0799" = "1" /* THYMUS OPERATION NEC */
"0811" = "1" /* EYELID BIOPSY */
"0820" = "1" /* REMOVE EYELID LESION NOS */
"0821" = "1" /* CHALAZION EXCISION */
"0822" = "1" /* EXCISE MINOR LES LID NEC */
"0823" = "1" /* EXC MAJ LES LID PRT-THIC */
"0824" = "1" /* EXC MAJ LES LID FUL-THIC */
"0825" = "1" /* DESTRUCTION LID LESION */
"0831" = "1" /* PTOSIS REP-FRONT MUS SUT */
"0832" = "1" /* PTOSIS REP-FRON MUS SLNG */
"0833" = "1" /* PTOSIS REP-LEVAT MUS ADV */
"0834" = "1" /* PTOSIS REP-LEVAT MUS NEC */
"0835" = "1" /* PTOS REP-TARSAL TECHNIQ */
"0836" = "1" /* BLEPHAROPTOS REPAIR NEC */
"0837" = "1" /* REDUC OVERCORRECT PTOSIS */
"0838" = "1" /* CORRECT LID RETRACTION */
"0841" = "1" /* THERMOCAUT/ENTROPION REP */
"0842" = "1" /* SUTURE ENTROPION REPAIR */
"0843" = "1" /* WEDG RESEC ENTROPION REP */
"0844" = "1" /* LID RECONS ENTROPION REP */
"0849" = "1" /* ENTROPION/ECTROP REP NEC */
"0851" = "1" /* CANTHOTOMY */
"0852" = "1" /* BLEPHARORRHAPHY */
"0859" = "1" /* ADJUST LID POSITION NEC */
"0861" = "1" /* LID RECONST W SKIN GRAFT */
"0862" = "1" /* LID RECONST W MUC GRAFT */
"0863" = "1" /* LID RECONST W HAIR GRAFT */
"0864" = "1" /* LID RECON-TARSOCONJ FLAP */
"0869" = "1" /* LID RECONSTR W GRAFT NEC */
"0870" = "1" /* LID RECONSTRUCTION NOS */
"0871" = "1" /* LID MARG RECON-PART THIC */
"0872" = "1" /* LID RECONS-PART THIC NEC */
"0873" = "1" /* LID MARG RECONS FUL THIC */
"0874" = "1" /* LID RECONST-FUL THIC NEC */
"0891" = "1" /* ELECTROSURG LID EPILAT */
"0892" = "1" /* CRYOSURG LID EPILATION */
"0893" = "1" /* EYELID EPILATION NEC */
"0899" = "1" /* EYELID OPERATION NEC */
"090 " = "1" /* LACRIMAL GLAND INCISION */
"0911" = "1" /* LACRIMAL GLAND BIOPSY */
"0912" = "1" /* LACRIMAL SAC BIOPSY */
"0919" = "1" /* LACRIMAL SYS DX PROC NEC */
"0920" = "1" /* EXC LACRIMAL GLAND NOS */
"0921" = "1" /* EXCIS LES LACRIMAL GLAND */
"0922" = "1" /* PART DACRYOADENECT NEC */
"0923" = "1" /* TOTAL DACRYOADENECTOMY */
"093 " = "1" /* OTHER LACRIMAL GLAND OPS */
"0941" = "1" /* LACRIMAL PUNCTUM PROBE */
"0942" = "1" /* LAC CANALICULI PROBE */
"0943" = "1" /* NASOLACRIMAL DUCT PROBE */
"0944" = "1" /* NASOLAC DUCT INTUBAT */
"0949" = "1" /* LAC PASSAGE MANIP NEC */
"0951" = "1" /* LAC PUNCTUM INCISION */
"0952" = "1" /* LAC CANALICULI INCISION */
"0953" = "1" /* LACRIMAL SAC INCISION */
"0959" = "1" /* LACRIM PASSAGE INCIS NEC */
"096 " = "1" /* LACRIM SAC/PASSAGE EXCIS */
"0971" = "1" /* CORRECT EVERTED PUNCTUM */
"0972" = "1" /* PUNCTUM REPAIR NEC */
"0973" = "1" /* CANALICULUS REPAIR */
"0981" = "1" /* DACRYOCYSTORHINOSTOMY */
"0982" = "1" /* CONJUNCTIVOCYSTORHINOST */
"0983" = "1" /* CONJUNCTIVORHINOS W TUBE */
"0991" = "1" /* LAC PUNCTUM OBLITERATION */
"0999" = "1" /* LACRIMAL SYSTEM OP NEC */
"100 " = "1" /* INCISE/REMOV CONJUNCT FB */
"101 " = "1" /* CONJUNCTIVA INCISION NEC */
"1021" = "1" /* CONJUNCTIVAL BIOPSY */
"1029" = "1" /* CONJUNCTIVA DX PROC NEC */
"1031" = "1" /* EXCISE CONJUNCTIV LESION */
"1032" = "1" /* DESTRUCT CONJUNC LES NEC */
"1033" = "1" /* OTH CONJUNC DESTRUC PROC */
"1041" = "1" /* SYMBLEPH REP W FREE GRFT */
"1042" = "1" /* GRAFT CONJUNC CUL-DE-SAC */
"1043" = "1" /* CONJUN CUL-DE-SAC RX NEC */
"1044" = "1" /* CONJUNC FREE GRAFT NEC */
"1049" = "1" /* CONJUNCTIVOPLASTY NEC */
"105 " = "1" /* CONJUNC/LID ADHESIOLYSIS */
"106 " = "1" /* REPAIR CONJUNCT LACERAT */
"1091" = "1" /* SUBCONJUNCTIVAL INJECT */
"1099" = "1" /* CONJUNCTIVAL OP NEC */
"110 " = "1" /* MAGNET REMOVAL CORNEA FB */
"111 " = "1" /* CORNEAL INCISION */
"1121" = "1" /* CORNEAL SCRAPE FOR SMEAR */
"1122" = "1" /* CORNEAL BIOPSY */
"1129" = "1" /* CORNEAL DX PROC NEC */
"1131" = "1" /* PTERYGIUM TRANSPOSITION */
"1132" = "1" /* PTERYG EXC W CORNEA GRFT */
"1139" = "1" /* PTERYGIUM EXCISION NEC */
"1141" = "1" /* MECH REMOV CORNEA EPITH */
"1142" = "1" /* THERMOCAUT CORNEA LESION */
"1143" = "1" /* CRYOTHERAP CORNEA LESION */
"1149" = "1" /* DESTRUCT CORNEA LES NEC */
"1151" = "1" /* SUTURE CORNEA LACERATION */
"1152" = "1" /* REP CORNEA POSTOP DEHISC */
"1153" = "1" /* RX CORNEA LAC W CONJ FLP */
"1159" = "1" /* CORNEAL REPAIR NEC */
"1160" = "1" /* CORNEAL TRANSPLANT NOS */
"1161" = "1" /* LAM KERATPLAST W AUTGRFT */
"1162" = "1" /* LAMELLAR KERATOPLAST NEC */
"1163" = "1" /* PERF KERATOPL W AUTOGRFT */
"1164" = "1" /* PERFORAT KERATOPLAST NEC */
"1169" = "1" /* CORNEAL TRANSPLANT NEC */
"1171" = "1" /* KERATOMILEUSIS */
"1172" = "1" /* KERATOPHAKIA */
"1173" = "1" /* KERATOPROSTHESIS */
"1174" = "1" /* THERMOKERATOPLASTY */
"1175" = "1" /* RADIAL KERATOTOMY */
"1176" = "1" /* EPIKERATOPHAKIA */
"1179" = "1" /* CORNEA RECONSTRUCT NEC */
"1191" = "1" /* CORNEAL TATTOOING */
"1192" = "1" /* REMOVE CORNEAL IMPLANT */
"1199" = "1" /* CORNEAL OPERATION NEC */
"1200" = "1" /* REMOV ANT SEGMNT FB NOS */
"1201" = "1" /* MAGNET REMOV ANT SEG FB */
"1202" = "1" /* NONMAG REMOV ANT SEG FB */
"1211" = "1" /* IRIDOTOMY W TRANSFIXION */
"1212" = "1" /* IRIDOTOMY NEC */
"1213" = "1" /* PROLAPSED IRIS EXCISION */
"1214" = "1" /* IRIDECTOMY NEC */
"1221" = "1" /* DX ASPIRAT-ANT CHAMBER */
"1222" = "1" /* IRIS BIOPSY */
"1229" = "1" /* ANT SEGMENT DX PROC NEC */
"1231" = "1" /* GONIOSYNECHIAE LYSIS */
"1232" = "1" /* ANT SYNECHIA LYSIS NEC */
"1233" = "1" /* POST SYNECHIAE LYSIS */
"1234" = "1" /* CORNEOVITREAL ADHESIOLYS */
"1235" = "1" /* COREOPLASTY */
"1239" = "1" /* IRIDOPLASTY NEC */
"1240" = "1" /* REMOV ANT SEGMNT LES NOS */
"1241" = "1" /* NONEXC DESTRUC IRIS LES */
"1242" = "1" /* EXCISION OF IRIS LESION */
"1243" = "1" /* NONEXC DESTR CIL BOD LES */
"1244" = "1" /* EXCISE CILIARY BODY LES */
"1251" = "1" /* GONIOPUNCTURE */
"1252" = "1" /* GONIOTOMY */
"1253" = "1" /* GONIOTOMY W GONIOPUNCTUR */
"1254" = "1" /* TRABECULOTOMY AB EXTERNO */
"1255" = "1" /* CYCLODIALYSIS */
"1259" = "1" /* FACILIT INTRAOC CIRC NEC */
"1261" = "1" /* TREPHIN SCLERA W IRIDECT */
"1262" = "1" /* THERMCAUT SCLER W IRIDEC */
"1263" = "1" /* IRIDENCLEISIS/IRIDOTASIS */
"1264" = "1" /* TRABECULECTOM AB EXTERNO */
"1265" = "1" /* SCLER FISTULIZ W IRIDECT */
"1266" = "1" /* POSTOP REVIS SCL FISTUL */
"1269" = "1" /* SCLER FISTULIZING OP NEC */
"1271" = "1" /* CYCLODIATHERMY */
"1272" = "1" /* CYCLOCRYOTHERAPY */
"1273" = "1" /* CYCLOPHOTOCOAGULATION */
"1274" = "1" /* CIL BODY DIMINUTION NOS */
"1279" = "1" /* GLAUCOMA PROCEDURE NEC */
"1281" = "1" /* SUTURE SCLERAL LACER */
"1282" = "1" /* SCLERAL FISTULA REPAIR */
"1283" = "1" /* REVIS ANT SEG OP WND NEC */
"1284" = "1" /* DESTRUCT SCLERAL LESION */
"1285" = "1" /* REPAIR STAPHYLOM W GRAFT */
"1286" = "1" /* REP SCLER STAPHYLOMA NEC */
"1287" = "1" /* GRAFT REINFORCE SCLERA */
"1288" = "1" /* SCLERA REINFORCEMENT NEC */
"1289" = "1" /* SCLERAL OPERATION NEC */
"1291" = "1" /* THERAPEUT EVAC ANT CHAMB */
"1292" = "1" /* ANTERIOR CHAMBER INJECT */
"1293" = "1" /* REMOV EPITHEL DOWNGROWTH */
"1297" = "1" /* IRIS OPERATION NEC */
"1298" = "1" /* CILIARY BODY OP NEC */
"1299" = "1" /* ANTERIOR CHAMBER OP NEC */
"1300" = "1" /* REMOVE FB LENS NOS */
"1301" = "1" /* MAGNET REMOVE FB LENS */
"1302" = "1" /* NONMAGNET REMOVE FB LENS */
"1311" = "1" /* TEMP-INF INTRCAP LENS EX */
"1319" = "1" /* INTRACAPSUL LENS EXT NEC */
"132 " = "1" /* LINEAR EXTRACAP LENS EXT */
"133 " = "1" /* SIMPL ASPIR LENS EXTRACT */
"1341" = "1" /* CATARAC PHACOEMULS/ASPIR */
"1342" = "1" /* POST CATARAC FRAG/ASPIR */
"1343" = "1" /* CATARACT FRAG/ASPIR NEC */
"1351" = "1" /* TEMP-INF XTRACAP LENS EX */
"1359" = "1" /* EXTRACAP LENS EXTRAC NEC */
"1361" = "1" /* EXTRACAP LENS EXTRAC NEC */
"1362" = "1" /* EXTRACAP LENS EXTRAC NEC */
"1363" = "1" /* EXTRACAP LENS EXTRAC NEC */
"1364" = "1" /* AFTER-CATAR DISCISSION */
"1365" = "1" /* AFTER-CATARACT EXCISION */
"1366" = "1" /* AFTER CATAR FRAGMNTATION */
"1369" = "1" /* CATARACT EXTRACTION NEC */
"1370" = "1" /* INSERT PSEUDOPHAKOS NOS */
"1371" = "1" /* INSERT LENS AT CATAR EXT */
"1372" = "1" /* SECONDARY INSERT LENS */
"138 " = "1" /* IMPLANTED LENS REMOVAL */
"139 " = "1" /* OTHER OPERATIONS ON LENS */
"1390" = "1" /* OPERATION ON LENS NEC */
"1391" = "1" /* IMPL INTRAOC TELESC PROS */
"1400" = "1" /* REMOV POST SEGMNT FB NOS */
"1401" = "1" /* MAGNET REMOV POST SEG FB */
"1402" = "1" /* NONMAG REMOV POST SEG FB */
"1411" = "1" /* DIAGNOST VITREOUS ASPIR */
"1419" = "1" /* DX PROC POST SEG NEC */
"1421" = "1" /* CHORIORET LES DIATHERMY */
"1422" = "1" /* CHORIORETIN LES CRYOTHER */
"1426" = "1" /* CHORIORET LES RADIOTHER */
"1427" = "1" /* CHORIORET LES RAD IMPLAN */
"1429" = "1" /* CHORIORET LES DESTR NEC */
"1431" = "1" /* RETINAL TEAR DIATHERMY */
"1432" = "1" /* RETINAL TEAR CRYOTHERAPY */
"1439" = "1" /* RETINAL TEAR REPAIR NEC */
"1441" = "1" /* SCLERAL BUCKLE W IMPLANT */
"1449" = "1" /* SCLERAL BUCKLING NEC */
"1451" = "1" /* DETACH RETINA-DIATHERMY */
"1452" = "1" /* DETACH RETINA-CRYOTHERAP */
"1453" = "1" /* DETACH RETINA XENON COAG */
"1454" = "1" /* DETACH RETINA LASER COAG */
"1455" = "1" /* DETACH RET PHOTOCOAG NOS */
"1459" = "1" /* REPAIR RETINA DETACH NEC */
"146 " = "1" /* REMOV PROS MAT POST SEG */
"1471" = "1" /* ANTERIOR REMOV VITREOUS */
"1472" = "1" /* VITREOUS REMOVAL NEC */
"1473" = "1" /* ANTERIOR MECHAN VITRECT */
"1474" = "1" /* MECH VITRECTOMY NEC */
"1475" = "1" /* VITREOUS SUBSTITUT INJEC */
"1479" = "1" /* VITREOUS OPERATION NEC */
"149 " = "1" /* OTHER POST SEGMENT OPS */
"1501" = "1" /* EXTRAOC MUSC-TEND BIOPSY */
"1509" = "1" /* EXTRAOC MUSC DX PROC NEC */
"1511" = "1" /* ONE EXTRAOC MUS RECESS */
"1512" = "1" /* 1 EXTRAOC MUSCL ADVANCE */
"1513" = "1" /* 1 EXTRAOC MUSCL RESECT */
"1519" = "1" /* XTRAOC MUS OP/DETACH NEC */
"1521" = "1" /* LENGTHEN 1 EXTRAOC MUSC */
"1522" = "1" /* SHORTEN 1 EXTRAOC MUSC */
"1529" = "1" /* OP ON 1 EXTRAOC MUSC NEC */
"153 " = "1" /* TEMP DETACH >1 XTROC MUS */
"154 " = "1" /* OTH OP ON >L EXTRAOC MUS */
"155 " = "1" /* EXTRAOCUL MUS TRANSPOSIT */
"156 " = "1" /* REVIS EXTRAOC MUSC SURG */
"157 " = "1" /* EXTRAOC MUSC INJ REPAIR */
"159 " = "1" /* OTH EXTRAOC MUS-TEND OP */
"1601" = "1" /* ORBITOTOMY W BONE FLAP */
"1602" = "1" /* ORBITOTOMY W IMPLANT */
"1609" = "1" /* ORBITOTOMY NEC */
"161 " = "1" /* REMOVE PENETRAT FB EYE */
"1622" = "1" /* DIAGNOSTIC ASP OF ORBIT */
"1623" = "1" /* EYEBALL & ORBIT BIOPSY */
"1629" = "1" /* EYEBAL/ORBIT DX PROC NEC */
"1631" = "1" /* EYE EVISC W SYNCH IMPLAN */
"1639" = "1" /* EYEBALL EVISCERATION NEC */
"1641" = "1" /* EYE ENUC/IMPLAN/MUSC ATT */
"1642" = "1" /* EYE ENUC W IMPLANT NEC */
"1649" = "1" /* EYEBALL ENUCLEATION NEC */
"1651" = "1" /* RADICAL ORBITOMAXILLECT */
"1652" = "1" /* ORBIT EXENT W BONE REMOV */
"1659" = "1" /* ORBITAL EXENTERATION NEC */
"1661" = "1" /* 2NDRY OCULAR IMP INSERT */
"1662" = "1" /* REVIS/REINSERT OCUL IMP */
"1663" = "1" /* REVIS ENUC SOCKET W GRFT */
"1664" = "1" /* ENUC SOCKET REVIS NEC */
"1665" = "1" /* 2NDRY EXENT CAVITY GRAFT */
"1666" = "1" /* REVIS EXENTER CAVITY NEC */
"1669" = "1" /* 2ND OP POST EYE REM NEC */
"1671" = "1" /* REMOVE OCULAR IMPLANT */
"1672" = "1" /* REMOVE ORBITAL IMPLANT */
"1681" = "1" /* REPAIR OF ORBITAL WOUND */
"1682" = "1" /* REPAIR EYEBALL RUPTURE */
"1689" = "1" /* EYE/ORBIT INJ REPAIR NEC */
"1692" = "1" /* EXCISION ORBITAL LESION */
"1693" = "1" /* EXCISION EYE LESION NOS */
"1698" = "1" /* OPERATION ON ORBIT NEC */
"1699" = "1" /* OPERATION ON EYEBALL NEC */
"1821" = "1" /* PREAURICULAR SINUS EXCIS */
"1831" = "1" /* RAD EXCIS EXT EAR LES */
"1839" = "1" /* EXCIS EXTERNAL EAR NEC */
"185 " = "1" /* CORRECTION PROMINENT EAR */
"186 " = "1" /* EXT AUDIT CANAL RECONSTR */
"1871" = "1" /* CONSTRUCTION EAR AURICLE */
"1872" = "1" /* REATTACH AMPUTATED EAR */
"1879" = "1" /* PLASTIC REP EXT EAR NEC */
"189 " = "1" /* OTHER EXT EAR OPERATIONS */
"190 " = "1" /* STAPES MOBILIZATION */
"1911" = "1" /* STAPEDECT W REPLAC INCUS */
"1919" = "1" /* STAPEDECTOMY NEC */
"1921" = "1" /* REV STAPDEC W INCUS REPL */
"1929" = "1" /* STAPEDECTOMY REVIS NEC */
"193 " = "1" /* OSSICULAR CHAIN OP NEC */
"194 " = "1" /* MYRINGOPLASTY */
"1952" = "1" /* TYPE 2 TYMPANOPLASTY */
"1953" = "1" /* TYPE 3 TYMPANOPLASTY */
"1954" = "1" /* TYPE 4 TYMPANOPLASTY */
"1955" = "1" /* TYPE 5 TYMPANOPLASTY */
"196 " = "1" /* TYMPANOPLASTY REVISION */
"199 " = "1" /* MIDDLE EAR REPAIR NEC */
"2001" = "1" /* MYRINGOTOMY W INTUBATION */
"2021" = "1" /* MASTOID INCISION */
"2022" = "1" /* PETRUS PYRAM AIR CEL INC */
"2023" = "1" /* MIDDLE EAR INCISION */
"2032" = "1" /* MID & INNER EAR BIOPSY */
"2039" = "1" /* MID/IN EAR DX PROC NEC */
"2041" = "1" /* SIMPLE MASTOIDECTOMY */
"2042" = "1" /* RADICAL MASTOIDECTOMY */
"2049" = "1" /* MASTOIDECTOMY NEC */
"2051" = "1" /* EXCISE MIDDLE EAR LESION */
"2059" = "1" /* MIDDLE EAR EXCISION NEC */
"2061" = "1" /* INNER EAR FENESTRATION */
"2062" = "1" /* REVIS INNER EAR FENESTRA */
"2071" = "1" /* ENDOLYMPHATIC SHUNT */
"2072" = "1" /* INNER EAR INJECTION */
"2079" = "1" /* INC/EXC/DESTR IN EAR NEC */
"2091" = "1" /* TYMPANOSYMPATHECTOMY */
"2092" = "1" /* MASTOIDECTOMY REVISION */
"2093" = "1" /* REPAIR OVAL/ROUND WINDOW */
"2095" = "1" /* ELECMAG HEAR DEV IMPLANT */
"2096" = "1" /* IMPLT COCHLEAR PROST NOS */
"2097" = "1" /* IMP/REP SCHAN COCH PROS */
"2098" = "1" /* IMP/REP MCHAN COCHL PROS */
"2099" = "1" /* MID-INNER EAR OPS NEC */
"2104" = "1" /* ETHMOID ART LIGAT-EPIST */
"2105" = "1" /* MAX ART LIG FOR EPISTAX */
"2106" = "1" /* EXT CAROT ART LIG-EPIST */
"2107" = "1" /* NASAL SEPT GRFT-EPISTAX */
"2109" = "1" /* EPISTAXIS CONTROL NEC */
"214 " = "1" /* RESECTION OF NOSE */
"215 " = "1" /* SUBMUC NASAL SEPT RESECT */
"2161" = "1" /* DIATHER/CRYO TURBINECTOM */
"2162" = "1" /* TURBINATE FRACTURE */
"2169" = "1" /* TURBINECTOMY NEC */
"2172" = "1" /* OPEN REDUCTION NASAL FX */
"2182" = "1" /* NASAL FISTULA CLOSURE */
"2183" = "1" /* TOT NASAL RECONSTRUCTION */
"2184" = "1" /* REVISION RHINOPLASTY */
"2185" = "1" /* AUGMENTATION RHINOPLASTY */
"2186" = "1" /* LIMITED RHINOPLASTY */
"2187" = "1" /* RHINOPLASTY NEC */
"2188" = "1" /* SEPTOPLASTY NEC */
"2189" = "1" /* NASAL REPAIR NEC */
"2199" = "1" /* NASAL OPERATION NEC */
"2212" = "1" /* OPEN BIOPSY NASAL SINUS */
"2231" = "1" /* RADICAL MAXILLARY ANTROT */
"2239" = "1" /* EXT MAXILLARY ANTROT NEC */
"2241" = "1" /* FRONTAL SINUSOTOMY */
"2242" = "1" /* FRONTAL SINUSECTOMY */
"2250" = "1" /* SINUSOTOMY NOS */
"2251" = "1" /* ETHMOIDOTOMY */
"2252" = "1" /* SPHENOIDOTOMY */
"2253" = "1" /* MULTIPLE SINUS INCISION */
"2260" = "1" /* SINUSECTOMY NOS */
"2261" = "1" /* C-LUC EXC MAX SINUS LES */
"2262" = "1" /* EXC MAX SINUS LESION NEC */
"2263" = "1" /* ETHMOIDECTOMY */
"2264" = "1" /* SPHENOIDECTOMY */
"2271" = "1" /* NASAL SINUS FISTULA CLOS */
"2279" = "1" /* NASAL SINUS REPAIR NEC */
"229 " = "1" /* OTHER NASAL SINUS OPS */
"242 " = "1" /* GINGIVOPLASTY */
"244 " = "1" /* EXC OF DENTAL LES OF JAW */
"245 " = "1" /* ALVEOLOPLASTY */
"2502" = "1" /* OPEN BIOPSY OF TONGUE */
"251 " = "1" /* DESTRUCTION TONGUE LES */
"252 " = "1" /* PARTIAL GLOSSECTOMY */
"253 " = "1" /* COMPLETE GLOSSECTOMY */
"254 " = "1" /* RADICAL GLOSSECTOMY */
"2559" = "1" /* REPAIR OF TONGUE NEC */
"2594" = "1" /* OTHER GLOSSOTOMY */
"2599" = "1" /* TONGUE OPERATION NEC */
"2612" = "1" /* OPEN BX SALIV GLAND/DUCT */
"2621" = "1" /* SALIVARY CYST MARSUPIAL */
"2629" = "1" /* SALIV LESION EXCIS NEC */
"2630" = "1" /* SIALOADENECTOMY NOS */
"2631" = "1" /* PARTIAL SIALOADENECTOMY */
"2632" = "1" /* COMPLETE SIALOADENECTOMY */
"2641" = "1" /* SUTURE OF SALIV GLND LAC */
"2642" = "1" /* SALIVARY FISTULA CLOSURE */
"2649" = "1" /* SALIVARY REPAIR NEC */
"2699" = "1" /* SALIVARY OPERATION NEC */
"270 " = "1" /* DRAIN FACE & MOUTH FLOOR */
"271 " = "1" /* INCISION OF PALATE */
"2721" = "1" /* BONY PALATE BIOPSY */
"2722" = "1" /* UVULA AND SOFT PALATE BX */
"2731" = "1" /* LOC EXC BONY PALATE LES */
"2732" = "1" /* WIDE EXC BONY PALATE LES */
"2742" = "1" /* WIDE EXCISION OF LIP LES */
"2743" = "1" /* EXCISION OF LIP LES NEC */
"2749" = "1" /* EXCISION OF MOUTH NEC */
"2753" = "1" /* CLOSURE OF MOUTH FISTULA */
"2754" = "1" /* REPAIR OF CLEFT LIP */
"2755" = "1" /* FULL-THICK GRFT TO MOUTH */
"2756" = "1" /* SKIN GRAFT TO MOUTH NEC */
"2757" = "1" /* PEDICLE ATTACH TO MOUTH */
"2759" = "1" /* MOUTH REPAIR NEC */
"2761" = "1" /* SUTURE OF PALATE LACERAT */
"2762" = "1" /* CLEFT PALATE CORRECTION */
"2763" = "1" /* REVIS CLEFT PALAT REPAIR */
"2769" = "1" /* OTH PLASTIC REPAIR PALAT */
"2771" = "1" /* INCISION OF UVULA */
"2772" = "1" /* EXCISION OF UVULA */
"2773" = "1" /* REPAIR OF UVULA */
"2779" = "1" /* OTHER UVULA OPERATIONS */
"2792" = "1" /* MOUTH INCISION NOS */
"2799" = "1" /* ORAL CAVITY OPS NEC */
"280 " = "1" /* PERITONSILLAR I & D */
"2811" = "1" /* TONSIL&ADENOID BIOPSY */
"2819" = "1" /* TONSIL&ADENOID DX OP NEC */
"282 " = "1" /* TONSILLECTOMY */
"283 " = "1" /* TONSILLECTOMY/ADENOIDEC */
"284 " = "1" /* EXCISION OF TONSIL TAG */
"285 " = "1" /* EXCISION LINGUAL TONSIL */
"286 " = "1" /* ADENOIDECTOMY */
"287 " = "1" /* HEMORR CONTRL POST T & A */
"2891" = "1" /* INCIS TO REMOV TONSIL FB */
"2892" = "1" /* EXCIS TONSIL/ADENOID LES */
"2899" = "1" /* TONSIL/ADENOID OPS NEC */
"290 " = "1" /* PHARYNGOTOMY */
"292 " = "1" /* EXC BRANCHIAL CLEFT CYST */
"293 " = "1" /* EXC BRANCHIAL CLEFT CYST */
"2931" = "1" /* CRICOPHARYNGEAL MYOTOMY */
"2932" = "1" /* PHARYNGEAL DIVERTICULEC */
"2933" = "1" /* PHARYNGECTOMY */
"2939" = "1" /* EXCIS/DESTR LES PHAR NEC */
"294 " = "1" /* PLASTIC OP ON PHARYNX */
"2951" = "1" /* SUTURE OF PHARYNGEAL LAC */
"2952" = "1" /* CLOS BRANCH CLEFT FISTUL */
"2953" = "1" /* CLOS PHARYNX FISTULA NEC */
"2954" = "1" /* LYSIS PHARYNGEAL ADHES */
"2959" = "1" /* PHARYNGEAL REPAIR NEC */
"2992" = "1" /* DIVIS GLOSSOPHARYNG NERV */
"2999" = "1" /* PHARYNGEAL OPERATION NEC */
"3001" = "1" /* LARYNX CYST MARSUPIALIZ */
"3009" = "1" /* DESTRUCT LARYNX LES NEC */
"301 " = "1" /* HEMILARYNGECTOMY */
"3021" = "1" /* EPIGLOTTIDECTOMY */
"3022" = "1" /* VOCAL CORDECTOMY */
"3029" = "1" /* OTHER PART LARYNGECTOMY */
"303 " = "1" /* COMPLETE LARYNGECTOMY */
"304 " = "1" /* RADICAL LARYNGECTOMY */
"3121" = "1" /* MEDIASTINAL TRACHEOSTOMY */
"3129" = "1" /* OTHER PERM TRACHEOSTOMY */
"313 " = "1" /* INCIS LARYNX TRACHEA NEC */
"3145" = "1" /* OPN BX LARYNX OR TRACHEA */
"315 " = "1" /* LOCAL DESTRUC TRACH LES */
"3161" = "1" /* SUTURE OF LARYNGEAL LAC */
"3162" = "1" /* LARYNGEAL FISTULA CLOS */
"3163" = "1" /* LARYNGOSTOMY REVISION */
"3164" = "1" /* LARYNGEAL FX REPAIR */
"3169" = "1" /* OTHER LARYNGEAL REPAIR */
"3171" = "1" /* SUTURE OF TRACHEAL LACER */
"3172" = "1" /* CLOSURE OF TRACHEOSTOMY */
"3173" = "1" /* TRACHEA FISTULA CLOS NEC */
"3174" = "1" /* REVISION OF TRACHEOSTOMY */
"3175" = "1" /* TRACHEAL RECONSTRUCTION */
"3179" = "1" /* OTHER TRACHEAL REPAIR */
"3191" = "1" /* LARYNGEAL NERV DIVISION */
"3192" = "1" /* LYSIS TRACH/LARYNX ADHES */
"3198" = "1" /* OTH LARYNGEAL OPERATION */
"3199" = "1" /* OTHER TRACHEAL OPERATION */
"320 " = "1" /* OTHER TRACHEAL OPERATION */
"3209" = "1" /* OTHER DESTRUC BRONC LES */
"321 " = "1" /* OTHER BRONCHIAL EXCISION */
"3221" = "1" /* EMPHYSEMA BLEB PLICATION */
"3222" = "1" /* LUNG VOL REDUCTION SURG */
"3223" = "1" /* OPEN ABLTN LUNG LES/TISS */
"3224" = "1" /* PERC ABLTN LUNG LES/TISS */
"3225" = "1" /* THOR ABLTN LUNG LES/TISS */
"3226" = "1" /* ABLTN LUNG TISS NEC/NOS */
"3229" = "1" /* DESTROY LOC LUNG LES NEC */
"323 " = "1" /* SEGMENTAL LUNG RESECTION */
"324 " = "1" /* LOBECTOMY OF LUNG */
"325 " = "1" /* COMPLETE PNEUMONECTOMY */
"326 " = "1" /* RAD DISSEC THORAC STRUCT */
"329 " = "1" /* OTHER EXCISION OF LUNG */
"330 " = "1" /* INCISION OF BRONCHUS */
"331 " = "1" /* INCISION OF LUNG */
"3325" = "1" /* OPEN BRONCHIAL BIOPSY */
"3327" = "1" /* CLOS ENDOSCOPIC LUNG BX */
"3328" = "1" /* OPEN LUNG BIOPSY */
"3329" = "1" /* BRONCH/LUNG DX PROC NEC */
"3334" = "1" /* THORACOPLASTY */
"3339" = "1" /* SURG COLLAPS OF LUNG NEC */
"3341" = "1" /* BRONCHIAL LACERAT SUTURE */
"3342" = "1" /* BRONCHIAL FISTULA CLOS */
"3343" = "1" /* LUNG LACERATION CLOSURE */
"3348" = "1" /* BRONCHIAL REPAIR NEC */
"3349" = "1" /* LUNG REPAIR NEC */
"335 " = "1" /* LUNG REPAIR NEC */
"3350" = "1" /* LUNG TRANSPLANT NOS */
"3351" = "1" /* UNILAT LUNG TRANSPLANT */
"3352" = "1" /* BILAT LUNG TRANSPLANT */
"336 " = "1" /* COMB HEART/LUNG TRANSPLA */
"3392" = "1" /* BRONCHIAL LIGATION */
"3393" = "1" /* PUNCTURE OF LUNG */
"3398" = "1" /* BRONCHIAL OPERATION NEC */
"3399" = "1" /* LUNG OPERATION NEC */
"3402" = "1" /* EXPLORATORY THORACOTOMY */
"3403" = "1" /* REOPEN THORACOTOMY SITE */
"341 " = "1" /* INCISION OF MEDIASTINUM */
"3421" = "1" /* TRANSPLEURA THORACOSCOPY */
"3422" = "1" /* MEDIASTINOSCOPY */
"3426" = "1" /* OPEN MEDIASTINAL BIOPSY */
"3427" = "1" /* BIOPSY OF DIAPHRAGM */
"3428" = "1" /* DX PROCEDURE THORAX NEC */
"3429" = "1" /* DX PROC MEDIASTINUM NEC */
"343 " = "1" /* DESTRUCT MEDIASTIN LES */
"344 " = "1" /* DESTRUCT CHEST WALL LES */
"3451" = "1" /* DECORTICATION OF LUNG */
"3459" = "1" /* OTHER PLEURAL EXCISION */
"346 " = "1" /* SCARIFICATION OF PLEURA */
"3473" = "1" /* CLOS THORACIC FISTUL NEC */
"3474" = "1" /* PECTUS DEFORMITY REPAIR */
"3479" = "1" /* OTHER CHEST WALL REPAIR */
"3481" = "1" /* EXCISE DIAPHRAGM LESION */
"3482" = "1" /* SUTURE DIAPHRAGM LACERAT */
"3483" = "1" /* CLOSE DIAPHRAGM FISTULA */
"3484" = "1" /* OTHER DIAPHRAGM REPAIR */
"3485" = "1" /* IMPLANT DIAPHRA PACEMAKE */
"3489" = "1" /* DIAPHRAGM OPERATION NEC */
"3493" = "1" /* REPAIR OF PLEURA */
"3499" = "1" /* THORACIC OPERATION NEC */
"3500" = "1" /* CLOSED VALVOTOMY NOS */
"3501" = "1" /* CLOSED AORTIC VALVOTOMY */
"3502" = "1" /* CLOSED MITRAL VALVOTOMY */
"3503" = "1" /* CLOSED PULMON VALVOTOMY */
"3504" = "1" /* CLOSED TRICUSP VALVOTOMY */
"3510" = "1" /* OPEN VALVULOPLASTY NOS */
"3511" = "1" /* OPN AORTIC VALVULOPLASTY */
"3512" = "1" /* OPN MITRAL VALVULOPLASTY */
"3513" = "1" /* OPN PULMON VALVULOPLASTY */
"3514" = "1" /* OPN TRICUS VALVULOPLASTY */
"3520" = "1" /* REPLACE HEART VALVE NOS */
"3521" = "1" /* REPLACE AORT VALV-TISSUE */
"3522" = "1" /* REPLACE AORTIC VALVE NEC */
"3523" = "1" /* REPLACE MITR VALV-TISSUE */
"3524" = "1" /* REPLACE MITRAL VALVE NEC */
"3525" = "1" /* REPLACE PULM VALV-TISSUE */
"3526" = "1" /* REPLACE PULMON VALVE NEC */
"3527" = "1" /* REPLACE TRIC VALV-TISSUE */
"3528" = "1" /* REPLACE TRICUSP VALV NEC */
"3531" = "1" /* PAPILLARY MUSCLE OPS */
"3532" = "1" /* CHORDAE TENDINEAE OPS */
"3533" = "1" /* ANNULOPLASTY */
"3534" = "1" /* INFUNDIBULECTOMY */
"3535" = "1" /* TRABECUL CARNEAE CORD OP */
"3539" = "1" /* TISS ADJ TO VALV OPS NEC */
"3542" = "1" /* CREATE SEPTAL DEFECT */
"3550" = "1" /* PROSTH REP HRT SEPTA NOS */
"3551" = "1" /* PROS REP ATRIAL DEF-OPN */
"3552" = "1" /* PROS REPAIR ATRIA DEF-CL */
"3553" = "1" /* PROST REPAIR VENTRIC DEF */
"3554" = "1" /* PROS REP ENDOCAR CUSHION */
"3555" = "1" /* PROS REP VENTRC DEF-CLOS */
"3560" = "1" /* GRFT REPAIR HRT SEPT NOS */
"3561" = "1" /* GRAFT REPAIR ATRIAL DEF */
"3562" = "1" /* GRAFT REPAIR VENTRIC DEF */
"3563" = "1" /* GRFT REP ENDOCAR CUSHION */
"3570" = "1" /* HEART SEPTA REPAIR NOS */
"3571" = "1" /* ATRIA SEPTA DEF REP NEC */
"3572" = "1" /* VENTR SEPTA DEF REP NEC */
"3573" = "1" /* ENDOCAR CUSHION REP NEC */
"3581" = "1" /* TOT REPAIR TETRAL FALLOT */
"3582" = "1" /* TOTAL REPAIR OF TAPVC */
"3583" = "1" /* TOT REP TRUNCUS ARTERIOS */
"3584" = "1" /* TOT COR TRANSPOS GRT VES */
"3591" = "1" /* INTERAT VEN RETRN TRANSP */
"3592" = "1" /* CONDUIT RT VENT-PUL ART */
"3593" = "1" /* CONDUIT LEFT VENTR-AORTA */
"3594" = "1" /* CONDUIT ARTIUM-PULM ART */
"3595" = "1" /* HEART REPAIR REVISION */
"3596" = "1" /* PERC HEART VALVULOPLASTY */
"3598" = "1" /* OTHER HEART SEPTA OPS */
"3599" = "1" /* OTHER HEART VALVE OPS */
"3600" = "1" /* OTHER HEART VALVE OPS */
"3601" = "1" /* PTCA-1 VES/ATH W/O AGENT */
"3602" = "1" /* PTCA-1 VES/ATH W AGENT */
"3603" = "1" /* OPEN CORONRY ANGIOPLASTY */
"3605" = "1" /* PTCA-MULTIPLE VESSEL/ATH */
"3609" = "1" /* REM OF COR ART OBSTR NEC */
"3610" = "1" /* AORTOCORONARY BYPASS NOS */
"3611" = "1" /* AORTOCOR BYPAS-1 COR ART */
"3612" = "1" /* AORTOCOR BYPAS-2 COR ART */
"3613" = "1" /* AORTOCOR BYPAS-3 COR ART */
"3614" = "1" /* AORTCOR BYPAS-4+ COR ART */
"3615" = "1" /* 1 INT MAM-COR ART BYPASS */
"3616" = "1" /* 2 INT MAM-COR ART BYPASS */
"3617" = "1" /* ABD-CORON ARTERY BYPASS */
"3619" = "1" /* HRT REVAS BYPS ANAS NEC */
"362 " = "1" /* ARTERIAL IMPLANT REVASC */
"363 " = "1" /* ARTERIAL IMPLANT REVASC */
"3631" = "1" /* OPEN CHEST TRANS REVASC */
"3632" = "1" /* OTH TRANSMYO REVASCULAR */
"3633" = "1" /* ENDO TRANSMYO REVASCULAR */
"3634" = "1" /* PERC TRANSMYO REVASCULAR */
"3639" = "1" /* OTH HEART REVASCULAR */
"3691" = "1" /* CORON VESS ANEURYSM REP */
"3699" = "1" /* HEART VESSEL OP NEC */
"3710" = "1" /* INCISION OF HEART NOS */
"3711" = "1" /* CARDIOTOMY */
"3712" = "1" /* PERICARDIOTOMY */
"3724" = "1" /* PERICARDIAL BIOPSY */
"3731" = "1" /* PERICARDIECTOMY */
"3732" = "1" /* HEART ANEURYSM EXCISION */
"3733" = "1" /* EXC/DEST HRT LESION OPEN */
"3734" = "1" /* EXC/DEST HRT LES OTHER */
"3735" = "1" /* PARTIAL VENTRICULECTOMY */
"374 " = "1" /* HEART & PERICARD REPAIR */
"3741" = "1" /* IMPL CARDIAC SUPPORT DEV */
"3749" = "1" /* HEART/PERICARD REPR NEC */
"375 " = "1" /* HEART & PERICARD REPAIR */
"3751" = "1" /* HEART TRANSPLANTATION */
"3752" = "1" /* IMPLANT TOT REP HRT SYS */
"3753" = "1" /* REPL/REP THORAC UNIT HRT */
"3754" = "1" /* REPL/REP OTH TOT HRT SYS */
"3761" = "1" /* PULSATION BALLOON IMPLAN */
"3762" = "1" /* IMPLANT HRT ASST SYS NEC */
"3763" = "1" /* REPLACE HRT ASSIST SYST */
"3764" = "1" /* REMOVE HEART ASSIST SYS */
"3765" = "1" /* IMP EXT PUL HRT ASST SYS */
"3766" = "1" /* IMP IMP PUL HRT ASST SYS */
"3767" = "1" /* IMP CARDIOMYOSTIMUL SYS */
"3774" = "1" /* INT OR REPL LEAD EPICAR */
"3775" = "1" /* REVISION OF LEAD */
"3776" = "1" /* REPL TV ATRI-VENT LEAD */
"3777" = "1" /* REMOVAL OF LEAD W/O REPL */
"3779" = "1" /* REVIS OR RELOCATE POCKET */
"3780" = "1" /* INT OR REPL PERM PACEMKR */
"3785" = "1" /* REPL PACEM W 1-CHAM, NON */
"3786" = "1" /* REPL PACEM 1-CHAM, RATE */
"3787" = "1" /* REPL PACEM W DUAL-CHAM */
"3789" = "1" /* REVISE OR REMOVE PACEMAK */
"3790" = "1" /* INS LEFT ATR APPEND DEV */
"3791" = "1" /* OPN CHEST CARDIAC MASSAG */
"3794" = "1" /* IMPLT/REPL CARDDEFIB TOT */
"3795" = "1" /* IMPLT CARDIODEFIB LEADS */
"3796" = "1" /* IMPLT CARDIODEFIB GENATR */
"3797" = "1" /* REPL CARDIODEFIB LEADS */
"3798" = "1" /* REPL CARDIODEFIB GENRATR */
"3799" = "1" /* OTHER HEART/PERICARD OPS */
"3800" = "1" /* INCISION OF VESSEL NOS */
"3801" = "1" /* INTRACRAN VESSEL INCIS */
"3802" = "1" /* HEAD/NECK VES INCIS NEC */
"3803" = "1" /* UPPER LIMB VESSEL INCIS */
"3804" = "1" /* INCISION OF AORTA */
"3805" = "1" /* THORACIC VESSEL INC NEC */
"3806" = "1" /* ABDOMEN ARTERY INCISION */
"3807" = "1" /* ABDOMINAL VEIN INCISION */
"3808" = "1" /* LOWER LIMB ARTERY INCIS */
"3809" = "1" /* LOWER LIMB VEIN INCISION */
"3810" = "1" /* ENDARTERECTOMY NOS */
"3811" = "1" /* INTRACRAN ENDARTERECTOMY */
"3812" = "1" /* HEAD & NECK ENDARTER NEC */
"3813" = "1" /* UPPER LIMB ENDARTERECTOM */
"3814" = "1" /* ENDARTERECTOMY OF AORTA */
"3815" = "1" /* THORACIC ENDARTERECTOMY */
"3816" = "1" /* ABDOMINAL ENDARTERECTOMY */
"3818" = "1" /* LOWER LIMB ENDARTERECT */
"3821" = "1" /* BLOOD VESSEL BIOPSY */
"3829" = "1" /* BLOOD VESSEL DX PROC NEC */
"3830" = "1" /* VESSEL RESECT/ANAST NOS */
"3831" = "1" /* INTRACRAN VES RESEC-ANAS */
"3832" = "1" /* HEAD/NECK VES RESEC-ANAS */
"3833" = "1" /* ARM VESSEL RESECT/ANAST */
"3834" = "1" /* AORTA RESECTION & ANAST */
"3835" = "1" /* THOR VESSEL RESECT/ANAST */
"3836" = "1" /* ABD VESSEL RESECT/ANAST */
"3837" = "1" /* ABD VEIN RESECT & ANAST */
"3838" = "1" /* LEG ARTERY RESECT/ANAST */
"3839" = "1" /* LEG VEIN RESECT/ANASTOM */
"3840" = "1" /* VESSEL RESECT/REPLAC NOS */
"3841" = "1" /* INTRACRAN VES RESEC-REPL */
"3842" = "1" /* HEAD/NECK VES RESEC-REPL */
"3843" = "1" /* ARM VES RESECT W REPLACE */
"3844" = "1" /* RESECT ABDM AORTA W REPL */
"3845" = "1" /* RESECT THORAC VES W REPL */
"3846" = "1" /* ABD ARTERY RESEC W REPLA */
"3847" = "1" /* ABD VEIN RESECT W REPLAC */
"3848" = "1" /* LEG ARTERY RESEC W REPLA */
"3849" = "1" /* LEG VEIN RESECT W REPLAC */
"3850" = "1" /* VARICOSE V LIG-STRIP NOS */
"3851" = "1" /* INTCRAN VAR V LIG-STRIP */
"3852" = "1" /* HEAD/NECK VAR V LIG-STR */
"3853" = "1" /* ARM VARICOSE V LIG-STRIP */
"3855" = "1" /* THORAC VAR V LIG-STRIP */
"3857" = "1" /* ABD VARICOS V LIGA-STRIP */
"3859" = "1" /* LEG VARICOS V LIGA-STRIP */
"3860" = "1" /* EXCISION OF VESSEL NOS */
"3861" = "1" /* INTRACRAN VESSEL EXCIS */
"3862" = "1" /* HEAD/NECK VESSEL EXCIS */
"3863" = "1" /* ARM VESSEL EXCISION */
"3864" = "1" /* EXCISION OF AORTA */
"3865" = "1" /* THORACIC VESSEL EXCISION */
"3866" = "1" /* ABDOMINAL ARTERY EXCIS */
"3867" = "1" /* ABDOMINAL VEIN EXCISION */
"3868" = "1" /* LEG ARTERY EXCISION */
"3869" = "1" /* LEG VEIN EXCISION */
"387 " = "1" /* INTERRUPTION VENA CAVA */
"3880" = "1" /* SURG VESSEL OCCLUS NEC */
"3881" = "1" /* OCCLUS INTRACRAN VES NEC */
"3882" = "1" /* OCCLUS HEAD/NECK VES NEC */
"3883" = "1" /* OCCLUDE ARM VESSEL NEC */
"3884" = "1" /* OCCLUDE AORTA NEC */
"3885" = "1" /* OCCLUDE THORACIC VES NEC */
"3886" = "1" /* OCCLUDE ABD ARTERY NEC */
"3887" = "1" /* OCCLUDE ABD VEIN NEC */
"3888" = "1" /* OCCLUDE LEG ARTERY NEC */
"3889" = "1" /* OCCLUDE LEG VEIN NEC */
"390 " = "1" /* SYSTEMIC-PULM ART SHUNT */
"391 " = "1" /* INTRA-ABD VENOUS SHUNT */
"3921" = "1" /* CAVAL-PULMON ART ANASTOM */
"3922" = "1" /* AORTA-SUBCLV-CAROT BYPAS */
"3923" = "1" /* INTRATHORACIC SHUNT NEC */
"3924" = "1" /* AORTA-RENAL BYPASS */
"3925" = "1" /* AORTA-ILIAC-FEMOR BYPASS */
"3926" = "1" /* INTRA-ABDOMIN SHUNT NEC */
"3927" = "1" /* DIALYSIS ARTERIOVENOSTOM */
"3928" = "1" /* EXTRACRAN-INTRACR BYPASS */
"3929" = "1" /* VASC SHUNT & BYPASS NEC */
"3930" = "1" /* SUTURE OF VESSEL NOS */
"3931" = "1" /* SUTURE OF ARTERY */
"3932" = "1" /* SUTURE OF VEIN */
"3941" = "1" /* POSTOP VASC OP HEM CONTR */
"3942" = "1" /* REVIS REN DIALYSIS SHUNT */
"3943" = "1" /* REMOV REN DIALYSIS SHUNT */
"3949" = "1" /* VASC PROC REVISION NEC */
"3950" = "1" /* ANGIO/ATH NON-CORO VES */
"3951" = "1" /* CLIPPING OF ANEURYSM */
"3952" = "1" /* ANEURYSM REPAIR NEC */
"3953" = "1" /* ARTERIOVEN FISTULA REP */
"3954" = "1" /* RE-ENTRY OPERATION */
"3955" = "1" /* REIMPLAN ABERR RENAL VES */
"3956" = "1" /* REPAIR VESS W TIS PATCH */
"3957" = "1" /* REP VESS W SYNTH PATCH */
"3958" = "1" /* REPAIR VESS W PATCH NOS */
"3959" = "1" /* REPAIR OF VESSEL NEC */
"397 " = "1" /* PER CARDIOPULMON BYPASS */
"3971" = "1" /* ENDO IMPL GRFT ABD AORTA */
"3972" = "1" /* ENDOVASC REPAIR HEAD VES */
"3973" = "1" /* ENDO IMP GRFT THOR AORTA */
"3974" = "1" /* ENDO REM OBS HD/NECK VES */
"3979" = "1" /* ENDO REPAIR OTHER VESSEL */
"398 " = "1" /* VASCULAR BODY OPERATIONS */
"3991" = "1" /* FREEING OF VESSEL */
"3992" = "1" /* VEIN INJECT-SCLEROS AGNT */
"3993" = "1" /* INSERT VES-TO-VES CANNUL */
"3994" = "1" /* REPLAC VES-TO-VES CANNUL */
"3998" = "1" /* HEMORRHAGE CONTROL NOS */
"3999" = "1" /* VESSEL OPERATION NEC */
"400 " = "1" /* INCIS LYMPHATIC STRUCTUR */
"4011" = "1" /* LYMPHATIC STRUCT BIOPSY */
"4019" = "1" /* LYMPHATIC DIAG PROC NEC */
"4021" = "1" /* EXCIS DEEP CERVICAL NODE */
"4022" = "1" /* EXCISE INT MAMMARY NODE */
"4023" = "1" /* EXCISE AXILLARY NODE */
"4024" = "1" /* EXCISE INGUINAL NODE */
"4029" = "1" /* SIMP EXC LYMPH STRUC NEC */
"403 " = "1" /* REGIONAL LYMPH NODE EXC */
"4040" = "1" /* RAD NECK DISSECTION NOS */
"4041" = "1" /* UNILAT RAD NECK DISSECT */
"4042" = "1" /* BILAT RAD NECK DISSECT */
"4050" = "1" /* RAD NODE DISSECTION NOS */
"4051" = "1" /* RAD DISSEC AXILLARY NODE */
"4052" = "1" /* RAD DISSEC PERIAORT NODE */
"4053" = "1" /* RAD DISSECT ILIAC NODES */
"4054" = "1" /* RADICAL GROIN DISSECTION */
"4059" = "1" /* RAD NODE DISSECTION NEC */
"4061" = "1" /* THORAC DUCT CANNULATION */
"4062" = "1" /* THORACIC DUCT FISTULIZAT */
"4063" = "1" /* CLOSE THORACIC DUCT FIST */
"4064" = "1" /* LIGATE THORACIC DUCT */
"4069" = "1" /* THORACIC DUCT OP NEC */
"409 " = "1" /* LYMPH STRUCTURE OP NEC */
"410 " = "1" /* LYMPH STRUCTURE OP NEC */
"4100" = "1" /* BONE MARROW TRNSPLNT NOS */
"4101" = "1" /* AUTO BONE MT W/O PURG */
"4102" = "1" /* ALO BONE MARROW TRNSPLNT */
"4103" = "1" /* ALLOGRFT BONE MARROW NOS */
"4104" = "1" /* AUTO HEM STEM CT W/O PUR */
"4105" = "1" /* ALLO HEM STEM CT W/O PUR */
"4106" = "1" /* CORD BLD STEM CELL TRANS */
"4107" = "1" /* AUTO HEM STEM CT W PURG */
"4108" = "1" /* ALLO HEM STEM CT W PURG */
"4109" = "1" /* AUTO BONE MT W PURGING */
"412 " = "1" /* SPLENOTOMY */
"4133" = "1" /* OPEN SPLEEN BIOPSY */
"4141" = "1" /* SPLENIC CYST MARSUPIAL */
"4142" = "1" /* EXC SPLENIC LESION/TISS */
"4143" = "1" /* PARTIAL SPLENECTOMY */
"415 " = "1" /* TOTAL SPLENECTOMY */
"4193" = "1" /* EXC OF ACCESSORY SPLEEN */
"4194" = "1" /* SPLEEN TRANSPLANTATION */
"4195" = "1" /* REPAIR OF SPLEEN */
"4199" = "1" /* SPLEEN OPERATION NEC */
"4201" = "1" /* ESOPHAGEAL WEB INCISION */
"4209" = "1" /* ESOPHAGEAL INCISION NEC */
"4210" = "1" /* ESOPHAGOSTOMY NOS */
"4211" = "1" /* CERVICAL ESOPHAGOSTOMY */
"4212" = "1" /* ESOPH POUCH EXTERIORIZAT */
"4219" = "1" /* EXT FISTULIZAT ESOPH NEC */
"4221" = "1" /* ESOPHAGOSCOPY BY INCIS */
"4225" = "1" /* OPEN BIOPSY OF ESOPHAGUS */
"4231" = "1" /* LOC EXCIS ESOPH DIVERTIC */
"4232" = "1" /* LOCAL EXCIS ESOPHAG NEC */
"4239" = "1" /* DESTRUCT ESOPHAG LES NEC */
"4240" = "1" /* ESOPHAGECTOMY NOS */
"4241" = "1" /* PARTIAL ESOPHAGECTOMY */
"4242" = "1" /* TOTAL ESOPHAGECTOMY */
"4251" = "1" /* THORAC ESOPHAGOESOPHAGOS */
"4252" = "1" /* THORAC ESOPHAGOGASTROST */
"4253" = "1" /* THORAC SM BOWEL INTERPOS */
"4254" = "1" /* THORAC ESOPHAGOENTER NEC */
"4255" = "1" /* THORAC LG BOWEL INTERPOS */
"4256" = "1" /* THORAC ESOPHAGOCOLOS NEC */
"4258" = "1" /* THORAC INTERPOSITION NEC */
"4259" = "1" /* THORAC ESOPHAG ANAST NEC */
"4261" = "1" /* STERN ESOPHAGOESOPHAGOST */
"4262" = "1" /* STERN ESOPHAGOGASTROSTOM */
"4263" = "1" /* STERN SM BOWEL INTERPOS */
"4264" = "1" /* STERN ESOPHAGOENTER NEC */
"4265" = "1" /* STERN LG BOWEL INTERPOS */
"4266" = "1" /* STERN ESOPHAGOCOLOS NEC */
"4268" = "1" /* STERN INTERPOSITION NEC */
"4269" = "1" /* STERN ESOPHAG ANAST NEC */
"427 " = "1" /* ESOPHAGOMYOTOMY */
"4282" = "1" /* SUTURE ESOPHAGEAL LACER */
"4283" = "1" /* ESOPHAGOSTOMY CLOSURE */
"4284" = "1" /* ESOPH FISTULA REPAIR NEC */
"4285" = "1" /* ESOPHAG STRICTURE REPAIR */
"4286" = "1" /* PROD SUBQ TUNNEL NO ANAS */
"4287" = "1" /* ESOPHAGEAL GRAFT NEC */
"4289" = "1" /* ESOPHAGEAL REPAIR NEC */
"4291" = "1" /* LIGATION ESOPH VARIX */
"430 " = "1" /* GASTROTOMY */
"431 " = "1" /* GASTROTOMY */
"432 " = "1" /* OTHER GASTROSTOMY */
"433 " = "1" /* PYLOROMYOTOMY */
"4342" = "1" /* LOCAL GASTR EXCISION NEC */
"4349" = "1" /* LOCAL GASTR DESTRUCT NEC */
"435 " = "1" /* PROXIMAL GASTRECTOMY */
"436 " = "1" /* DISTAL GASTRECTOMY */
"437 " = "1" /* PART GASTREC W JEJ ANAST */
"4381" = "1" /* PART GAST W JEJ TRANSPOS */
"4389" = "1" /* PARTIAL GASTRECTOMY NEC */
"4391" = "1" /* TOT GAST W INTES INTERPO */
"4399" = "1" /* TOTAL GASTRECTOMY NEC */
"4400" = "1" /* VAGOTOMY NOS */
"4401" = "1" /* TRUNCAL VAGOTOMY */
"4402" = "1" /* HIGHLY SELECT VAGOTOMY */
"4403" = "1" /* SELECTIVE VAGOTOMY NEC */
"4411" = "1" /* TRANSABDOMIN GASTROSCOPY */
"4415" = "1" /* OPEN GASTRIC BIOPSY */
"442 " = "1" /* GASTRIC DIAGNOS PROC NEC */
"4421" = "1" /* DILATE PYLORUS, INCISION */
"4429" = "1" /* OTHER PYLOROPLASTY */
"4431" = "1" /* HIGH GASTRIC BYPASS */
"4432" = "1" /* PERCU GASTROJEJUNOSTOMY */
"4438" = "1" /* LAP GASTROENTEROSTOMY */
"4439" = "1" /* GASTROENTEROSTOMY NEC */
"4440" = "1" /* SUTURE PEPTIC ULCER NOS */
"4441" = "1" /* SUT GASTRIC ULCER SITE */
"4442" = "1" /* SUTURE DUODEN ULCER SITE */
"445 " = "1" /* REVISION GASTRIC ANASTOM */
"4461" = "1" /* SUTURE GASTRIC LACERAT */
"4463" = "1" /* CLOSE GASTRIC FISTUL NEC */
"4464" = "1" /* GASTROPEXY */
"4465" = "1" /* ESOPHAGOGASTROPLASTY */
"4466" = "1" /* CREAT ESOPHAGASTR SPHINC */
"4467" = "1" /* LAP CREAT ESOPH SPHINCT */
"4468" = "1" /* LAPAROSCOP GASTROPLASTY */
"4469" = "1" /* GASTRIC REPAIR NEC */
"4491" = "1" /* LIGATE GASTRIC VARICES */
"4492" = "1" /* INTRAOP GASTRIC MANIPUL */
"4495" = "1" /* LAP GASTRIC RESTRIC PROC */
"4496" = "1" /* LAP REV GAST RESTRI PROC */
"4497" = "1" /* LAP REM GAST RESTRIC DEV */
"4498" = "1" /* ADJUST GAST RESTRICT DEV */
"4499" = "1" /* GASTRIC OPERATION NEC */
"4500" = "1" /* INTESTINAL INCISION NOS */
"4501" = "1" /* DUODENAL INCISION */
"4502" = "1" /* SMALL BOWEL INCISION NEC */
"4503" = "1" /* LARGE BOWEL INCISION */
"4511" = "1" /* TRANSAB SM BOWEL ENDOSC */
"4515" = "1" /* OPEN SMALL BOWEL BIOPSY */
"4521" = "1" /* TRANSAB LG BOWEL ENDOSC */
"4526" = "1" /* OPEN LARGE BOWEL BIOPSY */
"4531" = "1" /* OTH EXCISE DUODENUM LES */
"4532" = "1" /* DESTRUCT DUODEN LES NEC */
"4533" = "1" /* LOCAL EXCIS SM BOWEL NEC */
"4534" = "1" /* DESTR SM BOWEL LES NEC */
"4541" = "1" /* EXCISE LG INTESTINE LES */
"4549" = "1" /* DESTRUC LG BOWEL LES NEC */
"4550" = "1" /* INTEST SEG ISOLAT NOS */
"4551" = "1" /* SM BOWEL SEGMENT ISOLAT */
"4552" = "1" /* LG BOWEL SEGMENT ISOLAT */
"4561" = "1" /* MULT SEG SM BOWEL EXCIS */
"4562" = "1" /* PART SM BOWEL RESECT NEC */
"4563" = "1" /* TOTAL REMOVAL SM BOWEL */
"4571" = "1" /* MULT SEG LG BOWEL EXCIS */
"4572" = "1" /* CECECTOMY */
"4573" = "1" /* RIGHT HEMICOLECTOMY */
"4574" = "1" /* TRANSVERSE COLON RESECT */
"4575" = "1" /* LEFT HEMICOLECTOMY */
"4576" = "1" /* SIGMOIDECTOMY */
"4579" = "1" /* PART LG BOWEL EXCIS NEC */
"458 " = "1" /* TOT INTRA-ABD COLECTOMY */
"4590" = "1" /* INTESTINAL ANASTOM NOS */
"4591" = "1" /* SM-TO-SM BOWEL ANASTOM */
"4592" = "1" /* SM BOWEL-RECT STUMP ANAS */
"4593" = "1" /* SMALL-TO-LARGE BOWEL NEC */
"4594" = "1" /* LG-TO-LG BOWEL ANASTOM */
"4595" = "1" /* ANAL ANASTOMOSIS */
"4601" = "1" /* SM BOWEL EXTERIORIZATION */
"4602" = "1" /* RESECT EXT SEG SM BOWEL */
"4603" = "1" /* LG BOWEL EXTERIORIZATION */
"4604" = "1" /* RESECT EXT SEG LG BOWEL */
"4610" = "1" /* COLOSTOMY NOS */
"4611" = "1" /* TEMPORARY COLOSTOMY */
"4612" = "1" /* TEMPORARY COLOSTOMY */
"4613" = "1" /* PERMANENT COLOSTOMY */
"4620" = "1" /* ILEOSTOMY NOS */
"4621" = "1" /* TEMPORARY ILEOSTOMY */
"4622" = "1" /* CONTINENT ILEOSTOMY */
"4623" = "1" /* PERMANENT ILEOSTOMY NEC */
"4640" = "1" /* INTEST STOMA REVIS NOS */
"4641" = "1" /* SM BOWEL STOMA REVISION */
"4642" = "1" /* PERICOLOST HERNIA REPAIR */
"4643" = "1" /* LG BOWEL STOMA REVIS NEC */
"4650" = "1" /* INTEST STOMA CLOSURE NOS */
"4651" = "1" /* SM BOWEL STOMA CLOSURE */
"4652" = "1" /* LG BOWEL STOMA CLOSURE */
"4660" = "1" /* INTESTINAL FIXATION NOS */
"4661" = "1" /* SM BOWEL-ABD WALL FIXAT */
"4662" = "1" /* SMALL BOWEL FIXATION NEC */
"4663" = "1" /* LG BOWEL-ABD WALL FIXAT */
"4664" = "1" /* LARGE BOWEL FIXATION NEC */
"4671" = "1" /* DUODENAL LACERAT SUTURE */
"4672" = "1" /* DUODENAL FISTULA CLOSURE */
"4673" = "1" /* SMALL BOWEL SUTURE NEC */
"4674" = "1" /* CLOSE SM BOWEL FIST NEC */
"4675" = "1" /* SUTURE LG BOWEL LACERAT */
"4676" = "1" /* CLOSE LG BOWEL FISTULA */
"4679" = "1" /* REPAIR OF INTESTINE NEC */
"4680" = "1" /* INTRA-AB BOWEL MANIP NOS */
"4681" = "1" /* INTRA-ABD SM BOWEL MANIP */
"4682" = "1" /* INTRA-ABD LG BOWEL MANIP */
"4691" = "1" /* MYOTOMY OF SIGMOID COLON */
"4692" = "1" /* MYOTOMY OF COLON NEC */
"4693" = "1" /* REVISE SM BOWEL ANASTOM */
"4694" = "1" /* REVISE LG BOWEL ANASTOM */
"4697" = "1" /* TRANSPLANT OF INTESTINE */
"4699" = "1" /* INTESTINAL OP NEC */
"470 " = "1" /* INTESTINAL OP NEC */
"4701" = "1" /* LAP APPENDECTOMY */
"4709" = "1" /* OTHER APPENDECTOMY */
"471 " = "1" /* OTHER APPENDECTOMY */
"4711" = "1" /* LAP INCID APPENDECTOMY */
"4719" = "1" /* OTHER INCID APPENDECTOMY */
"472 " = "1" /* DRAIN APPENDICEAL ABSC */
"4791" = "1" /* APPENDICOSTOMY */
"4792" = "1" /* CLOSE APPENDICEAL FISTUL */
"4799" = "1" /* APPENDICEAL OPS NEC */
"480 " = "1" /* PROCTOTOMY */
"481 " = "1" /* PROCTOSTOMY */
"4821" = "1" /* TRANSAB PROCTOSIGMOIDOSC */
"4825" = "1" /* OPEN RECTAL BIOPSY */
"4835" = "1" /* LOCAL EXCIS RECTAL LES */
"4841" = "1" /* SOAVE SUBMUC RECT RESECT */
"4849" = "1" /* PULL-THRU RECT RESEC NEC */
"485 " = "1" /* ABD-PERINEAL RECT RESECT */
"4861" = "1" /* TRANSSAC RECTOSIGMOIDECT */
"4862" = "1" /* ANT RECT RESECT W COLOST */
"4863" = "1" /* ANTERIOR RECT RESECT NEC */
"4864" = "1" /* POSTERIOR RECT RESECTION */
"4865" = "1" /* DUHAMEL RECTAL RESECTION */
"4866" = "1" /* DUHAMEL RECTAL RESECTION */
"4869" = "1" /* RECTAL RESECTION NEC */
"4871" = "1" /* SUTURE OF RECTAL LACER */
"4872" = "1" /* CLOSURE OF PROCTOSTOMY */
"4873" = "1" /* CLOSE RECTAL FIST NEC */
"4874" = "1" /* RECTORECTOSTOMY */
"4875" = "1" /* ABDOMINAL PROCTOPEXY */
"4876" = "1" /* PROCTOPEXY NEC */
"4879" = "1" /* REPAIR OF RECTUM NEC */
"4881" = "1" /* PERIRECTAL INCISION */
"4882" = "1" /* PERIRECTAL EXCISION */
"4891" = "1" /* INCIS RECTAL STRICTURE */
"4892" = "1" /* ANORECTAL MYOMECTOMY */
"4893" = "1" /* REPAIR PERIRECT FISTULA */
"4899" = "1" /* RECTAL PERIRECT OP NEC */
"4901" = "1" /* INCIS PERIANAL ABSCESS */
"4902" = "1" /* PERIANAL INCISION NEC */
"4904" = "1" /* PERIANAL EXCISION NEC */
"4911" = "1" /* ANAL FISTULOTOMY */
"4912" = "1" /* ANAL FISTULECTOMY */
"493 " = "1" /* ANAL/PERIAN DX PROC NEC */
"4939" = "1" /* OTHER DESTRUC ANUS LES */
"4944" = "1" /* HEMORRHOID CRYOTHERAPY */
"4945" = "1" /* HEMORRHOID LIGATION */
"4946" = "1" /* HEMORRHOIDECTOMY */
"4949" = "1" /* HEMORRHOID PROCEDURE NEC */
"4951" = "1" /* LEFT LAT SPHINCTEROTOMY */
"4952" = "1" /* POST SPHINCTEROTOMY */
"4959" = "1" /* ANAL SPHINCTEROTOMY NEC */
"496 " = "1" /* EXCISION OF ANUS */
"4971" = "1" /* SUTURE ANAL LACERATION */
"4972" = "1" /* ANAL CERCLAGE */
"4973" = "1" /* CLOSURE OF ANAL FISTULA */
"4974" = "1" /* GRACILIS MUSC TRANSPLAN */
"4975" = "1" /* IMPL OR REV ART ANAL SPH */
"4976" = "1" /* REMOV ART ANAL SPHINCTER */
"4979" = "1" /* ANAL SPHINCT REPAIR NEC */
"4991" = "1" /* INCISION OF ANAL SEPTUM */
"4992" = "1" /* INSERT SUBQ ANAL STIMUL */
"4993" = "1" /* ANAL INCISION NEC */
"4994" = "1" /* REDUCTION ANAL PROLAPSE */
"4995" = "1" /* CONTROL ANAL HEMORRHAGE */
"4999" = "1" /* ANAL OPERATION NEC */
"500 " = "1" /* HEPATOTOMY */
"5012" = "1" /* OPEN LIVER BIOPSY */
"5019" = "1" /* HEPATIC DX PROC NEC */
"5021" = "1" /* MARSUPIALIZAT LIVER LES */
"5022" = "1" /* PARTIAL HEPATECTOMY */
"5023" = "1" /* OPN ABLTN LIVER LES/TISS */
"5024" = "1" /* PERC ABLTN LIVER LES/TIS */
"5025" = "1" /* LAP ABLTN LIVER LES/TISS */
"5026" = "1" /* ABLTN LIVER LES/TISS NEC */
"5029" = "1" /* DESTRUC HEPATIC LES NEC */
"503 " = "1" /* HEPATIC LOBECTOMY */
"504 " = "1" /* TOTAL HEPATECTOMY */
"5051" = "1" /* AUXILIARY LIVER TRANSPL */
"5059" = "1" /* LIVER TRANSPLANT NEC */
"5061" = "1" /* CLOSURE LIVER LACERAT */
"5069" = "1" /* LIVER REPAIR NEC */
"5102" = "1" /* TROCAR CHOLECYSTOSTOMY */
"5103" = "1" /* CHOLECYSTOSTOMY NEC */
"5104" = "1" /* CHOLECYSTOTOMY NEC */
"5113" = "1" /* OPEN BILIARY TRACT BX */
"5119" = "1" /* BILIARY TR DX PROC NEC */
"5121" = "1" /* OTH PART CHOLECYSTECTOMY */
"5122" = "1" /* CHOLECYSTECTOMY */
"5123" = "1" /* LAPAROSCOPIC CHOLECYSTEC */
"5124" = "1" /* LAP PART CHOLECYSTECTOMY */
"5131" = "1" /* GB-TO-HEPAT DUCT ANAST */
"5132" = "1" /* GB-TO-INTESTINE ANASTOM */
"5133" = "1" /* GB-TO-PANCREAS ANASTOM */
"5134" = "1" /* GB-TO-STOMACH ANASTOMOS */
"5135" = "1" /* GALLBLADDER ANASTOM NEC */
"5136" = "1" /* CHOLEDOCHOENTEROSTOMY */
"5137" = "1" /* HEPATIC DUCT-GI ANASTOM */
"5139" = "1" /* BILE DUCT ANASTOMOS NEC */
"5141" = "1" /* CDE FOR CALCULUS REMOV */
"5142" = "1" /* CDE FOR OBSTRUCTION NEC */
"5143" = "1" /* CHOLEDOCHOHEPAT INTUBAT */
"5149" = "1" /* INCIS OBSTR BILE DUC NEC */
"5151" = "1" /* COMMON DUCT EXPLORATION */
"5159" = "1" /* BILE DUCT INCISION NEC */
"5161" = "1" /* EXCIS CYST DUCT REMNANT */
"5162" = "1" /* EXCIS AMPULLA OF VATER */
"5163" = "1" /* COMMON DUCT EXCIS NEC */
"5169" = "1" /* BILE DUCT EXCISION NEC */
"5171" = "1" /* SIMPLE SUT-COMMON DUCT */
"5172" = "1" /* CHOLEDOCHOPLASTY */
"5179" = "1" /* BILE DUCT REPAIR NEC */
"5181" = "1" /* SPHINCTER OF ODDI DILAT */
"5182" = "1" /* PANCREAT SPHINCTEROTOM */
"5183" = "1" /* PANCREAT SPHINCTEROPLAS */
"5189" = "1" /* SPHINCT OF ODDI OP NEC */
"5191" = "1" /* REPAIR GB LACERATION */
"5192" = "1" /* CLOSURE CHOLECYSTOSTOMY */
"5193" = "1" /* CLOS BILIARY FISTUL NEC */
"5194" = "1" /* REVIS BILE TRACT ANASTOM */
"5195" = "1" /* REMOVE BILE DUCT PROSTH */
"5199" = "1" /* BILIARY TRACT OP NEC */
"5201" = "1" /* CATH DRAIN-PANCREAT CYST */
"5209" = "1" /* PANCREATOTOMY NEC */
"5212" = "1" /* OPEN PANCREATIC BIOPSY */
"5219" = "1" /* PANCREATIC DX PROC NEC */
"522 " = "1" /* PANCREATIC DX PROC NEC */
"5222" = "1" /* OTHER DESTRU PANCREA LES */
"523 " = "1" /* PANCREAT CYST MARSUPIALI */
"524 " = "1" /* INT DRAIN PANCREAT CYST */
"5251" = "1" /* PROXIMAL PANCREATECTOMY */
"5252" = "1" /* DISTAL PANCREATECTOMY */
"5253" = "1" /* RAD SUBTOT PANCREATECTOM */
"5259" = "1" /* PARTIAL PANCREATECT NEC */
"526 " = "1" /* TOTAL PANCREATECTOMY */
"527 " = "1" /* RAD PANCREATICODUODENECT */
"5280" = "1" /* PANCREAT TRANSPLANT NOS */
"5281" = "1" /* REIMPLANT PANCREATIC TIS */
"5282" = "1" /* PANCREATIC HOMOTRANSPLAN */
"5283" = "1" /* PANCREATIC HETEROTRANSPL */
"5291" = "1" /* TRNSPLNT ISLETS LANG NOS */
"5292" = "1" /* CANNULATION PANCREA DUC */
"5295" = "1" /* PANCREATIC REPAIR NEC */
"5296" = "1" /* PANCREATIC ANASTOMOSIS */
"5299" = "1" /* PANCREATIC OPERATION NEC */
"5300" = "1" /* UNILAT ING HERN REP NOS */
"5301" = "1" /* REPAIR DIRECT ING HERNIA */
"5302" = "1" /* REPAIR INDIR ING HERNIA */
"5303" = "1" /* DIR ING HERNIA REP-GRAFT */
"5304" = "1" /* IND ING HERNIA REP-GRAFT */
"5305" = "1" /* ING HERNIA REP-GRAFT NOS */
"5310" = "1" /* BILAT ING HERNIA REP NOS */
"5311" = "1" /* BILAT DIR ING HERN REP */
"5312" = "1" /* BILAT IND ING HERN REP */
"5313" = "1" /* BIL DIR/IND ING HRN REP */
"5314" = "1" /* BIL DIR ING HRN REP-GRFT */
"5315" = "1" /* BIL IND ING HRN REP-GRFT */
"5316" = "1" /* BIL DIR/IND ING HERN-PRO */
"5317" = "1" /* BIL ING HRN REP-GRFT NOS */
"5321" = "1" /* UNIL FEMOR HRN REP-GRFT */
"5329" = "1" /* UNIL FEMOR HERN REP NEC */
"5331" = "1" /* BIL FEM HERN REPAIR-GRFT */
"5339" = "1" /* BIL FEM HERN REPAIR NEC */
"5341" = "1" /* UMBIL HERNIA REPAIR-GRFT */
"5349" = "1" /* UMBIL HERNIA REPAIR NEC */
"5351" = "1" /* INCISIONAL HERNIA REPAIR */
"5359" = "1" /* ABD WALL HERN REPAIR NEC */
"5361" = "1" /* INCIS HERNIA REPAIR-GRFT */
"5369" = "1" /* ABD HERN REPAIR-GRFT NEC */
"537 " = "1" /* ABD REPAIR-DIAPHR HERNIA */
"5380" = "1" /* THOR REP-DIAPH HERN NOS */
"5381" = "1" /* DIAPHRAGMATIC PLICATION */
"5382" = "1" /* PARASTERN HERNIA REPAIR */
"539 " = "1" /* OTHER HERNIA REPAIR */
"540 " = "1" /* ABDOMINAL WALL INCISION */
"5411" = "1" /* EXPLORATORY LAPAROTOMY */
"5412" = "1" /* REOPEN RECENT LAP SITE */
"5419" = "1" /* LAPAROTOMY NEC */
"5421" = "1" /* LAPAROSCOPY */
"5422" = "1" /* ABDOMINAL WALL BIOPSY */
"5423" = "1" /* PERITONEAL BIOPSY */
"5429" = "1" /* ABD REGION DX PROC NEC */
"543 " = "1" /* DESTRUCT ABD WALL LESION */
"544 " = "1" /* DESTRUCT PERITONEAL TISS */
"545 " = "1" /* DESTRUCT PERITONEAL TISS */
"5451" = "1" /* LAP PERITON ADHESIOLYSIS */
"5459" = "1" /* OTH PERITON ADHESIOLYSIS */
"5461" = "1" /* RECLOSE POST OP DISRUPT */
"5462" = "1" /* DELAYED CLOS ABD WOUND */
"5463" = "1" /* ABD WALL SUTURE NEC */
"5464" = "1" /* PERITONEAL SUTURE */
"5471" = "1" /* REPAIR OF GASTROSCHISIS */
"5472" = "1" /* ABDOMEN WALL REPAIR NEC */
"5473" = "1" /* PERITONEAL REPAIR NEC */
"5474" = "1" /* OMENTAL REPAIR NEC */
"5475" = "1" /* MESENTERIC REPAIR NEC */
"5492" = "1" /* REMOVE FB FROM PERITON */
"5493" = "1" /* CREATE CUTANPERITON FIST */
"5494" = "1" /* CREAT PERITONEOVAS SHUNT */
"5495" = "1" /* PERITONEAL INCISION */
"5501" = "1" /* NEPHROTOMY */
"5502" = "1" /* NEPHROSTOMY */
"5503" = "1" /* PERCU NEPHROSTM W/O FRAG */
"5504" = "1" /* PERCU NEPHROSTMY W FRAG */
"5511" = "1" /* PYELOTOMY */
"5512" = "1" /* PYELOSTOMY */
"5524" = "1" /* OPEN RENAL BIOPSY */
"5529" = "1" /* RENAL DIAGNOST PROC NEC */
"5531" = "1" /* RENAL LES MARSUPIALIZAT */
"5532" = "1" /* OPN ABLTN RENAL LES/TISS */
"5533" = "1" /* PERC ABLTN RENL LES/TISS */
"5534" = "1" /* LAP ABLTN RENAL LES/TISS */
"5535" = "1" /* ABLTN RENAL LES/TISS NEC */
"5539" = "1" /* LOC DESTR RENAL LES NEC */
"554 " = "1" /* PARTIAL NEPHRECTOMY */
"5551" = "1" /* NEPHROURETERECTOMY */
"5552" = "1" /* SOLITARY KIDNEY NEPHRECT */
"5553" = "1" /* REJECTED KIDNEY NEPHRECT */
"5554" = "1" /* BILATERAL NEPHRECTOMY */
"5561" = "1" /* RENAL AUTOTRANSPLANT */
"5569" = "1" /* KIDNEY TRANSPLANT NEC */
"557 " = "1" /* NEPHROPEXY */
"5581" = "1" /* SUTURE KIDNEY LACERATION */
"5582" = "1" /* CLOSE NEPHROST & PYELOST */
"5583" = "1" /* CLOSE RENAL FISTULA NEC */
"5584" = "1" /* REDUCE RENAL PEDICL TORS */
"5585" = "1" /* SYMPHYSIOTOMY */
"5586" = "1" /* RENAL ANASTOMOSIS */
"5587" = "1" /* CORRECT URETEROPELV JUNC */
"5589" = "1" /* RENAL REPAIR NEC */
"5591" = "1" /* RENAL DECAPSULATION */
"5597" = "1" /* IMPLANT MECHANIC KIDNEY */
"5598" = "1" /* REMOV MECHANICAL KIDNEY */
"5599" = "1" /* RENAL OPERATION NEC */
"560 " = "1" /* TU REMOV URETER OBSTRUCT */
"561 " = "1" /* URETERAL MEATOTOMY */
"562 " = "1" /* URETEROTOMY */
"5634" = "1" /* OPEN URETERAL BIOPSY */
"5639" = "1" /* URETERAL DX PROCEDUR NEC */
"5640" = "1" /* URETERECTOMY NOS */
"5641" = "1" /* PARTIAL URETERECTOMY */
"5642" = "1" /* TOTAL URETERECTOMY */
"5651" = "1" /* FORM CUTAN ILEOURETEROST */
"5652" = "1" /* REVIS CUTAN ILEOURETEROS */
"5661" = "1" /* FORM CUTAN URETEROSTOMY */
"5662" = "1" /* REVIS CUTAN URETEROS NEC */
"5671" = "1" /* URIN DIVERSION TO BOWEL */
"5672" = "1" /* REVIS URETEROENTEROSTOMY */
"5673" = "1" /* NEPHROCYSTANASTOMOSI NOS */
"5674" = "1" /* URETERONEOCYSTOSTOMY */
"5675" = "1" /* TRANSURETEROURETEROSTOMY */
"5679" = "1" /* URETERAL ANASTOMOSIS NEC */
"5681" = "1" /* INTRALUM URETE ADHESIOLY */
"5682" = "1" /* SUTURE URETERAL LACERAT */
"5683" = "1" /* URETEROSTOMY CLOSURE */
"5684" = "1" /* CLOSE URETER FISTULA NEC */
"5685" = "1" /* URETEROPEXY */
"5686" = "1" /* REMOVE URETERAL LIGATURE */
"5689" = "1" /* REPAIR OF URETER NEC */
"5692" = "1" /* IMPLANT URETERAL STIMUL */
"5693" = "1" /* REPLACE URETERAL STIMUL */
"5694" = "1" /* REMOVE URETERAL STIMULAT */
"5695" = "1" /* LIGATION OF URETER */
"5699" = "1" /* URETERAL OPERATION NEC */
"5712" = "1" /* CYSTOTOMY & ADHESIOLYSIS */
"5718" = "1" /* OTHER SUPRAPU CYSTOSTOMY */
"5719" = "1" /* CYSTOTOMY NEC */
"5721" = "1" /* VESICOSTOMY */
"5722" = "1" /* REVISE CLO VESICOSTOMY */
"5733" = "1" /* CLOS TRANSURETH BLADD BX */
"5734" = "1" /* OPEN BLADDER BIOPSY */
"5739" = "1" /* BLADDER DIAGNOS PROC NEC */
"5741" = "1" /* TU ADHESIOLYSIS BLADDER */
"5749" = "1" /* TU DESTRUC BLADD LES NEC */
"5751" = "1" /* EXCISION OF URACHUS */
"5759" = "1" /* BLADDER LES DESTRUCT NEC */
"576 " = "1" /* PARTIAL CYSTECTOMY */
"5771" = "1" /* RADICAL CYSTECTOMY */
"5779" = "1" /* TOTAL CYSTECTOMY NEC */
"5781" = "1" /* SUTURE BLADDER LACERAT */
"5782" = "1" /* CYSTOSTOMY CLOSURE */
"5783" = "1" /* ENTEROVESICO FIST REPAIR */
"5784" = "1" /* VESIC FISTULA REPAIR NEC */
"5785" = "1" /* CYSTOURETHROPLASTY */
"5786" = "1" /* BLADDER EXSTROPHY REPAIR */
"5787" = "1" /* BLADDER RECONSTRUCTION */
"5788" = "1" /* BLADDER ANASTOMOSIS NEC */
"5789" = "1" /* BLADDER REPAIR NEC */
"5791" = "1" /* BLADDER SPHINCTEROTOMY */
"5793" = "1" /* CONTROL BLADD HEMORRHAGE */
"5796" = "1" /* IMPLANT BLADDER STIMULAT */
"5797" = "1" /* REPLACE BLADDER STIMULAT */
"5798" = "1" /* REMOVE BLADDER STIMULAT */
"5799" = "1" /* BLADDER OPERATION NEC */
"580 " = "1" /* URETHROTOMY */
"581 " = "1" /* URETHRAL MEATOTOMY */
"5841" = "1" /* SUTURE URETHRAL LACERAT */
"5842" = "1" /* URETHROSTOMY CLOSURE */
"5843" = "1" /* CLOSE URETH FISTULA NEC */
"5844" = "1" /* URETHRAL REANASTOMOSIS */
"5845" = "1" /* HYPO-EPISPADIUS REPAIR */
"5846" = "1" /* URETH RECONSTRUCTION NEC */
"5847" = "1" /* URETHRAL MEATOPLASTY */
"5849" = "1" /* URETHRAL REPAIR NEC */
"585 " = "1" /* URETH STRICTURE RELEASE */
"5891" = "1" /* PERIURETHRAL INCISION */
"5892" = "1" /* PERIURETHRAL EXCISION */
"5893" = "1" /* IMPLT ARTF URIN SPHINCT */
"5899" = "1" /* URETH/PERIURETH OP NEC */
"5900" = "1" /* RETROPERIT DISSECT NOS */
"5901" = "1" /* RETROPERIT DISSECT NOS */
"5902" = "1" /* PERIREN ADHESIOLYS NEC */
"5903" = "1" /* LAP LYS PERIREN/URET ADH */
"5909" = "1" /* PERIREN/URETER INCIS NEC */
"5911" = "1" /* OTH LYS PERIVES ADHESIO */
"5912" = "1" /* LAP LYS PERIVESURETH ADH */
"5919" = "1" /* PERIVESICAL INCISION NEC */
"5921" = "1" /* PERIREN/URETERAL BIOPSY */
"5929" = "1" /* PERIREN/URET DX PROC NEC */
"593 " = "1" /* URETHROVES JUNCT PLICAT */
"594 " = "1" /* SUPRAPUBIC SLING OP */
"595 " = "1" /* RETROPUBIC URETH SUSPENS */
"596 " = "1" /* PARAURETHRAL SUSPENSION */
"5971" = "1" /* LEVATOR MUSC SUSPENSION */
"5979" = "1" /* URIN INCONTIN REPAIR NEC */
"5991" = "1" /* PERIREN/VESICLE EXCISION */
"5992" = "1" /* PERIREN/VESICLE OP NEC */
"600 " = "1" /* INCISION OF PROSTATE */
"6012" = "1" /* OPEN PROSTATIC BIOPSY */
"6014" = "1" /* OPEN SEMINAL VESICLES BX */
"6015" = "1" /* PERIPROSTATIC BIOPSY */
"6018" = "1" /* PROSTATIC DX PROCED NEC */
"6019" = "1" /* SEMIN VES DX PROCED NEC */
"602 " = "1" /* SEMIN VES DX PROCED NEC */
"6021" = "1" /* TRANSURETH PROSTATECTOMY */
"6029" = "1" /* OTH TRANSURETH PROSTATEC */
"603 " = "1" /* SUPRAPUBIC PROSTATECTOMY */
"604 " = "1" /* RETROPUBIC PROSTATECTOMY */
"605 " = "1" /* RADICAL PROSTATECTOMY */
"6061" = "1" /* LOS EXCIS PROSTATIC LES */
"6062" = "1" /* PERINEAL PROSTATECTOMY */
"6069" = "1" /* PROSTATECTOMY NEC */
"6072" = "1" /* SEMINAL VESICLE INCISION */
"6073" = "1" /* SEMINAL VESICLE EXCISION */
"6079" = "1" /* SEMINAL VESICLE OP NEC */
"6081" = "1" /* PERIPROSTATIC INCISION */
"6082" = "1" /* PERIPROSTATIC EXCISION */
"6093" = "1" /* REPAIR OF PROSTATE */
"6094" = "1" /* CONTROL PROSTATE HEMORR */
"6095" = "1" /* TRANS BAL DIL PROS URETH */
"6096" = "1" /* TU DESTR PROSTATE BY MT */
"6097" = "1" /* OTH TU DESTR PROS - RT */
"6099" = "1" /* PROSTATIC OPERATION NEC */
"612 " = "1" /* EXCISION OF HYDROCELE */
"6142" = "1" /* SCROTAL FISTULA REPAIR */
"6149" = "1" /* SCROTUM/TUNIC REPAIR NEC */
"6192" = "1" /* EXCISION TUNICA LES NEC */
"6199" = "1" /* SCROTUM & TUNICA OP NEC */
"620 " = "1" /* INCISION OF TESTES */
"6212" = "1" /* OPEN TESTICULAR BIOPSY */
"6219" = "1" /* TESTES DX PROCEDURE NEC */
"622 " = "1" /* TESTICULAR LES DESTRUCT */
"623 " = "1" /* UNILATERAL ORCHIECTOMY */
"6241" = "1" /* REMOVE BOTH TESTES */
"6242" = "1" /* REMOVE SOLITARY TESTIS */
"625 " = "1" /* ORCHIOPEXY */
"6261" = "1" /* SUTURE TESTICULAR LACER */
"6269" = "1" /* TESTICULAR REPAIR NEC */
"627 " = "1" /* INSERT TESTICULAR PROSTH */
"6299" = "1" /* TESTICULAR OPERATION NEC */
"6309" = "1" /* SPERMAT CORD/VAS DX NEC */
"631 " = "1" /* EXC SPERMATIC VARICOCELE */
"632 " = "1" /* EXCISE EPIDIDYMIS CYST */
"633 " = "1" /* EXCISE CORD/EPID LES NEC */
"634 " = "1" /* EPIDIDYMECTOMY */
"6351" = "1" /* SUTURE CORD & EPID LACER */
"6353" = "1" /* TRANSPLANT SPERMAT CORD */
"6359" = "1" /* CORD & EPIDID REPAIR NEC */
"6381" = "1" /* SUTURE VAS & EPIDID LAC */
"6382" = "1" /* POSTOP VAS RECONSTRUCT */
"6383" = "1" /* EPIDIDYMOVASOSTOMY */
"6385" = "1" /* REMOV VAS DEFERENS VALVE */
"6389" = "1" /* VAS & EPIDIDY REPAIR NEC */
"6392" = "1" /* EPIDIDYMOTOMY */
"6393" = "1" /* SPERMATIC CORD INCISION */
"6394" = "1" /* SPERM CORD ADHESIOLYSIS */
"6395" = "1" /* INSERT VALVE IN VAS DEF */
"6399" = "1" /* CORD/EPID/VAS OPS NEC */
"640 " = "1" /* CIRCUMCISION */
"6411" = "1" /* PENILE BIOPSY */
"642 " = "1" /* LOCAL EXCIS PENILE LES */
"643 " = "1" /* AMPUTATION OF PENIS */
"6441" = "1" /* SUTURE PENILE LACERATION */
"6442" = "1" /* RELEASE OF CHORDEE */
"6443" = "1" /* CONSTRUCTION OF PENIS */
"6444" = "1" /* RECONSTRUCTION OF PENIS */
"6445" = "1" /* REPLANTATION OF PENIS */
"6449" = "1" /* PENILE REPAIR NEC */
"645 " = "1" /* SEX TRANSFORMAT OP NEC */
"6492" = "1" /* INCISION OF PENIS */
"6493" = "1" /* DIVISION OF PENILE ADHES */
"6495" = "1" /* INS NONINFL PENIS PROSTH */
"6496" = "1" /* REMOVE INT PENILE PROSTH */
"6497" = "1" /* INS INFLATE PENIS PROSTH */
"6498" = "1" /* PENILE OPERATION NEC */
"6499" = "1" /* MALE GENITAL OP NEC */
"650 " = "1" /* MALE GENITAL OP NEC */
"6501" = "1" /* LAPAROSCOPIC OOPHOROTOMY */
"6509" = "1" /* OTHER OOPHOROTOMY */
"6511" = "1" /* OVARIAN ASPIRAT BIOPSY */
"6512" = "1" /* OVARIAN BIOPSY NEC */
"6513" = "1" /* LAP BIOPSY OF OVARY */
"6514" = "1" /* OTH LAP DX PROC OVARIES */
"6519" = "1" /* OVARIAN DX PROCEDURE NEC */
"6521" = "1" /* OVARIAN CYST MARSUPIALIZ */
"6522" = "1" /* OVARIAN WEDGE RESECTION */
"6523" = "1" /* LAP MARSUP OVARIAN CYST */
"6524" = "1" /* LAP WEDGE RESECT OVARY */
"6525" = "1" /* OTH LAP LOC EXC DEST OVA */
"6529" = "1" /* LOCAL DESTR OVA LES NEC */
"653 " = "1" /* LOCAL DESTR OVA LES NEC */
"6531" = "1" /* LAP UNILAT OOPHORECTOMY */
"6539" = "1" /* OTH UNILAT OOPHORECTOMY */
"654 " = "1" /* OTH UNILAT OOPHORECTOMY */
"6541" = "1" /* LAP UNI SALPINGO-OOPHOR */
"6549" = "1" /* OTH UNI SALPINGO-OOPHOR */
"6551" = "1" /* OTH REMOVE BOTH OVARIES */
"6552" = "1" /* OTH REMOVE REMAIN OVARY */
"6553" = "1" /* LAP REMOVE BOTH OVARIES */
"6554" = "1" /* LAP REMOVE REMAIN OVARY */
"6561" = "1" /* OTH REMOVE OVARIES/TUBES */
"6562" = "1" /* OTH REMOVE REM OVA/TUBE */
"6563" = "1" /* LAP REMOVE OVARIES/TUBES */
"6564" = "1" /* LAP REMOVE REM OVA/TUBE */
"6571" = "1" /* OTH SIMPLE SUTURE OVARY */
"6572" = "1" /* OTH REIMPLANT OF OVARY */
"6573" = "1" /* OTH SALPINGO-OOPHOROPLAS */
"6574" = "1" /* LAP SIMPLE SUTURE OVARY */
"6575" = "1" /* LAP REIMPLANT OF OVARY */
"6576" = "1" /* LAP SALPINGO-OOPHOROPLAS */
"6579" = "1" /* REPAIR OF OVARY NEC */
"658 " = "1" /* REPAIR OF OVARY NEC */
"6581" = "1" /* LAP ADHESIOLYS OVA/TUBE */
"6589" = "1" /* ADHESIOLYSIS OVARY/TUBE */
"6591" = "1" /* ASPIRATION OF OVARY */
"6592" = "1" /* TRANSPLANTATION OF OVARY */
"6593" = "1" /* MANUAL RUPT OVARIAN CYST */
"6594" = "1" /* OVARIAN DENERVATION */
"6595" = "1" /* OVARIAN TORSION RELEASE */
"6599" = "1" /* OVARIAN OPERATION NEC */
"660 " = "1" /* OVARIAN OPERATION NEC */
"6601" = "1" /* SALPINGOTOMY */
"6602" = "1" /* SALPINGOSTOMY */
"6611" = "1" /* FALLOPIAN TUBE BIOPSY */
"6619" = "1" /* FALLOP TUBE DX PROC NEC */
"6621" = "1" /* BILAT ENDOSC CRUSH TUBE */
"6622" = "1" /* BILAT ENDOSC DIVIS TUBE */
"6629" = "1" /* BILAT ENDOS OCC TUBE NEC */
"6631" = "1" /* BILAT TUBAL CRUSHING NEC */
"6632" = "1" /* BILAT TUBAL DIVISION NEC */
"6639" = "1" /* BILAT TUBAL DESTRUCT NEC */
"664 " = "1" /* TOTAL UNILAT SALPINGECT */
"6651" = "1" /* REMOVE BOTH FALLOP TUBES */
"6652" = "1" /* REMOVE SOLITARY FAL TUBE */
"6661" = "1" /* DESTROY FALLOP TUBE LES */
"6662" = "1" /* REMOV TUBE & ECTOP PREG */
"6663" = "1" /* BILAT PART SALPINGEC NOS */
"6669" = "1" /* PARTIAL SALPINGECTOM NEC */
"6671" = "1" /* SIMPL SUTURE FALLOP TUBE */
"6672" = "1" /* SALPINGO-OOPHOROSTOMY */
"6673" = "1" /* SALPINGO-SALPINGOSTOMY */
"6674" = "1" /* SALPINGO-UTEROSTOMY */
"6679" = "1" /* FALLOP TUBE REPAIR NEC */
"6692" = "1" /* UNILAT FALLOP TUBE DESTR */
"6693" = "1" /* IMPL FALLOP TUBE PROSTH */
"6694" = "1" /* REMOV FALLOP TUBE PROSTH */
"6695" = "1" /* BLOW THERAPEUT INTO TUBE */
"6696" = "1" /* FALLOPIAN TUBE DILATION */
"6697" = "1" /* BURY FIMBRIAE IN UTERUS */
"6699" = "1" /* FALLOPIAN TUBE OP NEC */
"6711" = "1" /* ENDOCERVICAL BIOPSY */
"6712" = "1" /* CERVICAL BIOPSY NEC */
"6719" = "1" /* CERVICAL DX PROCEDUR NEC */
"672 " = "1" /* CONIZATION OF CERVIX */
"6731" = "1" /* CERVICAL CYST MARSUPIAL */
"6732" = "1" /* CERVICAL LES CAUTERIZAT */
"6733" = "1" /* CERVICAL LES CRYOTHERAPY */
"6739" = "1" /* CERVICAL LES DESTRUC NEC */
"674 " = "1" /* AMPUTATION OF CERVIX */
"675 " = "1" /* AMPUTATION OF CERVIX */
"6751" = "1" /* TRANSAB CERCLAGE CERVIX */
"6759" = "1" /* OTH REP INT CERVICAL OS */
"6761" = "1" /* SUTURE CERVICAL LACERAT */
"6762" = "1" /* CERVICAL FISTULA REPAIR */
"6769" = "1" /* CERVICAL REPAIR NEC */
"680 " = "1" /* HYSTEROTOMY */
"6813" = "1" /* OPEN UTERINE BIOPSY */
"6814" = "1" /* OPEN UTERINE LIGAMENT BX */
"6815" = "1" /* CLOS UTERINE LIGAMENT BX */
"6816" = "1" /* CLOSED UTERINE BIOPSY */
"6819" = "1" /* UTERUS/ADNEX DX PROC NEC */
"6821" = "1" /* ENDOMET SYNECHIAE DIVIS */
"6822" = "1" /* INCISION UTERINE SEPTUM */
"6823" = "1" /* ENDOMETRIAL ABLATION */
"6829" = "1" /* UTERINE LES DESTRUCT NEC */
"683 " = "1" /* UTERINE LES DESTRUCT NEC */
"6831" = "1" /* LAP SCERVIC HYSTERECTOMY */
"6839" = "1" /* OTH SUBTOT ABD HYSTERECT */
"684 " = "1" /* TOTAL ABD HYSTERECTOMY */
"6841" = "1" /* LAP TOTAL ABDOMINAL HYST */
"6849" = "1" /* TOTAL ABD HYST NEC/NOS */
"685 " = "1" /* TOTAL ABD HYSTERECTOMY */
"6851" = "1" /* LAP AST VAG HYSTERECTOMY */
"6859" = "1" /* OTHER VAG HYSTERECTOMY */
"686 " = "1" /* RADICAL ABD HYSTERECTOMY */
"6861" = "1" /* LAP RADICAL ABDOMNL HYST */
"6869" = "1" /* RADICAL ABD HYST NEC/NOS */
"687 " = "1" /* RADICAL VAG HYSTERECTOMY */
"6871" = "1" /* LAP RADICAL VAGINAL HYST */
"6879" = "1" /* RADICAL VAG HYST NEC/NOS */
"688 " = "1" /* PELVIC EVISCERATION */
"689 " = "1" /* HYSTERECTOMY NEC/NOS */
"6901" = "1" /* D & C FOR PREG TERMINAT */
"6902" = "1" /* D & C POST DELIVERY */
"6909" = "1" /* D & C NEC */
"6911" = "1" /* D & C NEC */
"6919" = "1" /* DESTRUC UTER SUPPORT NEC */
"6921" = "1" /* INTERPOSIT OP UTERIN LIG */
"6922" = "1" /* UTERINE SUSPENSION NEC */
"6923" = "1" /* VAG REPAIR INVERS UTERUS */
"6929" = "1" /* UTERUS/ADNEXA REPAIR NEC */
"693 " = "1" /* PARACERV UTERINE DENERV */
"6941" = "1" /* SUTURE UTERINE LACERAT */
"6942" = "1" /* CLOSURE UTERINE FISTULA */
"6949" = "1" /* UTERINE REPAIR NEC */
"6951" = "1" /* ASPIRAT CURET-PREG TERMI */
"6952" = "1" /* ASPIRAT CURET-POST DELIV */
"6995" = "1" /* INCISION OF CERVIX */
"6997" = "1" /* REMOVE PENETRAT CERV FB */
"6998" = "1" /* UTERINE SUPPORT OP NEC */
"6999" = "1" /* UTERINE OPERATION NEC */
"7012" = "1" /* CULDOTOMY */
"7013" = "1" /* INTRALUM VAG ADHESIOLYS */
"7014" = "1" /* VAGINOTOMY NEC */
"7023" = "1" /* CUL-DE-SAC BIOPSY */
"7024" = "1" /* VAGINAL BIOPSY */
"7029" = "1" /* VAGIN/CUL-DE-SAC DX NEC */
"7031" = "1" /* HYMENECTOMY */
"7032" = "1" /* EXCIS CUL-DE-SAC LESION */
"7033" = "1" /* EXCISION VAGINAL LESION */
"704 " = "1" /* VAGINAL OBLITERATION */
"7050" = "1" /* CYSTOCEL/RECTOCEL REPAIR */
"7051" = "1" /* CYSTOCELE REPAIR */
"7052" = "1" /* RECTOCELE REPAIR */
"7061" = "1" /* VAGINAL CONSTRUCTION */
"7062" = "1" /* VAGINAL RECONSTRUCTION */
"7071" = "1" /* SUTURE VAGINA LACERATION */
"7072" = "1" /* REPAIR COLOVAGIN FISTULA */
"7073" = "1" /* REPAIR RECTOVAG FISTULA */
"7074" = "1" /* REP VAGINOENT FISTUL NEC */
"7075" = "1" /* REPAIR VAG FISTULA NEC */
"7076" = "1" /* HYMENORRHAPHY */
"7077" = "1" /* VAGINAL SUSPENS & FIXAT */
"7079" = "1" /* VAGINAL REPAIR NEC */
"708 " = "1" /* VAGINAL VAULT OBLITERAT */
"7091" = "1" /* VAGINAL OPERATION NEC */
"7092" = "1" /* CUL-DE-SAC OPERATION NEC */
"7101" = "1" /* VULVAR ADHESIOLYSIS */
"7109" = "1" /* INCIS VULVA/PERINEUM NEC */
"7111" = "1" /* VULVAR BIOPSY */
"7119" = "1" /* VULVAR DIAGNOS PROC NEC */
"7122" = "1" /* INCISE BARTHOLIN"S GLAND */
"7123" = "1" /* BARTHOLIN GLAND MARSUP */
"7124" = "1" /* DESTRUC BARTHOLIN GLAND */
"7129" = "1" /* BARTHOLIN"S GLAND OP NEC */
"713 " = "1" /* LOCAL VULVAR EXCIS NEC */
"714 " = "1" /* OPERATIONS ON CLITORIS */
"715 " = "1" /* RADICAL VULVECTOMY */
"7161" = "1" /* UNILATERAL VULVECTOMY */
"7162" = "1" /* BILATERAL VULVECTOMY */
"7171" = "1" /* SUTURE VULVAR LACERATION */
"7172" = "1" /* REPAIR VULVAR FISTULA */
"7179" = "1" /* VULVAR/PERIN REPAIR NEC */
"718 " = "1" /* OTHER VULVAR OPERATIONS */
"719 " = "1" /* OTHER FEMALE GENITAL OPS */
"7394" = "1" /* PUBIOTOMY TO ASSIST DEL */
"7399" = "1" /* OPS ASSISTING DELIV NEC */
"740 " = "1" /* CLASSICAL C-SECTION */
"741 " = "1" /* LOW CERVICAL C-SECTION */
"742 " = "1" /* EXTRAPERITONEAL C-SECT */
"743 " = "1" /* REM EXTRATUB ECTOP PREG */
"744 " = "1" /* CESAREAN SECTION NEC */
"7491" = "1" /* HYSTEROTOMY TO TERMIN PG */
"7499" = "1" /* CESAREAN SECTION NOS */
"7536" = "1" /* CORRECTION FETAL DEFECT */
"7550" = "1" /* REPAIR OB LAC UTERUS NOS */
"7551" = "1" /* REPAIR OB LACERAT CERVIX */
"7552" = "1" /* REPAIR OB LAC CORP UTERI */
"7561" = "1" /* REPAIR OB LAC BLAD/URETH */
"7593" = "1" /* SURG CORR INVERT UTERUS */
"7599" = "1" /* OBSTETRIC OPERATION NEC */
"7601" = "1" /* FACIAL BONE SEQUESTRECT */
"7609" = "1" /* FACIAL BONE INCISION NEC */
"7611" = "1" /* FACIAL BONE BIOPSY */
"7619" = "1" /* FACIAL BONE DX PROC NEC */
"762 " = "1" /* DESTRUCT FACIAL BONE LES */
"7631" = "1" /* PARTIAL MANDIBULECTOMY */
"7639" = "1" /* PART FACIAL OSTECTOM NEC */
"7641" = "1" /* TOT MANDIBULEC W RECONST */
"7642" = "1" /* TOTAL MANDIBULECTOMY NEC */
"7643" = "1" /* MANDIBULAR RECONST NEC */
"7644" = "1" /* TOT FACE OSTECT W RECONS */
"7645" = "1" /* TOT FACE BONE OSTECT NEC */
"7646" = "1" /* FACIAL BONE RECONSTR NEC */
"765 " = "1" /* TEMPOROMAND ARTHROPLASTY */
"7661" = "1" /* CL OSTEOPLASTY MAND RAMI */
"7662" = "1" /* OPEN OSTEOPLAS MAND RAMI */
"7663" = "1" /* OSTEOPLASTY MANDIBLE BDY */
"7664" = "1" /* MAND ORTHOGNATHIC OP NEC */
"7665" = "1" /* SEG OSTEOPLASTY MAXILLA */
"7666" = "1" /* TOT OSTEOPLASTY MAXILLA */
"7667" = "1" /* REDUCTION GENIOPLASTY */
"7668" = "1" /* AUGMENTATION GENIOPLASTY */
"7669" = "1" /* FACIAL BONE REPAIR NEC */
"7670" = "1" /* REDUCTION FACIAL FX NOS */
"7672" = "1" /* OPN REDUCT MALAR/ZYGO FX */
"7674" = "1" /* OPEN REDUCT MAXILLARY FX */
"7676" = "1" /* OPEN REDUCT MANDIBLE FX */
"7677" = "1" /* OPEN REDUCT ALVEOLAR FX */
"7679" = "1" /* OPEN REDUCT FACE FX NEC */
"7691" = "1" /* BONE GRAFT TO FACE BONE */
"7692" = "1" /* SYN IMPLANT TO FACE BONE */
"7694" = "1" /* OPEN REDUCT TM DISLOCAT */
"7697" = "1" /* REMOVE INT FIX FACE BONE */
"7699" = "1" /* FACIAL BONE/JNT OP NEC */
"7700" = "1" /* SEQUESTRECTOMY NOS */
"7701" = "1" /* CHEST CAGE SEQUESTREC */
"7702" = "1" /* HUMERUS SEQUESTRECTOMY */
"7703" = "1" /* RADIUS & ULNA SEQUESTREC */
"7704" = "1" /* METACARP/CARP SEQUESTREC */
"7705" = "1" /* FEMORAL SEQUESTRECTOMY */
"7706" = "1" /* PATELLAR SEQUESTRECTOMY */
"7707" = "1" /* TIBIA/FIBULA SEQUESTREC */
"7708" = "1" /* METATAR/TAR SEQUESTREC */
"7709" = "1" /* SEQUESTRECTOMY NEC */
"7710" = "1" /* OTHER BONE INCISION NOS */
"7711" = "1" /* OTHER CHEST CAGE INCIS */
"7712" = "1" /* OTHER HUMERUS INCISION */
"7713" = "1" /* OTHER RADIUS/ULNA INCIS */
"7714" = "1" /* OTH METACARP/CARP INCIS */
"7715" = "1" /* OTHER FEMORAL INCISION */
"7716" = "1" /* OTHER PATELLAR INCISION */
"7717" = "1" /* OTHER TIBIA/FIBULA INCIS */
"7718" = "1" /* OTH METATARS/TARS INCIS */
"7719" = "1" /* BONE INCIS W/O DIV NEC */
"7720" = "1" /* WEDGE OSTEOTOMY NOS */
"7721" = "1" /* CHEST CAGE WEDG OSTEOTOM */
"7722" = "1" /* HUMERUS WEDGE OSTEOTOMY */
"7723" = "1" /* RADIUS/ULNA WEDG OSTEOTO */
"7724" = "1" /* METACAR/CAR WEDG OSTEOTO */
"7725" = "1" /* FEMORAL WEDGE OSTEOTOMY */
"7726" = "1" /* PATELLAR WEDGE OSTEOTOMY */
"7727" = "1" /* TIBIA/FIBUL WEDG OSTEOT */
"7728" = "1" /* METATAR/TAR WEDG OSTEOT */
"7729" = "1" /* WEDGE OSTEOTOMY NEC */
"7730" = "1" /* OTHER BONE DIVISION NOS */
"7731" = "1" /* CHEST CAGE BONE DIV NEC */
"7732" = "1" /* HUMERUS DIVISION NEC */
"7733" = "1" /* RADIUS/ULNA DIVISION NEC */
"7734" = "1" /* METACAR/CAR DIVISION NEC */
"7735" = "1" /* FEMORAL DIVISION NEC */
"7736" = "1" /* PATELLAR DIVISION NEC */
"7737" = "1" /* TIBIA/FIBULA DIV NEC */
"7738" = "1" /* METATAR/TAR DIVISION NEC */
"7739" = "1" /* BONE DIVISION NEC */
"7740" = "1" /* BONE BIOPSY NOS */
"7741" = "1" /* CHEST CAGE BONE BIOPSY */
"7742" = "1" /* HUMERUS BIOPSY */
"7743" = "1" /* RADIUS & ULNA BIOPSY */
"7744" = "1" /* METACARPAL/CARPAL BIOPSY */
"7745" = "1" /* FEMORAL BIOPSY */
"7746" = "1" /* PATELLAR BIOPSY */
"7747" = "1" /* TIBIA & FIBULA BIOPSY */
"7748" = "1" /* METATARSAL/TARSAL BIOPSY */
"7749" = "1" /* BONE BIOPSY NEC */
"7751" = "1" /* BUNIONECT/SFT/OSTEOTOMY */
"7752" = "1" /* BUNIONECT/SFT/ARTHRODES */
"7753" = "1" /* OTH BUNIONECT W SFT CORR */
"7754" = "1" /* EXC CORRECT BUNIONETTE */
"7756" = "1" /* REPAIR OF HAMMER TOE */
"7757" = "1" /* REPAIR OF CLAW TOE */
"7758" = "1" /* OTH EXC, FUS, REPAIR TOE */
"7759" = "1" /* BUNIONECTOMY NEC */
"7760" = "1" /* LOC EXC BONE LESION NOS */
"7761" = "1" /* EXC CHEST CAGE BONE LES */
"7762" = "1" /* LOC EXC BONE LES HUMERUS */
"7763" = "1" /* LOC EXC LES RADIUS/ULNA */
"7764" = "1" /* LOC EXC LES METACAR/CAR */
"7765" = "1" /* LOC EXC BONE LES FEMUR */
"7766" = "1" /* LOC EXC BONE LES PATELLA */
"7767" = "1" /* LOC EXC LES TIBIA/FIBULA */
"7768" = "1" /* LOC EXC LES METATAR/TAR */
"7769" = "1" /* LOC EXC BONE LESION NEC */
"7770" = "1" /* EXCISE BONE FOR GRFT NOS */
"7771" = "1" /* EX CHEST CAGE BONE-GFT */
"7772" = "1" /* EXCISE HUMERUS FOR GRAFT */
"7773" = "1" /* EXCIS RADIUS/ULNA-GRAFT */
"7774" = "1" /* EXCIS METACAR/CAR-GRAFT */
"7775" = "1" /* EXCISE FEMUR FOR GRAFT */
"7776" = "1" /* EXCISE PATELLA FOR GRAFT */
"7777" = "1" /* EXCISE TIB/FIB FOR GRAFT */
"7778" = "1" /* EXCIS METATAR/TAR-GRAFT */
"7779" = "1" /* EXCISE BONE FOR GFT NEC */
"7780" = "1" /* OTH PART OSTECTOMY NOS */
"7781" = "1" /* OTH CHEST CAGE OSTECTOMY */
"7782" = "1" /* PARTIAL HUMERECTOMY NEC */
"7783" = "1" /* PART OSTECT-RADIUS/ULNA */
"7784" = "1" /* PART OSTECT-METACAR/CAR */
"7785" = "1" /* PART OSTECTOMY-FEMUR */
"7786" = "1" /* PARTIAL PATELLECTOMY */
"7787" = "1" /* PART OSTECT-TIBIA/FIBULA */
"7788" = "1" /* PART OSTECT-METATAR/TAR */
"7789" = "1" /* PARTIAL OSTECTOMY NEC */
"7790" = "1" /* TOTAL OSTECTOMY NOS */
"7791" = "1" /* TOT CHEST CAGE OSTECTOMY */
"7792" = "1" /* TOTAL OSTECTOMY-HUMERUS */
"7793" = "1" /* TOT OSTECT-RADIUS/ULNA */
"7794" = "1" /* TOT OSTECT-METACARP/CARP */
"7795" = "1" /* TOT OSTECTOMY-FEMUR */
"7796" = "1" /* TOTAL PATELLECTOMY */
"7797" = "1" /* TOT OSTECT-TIBIA/FIBULA */
"7798" = "1" /* TOT OSTECT-METATARS/TARS */
"7799" = "1" /* TOTAL OSTECTOMY NEC */
"7800" = "1" /* BONE GRAFT NOS */
"7801" = "1" /* BONE GRAFT TO CHEST CAGE */
"7802" = "1" /* BONE GRAFT TO HUMERUS */
"7803" = "1" /* BONE GRAFT-RADIUS/ULNA */
"7804" = "1" /* BONE GRFT TO METACAR/CAR */
"7805" = "1" /* BONE GRAFT TO FEMUR */
"7806" = "1" /* BONE GRAFT TO PATELLA */
"7807" = "1" /* BONE GRAFT-TIBIA/FIBULA */
"7808" = "1" /* BONE GRAFT-METATAR/TAR */
"7809" = "1" /* BONE GRAFT NEC */
"7810" = "1" /* APPLIC EXT FIX DEV NOS */
"7811" = "1" /* APPL EXT FIX-CHEST CAGE */
"7812" = "1" /* APPLIC EXT FIX-HUMERUS */
"7813" = "1" /* APPL EXT FIX-RADIUS/ULNA */
"7814" = "1" /* APPL EXT FIX-METACAR/CAR */
"7815" = "1" /* APPLIC EXT FIX DEV-FEMUR */
"7816" = "1" /* APPL EXT FIX DEV-PATELLA */
"7817" = "1" /* APPL EXT FIX-TIB/FIBULA */
"7818" = "1" /* APPL EXT FIX-METATAR/TAR */
"7819" = "1" /* APPLIC EXT FIX DEV NEC */
"7820" = "1" /* LIMB SHORTEN PROC NOS */
"7822" = "1" /* LIMB SHORT PROC-HUMERUS */
"7823" = "1" /* LIMB SHORTEN-RADIUS/ULNA */
"7824" = "1" /* LIMB SHORTEN-METACAR/CAR */
"7825" = "1" /* LIMB SHORT PROC-FEMUR */
"7827" = "1" /* LIMB SHORTEN-TIB/FIBULA */
"7828" = "1" /* LIMB SHORTEN-METATAR/TAR */
"7829" = "1" /* LIMB SHORTEN PROC NEC */
"7830" = "1" /* LIMB LENGTHEN PROC NOS */
"7831" = "1" /* LIMB LENGTHEN PROC NOS */
"7832" = "1" /* LIMB LENGTH PROC-HUMERUS */
"7833" = "1" /* LIMB LENGTH-RADIUS/ULNA */
"7834" = "1" /* LIMB LENGTH-METACAR/CAR */
"7835" = "1" /* LIMB LENGTH PROC-FEMUR */
"7837" = "1" /* LIMB LENGTHEN-TIB/FIBULA */
"7838" = "1" /* LIMB LENGTHN-METATAR/TAR */
"7839" = "1" /* LIMB LENGTHEN PROC NEC */
"7840" = "1" /* OTH BONE REPAIR/PLAST OP */
"7841" = "1" /* OTH CHEST CAGE REP/PLAST */
"7842" = "1" /* OTH HUMERUS REPAIR/PLAST */
"7843" = "1" /* OTH RAD/ULN REPAIR/PLAST */
"7844" = "1" /* OTH METAC/CARP REP/PLAST */
"7845" = "1" /* OTH FEMUR REPAIR/PLASTIC */
"7846" = "1" /* OTH PATELLA REPAIR/PLAST */
"7847" = "1" /* OTH TIB/FIB REPAIR/PLAST */
"7848" = "1" /* OTH META/TAR REPA/PLAST */
"7849" = "1" /* OTH BONE REPA/PLAST NEC */
"7850" = "1" /* INT FIX W/O FX REDUC NOS */
"7851" = "1" /* INT FIXATION-CHEST CAGE */
"7852" = "1" /* INT FIXATION-HUMERUS */
"7853" = "1" /* INT FIXATION-RADIUS/ULNA */
"7854" = "1" /* INT FIXATION-METACAR/CAR */
"7855" = "1" /* INTERNAL FIXATION-FEMUR */
"7856" = "1" /* INTERNAL FIX-PATELLA */
"7857" = "1" /* INT FIXATION-TIBIA/FIBUL */
"7858" = "1" /* INT FIXATION-METATAR/TAR */
"7859" = "1" /* INT FIX-NO FX REDUCT NEC */
"7860" = "1" /* REMOVE IMP DEVICE NOS */
"7861" = "1" /* REMOV IMP DEV-CHEST CAGE */
"7862" = "1" /* REMOVE IMPL DEV-HUMERUS */
"7863" = "1" /* REMOV IMP DEV-RADIUS/ULN */
"7864" = "1" /* REMOV IMP DEV-METAC/CARP */
"7865" = "1" /* REMOVE IMP DEVICE-FEMUR */
"7866" = "1" /* REMOV IMP DEVICE-PATELLA */
"7867" = "1" /* REMOV IMP DEV-TIB/FIBULA */
"7868" = "1" /* REMOVE IMP DEV-METAT/TAR */
"7869" = "1" /* REMOVE IMPL DEVICE NEC */
"7870" = "1" /* OSTEOCLASIS NOS */
"7871" = "1" /* OSTEOCLASIS-CHEST CAGE */
"7872" = "1" /* OSTEOCLASIS-HUMERUS */
"7873" = "1" /* OSTEOCLASIS-RADIUS/ULNA */
"7874" = "1" /* OSTEOCLASIS-METACAR/CAR */
"7875" = "1" /* OSTEOCLASIS-FEMUR */
"7876" = "1" /* OSTEOCLASIS-PATELLA */
"7877" = "1" /* OSTEOCLASIS-TIBIA/FIBULA */
"7878" = "1" /* OSTEOCLASIS-METATAR/TAR */
"7879" = "1" /* OSTEOCLASIS NEC */
"7880" = "1" /* OTHER BONE DX PROC NOS */
"7881" = "1" /* OTH DX PROCED-CHEST CAGE */
"7882" = "1" /* OTH DX PROCED-HUMERUS */
"7883" = "1" /* OTH DX PROC-RADIUS/ULNA */
"7884" = "1" /* OTH DX PROC-METACAR/CAR */
"7885" = "1" /* OTH DX PROCED-FEMUR */
"7886" = "1" /* OTH DX PROCED-PATELLA */
"7887" = "1" /* OTH DX PROC-TIBIA/FIBULA */
"7888" = "1" /* OTH DX PROC-METATAR/TAR */
"7889" = "1" /* OTHER BONE DX PROC NEC */
"7890" = "1" /* INSERT BONE STIMUL NOS */
"7891" = "1" /* INSERT BONE STIMUL-CHEST */
"7892" = "1" /* INSERT BONE STIM-HUMERUS */
"7893" = "1" /* INSER BONE STIM-RAD/ULNA */
"7894" = "1" /* INSER BONE STIM-META/CAR */
"7895" = "1" /* INSERT BONE STIM-FEMUR */
"7896" = "1" /* INSERT BONE STIM-PATELLA */
"7897" = "1" /* INSER BONE STIM-TIB/FIB */
"7898" = "1" /* INSER BONE STIM-META/TAR */
"7899" = "1" /* INSERT BONE STIMUL NEC */
"7910" = "1" /* CL FX REDUC-INT FIX NOS */
"7911" = "1" /* CLOS RED-INT FIX HUMERUS */
"7912" = "1" /* CL RED-INT FIX RAD/ULNA */
"7913" = "1" /* CL RED-INT FIX METAC/CAR */
"7914" = "1" /* CLOSE RED-INT FIX FINGER */
"7915" = "1" /* CLOSED RED-INT FIX FEMUR */
"7916" = "1" /* CL RED-INT FIX TIB/FIBU */
"7917" = "1" /* CL RED-INT FIX METAT/TAR */
"7918" = "1" /* CLOSE RED-INT FIX TOE FX */
"7919" = "1" /* CL FX REDUC-INT FIX NEC */
"7920" = "1" /* OPEN FX REDUCTION NOS */
"7921" = "1" /* OPEN REDUC-HUMERUS FX */
"7922" = "1" /* OPEN REDUC-RADIUS/ULN FX */
"7923" = "1" /* OPEN REDUC-METAC/CAR FX */
"7924" = "1" /* OPEN REDUCTION-FINGER FX */
"7925" = "1" /* OPEN REDUCTION-FEMUR FX */
"7926" = "1" /* OPEN REDUC-TIBIA/FIB FX */
"7927" = "1" /* OPEN REDUC-METAT/TARS FX */
"7928" = "1" /* OPEN REDUCTION-TOE FX */
"7929" = "1" /* OPEN FX REDUCTION NEC */
"7930" = "1" /* OPN FX RED W INT FIX NOS */
"7931" = "1" /* OPEN RED-INT FIX HUMERUS */
"7932" = "1" /* OP RED-INT FIX RAD/ULNA */
"7933" = "1" /* OP RED-INT FIX METAC/CAR */
"7934" = "1" /* OPEN RED-INT FIX FINGER */
"7935" = "1" /* OPEN REDUC-INT FIX FEMUR */
"7936" = "1" /* OP RED-INT FIX TIB/FIBUL */
"7937" = "1" /* OP RED-INT FIX METAT/TAR */
"7938" = "1" /* OPEN REDUCT-INT FIX TOE */
"7939" = "1" /* OPN FX RED W INT FIX NEC */
"7940" = "1" /* CLS REDUC-SEP EPIPHY NOS */
"7941" = "1" /* CLOSE RED-HUMERUS EPIPHY */
"7942" = "1" /* CLS RED-RADIUS/UL EPIPHY */
"7945" = "1" /* CLOSE REDUC-FEMUR EPIPHY */
"7946" = "1" /* CLS RED-TIBIA/FIB EPIPHY */
"7949" = "1" /* CLS REDUC-SEP EPIPHY NEC */
"7950" = "1" /* OPEN RED-SEP EPIPHY NOS */
"7951" = "1" /* OPN RED-SEP EPIPHY-HUMER */
"7952" = "1" /* OP RED-RADIUS/ULN EPIPHY */
"7955" = "1" /* OPN RED-SEP EPIPHY-FEMUR */
"7956" = "1" /* OP RED-TIBIA/FIB EPIPHYS */
"7959" = "1" /* OPEN RED-SEP EPIPHY NEC */
"7960" = "1" /* OPEN FX SITE DEBRIDE NOS */
"7961" = "1" /* DEBRID OPEN FX-HUMERUS */
"7962" = "1" /* DEBRID OPN FX-RADIUS/ULN */
"7963" = "1" /* DEBRID OPN FX-METAC/CAR */
"7964" = "1" /* DEBRID OPN FX-FINGER */
"7965" = "1" /* DEBRID OPN FX-FEMUR */
"7966" = "1" /* DEBRID OPN FX-TIBIA/FIB */
"7967" = "1" /* DEBRID OPN FX-METAT/TAR */
"7968" = "1" /* DEBRID OPN FX-TOE */
"7969" = "1" /* OPEN FX SITE DEBRIDE NEC */
"7980" = "1" /* OPEN REDUC-DISLOCAT NOS */
"7981" = "1" /* OPN REDUC DISLOC-SHOULDR */
"7982" = "1" /* OPEN REDUC-ELBOW DISLOC */
"7983" = "1" /* OPEN REDUC-WRIST DISLOC */
"7984" = "1" /* OPN REDUC DISLOC-HAND */
"7985" = "1" /* OPEN REDUC-HIP DISLOCAT */
"7986" = "1" /* OPEN REDUC-KNEE DISLOCAT */
"7987" = "1" /* OPEN REDUC-ANKLE DISLOC */
"7988" = "1" /* OPN REDUC DISLOC-FT/TOE */
"7989" = "1" /* OPEN REDUC-DISLOCAT NEC */
"7990" = "1" /* UNSPEC OP BONE INJ NOS */
"7991" = "1" /* HUMERUS INJURY OP NOS */
"7992" = "1" /* RADIUS/ULNA INJ OP NOS */
"7993" = "1" /* METACARP/CARP INJ OP NOS */
"7994" = "1" /* FINGER INJURY OP NOS */
"7995" = "1" /* FEMUR INJURY OP NOS */
"7996" = "1" /* TIBIA/FIBULA INJ OP NOS */
"7997" = "1" /* METATARS/TARS INJ OP NOS */
"7998" = "1" /* TOE INJURY OPERATION NOS */
"7999" = "1" /* UNSPEC OP-BONE INJ NEC */
"8000" = "1" /* ARTHROT & PROS REMOV NOS */
"8001" = "1" /* ARTHROT/PROS REMOV-SHLDR */
"8002" = "1" /* ARTHROT/PROS REMOV-ELBOW */
"8003" = "1" /* ARTHROT/PROS REMOV-WRIST */
"8004" = "1" /* ARTHROT/PROS REMOV-HAND */
"8005" = "1" /* ARTHROT/PROS REMOV-HIP */
"8006" = "1" /* ARTHROT/PROS REMOV-KNEE */
"8007" = "1" /* ARTHROT/PROS REMOV-ANKLE */
"8008" = "1" /* ARTHROT/PROS REMOV-FOOT */
"8009" = "1" /* ARTHROT & PROS REMOV NEC */
"8010" = "1" /* OTHER ARTHROTOMY NOS */
"8011" = "1" /* OTH ARTHROTOMY-SHOULDER */
"8012" = "1" /* OTH ARTHROTOMY-ELBOW */
"8013" = "1" /* OTH ARTHROTOMY-WRIST */
"8014" = "1" /* OTH ARTHROTOMY-HAND/FNGR */
"8015" = "1" /* OTH ARTHROTOMY-HIP */
"8016" = "1" /* OTH ARTHROTOMY-KNEE */
"8017" = "1" /* OTH ARTHROTOMY-ANKLE */
"8018" = "1" /* OTH ARTHROTOMY-FOOT/TOE */
"8019" = "1" /* OTHER ARTHROTOMY NEC */
"8020" = "1" /* ARTHROSCOPY NOS */
"8021" = "1" /* SHOULDER ARTHROSCOPY */
"8022" = "1" /* ELBOW ARTHROSCOPY */
"8023" = "1" /* WRIST ARTHROSCOPY */
"8024" = "1" /* HAND & FINGER ARTHROSCOP */
"8025" = "1" /* HIP ARTHROSCOPY */
"8026" = "1" /* KNEE ARTHROSCOPY */
"8027" = "1" /* ANKLE ARTHROSCOPY */
"8028" = "1" /* FOOT & TOE ARTHROSCOPY */
"8029" = "1" /* ARTHROSCOPY NEC */
"8040" = "1" /* JT STRUCTUR DIVISION NOS */
"8041" = "1" /* SHOULDER STRUCT DIVISION */
"8042" = "1" /* ELBOW STRUCTURE DIVISION */
"8043" = "1" /* WRIST STRUCTURE DIVISION */
"8044" = "1" /* HAND JOINT STRUCT DIVIS */
"8045" = "1" /* HIP STRUCTURE DIVISION */
"8046" = "1" /* KNEE STRUCTURE DIVISION */
"8047" = "1" /* ANKLE STRUCTURE DIVISION */
"8048" = "1" /* FOOT JOINT STRUCT DIVIS */
"8049" = "1" /* JT STRUCTUR DIVISION NEC */
"805 " = "1" /* JT STRUCTUR DIVISION NEC */
"8050" = "1" /* EXC/DEST INTVRT DISC NOS */
"8051" = "1" /* EXCISION INTERVERT DISC */
"8059" = "1" /* OTH EXC/DEST INTVRT DISC */
"806 " = "1" /* EXCIS KNEE SEMILUN CARTL */
"8070" = "1" /* SYNOVECTOMY-SITE NOS */
"8071" = "1" /* SHOULDER SYNOVECTOMY */
"8072" = "1" /* ELBOW SYNOVECTOMY */
"8073" = "1" /* WRIST SYNOVECTOMY */
"8074" = "1" /* HAND SYNOVECTOMY */
"8075" = "1" /* HIP SYNOVECTOMY */
"8076" = "1" /* KNEE SYNOVECTOMY */
"8077" = "1" /* ANKLE SYNOVECTOMY */
"8078" = "1" /* FOOT SYNOVECTOMY */
"8079" = "1" /* SYNOVECTOMY-SITE NEC */
"8080" = "1" /* DESTRUCT JOINT LES NOS */
"8081" = "1" /* DESTRUC-SHOULDER LES NEC */
"8082" = "1" /* DESTRUC-ELBOW LESION NEC */
"8083" = "1" /* DESTRUC-WRIST LESION NEC */
"8084" = "1" /* DESTRUC-HAND JT LES NEC */
"8085" = "1" /* DESTRUCT-HIP LESION NEC */
"8086" = "1" /* DESTRUCT-KNEE LESION NEC */
"8087" = "1" /* DESTRUC-ANKLE LESION NEC */
"8088" = "1" /* DESTRUC-FOOT JT LES NEC */
"8089" = "1" /* DESTRUCT JOINT LES NEC */
"8090" = "1" /* EXCISION OF JOINT NOS */
"8091" = "1" /* EXCISION OF SHOULDER NEC */
"8092" = "1" /* EXCISION OF ELBOW NEC */
"8093" = "1" /* EXCISION OF WRIST NEC */
"8094" = "1" /* EXCISION HAND JOINT NEC */
"8095" = "1" /* EXCISION OF HIP NEC */
"8096" = "1" /* EXCISION OF KNEE NEC */
"8097" = "1" /* EXCISION OF ANKLE NEC */
"8098" = "1" /* EXCISION FOOT JOINT NEC */
"8099" = "1" /* EXCISION OF JOINT NEC */
"8100" = "1" /* SPINAL FUSION NOS */
"8101" = "1" /* ATLAS-AXIS FUSION */
"8102" = "1" /* OTHER CERVICAL FUS ANT */
"8103" = "1" /* OTHER CERVICAL FUS POST */
"8104" = "1" /* DORSAL/DORSOLUM FUS ANT */
"8105" = "1" /* DORSAL/DORSOLUM FUS POST */
"8106" = "1" /* LUMBAR/LUMBOSAC FUS ANT */
"8107" = "1" /* LUMBAR/LUMBOSAC FUS LAT */
"8108" = "1" /* LUMBAR/LUMBOSAC FUS POST */
"8109" = "1" /* LUMBAR/LUMBOSAC FUS POST */
"8111" = "1" /* ANKLE FUSION */
"8112" = "1" /* TRIPLE ARTHRODESIS */
"8113" = "1" /* SUBTALAR FUSION */
"8114" = "1" /* MIDTARSAL FUSION */
"8115" = "1" /* TARSOMETATARSAL FUSION */
"8116" = "1" /* METATARSOPHALANGEAL FUS */
"8117" = "1" /* OTHER FUSION OF FOOT */
"8118" = "1" /* OTHER FUSION OF FOOT */
"8120" = "1" /* ARTHRODESIS NOS */
"8121" = "1" /* ARTHRODESIS OF HIP */
"8122" = "1" /* ARTHRODESIS OF KNEE */
"8123" = "1" /* ARTHRODESIS OF SHOULDER */
"8124" = "1" /* ARTHRODESIS OF ELBOW */
"8125" = "1" /* CARPORADIAL FUSION */
"8126" = "1" /* METACARPOCARPAL FUSION */
"8127" = "1" /* METACARPOPHALANGEAL FUS */
"8128" = "1" /* INTERPHALANGEAL FUSION */
"8129" = "1" /* ARTHRODESIS NEC */
"8130" = "1" /* SPINAL REFUSION NOS */
"8131" = "1" /* REFUSION OF ATLAS-AXIS */
"8132" = "1" /* REFUSION OF OTH CERV ANT */
"8133" = "1" /* REFUS OF OTH CERV POST */
"8134" = "1" /* REFUSION OF DORSAL ANT */
"8135" = "1" /* REFUSION OF DORSAL POST */
"8136" = "1" /* REFUSION OF LUMBAR ANT */
"8137" = "1" /* REFUSION OF LUMBAR LAT */
"8138" = "1" /* REFUSION OF LUMBAR POST */
"8139" = "1" /* REFUSION OF SPINE NEC */
"8140" = "1" /* REPAIR OF HIP, NEC */
"8141" = "1" /* REPAIR OF HIP, NEC */
"8142" = "1" /* FIVE-IN-ONE KNEE REPAIR */
"8143" = "1" /* TRIAD KNEE REPAIR */
"8144" = "1" /* PATELLAR STABILIZATION */
"8145" = "1" /* CRUCIATE LIG REPAIR NEC */
"8146" = "1" /* COLLATERL LIG REPAIR NEC */
"8147" = "1" /* OTHER REPAIR OF KNEE */
"8148" = "1" /* OTHER REPAIR OF KNEE */
"8149" = "1" /* OTHER REPAIR OF ANKLE */
"8151" = "1" /* TOTAL HIP REPLACEMENT */
"8152" = "1" /* PARTIAL HIP REPLACEMENT */
"8153" = "1" /* REVISE HIP REPLACEMENT */
"8154" = "1" /* TOTAL KNEE REPLACEMENT */
"8155" = "1" /* REVISE KNEE REPLACEMENT */
"8156" = "1" /* TOTAL ANKLE REPLACEMENT */
"8157" = "1" /* REPL JOINT OF FOOT, TOE */
"8159" = "1" /* REV JT REPL LOW EXT NEC */
"8161" = "1" /* 360 SPINAL FUSION */
"8162" = "1" /* FUS/REFUS 2-3 VERTEBRAE */
"8163" = "1" /* FUS/REFUS 4-8 VERTEBRAE */
"8164" = "1" /* FUS/REFUS 9 VERTEBRAE */
"8165" = "1" /* VERTEBROPLASTY */
"8166" = "1" /* KYPHOPLASTY */
"8169" = "1" /* OTH HIP REPAIR  JAN80--SEP89 */
"8171" = "1" /* ARTHROPLAS METACARP WIT */
"8172" = "1" /* ARTHROPLASTY METACAR W/O */
"8173" = "1" /* TOTAL WRIST REPLACEMENT */
"8174" = "1" /* ARTHROPLASTY CARPAL WIT */
"8175" = "1" /* ARTHROPLASTY CARPAL W/O */
"8179" = "1" /* OTH REPAIR HAN/FIN/WRIS */
"8180" = "1" /* TOTAL SHOULDER REPLACE */
"8181" = "1" /* PARTIAL SHOULDER REPLACE */
"8182" = "1" /* REP RECUR SHLDER DISLOC */
"8183" = "1" /* SHOULDER ARTHROPLAST NEC */
"8184" = "1" /* TOTAL ELBOW REPLACEMENT */
"8185" = "1" /* ELBOW ARTHROPLASTY NEC */
"8186" = "1" /* ELBOW ARTHROPLASTY NEC */
"8187" = "1" /* ELBOW ARTHROPLASTY NEC */
"8193" = "1" /* SUTUR CAPSUL/LIGAMEN ARM */
"8194" = "1" /* SUTURE CAPSUL/LIG ANK/FT */
"8195" = "1" /* SUTUR CAPSUL/LIG LEG NEC */
"8196" = "1" /* OTHER REPAIR OF JOINT */
"8197" = "1" /* REV JT REPL UPPER EXTREM */
"8198" = "1" /* OTHER JOINT DX PROCEDURE */
"8199" = "1" /* JOINT STRUCTURE OP NEC */
"8201" = "1" /* EXPLOR TEND SHEATH-HAND */
"8202" = "1" /* MYOTOMY OF HAND */
"8203" = "1" /* BURSOTOMY OF HAND */
"8209" = "1" /* INC SOFT TISSUE HAND NEC */
"8211" = "1" /* TENOTOMY OF HAND */
"8212" = "1" /* FASCIOTOMY OF HAND */
"8219" = "1" /* DIV SOFT TISSUE HAND NEC */
"8221" = "1" /* EXC LES TEND SHEATH HAND */
"8222" = "1" /* EXCISION HAND MUSCLE LES */
"8229" = "1" /* EXC LES SFT TISS HND NEC */
"8231" = "1" /* BURSECTOMY OF HAND */
"8232" = "1" /* EXCIS HAND TEND FOR GRFT */
"8233" = "1" /* HAND TENONECTOMY NEC */
"8234" = "1" /* EXC HND MUS/FAS FOR GRFT */
"8235" = "1" /* HAND FASCIECTOMY NEC */
"8236" = "1" /* OTHER MYECTOMY OF HAND */
"8239" = "1" /* HAND SOFT TISSUE EXC NEC */
"8241" = "1" /* SUTURE TENDN SHEATH HAND */
"8242" = "1" /* DELAY SUT FLEX TEND HAND */
"8243" = "1" /* DELAY SUT HAND TEND NEC */
"8244" = "1" /* SUTUR FLEX TEND HAND NEC */
"8245" = "1" /* SUTURE HAND TENDON NEC */
"8246" = "1" /* SUTURE HAND MUSCLE/FASC */
"8251" = "1" /* HAND TENDON ADVANCEMENT */
"8252" = "1" /* HAND TENDON RECESSION */
"8253" = "1" /* HAND TENDON REATTACHMENT */
"8254" = "1" /* HAND MUSCLE REATTACHMENT */
"8255" = "1" /* CHNG HND MUS/TEN LNG NEC */
"8256" = "1" /* TRANSPLANT HAND TEND NEC */
"8257" = "1" /* TRANSPOSIT HAND TEND NEC */
"8258" = "1" /* TRANSPLANT HAND MUSC NEC */
"8259" = "1" /* TRANSPOSIT HAND MUSC NEC */
"8261" = "1" /* POLLICIZATION OPERATION */
"8269" = "1" /* THUMB RECONSTRUCTION NEC */
"8271" = "1" /* HAND TEND PULLEY RECONST */
"8272" = "1" /* PLAST OP HND-MUS/FAS GRF */
"8279" = "1" /* PLAST OP HAND W GRFT NEC */
"8281" = "1" /* TRANSFER OF FINGER */
"8282" = "1" /* REPAIR OF CLEFT HAND */
"8283" = "1" /* REPAIR OF MACRODACTYLY */
"8284" = "1" /* REPAIR OF MALLET FINGER */
"8285" = "1" /* OTHER TENODESIS OF HAND */
"8286" = "1" /* OTHER TENOPLASTY OF HAND */
"8289" = "1" /* HAND PLASTIC OP NEC */
"8291" = "1" /* LYSIS OF HAND ADHESIONS */
"8299" = "1" /* HAND MUS/TEN/FAS/OPS NEC */
"8301" = "1" /* TENDON SHEATH EXPLORAT */
"8302" = "1" /* MYOTOMY */
"8303" = "1" /* BURSOTOMY */
"8309" = "1" /* SOFT TISSUE INCISION NEC */
"8311" = "1" /* ACHILLOTENOTOMY */
"8312" = "1" /* ADDUCTOR TENOTOMY OF HIP */
"8313" = "1" /* OTHER TENOTOMY */
"8314" = "1" /* FASCIOTOMY */
"8319" = "1" /* SOFT TISSUE DIVISION NEC */
"8321" = "1" /* SOFT TISSUE BIOPSY */
"8329" = "1" /* SOFT TISSUE DX PROC NEC */
"8331" = "1" /* EXCIS LES TENDON SHEATH */
"8332" = "1" /* EXCIS LESION OF MUSCLE */
"8339" = "1" /* EXC LES SOFT TISSUE NEC */
"8341" = "1" /* TENDON EXCISION FOR GRFT */
"8342" = "1" /* OTHER TENONECTOMY */
"8343" = "1" /* MUSC/FASC EXCIS FOR GRFT */
"8344" = "1" /* OTHER FASCIECTOMY */
"8345" = "1" /* OTHER MYECTOMY */
"8349" = "1" /* OTHER SOFT TISSUE EXCIS */
"835 " = "1" /* BURSECTOMY */
"8361" = "1" /* TENDON SHEATH SUTURE */
"8362" = "1" /* DELAYED TENDON SUTURE */
"8363" = "1" /* ROTATOR CUFF REPAIR */
"8364" = "1" /* OTHER SUTURE OF TENDON */
"8365" = "1" /* OTHER MUSCLE/FASC SUTURE */
"8371" = "1" /* TENDON ADVANCEMENT */
"8372" = "1" /* TENDON RECESSION */
"8373" = "1" /* TENDON REATTACHMENT */
"8374" = "1" /* MUSCLE REATTACHMENT */
"8375" = "1" /* TENDON TRNSFR/TRANSPLANT */
"8376" = "1" /* OTHER TENDON TRANSPOSIT */
"8377" = "1" /* MUSCLE TRNSFR/TRANSPLANT */
"8379" = "1" /* OTHER MUSCLE TRANSPOSIT */
"8381" = "1" /* TENDON GRAFT */
"8382" = "1" /* MUSCLE OR FASCIA GRAFT */
"8383" = "1" /* TENDON PULLEY RECONSTRUC */
"8384" = "1" /* CLUBFOOT RELEASE NEC */
"8385" = "1" /* MUSC/TEND LNG CHANGE NEC */
"8386" = "1" /* QUADRICEPSPLASTY */
"8387" = "1" /* OTHER PLASTIC OPS MUSCLE */
"8388" = "1" /* OTHER PLASTIC OPS TENDON */
"8389" = "1" /* OTHER PLASTIC OPS FASCIA */
"8391" = "1" /* ADHESIOLYSIS MUS/TEN/FAS */
"8392" = "1" /* INSERT SKEL MUSC STIMULA */
"8393" = "1" /* REMOV SKEL MUSC STIMULAT */
"8399" = "1" /* MUS/TEN/FAS/BUR OP NEC */
"8400" = "1" /* UPPER LIMB AMPUTAT NOS */
"8401" = "1" /* FINGER AMPUTATION */
"8402" = "1" /* THUMB AMPUTATION */
"8403" = "1" /* AMPUTATION THROUGH HAND */
"8404" = "1" /* DISARTICULATION OF WRIST */
"8405" = "1" /* AMPUTATION THRU FOREARM */
"8406" = "1" /* DISARTICULATION OF ELBOW */
"8407" = "1" /* AMPUTATION THRU HUMERUS */
"8408" = "1" /* SHOULDER DISARTICULATION */
"8409" = "1" /* FOREQUARTER AMPUTATION */
"8410" = "1" /* LOWER LIMB AMPUTAT NOS */
"8411" = "1" /* TOE AMPUTATION */
"8412" = "1" /* AMPUTATION THROUGH FOOT */
"8413" = "1" /* DISARTICULATION OF ANKLE */
"8414" = "1" /* AMPUTAT THROUGH MALLEOLI */
"8415" = "1" /* BELOW KNEE AMPUTAT NEC */
"8416" = "1" /* DISARTICULATION OF KNEE */
"8417" = "1" /* ABOVE KNEE AMPUTATION */
"8418" = "1" /* DISARTICULATION OF HIP */
"8419" = "1" /* HINDQUARTER AMPUTATION */
"8421" = "1" /* THUMB REATTACHMENT */
"8422" = "1" /* FINGER REATTACHMENT */
"8423" = "1" /* FOREARM/WRIST/HAND REATT */
"8424" = "1" /* UPPER ARM REATTACHMENT */
"8425" = "1" /* TOE REATTACHMENT */
"8426" = "1" /* FOOT REATTACHMENT */
"8427" = "1" /* LOWER LEG/ANKLE REATTACH */
"8428" = "1" /* THIGH REATTACHMENT */
"8429" = "1" /* REATTACHMENT NEC */
"843 " = "1" /* AMPUTATION STUMP REVIS */
"8440" = "1" /* IMPLNT/FIT PROS LIMB NOS */
"8444" = "1" /* IMPLANT ARM PROSTHESIS */
"8448" = "1" /* IMPLANT LEG PROSTHESIS */
"8458" = "1" /* IMP INTRSPINE DECOMP DEV */
"8459" = "1" /* INSERT OTHR SPIN DEVICE */
"8460" = "1" /* INSERT DISC PROS NOS */
"8461" = "1" /* INS PART DISC PROS CERV */
"8462" = "1" /* INS TOT DISC PROST CERV */
"8463" = "1" /* INS SPIN DISC PROS THOR */
"8464" = "1" /* INS PART DISC PROS LUMB */
"8465" = "1" /* INS TOTL DISC PROS LUMB */
"8466" = "1" /* REVISE DISC PROST CERV */
"8467" = "1" /* REVISE DISC PROST THORA */
"8468" = "1" /* REVISE DISC PROSTH LUMB */
"8469" = "1" /* REVISE DISC PROSTH NOS */
"8472" = "1" /* APP EXT FIX DEV-RING SYS */
"8473" = "1" /* APP HYBRID EXT FIX DEV */
"8491" = "1" /* AMPUTATION NOS */
"8492" = "1" /* SEPARAT EQUAL JOIN TWIN */
"8493" = "1" /* SEPARAT UNEQUL JOIN TWIN */
"8499" = "1" /* MUSCULOSKELETAL OP NEC */
"8512" = "1" /* OPEN BREAST BIOPSY */
"8520" = "1" /* BREAST TISSU DESTRUC NOS */
"8521" = "1" /* LOCAL EXCIS BREAST LES */
"8522" = "1" /* QUADRANT RESECT BREAST */
"8523" = "1" /* SUBTOTAL MASTECTOMY */
"8524" = "1" /* EXC ECTOPIC BREAST TISSU */
"8525" = "1" /* EXCISION OF NIPPLE */
"8531" = "1" /* UNILAT REDUCT MAMMOPLAST */
"8532" = "1" /* BILAT REDUCT MAMMOPLASTY */
"8533" = "1" /* UNIL SUBQ MAMMECT-IMPLNT */
"8534" = "1" /* UNILAT SUBQ MAMMECT NEC */
"8535" = "1" /* BIL SUBQ MAMMECT-IMPLANT */
"8536" = "1" /* BILAT SUBQ MAMMECTOM NEC */
"8541" = "1" /* UNILAT SIMPLE MASTECTOMY */
"8542" = "1" /* BILAT SIMPLE MASTECTOMY */
"8543" = "1" /* UNILAT EXTEN SIMP MASTEC */
"8544" = "1" /* BILAT EXTEND SIMP MASTEC */
"8545" = "1" /* UNILAT RADICAL MASTECTOM */
"8546" = "1" /* BILAT RADICAL MASTECTOMY */
"8547" = "1" /* UNIL EXT RAD MASTECTOMY */
"8548" = "1" /* BIL EXTEN RAD MASTECTOMY */
"8550" = "1" /* AUGMENT MAMMOPLASTY NOS */
"8553" = "1" /* UNILAT BREAST IMPLANT */
"8554" = "1" /* BILATERAL BREAST IMPLANT */
"856 " = "1" /* MASTOPEXY */
"857 " = "1" /* TOTAL BREAST RECONSTRUCT */
"8582" = "1" /* BREAST SPLIT-THICK GRAFT */
"8583" = "1" /* BREAST FULL-THICK GRAFT */
"8584" = "1" /* BREAST PEDICLE GRAFT */
"8585" = "1" /* BREAST MUSCLE FLAP GRAFT */
"8586" = "1" /* TRANSPOSITION OF NIPPLE */
"8587" = "1" /* NIPPLE REPAIR NEC */
"8589" = "1" /* MAMMOPLASTY NEC */
"8593" = "1" /* BREAST IMPLANT REVISION */
"8594" = "1" /* BREAST IMPLANT REMOVAL */
"8595" = "1" /* INSER BREAST TISSU EXPAN */
"8596" = "1" /* REMOV BREAST TISSU EXPAN */
"8599" = "1" /* BREAST OPERATION NEC */
"8606" = "1" /* INSERT INFUSION PUMP */
"8621" = "1" /* EXCISION OF PILONID CYST */
"8622" = "1" /* EXC WOUND DEBRIDEMENT */
"8625" = "1" /* DERMABRASION */
"864 " = "1" /* RADICAL EXCIS SKIN LES */
"8660" = "1" /* FREE SKIN GRAFT NOS */
"8661" = "1" /* FULL-THICK HAND SKIN GRF */
"8662" = "1" /* HAND SKIN GRAFT NEC */
"8663" = "1" /* FULL-THICK SKIN GRFT NEC */
"8665" = "1" /* HETEROGRAFT TO SKIN */
"8666" = "1" /* HOMOGRAFT TO SKIN */
"8667" = "1" /* DERMAL REGENER GRAFT */
"8669" = "1" /* FREE SKIN GRAFT NEC */
"8670" = "1" /* PEDICLE GRAFT/FLAP NOS */
"8671" = "1" /* CUT & PREP PEDICLE GRAFT */
"8672" = "1" /* PEDICLE GRAFT ADVANCEMEN */
"8673" = "1" /* ATTACH PEDICLE TO HAND */
"8674" = "1" /* ATTACH PEDICLE GRAFT NEC */
"8675" = "1" /* REVISION OF PEDICLE GRFT */
"8681" = "1" /* REPAIR FACIAL WEAKNESS */
"8682" = "1" /* FACIAL RHYTIDECTOMY */
"8683" = "1" /* SIZE REDUCT PLASTIC OP */
"8684" = "1" /* RELAXATION OF SCAR */
"8685" = "1" /* SYNDACTYLY CORRECTION */
"8686" = "1" /* ONYCHOPLASTY */
"8689" = "1" /* SKIN REPAIR & PLASTY NEC */
"8691" = "1" /* SKIN EXCISION FOR GRAFT */
"8693" = "1" /* INSERT TISSUE EXPANDER */
"8694" = "1" /* INS/REPL SINGLE PUL GEN */
"8695" = "1" /* INS/REPL DUAL PULSE GEN */
"8696" = "1" /* INSERT/REPL OTH NEUROST */
"8697" = "1" /* INS/REP 1 PUL GEN */
"8698" = "1" /* INS/REP 2 PUL GEN */
"8753" = "1" /* INTRAOPER CHOLANGIOGRAM */
"9504" = "1" /* ANESTHETIZED EYE EXAM */
OTHER  = "0" ;
;

VALUE $POVCAT
"01001" = 5
"01003" = 5
"01005" = 10
"01007" = 9
"01009" = 6
"01011" = 10
"01013" = 10
"01015" = 9
"01017" = 9
"01019" = 9
"01021" = 8
"01023" = 9
"01025" = 10
"01027" = 7
"01029" = 8
"01031" = 8
"01033" = 8
"01035" = 10
"01037" = 8
"01039" = 9
"01041" = 10
"01043" = 7
"01045" = 9
"01047" = 10
"01049" = 8
"01051" = 6
"01053" = 9
"01055" = 9
"01057" = 9
"01059" = 9
"01061" = 10
"01063" = 10
"01065" = 10
"01067" = 9
"01069" = 9
"01071" = 8
"01073" = 7
"01075" = 9
"01077" = 8
"01079" = 7
"01081" = 6
"01083" = 6
"01085" = 10
"01087" = 10
"01089" = 5
"01091" = 10
"01093" = 9
"01095" = 9
"01097" = 10
"01099" = 9
"01101" = 10
"01103" = 7
"01105" = 10
"01107" = 10
"01109" = 10
"01111" = 9
"01113" = 10
"01115" = 7
"01117" = 2
"01119" = 10
"01121" = 9
"01123" = 9
"01125" = 8
"01127" = 8
"01129" = 9
"01131" = 10
"01133" = 10
"02013" = 2
"02016" = 1
"02020" = 2
"02050" = 10
"02060" = 2
"02068" = 1
"02070" = 8
"02090" = 2
"02100" = 4
"02110" = 2
"02122" = 3
"02130" = 2
"02150" = 3
"02164" = 7
"02170" = 4
"02180" = 8
"02185" = 3
"02188" = 7
"02201" = 7
"02220" = 2
"02232" = 5
"02240" = 9
"02261" = 2
"02270" = 10
"02280" = 3
"02282" = 4
"02290" = 7
"04001" = 10
"04003" = 9
"04005" = 8
"04007" = 10
"04009" = 9
"04011" = 4
"04012" = 10
"04013" = 6
"04015" = 9
"04017" = 10
"04019" = 8
"04021" = 8
"04023" = 10
"04025" = 7
"04027" = 10
"05001" = 9
"05003" = 10
"05005" = 9
"05007" = 4
"05009" = 9
"05011" = 10
"05013" = 7
"05015" = 9
"05017" = 10
"05019" = 8
"05021" = 9
"05023" = 9
"05025" = 8
"05027" = 10
"05029" = 9
"05031" = 9
"05033" = 8
"05035" = 10
"05037" = 10
"05039" = 10
"05041" = 10
"05043" = 10
"05045" = 5
"05047" = 9
"05049" = 10
"05051" = 10
"05053" = 5
"05055" = 8
"05057" = 10
"05059" = 9
"05061" = 9
"05063" = 9
"05065" = 10
"05067" = 10
"05069" = 10
"05071" = 9
"05073" = 10
"05075" = 10
"05077" = 10
"05079" = 10
"05081" = 9
"05083" = 9
"05085" = 5
"05087" = 9
"05089" = 10
"05091" = 10
"05093" = 10
"05095" = 10
"05097" = 10
"05099" = 10
"05101" = 10
"05103" = 10
"05105" = 8
"05107" = 10
"05109" = 8
"05111" = 10
"05113" = 10
"05115" = 8
"05117" = 9
"05119" = 8
"05121" = 10
"05123" = 10
"05125" = 4
"05127" = 10
"05129" = 10
"05131" = 8
"05133" = 10
"05135" = 10
"05137" = 10
"05139" = 9
"05141" = 10
"05143" = 6
"05145" = 8
"05147" = 10
"05149" = 9
"06001" = 4
"06003" = 8
"06005" = 3
"06007" = 8
"06009" = 4
"06011" = 7
"06013" = 2
"06015" = 9
"06017" = 2
"06019" = 10
"06021" = 8
"06023" = 8
"06025" = 10
"06027" = 5
"06029" = 9
"06031" = 9
"06033" = 8
"06035" = 5
"06037" = 9
"06039" = 10
"06041" = 1
"06043" = 5
"06045" = 8
"06047" = 10
"06049" = 9
"06051" = 2
"06053" = 7
"06055" = 2
"06057" = 3
"06059" = 4
"06061" = 1
"06063" = 4
"06065" = 6
"06067" = 7
"06069" = 3
"06071" = 8
"06073" = 5
"06075" = 6
"06077" = 7
"06079" = 4
"06081" = 1
"06083" = 6
"06085" = 3
"06087" = 5
"06089" = 7
"06091" = 4
"06093" = 9
"06095" = 3
"06097" = 2
"06099" = 7
"06101" = 7
"06103" = 8
"06105" = 8
"06107" = 10
"06109" = 6
"06111" = 4
"06113" = 4
"06115" = 9
"08001" = 4
"08003" = 9
"08005" = 2
"08007" = 5
"08009" = 9
"08011" = 9
"08013" = 2
"08014" = 1
"08015" = 5
"08017" = 5
"08019" = 2
"08021" = 9
"08023" = 10
"08025" = 10
"08027" = 6
"08029" = 7
"08031" = 8
"08033" = 5
"08035" = 1
"08037" = 1
"08039" = 1
"08041" = 3
"08043" = 6
"08045" = 2
"08047" = 1
"08049" = 2
"08051" = 3
"08053" = 8
"08055" = 10
"08057" = 7
"08059" = 1
"08061" = 4
"08063" = 6
"08065" = 4
"08067" = 3
"08069" = 2
"08071" = 8
"08073" = 6
"08075" = 5
"08077" = 5
"08079" = 4
"08081" = 3
"08083" = 8
"08085" = 6
"08087" = 5
"08089" = 9
"08091" = 2
"08093" = 2
"08095" = 4
"08097" = 1
"08099" = 9
"08101" = 7
"08103" = 3
"08105" = 10
"08107" = 1
"08109" = 10
"08111" = 8
"08113" = 2
"08115" = 5
"08117" = 1
"08119" = 3
"08121" = 4
"08123" = 3
"08125" = 5
"09001" = 2
"09003" = 3
"09005" = 1
"09007" = 1
"09009" = 4
"09011" = 2
"09013" = 1
"09015" = 3
"10001" = 5
"10003" = 3
"10005" = 5
"11001" = 10
"12001" = 7
"12003" = 6
"12005" = 7
"12007" = 8
"12009" = 4
"12011" = 5
"12013" = 9
"12015" = 5
"12017" = 8
"12019" = 2
"12021" = 4
"12023" = 9
"12027" = 10
"12029" = 10
"12031" = 6
"12033" = 8
"12035" = 4
"12037" = 8
"12039" = 9
"12041" = 7
"12043" = 6
"12045" = 8
"12047" = 10
"12049" = 10
"12051" = 9
"12053" = 7
"12055" = 8
"12057" = 7
"12059" = 10
"12061" = 5
"12063" = 8
"12065" = 8
"12067" = 9
"12069" = 6
"12071" = 5
"12073" = 5
"12075" = 9
"12077" = 8
"12079" = 9
"12081" = 5
"12083" = 9
"12085" = 4
"12086" = 10
"12087" = 4
"12089" = 3
"12091" = 4
"12093" = 8
"12095" = 6
"12097" = 7
"12099" = 5
"12101" = 6
"12103" = 6
"12105" = 7
"12107" = 10
"12109" = 2
"12111" = 7
"12113" = 4
"12115" = 3
"12117" = 3
"12119" = 9
"12121" = 9
"12123" = 9
"12125" = 7
"12127" = 7
"12129" = 5
"12131" = 8
"12133" = 9
"13001" = 9
"13003" = 10
"13005" = 9
"13007" = 10
"13009" = 8
"13011" = 6
"13013" = 4
"13015" = 5
"13017" = 10
"13019" = 9
"13021" = 10
"13023" = 9
"13025" = 9
"13027" = 10
"13029" = 4
"13031" = 9
"13033" = 10
"13035" = 6
"13037" = 10
"13039" = 4
"13043" = 10
"13045" = 7
"13047" = 5
"13049" = 9
"13051" = 9
"13053" = 5
"13055" = 7
"13057" = 2
"13059" = 8
"13061" = 10
"13063" = 7
"13065" = 10
"13067" = 3
"13069" = 10
"13071" = 10
"13073" = 2
"13075" = 9
"13077" = 3
"13079" = 7
"13081" = 10
"13083" = 5
"13085" = 4
"13087" = 10
"13089" = 6
"13091" = 9
"13093" = 10
"13095" = 10
"13097" = 4
"13099" = 10
"13101" = 9
"13103" = 4
"13105" = 8
"13107" = 10
"13109" = 10
"13111" = 8
"13113" = 1
"13115" = 7
"13117" = 1
"13119" = 8
"13121" = 8
"13123" = 7
"13125" = 6
"13127" = 9
"13129" = 6
"13131" = 10
"13133" = 10
"13135" = 2
"13137" = 5
"13139" = 5
"13141" = 10
"13143" = 8
"13145" = 3
"13147" = 7
"13149" = 8
"13151" = 2
"13153" = 5
"13155" = 10
"13157" = 6
"13159" = 8
"13161" = 9
"13163" = 10
"13165" = 10
"13167" = 10
"13169" = 4
"13171" = 8
"13173" = 10
"13175" = 9
"13177" = 3
"13179" = 7
"13181" = 8
"13183" = 10
"13185" = 9
"13187" = 6
"13189" = 9
"13191" = 10
"13193" = 10
"13195" = 6
"13197" = 10
"13199" = 9
"13201" = 10
"13205" = 10
"13207" = 6
"13209" = 9
"13211" = 6
"13213" = 6
"13215" = 9
"13217" = 6
"13219" = 2
"13221" = 6
"13223" = 2
"13225" = 10
"13227" = 6
"13229" = 9
"13231" = 5
"13233" = 7
"13235" = 8
"13237" = 8
"13239" = 10
"13241" = 7
"13243" = 10
"13245" = 10
"13247" = 5
"13249" = 8
"13251" = 9
"13253" = 10
"13255" = 9
"13257" = 8
"13259" = 10
"13261" = 10
"13263" = 10
"13265" = 10
"13267" = 10
"13269" = 10
"13271" = 10
"13273" = 10
"13275" = 9
"13277" = 9
"13279" = 10
"13281" = 7
"13283" = 10
"13285" = 8
"13287" = 10
"13289" = 9
"13291" = 7
"13293" = 9
"13295" = 7
"13297" = 5
"13299" = 10
"13301" = 10
"13303" = 10
"13305" = 9
"13307" = 9
"13309" = 10
"13311" = 6
"13313" = 5
"13315" = 10
"13317" = 9
"13319" = 8
"13321" = 9
"15001" = 6
"15003" = 4
"15005" = 1
"15007" = 4
"15009" = 4
"16001" = 3
"16003" = 8
"16005" = 6
"16007" = 5
"16009" = 7
"16011" = 6
"16013" = 1
"16015" = 5
"16017" = 7
"16019" = 5
"16021" = 7
"16023" = 6
"16025" = 1
"16027" = 7
"16029" = 4
"16031" = 7
"16033" = 8
"16035" = 7
"16037" = 5
"16039" = 4
"16041" = 4
"16043" = 7
"16045" = 6
"16047" = 5
"16049" = 7
"16051" = 5
"16053" = 6
"16055" = 5
"16057" = 4
"16059" = 7
"16061" = 7
"16063" = 5
"16065" = 4
"16067" = 6
"16069" = 5
"16071" = 4
"16073" = 9
"16075" = 7
"16077" = 7
"16079" = 9
"16081" = 3
"16083" = 6
"16085" = 4
"16087" = 7
"17001" = 5
"17003" = 10
"17005" = 4
"17007" = 2
"17009" = 3
"17011" = 3
"17013" = 2
"17015" = 4
"17017" = 4
"17019" = 4
"17021" = 4
"17023" = 5
"17025" = 5
"17027" = 2
"17029" = 5
"17031" = 8
"17033" = 5
"17035" = 4
"17037" = 2
"17039" = 4
"17041" = 3
"17043" = 1
"17045" = 6
"17047" = 4
"17049" = 3
"17051" = 6
"17053" = 3
"17055" = 8
"17057" = 5
"17059" = 9
"17061" = 6
"17063" = 1
"17065" = 6
"17067" = 4
"17069" = 8
"17071" = 5
"17073" = 3
"17075" = 4
"17077" = 9
"17079" = 4
"17081" = 7
"17083" = 3
"17085" = 2
"17087" = 6
"17089" = 2
"17091" = 5
"17093" = 1
"17095" = 6
"17097" = 2
"17099" = 4
"17101" = 6
"17103" = 3
"17105" = 3
"17107" = 4
"17109" = 6
"17111" = 1
"17113" = 2
"17115" = 7
"17117" = 4
"17119" = 4
"17121" = 8
"17123" = 3
"17125" = 5
"17127" = 7
"17129" = 3
"17131" = 3
"17133" = 1
"17135" = 5
"17137" = 5
"17139" = 3
"17141" = 2
"17143" = 7
"17145" = 5
"17147" = 1
"17149" = 6
"17151" = 7
"17153" = 10
"17155" = 2
"17157" = 5
"17159" = 5
"17161" = 6
"17163" = 8
"17165" = 9
"17167" = 4
"17169" = 3
"17171" = 4
"17173" = 4
"17175" = 4
"17177" = 4
"17179" = 2
"17181" = 7
"17183" = 7
"17185" = 5
"17187" = 4
"17189" = 2
"17191" = 5
"17193" = 6
"17195" = 4
"17197" = 1
"17199" = 7
"17201" = 6
"17203" = 1
"18001" = 5
"18003" = 4
"18005" = 2
"18007" = 2
"18009" = 4
"18011" = 1
"18013" = 2
"18015" = 2
"18017" = 4
"18019" = 4
"18021" = 4
"18023" = 3
"18025" = 6
"18027" = 6
"18029" = 2
"18031" = 3
"18033" = 2
"18035" = 5
"18037" = 1
"18039" = 4
"18041" = 4
"18043" = 3
"18045" = 3
"18047" = 2
"18049" = 4
"18051" = 2
"18053" = 6
"18055" = 4
"18057" = 1
"18059" = 1
"18061" = 2
"18063" = 1
"18065" = 4
"18067" = 4
"18069" = 2
"18071" = 3
"18073" = 2
"18075" = 5
"18077" = 4
"18079" = 4
"18081" = 1
"18083" = 6
"18085" = 2
"18087" = 3
"18089" = 6
"18091" = 4
"18093" = 3
"18095" = 5
"18097" = 6
"18099" = 2
"18101" = 3
"18103" = 4
"18105" = 3
"18107" = 3
"18109" = 2
"18111" = 2
"18113" = 2
"18115" = 2
"18117" = 6
"18119" = 4
"18121" = 5
"18123" = 3
"18125" = 3
"18127" = 1
"18129" = 2
"18131" = 4
"18133" = 2
"18135" = 4
"18137" = 2
"18139" = 3
"18141" = 5
"18143" = 5
"18145" = 2
"18147" = 2
"18149" = 5
"18151" = 2
"18153" = 5
"18155" = 4
"18157" = 3
"18159" = 1
"18161" = 3
"18163" = 5
"18165" = 3
"18167" = 6
"18169" = 3
"18171" = 2
"18173" = 1
"18175" = 4
"18177" = 5
"18179" = 2
"18181" = 3
"18183" = 1
"19001" = 3
"19003" = 5
"19005" = 3
"19007" = 7
"19009" = 3
"19011" = 1
"19013" = 5
"19015" = 2
"19017" = 1
"19019" = 3
"19021" = 3
"19023" = 2
"19025" = 4
"19027" = 2
"19029" = 4
"19031" = 1
"19033" = 3
"19035" = 3
"19037" = 3
"19039" = 4
"19041" = 3
"19043" = 3
"19045" = 4
"19047" = 4
"19049" = 1
"19051" = 6
"19053" = 7
"19055" = 2
"19057" = 6
"19059" = 2
"19061" = 2
"19063" = 3
"19065" = 4
"19067" = 4
"19069" = 3
"19071" = 3
"19073" = 4
"19075" = 1
"19077" = 3
"19079" = 2
"19081" = 2
"19083" = 3
"19085" = 3
"19087" = 2
"19089" = 3
"19091" = 3
"19093" = 3
"19095" = 1
"19097" = 4
"19099" = 2
"19101" = 5
"19103" = 2
"19105" = 2
"19107" = 4
"19109" = 3
"19111" = 5
"19113" = 2
"19115" = 3
"19117" = 5
"19119" = 2
"19121" = 2
"19123" = 3
"19125" = 2
"19127" = 4
"19129" = 2
"19131" = 3
"19133" = 4
"19135" = 4
"19137" = 4
"19139" = 4
"19141" = 2
"19143" = 2
"19145" = 4
"19147" = 3
"19149" = 1
"19151" = 4
"19153" = 3
"19155" = 4
"19157" = 2
"19159" = 7
"19161" = 4
"19163" = 4
"19165" = 2
"19167" = 1
"19169" = 1
"19171" = 2
"19173" = 5
"19175" = 5
"19177" = 5
"19179" = 6
"19181" = 1
"19183" = 2
"19185" = 7
"19187" = 4
"19189" = 2
"19191" = 1
"19193" = 5
"19195" = 2
"19197" = 3
"20001" = 6
"20003" = 5
"20005" = 4
"20007" = 5
"20009" = 5
"20011" = 7
"20013" = 6
"20015" = 2
"20017" = 4
"20019" = 7
"20021" = 7
"20023" = 4
"20025" = 4
"20027" = 4
"20029" = 4
"20031" = 3
"20033" = 3
"20035" = 6
"20037" = 7
"20039" = 5
"20041" = 3
"20043" = 4
"20045" = 3
"20047" = 5
"20049" = 8
"20051" = 2
"20053" = 2
"20055" = 5
"20057" = 4
"20059" = 4
"20061" = 6
"20063" = 4
"20065" = 5
"20067" = 3
"20069" = 3
"20071" = 3
"20073" = 7
"20075" = 5
"20077" = 5
"20079" = 3
"20081" = 4
"20083" = 2
"20085" = 3
"20087" = 2
"20089" = 6
"20091" = 1
"20093" = 3
"20095" = 4
"20097" = 5
"20099" = 7
"20101" = 3
"20103" = 2
"20105" = 5
"20107" = 6
"20109" = 3
"20111" = 4
"20113" = 2
"20115" = 3
"20117" = 3
"20119" = 4
"20121" = 2
"20123" = 4
"20125" = 7
"20127" = 3
"20129" = 4
"20131" = 2
"20133" = 5
"20135" = 3
"20137" = 3
"20139" = 3
"20141" = 5
"20143" = 2
"20145" = 4
"20147" = 4
"20149" = 3
"20151" = 4
"20153" = 4
"20155" = 5
"20157" = 5
"20159" = 5
"20161" = 3
"20163" = 4
"20165" = 5
"20167" = 5
"20169" = 4
"20171" = 2
"20173" = 5
"20175" = 6
"20177" = 4
"20179" = 4
"20181" = 6
"20183" = 4
"20185" = 6
"20187" = 4
"20189" = 5
"20191" = 4
"20193" = 3
"20195" = 4
"20197" = 2
"20199" = 7
"20201" = 4
"20203" = 5
"20205" = 6
"20207" = 6
"20209" = 8
"21001" = 10
"21003" = 7
"21005" = 3
"21007" = 7
"21009" = 8
"21011" = 10
"21013" = 10
"21015" = 2
"21017" = 7
"21019" = 9
"21021" = 7
"21023" = 5
"21025" = 10
"21027" = 9
"21029" = 4
"21031" = 9
"21033" = 8
"21035" = 7
"21037" = 4
"21039" = 7
"21041" = 7
"21043" = 10
"21045" = 10
"21047" = 8
"21049" = 7
"21051" = 10
"21053" = 10
"21055" = 9
"21057" = 10
"21059" = 6
"21061" = 9
"21063" = 10
"21065" = 10
"21067" = 6
"21069" = 9
"21071" = 10
"21073" = 5
"21075" = 10
"21077" = 7
"21079" = 8
"21081" = 6
"21083" = 8
"21085" = 9
"21087" = 9
"21089" = 8
"21091" = 4
"21093" = 5
"21095" = 10
"21097" = 6
"21099" = 10
"21101" = 7
"21103" = 6
"21105" = 8
"21107" = 9
"21109" = 10
"21111" = 7
"21113" = 6
"21115" = 10
"21117" = 4
"21119" = 10
"21121" = 10
"21123" = 8
"21125" = 10
"21127" = 10
"21129" = 10
"21131" = 10
"21133" = 10
"21135" = 10
"21137" = 9
"21139" = 7
"21141" = 8
"21143" = 6
"21145" = 8
"21147" = 10
"21149" = 7
"21151" = 7
"21153" = 10
"21155" = 8
"21157" = 6
"21159" = 10
"21161" = 9
"21163" = 4
"21165" = 10
"21167" = 6
"21169" = 10
"21171" = 10
"21173" = 8
"21175" = 10
"21177" = 9
"21179" = 5
"21181" = 8
"21183" = 9
"21185" = 1
"21187" = 8
"21189" = 10
"21191" = 5
"21193" = 10
"21195" = 10
"21197" = 10
"21199" = 9
"21201" = 9
"21203" = 10
"21205" = 9
"21207" = 10
"21209" = 4
"21211" = 4
"21213" = 7
"21215" = 3
"21217" = 9
"21219" = 9
"21221" = 6
"21223" = 6
"21225" = 7
"21227" = 7
"21229" = 6
"21231" = 10
"21233" = 7
"21235" = 10
"21237" = 10
"21239" = 3
"22001" = 10
"22003" = 10
"22005" = 5
"22007" = 9
"22009" = 10
"22011" = 8
"22013" = 10
"22015" = 7
"22017" = 10
"22019" = 9
"22021" = 10
"22023" = 5
"22025" = 10
"22027" = 10
"22029" = 10
"22031" = 10
"22033" = 9
"22035" = 10
"22037" = 9
"22039" = 10
"22041" = 10
"22043" = 10
"22045" = 10
"22047" = 10
"22049" = 9
"22051" = 9
"22053" = 9
"22055" = 8
"22057" = 8
"22059" = 9
"22061" = 10
"22063" = 6
"22065" = 10
"22067" = 10
"22069" = 10
"22071" = 10
"22073" = 10
"22075" = 8
"22077" = 9
"22079" = 10
"22081" = 10
"22083" = 10
"22085" = 10
"22087" = 8
"22089" = 6
"22091" = 10
"22093" = 9
"22095" = 9
"22097" = 10
"22099" = 10
"22101" = 10
"22103" = 5
"22105" = 10
"22107" = 10
"22109" = 9
"22111" = 9
"22113" = 9
"22115" = 7
"22117" = 10
"22119" = 10
"22121" = 8
"22123" = 10
"22125" = 6
"22127" = 10
"23001" = 5
"23003" = 7
"23005" = 2
"23007" = 6
"23009" = 4
"23011" = 5
"23013" = 5
"23015" = 4
"23017" = 6
"23019" = 5
"23021" = 7
"23023" = 3
"23025" = 8
"23027" = 7
"23029" = 9
"23031" = 2
"24001" = 6
"24003" = 1
"24005" = 2
"24009" = 1
"24011" = 4
"24013" = 1
"24015" = 3
"24017" = 2
"24019" = 7
"24021" = 1
"24023" = 6
"24025" = 1
"24027" = 1
"24029" = 4
"24031" = 1
"24033" = 3
"24035" = 1
"24037" = 2
"24039" = 9
"24041" = 3
"24043" = 4
"24045" = 5
"24047" = 5
"24510" = 9
"25001" = 2
"25003" = 4
"25005" = 4
"25007" = 2
"25009" = 3
"25011" = 3
"25013" = 8
"25015" = 2
"25017" = 1
"25019" = 1
"25021" = 1
"25023" = 2
"25025" = 9
"25027" = 3
"26001" = 8
"26003" = 4
"26005" = 2
"26007" = 6
"26009" = 4
"26011" = 8
"26013" = 4
"26015" = 2
"26017" = 4
"26019" = 3
"26021" = 6
"26023" = 5
"26025" = 5
"26027" = 5
"26029" = 3
"26031" = 6
"26033" = 5
"26035" = 8
"26037" = 1
"26039" = 7
"26041" = 4
"26043" = 3
"26045" = 2
"26047" = 2
"26049" = 7
"26051" = 7
"26053" = 6
"26055" = 2
"26057" = 4
"26059" = 5
"26061" = 5
"26063" = 5
"26065" = 5
"26067" = 3
"26069" = 8
"26071" = 5
"26073" = 4
"26075" = 4
"26077" = 4
"26079" = 6
"26081" = 3
"26083" = 4
"26085" = 10
"26087" = 2
"26089" = 2
"26091" = 2
"26093" = 1
"26095" = 7
"26097" = 5
"26099" = 2
"26101" = 5
"26103" = 3
"26105" = 5
"26107" = 7
"26109" = 4
"26111" = 2
"26113" = 6
"26115" = 2
"26117" = 5
"26119" = 8
"26121" = 6
"26123" = 5
"26125" = 1
"26127" = 7
"26129" = 8
"26131" = 5
"26133" = 6
"26135" = 8
"26137" = 4
"26139" = 1
"26141" = 5
"26143" = 9
"26145" = 7
"26147" = 3
"26149" = 5
"26151" = 6
"26153" = 6
"26155" = 3
"26157" = 4
"26159" = 6
"26161" = 2
"26163" = 9
"26165" = 5
"27001" = 5
"27003" = 1
"27005" = 5
"27007" = 8
"27009" = 2
"27011" = 4
"27013" = 2
"27015" = 1
"27017" = 2
"27019" = 1
"27021" = 6
"27023" = 3
"27025" = 1
"27027" = 3
"27029" = 6
"27031" = 2
"27033" = 4
"27035" = 3
"27037" = 1
"27039" = 1
"27041" = 2
"27043" = 3
"27045" = 3
"27047" = 3
"27049" = 1
"27051" = 3
"27053" = 3
"27055" = 1
"27057" = 5
"27059" = 1
"27061" = 4
"27063" = 2
"27065" = 3
"27067" = 3
"27069" = 2
"27071" = 4
"27073" = 2
"27075" = 2
"27077" = 2
"27079" = 1
"27081" = 2
"27083" = 2
"27085" = 1
"27087" = 9
"27089" = 3
"27091" = 4
"27093" = 2
"27095" = 3
"27097" = 3
"27099" = 3
"27101" = 2
"27103" = 1
"27105" = 3
"27107" = 3
"27109" = 1
"27111" = 3
"27113" = 3
"27115" = 4
"27117" = 3
"27119" = 4
"27121" = 2
"27123" = 4
"27125" = 2
"27127" = 3
"27129" = 3
"27131" = 1
"27133" = 2
"27135" = 1
"27137" = 4
"27139" = 1
"27141" = 1
"27143" = 2
"27145" = 2
"27147" = 1
"27149" = 1
"27151" = 3
"27153" = 5
"27155" = 4
"27157" = 1
"27159" = 5
"27161" = 2
"27163" = 1
"27165" = 3
"27167" = 2
"27169" = 2
"27171" = 1
"27173" = 2
"28001" = 10
"28003" = 9
"28005" = 10
"28007" = 10
"28009" = 10
"28011" = 10
"28013" = 10
"28015" = 10
"28017" = 9
"28019" = 10
"28021" = 10
"28023" = 10
"28025" = 10
"28027" = 10
"28029" = 10
"28031" = 10
"28033" = 3
"28035" = 10
"28037" = 10
"28039" = 9
"28041" = 9
"28043" = 10
"28045" = 9
"28047" = 9
"28049" = 10
"28051" = 10
"28053" = 10
"28055" = 10
"28057" = 7
"28059" = 9
"28061" = 10
"28063" = 10
"28065" = 10
"28067" = 10
"28069" = 10
"28071" = 6
"28073" = 7
"28075" = 10
"28077" = 10
"28079" = 10
"28081" = 7
"28083" = 10
"28085" = 9
"28087" = 10
"28089" = 7
"28091" = 10
"28093" = 9
"28095" = 9
"28097" = 10
"28099" = 10
"28101" = 9
"28103" = 10
"28105" = 10
"28107" = 10
"28109" = 10
"28111" = 10
"28113" = 10
"28115" = 6
"28117" = 8
"28119" = 10
"28121" = 5
"28123" = 10
"28125" = 10
"28127" = 10
"28129" = 9
"28131" = 10
"28133" = 10
"28135" = 10
"28137" = 8
"28139" = 9
"28141" = 7
"28143" = 10
"28145" = 7
"28147" = 10
"28149" = 10
"28151" = 10
"28153" = 10
"28155" = 10
"28157" = 10
"28159" = 10
"28161" = 10
"28163" = 10
"29001" = 6
"29003" = 3
"29005" = 5
"29007" = 7
"29009" = 8
"29011" = 6
"29013" = 7
"29015" = 9
"29017" = 7
"29019" = 4
"29021" = 6
"29023" = 10
"29025" = 5
"29027" = 4
"29029" = 7
"29031" = 5
"29033" = 7
"29035" = 10
"29037" = 2
"29039" = 9
"29041" = 4
"29043" = 4
"29045" = 6
"29047" = 2
"29049" = 2
"29051" = 3
"29053" = 4
"29055" = 8
"29057" = 7
"29059" = 8
"29061" = 9
"29063" = 4
"29065" = 9
"29067" = 10
"29069" = 10
"29071" = 3
"29073" = 4
"29075" = 5
"29077" = 6
"29079" = 8
"29081" = 7
"29083" = 7
"29085" = 10
"29087" = 5
"29089" = 5
"29091" = 9
"29093" = 10
"29095" = 6
"29097" = 8
"29099" = 2
"29101" = 4
"29103" = 8
"29105" = 8
"29107" = 4
"29109" = 7
"29111" = 6
"29113" = 3
"29115" = 7
"29117" = 6
"29119" = 9
"29121" = 6
"29123" = 9
"29125" = 6
"29127" = 6
"29129" = 6
"29131" = 7
"29133" = 10
"29135" = 4
"29137" = 5
"29139" = 5
"29141" = 9
"29143" = 10
"29145" = 6
"29147" = 3
"29149" = 10
"29151" = 2
"29153" = 10
"29155" = 10
"29157" = 3
"29159" = 7
"29161" = 7
"29163" = 7
"29165" = 1
"29167" = 8
"29169" = 4
"29171" = 7
"29173" = 4
"29175" = 6
"29177" = 3
"29179" = 10
"29181" = 10
"29183" = 1
"29185" = 9
"29186" = 3
"29187" = 7
"29189" = 3
"29195" = 7
"29197" = 7
"29199" = 7
"29201" = 8
"29203" = 10
"29205" = 7
"29207" = 8
"29209" = 8
"29211" = 7
"29213" = 8
"29215" = 10
"29217" = 8
"29219" = 3
"29221" = 9
"29223" = 10
"29225" = 8
"29227" = 7
"29229" = 10
"29510" = 10
"30001" = 8
"30003" = 10
"30005" = 10
"30007" = 7
"30009" = 5
"30011" = 6
"30013" = 7
"30015" = 7
"30017" = 7
"30019" = 6
"30021" = 6
"30023" = 7
"30025" = 3
"30027" = 8
"30029" = 6
"30031" = 3
"30033" = 7
"30035" = 10
"30037" = 9
"30039" = 8
"30041" = 9
"30043" = 3
"30045" = 8
"30047" = 10
"30049" = 5
"30051" = 4
"30053" = 10
"30055" = 7
"30057" = 6
"30059" = 9
"30061" = 10
"30063" = 7
"30065" = 10
"30067" = 6
"30069" = 6
"30071" = 8
"30073" = 8
"30075" = 4
"30077" = 8
"30079" = 7
"30081" = 8
"30083" = 6
"30085" = 10
"30087" = 9
"30089" = 10
"30091" = 6
"30093" = 8
"30095" = 3
"30097" = 4
"30099" = 6
"30101" = 5
"30103" = 7
"30105" = 8
"30107" = 10
"30109" = 6
"30111" = 5
"31001" = 3
"31003" = 5
"31005" = 4
"31007" = 3
"31009" = 9
"31011" = 3
"31013" = 5
"31015" = 5
"31017" = 6
"31019" = 3
"31021" = 4
"31023" = 3
"31025" = 2
"31027" = 2
"31029" = 4
"31031" = 6
"31033" = 3
"31035" = 4
"31037" = 3
"31039" = 3
"31041" = 6
"31043" = 4
"31045" = 7
"31047" = 4
"31049" = 5
"31051" = 3
"31053" = 3
"31055" = 4
"31057" = 5
"31059" = 4
"31061" = 5
"31063" = 4
"31065" = 5
"31067" = 3
"31069" = 7
"31071" = 4
"31073" = 3
"31075" = 5
"31077" = 6
"31079" = 5
"31081" = 2
"31083" = 5
"31085" = 4
"31087" = 7
"31089" = 5
"31091" = 4
"31093" = 4
"31095" = 4
"31097" = 3
"31099" = 2
"31101" = 4
"31103" = 7
"31105" = 7
"31107" = 7
"31109" = 3
"31111" = 5
"31113" = 4
"31115" = 6
"31117" = 7
"31119" = 4
"31121" = 4
"31123" = 7
"31125" = 5
"31127" = 3
"31129" = 5
"31131" = 2
"31133" = 5
"31135" = 4
"31137" = 3
"31139" = 3
"31141" = 2
"31143" = 2
"31145" = 4
"31147" = 5
"31149" = 8
"31151" = 2
"31153" = 1
"31155" = 2
"31157" = 8
"31159" = 1
"31161" = 8
"31163" = 5
"31165" = 4
"31167" = 2
"31169" = 4
"31171" = 5
"31173" = 10
"31175" = 5
"31177" = 1
"31179" = 3
"31181" = 3
"31183" = 7
"31185" = 3
"32001" = 4
"32003" = 5
"32005" = 2
"32007" = 3
"32009" = 4
"32011" = 3
"32013" = 4
"32015" = 3
"32017" = 7
"32019" = 4
"32021" = 8
"32023" = 7
"32027" = 5
"32029" = 1
"32031" = 3
"32033" = 5
"32510" = 4
"33001" = 2
"33003" = 2
"33005" = 2
"33007" = 4
"33009" = 2
"33011" = 1
"33013" = 1
"33015" = 1
"33017" = 2
"33019" = 2
"34001" = 4
"34003" = 1
"34005" = 1
"34007" = 4
"34009" = 4
"34011" = 7
"34013" = 7
"34015" = 1
"34017" = 8
"34019" = 1
"34021" = 2
"34023" = 2
"34025" = 1
"34027" = 1
"34029" = 3
"34031" = 6
"34033" = 3
"34035" = 1
"34037" = 1
"34039" = 3
"34041" = 1
"35001" = 7
"35003" = 10
"35005" = 10
"35006" = 10
"35007" = 9
"35009" = 10
"35011" = 10
"35013" = 10
"35015" = 9
"35017" = 10
"35019" = 10
"35021" = 6
"35023" = 10
"35025" = 9
"35027" = 10
"35028" = 1
"35029" = 10
"35031" = 10
"35033" = 10
"35035" = 9
"35037" = 10
"35039" = 9
"35041" = 10
"35043" = 5
"35045" = 10
"35047" = 10
"35049" = 6
"35051" = 10
"35053" = 10
"35055" = 10
"35057" = 10
"35059" = 9
"35061" = 9
"36001" = 4
"36003" = 7
"36005" = 10
"36007" = 6
"36009" = 7
"36011" = 5
"36013" = 8
"36015" = 7
"36017" = 7
"36019" = 5
"36021" = 4
"36023" = 6
"36025" = 7
"36027" = 2
"36029" = 6
"36031" = 5
"36033" = 7
"36035" = 7
"36037" = 3
"36039" = 6
"36041" = 4
"36043" = 6
"36045" = 7
"36047" = 10
"36049" = 6
"36051" = 4
"36053" = 4
"36055" = 6
"36057" = 7
"36059" = 1
"36061" = 10
"36063" = 5
"36065" = 6
"36067" = 5
"36069" = 3
"36071" = 4
"36073" = 6
"36075" = 6
"36077" = 5
"36079" = 1
"36081" = 8
"36083" = 4
"36085" = 4
"36087" = 4
"36089" = 7
"36091" = 1
"36093" = 5
"36095" = 5
"36097" = 6
"36099" = 5
"36101" = 7
"36103" = 2
"36105" = 7
"36107" = 4
"36109" = 4
"36111" = 4
"36113" = 4
"36115" = 5
"36117" = 4
"36119" = 3
"36121" = 4
"36123" = 7
"37001" = 5
"37003" = 5
"37005" = 9
"37007" = 9
"37009" = 8
"37011" = 7
"37013" = 9
"37015" = 10
"37017" = 10
"37019" = 8
"37021" = 7
"37023" = 7
"37025" = 4
"37027" = 7
"37029" = 3
"37031" = 6
"37033" = 7
"37035" = 6
"37037" = 4
"37039" = 9
"37041" = 9
"37043" = 8
"37045" = 8
"37047" = 10
"37049" = 7
"37051" = 8
"37053" = 4
"37055" = 3
"37057" = 6
"37059" = 3
"37061" = 9
"37063" = 7
"37065" = 10
"37067" = 6
"37069" = 6
"37071" = 7
"37073" = 6
"37075" = 9
"37077" = 5
"37079" = 9
"37081" = 7
"37083" = 10
"37085" = 8
"37087" = 8
"37089" = 6
"37091" = 10
"37093" = 8
"37095" = 10
"37097" = 4
"37099" = 7
"37101" = 6
"37103" = 8
"37105" = 7
"37107" = 9
"37109" = 5
"37111" = 6
"37113" = 7
"37115" = 8
"37117" = 10
"37119" = 5
"37121" = 8
"37123" = 8
"37125" = 6
"37127" = 7
"37129" = 7
"37131" = 10
"37133" = 7
"37135" = 3
"37137" = 9
"37139" = 9
"37141" = 8
"37143" = 9
"37145" = 5
"37147" = 8
"37149" = 5
"37151" = 5
"37153" = 9
"37155" = 10
"37157" = 6
"37159" = 5
"37161" = 8
"37163" = 9
"37165" = 10
"37167" = 5
"37169" = 4
"37171" = 7
"37173" = 8
"37175" = 7
"37177" = 10
"37179" = 3
"37181" = 10
"37183" = 2
"37185" = 10
"37187" = 10
"37189" = 5
"37191" = 8
"37193" = 7
"37195" = 8
"37197" = 5
"37199" = 9
"38001" = 4
"38003" = 4
"38005" = 10
"38007" = 6
"38009" = 5
"38011" = 3
"38013" = 4
"38015" = 2
"38017" = 2
"38019" = 4
"38021" = 6
"38023" = 4
"38025" = 5
"38027" = 3
"38029" = 6
"38031" = 2
"38033" = 6
"38035" = 4
"38037" = 9
"38039" = 4
"38041" = 4
"38043" = 6
"38045" = 4
"38047" = 6
"38049" = 7
"38051" = 6
"38053" = 7
"38055" = 6
"38057" = 2
"38059" = 3
"38061" = 7
"38063" = 4
"38065" = 3
"38067" = 3
"38069" = 5
"38071" = 6
"38073" = 2
"38075" = 4
"38077" = 2
"38079" = 10
"38081" = 2
"38083" = 9
"38085" = 10
"38087" = 5
"38089" = 4
"38091" = 2
"38093" = 4
"38095" = 4
"38097" = 2
"38099" = 4
"38101" = 4
"38103" = 4
"38105" = 5
"39001" = 8
"39003" = 5
"39005" = 3
"39007" = 6
"39009" = 8
"39011" = 1
"39013" = 7
"39015" = 4
"39017" = 2
"39019" = 5
"39021" = 3
"39023" = 6
"39025" = 2
"39027" = 3
"39029" = 6
"39031" = 4
"39033" = 5
"39035" = 7
"39037" = 2
"39039" = 2
"39041" = 1
"39043" = 4
"39045" = 2
"39047" = 5
"39049" = 5
"39051" = 2
"39053" = 9
"39055" = 1
"39057" = 2
"39059" = 7
"39061" = 5
"39063" = 2
"39065" = 4
"39067" = 6
"39069" = 2
"39071" = 5
"39073" = 5
"39075" = 4
"39077" = 3
"39079" = 8
"39081" = 7
"39083" = 4
"39085" = 2
"39087" = 9
"39089" = 3
"39091" = 3
"39093" = 4
"39095" = 6
"39097" = 3
"39099" = 7
"39101" = 5
"39103" = 1
"39105" = 9
"39107" = 2
"39109" = 2
"39111" = 7
"39113" = 5
"39115" = 8
"39117" = 4
"39119" = 7
"39121" = 5
"39123" = 2
"39125" = 3
"39127" = 6
"39129" = 4
"39131" = 9
"39133" = 2
"39135" = 2
"39137" = 1
"39139" = 5
"39141" = 6
"39143" = 2
"39145" = 9
"39147" = 3
"39149" = 2
"39151" = 4
"39153" = 5
"39155" = 5
"39157" = 4
"39159" = 1
"39161" = 2
"39163" = 8
"39165" = 1
"39167" = 5
"39169" = 3
"39171" = 2
"39173" = 1
"39175" = 1
"40001" = 10
"40003" = 6
"40005" = 10
"40007" = 5
"40009" = 8
"40011" = 9
"40013" = 9
"40015" = 10
"40017" = 3
"40019" = 8
"40021" = 10
"40023" = 10
"40025" = 7
"40027" = 4
"40029" = 10
"40031" = 8
"40033" = 7
"40035" = 8
"40037" = 7
"40039" = 8
"40041" = 10
"40043" = 7
"40045" = 6
"40047" = 7
"40049" = 9
"40051" = 7
"40053" = 6
"40055" = 10
"40057" = 10
"40059" = 5
"40061" = 10
"40063" = 10
"40065" = 7
"40067" = 9
"40069" = 9
"40071" = 9
"40073" = 4
"40075" = 10
"40077" = 10
"40079" = 10
"40081" = 7
"40083" = 6
"40085" = 7
"40087" = 5
"40089" = 10
"40091" = 10
"40093" = 5
"40095" = 8
"40097" = 9
"40099" = 8
"40101" = 9
"40103" = 6
"40105" = 8
"40107" = 10
"40109" = 8
"40111" = 10
"40113" = 7
"40115" = 9
"40117" = 7
"40119" = 7
"40121" = 9
"40123" = 9
"40125" = 9
"40127" = 10
"40129" = 6
"40131" = 4
"40133" = 10
"40135" = 10
"40137" = 8
"40139" = 5
"40141" = 10
"40143" = 6
"40145" = 5
"40147" = 6
"40149" = 8
"40151" = 7
"40153" = 6
"41001" = 7
"41003" = 3
"41005" = 3
"41007" = 6
"41009" = 3
"41011" = 9
"41013" = 6
"41015" = 7
"41017" = 4
"41019" = 7
"41021" = 3
"41023" = 6
"41025" = 7
"41027" = 6
"41029" = 7
"41031" = 8
"41033" = 9
"41035" = 8
"41037" = 8
"41039" = 7
"41041" = 8
"41043" = 6
"41045" = 9
"41047" = 7
"41049" = 6
"41051" = 6
"41053" = 4
"41055" = 6
"41057" = 6
"41059" = 7
"41061" = 5
"41063" = 5
"41065" = 7
"41067" = 2
"41069" = 7
"41071" = 4
"42001" = 2
"42003" = 4
"42005" = 5
"42007" = 4
"42009" = 5
"42011" = 4
"42013" = 6
"42015" = 5
"42017" = 1
"42019" = 2
"42021" = 5
"42023" = 5
"42025" = 4
"42027" = 3
"42029" = 1
"42031" = 5
"42033" = 7
"42035" = 6
"42037" = 4
"42039" = 7
"42041" = 1
"42043" = 3
"42045" = 3
"42047" = 3
"42049" = 6
"42051" = 9
"42053" = 7
"42055" = 3
"42057" = 4
"42059" = 8
"42061" = 5
"42063" = 7
"42065" = 6
"42067" = 3
"42069" = 4
"42071" = 3
"42073" = 6
"42075" = 3
"42077" = 4
"42079" = 5
"42081" = 5
"42083" = 6
"42085" = 7
"42087" = 7
"42089" = 3
"42091" = 1
"42093" = 3
"42095" = 2
"42097" = 4
"42099" = 3
"42101" = 10
"42103" = 2
"42105" = 6
"42107" = 4
"42109" = 3
"42111" = 6
"42113" = 5
"42115" = 5
"42117" = 6
"42119" = 3
"42121" = 7
"42123" = 5
"42125" = 3
"42127" = 5
"42129" = 3
"42131" = 4
"42133" = 2
"44001" = 1
"44003" = 2
"44005" = 2
"44007" = 8
"44009" = 1
"45001" = 6
"45003" = 7
"45005" = 10
"45007" = 5
"45009" = 10
"45011" = 9
"45013" = 6
"45015" = 5
"45017" = 7
"45019" = 7
"45021" = 8
"45023" = 8
"45025" = 9
"45027" = 10
"45029" = 10
"45031" = 10
"45033" = 10
"45035" = 5
"45037" = 7
"45039" = 8
"45041" = 9
"45043" = 9
"45045" = 5
"45047" = 7
"45049" = 10
"45051" = 8
"45053" = 10
"45055" = 6
"45057" = 7
"45059" = 8
"45061" = 10
"45063" = 5
"45065" = 8
"45067" = 10
"45069" = 10
"45071" = 8
"45073" = 6
"45075" = 10
"45077" = 5
"45079" = 6
"45081" = 7
"45083" = 6
"45085" = 9
"45087" = 8
"45089" = 10
"45091" = 5
"46003" = 6
"46005" = 5
"46007" = 10
"46009" = 5
"46011" = 2
"46013" = 3
"46015" = 5
"46017" = 10
"46019" = 7
"46021" = 3
"46023" = 10
"46025" = 6
"46027" = 6
"46029" = 4
"46031" = 10
"46033" = 5
"46035" = 4
"46037" = 7
"46039" = 3
"46041" = 10
"46043" = 5
"46045" = 4
"46047" = 8
"46049" = 4
"46051" = 3
"46053" = 9
"46055" = 5
"46057" = 4
"46059" = 3
"46061" = 3
"46063" = 5
"46065" = 4
"46067" = 4
"46069" = 4
"46071" = 10
"46073" = 4
"46075" = 6
"46077" = 3
"46079" = 3
"46081" = 4
"46083" = 1
"46085" = 9
"46087" = 3
"46089" = 5
"46091" = 4
"46093" = 4
"46095" = 10
"46097" = 4
"46099" = 3
"46101" = 3
"46103" = 6
"46105" = 6
"46107" = 4
"46109" = 9
"46111" = 5
"46113" = 10
"46115" = 5
"46117" = 3
"46119" = 2
"46121" = 10
"46123" = 9
"46125" = 2
"46127" = 1
"46129" = 9
"46135" = 4
"46137" = 10
"47001" = 7
"47003" = 6
"47005" = 9
"47007" = 9
"47009" = 5
"47011" = 6
"47013" = 10
"47015" = 6
"47017" = 8
"47019" = 9
"47021" = 3
"47023" = 7
"47025" = 10
"47027" = 10
"47029" = 10
"47031" = 7
"47033" = 7
"47035" = 9
"47037" = 7
"47039" = 8
"47041" = 7
"47043" = 5
"47045" = 8
"47047" = 5
"47049" = 10
"47051" = 6
"47053" = 7
"47055" = 7
"47057" = 8
"47059" = 7
"47061" = 10
"47063" = 7
"47065" = 7
"47067" = 10
"47069" = 9
"47071" = 10
"47073" = 8
"47075" = 9
"47077" = 7
"47079" = 8
"47081" = 8
"47083" = 7
"47085" = 6
"47087" = 9
"47089" = 7
"47091" = 10
"47093" = 6
"47095" = 10
"47097" = 8
"47099" = 7
"47101" = 9
"47103" = 6
"47105" = 5
"47107" = 7
"47109" = 9
"47111" = 8
"47113" = 7
"47115" = 7
"47117" = 5
"47119" = 5
"47121" = 9
"47123" = 8
"47125" = 5
"47127" = 4
"47129" = 9
"47131" = 7
"47133" = 9
"47135" = 7
"47137" = 9
"47139" = 8
"47141" = 7
"47143" = 8
"47145" = 7
"47147" = 4
"47149" = 3
"47151" = 10
"47153" = 8
"47155" = 7
"47157" = 8
"47159" = 5
"47161" = 7
"47163" = 7
"47165" = 4
"47167" = 6
"47169" = 7
"47171" = 7
"47173" = 9
"47175" = 8
"47177" = 8
"47179" = 6
"47181" = 9
"47183" = 7
"47185" = 8
"47187" = 1
"47189" = 2
"48001" = 9
"48003" = 7
"48005" = 9
"48007" = 10
"48009" = 2
"48011" = 4
"48013" = 10
"48015" = 4
"48017" = 10
"48019" = 7
"48021" = 6
"48023" = 9
"48025" = 10
"48027" = 7
"48029" = 9
"48031" = 4
"48033" = 3
"48035" = 8
"48037" = 9
"48039" = 4
"48041" = 7
"48043" = 9
"48045" = 9
"48047" = 10
"48049" = 10
"48051" = 8
"48053" = 7
"48055" = 7
"48057" = 8
"48059" = 8
"48061" = 10
"48063" = 10
"48065" = 3
"48067" = 9
"48069" = 10
"48071" = 4
"48073" = 9
"48075" = 10
"48077" = 5
"48079" = 10
"48081" = 8
"48083" = 10
"48085" = 1
"48087" = 10
"48089" = 8
"48091" = 4
"48093" = 10
"48095" = 9
"48097" = 7
"48099" = 6
"48101" = 10
"48103" = 5
"48105" = 7
"48107" = 10
"48109" = 10
"48111" = 7
"48113" = 8
"48115" = 10
"48117" = 9
"48119" = 9
"48121" = 2
"48123" = 9
"48125" = 9
"48127" = 10
"48129" = 9
"48131" = 10
"48133" = 10
"48135" = 10
"48137" = 10
"48139" = 4
"48141" = 10
"48143" = 7
"48145" = 10
"48147" = 7
"48149" = 6
"48151" = 8
"48153" = 10
"48155" = 8
"48157" = 2
"48159" = 8
"48161" = 8
"48163" = 10
"48165" = 10
"48167" = 6
"48169" = 9
"48171" = 5
"48173" = 6
"48175" = 8
"48177" = 10
"48179" = 7
"48181" = 7
"48183" = 9
"48185" = 8
"48187" = 6
"48189" = 10
"48191" = 10
"48193" = 9
"48195" = 6
"48197" = 9
"48199" = 5
"48201" = 8
"48203" = 8
"48205" = 2
"48207" = 10
"48209" = 4
"48211" = 3
"48213" = 9
"48215" = 10
"48217" = 9
"48219" = 9
"48221" = 6
"48223" = 8
"48225" = 10
"48227" = 10
"48229" = 10
"48231" = 8
"48233" = 6
"48235" = 5
"48237" = 6
"48239" = 7
"48241" = 9
"48243" = 9
"48245" = 9
"48247" = 10
"48249" = 10
"48251" = 5
"48253" = 10
"48255" = 10
"48257" = 5
"48259" = 3
"48261" = 7
"48263" = 3
"48265" = 8
"48267" = 10
"48269" = 7
"48271" = 10
"48273" = 10
"48275" = 10
"48277" = 10
"48279" = 10
"48281" = 7
"48283" = 10
"48285" = 7
"48287" = 5
"48289" = 8
"48291" = 8
"48293" = 9
"48295" = 7
"48297" = 8
"48299" = 8
"48301" = 2
"48303" = 9
"48305" = 10
"48307" = 10
"48309" = 9
"48311" = 4
"48313" = 9
"48315" = 10
"48317" = 9
"48319" = 8
"48321" = 9
"48323" = 10
"48325" = 8
"48327" = 10
"48329" = 8
"48331" = 9
"48333" = 9
"48335" = 9
"48337" = 8
"48339" = 4
"48341" = 6
"48343" = 9
"48345" = 8
"48347" = 9
"48349" = 9
"48351" = 10
"48353" = 10
"48355" = 10
"48357" = 5
"48359" = 6
"48361" = 8
"48363" = 9
"48365" = 7
"48367" = 5
"48369" = 7
"48371" = 10
"48373" = 10
"48375" = 10
"48377" = 10
"48379" = 8
"48381" = 2
"48383" = 6
"48385" = 10
"48387" = 10
"48389" = 10
"48391" = 8
"48393" = 2
"48395" = 10
"48397" = 2
"48399" = 9
"48401" = 7
"48403" = 10
"48405" = 10
"48407" = 9
"48409" = 9
"48411" = 10
"48413" = 8
"48415" = 9
"48417" = 6
"48419" = 9
"48421" = 5
"48423" = 7
"48425" = 4
"48427" = 10
"48429" = 10
"48431" = 5
"48433" = 8
"48435" = 6
"48437" = 10
"48439" = 6
"48441" = 8
"48443" = 9
"48445" = 10
"48447" = 6
"48449" = 8
"48451" = 9
"48453" = 6
"48455" = 10
"48457" = 9
"48459" = 9
"48461" = 8
"48463" = 10
"48465" = 10
"48467" = 7
"48469" = 8
"48471" = 8
"48473" = 8
"48475" = 9
"48477" = 6
"48479" = 10
"48481" = 7
"48483" = 8
"48485" = 7
"48487" = 9
"48489" = 10
"48491" = 2
"48493" = 5
"48495" = 8
"48497" = 4
"48499" = 9
"48501" = 7
"48503" = 9
"48505" = 10
"48507" = 10
"49001" = 3
"49003" = 2
"49005" = 2
"49007" = 5
"49009" = 1
"49011" = 2
"49013" = 6
"49015" = 4
"49017" = 4
"49019" = 7
"49021" = 6
"49023" = 3
"49025" = 3
"49027" = 4
"49029" = 1
"49031" = 6
"49033" = 2
"49035" = 3
"49037" = 9
"49039" = 5
"49041" = 5
"49043" = 1
"49045" = 2
"49047" = 5
"49049" = 3
"49051" = 2
"49053" = 5
"49055" = 5
"49057" = 3
"50001" = 2
"50003" = 4
"50005" = 6
"50007" = 1
"50009" = 7
"50011" = 3
"50013" = 2
"50015" = 3
"50017" = 3
"50019" = 7
"50021" = 4
"50023" = 2
"50025" = 4
"50027" = 3
"51001" = 9
"51003" = 2
"51005" = 5
"51007" = 4
"51009" = 5
"51011" = 6
"51013" = 2
"51015" = 2
"51017" = 2
"51019" = 2
"51021" = 5
"51023" = 1
"51025" = 9
"51027" = 10
"51029" = 8
"51031" = 5
"51033" = 5
"51035" = 8
"51036" = 5
"51037" = 9
"51041" = 2
"51043" = 1
"51045" = 4
"51047" = 3
"51049" = 8
"51051" = 9
"51053" = 4
"51057" = 7
"51059" = 1
"51061" = 1
"51063" = 5
"51065" = 2
"51067" = 6
"51069" = 2
"51071" = 5
"51073" = 3
"51075" = 2
"51077" = 9
"51079" = 2
"51081" = 6
"51083" = 9
"51085" = 1
"51087" = 2
"51089" = 7
"51091" = 5
"51093" = 4
"51095" = 2
"51097" = 5
"51099" = 2
"51101" = 2
"51103" = 8
"51105" = 10
"51107" = 1
"51109" = 4
"51111" = 8
"51113" = 4
"51115" = 4
"51117" = 8
"51119" = 7
"51121" = 4
"51125" = 5
"51127" = 2
"51131" = 10
"51133" = 8
"51135" = 8
"51137" = 3
"51139" = 5
"51141" = 7
"51143" = 6
"51145" = 1
"51147" = 8
"51149" = 2
"51153" = 1
"51155" = 7
"51157" = 2
"51159" = 7
"51161" = 2
"51163" = 4
"51165" = 3
"51167" = 9
"51169" = 7
"51171" = 3
"51173" = 8
"51175" = 7
"51177" = 1
"51179" = 1
"51181" = 5
"51183" = 8
"51185" = 9
"51187" = 3
"51191" = 6
"51193" = 8
"51195" = 10
"51197" = 7
"51199" = 1
"51510" = 4
"51515" = 8
"51520" = 10
"51530" = 5
"51540" = 8
"51550" = 3
"51570" = 2
"51580" = 7
"51590" = 10
"51595" = 9
"51600" = 1
"51610" = 1
"51620" = 10
"51630" = 7
"51640" = 10
"51650" = 6
"51660" = 7
"51670" = 9
"51678" = 4
"51680" = 9
"51683" = 2
"51685" = 2
"51690" = 9
"51700" = 7
"51710" = 10
"51720" = 10
"51730" = 10
"51735" = 1
"51740" = 9
"51750" = 5
"51760" = 10
"51770" = 10
"51775" = 3
"51790" = 7
"51800" = 5
"51810" = 2
"51820" = 7
"51830" = 7
"51840" = 6
"53001" = 8
"53003" = 8
"53005" = 4
"53007" = 6
"53009" = 6
"53011" = 4
"53013" = 7
"53015" = 6
"53017" = 5
"53019" = 9
"53021" = 8
"53023" = 5
"53025" = 8
"53027" = 8
"53029" = 2
"53031" = 6
"53033" = 3
"53035" = 3
"53037" = 4
"53039" = 7
"53041" = 7
"53043" = 5
"53045" = 6
"53047" = 10
"53049" = 8
"53051" = 9
"53053" = 4
"53055" = 4
"53057" = 5
"53059" = 5
"53061" = 3
"53063" = 6
"53065" = 8
"53067" = 3
"53069" = 4
"53071" = 7
"53073" = 5
"53075" = 4
"53077" = 9
"54001" = 10
"54003" = 6
"54005" = 10
"54007" = 10
"54009" = 5
"54011" = 9
"54013" = 10
"54015" = 10
"54017" = 10
"54019" = 10
"54021" = 10
"54023" = 8
"54025" = 9
"54027" = 8
"54029" = 5
"54031" = 7
"54033" = 9
"54035" = 8
"54037" = 3
"54039" = 9
"54041" = 10
"54043" = 10
"54045" = 10
"54047" = 10
"54049" = 9
"54051" = 8
"54053" = 9
"54055" = 10
"54057" = 8
"54059" = 10
"54061" = 6
"54063" = 8
"54065" = 5
"54067" = 10
"54069" = 7
"54071" = 7
"54073" = 5
"54075" = 9
"54077" = 9
"54079" = 4
"54081" = 10
"54083" = 9
"54085" = 9
"54087" = 10
"54089" = 10
"54091" = 9
"54093" = 8
"54095" = 9
"54097" = 10
"54099" = 10
"54101" = 10
"54103" = 9
"54105" = 10
"54107" = 8
"54109" = 10
"55001" = 6
"55003" = 6
"55005" = 3
"55007" = 4
"55009" = 2
"55011" = 2
"55013" = 5
"55015" = 1
"55017" = 3
"55019" = 5
"55021" = 1
"55023" = 4
"55025" = 2
"55027" = 1
"55029" = 2
"55031" = 5
"55033" = 3
"55035" = 2
"55037" = 2
"55039" = 1
"55041" = 4
"55043" = 3
"55045" = 2
"55047" = 3
"55049" = 2
"55051" = 4
"55053" = 3
"55055" = 1
"55057" = 4
"55059" = 3
"55061" = 1
"55063" = 2
"55065" = 3
"55067" = 4
"55069" = 2
"55071" = 2
"55073" = 2
"55075" = 3
"55077" = 3
"55078" = 10
"55079" = 9
"55081" = 4
"55083" = 2
"55085" = 2
"55087" = 1
"55089" = 1
"55091" = 3
"55093" = 1
"55095" = 2
"55097" = 2
"55099" = 3
"55101" = 3
"55103" = 4
"55105" = 3
"55107" = 5
"55109" = 1
"55111" = 2
"55113" = 6
"55115" = 3
"55117" = 1
"55119" = 3
"55121" = 2
"55123" = 5
"55125" = 3
"55127" = 2
"55129" = 4
"55131" = 1
"55133" = 1
"55135" = 2
"55137" = 4
"55139" = 1
"55141" = 2
"56001" = 5
"56003" = 7
"56005" = 2
"56007" = 5
"56009" = 4
"56011" = 2
"56013" = 7
"56015" = 7
"56017" = 5
"56019" = 4
"56021" = 4
"56023" = 3
"56025" = 6
"56027" = 7
"56029" = 5
"56031" = 5
"56033" = 5
"56035" = 2
"56037" = 3
"56039" = 1
"56041" = 4
"56043" = 5
"56045" = 3
OTHER = 0
;

RUN;
